
export elastix_webmodule_path="/var/www/html/modules"

elastix_is_elastix()
{
    if ! which rpm &> /dev/null
    then
        return 1
    fi

    if rpm -q elastix &> /dev/null
    then
        return 0
    fi

    return 1
}

elastix_is_version_supported()
{
    if elastix_is_elastix
    then
        export DETECTED_ELASTIX_VERSION=`rpm -q elastix --qf "%{v}"`
        elastix_version=${DETECTED_ELASTIX_VERSION}

        major=`echo ${elastix_version} | cut -d. -f1`
        minor=`echo ${elastix_version} | cut -d. -f2`
        build=`echo ${elastix_version} | cut -d. -f3`

        if [ $major -eq "4" ] 
        then
            return 0
        elif [ $major -eq "2" -a $minor -ge "3" ] 
        then
            return 0
        else
            return 1
        fi
    fi

    return 1
}

elastix_adjust_khomp_config ()
{
	echo "Adjusting channel configuration for Elastix"

	if [ ! -f "${ROOT}/etc/asterisk/khomp.conf" ]
	then
		echo "Unable to find khomp.conf"
		exit 1
	fi

	sed -i 's/; context-fxs = khomp-DD-CC/context-fxs = from-internal/g' ${ROOT}/etc/asterisk/khomp.conf
	sed -i 's/; context-fxo = khomp-DD-CC/context-fxo = from-pstn/g' ${ROOT}/etc/asterisk/khomp.conf
	sed -i 's/; context-gsm-call = khomp-DD-CC/context-gsm-call = from-pstn/g' ${ROOT}/etc/asterisk/khomp.conf
	sed -i 's/; context-digital = khomp-DD-LL/context-digital = from-pstn/g' ${ROOT}/etc/asterisk/khomp.conf

    sed -i 's/; load-error = failure/load-error = skip/g' ${ROOT}/etc/asterisk/khomp.conf
}

elastix_install_webmodule ()
{
    echo "Installing Khomp web module for Elastix"

    make_dir_copy "${ROOT}/${elastix_webmodule_path}/khomp" "${PACKDIR}/webportal/" tree 755
    make_dir_copy "${ROOT}/${elastix_webmodule_path}/kconfig" "${PACKDIR}/webportal/KWebConfig" tree 755
    make_dir_copy "${ROOT}/${elastix_webmodule_path}/kmonitor" "${PACKDIR}/webportal/KWebMonitor" tree 755
    make_dir_copy "${ROOT}/${elastix_webmodule_path}/kdiagnostic" "${PACKDIR}/webportal/KWebLogViewer" tree 755

    rm -rf "${ROOT}/${elastix_webmodule_path}/khomp/KWebConfig"
    rm -rf "${ROOT}/${elastix_webmodule_path}/khomp/KWebMonitor"
    rm -rf "${ROOT}/${elastix_webmodule_path}/khomp/KWebLogViewer"

    mv -fv "${ROOT}/${elastix_webmodule_path}/kconfig/elastix-index.php" "${ROOT}/${elastix_webmodule_path}/kconfig/index.php"
    mv -fv "${ROOT}/${elastix_webmodule_path}/kmonitor/elastix-index.php" "${ROOT}/${elastix_webmodule_path}/kmonitor/index.php"
    mv -fv "${ROOT}/${elastix_webmodule_path}/kdiagnostic/elastix-index.php" "${ROOT}/${elastix_webmodule_path}/kdiagnostic/index.php"

    touch "${ROOT}/opt/khomp/webserver/skip-kwebserver"
}

# Khomp installations are kind of magic!
# The right channel will be chosen during
# the installation process ("post" RPM scripts)
elastix_install_all_channel_binaries ()
{
    source ./functions-generic.sh
    source ./instalador-channel.sh
    source ./distribution-versions.sh

    packdir=$1

    if [ ! -d ${packdir}/channel${arch_numbits} ]
    then
        echo "COULD NOT FIND MODULES FOR INSTALATION"
        exit
    fi

    # The Elastix update repo has only 
    # Asterisk versions greater than 11.4.0
    # for the 11.x line.
    channel_versions="11.1.1 1.8.19.1 1.8.0"

    get_distro_version "chan_khomp"

    for channel_version in ${channel_versions}
    do
        check_specific_files ()
        {
            test -f "${packdir}/channel${arch_numbits}/${channel_version}/chan_khomp-$1.so"
            return $?
        }  

        enum_specific_files ()
        {
            (find "${packdir}/channel${arch_numbits}/${channel_version}" -mindepth 1 -maxdepth 1 \
                -type f -printf '%f\n' | grep 'chan_khomp' | sed -e 's/chan_khomp-\(.\+\)\.so/\1/' | sort -r | uniq)
            return $?
        }

        compatible_version="`search_compatible_version "${GLIBC_VERSION}" "${GCC_VERSION}" check_specific_files enum_specific_files`"

        if [ "@$compatible_version" != "@" ]
        then
                cp ${packdir}/channel${arch_numbits}/${channel_version}/chan_khomp-${compatible_version}.so ${ROOT}/usr/lib${arch_numbits}/chan_khomp-${channel_version}.so
                cp ${packdir}/channel${arch_numbits}/${channel_version}/libkchannel-${compatible_version}.so ${ROOT}/usr/lib${arch_numbits}/libkchannel-${channel_version}.so
        else
            echo "NO CHANNEL MODULE FOR VERSION ${channel_version}"
            exit
        fi
    done

    rm -f ${ROOT}/usr/lib${arch_numbits}/libkchannel.so
    rm -f ${ROOT}/usr/lib${arch_numbits}/asterisk/modules/chan_khomp.so
    
    ln -sfn libkchannel.so ${ROOT}/usr/lib${arch_numbits}/libkchannel.so
    ln -sfn chan_khomp.so ${ROOT}/usr/lib${arch_numbits}/asterisk/modules/chan_khomp.so
}

# Installing to a RPM buildroot
elastix_rpm_install ()
{
    ROOT=$1
    LIBDIR=$2

    mkdir -p ${ROOT}/usr/lib ${ROOT}/etc ${ROOT}${LIBDIR}/asterisk/modules
    export PATH=/usr/sbin:/sbin:$PATH

    export OPT_PURGE="no"
    export OPT_NO_INSTALL="no"
    export OPT_UNCOMPRESS_ONLY="no"
    export OPT_REPACKAGE="no"
    export OPT_VERBOSE="no"

    export OPT_BUILD="auto"
    export OPT_ASTERISK="auto"

    export OPT_SKIP_CHAN_KHOMP="no"

    export OPT_SKIP_KERNEL_MOD="no"
    export OPT_SKIP_WEBPORTAL="yes"
    export OPT_SKIP_WIZARD="yes"

    export PACKDIR=.

    if [ $LIBDIR == "/usr/lib64" ] ; then
        export arch_numbits=64
    else
        export arch_numbits=""
    fi

    msg ()
    {
	    echo "$@"
    }	

    export OPT_INSTALLING_RPM_ROOT="yes"

    source ./functions-generic.sh
    source ./instalador-k3l.sh && k3l_install
    source ./instalador-channel.sh && channel_install

    elastix_install_all_channel_binaries ${PACKDIR}
}

elastix_fix_chkconfig_services ()
{
    sed -i '/#!\/bin\/sh/a \
\
# \
# k3lserver      Startup script for the KHOMP server daemon \
# \
# chkconfig: - 88 15 \
# description: The KHOMP server manages communication between Asterisk and KHOMP telephony cards \
\
' ${ROOT}/etc/init.d/k3lserver

    sed -i '/#!\/bin\/bash/a \
\
# \
# khompdrv     Startup script for the KHOMP drivers \
# \
# chkconfig: - 87 07 \
# description: KHOMP telephony card drivers \
\
' ${ROOT}/etc/init.d/khompdrv

    sed -i '/#!\/bin\/sh/a \
\
# \
# khomp-services      Startup script for the KHOMP services daemons \
# \
# chkconfig: - 88 15 \
# description: The KHOMP server manages communication between Asterisk and KHOMP telephony cards \
\
' ${ROOT}/etc/init.d/khomp-services


    sed -i '/#!\/bin\/sh/a \
\
# \
# kibs      Startup script for the KHOMP Integrated Boot Server daemon \
# \
# chkconfig: - 88 15 \
# description: The KHOMP server manages communication between Asterisk and KHOMP telephony cards \
\
' ${ROOT}/etc/init.d/kibs

    sed -i '/#!\/bin\/sh/a \
\
# \
# klogserver      Startup script for the KHOMP log server daemon \
# \
# chkconfig: - 88 15 \
# description: The KHOMP server manages communication between Asterisk and KHOMP telephony cards \
\
' ${ROOT}/etc/init.d/klogserver

    sed -i '/#!\/bin\/sh/a \
\
# \
# kqueryserver      Startup script for the KHOMP query server daemon \
# \
# chkconfig: - 88 15 \
# description: The KHOMP server manages communication between Asterisk and KHOMP telephony cards \
\
' ${ROOT}/etc/init.d/kqueryserver

    sed -i '/#!\/bin\/sh/a \
\
# \
# kwebserver      Startup script for the KHOMP webserver daemon  \
# \
# chkconfig: - 88 15 \
# description: The KHOMP server manages communication between Asterisk and KHOMP telephony cards \
\
' ${ROOT}/etc/init.d/kwebserver


}

elastix_adjust_kibs_config ()
{
	local OPT_FIRMWARE_DIR='/tftpboot'

    if grep -q "AlternativePath" ${ROOT}/etc/khomp/config/kibs.yaml 
    then
        sed -i 's,AlternativePath:.*,AlternativePath: /'$OPT_FIRMWARE_DIR',g' ${ROOT}/etc/khomp/config/kibs.yaml
    else
        sed -i 's,\(^[ ]*["]\?tftp["]\?:.*$\),\1\n        AlternativePath: /'$OPT_FIRMWARE_DIR',g' ${ROOT}/etc/khomp/config/kibs.yaml
    fi
}

elastix_support_install ()
{
	elastix_adjust_kibs_config
	elastix_adjust_khomp_config
	elastix_fix_chkconfig_services
	elastix_install_webmodule
}

# These are post install functions
elastix_add_services ()
{
	for i in kibs klogserver kqueryserver k3lserver khompdrv ; do
		/sbin/chkconfig --add $i
		/sbin/chkconfig $i on
	done

	/sbin/chkconfig --add khomp-services
	/sbin/chkconfig khomp-services off

	/sbin/chkconfig --add kwebserver
	/sbin/chkconfig kwebserver off
}

elastix_menumerge ()
{
	if type "elastix-menumerge" &> /dev/null && [ -e "${elastix_webmodule_path}/khomp/elastix-menu.xml" ];
	then
		elastix-menumerge "${elastix_webmodule_path}/khomp/elastix-menu.xml"
	else
	    echo "Unable to execute 'elastix-menumerge'"
		exit 1
	fi
}

elastix_services_start ()
{
	/sbin/service khomp-services start
}

elastix_make_logdirs ()
{
    if [ ! -d /var/log/khomp/old ]
    then
        mkdir /var/log/khomp/old
    fi
}

elastix_set_permissions ()
{
	chown -R "asterisk:asterisk" /etc/khomp/config/*
	chown -R "asterisk:asterisk" /etc/khomp/config/.default.*

	chown -R "asterisk:asterisk" /var/www/html/modules/khomp
	chown -R "asterisk:asterisk" /var/www/html/modules/kmonitor
	chown -R "asterisk:asterisk" /var/www/html/modules/kconfig
	chown -R "asterisk:asterisk" /var/www/html/modules/kdiagnostic

    chown -R "asterisk:asterisk" /var/log/khomp/old
}

elastix_set_sudoers ()
{
    PERM=`stat -c %a /etc/sudoers`
    chmod 640 /etc/sudoers
    grep khomp-sudo /etc/sudoers > /dev/null || echo "asterisk ALL = NOPASSWD: /usr/sbin/khomp-sudo" >> /etc/sudoers
    chmod $PERM /etc/sudoers
}

elastix_nearest_asterisk_version ()
{
    if [ "$1" -eq "11" ]
    then
        if [ "$2" -gt "1" ]; then echo "11.1.1"
        else                      echo  "unknown"
        fi
    elif [ "$2" -eq "8" ]
    then
        if [ "$3" -eq "19" ]
        then
            if [ "$4" -ge "1" ]; then echo "1.8.19.1"
            else                      echo "1.8.0"
            fi
        elif [ "$3" -gt "19" ]; then echo "1.8.19.1"
        else                         echo "1.8.0"
        fi
    else
        echo "unknown"
    fi
}

elastix_relink_channel_libraries ()
{
    if [ -d "/usr/lib64" ] ; then
        local arch_numbits=64
    else
        local arch_numbits=""
    fi

    local asterisk_version="`asterisk -V | grep -o '[0-9]\+\.[0-9]\+\.[0-9]\+\(\.[0-9]\+\)\?'`"

    if [ "w${asterisk_version}" == "w" ]; then
        echo "This is bad: impossible to determine Asterisk version. Is Asterisk installed?"
        echo "Try to reinstall your Asterisk package, later reinstall the Khomp channel driver."
        exit 1
    fi

    ast_major_ver="$(echo ${asterisk_version} | cut -d. -f1)" 
    ast_minor_ver="$(echo ${asterisk_version} | cut -d. -f2)" 
    ast_build_ver="$(echo ${asterisk_version} | cut -d. -f3 | cut -d- -f1)" 
    ast_micro_ver="$(echo ${asterisk_version} | cut -d. -f4 | cut -d- -f1)"

    nearest_version=`elastix_nearest_asterisk_version $ast_major_ver $ast_minor_ver $ast_build_ver $ast_micro_ver`

    if [ -f "/usr/lib${arch_numbits}/libkchannel-${nearest_version}.so" -a -f "/usr/lib${arch_numbits}/chan_khomp-${nearest_version}.so" ]
    then
        echo "Binary for Asterisk ${asterisk_version} found: ${nearest_version}"

        ln -sfn /usr/lib${arch_numbits}/libkchannel-${nearest_version}.so /usr/lib${arch_numbits}/libkchannel.so
        ln -sfn /usr/lib${arch_numbits}/chan_khomp-${nearest_version}.so /usr/lib${arch_numbits}/asterisk/modules/chan_khomp.so    
    else
        echo "This is bad: the Asterisk $asterisk_version is not compatible with the installed Khomp Channel!"
        exit 1
    fi
}

# It should be independent of any variable built by the other installation scripts
elastix_post_install ()
{
    elastix_set_sudoers
    elastix_make_logdirs
    elastix_set_permissions
    elastix_add_services
    elastix_menumerge
}

# Uninstall stuff down here
elastix_menuremove ()
{
	if type "elastix-menuremove" &> /dev/null && [ -e "${elastix_webmodule_path}/khomp/elastix-menu.xml" ];
	then
		elastix-menuremove "chan_khomp"
	else
		echo "Unable to find command 'elastix-menuremove'"
		exit 1
	fi
}

elastix_unset_sudoers ()
{
	sed -i '/asterisk[[:space:]]*ALL[[:space:]]*=[[:space:]]*NOPASSWD[[:space:]]*:[[:space:]]\/usr\/sbin\/khomp-sudo/d' /etc/sudoers
}

elastix_pre_uninstall ()
{
	elastix_unset_sudoers
	elastix_menuremove
}


