<?php

class Dev
{
    public $board = -1;
    public $serial = -1;
    public $callerid_base = -1;
    public $callerid_end = -1;
    public $branchToCallerid = array(); // aliases
    public $calleridToChan = array();
    public $chanToBranch = array();
    public $branchToChan = array();
}   


class branchManager
{
    private $devs = array();

    public function __construct()
    {
        $this->init();
    }
         
    private function init()
    {
        Global $sistema;
        $ids = $this->mapBranches();
        $ranges = $sistema->getTechChannels('FXS');
        $globalOrig = getSpecificOptValue('fxs-global-orig');

        $chans = array();
        $branches = array();
        
        foreach($ids as $s => $callerid_range)
        {
            $dev = new Dev();
            $dev->callerid_base = $callerid_range[0];
            $dev->callerid_end  = $callerid_range[1];
            $dev->serial = $s;

            if(!array_key_exists($s, $ranges))
                continue;
            $range = $ranges[$s];
            $dev->board = $range['board'];

            for($branch = $dev->callerid_base; $branch <= $dev->callerid_end; ++$branch)
            {
                $bxcy = 'B'.$dev->board.'C'.($branch - $dev->callerid_base + $range[0]);
                
                if( $globalOrig < 100 )
                    $b_str = str_pad( $branch, strlen( $globalOrig ), "0", STR_PAD_LEFT );
                else
                    $b_str = (string) $branch;
                
                $dev->branchToChan[$b_str] = $bxcy;
                $dev->chanToBranch[$bxcy] = $b_str;
                $dev->branchToCallerid[$b_str] = $b_str;
                $dev->calleridToChan[$b_str] = $bxcy;
            }

            $this->devs[$dev->board] = $dev;
        }

        // now process fxs-options
        $Kconf_arr = loadKhompConfig( );
        $FXSOpts = $Kconf_arr['geral']['fxs-options'];

        foreach( $FXSOpts as $branch => $opts )
        {
            if(preg_match('/calleridnum\s*:\s*(\d+)/', $opts))
            {
                $alias = preg_replace('/.*calleridnum\s*:\s*(\d+).*/', '$1', $opts);

                if(is_numeric($branch))
                {
                    if( ! $alias )
                        $alias = $branch;
                    $dev = $this->get_dev_by_branch_original( $branch );
                    $dev->branchToCallerid[$branch] = (string)$alias;
                    $bxcy = $dev->branchToChan[$branch];
                    unset( $dev->calleridToChan[$branch] );
                    $dev->calleridToChan[$alias] = $bxcy;
                }
                else
                {
                    $dev = $this->get_dev_by_bxcy($branch);
                    $b = $dev->chanToBranch[$branch];
                    $dev->branchToCallerid[$b] = (string)$alias;
                    unset( $dev->calleridToChan[$b] );
                    $dev->calleridToChan[$alias] = $branch;
                }
            }
        }
    }

    // Returns a mapping Serial -> [callerid_base,callerid_end]
    private function mapBranches()
    {
        Global $sistema;
        $idToSerial = getSpecificOptValue(false, 'fxs-branches');

        $map = array();
        $ranges = $sistema->getTechChannels('FXS');
        
        foreach($idToSerial as $id => $serial_list)
        {
            $serials = explode(',', str_replace(' ', '', $serial_list));
            $n_chans=0;
            foreach($serials as $s)
            {
                $n = $ranges[$s][1] - $ranges[$s][0] + 1;
                $map[$s] = array($id + $n_chans, $id + $n_chans + $n - 1);
                $n_chans += $n;
            }                
        }

        $allSerials = $sistema->getDeviceSerials();
        $globalOrig = getSpecificOptValue('fxs-global-orig');
        $n_chans=0;
        foreach($allSerials as $s)
        {
            if(array_key_exists($s, $map))
                continue;

            $n = $ranges[$s][1] - $ranges[$s][0] + 1;
            if($n <= 1)
                continue;
            $map[$s] = array($globalOrig + $n_chans, $globalOrig + $n_chans + $n - 1);
            $n_chans += $n;
        }

        return $map;
    }

    private function get_dev_by_serial($serial)
    {
        foreach( $this->devs as $d )
            if( $d->serial == $serial )
                return $d;
    }

    private function get_dev_by_board($board)
    {
        return $this->devs[$board];
    }
    
    private function get_dev_by_bxcy($bxcy)
    {
        $bc = getBoardChanFromStr($bxcy);
        if(!$bc)
            return null;
        return $this->get_dev_by_board( $bc[0] );
    }

    // alias not applied yet
    private function get_dev_by_branch_original($branch)
    {
        foreach( $this->devs as $d )
        {
            if( $branch >= $d->callerid_base and $branch <= $d->callerid_end )
                return $d;
        }
        return false;
    }

    public function get_callerid_from_branch($branch)
    {
        $dev = $this->get_dev_by_branch_original($branch);
        return $dev->branchToCallerid["{$branch}"];
    }
        
    public function get_callerid_from_bxcy($bxcy)
    {
        $dev = $this->get_dev_by_bxcy($bxcy);
        $branch = $dev->chanToBranch[$bxcy];
        return $dev->branchToCallerid[$branch];
    }

    public function get_callerids_from_serial_sorted( $serial )
    {
        $dev = $this->get_dev_by_serial( $serial );
        $alias = $dev->branchToCallerid;
        sort($alias);
        return $alias;
    }

    public function get_bxcy_from_callerid( $callerid )
    {
        foreach( $this->devs as $d )
        {
            if( array_key_exists( $callerid, $d->calleridToChan ) )
                return $d->calleridToChan[$callerid];
        }
        return '';
    }

    public function is_alias( $bxcy, $callerid )
    {
        $dev = $this->get_dev_by_bxcy($bxcy);
        $branch = $dev->chanToBranch[$bxcy];
        return $callerid != $branch;
    }
}
