<?php
    require_once KCONFIG . '/classes/classKGatewayNTP.php';

    define( 'KGATEWAYTZDIR', '/usr/share/zoneinfo' );
    define( 'KGATEWAYTZFILE', '/etc/localtime' );
    define( 'KGATEWAYTZINFO', '/etc/timezone' );

    $AjaxFunctions['showDateTimeGW'] = 'showDateTimeGW';
    $AjaxFunctions['saveDateTimeGW'] = 'saveDateTimeGW';

    function readTimeZoneRecursive( $directory, &$ignoreTimeZones )
    {
        $list = array( );
        $files = glob( KGATEWAYTZDIR . "{$directory}/*", GLOB_NOSORT );
        $nrfiles = count( $files );
        for( $c = 0; $c < $nrfiles; $c++ )
        {
            $basename = basename( $files[$c] );
            if( in_array( $basename, $ignoreTimeZones ) )
            {
                continue;
            }
            $f = "{$directory}/{$basename}";
            if( is_dir( KGATEWAYTZDIR . "/{$f}" ) )
            {
                $list = array_merge( $list, readTimeZoneRecursive( $f, $ignoreTimeZones ) );
            }
            else
            {
                // remove frist slash
                $f = substr( $f, 1 );
                $list[] = $f;
            }
        }
        return $list;
    }

    function getTimeZoneList( )
    {
        $ignoreTimeZones = array( 'posix', 'right', 'SystemV', 'zone.tab', 'localtime', 'iso3166.tab', 'posixrules' );
        $list = readTimeZoneRecursive( '', $ignoreTimeZones );
        asort( $list );
        return $list;
    }

    function getCurrentTimeZone( )
    {
        return trim( file_get_contents( KGATEWAYTZINFO ) );
    }

    function showDateTimeGW( )
    {
        $app = KSystem::getInstance( );

        try
        {
            $ntp = new KGatewayNTP( );
            $ntp->load( );
            $ntpconfig = $ntp->getConfiguration( );

            $buf = "<fieldset><legend>{$app->translate( 'gateway-datetime' )}</legend>";
            $buf .= "<div id='gateway-datetime'>";

            $now = new DateTime( );
            $buf .= "<div id='gateway-datetime-current'><table class='zebra simpleTable'>";
            $buf .= "<tr><td style='text-align:left;'><label for='gateway-datetime-date'>{$app->translate( 'gateway-datetime-date' )} (dd/mm/yyyy)</label></td><td style='text-align:right;'><input type='text' name='Date' id='gateway-datetime-date' value='{$now->format( 'd/m/Y' )}' /><input type='hidden' id='gateway-datetime-date-changed' value='no' /></td></tr>";
            $buf .= "<tr><td style='text-align:left;'><label for='gateway-datetime-time'>{$app->translate( 'gateway-datetime-time' )} (hh:mm:ss)</label></td><td style='text-align:right;'><input type='text' name='Time' id='gateway-datetime-time' value='{$now->format( 'H:i:s' )}' /><input type='hidden' id='gateway-datetime-time-changed' value='no' /></td></tr>";
            $timezonelist = getTimeZoneList( );
            $mytz = getCurrentTimeZone( );
            $options = '';
            foreach( $timezonelist as &$tz )
            {
                $selected = $tz == $mytz ? ' selected="selected"' : '';
                $options .= "<option value='{$tz}'{$selected}>{$tz}</option>";
            }
            $buf .= "<tr><td style='text-align:left;'><label for='gateway-datetime-timezone'>{$app->translate( 'gateway-datetime-timezone' )}</label></td><td style='text-align:right;'><select name='TimeZone' id='gateway-datetime-timezone'>{$options}</select></td></tr>";
            $buf .= "</table><br /><div class='textAlignCenter'>{$app->translate( 'gateway-datetime-timezone-warning' )}</div></div><br />";

            $checked = array_key_exists( 'ntp-enabled', $ntpconfig ) && $ntpconfig['ntp-enabled'] == 'yes' ? ' checked=\'checked\'' : '';
            $buf .= "<div id='gateway-datetime-ntp'><fieldset><legend><input type='checkbox' id='gateway-datetime-ntp-enable'{$checked}/> NTP / SNTP</legend>";
            $buf .= '<table class="zebra simpleTable"><tbody>';
            $buf .= "<tr><td style='text-align:left;'><label for='gateway-datetime-ntp-timeout'>{$app->translate( 'gateway-datetime-ntp-timeout')}</label></td><td style='text-align:right;'><input type='text' id='gateway-datetime-ntp-timeout' value='{$ntpconfig['ntp-timeout']}' /></td></tr>";
            $buf .= "<tr><td style='text-align:left;'>{$app->translate( 'gateway-datetime-servers' )}</td><td style='text-align:right;'><div id='gateway-datetime-ntp-server-list'>";
            $servers = array_key_exists( 'ntp-servers', $ntpconfig ) ? explode( ' ', $ntpconfig['ntp-servers'] ) : array( );
            foreach( $servers as $server )
            {
                $buf .= "<input type='text' class='gateway-datetime-ntp-server' value='{$server}' /><br />";
            }
            $buf .= "</div><a href='#' id='gateway-datetime-ntp-add'>{$app->translate( 'gateway-datetime-add-server' )}</a></td></tr>";
            $buf .= '</td></tr>';
            $buf .= '</tbody></table></fieldset></div><br />';

            $buf .= "<div class='textAlignRight'><input type='button' id='gateway-datetime-save' value='{$app->translate( 'save' )}' /></div>";
        }
        catch( KGatewayNTPException $e )
        {
            $buf .= $app->translate( $e->getMessage( ) );
        }
        $buf .= "</div></fieldset>";

        return $buf;
    }

    function saveDateTimeGW( )
    {
        require_once KROOT . '/json_util.php';
        
        $app = KSystem::getInstance( );
        $reply = array( 'success' => false, 'message' => $app->translate( 'gateway-datetime-save-error' ) );

        try
        {
            $ntp = new KGatewayNTP( );
            $ntp->load( );
            $ntpconfig = $ntp->getConfiguration( );
            $now = new DateTime( );
            $dateparts = null;
            $timeparts = null;

            // validate
            if( isset( $_POST['date'] ) )
            {
                $dateparts = explode( '/', $_POST['date'] );
                if(!checkdate($dateparts[1], $dateparts[0], $dateparts[2]))
                {
                    throw new Exception( 'gateway-datetime-error-invalid-date' );
                }
                if ( ( (int)$dateparts[2] < 1970 ) || ( (int)$dateparts[2] > 2069 ) )
                {
                    throw new Exception( 'gateway-datetime-error-invalid-date' );
                }
            }
            if( isset( $_POST['time'] ) )
            {
                $timeparts = explode( ':', $_POST['time'] );
                //@formatter:off
                if( count( $timeparts ) !== 3
                    || $timeparts[0] < 0 || $timeparts[0] > 23
                    || $timeparts[1] < 0 || $timeparts[1] > 59
                    || $timeparts[2] < 0 || $timeparts[2] > 59
                )
                //@formatter:on
                {
                    throw new Exception( 'gateway-datetime-error-invalid-time' );
                }
            }

            // ntp configuration
            $config = array( );
            foreach( $_POST['ntp'] as $key => $value )
            {
                $config["ntp-{$key}"] = $value;
            }
            $ntp->setConfiguration( $config );
            $ntp->save( );

            $mytz = getCurrentTimeZone( );
            if( $_POST['timezone'] != $mytz )
            {
                // changing timezone
                // first copy the new timezone file to /etc with a temporary
                // name
                $srcfile = KGATEWAYTZDIR . "/{$_POST['timezone']}";
                $dstfile = KGATEWAYTZFILE . '.kwebportal';
                // just to be sure ;)
                if( file_exists( $dstfile ) )
                {
                    unlink( $dstfile );
                }
                $src = fopen( $srcfile, 'r' );
                $dst = fopen( $dstfile, 'w' );
                do
                {
                    $data = fread( $src, 4096 );
                    if( $data === false )
                    {
                        //TODO: generic IO exception?
                        throw new Exception( 'gateway-datetime-error-io' );
                    }
                    if( fwrite( $dst, $data ) === false )
                    {
                        //TODO: generic IO exception?
                        throw new Exception( 'gateway-datetime-error-io' );
                    }
                }
                while( !feof( $src ) );
                fclose( $src );
                fclose( $dst );

                // now update TZINFO
                if( file_put_contents( KGATEWAYTZINFO, "{$_POST['timezone']}\n" ) === false )
                {
                    //TODO: generic IO exception?
                    throw new Exception( 'gateway-datetime-error-io' );
                }
                // commit localtime
                if( rename( $dstfile, KGATEWAYTZFILE ) === false )
                {
                    //TODO: generic IO exception?
                    throw new Exception( 'gateway-datetime-error-io' );
                }
            }

            $output = array( );
            $excode = -1;	
            // update date and time
            if( $dateparts && $timeparts )
            {
                exec( "/bin/date -s \"{$dateparts[2]}-{$dateparts[1]}-{$dateparts[0]} {$timeparts[0]}:{$timeparts[1]}:{$timeparts[2]}\" 2>&1", $output, $excode );
                if( $excode != 0 )
                {
                    throw new Exception( implode( ', ', $output ) );
                }
            }
            // update date only
            elseif( isset( $_POST['date'] ) )
            {
                exec( "/bin/date -s \"{$dateparts[2]}-{$dateparts[1]}-{$dateparts[0]} {$now->format('H:i:s')}\" 2>&1", $output, $excode );
                if( $excode != 0 )
                {
                    throw new Exception( implode( ', ', $output ) );
                }
            }
            // update time only
            elseif( isset( $_POST['time'] ) )
            {
                exec( "/bin/date -s \"{$now->format('Y-m-d')} {$timeparts[0]}:{$timeparts[1]}:{$timeparts[2]}\" 2>&1", $output, $excode );
                if( $excode != 0 )
                {
                    throw new Exception( implode( ', ', $output ) );
                }
            }
            // commit date or time update to system clock
            if( isset( $_POST['date'] ) || isset( $_POST['time'] ) )
            {
                exec( '/sbin/hwclock --systohc 2>&1', $output, $excode );
                if( $excode != 0 )
                {
                    throw new Exception( implode( ', ', $output ) );
                }
            }

            $reply = array( 'success' => true, 'message' => $app->translate( 'gateway-datetime-saved' ) );
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }
        return JSONencode( $reply );

    }
