<?php
    require_once KCONFIG . '/classes/classKGatewayUtil.php';
    require_once KCONFIG . '/classes/classKGatewayNetwork.php';
    require_once KCONFIG . '/classes/KGatewayYaml.php';
    require_once KROOT . '/classes/KFeatures.php';

    $AjaxFunctions['showNetworkGW'] = 'showNetworkGW';
    $AjaxFunctions['saveNetworkGW'] = 'saveNetworkGW';
    $AjaxFunctions['showVoIPOptionsGW'] = 'showVoIPOptionsGW';
    $AjaxFunctions['showVoIPTLSGW'] = 'showVoIPTLSGW';
    $AjaxFunctions['getHostName_'] = 'getHostName_';

    class KGatewayNetworkConfigException extends Exception
    {
        /* VOID */
    }

    function iseth( $intf )
    {
        // GAMBI: return ONLY ethX interfaces, rá!
        return strpos( $intf, 'eth' ) === 0 && strlen( $intf ) === 4;
    }

    function showNetworkGW( )
    {
        require_once KCONFIG . '/classes/KGatewayHardwareInfo.php';

        $app = KSystem::getInstance( );
        $hwinfo = KGatewayHardwareInfo::getInstance( );
        $parser = KGatewayNetwork::getInstance( );

        $buffer = '';

        $buffer .= '<div class="textAlignCenter displayNone sms-info" id="gateway-reboot-is-needed">';
        $buffer .= "<label>{$app->translate( 'gateway-network-alert-reboot' )}</label>";

        $buffer .= '<br /><br />';

        $buffer .= "<input id='gateway-alert-reboot-bt' type='button' onclick='rebootGW();' value='{$app->translate( 'gateway-reboot' )}' />";
        $buffer .= "</div>";

        $buffer .= '<br />';

        $buffer .= "<fieldset><legend>{$app->translate( 'gateway-network' )}</legend>";
        $buffer .= "<div id='gateway-network'>";

        $switchcores = count( $hwinfo->query( 'Switch.Cores' ) );

        if( $switchcores > 0 )
        {
            $buffer .= showNetworkVlan( $parser, $hwinfo );
        }
        else
        {
            $buffer .= showNetworkNoVlan( $parser );
        }

        // DNS SERVERS
        $servers = $parser->getDNSServers( );

        $buffer .= '<div id=\'gateway-network-dns\'><fieldset><legend>DNS</legend>';

        if( $switchcores > 0 )
        {
            $buffer .= '<input type=\'hidden\' id=\'gateway-network-dns-method\' value=\'static\' />';
        }
        else
        {
            $buffer .= "<div class='gateway-network-method'><label for='gateway-network-dns-method'>{$app->translate( 'gateway-network-dns-method' )}</label> <select id='gateway-network-dns-method'>";

            foreach( KGatewayNetwork::$dnsmethods as $method )
            {
                $selected = (!empty( $servers ) && $method == 'static') ? ' selected=\'true\'' : '';
                $name = $app->translate( "gateway-network-method-{$method}" );
                $buffer .= "<option value='{$method}'{$selected}>{$name}</option>";
            }

            $buffer .= '</select></div><br />';
        }

        $dnssearch = $parser->getDNSSearchDomain( );

        $buffer .= '<div id=\'gateway-network-dns-config\'><table class=\'gateway-table-form zebra simpleTable\'><tbody>';
        $buffer .= "<tr><td class='label'><label for='gateway-network-dns-search'>{$app->translate( 'gateway-network-dns-search' )}</label></td><td class='gateway-form-field-container-right'><input type='text' id='gateway-network-dns-search' value='{$dnssearch}' /></td></tr>";
        $buffer .= "<tr><td class='label'>{$app->translate( 'gateway-network-servers' )}</td><td class='gateway-form-field-container-right'>";
        for( $i = 0; $i < 3; ++$i )
        {
            $value = isset( $servers[$i] ) ? $servers[$i] : '';
            $buffer .= "<input type='text' class='gateway-network-dns-server' value='{$value}' /><br />";
        }
        $buffer .= "</td></tr>";
        $buffer .= '</tbody></table></fieldset></div><br />';
        unset( $servers );
        unset( $dnssearch );

        $buffer .= "<br /><div class='textAlignRight'><input type='button' id='gateway-network-save' value='{$app->translate( 'save' )}' /></div>";

        return "{$buffer}</div></fieldset>";
    }

    function __networkMethodSelect( $intf, $method, $extraoptions = array() )
    {
        $app = KSystem::getInstance( );

        $buf = "<select id='gateway-network-method-{$intf}' class='gateway-network-interface-method'>";
        foreach( array_merge( KGatewayNetwork::$methods['inet'], $extraoptions ) as $option )
        {
            $selected = $method === $option ? ' selected=\'selected\'' : '';

            $name = $app->translate( "gateway-network-method-{$option}" );
            $buf .= "<option value='{$option}'{$selected}>{$name}</option>";
        }
        return "{$buf}</select>";
    }

    function __networkVLANModeCheckbox( $intf, $mode )
    {
        $checked = ( $mode === 'MEMBER' ) ? " checked='checked'" : '';
        return "<input id='gateway-network-vlan-mode-{$intf}' type='checkbox' value='true' class='vlan-mode method-not-manual'{$checked}/>";
    }

    function __networkInput( $intf, $name, $value, $class = '' )
    {
        return "<input type='text' value='{$value}' id='gateway-network-{$name}-{$intf}' class='{$class}' />";
    }

    function __networkGatewayDefaultCheckbox( $intf, $name, $value, $class = '' )
    {
        $checked = $value ? " checked='checked'" : '';
        return "<input id='gateway-network-{$name}-{$intf}' type='checkbox' class='{$class}'{$checked}/>";
    }

    function showNetworkVlan( KGatewayNetwork $parser, KGatewayHardwareInfo $hwinfo )
    {
        require_once KCONFIG . '/classes/KGatewaySwitch.php';

        $app = KSystem::getInstance( );
        $cores = $hwinfo->query( 'Switch.Cores' );
        $switchyaml = new KGatewaySwitchYaml( );

        $buf = '<div id="gateway-network-vlan-interfaces"><input type="hidden" value="vlan" id="gateway-network-configuration-mode" />';

        $buf .= "<table class='gateway-table-form zebra simpleTable'><thead><tr><th rowspan='2'>{$app->translate( 'gateway-network-interface' )}</th><th rowspan='2'>{$app->translate( 'gateway-network-method' )}</th><th colspan='3'>IPv4</th><th colspan='3'>IPv6</th><th colspan='2' class='advanced-options'>VLAN <img src='../imagens/tooltip.png' title='{$app->translate( 'gateway-network-vlan-tooltip' )}' class='gateway-tooltip' /></th></tr>";
        $buf .= "<tr><th>{$app->translate( 'gateway-network-address' )}</th><th>{$app->translate( 'gateway-network-netmask' )}</th><th>{$app->translate( 'gateway-network-gateway' )} <img src='../imagens/tooltip.png' title='{$app->translate( 'gateway-network-gateway-tooltip' )}' class='gateway-tooltip' /></th><th>{$app->translate( 'gateway-network-address' )}</th><th>{$app->translate( 'gateway-network-netmask' )}</th><th>{$app->translate( 'gateway-network-gateway' )} <img src='../imagens/tooltip.png' title='{$app->translate( 'gateway-network-gateway-tooltip' )}' class='gateway-tooltip' /></th><th class='advanced-options'>ID</th><th class='advanced-options'>{$app->translate( 'gateway-network-vlan-propagate' )}</th></tr><tbody>";

        foreach( $hwinfo->getSwitchPortNames() as $idx => $name )
        {
            $port = $hwinfo->getSwitchPortConfig( $name );
            if( KFeatures::checkFeature( KFeatures::SWITCH_DATA_LINK ) )
            {
                $dlconfig = $switchyaml->getDataLinkConfigByPrivatePortName( $name );
            }
            else
            {
                $dlconfig = null;
            }

            if( is_null( $dlconfig ) )
            {
                $swconfig = $switchyaml->getVLANConfigByPortName( $name );
            }
            else
            {
                $swconfig = null;
            }

            if( is_null( $port ) || ( is_null( $dlconfig ) && is_null( $swconfig ) ) )
            {
                return $app->translate( 'gateway-network-interface-error-invalid-config' );
            }

            $intf = $cores[$port['CoreId']]['Interface'];
            if( !is_null( $swconfig ) )
            {
                $vlanintf = "{$intf}.{$swconfig['Id']}";
                $config = $parser->hasInterface( $vlanintf ) ? $parser->getConfiguration( $vlanintf ) : array( );
                $method = isset( $config['inet']['properties']['method'] ) ? $config['inet']['properties']['method'] : 'manual';
                $vlanmode = $swconfig['Mode'];
                $vlanid = $swconfig['Id'];
                unset( $vlanintf );
            }
            else
            {
                $config = array();
                $method = 'data-link';
                $vlanmode = $dlconfig['PrivatePortMode'];
                $vlanid = $dlconfig['Id'];
            }

            $ipv4address = isset( $config['inet']['config']['address'] ) ? $config['inet']['config']['address'] : '';
            $ipv4netmask = isset( $config['inet']['config']['netmask'] ) ? $config['inet']['config']['netmask'] : '';
            $ipv4gateway = '';
            $ipv4gatewaydefault = false;
            if( isset( $config['inet']['config']['x-khomp-gateway'] ) )
            {
                $ipv4gateway = $config['inet']['config']['x-khomp-gateway'];
            }
            else if( isset( $config['inet']['config']['gateway'] ) )
            {
                $ipv4gateway = $config['inet']['config']['gateway'];
                $ipv4gatewaydefault = true;
            }

            $ipv6address = isset( $config['inet6']['config']['address'] ) ? $config['inet6']['config']['address'] : '';
            $ipv6netmask = isset( $config['inet6']['config']['netmask'] ) ? $config['inet6']['config']['netmask'] : '';
            $ipv6gateway = '';
            $ipv6gatewaydefault = false;
            if( isset( $config['inet6']['config']['x-khomp-gateway'] ) )
            {
                $ipv6gateway = $config['inet6']['config']['x-khomp-gateway'];
            }
            else if( isset( $config['inet6']['config']['gateway'] ) )
            {
                $ipv6gateway = $config['inet6']['config']['gateway'];
                $ipv6gatewaydefault = true;
            }

            $extraoptions = KFeatures::checkFeature( KFeatures::SWITCH_DATA_LINK ) ? array( 'data-link' ) : array();
            $methodselect = __networkMethodSelect( $name, $method, $extraoptions );
            $vlanmodecheckbox = __networkVLANModeCheckbox( $name, $vlanmode );

            $ipv4addressinput = __networkInput( $name, 'ipv4-address', $ipv4address, "interface-{$name} method-static" );
            $ipv4netmaskinput = __networkInput( $name, 'ipv4-netmask', $ipv4netmask, "interface-{$name} method-static" );
            $ipv4gatewayinput = __networkInput( $name, 'ipv4-gateway', $ipv4gateway, "interface-{$name} method-static" );
            $ipv4gatewaydefaultcb = __networkGatewayDefaultCheckbox( $name, 'ipv4-gateway-default', $ipv4gatewaydefault, "interface-{$name} method-static ipv4-gateway-default" );

            $ipv6addressinput = __networkInput( $name, 'ipv6-address', $ipv6address, "interface-{$name} method-static" );
            $ipv6netmaskinput = __networkInput( $name, 'ipv6-netmask', $ipv6netmask, "interface-{$name} method-static" );
            $ipv6gatewayinput = __networkInput( $name, 'ipv6-gateway', $ipv6gateway, "interface-{$name} method-static" );
            $ipv6gatewaydefaultcb = __networkGatewayDefaultCheckbox( $name, 'ipv6-gateway-default', $ipv6gatewaydefault, "interface-{$name} method-static ipv6-gateway-default" );

            $vlaninput = __networkInput( $name, 'vlan', $vlanid, 'vlan method-not-manual' );

            $datalinkselect = make_select( "gateway-network-data-link-{$name}", $hwinfo->getSwitchPortNames(), is_null( $dlconfig ) ? '' : $dlconfig['PublicPortName'], false, array( 'class' => "interface-{$name} method-data-link" ) );

            $buf .= "<tr class='interface-{$name}'><td>{$name}<input type='hidden' class='gateway-network-interface' value='{$name}' /></td>";
            $buf .= "<td class='border-right'>{$methodselect}</td>";

            $buf .= "<td class='method-vlan'>{$ipv4addressinput}</td>";
            $buf .= "<td class='method-vlan'>{$ipv4netmaskinput}</td>";
            $buf .= "<td class='method-vlan border-right'>{$ipv4gatewayinput} {$ipv4gatewaydefaultcb}</td>";
            $buf .= "<td class='method-vlan'>{$ipv6addressinput}</td>";
            $buf .= "<td class='method-vlan'>{$ipv6netmaskinput}</td>";
            $buf .= "<td class='method-vlan border-right'>{$ipv6gatewayinput} {$ipv6gatewaydefaultcb}</td>";

            if( KFeatures::checkFeature( KFeatures::SWITCH_DATA_LINK ) )
            {
                $buf .= "<td class='method-data-link border-right' colspan='6'>{$app->translate( 'gateway-network-data-link-public-interface' )} {$datalinkselect}</td>";
            }

            $buf .= "<td class='advanced-options'>{$vlaninput}</td><td class='advanced-options'>{$vlanmodecheckbox}</td>";

            $buf .= '</tr>';
        }
        return "{$buf}</tbody></table></div><div class='floatRight'><span id='gateway-network-advanced-options' class='link'>{$app->translate( 'gateway-network-advanced-options' )}</span></div><br />";
    }

    function showNetworkNoVlan( KGatewayNetwork $parser )
    {
        $app = KSystem::getInstance( );

        $buffer = '<input type="hidden" value="no-vlan" id="gateway-network-configuration-mode" />';
        $ifaces = array_filter( $parser->getInterfaces( ), 'iseth' );
        $total_interfaces = count( $ifaces );

        try
        {
            foreach( $ifaces as $idx => $iface )
            {
                try
                {
                    $config = $parser->getConfiguration( $iface );
                    $macaddress = $parser->getMacAddress( $iface );
                }
                catch( Exception $e )
                {
                    if( $total_interfaces > 1 )
                    {
                        trigger_error( "{$iface}: {$e->getMessage()}", E_USER_NOTICE );
                        continue;
                    }
                    throw $e;
                }

                $inet = array( 'address' => isset( $config['inet']['config']['address'] ) ? $config['inet']['config']['address'] : '', 'netmask' => isset( $config['inet']['config']['netmask'] ) ? $config['inet']['config']['netmask'] : '', 'gateway' => isset( $config['inet']['config']['gateway'] ) ? $config['inet']['config']['gateway'] : '' );
                $inet6 = array( 'address' => isset( $config['inet6']['config']['address'] ) ? $config['inet6']['config']['address'] : '', 'netmask' => isset( $config['inet6']['config']['netmask'] ) ? $config['inet6']['config']['netmask'] : '', 'gateway' => isset( $config['inet6']['config']['gateway'] ) ? $config['inet6']['config']['gateway'] : '' );

                $buffer .= "<div id='gateway-network-{$iface}' class='gateway-network-interface'>";

                if( $total_interfaces > 1 )
                {
                    $buffer .= "<fieldset><legend>{$app->translate( 'gateway-network-interface' )} {$iface} [{$app->translate( 'gateway-network-interface-macaddress' )} {$macaddress}]</legend>";
                }
                else
                {
                    $buffer .= "<fieldset><legend>{$app->translate( 'gateway-network-interface' )} [{$app->translate( 'gateway-network-interface-macaddress' )} {$macaddress}]</legend>";
                }

                $method = 'dhcp';
                if( isset( $config['inet']['properties']['method'] ) )
                {
                    $method = $config['inet']['properties']['method'];
                }
                else if( isset( $config['inet6']['properties']['method'] ) )
                {
                    $method = $config['inet6']['properties']['method'];
                }

                $buffer .= "<div class='gateway-network-method'><label for='gateway-network-method-{$iface}'>{$app->translate( 'gateway-network-method' )}</label>";
                $buffer .= __networkMethodSelect( $iface, $method );
                $buffer .= '</div><br />';
                $buffer .= "<div id='gateway-network-config-dhcp-{$iface}' class='displayNone'>";
                $buffer .= '<table><tbody>';
                $buffer .= '</tbody></table>';
                $buffer .= "</div>";
                $buffer .= "<div id='gateway-network-config-static-{$iface}' class='displayNone'>";
                $buffer .= "<table class='gateway-table-form zebra ipv4'>";
                $buffer .= "<caption>IPv4</caption>";
                $buffer .= "<tr><td class='label'><label for='gateway-network-static-address-{$iface}'>{$app->translate( 'gateway-network-address' )}</label></td><td class='gateway-form-field-container-right'><input id='gateway-network-static-address-{$iface}' class='ipv4' type='text' value='{$inet['address']}' /></td></tr>";
                $buffer .= "<tr><td class='label'><label for='gateway-network-static-netmask-{$iface}'>{$app->translate( 'gateway-network-netmask' )}</label></td><td class='gateway-form-field-container-right'><input id='gateway-network-static-netmask-{$iface}' class='ipv4' type='text' value='{$inet['netmask']}' /></td></tr>";
                $buffer .= "<tr><td class='label'><label for='gateway-network-static-gateway-{$iface}'>{$app->translate( 'gateway-network-gateway' )}</label></td><td class='gateway-form-field-container-right'><input id='gateway-network-static-gateway-{$iface}' class='ipv4' type='text' value='{$inet['gateway']}' /></td></tr>";
                $buffer .= "</table>";
                $buffer .= "<br />";
                $buffer .= "<table class='gateway-table-form zebra ipv6'>";
                $buffer .= "<caption>IPv6</caption>";
                $buffer .= "<tr><td class='label'><label for='gateway-network-static-addressv6-{$iface}'>{$app->translate( 'gateway-network-address' )}</label></td><td class='gateway-form-field-container-right'><input id='gateway-network-static-addressv6-{$iface}' class='ipv6' type='text' maxlength='39' value='{$inet6['address']}' /></td></tr>";
                $buffer .= "<tr><td class='label'><label for='gateway-network-static-netmaskv6-{$iface}'>{$app->translate( 'gateway-network-netmask' )}</label></td><td class='gateway-form-field-container-right'><input id='gateway-network-static-netmaskv6-{$iface}' type='text' size='3' maxlength='3' value='{$inet6['netmask']}' /></td></tr>";
                $buffer .= "<tr><td class='label'><label for='gateway-network-static-gatewayv6-{$iface}'>{$app->translate( 'gateway-network-gateway' )}</label></td><td class='gateway-form-field-container-right'><input id='gateway-network-static-gatewayv6-{$iface}' class='ipv6' type='text' maxlength='39' value='{$inet6['gateway']}' /></td></tr>";
                $buffer .= "</table>";
                $buffer .= "</div>";

                unset( $inet );
                unset( $inet6 );

                $buffer .= "</fieldset><br />";
            }
        }
        catch( KGatewayNetworkException $e )
        {
            $buffer .= $app->translate( $e->getMessage( ) );
        }

        return $buffer;
    }

    function saveNetworkGW( )
    {
        require_once KROOT . '/json_util.php';
        require_once KCONFIG . '/classes/KGatewayHardwareInfo.php';

        $app = KSystem::getInstance( );
        $reply = array( 'success' => false, 'message' => $app->translate( 'gateway-network-error-save' ) );
        $hwinfo = KGatewayHardwareInfo::getInstance( );

        try
        {
            $switchcores = count( $hwinfo->query( 'Switch.Cores' ) );

            if( $switchcores > 0 )
            {
                saveNetworkVlan( $hwinfo );
            }
            else
            {
                saveNetworkNoVlan( $hwinfo );
            }

            $reply = array( 'success' => true, 'message' => $app->translate( 'gateway-network-saved' ) );
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }
        return JSONencode( $reply );
    }

    function saveNetworkVlan( KGatewayHardwareInfo $hwinfo )
    {
        require_once KCONFIG . '/classes/KGatewaySwitch.php';

        $parser = KGatewayNetwork::getInstance( );
        $switchyaml = new KGatewaySwitchYaml();
        $cores = $hwinfo->query( 'Switch.Cores' );

        // build list of current vlan interfaces that should be configured, so
        // we can remove the obsolete/renamed interfaces at the end
        $removeintf = $parser->getInterfaces();
        foreach( $hwinfo->getSwitchPortNames() as $idx => $name )
        {
            $port = $hwinfo->getSwitchPortConfig( $name );
            $swconfig = $switchyaml->getVLANConfigByPortName( $name );
            if( is_null( $port ) || is_null( $swconfig ) )
            {
                throw new KGatewayNetworkConfigException( 'gateway-network-error-invalid-interface' );
            }

            $intf = "{$cores[$port['CoreId']]['Interface']}";
            $vlanintf = "{$intf}.{$swconfig['Id']}";
            if( !in_array( $vlanintf, $removeintf ) )
            {
                $removeintf[] = $vlanintf;
            }
        }

        // update configuration/add interfaces
        foreach( $_POST as $name => &$config )
        {
            // Data-link configuration issue #15195
            if( isset( $config['datalink'] ) )
            {
                if( ! KFeatures::checkFeature( KFeatures::SWITCH_DATA_LINK ) )
                {
                    trigger_error( "This gateway does not support SWITCH_DATA_LINK, ignoring data-link configuration for interface {$name}", E_USER_WARNING );
                    continue;
                }

                $switchyaml->setDataLinkConfig( $config['datalink']['PublicPortName'], $name, $config['datalink']['PrivatePortMode'], $config['datalink']['Id'] );
                if( $switchyaml->hasVLANConfigWithPortName( $name ) )
                {
                    $switchyaml->deleteVLANConfigByPortName( $name );
                }

                // the operating system interfaces configuration will be removed
                continue;
            }
            else if( $switchyaml->hasDataLinkConfigWithPrivatePortName( $name ) )
            {
                $switchyaml->deleteDataLinkConfigByPrivatePortName( $name );
            }

            // Network interface configuration
            $port = $hwinfo->getSwitchPortConfig( $name );
            $core = $cores[$port['CoreId']];
            $intf = $core['Interface'];
            $vlanintf = "{$intf}.{$config['vlan']['id']}";

            if( ! empty( $config['iface']['inet']['properties'] ) )
            {
                $parser->setProperties( $vlanintf, $config['iface']['inet']['properties'], isset( $config['iface']['inet']['config'] ) ? $config['iface']['inet']['config'] : array() );

                //TODO: setProperties already set the configuration propperly
                if ( !empty( $config[ 'iface' ][ 'inet' ][ 'config' ] ) )
                {
                    $parser->setConfiguration( $vlanintf, $config[ 'iface' ][ 'inet' ][ 'config' ], $config[ 'iface' ][ 'inet' ][ 'properties' ][ 'method' ], $config[ 'iface' ][ 'inet' ][ 'properties' ][ 'family' ]);
                }
            }
            if( ! empty( $config['iface']['inet6']['properties'] ) )
            {
                $parser->setProperties( $vlanintf, $config['iface']['inet6']['properties'], isset( $config['iface']['inet6']['config'] ) ? $config['iface']['inet6']['config'] : array() );

                //TODO: setProperties already set the configuration propperly
                if ( !empty( $config[ 'iface' ][ 'inet6' ][ 'config' ] ) )
                {
                    $parser->setConfiguration( $vlanintf, $config[ 'iface' ][ 'inet6' ][ 'config' ], $config[ 'iface' ][ 'inet6' ][ 'properties' ][ 'method' ], $config[ 'iface' ][ 'inet6' ][ 'properties' ][ 'family' ]);
                }
            }
            $switchyaml->setVLANConfig( $name, $config['vlan']['id'], $config['vlan']['mode'] );

            $pos = array_search( $vlanintf, $removeintf );
            if( $pos !== false )
            {
                unset( $removeintf[$pos] );
            }
        }

        // remove obsolete/renamed interfaces
        foreach( $removeintf as $intf )
        {
            $parser->removeInterface( $intf );
        }

        $switchyaml->save( );
        $parser->save( );
    }

    function saveNetworkNoVlan( KGatewayHardwareInfo $hwinfo )
    {
        $parser = KGatewayNetwork::getInstance( );
        $ifaces = array();

        // interfaces configuration
        foreach( $_POST['ifaces'] as $iface => &$familyData )
        {
            foreach( $familyData as $family => &$data )
            {
                $config = array_key_exists( 'config', $data ) ? $data['config'] : array( );
                if( empty( $data['properties'] ) )
                {
                    trigger_error( "Interface {$iface} has no properties", E_USER_WARNING );
                    continue;
                }
                if( $family == 'inet6' && $data['properties']['method'] == 'dhcp' )
                {
                    trigger_error( "Changing {$iface} [{$family}] method from {$data['properties']['method']} to auto", E_USER_NOTICE );
                    $data['properties']['method'] = 'auto';
                }
                $parser->setProperties( $iface, $data['properties'], $config );
                $ifaces[] = $iface;
            }
        }

        // probably aliases and ebs network, configured by interfaces.header
        // file
        $ifacesremove = array_diff( $parser->getInterfaces(), $ifaces );
        foreach( $ifacesremove as $name )
        {
            $parser->removeInterface( $name );
        }

        $parser->save( );
    }

    function showVoIPOptionsGW( )
    {
        return showVoIPOptions( $_REQUEST, 'GW' );
    }

    function showVoIPTLSGW( )
    {
        return showVoIPTLS( $_REQUEST, 'GW' );
    }

    function getHostName_( )
    {
        $hostname = exec( "hostname -s" );
        return JSONencode( $hostname );
    }
