<?php
    require_once KCONFIG . '/classes/classKGatewayUtil.php';
    require_once KCONFIG . '/classes/KGatewayYaml.php';

    $AjaxFunctions['showNAPsGW'] = 'showNAPsGW';
    $AjaxFunctions['newNAP'] = 'newNAP';
    $AjaxFunctions['printNAPType'] = 'printNAPType';
    $AjaxFunctions['saveNAP'] = 'saveNAP';
    $AjaxFunctions['removeNAP'] = 'removeNAP';
    $AjaxFunctions['editNAP'] = 'editNAP';
    $AjaxFunctions['getServiceProvider'] = 'getServiceProvider';
    $AjaxFunctions['showSimCardsGW'] = 'showSimCardsGW';
    $AjaxFunctions['saveSimCards'] = 'saveSimCards';
    $AjaxFunctions['deleteSimCard'] = 'deleteSimCard';
    $AjaxFunctions['getConfiguredSimCards'] = 'getConfiguredSimCards';

    function showSimCardsGW( )
    {
        $app = KSystem::getInstance();
        $yaml = new KGatewayYaml( );

        $buf = "<fieldset><legend>{$app->translate( 'gateway-nap-gsm-sim-cards' )}</legend>";
        $buf .= "<div id='gateway-gsm-sim-cards'><table id='gateway-gsm-sim-cards-table' class='gateway-table zebra sortable'><thead><tr>";
        $buf .= "<th>{$app->translate( 'gateway-nap-gsm-sim-card' )}</th><th>{$app->translate( 'gateway-nap-gsm-sim-card-number' )}</th><th>NAP</th><th>{$app->translate( 'options' )}</th>";
        $buf .= '</tr></thead><tbody></tbody></table><br />';
        $buf .= "<input type='button' value='{$app->translate( 'gateway-nap-gsm-sim-cards-add-manually' )}' id='gateway-gsm-sim-cards-add' /><br /><br />";
        $buf .= "<div class='textAlignRight'><input type='button' value='{$app->translate( 'save' )}' id='gateway-gsm-sim-cards-save' /></div>";
        $buf .= '</div></fieldset>';

        // avaliable naps select template
        $buf .= '<select class="displayNone" id="gateway-gsm-sim-cards-nap-tpl">';
        $buf .= "<option value='' class='gateway-nap-gsm-sim-card-not-assigned' selected>{$app->translate( 'gateway-nap-gsm-sim-card-not-assigned' )}</option>";
        foreach( $yaml->getAllNaps() as $nap )
        {
            if( $nap['Type'] == 'GSM' )
            {
                $name = htmlspecialchars( $nap['Name'], ENT_QUOTES );
                $buf .= "<option value='{$name}'>{$name}</option>";
            }
        }
        $buf .= '</select>';
        return $buf;
    }

    function getConfiguredSimCards( )
    {
        require_once KROOT . '/json_util.php';

        $yaml = new KGatewayYaml( );

        $sims = array( );

        foreach( $yaml->getAllSIMCards() as $sim )
        {
            $nap = $yaml->getSIMCardNap( $sim['ICCID'] );
            if( is_null( $nap ) )
            {
                $sim['NAP'] = null;
            }
            else
            {
                $sim['NAP'] = $nap['Name'];
            }
            $sims[] = $sim;
        }
        return JSONencode( $sims );
    }

    function saveSimCards( )
    {
        require_once KROOT . '/json_util.php';
        $app = KSystem::getInstance( );
        $yaml = new KGatewayYaml( );
        $ret = array( 'message' => $app->translate( 'gateway-nap-save-error' ), 'success' => false );

        try
        {
            foreach( $yaml->getAllNaps( ) as $nap )
            {
                if( $nap['Type'] == 'GSM' )
                {
                    $nap['SIMs'] = array();
                    $yaml->setNap( $nap, $nap['Name'] );
                }
            }

            foreach( $_POST['SIMs'] as $sim )
            {
                if( $sim['NAP'] )
                {
                    $nap = $yaml->getNap( $sim['NAP'] );
                    if( is_null( $nap ) )
                    {
                        throw new Exception( 'gateway-nap-error-not-found' );
                    }
                    else if( $nap['Type'] != 'GSM' )
                    {
                        throw new Exception( 'gateway-nap-error-gsm-type-expected' );
                    }
                    $nap['SIMs'][] = $sim['ICCID'];
                    $yaml->setNap( $nap, $nap['Name'] );
                }
                unset( $sim['NAP'] );
                $yaml->setSIMCard( $sim );
            }
            $yaml->save( );
            $ret = array( 'message' => $app->translate( 'gateway-nap-gsm-sim-cards-saved' ), 'success' => true );
        }
        catch( Exception $e )
        {
            $ret = array( 'message' => $app->translate( $e->getMessage( ) ), 'success' => false );
        }
        return JSONencode( $ret );
    }

    function deleteSimCard( )
    {
        require_once KROOT . '/json_util.php';
        $app = KSystem::getInstance( );
        $yaml = new KGatewayYaml( );
        $ret = array( 'message' => $app->translate( 'gateway-nap-error-gsm-sim-card-delete' ), 'success' => false );

        try
        {
            $yaml->deleteSimCard( $_POST['ICCID'] );
            $yaml->save( );
            $ret = array( 'message' => $app->translate( 'gateway-nap-gsm-sim-card-deleted' ), 'success' => true );
        }
        catch( Exception $e )
        {
            $ret = array( 'message' => $app->translate( $e->getMessage( ) ), 'success' => false );
        }
        return JSONencode( $ret );
    }

    function showNAPsGW( )
    {
        $app = KSystem::getInstance( );

        $kyaml = new KGatewayYaml( );
        $yaml = $kyaml->getYaml( );
        $buffer = '';

        $buffer .= "<fieldset><legend>{$app->translate( 'gateway-nap' )}</legend>";
        $buffer .= "<div id='NAPsDivForm'>";
        $buffer .= "<input type='button' value='{$app->translate( 'gateway-nap-new' )}' onClick='newNAP();' /><br /><br />";

        if( sizeof((array_values( $yaml['NAPs'] ))) > 0 )
        {

            $buffer .= "<table id='NAPsTable' class='zebra gateway-table'><thead><tr>";
            $buffer .= "<th>{$app->translate( 'name' )}</th>";
            $buffer .= "<th>{$app->translate( 'type' )}</th>";
            $buffer .= "<th>{$app->translate( 'gateway-profile' ) }</th>";
            $buffer .= "<th class='gateway-table-options'>{$app->translate( 'options' )}</th></tr></thead><tbody>";
            foreach( $yaml['NAPs'] as $nap )
            {
                $p = isset( $nap['Profile'] ) && $nap['Profile'] ? $nap['Profile'] : "<i>{$app->translate( 'gateway-nap-use-default-profile' )}</i>";
                $ename = urlencode( $nap['Name'] );
                $buffer .= "<tr>";
                $buffer .= "<td>{$nap['Name']}</td>";
                $buffer .= "<td>{$nap['Type']}</td>";
                $buffer .= "<td>{$p}</td>";
                $buffer .= "<td class='gateway-table-options'><a href='#' onClick=\"editNAP('{$ename}');\"><img border='0' src='/imagens/edit.png' title='{$app->translate( 'edit' )}' /></a><a href='#' onClick=\"removeNAP('{$ename}');\"><img border='0' src='/imagens/remove.png'  title='{$app->translate( 'delete' )}' /></a></td>";
                $buffer .= "</tr>";
            }
            $buffer .= "</tbody></table>";
        }
        else
        {
            $buffer .= $app->translate( 'gateway-nap-empty-list' );
        }

        $buffer .= '</div>';
        $buffer .= '</fieldset>';

        return $buffer;
    }

    //TODO: Validate all incomming data
    //TODO: move all logic to KGatewayYaml class
    function saveNAP( )
    {
        require_once KROOT . '/json_util.php';
        $app = KSystem::getInstance( );

        $reply = array( 'success' => false, 'message' => $app->translate( 'gateway-nap-save-error' ) );

        try
        {
            $kyaml = new KGatewayYaml( );
            $yaml = $kyaml->getYaml( );

            $renamed = false;
            $duplicated = false;
            $duplicatedRegister = false;
            // if true, the yaml can be saved, so change to false to report
            // errors
            $canSave = true;
            $new = strlen( $_POST['oldName'] ) == 0;
            $myname = $_POST['Name'];

            if( !$new && $_POST['oldName'] != $_POST['Name'] )
            {
                $renamed = true;
                $myname = $_POST['oldName'];
            }
            if( $new || $renamed )
            {
                // if it is new or renamed, check if $_POST['name'] already
                // exists -- duplicate key check
                foreach( $yaml['NAPs'] as &$_nap )
                {
                    if( $_nap['Name'] == $_POST['Name'] )
                    {
                        $duplicated = true;
                        $canSave = false;
                        break;
                    }
                }
            }
            if( isset( $_POST['Register'] ) && $_POST['Register'] == 'true' )
            {
                foreach( $yaml['NAPs'] as &$_nap )
                {
                    if( $_nap['Name'] == $myname || $_nap['Type'] != 'SIP' || !$_nap['Register'] )
                    {
                        continue;
                    }
                    if( $_nap['Domain'] == $_POST['Domain'] && $_nap['DomainPort'] == $_POST['DomainPort'] && $_nap['AuthorizationUser'] == $_POST['AuthorizationUser'] )
                    {
                        $duplicatedRegister = true;
                        $canSave = false;
                        break;
                    }
                }
            }
            if( $duplicated )
            {
                $reply['message'] = $app->translate( 'gateway-nap-save-error-duplicated' );
            }
            elseif( $duplicatedRegister )
            {
                $reply['message'] = $app->translate( 'gateway-nap-save-error-duplicated-register' );
            }
            else
            {
                $missing = true;
                if( $new )
                {
                    // append the new
                    $nap = array( 'Name' => $_POST['Name'], 'Type' => $_POST['Type'] );
                    $yaml['NAPs'][] = &$nap;
                    $missing = false;
                }
                else if( $renamed )
                {
                    // update
                    foreach( $yaml['NAPs'] as &$nap )
                    {
                        if( $nap['Name'] == $_POST['oldName'] )
                        {
                            $nap['Name'] = $_POST['Name'];
                            $missing = false;
                            break;
                        }
                    }
                    if( !$missing )
                    {
                        // update any route that referrer to the old name
                        foreach( $yaml['Routes'] as &$_route )
                        {
                            if( $_route['OriginNAP'] == $_POST['oldName'] )
                            {
                                $_route['OriginNAP'] = $nap['Name'];
                            }
                            if( $_route['DestinationNAP'] == $_POST['oldName'] )
                            {
                                $_route['DestinationNAP'] = $nap['Name'];
                            }
                        }
                        foreach( $yaml['RoutesSMS'] as &$_route )
                        {
                            if( $_route['DestinationNAP'] == $_POST['oldName'] )
                            {
                                $_route['DestinationNAP'] = $nap['Name'];
                            }
                        }
                    }
                }
                else
                {
                    // update *only*
                    foreach( $yaml['NAPs'] as &$nap )
                    {
                        if( $nap['Name'] == $_POST['Name'] )
                        {
                            $missing = false;
                            break;
                        }
                    }
                }

                //POG: issue #8895
                //     This shouldn't be needed, but the YAML manipualtion is
                //     poorly implemented in this function
                $appendSimCard = array();
                if( $missing )
                {
                    $reply['message'] = $app->translate( 'gateway-nap-error-not-found' );
                    $canSave = false;
                }
                else
                {
                    if( isset( $_POST['Profile'] ) )
                    {
                        $nap['Profile'] = $_POST['Profile'];
                    }
                    elseif( isset( $nap['Profile'] ) )
                    {
                        unset( $nap['Profile'] );
                    }

                    if( isset( $_POST['ChannelAllocator'] ) )
                    {
                        $nap['ChannelAllocator'] = $_POST['ChannelAllocator'];
                    }
                    else if( isset( $nap['ChannelAllocator'] ) )
                    {
                        unset( $nap['ChannelAllocator'] );
                    }

                    // type specific data
                    switch( $nap['Type'] )
                    {
                        case 'SIP':
                            $nap['Register'] = $_POST['Register'] == 'true';
                            $nap['Domain'] = $_POST['Domain'];
                            $nap['DomainPort'] = $_POST['DomainPort'];
                            $nap['DomainUseIpPacketAddress'] = $_POST['DomainUseIpPacketAddress'] == 'true';
                            $nap['IgnoreSourcePort'] = $_POST['IgnoreSourcePort'] == 'true';
                            $nap['UseCalledFromRequestURI'] = $_POST['UseCalledFromRequestURI'] == 'true';
                            $nap['AuthorizationUser'] = $_POST['AuthorizationUser'];
                            $nap['SipTransportType'] = $_POST['SipTransportType'];
                            $nap['AudioTransportType'] = $_POST['AudioTransportType'];
                            $nap['Expires'] = $_POST['Expires'];
                            $nap['Proxy'] = $_POST['Proxy'];
                            $nap['ProxyPort'] = $_POST['ProxyPort'];
                            $nap['Username'] = $_POST['Username'];
                            $nap['Password'] = $_POST['Password'];
                            if( isset( $_POST['ForceSipInterface'] ) )
                            {
                                $nap['ForceSipInterface'] = $_POST['ForceSipInterface'];
                                $nap['ForceRtpInterface'] = $_POST['ForceRtpInterface'];
                            }
                            $nap['NATTraversalMode'] = $_POST['NATTraversalMode'];
                            switch( $nap['NATTraversalMode'] )
                            {
                                case 'STUN':
                                    $nap['StunServer'] = $_POST['StunServer'];
                                    $nap['StunServerPort'] = $_POST['StunServerPort'];
                                break;
                                case 'EXTERNAL_IP_ADDRESSES':
                                    $nap['ExternalSipAddress'] = $_POST['ExternalSipAddress'];
                                    $nap['ExternalRtpAddress'] = $_POST['ExternalRtpAddress'];
                                break;
                            }
                            $nap['KeepAliveInterval'] = $_POST['KeepAliveInterval'];
                            $nap['KeepAlive'] = isset( $_POST['KeepAliveInterval'] ) && $_POST['KeepAliveInterval'] != '0';
                            break;
                        case 'Trunk':
                            $nap['Links'] = array( );
                            foreach( $_POST['Links'] as $link )
                            {
                                $nap['Links'][] = str_replace( '-', '.', $link );
                            }

                            break;
                        case 'GSM':
                            $nap['MonthlyMinutes']          = $_POST['MonthlyMinutes'];
                            $nap['MonthlySms']              = $_POST['MonthlySms'];
                            $nap['MinutesToSwap']           = $_POST['MinutesToSwap'];
                            $nap['ContractDay']             = $_POST['ContractDay'];
                            $nap['SharedMinutes']           = $_POST['SharedMinutes'] == 'true';
                            $nap['TimeTrackPattern']        = $_POST['TimeTrackPattern'];
                            $nap['SIMs'] = array();
                            if( isset( $_POST['SIMs'] ) )
                            {
                                foreach( $_POST['SIMs'] as $sim )
                                {
                                    $nap['SIMs'][] = $sim;
                                    //issue #8895
                                    if( !$kyaml->hasSIMCard( $sim ) )
                                    {
                                        //POG: The correct way of doing this is
                                        //     commented bellow and should be fixed
                                        //     when the NAP manipulation logic got
                                        //     moved to KGatewayYaml class
                                        $appendSimCard[] = $sim;
                                        //$kyaml->setSIMCard( array( 'ICCID' => $sim, 'Number' => '' ) );
                                    }
                                }
                            }


                            break;
                        /*
                         case 'FXO':
                         $nap['Channels'] = array( );
                         foreach( $_POST['Channels'] as $channel )
                         {
                         $nap['Channels'][] = str_replace( '-', '.', $channel );
                         }
                         break;
                         case 'FXS':
                         $nap['Branches'] = array( );
                         foreach( $_POST['Branches'] as $dev=>$branch )
                         {
                         $dev = str_replace( '-', '.', $dev );
                         $nap['Branches'][$dev] = $branch;
                         }
                         break;
                         */
                        default:
                            $canSave = false;
                            break;
                    }
                }

                if( $canSave )
                {
                    $kyaml->setYaml( $yaml );
                    //POG: issue #8895
                    //     This shouldn't be here, but it is needed as setYaml
                    //     will erase any change done to the $kyaml object's
                    //     private YAML array
                    foreach( $appendSimCard as $sim )
                    {
                        $kyaml->setSIMCard( array( 'ICCID' => $sim, 'Number' => '' ) );
                    }
                    $kyaml->save( );
                    $reply = array( 'success' => true, 'message' => $app->translate( 'gateway-nap-saved' ) );
                }
            }
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ));
        }
        return JSONencode( $reply );
    }

function removeNAP( )
    {
        require_once KROOT . '/json_util.php';
        $app = KSystem::getInstance( );

        $reply = array( 'success' => false, 'message' => $app->translate( 'gateway-nap-delete-error' ) );

        $name = isset( $_POST['name'] ) ? urldecode( $_POST['name'] ) : null;
        if( !is_null( $name ) )
        {
            try
            {
                $kyaml = new KGatewayYaml( );
                $yaml = $kyaml->getYaml( );
                foreach( $yaml['NAPs'] as $idx => $nap )
                {
                    if( $nap['Name'] == $name )
                    {
                        $inUse = false;
                        foreach( $yaml['Routes'] as $route )
                        {
                            if( $route['OriginNAP'] == $nap['Name'] || $route['DestinationNAP'] == $nap['Name'] )
                            {
                                $inUse = true;
                                break;
                            }
                        }
                        foreach( $yaml['RoutesSMS'] as $routesms )
                        {
                            if( $routesms['DestinationNAP'] == $nap['Name'] )
                            {
                                $inUse = true;
                                break;
                            }
                        }
                        if( $inUse )
                        {
                            $reply['message'] = $app->translate( 'gateway-nap-delete-error-inuse' );
                        }
                        else
                        {
                            unset( $yaml['NAPs'][$idx] );
                            $kyaml->setYaml( $yaml );
                            //$kyaml->saveYaml( );
                            $kyaml->save( );

                            $reply = array( 'success' => true, 'message' => $app->translate( 'gateway-nap-deleted' ) );
                        }
                        break;
                    }
                }
            }
            catch( Exception $e )
            {
                $reply['message'] = $app->translate( $e->getMessage( ) );
            }
        }
        return JSONencode( $reply );
    }

function newNAP( )
    {
        require_once KCONFIG . '/classes/KGatewayYaml.php';

        return printNAPForm( new KGatewayYaml( ) );
    }

    function editNAP( )
    {
        $name = isset( $_POST['name'] ) ? urldecode( $_POST['name'] ) : null;
        $app = KSystem::getInstance( );

        if( !is_null( $name ) )
        {
            $kyaml = new KGatewayYaml( );
            $yaml = $kyaml->getYaml( );
            foreach( $yaml['NAPs'] as &$nap )
            {
                if( $nap['Name'] == $name )
                {
                    return printNAPForm( $kyaml, $nap );
                }
            }
        }
        return $app->translate( 'gateway-nap-error-not-found' );
    }

    function getServiceProvider()
    {
        $ret = array();
        $ret["success"] = false;
        if( !isset( $_POST['imsi'] ) )
        {
            $ret["message"] = "gateway-service-provider-invalid-params";
        }
        else
        {
            $imsi = $_POST['imsi'];
            $kyaml = new KGatewayYaml( );
            $yaml = $kyaml->getYaml( );
            if( strlen($imsi) > 0 )
            {
                foreach( $yaml["MobileServiceProviders"] as &$prov )
                {
                    $mccmnc = $prov['MCCMNC'];
                    if( substr_compare($imsi, $mccmnc, 0, strlen($mccmnc)) == 0)
                    {
                        $ret["success"] = true;
                        $ret["service_provider"] = $prov['Name'];
                        break;
                    }
                }
            }

        }
        return json_encode($ret);
    }

    function printNAPForm( KGatewayYaml $yaml, $data = array() )
    {
        $app = KSystem::getInstance( );
        $profiles = $yaml->getKGatewayProfileYaml( );

        $_napName = array_key_exists( 'Name', $data ) ? htmlspecialchars( $data['Name'], ENT_QUOTES ) : '';

        $buffer = '';
        $buffer .= "<input type='hidden' value='{$_napName}' id='gateway-nap-old-name' />";
        $buffer .= "<table class='zebra simpleTable gateway-table-form textAlignLeft'>";
        $buffer .= "<tr><td class='label'><label for='gateway-nap-name'>{$app->translate( 'NAPName' )}</label></td><td style='width:50%;text-align:right;'><input type='text' value='{$_napName}' id='gateway-nap-name' /></td></tr>";
        $buffer .= "<tr><td class='label'><label for='gateway-nap-profile'>{$app->translate( 'gateway-profile' )}</label></td><td style='width:50%;text-align:right;'><select id='gateway-nap-profile' name='Profile'>";
        $_profileName = isset( $data['Profile'] ) ? $data['Profile'] : null;
        $buffer .= "<option value=''>{$app->translate( 'gateway-nap-use-default-profile' )}</option>";
        foreach( $profiles->getAllProfiles() as $profile )
        {
            $s = $_profileName === $profile['Name'] ? ' selected' : '';
            $profileName = htmlspecialchars( $profile['Name'] );
            $buffer .= "<option value='{$profileName}'{$s}>{$profileName}</option>";
        }
        $buffer .= "</select></td></tr>";
        if( empty( $_napName ) )
        {
            $buffer .= "<tr><td class='label'><label for='gateway-nap-type'>{$app->translate( 'Type' )}</label></td><td style='width:50%;text-align:right;'><select id='gateway-nap-type' onChange='changeNAPType();'>";
            $types = getNAPTypes( $yaml );
            $buffer .= "<option value='<SELECT>'>{$app->translate( 'Select' )}</option>";

            foreach( $types as $t )
            {
                if( array_key_exists( 'Type', $data ) && $t == $data['Type'] )
                {
                    $buffer .= "<option value='{$t}' selected='selected'>{$t}</option>";
                }
                else
                {
                    $buffer .= "<option value='{$t}'>{$t}</option>";
                }
            }
            $buffer .= '</select></td></tr>';
        }
        else
        {
            $_type = $data['Type'];
            $buffer .= "<tr><td class='label'>{$app->translate( 'Type' )}</td><td class='gateway-form-field-container-right'><input type='hidden' id='gateway-nap-type' value='{$_type}' />{$_type}</td></tr>";
        }

        $buffer .= '</table><br />';
        $buffer .= '<div id="gateway-nap"></div>';

        $buffer .= "<br /><div class='textAlignRight'>";
        $buffer .= "<input type='button' onclick='backNAPs();' value='{$app->translate( 'back' )}' /> <input type='button' onclick='saveNAP();' value='{$app->translate( 'save' )}' /></div>";

        return $buffer;
    }

    function printNAPType( )
    {
        $app = KSystem::getInstance( );
        $type = isset( $_POST['type'] ) ? $_POST['type'] : NULL;

        if( !isset( $type ) )
            return;

        $yaml = new KGatewayYaml( );
        $_yaml = $yaml->getYaml( );

        $nap = null;
        $name = isset( $_POST['name'] ) ? $_POST['name'] : null;

        if( $name )
        {
            $nap = searchNAPValue( $name, $_yaml );
        }
        if( is_null( $nap ) )
        {
            $nap = array( );
        }

        switch($type)
        {
            case 'Trunk':
                return printNewNAPTrunk( $nap, $_yaml );
                break;
            case 'SIP':
                return printNewNAPSIP( $nap, $_yaml );
                break;
            case 'GSM':
                return printNewNAPGSM( $nap, $yaml );
                break;
            /*
             case 'FXO':
             return printNewNAPFXO( $nap, $_yaml );
             break;
             case 'FXS':
             return printNewNAPFXS( $nap, $_yaml );
             break;
             case 'FXSGroup':
             return printNewNAPFXSGroup( $nap, $_yaml );
             break;
             */
            case '<SELECT>':
                return $app->translate( 'gateway-nap-select-type' );
            default:
                return $app->translate( 'gateway-nap-unsupported-type' );
                break;
        }
        return $form_buffer;
    }

    function printNewNAPGSM( $nap, KGatewayYaml $yaml )
    {
        global $sistema;
        $app = KSystem::getInstance( );

        $devGSM = $sistema->getSpecificDevice( 'GSM' );

        $buffer = '';

        /// Opcoes
        $buffer .= "<fieldset><legend>{$app->translate( 'options' )}</legend>";
        $buffer .= '<table class="zebra simpleTable gateway-table-form textAlignLeft">' . getNAPAllocationAlgorithmHtml( $nap ) . '</table>';
        $buffer .= "</fieldset><br />";

        $selected_sims = isset( $nap["SIMs"] ) ? $nap["SIMs"] : array();
        $unavailable = array_diff(getAllSimCardsInUse(), $selected_sims );
        $unavailable = array_values($unavailable);

        $selected_sims = json_encode($selected_sims);
        $unavailable   = json_encode($unavailable);
        $configured_sims = json_encode( $yaml->getAllSIMCardICCIDs( ) );

        $buffer .= "<fieldset><legend>{$app->translate('gateway-nap-gsm-sim-cards')}</legend>";

        $buffer .= '<table class=\'simpleTable gateway-table-form\'><tbody>';
        $buffer .= '<tr>';
        $current = isset( $nap["MinutesToSwap"] ) ? $nap["MinutesToSwap"] : KGatewayYaml::GSMNAP_SIM_CARD_SWAP_DISABLED;
        $checked = $current == KGatewayYaml::GSMNAP_SIM_CARD_SWAP_DISABLED ? '' : ' checked=\'checked\'';
        $buffer .= "<td class='label'><input id='gateway-nap-gsm-minutes-swap-enabled' type='checkbox' {$checked}/><label for='gateway-nap-gsm-minutes-swap'>{$app->translate('gateway-nap-gsm-minutes-swap')}</label></td>";
        $buffer .= "<td class='textAlignLeft'><input id='gateway-nap-gsm-minutes-swap' type='text' value='{$current}' class='displayNone'></td>";
        $buffer .= '</tr>';
        $buffer .= '</tbody></table>';

        $buffer .= "<div id='gateway-nap-sim-card-list'></div><br/>";
        $buffer .= "</fieldset>";
        $buffer .= <<<EOS
        <script type='text/javascript'>
          (function ($){
            SimCardList( $('#gateway-nap-sim-card-list'), {$selected_sims}, {$unavailable}, {$configured_sims} );
          })(jQuery);
        </script>
EOS;

        $checked = isset( $nap['TimeTrackPattern'] ) && $nap['TimeTrackPattern'] ? ' checked=\'checked\'' : '';
        $buffer.= "<fieldset><legend><input type='checkbox' id='gateway-nap-gsm-enable-track' {$checked}/><label for='gateway-nap-gsm-enable-track'>{$app->translate('gateway-nap-gsm-track')}</label></legend>";
        $buffer.= '<table id="gateway-nap-gsm-track-table" class="zebra simpleTable gateway-table-form displayNone">';
        $buffer.= '<tbody>';

        $buffer.= '<tr>';
        $buffer.= "<td class='label'><label for='gateway-nap-gsm-monthly-minutes'>{$app->translate('gateway-nap-gsm-monthly-minutes')}</label></td>";
        $current = isset( $nap["MonthlyMinutes"] ) ? $nap["MonthlyMinutes"] : "";
        $buffer.= "<td><input id='gateway-nap-gsm-monthly-minutes' type='text' value='{$current}'></td>";
        $buffer.= '</tr>';

        $buffer.= '<tr>';
        $buffer.= "<td class='label'><label for='gateway-nap-gsm-monthly-sms'>{$app->translate('gateway-nap-gsm-monthly-sms')}</label>:</td>";
        $current = isset( $nap["MonthlySms"] ) ? $nap["MonthlySms"] : "";
        $buffer.= "<td><input id='gateway-nap-gsm-monthly-sms' type='text' value='{$current}'></td>";
        $buffer.= '</tr>';

        $buffer.= '<tr>';
        $buffer.= "<td class='label'><label for='gateway-nap-gsm-shared-minutes'>{$app->translate('gateway-nap-gsm-shared-minutes')}</label></td>";
        $current = isset( $nap['SharedMinutes'] ) && $nap['SharedMinutes'] ? "checked": "";
        $buffer.= "<td><input id='gateway-nap-gsm-shared-minutes' type='checkbox' $current></td>";
        $buffer.= '</tr>';

        $buffer.= '<tr>';
        $buffer.= "<td class='label'><label for='gateway-nap-gsm-contract-day'>{$app->translate('gateway-nap-gsm-contract-day')}</label></td>";
        $current = isset( $nap["ContractDay"] ) ? $nap["ContractDay"] : KGatewayYaml::GSMNAP_CONTRACT_RENEW_DISABLED;
        $contractDays = array( KGatewayYaml::GSMNAP_CONTRACT_RENEW_DISABLED => $app->translate( 'gateway-nap-gsm-contract-renew-disabled' ) );
        for( $i = 1; $i < 32; ++$i )
        {
            $contractDays[$i] = $i;
        }
        $buffer.= '<td>'. make_select( 'gateway-nap-gsm-contract-day', $contractDays, $current, true ) . '</td>';
        unset( $contractDays );
        $buffer.= '</tr>';

        $buffer.= '<tr>';
        $buffer.= '<td class=\'label\'><label for="gateway-nap-gsm-time-track-pattern-option">'. $app->translate('gateway-nap-gsm-billing-option'). '</label></td>';
        $current = isset( $nap['TimeTrackPattern'] ) ? $nap['TimeTrackPattern'] : '';
        $buffer.= '<td>'. make_select( 'gateway-nap-gsm-time-track-pattern-option', $yaml->getAllTimeTrackPatternsNames(), $current ) . '</td>';
        $buffer.= '</tr>';
        $buffer.= '</tbody>';
        $buffer.= '</table>';
        $buffer.= '</fieldset>';

        return $buffer;
    }

    function printNewNAPFXO( $nap, $yaml )
    {
        global $sistema;
        $app = KSystem::getInstance( );

        $buffer = '';
        $devFXO = $sistema->getSpecificDevice( 'FXO' );

        $buffer .= "{$app->translate( 'NAPChannels' )}<br /><table border='0'>";

        $_mylinks = array( );
        if( array_key_exists( 'Channels', $nap ) )
        {
            // OUTCH!!! YAML CAST DOTTED ELEMENTS TO FLOAT!!!
            // This ugly cast is to ensure that link ids (device.link)
            // are in string form and not as float
            foreach( $nap['Channels'] as $link )
            {
                $_mylinks[] = "{$link}";
            }
        }

        $_napName = array_key_exists( 'Name', $nap ) ? $nap['Name'] : '';

        foreach( $devFXO as $dev )
        {
            $buffer .= "<tr><td>Serial: {$dev}</td>";
            for( $i = 0; $i < 12; $i++ )
            {
                // html id
                $channelId = "{$dev}-{$i}";
                // device id
                $channelIdDev = "{$dev}.{$i}";
                if( in_array( $channelIdDev, $_mylinks ) )
                {
                    $extra = ' checked=\'true\'';
                }
                elseif( _isFXOChannelInUse( $yaml, $channelIdDev ) )
                {
                    $extra = ' disabled=\'disabled\'';
                }
                else
                {
                    $extra = '';
                }
                $buffer .= "<td>$i<br/><input id='{$channelId}' class='gateway-nap-fxo-channel' type='checkbox' {$extra}/></td>";
            }
        }
        $buffer .= '</tr>';
        $buffer .= '</table>';

        return $buffer;
    }

    function printNewNAPTrunk( $nap, $yaml )
    {
        global $sistema;
        $app = KSystem::getInstance( );
        $buffer = '';

        $configDir = $app->getConfigDir( );
        $devYaml = new KYaml( "{$app->getConfigDir()}/devices.yaml" );

        $devE1andT1 = $sistema->getSpecificDevice( array( 'T1', 'E1' ) );

        $_mylinks = array( );
        if( array_key_exists( 'Links', $nap ) )
        {
            // OUTCH!!! YAML CAST DOTTED ELEMENTS TO FLOAT!!!
            // This ugly cast is to ensure that link ids (device.link)
            // are in string form and not as float
            foreach( $nap['Links'] as $link )
            {
                $_mylinks[] = "{$link}";
            }
        }
        $_napName = array_key_exists( 'Name', $nap ) ? $nap['Name'] : '';

        /// Opcoes
        $buffer .= "<fieldset><legend>{$app->translate( 'options' )}</legend>";
        $buffer .= '<table class="zebra simpleTable gateway-table-form textAlignLeft">' . getNAPAllocationAlgorithmHtml( $nap ) . '</table>';
        $buffer .= "</fieldset><br />";

        foreach( $devE1andT1 as $e1 )
        {
            $dev = $devYaml->getValue( $e1 );
            $linksCount = sizeof( $dev['Links'] );

            $buffer .= "<fieldset><legend>{$app->translate( 'Device' )}: {$e1}</legend><table border='0' class='zebra simpleTable gateway-nap-trunk-device'>";

            for( $i = 0; $i < $linksCount; $i++ )
            {
                $channelsAllocatedByAnotherNap = 0;
                $channelsAllocatedByCurrentNap = 0;
                // yaml node reference
                $link = &$dev['Links'][$i][$i];
                // html id
                $idLink = "{$e1}-{$i}";
                // device id
                $idLinkDev = "{$e1}.{$i}";

                $buff = "<div class='gateway-nap-trunk-channels' id='channels-link-{$idLink}'><ul>";
                $total_channels = ( $link[ 'Type' ] === 'T1' ) ? 24 : 30;

                for( $l = 0; $l < $total_channels; $l++ )
                {
                    $idChannel = "{$idLink}-{$l}";
                    $idChannelDev = "{$idLinkDev}.{$l}";

                    if( in_array( $idChannelDev, $_mylinks, true ) )
                    {
                        $channelsAllocatedByCurrentNap++;
                        $extra = ' checked=\'true\'';
                    }
                    elseif( _isTrunkChannelInUse( $yaml, $idChannelDev ) )
                    {
                        $channelsAllocatedByAnotherNap++;
                        $extra = ' disabled=\'disabled\'';
                    }
                    else
                    {
                        $extra = '';
                    }
                    $buff .= "<li><label for='{$idChannel}'>{$l}</label><br /><input id='{$idChannel}' class='gateway-nap-trunk-link-channel' type='checkbox' {$extra}/></li>";
                    if( $l == 14 )
                    {
                        $buff .= "</ul><ul>";
                    }
                }
                $buff .= "</ul></div>";

                $name = isset( $link['Name'] ) ? "{$e1}.{$i}: {$link['Name']}" : "{$e1}.{$i}:";

                if( !in_array( $idLinkDev, $_mylinks, true ) && ( _isTrunkChannelInUse( $yaml, $idLinkDev )) || $channelsAllocatedByAnotherNap == 30 )
                {
                    $options = "<span class='gateway-nap-fully-allocated'>{$app->translate( 'gateway-nap-trunk-fully-allocated' )}</span>";
                }
                else
                {
                    $options = "<select id='{$idLink}' name='link-{$idLink}'>";
                    foreach( array( 'unassigned', 'full', 'partial' ) as $opt )
                    {
                        $select = '';
                        if( ($opt == 'full' && in_array( $idLinkDev, $_mylinks, true )) || ($opt == 'partial' && $channelsAllocatedByCurrentNap > 0) )
                        {
                            $select = 'selected=\'selected\'';
                        }
                        // hide full option if has any channel allocated by
                        // another NAP
                        elseif( $opt == 'full' && $channelsAllocatedByAnotherNap > 0 )
                        {
                            continue;
                        }
                        elseif( $opt == 'unassigned' && !in_array( $idLinkDev, $_mylinks, true ) && $channelsAllocatedByCurrentNap == 0 )
                        {
                            $select = 'selected=\'selected\'';
                        }
                        $optid = "gateway-nap-trunk-allocate-{$opt}";
                        $options .= "<option value='{$opt}' {$select}>{$app->translate( $optid )}</option>";
                    }
                    $options .= '</select>';
                }

                $buffer .= "<tr><td style='text-align:left; class='width150'><label for='{$idLink}'>{$name}</label></td><td style='width:40%;text-align:right;'>{$options}<br />{$buff}</td></tr>";
            }
            $buffer .= '</table></fieldset><br />';
        }

        return $buffer;
    }

    function printNewNAPFXS( $nap, $yaml )
    {
        global $sistema;
        $app = KSystem::getInstance( );

        $buffer = '';
        $devFXS = $sistema->getSpecificDevice( 'FXS' );

        $buffer .= "{$app->translate( 'Branches' )}:<br /><table border='0'>";

        $_mylinks = array( );
        if( array_key_exists( 'Branches', $nap ) )
        {
            // OUTCH!!! YAML CAST DOTTED ELEMENTS TO FLOAT!!!
            // This ugly cast is to ensure that link ids (device.link)
            // are in string form and not as float
            foreach( $nap['Branches'] as $device => $branch )
            {
                $_mylinks["{$device}"] = $branch;
            }
        }

        foreach( $devFXS as $fxs )
        {
            $buffer .= '<tr>';
            $buffer .= '<td>';
            $buffer .= "<table>";
            $buffer .= '<tr>';
            $buffer .= "<td>Serial: {$fxs}<br>";
            $buffer .= "<span class='cursorHand' onclick='autoCompleteFXS( {$fxs} );'><img src='/imagens/seta3.png' border='0' /></span>";
            $buffer .= "<span class='cursorHand' onclick='cleanFXS( {$fxs} );'><img src='/imagens/broom2.jpg' border='0' /></span>";
            $buffer .= "</td>";
            $buffer .= '</tr>';
            $buffer .= '</table>';
            $buffer .= '</td>';
            $buffer .= '<td>';
            $buffer .= "<table id='branches-{$fxs}' border='0'><tr>";
            for( $i = 0; $i < 24; $i++ )
            {
                // html id
                $branchId = "{$fxs}-{$i}";
                // device id
                $branchIdDev = "{$fxs}.{$i}";

                if( array_key_exists( $branchIdDev, $_mylinks ) )
                {
                    $extra = " value='{$_mylinks[$branchIdDev]}'";
                }
                elseif( _isFXSChannelInUse( $yaml, $branchIdDev ) )
                {
                    $extra = ' disabled=\'disabled\'';
                }
                else
                {
                    $extra = ' value=\'\'';
                }

                if( $i > 0 && $i % 8 == 0 )
                {
                    $tr = '</tr><tr>';
                }
                else
                {
                    $tr = '';
                }
                $buffer .= "{$tr}<td width='40'>{$i}: <input id='{$branchId}' class='gateway-nap-fxs-branch' type='text' size='4' onkeydown='return onlyIntegerNumbers(event);' {$extra}/></td>";
            }
            $buffer .= '</tr></table></td></tr>';
        }
        $buffer .= '</table>';

        return $buffer;
    }

    //TODO: port this function to work in the same way as the other printNewNAP*
    function printNewNAPFXSGroup( $nap, $yaml )
    {
        return 'TODO: FIXME';
        global $sistema;
        $buffer = '';

        $buffer .= $sistema->retTranslate( 'NAPChannels' ) . ':<br/><table border="0">';
        $buffer .= '<tr><td><table>';
        $buffer .= '<tr>';

        $nap = getNAPFXS( );
        if( $nap != '' )
        {
            $ramais = array( );
            $seriais = '';

            if( sizeof( $nap['Branches'] ) > 0 )
            {
                for( $i = 0; $i < sizeof( $nap['Branches'] ); $i++ )
                {
                    $tmp = array_values( $nap['Branches'][$i] );
                    $ramais[] = $tmp[0];
                }
                $ctrl = 0;
                $seriais = '';
                foreach( $ramais as $r )
                {
                    $seriais .= "$r|";
                    $disabled = verificaSeCanalJaUtilizado( 'FXSGroup', "$r" ) ? 'disabled' : '';
                    $buffer .= "<td>$r<input $disabled id='$r' type='checkbox' /></td>";
                    if( $ctrl == 7 )
                    {
                        $buffer .= '</tr>';
                        $ctrl = 0;
                    }
                    else
                        $ctrl++;
                }
            }
            else
                $buffer .= "<td>" . $sistema->retTranslate( 'noFXSBranches' ) . "</td>";
            $buffer .= '</table></td></tr>';
            $buffer .= '</table>';

            $seriais = substr( $seriais, 0, strlen( $seriais ) - 1 );
            // clear the last pipe
            $buffer .= "<input id='seriais' type='hidden' value='$seriais' />";
            if( sizeof( $nap['Branches'] ) > 0 )
                $buffer .= "</br><input type='button' onclick='saveFXSGroupNAP();' value='" . $sistema->retTranslate( 'save' ) . "' />";
            $buffer .= "<input type='button' onclick='backNAPs();' value='" . $sistema->retTranslate( 'back' ) . "' />";
        }

        return $buffer;
    }

    function printNewNAPSIP( $nap, $yaml )
    {
        require_once KCONFIG . '/classes/KGatewayHardwareInfo.php';
        require_once KCONFIG . '/classes/KGatewaySwitch.php';

        $app = KSystem::getInstance( );

        if( is_null( $nap ) || count( $nap ) == 0 )
        {
            $nap = array(
                'Register' => false,
                'Username' => '',
                'Domain' => '',
                'DomainPort' => 5060,
                'DomainUseIpPacketAddress' => false,
                'IgnoreSourcePort' => false,
                'Proxy' => '',
                'ProxyPort' => '',
                'AuthorizationUser' => '',
                'Password' => '',
                'Expires' => '',
                'UseCalledFromRequestURI' => false,
                'SipTransportType' => 'UDP',
                'AudioTransportType' => 'UDP',
                'StunServer' => '',
                'StunServerPort' => '',
                'NATTraversalMode' => 'DISABLED',
                'ForceSipInterface' => '',
                'ForceRtpInterface' => '',
                'ExternalRtpAddress' => '',
                'ExternalSipAddress' => '',
                'KeepAlive' => false,
                'KeepAliveInterval' => 0
            );
        }

        $buffer = '';

        $buffer .= '<table class="zebra simpleTable textAlignLeft">';

        $buffer .= "<tr><td class='label'><label for='gateway-nap-sip-domain'>{$app->translate( 'gateway-nap-sip-domain' )}</label></td>";
        $buffer .= "<td style='width:40%;text-align:right;'><input id='gateway-nap-sip-domain' type='text' value='{$nap['Domain']}' /></td>";
        $buffer .= '</tr>';

        $buffer .= "<tr><td class='label'><label for='gateway-nap-sip-domain-port'>{$app->translate( 'gateway-nap-sip-domain-port' )}</label></td>";
        $buffer .= "<td style='width:40%;text-align:right;'><input id='gateway-nap-sip-domain-port' type='text' value='{$nap['DomainPort']}' /></td>";
        $buffer .= '</tr></table><br  />';

        $buffer .= "<fieldset><legend>{$app->translate( 'options' )}</legend>";

        $checked = $nap['DomainUseIpPacketAddress'] ? ' checked=\'checked\'' : '';
        $buffer .= "<br /><input id='gateway-nap-sip-domain-use-ip-packet-address' name='DomainUseIpPacketAddress' type='checkbox' {$checked}/> <label for='gateway-nap-sip-domain-use-ip-packet-address'>{$app->translate( 'gateway-nap-sip-domain-use-ip-packet-address' )}</label><br />";

        $checked = $nap['IgnoreSourcePort'] ? ' checked=\'checked\'' : '';
        $buffer .= "<input id='gateway-nap-sip-ignore-source-port' name='IgnoreSourcePort' type='checkbox' {$checked}/> <label for='gateway-nap-sip-ignore-source-port'>{$app->translate( 'gateway-nap-sip-ignore-source-port' )}</label><br />";

        $checked = $nap['UseCalledFromRequestURI'] ? ' checked=\'checked\'' : '';
        $buffer .= "<input id='gateway-nap-sip-use-called-from-uri' name='UseCalledFromRequestURI' type='checkbox' {$checked}/> <label for='gateway-nap-sip-use-called-from-uri'>{$app->translate( 'gateway-nap-sip-use-called-from-uri' )}</label><br />";

        $buffer .= "<br /><table class='zebra simpleTable textAlignLeft'><tbody>";

        $buffer .= "<tr id='gateway-nap-sip-keep-alive-interval-tr'><td><label for='gateway-nap-sip-keep-alive-interval'>{$app->translate('gateway-nap-sip-keep-alive-interval')}</label></td><td style='width:40%;text-align:right;'>";
        $selected = $nap['KeepAlive'] ? $nap['KeepAliveInterval'] : 0;
        $options = array( 0 => $app->translate( 'disabled' ) );
        for( $i = 1; $i < 11; ++$i )
        {
            $v = $i * 1000;
            $options["{$v}"] = $v;
        }
        $buffer .= make_select( 'gateway-nap-sip-keep-alive-interval', $options, $selected, true );
        $buffer .= "</td></tr>";
        unset( $options );

        $buffer .= "<tr><td><label for='gateway-nap-sip-sip-transport-type'>{$app->translate('gateway-nap-sip-sip-transport-type')}</label></td><td style='width:40%;text-align:right;'>";
        $current = $nap['SipTransportType'] ? $nap['SipTransportType'] : 'UDP';
        $buffer .= make_select( 'gateway-nap-sip-sip-transport-type', array('UDP', 'TCP', 'TLS'), $current );
        $buffer .= "</td></tr>";

        $buffer .= "<tr><td><label for='gateway-nap-sip-audio-transport-type'>{$app->translate('gateway-nap-sip-audio-transport-type')}</label></td><td style='width:40%;text-align:right;'>";
        $current = $nap['AudioTransportType'] ? $nap['AudioTransportType'] : 'UDP';
        $buffer .= make_select( 'gateway-nap-sip-audio-transport-type', array('UDP', 'TCP', 'SRTP'), $current );
        $buffer .= "</td></tr>";

        $limit = isset( $nap['ChannelAllocator']['Limit'] ) ? $nap['ChannelAllocator']['Limit'] : '';
        $checked = ( empty( $limit ) || (int)$limit === 0 ) ? ' checked="checked"' : '';
        $buffer .= "<tr><td width='50%'><label for='gateway-nap-sip-channel-limit'>{$app->translate( 'gateway-nap-sip-channel-limit' )}</label>:</td>";
        $buffer .= "<td style='width:40%;text-align:right;'><input id='gateway-nap-sip-channel-limit' type='text' value='{$limit}' /> <input type='checkbox' id='gateway-nap-sip-channel-no-limit'{$checked} /> <label for='gateway-nap-sip-channel-no-limit'>{$app->translate( 'gateway-nap-sip-channel-no-limit')}</label></td>";
        $buffer .= "</tr>";

        $buffer .= "</tbody></table>";
        $buffer .= '</fieldset><br/>';

        $buffer .= "<fieldset><legend>{$app->translate( 'gateway-nap-sip-network-options' )}</legend>";
        $buffer .= "<table class='zebra simpleTable textAlignLeft'><tbody>";

        $hwinfo = KGatewayHardwareInfo::getInstance();
        $swinfo = new KGatewaySwitchYaml();
        if( count( $hwinfo->query( 'Switch.Cores' ) ) > 0 )
        {
            $options = array( '' => $app->translate( 'gateway-nap-sip-force-interface-AUTOMATIC' ) );
            foreach( $hwinfo->getSwitchPortNames() as $intf )
            {
                if( $swinfo->hasVLANConfigWithPortName( $intf ) )
                {
                    $options["{$intf}.ipv4"] = "{$intf} (IPv4)";
                    $options["{$intf}.ipv6"] = "{$intf} (IPv6)";
                }
            }

            $buffer .= "<tr><td><label for='gateway-nap-sip-force-sip-interface'>{$app->translate( 'gateway-nap-sip-force-sip-interface' )}</label>&nbsp;<img title='{$app->translate( 'gateway-nap-sip-disabled-interface-tooltip' )}' src='/imagens/tooltip.png' class='gateway-tooltip' /></td>";
            $buffer .= "<td style='width:40%;text-align:right;'>";
            $buffer .= make_select( 'gateway-nap-sip-force-sip-interface', $options, isset( $nap['ForceSipInterface'] ) ? $nap['ForceSipInterface'] : '', true );
            $buffer .= "</td></tr>";

            $buffer .= "<tr><td><label for='gateway-nap-sip-force-rtp-interface'>{$app->translate( 'gateway-nap-sip-force-rtp-interface' )}</label>&nbsp;<img title='{$app->translate( 'gateway-nap-sip-disabled-interface-tooltip' )}' src='/imagens/tooltip.png' class='gateway-tooltip' /></td>";
            $buffer .= "<td style='width:40%;text-align:right;'>";
            $buffer .= make_select( 'gateway-nap-sip-force-rtp-interface', $options, isset( $nap['ForceRtpInterface'] ) ? $nap['ForceRtpInterface'] : '' , true );
            $buffer .= "</td></tr>";
            unset( $options );
        }

        if( !isset( $nap['NATTraversalMode'] ) || empty( $nap['NATTraversalMode'] ) )
        {
            if( isset( $nap['StunServer'] ) && !empty( $nap['StunServer'] ) )
            {
                $nap['NATTraversalMode'] = 'STUN';
            }
            else
            {
                $nap['NATTraversalMode'] = 'DISABLED';
            }
        }

        $options = array();
        foreach( KGatewayYaml::$nattraversalmodes as $mode )
        {
            $options[$mode] = $app->translate( "gateway-nap-sip-nat-traversal-mode-{$mode}" );
        }
        $buffer .= "<tr><td><label for='gateway-nap-sip-nat-traversal-mode'>{$app->translate( 'gateway-nap-sip-nat-traversal-mode' )}</label></td>";
        $buffer .= "<td style='width:40%;text-align:right;'>";
        $buffer .= make_select( 'gateway-nap-sip-nat-traversal-mode', $options, $nap['NATTraversalMode'], true );
        $buffer .= "</td></tr>";
        unset( $options );

        $buffer .= "<tr class='gateway-nap-sip-stun-tr'><td><label for='gateway-nap-sip-stun-server'>{$app->translate( 'gateway-nap-sip-stun-server' )}</label></td>";
        $buffer .= "<td style='width:40%;text-align:right;'><input id='gateway-nap-sip-stun-server' type='text' value='{$nap['StunServer']}' /></td>";
        $buffer .= "</tr>";

        $buffer .= "<tr class='gateway-nap-sip-stun-tr'><td><label for='gateway-nap-sip-stun-server-port'>{$app->translate( 'gateway-nap-sip-stun-server-port' )}</label></td>";
        $buffer .= "<td style='width:40%;text-align:right;'><input id='gateway-nap-sip-stun-server-port' type='text' value='{$nap['StunServerPort']}' /></td>";
        $buffer .= "</tr>";

        $externaladdr = isset( $nap['ExternalSipAddress'] ) ? $nap['ExternalSipAddress'] : '';
        $buffer .= "<tr class='gateway-nap-sip-external-address-tr'><td><label for='gateway-nap-sip-external-sip-address'>{$app->translate( 'gateway-nap-sip-external-sip-address' )}</label></td>";
        $buffer .= "<td style='width:40%;text-align:right;'><input id='gateway-nap-sip-external-sip-address' type='text' value='{$externaladdr}' /></td>";
        $buffer .= "</tr>";

        $externaladdr = isset( $nap['ExternalRtpAddress'] ) ? $nap['ExternalRtpAddress'] : '';
        $buffer .= "<tr class='gateway-nap-sip-external-address-tr'><td><label for='gateway-nap-sip-external-rtp-address'>{$app->translate( 'gateway-nap-sip-external-rtp-address' )}</label></td>";
        $buffer .= "<td style='width:40%;text-align:right;'><input id='gateway-nap-sip-external-rtp-address' type='text' value='{$externaladdr}' /></td>";
        $buffer .= "</tr>";
        unset( $externaladdr );

        $buffer .= '</tbody></table></fieldset><br />';


        $checked = $nap['Register'] ? ' checked=\'checked\'' : '';
        $buffer .= "<fieldset><legend><input id='gateway-nap-sip-register' type='checkbox' {$checked} /> <label for='gateway-nap-sip-register'>{$app->translate( 'Register' )}</label></legend>";

        $buffer .= '<table class="zebra simpleTable textAlignLeft">';
        $buffer .= "<tr><td width='50%'><label for='gateway-nap-sip-username'>{$app->translate( 'Username' )}</label></td>";
        $buffer .= "<td style='width:40%;text-align:right;'><input id='gateway-nap-sip-username' type='text' value='{$nap['Username']}' /></td>";
        $buffer .= '</tr>';

        $buffer .= "<tr><td width='50%'><label for='gateway-nap-sip-authorization-user'>{$app->translate( 'gateway-nap-sip-authorization-user' )}</label></td>";
        $buffer .= "<td style='width:40%;text-align:right;'><input id='gateway-nap-sip-authorization-user' type='text' value='{$nap['AuthorizationUser']}' /></td>";
        $buffer .= '</tr>';

        $buffer .= "<tr><td width='50%'><label for='gateway-nap-sip-password'>{$app->translate( 'Password' )}</label></td>";
        $buffer .= "<td style='width:40%;text-align:right;'><input id='gateway-nap-sip-password' type='password' value='{$nap['Password']}' /></td>";
        $buffer .= '</tr>';

        $buffer .= "<tr><td width='50%'><label for='gateway-nap-sip-proxy'>{$app->translate( 'Proxy' )}</label></td>";
        $buffer .= "<td style='width:40%;text-align:right;'><input id='gateway-nap-sip-proxy' type='text' value='{$nap['Proxy']}' /></td>";
        $buffer .= '</tr>';

        $buffer .= "<tr><td width='50%'><label for='gateway-nap-sip-proxy-port'>{$app->translate( 'ProxyPort' )}</label></td>";
        $buffer .= "<td style='width:40%;text-align:right;'><input id='gateway-nap-sip-proxy-port' type='text' value='{$nap['ProxyPort']}' /></td>";
        $buffer .= '</tr>';

        $buffer .= "<tr><td width='50%'><label for='gateway-nap-sip-expires'>{$app->translate( 'gateway-nap-sip-expires' )}</label></td>";
        $buffer .= "<td style='width:40%;text-align:right;'><input id='gateway-nap-sip-expires' type='text' value='{$nap['Expires']}' /></td>";
        $buffer .= '</tr></table></fieldset><br  />';

        return $buffer;
    }

    //TODO: remove this function
    function getConfigGW( )
    {
        $kyaml = new KYaml( GatewayUtil::getGatewayConfigPath( ) );
        return $kyaml;
    }

    function getNAPTypes( )
    {
        global $sistema;

        $types = array( 'SIP' );
        /*
         if( sizeof( $sistema->getSpecificDevice( "FXO" ) ) > 0 )
         $types[] = "FXO";
         */
        if( sizeof( $sistema->getSpecificDevice( 'E1' ) ) )
        {
            $types[] = 'Trunk';
        }
        if( sizeof( $sistema->getSpecificDevice( "GSM" ) ) > 0 )
        {
            $types[] = "GSM";
        }
        /*
         if( sizeof( $sistema->getSpecificDevice( "FXS" ) ) )
         {
         $types[] = "FXS";
         if( existsNAPFXS( ) )
         $types[] = "FXSGroup";
         }
         */

        return $types;
    }

    //TODO: port this function to KGatewayYaml class
    function searchNAPValue( $value, $yaml = null )
    {
        if( is_null( $yaml ) )
        {
            $kyaml = new KGatewayYaml( );
            $yaml = $kyaml->getYaml( );
            unset( $kyaml );
        }
        for( $i = 0; $i < sizeof( $yaml['NAPs'] ); $i++ )
        {
            if( $yaml['NAPs'][$i]['Name'] == $value )
                return $yaml['NAPs'][$i];
        }
        return null;
    }

    function existsNAPFXS( )
    {
        $kyaml = getConfigGW( );
        $yaml = $kyaml->getYaml( );

        foreach( $yaml['NAPs'] as $nap )
        {
            if( $nap['Type'] == 'FXS' )
                return true;
        }
        return false;
    }

    //TODO: port this function to KGatewayYaml class
    function getAllSimCardsInUse()
    {
        $kyaml = getConfigGW( );
        $yaml = $kyaml->getYaml( );

        $ret = array();

        foreach( $yaml['NAPs'] as $nap )
        {
            if($nap['Type'] == 'GSM')
            {
                $ret = array_merge( $ret, $nap['SIMs']);
            }
        }
        return $ret;
    }

    //TODO: port this function to KGatewayYaml class
    function _isFXSGroupChannelInUse( &$yaml, $channel )
    {
        return _isChannelInUse( $yaml, $channel, 'FXSGroup', 'Branches' );
    }

    //TODO: port this function to KGatewayYaml class
    function _isFXSChannelInUse( &$yaml, $channel )
    {
        return _isChannelInUse( $yaml, $channel, 'FXS', 'Branches' );
    }

    //TODO: port this function to KGatewayYaml class
    function _isFXOChannelInUse( &$yaml, $channel )
    {
        return _isChannelInUse( $yaml, $channel, 'FXO', 'Channels' );
    }

    //TODO: port this function to KGatewayYaml class
    function _isGSMChannelInUse( &$yaml, $channel )
    {
        return _isChannelInUse( $yaml, $channel, 'GSM', 'Channels' );
    }

    //TODO: port this function to KGatewayYaml class
    function _isTrunkChannelInUse( &$yaml, $channel )
    {
        return _isChannelInUse( $yaml, $channel, 'Trunk', 'Links' );
    }

    /**
     * Internal use only, use _is[type]ChannelInUse instead.
     *
     * TODO: port this function to KGatewayYaml class
     */
    function _isChannelInUse( &$yaml, &$channel, $type, $napkey )
    {
        foreach( $yaml['NAPs'] as $_nap )
        {
            if( $_nap['Type'] == $type && array_key_exists( $napkey, $_nap ) )
            {
                foreach( $_nap[$napkey] as $link )
                {
                    if( "{$link}" === $channel )
                    {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    //TODO: delete this function
    function verificaSeCanalJaUtilizado( $NAPType, $canal, $NAPAtual = '' )
    {
        $kyaml = getConfigGW( );
        $yaml = $kyaml->getYaml( );
        $NAPs = $yaml['NAPs'];

        foreach( $NAPs as $nap )
        {
            if( $nap['Type'] == $NAPType && $nap['Name'] != $NAPAtual )
            {
                $canais = $nap[getSectionName( $NAPType )];
                foreach( $canais as $ch )
                {
                    if( @ereg( '\[', $ch ) )
                    {
                        $tmpCh = explode( '.', $ch );
                        if( sizeof( $tmpCh ) == 2 )
                        {
                            $c = explode( '.', $canal );
                            if( $c[0] != $tmpCh[0] )
                            {
                                // return false;
                            }
                            else
                            {
                                $tmp = explode( '-', $tmpCh[1] );
                                $tmp[0] = str_replace( '[', '', $tmp[0] );
                                $tmp[1] = str_replace( ']', '', $tmp[1] );
                                if( $c[1] < $tmp[0] || $c[1] > $tmp[1] )
                                {
                                    // return false;
                                }
                                else
                                    return true;
                            }
                        }
                        else
                        {
                            $c = explode( '.', $canal );
                            $tmpCh = explode( '.', $ch );
                            if( $c[0] != $tmpCh[0] || $c[1] != $tmpCh[1] )
                            {
                                // return false;
                            }
                            else
                            {
                                $tmp = str_replace( '[', '', $ch[2] );
                                $tmp = str_replace( ']', '', $tmp );
                                $tmp = explode( '-', $tmp );
                                if( $c[2] < $tmp[0] || $c[2] > $tmp[1] )
                                {
                                    // return false;
                                }
                                else
                                    return true;
                            }
                        }
                    }
                    else
                    {
                        $tmpCH = explode( '.', $ch );
                        if( sizeof( $tmpCH ) == 2 )
                        {
                            $c = explode( '.', $canal );
                            if( $tmpCH[0] == $c[0] && $tmpCH[1] == $c[1] )
                            {
                                return true;
                            }
                        }
                        else
                        {
                            if( $canal == $ch )
                                return true;
                        }
                    }
                }
            }
        }
        return false;
    }

    //TODO: delete this function
    function checkRealValue( $var )
    {
        if( $var == 'true' )
            return true;
        elseif( $var == 'false' )
            return false;
        else
            return $var;
    }

    //TODO: delete this function
    function getSectionName( $type )
    {
        switch ($type)
        {
            /*
             case 'GSM':
             case 'FXO':
             return 'Channels';
             */
            case 'Trunk':
                return 'Links';
            /*
             case 'FXS':
             case 'FXSGroup':
             return 'Branches';
             */
            default:
                return '';
        }
    }

    //TODO: delete this function
    function getNAPFXS( )
    {
        $kyaml = getConfigGW( );
        $yaml = $kyaml->getYaml( );

        foreach( $yaml['NAPs'] as $nap )
        {
            if( $nap['Type'] == 'FXS' )
                return $nap;
        }
        return '';
    }

    // Generate the html for NAP ChannelAllocator
    function getNAPAllocationAlgorithmHtml( &$data )
    {
        $app = KSystem::getInstance();

        $_algorithm = isset( $data['ChannelAllocator']['Algorithm'] ) ? $data['ChannelAllocator']['Algorithm'] : KGatewayYaml::$channelallocatoralgorithms[0];
        $_algorithm_sort = isset( $data['ChannelAllocator']['Sort'] ) ? $data['ChannelAllocator']['Sort'] : KGatewayYaml::$channelallocatoralgorithms[0];

        $buffer = '<tr><td class="label">';
        $buffer .= '<label for="gateway-nap-channel-algorithm">' . $app->translate( 'gateway-nap-channel-algorithm' ) . '</label> ';
        $buffer .= '</td><td style="width:50%;text-align:right;">';
        $buffer .= '<select id="gateway-nap-channel-algorithm">';
        $buffer .= '<optgroup label="' . $app->translate('gateway-nap-channel-algorithm-label') . '">';
        foreach( KGatewayYaml::$channelallocatoralgorithms as $algorithm )
        {
            // LIMIT is currently avliable only to SIP NAPs
            if( $algorithm == 'LIMIT' )
            {
                continue;
            }
            $selected = $_algorithm === $algorithm ? ' selected="selected"' : '';
            $key = "gateway-nap-channel-algorithm-{$algorithm}";
            $buffer .= '<option value="' . $algorithm . '" ' . $selected . '>' . $app->translate( $key ) . '</option>';
        }
        $buffer .= '</optgroup>';
        $buffer .= '</select>';

        $buffer .= '<select id="gateway-nap-channel-algorithm-sort" style="min-width: 200px;">';
        $buffer .= '<optgroup label="' . $app->translate('gateway-nap-channel-algorithm-sort') . '">';
        $buffer .= '<option value="SORT_BY_DEVICE_ASC"' . ($_algorithm_sort === 'SORT_BY_DEVICE_ASC' ? ' selected="selected"' : '') . '>' . $app->translate( 'gateway-nap-channel-algorithm-sort-sort_by_device_asc' ) . '</option>';
        $buffer .= '<option value="SORT_BY_DEVICE_DESC"' . ($_algorithm_sort === 'SORT_BY_DEVICE_DESC' ? ' selected="selected"' : '') . '>' . $app->translate( 'gateway-nap-channel-algorithm-sort-sort_by_device_desc' ) . '</option>';
        $buffer .= '</optgroup>';

        if ( !empty( $_REQUEST['type'] ) && $_REQUEST['type'] === 'Trunk' ) {
            $buffer .= '<optgroup label="' . $app->translate('gateway-nap-channel-algorithm-sort-by-link') . '">';
            $buffer .= '<option value="SORT_BY_LINK_ASC"' . ($_algorithm_sort === 'SORT_BY_LINK_ASC' ? ' selected="selected"' : '') . '>' . $app->translate( 'gateway-nap-channel-algorithm-sort-sort_by_link_asc' ) . '</option>';
            $buffer .= '<option value="SORT_BY_LINK_DESC"' . ($_algorithm_sort === 'SORT_BY_LINK_DESC' ? ' selected="selected"' : '') . '>' . $app->translate( 'gateway-nap-channel-algorithm-sort-sort_by_link_desc' ) . '</option>';
        }

        $buffer .= '</select>';

        $buffer .= '</td></tr>';

        return $buffer;
    }

