<?php
    require_once KCONFIG . '/classes/classKGatewayUtil.php';
    require_once KCONFIG . '/classes/KGatewayYaml.php';

    $AjaxFunctions['showRoutesGW'] = 'showRoutesGW';
    $AjaxFunctions['newRouteGW'] = 'newRouteGW';
    $AjaxFunctions['editRouteGW'] = 'editRouteGW';
    $AjaxFunctions['saveRouteGW'] = 'saveRouteGW';
    $AjaxFunctions['removeRouteGW'] = 'removeRouteGW';
    $AjaxFunctions['routesOrder'] = 'routesOrder';

    function showRoutesGW( )
    {
        require_once KROOT . '/classes/KLicense.php';

        $app = KSystem::getInstance( );
        $kyaml = new KGatewayYaml( );
        $manager = new KLicenseManager( );

        $buffer = '';

        $buffer .= "<fieldset><legend>{$app->translate( 'Routes' )}</legend><div id='gateway-route'>";
        $buffer .= "<input type='button' value='{$app->translate( 'newRoute' )}' class='gateway-route-add' /><br /><br />";

        if( $kyaml->countRoutes( ) > 0 )
        {
            $extra = '';

            if( $manager->sumLicensesFieldValue( KLicense::TYPE_GATEWAYVOIPCALLS ) == 0 )
            {
                $extra = "&nbsp;<img src='/imagens/tooltip.png' class='gateway-tooltip' khelp='gateway-route-tooltip-any-nap-may-ignore-sip-nap-destination' />";
            }

            $buffer .= '<div id=\'gateway-route-list\'>';
            $buffer .= "<table id='gateway-route-list-table' class='zebra gateway-table'>";
            $buffer .= '<thead><tr>';
            $buffer .= "<th rowspan='2'>{$app->translate( 'name' )}</th>";
            $buffer .= "<th colspan='2'>{$app->translate( 'gateway-route-incoming' )}</th>";
            $buffer .= "<th colspan='3'>{$app->translate( 'gateway-route-outgoing' )}</th>";
            $buffer .= "<th rowspan='2'>{$app->translate( 'priority' )}</th>";
            $buffer .= "<th class='gateway-table-options' rowspan='2'>{$app->translate( 'Options' )}</th>";
            $buffer .= '</tr><tr>';
            $buffer .= "<th>{$app->translate( 'OriginNAP' )}{$extra}</th>";
            $buffer .= "<th>{$app->translate( 'numberB' )}</th>";
            $buffer .= "<th>{$app->translate( 'DestinationNAP' )}</th>";
            $buffer .= "<th>{$app->translate( 'changingNumberB' )}</th>";
            $buffer .= "<th>{$app->translate( 'gateway-profile' )}</th>";
            $buffer .= '</tr></thead><tbody>';
            foreach( $kyaml->getAllRoutes() as $route )
            {
                $origin = $route['OriginNAP'] == '<ANY>' ? $app->translate( 'gateway-route-any' ) : $route['OriginNAP'];
                $destination = $route['DestinationNAP'] == '<ANY>' ? $app->translate( 'gateway-route-any' ) : $route['DestinationNAP'];
                $profile = isset( $route['DestinationProfile'] ) ? $route['DestinationProfile'] : "<i>{$app->translate( 'gateway-route-profile-use-from-nap' )}</i>";
                $buffer .= '<tr>';
                $buffer .= "<td>{$route['Name']}</td>";
                $buffer .= "<td>{$origin}</td>";
                $buffer .= "<td>{$route['Called']}</td>";
                $buffer .= "<td>{$destination}</td>";
                $buffer .= "<td>{$route['RemapCalled']}</td>";
                $buffer .= "<td>{$profile}</td>";
                $buffer .= "<td align='right'>{$route['Priority']}</td>";
                $buffer .= "<td class='gateway-table-options'><img border='0' src='/imagens/edit.png' class='gateway-route-edit' title='{$app->translate( 'edit' )}' /><img border='0' src='/imagens/remove.png' class='gateway-route-delete' title='{$app->translate( 'delete' )}' /></td>";
                $buffer .= '</tr>';
            }
            $buffer .= '</tbody></table><br />';
            $buffer .= '</div>';

        }
        else
        {
            $buffer .= $app->translate( 'gateway-route-empty-list' );
        }
        $buffer .= '</div></fieldset>';
        return $buffer;
    }

    function saveRouteGW( )
    {
        require_once KROOT . '/json_util.php';

        $app = KSystem::getInstance( );
        $reply = array( 'success' => false, 'message' => $app->translate( 'gateway-route-error-save' ) );

        if( array_key_exists( '__oldname__', $_POST ) )
        {
            $oldName = $_POST['__oldname__'];
            unset( $_POST['__oldname__'] );
        }
        else
        {
            $oldName = '';
        }

        try
        {
            // TimeRestraints can be null, so check for the value <NONE> and
            // change it to empty string, if match
            $_POST['Time'] = ($_POST['Time'] === '<NONE>') ? '' : $_POST['Time'];

            // Priority is numeric and cannot be null, 0 is default
            $priority = 0;
            if( is_numeric( $_POST['Priority'] ) )
                $priority = intval( $_POST['Priority'] );
            unset( $_POST['Priority'] );
            $_POST['Priority'] = $priority;
            if( $priority < 0 ) $priority = 0;
            else if( $priority > 99 ) $priority = 99;

            $kyaml = new KGatewayYaml( );
            $kyaml->setRoute( $_POST, $oldName );
            $kyaml->save( );

            $reply = array( 'success' => true, 'message' => $app->translate( 'gateway-route-saved' ) );
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }
        return JSONencode( $reply );
    }

    function _formRouteGW( $route = null, $kyaml = null )
    {
        require_once KROOT . '/classes/KJsonRpcClient.php';
        require_once KROOT . '/classes/KLicense.php';

        $app = KSystem::getInstance( );
        $jsonrpc = new KJsonRpcClient( 'kgateway' );
        $manager = new KLicenseManager( );

        $availableServiceProviders = ( $jsonrpc->PortabilityDriver( ) !== 'DISABLED' ) ? $jsonrpc->PortabilityServiceProviders( ) : array( );

        if( is_null( $route ) )
        {
            $route = array( 'Name' => '', 'OriginNAP' => '', 'Called' => '', 'Caller' => '', 'RemapCalled' => '', 'RemapCaller' => '', 'DestinationNAP' => '', 'Time' => '<NONE>', 'CodecProfile' => '', 'DestinationServiceProvider' => array( ), 'PortabilityRegex' => '' );
        }
        if( !isset( $route['Priority'] ) )
        {
            $route['Priority'] = 0;
        }
        else
        {
            if( $route['Priority'] < 0 ) $route['Priority'] = 0;
            else if( $route['Priority'] > 99 ) $route['Priority'] = 99;
        }
        if( is_null( $kyaml ) || !($kyaml instanceof KGatewayYaml) )
        {
            $kyaml = new KGatewayYaml( );
        }
        $yaml = $kyaml->getYaml( );
        $naps = &$yaml['NAPs'];
        $timeRestraints = &$yaml['TimeRestraints'];
        $serviceProviders = isset( $route['DestinationServiceProvider'] ) ? $route['DestinationServiceProvider'] : array( );

        $routeName = htmlspecialchars( $route['Name'], ENT_QUOTES );
        $routePriority = strval( $route['Priority'] );

        $buf = '';
        $buf .= '<table class=\'zebra simpleTable gateway-table-form\'>';
        $buf .= '<tr>';
        $buf .= "<td class='label'><label for='gateway-route-form-name'>{$app->translate( 'name' )}</label></td>";
        $buf .= "<td class='gateway-form-field-container-right'><input id='gateway-route-form-name' name='Name' type='text' value='{$routeName}' /><input type='hidden' name='__oldname__' value='{$routeName}' /></td>";
        $buf .= '</tr>';

        $buf .= '<tr>';
        $buf .= "<td class='label'><label for='gateway-route-form-priority'>{$app->translate( 'priority' )}</label> <img khelp='{$app->translate( 'tooltip-priority' )}' src='/imagens/tooltip.png' class='gateway-tooltip' /></td>";
        $buf .= "<td class='gateway-form-field-container-right'><input id='gateway-route-form-priority' name='Priority' type='text' value='{$routePriority}' /></td>";
        $buf .= '</tr>';

        $extra = '';
        if( $manager->sumLicensesFieldValue( KLicense::TYPE_GATEWAYVOIPCALLS ) == 0 )
        {
            $extra = "<br /><br />{$app->translate( 'gateway-route-tooltip-any-nap-may-ignore-sip-nap-destination' )}";
        }

        $buf .= '<tr>';
        $buf .= "<td class='label'><label for='gateway-route-form-origin-nap'>{$app->translate( 'OriginNAP' )}</label> <img khelp='{$app->translate( 'tooltip-OriginNAP' )}{$extra}' src='/imagens/tooltip.png' class='gateway-tooltip' /></td>";

        $opts = '';
        if( $route['Name'] == '' )
        {
            $opts .= "<option value='<SELECT>' selected>{$app->translate( 'Select' )}</option>";
        }
        foreach( $naps as &$nap )
        {
            $selected = ($nap['Name'] == $route['OriginNAP']) ? ' selected' : '';
            $napName = htmlspecialchars( $nap['Name'], ENT_QUOTES );
            $opts .= "<option value='{$nap['Type']}>{$napName}'{$selected}>{$napName}</option>";
        }
        $selected = ($route['OriginNAP']) == '<ANY>' ? ' selected' : '';
        $opts .= "<option value='<ANY>'{$selected}>&lt;{$app->translate( 'any' )}&gt;</option>";

        $buf .= "<td class='gateway-form-field-container-right'><select id='gateway-route-form-origin-nap' name='OriginNAP'>{$opts}</select></td>";
        $buf .= '</tr>';

        $buf .= '<tr>';
        $buf .= "<td class='label'><label for='gateway-route-form-called'>{$app->translate( 'numberB' )}</label> <img khelp='{$app->translate( 'tooltip-numberB' )}' src='/imagens/tooltip.png' class='gateway-tooltip' /></td>";
        $buf .= "<td class='gateway-form-field-container-right'><input id='gateway-route-form-called' name='Called' type='text' value='{$route['Called']}' /></td>";
        $buf .= '</tr>';

        $buf .= '<tr>';
        $buf .= "<td class='label'><label for='gateway-route-form-caller'>{$app->translate( 'numberA' )}</label> <img khelp='{$app->translate( 'tooltip-numberA' )}' src='/imagens/tooltip.png' class='gateway-tooltip' /></td>";
        $buf .= "<td class='gateway-form-field-container-right'><input id='gateway-route-form-caller' name='Caller' type='text' value='{$route['Caller']}' /></td>";
        $buf .= '</tr>';

        if( count( $availableServiceProviders ) > 0 )
        {
            $opts = '';
            foreach( $availableServiceProviders as &$serviceProvider )
            {
                $selected = in_array( $serviceProvider->Code, $serviceProviders, true ) ? ' selected' : '';
                $opts .= "<option value='{$serviceProvider->Code}'{$selected}>{$serviceProvider->Name} - {$serviceProvider->Code}</option>";
            }
            $checked = count( $serviceProviders ) > 0 ? ' checked' : '';
            $regex = isset( $route['PortabilityRegex'] ) ? $route['PortabilityRegex'] : '';

            $buf .= '<tr>';
            $buf .= "<td class='label'><label for='gateway-route-form-destination-check-portability'>{$app->translate( 'gateway-route-form-destination-check-portability' )}</label></td><td class='gateway-form-field-container-right form-checkbox-alone'><input id='gateway-route-form-destination-check-portability' type='checkbox' {$checked}/></td>";
            $buf .= '</tr><tr class="gateway-route-form-destination-portability">';
            $buf .= "<td class='label'><label for='gateway-route-form-destination-service-providers'>{$app->translate( 'gateway-route-form-destination-service-providers' )}</label> <img khelp='{$app->translate( 'gateway-route-form-destination-service-providers-tooltip' )}' src='/imagens/tooltip.png' class='gateway-tooltip' /></td>";
            $buf .= "<td class='gateway-form-field-container-right'><select id='gateway-route-form-destination-service-providers' name='DestinationServiceProvider' multiple data-placeholder='{$app->translate( 'gateway-route-form-destination-service-providers-click-to-add' )}'>{$opts}</select></td>";
            $buf .= '</tr><tr class="gateway-route-form-destination-portability">';
            $buf .= "<td class='label'><label for='gateway-route-form-destination-portability-regex'>{$app->translate( 'gateway-route-form-destination-portability-regex' )}</label> <img khelp='{$app->translate( 'gateway-route-form-destination-portability-regex-tooltip' )}' src='/imagens/tooltip.png' class='gateway-tooltip' /></td>";
            $buf .= "<td class='gateway-form-field-container-right'><input type='text' id='gateway-route-form-destination-portability-regex' name='PortabilityRegex' value='{$regex}' /></td>";
            $buf .= '</tr>';
        }

        $buf .= '<tr>';
        $buf .= "<td class='label'><label for='gateway-route-form-destination-nap'>{$app->translate( 'DestinationNAP' )} <img khelp='{$app->translate( 'tooltip-destinationNAP' )}' src='/imagens/tooltip.png' class='gateway-tooltip' /></td>";

        $opts = '';
        if( $route['Name'] == '' )
        {
            $opts .= "<option value='<SELECT>' selected>{$app->translate( 'Select' )}</option>";
        }
        foreach( $naps as &$nap )
        {
            $selected = ($nap['Name'] == $route['DestinationNAP']) ? ' selected' : '';
            $napName = htmlspecialchars( $nap['Name'], ENT_QUOTES );
            $opts .= "<option value='{$nap['Type']}>{$napName}'{$selected}>{$napName}</option>";
        }

        $buf .= "<td class='gateway-form-field-container-right'><select id='gateway-route-form-destination-nap' name='DestinationNAP'>{$opts}</select></td>";
        $buf .= '</tr>';

        $buf .= '<tr>';
        $buf .= "<td class='label'><label for='gateway-route-form-remap-called'>{$app->translate( 'changingNumberB' )}</label> <img khelp='{$app->translate( 'tooltip-changingNumberB' )}' src='/imagens/tooltip.png' class='gateway-tooltip' /></td>";
        $buf .= "<td class='gateway-form-field-container-right'><input id='gateway-route-form-remap-called' name='RemapCalled' type='text' value='{$route['RemapCalled']}' /></td>";
        $buf .= '</tr>';

        $buf .= '<tr>';
        $buf .= "<td class='label'><label for='gateway-route-form-remap-caller'>{$app->translate( 'changingNumberA' )}</label> <img khelp='{$app->translate( 'tooltip-changingNumberA' )}' src='/imagens/tooltip.png' class='gateway-tooltip' /></td>";
        $buf .= "<td class='gateway-form-field-container-right'><input id='gateway-route-form-remap-caller' name='RemapCaller' type='text' value='{$route['RemapCaller']}' /></td>";
        $buf .= '</tr>';

        $buf .= "<tr><td class='label'><label for='gateway-route-form-destination-profile'>{$app->translate( 'gateway-route-destination-profile' )}</label></td>";
        $buf .= "<td class='gateway-form-field-container-right'><select id='gateway-route-form-destination-profile' name='DestinationProfile'>";
        $buf .= "<option value=''>{$app->translate( 'gateway-route-profile-use-from-nap' )}</option>";
        $p = isset( $route['DestinationProfile'] ) ? $route['DestinationProfile'] : null;
        foreach( $kyaml->getKGatewayProfileYaml()->getAllProfiles() as $profile )
        {
            $s = $profile['Name'] === $p ? ' selected' : '';
            $profileName = htmlspecialchars( $profile['Name'], ENT_QUOTES );
            $buf .= "<option value='{$profileName}'{$s}>{$profileName}</option>";
        }
        $buf .= '</select></td></tr>';

        $buf .= '<tr>';
        $buf .= "<td class='label'><label for='gateway-route-form-time-restraints'>{$app->translate( 'TimeRestraints' )}</label> <img khelp='{$app->translate( 'tooltip-timeRestraints' )}' src='/imagens/tooltip.png' class='gateway-tooltip' /></td>";

        $opts = '';
        $selected = ($route['Time'] === '') ? ' selected' : '';
        $opts .= "<option value='<NONE>' {$selected}>&lt;{$app->translate( 'none' )}&gt;</option>";
        foreach( $timeRestraints as &$tr )
        {
            $selected = ($route['Time'] == $tr['Name']) ? ' selected' : '';
            $trName = htmlspecialchars( $tr['Name'], ENT_QUOTES );
            $opts .= "<option value='{$trName}'{$selected}>{$trName}</option>";
        }
        $buf .= "<td class='gateway-form-field-container-right'><select id='gateway-route-form-time-restraints' name='Time'>{$opts}</select></td>";
        $buf .= '</tr>';

        // Per route codec profile
        $buf .= "<tr id='gateway-route-form-codec-profile-list' style='display: none;'><td class='label'><label for='gateway-route-form-codec-profile'>{$app->translate( 'gateway-route-codec-profile' )}</label></td>";
        $opts = '';
        if( !array_key_exists( 'CodecProfile', $route ) || $route['CodecProfile'] == '' )
        {
            $opts .= "<option value='<SELECT>' selected>{$app->translate( 'Select' )}</option>";
        }
        foreach( $yaml['CodecProfiles'] as &$profile )
        {
            $codecs = implode( ',', $profile['CodecList'] );
            $selected = $route['CodecProfile'] == $profile['Name'] ? ' selected=\'selected\'' : '';
            $opts .= "<option value='{$profile['Name']}'{$selected}>{$profile['Name']} ({$codecs})</option>";
        }
        $buf .= "<td class='gateway-form-field-container-right'><select id='gateway-route-form-codec-profile' name='CodecProfile'>{$opts}</select></td></tr>";

        $buf .= '</table><br /><div class="textAlignRight">';
        $buf .= "<input type='button' value='{$app->translate( 'back' )}' id='gateway-route-form-back' />";
        $buf .= "<input type='button' value='{$app->translate( 'save' )}' id='gateway-route-form-save' /></div>";

        $buf .= "<p>{$app->translate( 'gateway-route-help-regexp' )}:<br /><a href='http://pubs.opengroup.org/onlinepubs/9699919799/basedefs/V1_chap09.html#tag_09_04' target='_blank'>http://pubs.opengroup.org/onlinepubs/9699919799/basedefs/V1_chap09.html#tag_09_04</a></p>";

        $buf .= "<script>kgatewayVoipCalls = {$manager->sumLicensesFieldValue( KLicense::TYPE_GATEWAYVOIPCALLS )};</script>";

        return $buf;
    }

    function newRouteGW( )
    {
        return _formRouteGW( );
    }

    function editRouteGW( )
    {
        $name = array_key_exists( 'name', $_POST ) ? $_POST['name'] : null;
        $app = KSystem::getInstance( );

        try
        {
            if( is_null( $name ) )
            {
                return $app->translate( 'gateway-route-error-not-found' );
            }
            else
            {
                $kyaml = new KGatewayYaml( );
                $route = $kyaml->getRoute( $name );
                if( is_null( $route ) )
                {
                    return $app->translate( 'gateway-route-error-not-found' );
                }
                return _formRouteGW( $route, $kyaml );
            }
        }
        catch( Exception $e )
        {
            return $app->translate( $e->getMessage( ) );
        }
    }

    function removeRouteGW( )
    {
        require_once KROOT . '/json_util.php';
        $app = KSystem::getInstance( );
        $reply = array( 'success' => false, 'message' => $app->translate( 'gateway-route-error-delete' ) );

        $name = isset( $_POST['name'] ) ? $_POST['name'] : null;

        try
        {
            $kyaml = new KGatewayYaml( );
            if( $kyaml->deleteRoute( $name ) )
            {
                $kyaml->save( );
                $reply = array( 'success' => true, 'message' => $app->translate( 'gateway-route-deleted' ) );
            }
            else
            {
                throw new Exception( 'gateway-route-error-not-found' );
            }
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }
        return JSONencode( $reply );
    }

    function routesOrder( )
    {
        require_once KROOT . '/json_util.php';
        $app = KSystem::getInstance( );
        $reply = array( 'success' => false, 'message' => $app->translate( 'gateway-route-error-reorder' ) );

        $order = isset( $_POST['order'] ) ? $_POST['order'] : null;
        try
        {
            $kyaml = new KGatewayYaml( );
            $kyaml->setRoutesOrder( $order );
            $kyaml->save( );
            $reply = array( 'success' => true, 'message' => '' );
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }
        return JSONencode( $reply );
    }
