<?php
    class KGatewayHardwareInfo
    {
        /**
         * Single instance
         */
        private static $me = null;
        /**
         * Info read from json reference file
         */
        private $info = array( );

        /**
         * Singleton constructor
         */
        private function __construct( )
        {
            $data = file_get_contents( '/etc/khomp/hardwareinfo.json' );

            if( $data )
            {
                $this->info = json_decode( $data, true );
            }
        }

        /**
         * Get singleton instance
         */
        static public function getInstance( )
        {
            if( is_null( self::$me ) )
            {
                self::$me = new KGatewayHardwareInfo( );
            }
            return self::$me;
        }

        /**
         * Get all info
         *
         * @return array
         */
        public function getInfo( )
        {
            return $this->info;
        }

        /**
         * Get switch port config
         *
         * @param string $name Port name
         * @return array|null
         */
        public function getSwitchPortConfig( $name )
        {
            foreach( $this->info['Switch']['PortMap'] as $map )
            {
                if( $map['Name'] === $name )
                {
                    return $map;
                }
            }

            return null;
        }

        /**
         * Get switch port names defined
         *
         * @return array
         */
        public function getSwitchPortNames( )
        {
            $names = array( );
            foreach( $this->info['Switch']['PortMap'] as $map )
            {
                $names[] = $map['Name'];
            }

            return $names;
        }

        /**
         * Get switch core interface
         *
         * @param  integer
         * @return string
         */
        public function getSwitchPortName( $id )
        {
            foreach( $this->info['Switch']['PortMap'] as $map )
            {
                if ( $map["DefaultVlanId"] == $id )
                {
                    return $map['Name'];
                }
            }

            return null;
        }

        /**
         * Query data
         *
         * @return mixed
         */
        public function query( $path, $default = null )
        {
            $node = $this->info;
            $pathparts = explode( '.', $path );
            while( count( $pathparts ) > 1 )
            {
                if( isset( $node[$pathparts[0]] ) )
                {
                    $node = $node[$pathparts[0]];
                    array_shift( $pathparts );
                }
                else
                {
                    return $default;
                }
            }
            return isset( $node[$pathparts[0]] ) ? $node[$pathparts[0]] : $default;
        }

    }
