<?php
    require_once KCONFIG . '/classes/classKYaml.php';
    require_once KCONFIG . '/classes/KGatewayHardwareInfo.php';

    class KGatewaySwitchYamlException extends KYamlException
    {
        /* VOID */
    }

    class KGatewaySwitchYamlVLANModeValidator implements KYamlValidator
    {
        /**
         * Return validator name
         *
         * @return string name
         */
        public function getName( )
        {
            return 'VLANMode';
        }

        /**
         * Check if $data is valid and return true if it is, anything else
         * otherwise.
         * Throw exception in case of invalid data
         *
         * @param KYaml $yaml YAML object being validated
         * @param array $rules Current rule set (as in KYaml::validate)
         * @param mixed $data Current data (as in KYaml::validate)
         * @param string $path Parent
         */
        public function validate( KYaml $yaml, &$rule, &$data, &$path )
        {
            if( !in_array( $data, KGatewaySwitchYaml::$vlanmodes ) )
            {
                throw new KGatewaySwitchYamlException( 'gateway-switch-error-invalid-vlan-mode' );
            }
        }
    }

    class KGatewaySwitchYamlVLANPortNameReferenceValidator implements KYamlValidator
    {
        /**
         * Return validator name
         *
         * @return string name
         */
        public function getName( )
        {
            return 'VLANPortNameReference';
        }

        /**
         * Check if $data is valid and return true if it is, anything else
         * otherwise.
         * Throw exception in case of invalid data
         *
         * @param KYaml $yaml YAML object being validated
         * @param array $rules Current rule set (as in KYaml::validate)
         * @param mixed $data Current data (as in KYaml::validate)
         * @param string $path Parent
         */
        public function validate( KYaml $yaml, &$rule, &$data, &$path )
        {
            $info = KGatewayHardwareInfo::getInstance( );

            if( is_null( $info->getSwitchPortConfig( $data ) ) )
            {
                throw new KGatewaySwitchYamlException( 'gateway-switch-error-invalid-vlan-mode' );
            }
        }
    }

    class KGatewaySwitchYamlDataLinkPublicPortNameReferenceValidator extends KGatewaySwitchYamlVLANPortNameReferenceValidator
    {
        /**
         * Return validator name
         *
         * @return string name
         */
        public function getName( )
        {
            return 'DataLinkPublicPortNameReference';
        }

        /**
         * Check if $data is valid and return true if it is, anything else
         * otherwise.
         * Throw exception in case of invalid data
         *
         * @param KYaml $yaml YAML object being validated
         * @param array $rules Current rule set (as in KYaml::validate)
         * @param mixed $data Current data (as in KYaml::validate)
         * @param string $path Parent
         */
        public function validate( KYaml $yaml, &$rule, &$data, &$path )
        {
            parent::validate( $yaml, $rule, $data, $path );
            if( ! $yaml->hasVLANConfigWithPortName( $data ) )
            {
                throw new KGatewaySwitchYamlException( 'gateway-switch-error-data-link-public-port-does-not-have-vlan-configuration' );
            }
        }
    }

    class KGatewaySwitchYamlDataLinkPrivatePortNameReferenceValidator extends KGatewaySwitchYamlVLANPortNameReferenceValidator
    {
        /**
         * Return validator name
         *
         * @return string name
         */
        public function getName( )
        {
            return 'DataLinkPrivatePortNameReference';
        }

        /**
         * Check if $data is valid and return true if it is, anything else
         * otherwise.
         * Throw exception in case of invalid data
         *
         * @param KYaml $yaml YAML object being validated
         * @param array $rules Current rule set (as in KYaml::validate)
         * @param mixed $data Current data (as in KYaml::validate)
         * @param string $path Parent
         */
        public function validate( KYaml $yaml, &$rule, &$data, &$path )
        {
            parent::validate( $yaml, $rule, $data, $path );
            if( $yaml->hasVLANConfigWithPortName( $data ) )
            {
                throw new KGatewaySwitchYamlException( 'gateway-switch-error-data-link-private-port-should-not-have-vlan-configuration' );
            }
        }
    }

    class KGatewaySwitchYamlDataLinkUniqueValidator implements KYamlValidator
    {
        /**
         * Return validator name
         *
         * @return string name
         */
        public function getName( )
        {
            return 'DataLinkUnique';
        }

        /**
         * Check if $data is valid and return true if it is, anything else
         * otherwise.
         * Throw exception in case of invalid data
         *
         * @param KYaml $yaml YAML object being validated
         * @param array $rules Current rule set (as in KYaml::validate)
         * @param mixed $data Current data (as in KYaml::validate)
         * @param string $path Parent
         */
        public function validate( KYaml $yaml, &$rule, &$data, &$path )
        {
            if( count( $data ) > 1 )
            {
                throw new KGatewaySwitchYamlException( 'gateway-switch-error-data-link-only-one-definition-is-supported' );
            }
        }
    }

    class KGatewaySwitchYamlVLANIdValidator implements KYamlValidator
    {
        /**
         * Return validator name
         *
         * @return string name
         */
        public function getName( )
        {
            return 'VLANId';
        }

        /**
         * Check if $data is valid and return true if it is, anything else
         * otherwise.
         * Throw exception in case of invalid data
         *
         * @param KYaml $yaml YAML object being validated
         * @param array $rules Current rule set (as in KYaml::validate)
         * @param mixed $data Current data (as in KYaml::validate)
         * @param string $path Parent
         */
        public function validate( KYaml $yaml, &$rule, &$data, &$path )
        {
            if( $data < 2 || $data > 3999 )
            {
                throw new KGatewaySwitchYamlException( 'gateway-switch-error-restricted-vlan-id' );
            }

            $yamldoc = $yaml->getYaml();
            $counter = 0;

            foreach( $yamldoc['VLAN'] as &$cfg )
            {
                if( $cfg['Id'] == $data )
                {
                    ++$counter;
                }
            }

            foreach( $yamldoc['DataLink'] as &$cfg )
            {
                if( $cfg['Id'] == $data )
                {
                    ++$counter;
                }
            }

            if( $counter > 1 )
            {
                throw new KGatewaySwitchYamlException( 'gateway-switch-error-duplicated-vlan-id' );
            }
        }
    }

    class KGatewaySwitchYaml extends KYaml
    {
        public static $vlanmodes = array( 'UNTAGGED', 'MEMBER' );
        const DEFAULT_VLAN_MODE = 0;
        // UNTAGGED

        public function __construct( )
        {
            //@formatter:off
            $this->rules = array(
                'VLAN' => array(
                    'type' => KYaml::TYPE_LIST,
                    'flags' => KYaml::FLAG_NORMALIZE | KYaml::FLAG_EMPTY,
                    'children' => array(
                        '?' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'children' => array(
                                'PortName' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'validator' => 'VLANPortNameReference'
                                ), // VLAN.?.PortName
                                'Id' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'validator' => 'VLANId'
                                ), // VLAN.?.Id
                                'Mode' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'validator' => 'VLANMode'
                                ) // VLAN.?.Mode
                            )
                        ) // VLAN.?
                    )
                ), // VLAN
                'DataLink' => array(
                    'type' => KYaml::TYPE_LIST,
                    'flags' => KYaml::FLAG_NORMALIZE | KYaml::FLAG_EMPTY | KYaml::FLAG_NOTREQUIRED,
                    'validator' => 'DataLinkUnique',
                    'children' => array(
                        '?' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'children' => array(
                                'PublicPortName' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'validator' => 'DataLinkPublicPortNameReference'
                                ), // DataLink.?.PublicPortName
                                'PrivatePortName' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'validator' => 'DataLinkPrivatePortNameReference'
                                ), // DataLink.?.PrivatePortName
                                'Id' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'validator' => 'VLANId'
                                ), // DataLink.?.Id
                                'PrivatePortMode' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'validator' => 'VLANMode'
                                )
                            )
                        ) // DataLink.?
                    )
                ) // DataLink
            );
            //@formatter:on
            $app = KSystem::getInstance( );
            parent::__construct( $app->getConfigDir( ) . DIRECTORY_SEPARATOR . 'kgateway-switch.yaml' );

            // validators
            $this->addValidator( new KGatewaySwitchYamlVLANModeValidator( ) );
            $this->addValidator( new KGatewaySwitchYamlVLANPortNameReferenceValidator( ) );
            $this->addValidator( new KGatewaySwitchYamlVLANIdValidator( ) );
            $this->addValidator( new KGatewaySwitchYamlDataLinkPublicPortNameReferenceValidator( ) );
            $this->addValidator( new KGatewaySwitchYamlDataLinkPrivatePortNameReferenceValidator( ) );
            $this->addValidator( new KGatewaySwitchYamlDataLinkUniqueValidator( ) );
        }

        ////////////////////////////////////////////////////////////////////////
        // VLAN maninpulation
        ////////////////////////////////////////////////////////////////////////

        /**
         * Get VLAN configuration array reference.
         *
         * @param string $name Port name
         * @return &array|null
         */
        protected function &_getVLANConfigurationByPortName( $name )
        {
            static $null = null;

            foreach( $this->yaml['VLAN'] as &$vlan )
            {
                if( $vlan['PortName'] === $name )
                {
                    return $vlan;
                }
            }

            return $null;
        }

        /**
         * Create configuration array for VLAN port.
         *
         * @param string $name Port name
         * @param string|int $id VLAN id
         * @param string $mode VLAN mode
         * @return array
         */
        protected function _createVLANConfiguration( $name, $id, $mode )
        {
            //@formatter:off
            return array(
                'PortName' => $name,
                'Id' => $id,
                'Mode' => $mode
            );
            //@formatter:on
        }

        /**
         * Check if VLAN configuration exists
         *
         * @param string $pname Port name
         * @return bool
         */
        public function hasVLANConfigWithPortName( $pname )
        {
            foreach( $this->yaml['VLAN'] as &$vlan )
            {
                if( $vlan['PortName'] === $pname )
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * Get the VLAN configuration array, filtered by PortName. If does not
         * exists, then use hardware info to create a configuration using the
         * default VLAN id. If the hardware info does not exists, then this
         * method returns null.
         *
         * @param string $name Port name
         * @return array|null
         */
        public function getVLANConfigByPortName( $name )
        {
            $myconfig = &$this->_getVLANConfigurationByPortName( $name );
            if( !is_null( $myconfig ) )
            {
                return $myconfig;
            }

            $hwinfo = KGatewayHardwareInfo::getInstance( );
            $port = $hwinfo->getSwitchPortConfig( $name );
            if( !is_null( $port ) )
            {
                return $this->_createVLANConfiguration( $name, $port['DefaultVlanId'], self::$vlanmodes[self::DEFAULT_VLAN_MODE] );
            }

            return null;
        }

        /**
         * Delete VLAN configuration for the supplied port name
         *
         * @param string $pname Port name
         */
        public function deleteVLANConfigByPortName( $pname )
        {
            foreach( $this->yaml['VLAN'] as $idx => $cfg )
            {
                if( $cfg['PortName'] === $pname )
                {
                    unset( $this->yaml['VLAN'][$idx] );
                    break;
                }
            }
        }

        /**
         * Get the VLAN Port Name filtered by Id.
         *
         * @param  integer $id
         * @return string|null
         */
        public function getVLANPortNameById( $id )
        {
            foreach( $this->yaml['VLAN'] as $vlan )
            {
                if( $vlan['Id'] == $id )
                {
                    return $vlan['PortName'];
                }
            }

            $hwinfo = KGatewayHardwareInfo::getInstance( );

            return $hwinfo->getSwitchPortName( $id );
        }

        /**
         * Set VLAN configuration.
         *
         * @param string $name Port name
         * @param string|int $id VLAN id
         * @param string $mode VLAN mode
         */
        public function setVLANConfig( $name, $id, $mode )
        {
            $myconfig = &$this->_getVLANConfigurationByPortName( $name );
            if( is_null( $myconfig ) )
            {
                $this->yaml['VLAN'][] = $this->_createVLANConfiguration( $name, $id, $mode );
            }
            else
            {
                $myconfig['Id'] = $id;
                $myconfig['Mode'] = $mode;
            }
        }

        ////////////////////////////////////////////////////////////////////////
        // Data link configurations
        ////////////////////////////////////////////////////////////////////////
        /**
         * Get Data link information reference for supplied (private) port name
         *
         * @param string $pname Private port name
         * @return array|null
         */
        protected function & _getDataLinkConfigByPrivatePortName( $pname )
        {
            static $null = null;

            foreach( $this->yaml['DataLink'] as &$cfg )
            {
                if( $cfg['PrivatePortName'] === $pname )
                {
                    return $cfg;
                }
            }
            return $null;
        }

        /**
         * Check if Data link configuration exists (private port)
         *
         * @param string $pname Private port name
         * @return bool
         */
        public function hasDataLinkConfigWithPrivatePortName( $pname )
        {
            foreach( $this->yaml['DataLink'] as &$cfg )
            {
                if( $cfg['PrivatePortName'] === $pname )
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * Get Data link information for supplied (private) port name
         *
         * @param string $pname Private port name
         * @return array|null
         */
        public function getDataLinkConfigByPrivatePortName( $pname )
        {
            return $this->_getDataLinkConfigByPrivatePortName( $pname );
        }

        /**
         * Delete Data link information for the supplied (private) port name
         *
         * @param string $pname Private port name
         */
        public function deleteDataLinkConfigByPrivatePortName( $pname )
        {
            foreach( $this->yaml['DataLink'] as $idx => $cfg )
            {
                if( $cfg['PrivatePortName'] === $pname )
                {
                    unset( $this->yaml['DataLink'][$idx] );
                    break;
                }
            }
        }

        /**
         * Set Data link information
         *
         *
         */
        public function setDataLinkConfig( $publicportname, $privateportname, $privateportmode, $vlanid )
        {
            if( is_null( $publicportname ) || is_null( $privateportname ) || is_null( $privateportmode ) || is_null( $vlanid ) )
            {
                throw new KGatewaySwitchYamlException( 'gateway-switch-error-invalid-parameters' );
            }

            $currentcfg = &$this->_getDataLinkConfigByPrivatePortName( $privateportname );
            if( is_null( $currentcfg ) )
            {
                if( !isset( $this->yaml['DataLink'] ) )
                {
                    $this->yaml['DataLink'] = array();
                }
                $this->yaml['DataLink'][] = array( 'PublicPortName' => $publicportname, 'PrivatePortName' => $privateportname, 'PrivatePortMode' => $privateportmode, 'Id' => $vlanid );
            }
            else
            {
                $currentcfg['PublicPortName'] = $publicportname;
                $currentcfg['PrivatePortMode'] = $privateportmode;
                $currentcfg['Id'] = $vlanid;
            }
        }
    }
