<?php
    require_once KCONFIG . '/classes/classKYaml.php';

    class KLcdYamlException extends KYamlException
    {
        /* VOID */
    }

    class KLcdYaml extends KYaml
    {
        public static $screenTypes = array( 'Message', 'Resource', 'Network', 'Process', 'Custom' );

        const SCREEN_TYPE_MESSAGE = 0;
        const SCREEN_TYPE_RESOURCE = 1;
        const SCREEN_TYPE_NETWORK = 2;
        const SCREEN_TYPE_PROCESS = 3;
        const SCREEN_TYPE_CUSTOM = 4;

        const DEFAULT_REFRESH_TIMEOUT = 2000;

        public function __construct( )
        {
            // validation rules
            //@formatter:off
            $this->rules = array(
                'Lcd' => array(
                    'type' => KYaml::TYPE_LISTHASH,
                    'children' => array(
                        'DefaultScreen' => array(
                            'type' => KYaml::TYPE_INTEGER,
                            'default' => 0
                        ), // Lcd.DefaultScreen
                        'RefreshTimeout' => array(
                            'type' => KYaml::TYPE_INTEGER,
                            'default' => KLcdYaml::DEFAULT_REFRESH_TIMEOUT
                        ), // Lcd.RefreshTimeout
                        'Screens' => array(
                            'type' => KYaml::TYPE_LIST,
                            'flags' => KYaml::FLAG_NORMALIZE | KYaml::FLAG_EMPTY,
                            'default' => array(),
                            'children' => array(
                                '?' => array(
                                    'type' => KYaml::TYPE_LISTHASH,
                                    'children' => array(
                                        'Type' => array(
                                            'type' => KYaml::TYPE_STRING,
                                        ), // Lcd.Screens.?.Type
                                        'Line1' => array(
                                            'type' => KYaml::TYPE_STRING,
                                            'flags' => KYaml::FLAG_NOTREQUIRED
                                        ), // Lcd.Screens.?.Line1
                                        'Line2' => array(
                                            'type' => KYaml::TYPE_STRING,
                                            'flags' => KYaml::FLAG_NOTREQUIRED
                                        ), // Lcd.Screens.?.Line2
                                        'Label1' => array(
                                            'type' => KYaml::TYPE_STRING,
                                            'flags' => KYaml::FLAG_NOTREQUIRED
                                        ), // Lcd.Screens.?.Label1
                                        'Label2' => array(
                                            'type' => KYaml::TYPE_STRING,
                                            'flags' => KYaml::FLAG_NOTREQUIRED
                                        ), // Lcd.Screens.?.Label2
                                        'Name1' => array(
                                            'type' => KYaml::TYPE_STRING,
                                            'flags' => KYaml::FLAG_NOTREQUIRED
                                        ), // Lcd.Screens.?.Name1
                                        'Name2' => array(
                                            'type' => KYaml::TYPE_STRING,
                                            'flags' => KYaml::FLAG_NOTREQUIRED
                                        ), // Lcd.Screens.?.Name2
                                        'Command1' => array(
                                            'type' => KYaml::TYPE_STRING,
                                            'flags' => KYaml::FLAG_NOTREQUIRED
                                        ), // Lcd.Screens.?.Command1
                                        'Command2' => array(
                                            'type' => KYaml::TYPE_STRING,
                                            'flags' => KYaml::FLAG_NOTREQUIRED
                                        ), // Lcd.Screens.?.Command2
                                    )
                                ) // Lcd.Screens.?
                            )
                        ) // Lcd.Screens
                    )
                ) // Lcd
            );
            //@formatter:on
            $app = KSystem::getInstance( );
            parent::__construct( $app->getConfigDir( ) . DIRECTORY_SEPARATOR . 'lcd.yaml' );

            if( !isset( $this->yaml['Lcd'] ) )
            {
                //@formatter:off
                $this->yaml = array(
                    'Lcd' => array(
                        'DefaultScreen' => 0,
                        'RefreshTimeout' => KLcdYaml::DEFAULT_REFRESH_TIMEOUT,
                        'Screens' => array()
                    )
                );
                //@formatter:on
            }
        }

        ////////////////////////////////////////////////////////////////////////
        // Global values
        ////////////////////////////////////////////////////////////////////////
        /**
         * Get available screen types
         *
         * @return array
         */
        public function getAllScreenTypes( )
        {
            return self::$screenTypes;
        }

        /**
         * Get screen type name by its internal id
         *
         * @param int $id valid $this->screenTypes array position
         * @return string
         * @throws KLcdYamlException
         */
        public function getScreenType( $id )
        {
            if( !is_int( $id ) || !array_key_exists( $id, self::$screenTypes ) )
            {
                throw new KLcdYamlException( 'klcd-error-invalid-screen-type' );
            }
            return self::$screenTypes[$id];
        }

        /**
         * Get the default screen value
         *
         * @return int
         */
        public function getDefaultScreen( )
        {
            return $this->yaml['Lcd']['DefaultScreen'];
        }

        /**
         * Set the default screen value
         *
         * @param int $screen Screen number (must be defined in Screens array)
         * @throws KLcdYamlException
         */
        public function setDefaultScreen( $screen )
        {
            if( !is_int( $screen ) || $this->countScreens( ) < $screen )
            {
                throw new KLcdYamlException( 'klcd-error-invalid-default-screen' );
            }
            $this->yaml['Lcd']['DefaultScreen'] = $screen;
        }

        /**
         * Get refresh timeout
         *
         * @return int
         */
        public function getRefreshTimeout( )
        {
            return $this->yaml['Lcd']['RefreshTimeout'];
        }

        /**
         * Set the refresh timeout value
         *
         * @param int $timeout Timeout value in milliseconds
         * @throws KLcdYamlException
         */
        public function setRefreshTimeout( $timeout )
        {
            if( !is_int( $timeout ) )
            {
                throw new KLcdYamlException( 'klcd-error-invalid-refresh-timeout' );
            }
            $this->yaml['Lcd']['RefreshTimeout'] = $timeout;
        }

        ////////////////////////////////////////////////////////////////////////
        // Screens manipulation
        ////////////////////////////////////////////////////////////////////////

        /**
         * Update screen
         *
         * @param int $index screen array index
         * @param array $data screen data
         * @throws KLcdYamlException
         */
        public function updateScreen( $index, $data )
        {
            if( !is_int( $index ) || !array_key_exists( $index, $this->yaml['Lcd']['Screens'] ) )
            {
                throw new KLcdYamlException( 'klcd-error-invalid-screen-index' );
            }

            $this->yaml['Lcd']['Screens'][$index] = $data;
        }

        /**
         * Append screen
         *
         * @param array $data screen data
         */
        public function appendScreen( $data )
        {
            $this->yaml['Lcd']['Screens'][] = $data;
        }

        /**
         * Delete screen
         *
         * @param int $index screen array index
         * @throws KLcdYamlException
         */
        public function deleteScreen( $index )
        {
            if( !is_int( $index ) || !array_key_exists( $index, $this->yaml['Lcd']['Screens'] ) )
            {
                throw new KLcdYamlException( 'klcd-error-invalid-screen-index' );
            }

            unset( $this->yaml['Lcd']['Screens'][$index] );
        }

        /**
         * Delete all screens
         */
        public function deleteAllScreens( )
        {
            $this->yaml['Lcd']['Screens'] = array( );
        }

        /**
         * Count defined screens
         *
         * @return int
         */
        public function countScreens( )
        {
            return count( $this->yaml['Lcd']['Screens'] );
        }

    }
