<?php
    require_once KCONFIG . '/classes/classKYaml.php';

    class KNotificationYamlException extends KYamlException
    {
        /* VOID */
    }

    // check if $data is a valid transport reference name
    class KNotificationTransportReferenceValidator implements KYamlValidator
    {
        public function getName()
        {
            return 'TransportReference';
        }
        public function validate( KYaml $yaml, &$rule, &$data, &$path )
        {
            if( ! empty( $data ) && ! $yaml->hasTransport( $data ) )
            {
                throw new KNotificationYamlException( 'gateway-notification-invalid-transport-reference' );
            }
        }
    }

    // check if $data is a valid transport object
    class KNotificationTransportConsistencyValidator implements KYamlValidator
    {
        public function getName()
        {
            return 'TransportConsistency';
        }
        public function validate( KYaml $yaml, &$rule, &$data, &$path )
        {
            if( $data['Protocol'] == 'HTTP' )
            {
                if( empty( $data['HTTP']['Method'] ) )
                {
                    throw new KNotificationYamlException( 'gateway-notification-invalid-transport' );
                }
            }
        }
    }

    class KNotificationYaml extends KYaml
    {
        public static $protocols = array( 'HTTP' );
        public static $httpmethods = array( 'GET', 'POST' );

        public function __construct( )
        {
            // validation rules
            //@formatter:off
            $this->rules = array(
                'TransportMapping' => array(
                    'type' => KYaml::TYPE_LIST,
                    'flags' => KYaml::FLAG_NORMALIZE | KYaml::FLAG_EMPTY,
                    'default' => array(),
                    'children' => array(
                        '?' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'default' => array(),
                            'children' => array(
                                'Type' => array(
                                    'type' => KYaml::TYPE_STRING,
                                ), // TransportMapping.?.Type
                                'Transport' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_EMPTY,
                                    'validator' => 'TransportReference'
                                ), // TransportMapping.?.Transport
                            )
                        ) // TransportMapping.?
                    )
                ), // TransportMapping
                'Transports' => array(
                    'type' => KYaml::TYPE_LIST,
                    'flags' => KYaml::FLAG_NORMALIZE | KYaml::FLAG_EMPTY,
                    'default' => array(),
                    'children' => array(
                        '?' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'validator' => 'TransportConsistency',
                            'children' => array(
                                'Name' => array(
                                    'type' => KYaml::TYPE_STRING
                                ), // Transports.?.Name
                                'Protocol' => array(
                                    'type' => KYaml::TYPE_STRING
                                ), // Transports.?.Protocol
                                'Username' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ), // Transports.?.Username
                                'Password' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ), // Transports.?.Password
                                'Host' => array(
                                    'type' => KYaml::TYPE_STRING
                                ), // Transports.?.Host
                                'Port' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ), // Transports.?.Port
                                'Path' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_NOTREQUIRED,
                                    'default' => '/'
                                ), // Transports.?.Path
                                ////////////////////////////////////////////////
                                // Per protocol options
                                ////////////////////////////////////////////////
                                'HTTP' => array(
                                    'type' => KYaml::TYPE_LISTHASH,
                                    'flags' => KYaml::FLAG_NOTREQUIRED,
                                    'children' => array(
                                        'Method' => array(
                                            'type' => KYaml::TYPE_STRING
                                        ), // Transports.?.HTTP.Method
                                        ////////////////////////////////////////
                                        // Per HTTP method options
                                        ////////////////////////////////////////
                                        'POST' => array(
                                            'type' => KYaml::TYPE_LISTHASH,
                                            'flags' => KYaml::FLAG_NOTREQUIRED,
                                            'children' => array(
                                                'AdditionalParams' => array(
                                                    'type' => KYaml::TYPE_LIST,
                                                    'default' => array(),
                                                    'children' => array(
                                                        '?' => array(
                                                            'type' => KYaml::TYPE_LISTHASH,
                                                            'flags' => KYaml::FLAG_EMPTY,
                                                            'children' => array(
                                                                'Name' => array(
                                                                    'type' => KYaml::TYPE_STRING
                                                                ), // Transports.?.POST.AdditionalParams.?.Name
                                                                'Value' => array(
                                                                    'type' => KYaml::TYPE_STRING,
                                                                    'flags' => KYaml::FLAG_EMPTY
                                                                ), // Transports.?.POST.AdditionalParams.?.Value
                                                            )
                                                        ) // Transports.?.POST.AdditionalParams.?
                                                    )
                                                ) // Transports.?.POST.AdditionalParams
                                            )
                                        ) // Transports.?.POST
                                    )
                                ) // Transports.?.HTTP
                            )
                        ) // Transports.?
                    )
                ), // Transports
                'Settings' => array(
                    'type' => KYaml::TYPE_LISTHASH,
                    'flags' => KYaml::FLAG_NOTREQUIRED,
                    'children' => array(
                        'WorkReadAhead' => array(
                            'type' => KYaml::TYPE_INTEGER,
                            'flags' => KYaml::FLAG_NOTREQUIRED
                        ), // Settings.WorkReadAhead
                        'DeferredMaximumRetry' => array(
                            'type' => KYaml::TYPE_INTEGER,
                            'flags' => KYaml::FLAG_NOTREQUIRED
                        ), // Settings.DeferredMaximumRetry
                        'DeferredRetryDelay' => array(
                            'type' => KYaml::TYPE_INTEGER,
                            'flags' => KYaml::FLAG_NOTREQUIRED
                        ), // Settings.DeferredRetryDelay
                        'TrashTimeout' => array(
                            'type' => KYaml::TYPE_INTEGER,
                            'flags' => KYaml::FLAG_NOTREQUIRED
                        ) // Settings.TrashTimeout
                    )
                ) // Settings
            );
            //@formatter:on
            $app = KSystem::getInstance( );
            parent::__construct( $app->getConfigDir( ) . DIRECTORY_SEPARATOR . 'knotification.yaml' );

            // validators
            $this->addValidator( new KNotificationTransportReferenceValidator() );
            $this->addValidator( new KNotificationTransportConsistencyValidator() );
        }

        ////////////////////////////////////////////////////////////////////////
        // Global values
        ////////////////////////////////////////////////////////////////////////

        ////////////////////////////////////////////////////////////////////////
        // Transports manipulation
        ////////////////////////////////////////////////////////////////////////

        /**
         * Check if $name transport exists
         * 
         * @param string $name transport name
         * @return bool
         */
        public function hasTransport( $name )
        {
            foreach( $this->yaml['Transports'] as &$transport )
            {
                if( $transport['Name'] === $name )
                {
                    return true;
                }
            }
            return false;
        }
    }
