<?php
    require_once (dirname( __FILE__ ) . "/classKYaml.php");
    require_once (dirname( __FILE__ ) . "/classKProfiles.php");
    require_once (dirname( __FILE__ ) . "/../placas.php");

    class KSistema
    {
        var $language, $placas, $devices, $CTbusBoards, $boardError, $SS7Boards, $installDir;

        function __construct( $lang )
        {
            $this->setLanguage( $lang );
            $this->boardError = "";
            $this->setPlacas( );
            $this->setDevices( );
            $this->setCTbusBoards( );
            $this->setSS7Boards( );

            $this->installDir  = $this->getInstallDir( );
        }

        function retTranslate( $word )
        {
            Global $dicionario;

            if( isset( $dicionario[$this->getLanguage( )][$word] ) )
            {
                return $dicionario[$this->getLanguage( )][$word];
            }
            else
            {
                trigger_error( "Undefined translation key {$word} ({$this->getLanguage()})" );
                return $word;
            }
        }

        function setLanguage( $lang )
        {
            $this->language = $lang;
        }

        function getLanguage( )
        {
            return $this->language;
        }

        function getPlacas( )
        {
            return $this->placas;
        }

        function getDevices( )
        {
            return $this->devices;
        }

        function getDeviceLinksList( $device )
        {
            $E1DeviceLinks = $this->getDeviceLinks( $device );
            return $E1DeviceLinks;
        }

        function changeDeviceOrder( $deviceList )
        {
            require_once KCONFIG . '/classes/KDevicesYaml.php';
            $devicesYaml = new KDevicesYaml( );
            $dev = $devicesYaml->getYaml( );
            $devices = $dev['Devices'];

            $newDeviceOrder = array( );
            $deviceSerials = explode( "|", $deviceList );
            for( $i = 0; $i < sizeof( $deviceSerials ); $i++ )
            {
                for( $j = 0; $j < sizeof( $devices ); $j++ )
                {
                    $key = array_keys( $devices[$j] );
                    if( $key[0] == $deviceSerials[$i] )
                        $newDeviceOrder[] = $devices[$j];
                }
            }
            $dev['Devices'] = $newDeviceOrder;
            $devicesYaml->setYaml( $dev );
            $devicesYaml->selfAssignEbsIpAddresses( );
            $devicesYaml->saveYaml( true );
        }

        function getDeviceChannelList( $device )
        {
            $channels = array( );
            $totalChannels = 0;
            $selectedDevice = $this->getYAMLDevice( $device );

            $isT1 = $selectedDevice[$device]['Links'][0][0]['Type'] == 'T1' ? true : false;
            $multi = $isT1 ? 24 : 30;

            if( isset( $selectedDevice[$device]['ChannelGroups'] ) )
            {
                $CGs = $selectedDevice[$device]['ChannelGroups'];
                for( $j = 0; $j < sizeof( $CGs ); $j++ )
                {
                    $keys = array_keys( $CGs[$j] );
                    $channelPiece = explode( "-", $keys[0] );
                    $totalChannels = ($channelPiece[1] + 1);
                }
            }
            elseif( isset( $selectedDevice[$device]['Links'] ) )
                $totalChannels = sizeof( $selectedDevice[$device]['Links'] ) * $multi;

            for( $i = 0; $i < $totalChannels; $i++ )
                $channels[] = $i;

            return $channels;
        }

        function getDeviceSerialByPosition( $position )
        {
            $devices = $this->getDeviceSerials( );
            return $serials[$position];
        }
        function getDevicePositionBySerial( $serial )
        {
            $serials = $this->getDeviceSerials();
            for($i=0; $i < count($serials); ++$i)
                if($serial == $serials[$i])
                    return $i;
            return -1;
        }

        function getDeviceSerials( )
        {
            $DevicesYAML = new KYaml( $this->getConfigDir( ) . "/devices.yaml" );
            $yaml = $DevicesYAML->getYaml( );
            $devices = $yaml['Devices'];

            $serials = array( );
            for( $i = 0; $i < sizeof( $devices ); $i++ )
            {
                $ch = array_keys( $devices[$i] );
                $serials[] = $ch[0];
            }

            return $serials;
        }

        function getCTbusBoards( )
        {
            return $this->CTbusBoards;
        }

        function getConfigDir( )
        {
            Global $_SESSION;

            if( PHP_OS == "WINNT" )
            {
                if( !isset( $_SESSION['configPath'] ) )
                {
                    $winVersion = PHP_WINDOWS_VERSION_MAJOR . "." . PHP_WINDOWS_VERSION_MINOR;
                    exec( KROOT . '\\..\\..\\Tools\\kwebhelper configdir', $sysdir );
                    if( $winVersion >= 6.1 )
                    {
                        $sysdata = explode( "\\", $sysdir[0] );
                        if( sizeof( $sysdata ) <= 3 )
                            $sysdir = $sysdata[0] . "\\ProgramData\\" . $sysdata[1] . "\\" . $sysdata[2];
                    }
                    $_SESSION['configPath'] = $sysdir[0];
                }
            }
            else
                $_SESSION['configPath'] = "/etc/khomp/config";

            return $_SESSION['configPath'];
        }

        function getInstallDir( )
        {
            Global $_SESSION;

            if( !isset( $_SESSION['installDir'] ) )
            {
                $configDir = $this->getConfigDir( );
                if( PHP_OS == "WINNT" )
                {
                    $tmpDir = explode( "\\", $configDir );
                    $workDirectory = '';
                    for( $i = 0; $i < (sizeof( $tmpDir ) - 1); $i++ )
                        $workDirectory .= $tmpDir[$i] . "\\";
                }
                else
                    $workDirectory = "/etc/khomp/";
                $_SESSION['installDir'] = $workDirectory;
            }
            else
                $workDirectory = $_SESSION['installDir'];
            return $workDirectory;
        }

        function setPlacas( )
        {

            Global $boardNames, $_SESSION;
            $data = array( );
            $placas = array( );

            $prog = PHP_OS == "WINNT" ? KROOT . '\\..\\..\\Tools\\kwebhelper.exe' : '/usr/sbin/kwebhelper';

            if( !file_exists( $prog ) )
            {
                $this->boardError .= "Recurso necessário para o funcionamento do KWebConfig não encontrado. Favor reinstalar a API K3L.";
                $this->placas = $placas;
                return;
            }

            exec( $prog . ' boarddump', $data );

            for( $a = 0; $a < sizeof( $data ); $a++ )
            {
                $dadosPlaca = explode( ":", $data[$a] );
                if( sizeof( $dadosPlaca ) == 5 )
                {
                    $rawboardData[$a] = $dadosPlaca;
                    $placas[$a][0] = trim( $dadosPlaca[4] );
                    //serial
                    $placas[$a][1] = $boardNames[trim( $dadosPlaca[0] )][trim( $dadosPlaca[1] )][trim( $dadosPlaca[2] )];
                    //nome
                    $placas[$a][2] = trim( $dadosPlaca[3] );
                    //canais
                    $placas[$a][3] = trim( $dadosPlaca[0] );
                    //type
                    $placas[$a][4] = trim( $dadosPlaca[2] );
                }
            }

            $_SESSION['boards'] = $placas;
            $this->placas = $placas;
        }

        function getBoardDevices( )
        {
            Global $boardNames;
            $count = 0;
            for( $x = 0; $x <= 16; $x++ )
            {
                if( isset( $boardNames[$x] ) )
                {
                    $fkeys = array_keys( $boardNames[$x] );
                    for( $y = 0; $y < sizeof( $fkeys ); $y++ )
                    {
                        $skeys = array_keys( $boardNames[$x][$fkeys[$y]] );
                        for( $z = 0; $z < sizeof( $skeys ); $z++ )
                        {
                            $boards[$count]['name'] = $boardNames[$x][$fkeys[$y]][$skeys[$z]];
                            $boards[$count]['id'] = $x . "|" . $fkeys[$y] . "|" . $skeys[$z];
                            $boards[$count]['link'] = $skeys[$z];
                            $count++;
                        }
                    }
                }
            }
            return $boards;
        }

        function deleteDevice( $serial )
        {
            Global $devices, $boardNames, $_SESSION;

            $devicesYaml = new KYaml( $this->getConfigDir( ) . "/devices.yaml" );
            $dev = $devicesYaml->getYaml( );
            $serialId = $this->getSerialId( $serial, $dev );

            $devicesYaml->deleteVar( $serialId, 'Devices' );
            $devicesYaml->sortKeys( 'Devices' );

            $devicesYaml->saveYaml( true );
            $this->removeDeviceFromConfig( $serial );

        }

        function removeDeviceFromConfig( $serial )
        {
            $systemYAML = new KYaml( $this->getConfigDir( ) . "/system.yaml" );
            $yaml = $systemYAML->getYaml( );
            $EnableLinkAsReference = $yaml['System']['EnableLinkAsReference'];

            if( $EnableLinkAsReference != '' )
            {
                $tmp = explode( ',', $EnableLinkAsReference );
                $max = sizeof( $tmp );
                for( $i = 0; $i < $max; $i++ )
                {
                    $aux = explode( '.', $tmp[$i] );
                    if( $aux[0] == $serial )
                        unset( $tmp[$i] );
                }
                $yaml['System']['EnableLinkAsReference'] = sizeof( $tmp ) > 0 ? implode( ',', $tmp ) : '';
            }

            $IncomingDigitsCount = $yaml['System']['IncomingDigitsCount'];
            if( sizeof( $IncomingDigitsCount ) > 0 )
            {
                $keys = array_keys( $IncomingDigitsCount );
                foreach( $keys as $k )
                {
                    $tmp = explode( '.', $k );
                    if( $tmp[0] == $serial )
                        unset( $IncomingDigitsCount[$k] );
                }
            }
            $yaml['System']['IncomingDigitsCount'] = $IncomingDigitsCount;

            $CRC4 = $yaml['System']['CRC4'];
            if( $CRC4 != '' )
            {
                $finalCRC = array( );
                $CRC4 = explode( ',', $CRC4 );
                foreach( $CRC4 as $crc )
                {
                    $tmp = explode( '.', $crc );
                    if( $tmp[0] != $serial )
                        $finalCRC[] = $crc;
                }

                if( sizeof( $finalCRC ) == 0 )
                    $yaml['System']['CRC4'] = '';
                else
                    $yaml['System']['CRC4'] = implode( ',', $finalCRC );
            }

            $UseCentralOfficePrefixOnIncomingCalls = $yaml['System']['UseCentralOfficePrefixOnIncomingCalls'];
            if( $UseCentralOfficePrefixOnIncomingCalls != '' )
            {
                $final = array( );
                $UseCentralOfficePrefixOnIncomingCalls = explode( ',', $UseCentralOfficePrefixOnIncomingCalls );
                foreach( $UseCentralOfficePrefixOnIncomingCalls as $u )
                {
                    $tp = explode( '.', $u );
                    if( $tp[0] != $serial )
                        $final[] = $u;
                }
                if( sizeof( $final ) == 0 )
                    $yaml['System']['UseCentralOfficePrefixOnIncomingCalls'] = '';
                else
                    $yaml['System']['UseCentralOfficePrefixOnIncomingCalls'] = implode( ',', $final );
            }

            $CentralOfficePrefix = $yaml['System']['CentralOfficePrefix'];
            if( sizeof( $CentralOfficePrefix ) > 0 )
            {
                $finalCOP = array( );
                foreach( $CentralOfficePrefix as $k => $v )
                {
                    $temp = explode( '.', $k );
                    if( $temp[0] != $serial )
                        $finalCOP[$k] = $v;
                }

                $yaml['System']['CentralOfficePrefix'] = $finalCOP;
            }

            $CasLineValidateTime = $yaml['System']['CasLineValidateTime'];
            if( sizeof( $CasLineValidateTime ) > 0 )
            {
                $finalCLVT = array( );
                foreach( $CasLineValidateTime as $k => $v )
                {
                    $temp = explode( '.', $k );
                    if( $temp[0] != $serial )
                        $finalCLVT[$k] = $v;
                }

                $yaml['System']['CasLineValidateTime'] = $finalCLVT;
            }

            $LinkTimers = $yaml['System']['LinkTimers'];
            if( sizeof( $LinkTimers ) > 0 )
            {
                $finalLT = array( );
                $keys = array_keys( $LinkTimers );
                foreach( $keys as $k )
                {
                    $tmp = explode( '.', $k );
                    if( $tmp[0] != $serial )
                        $finalLT[$k] = $LinkTimers[$k];
                }
                $yaml['System']['LinkTimers'] = $finalLT;
            }

            $AutoCallAnalyzer = $yaml['Features']['AutoCallAnalyzer'];
            if( $AutoCallAnalyzer != '' )
            {
                $tmp = explode( ',', $AutoCallAnalyzer );
                $size = sizeof( $tmp );
                for( $j = 0; $j < $size; $j++ )
                {
                    $aux = explode( '.', $tmp[$j] );
                    if( $aux[0] == $serial )
                        unset( $tmp[$j] );
                }
                $final = sizeof( $tmp ) > 0 ? implode( ',', $tmp ) : '';
                $yaml['Features']['AutoCallAnalyzer'] = $final;
            }

            $AutoEchoCanceller = $yaml['Features']['AutoEchoCanceller'];
            if( $AutoEchoCanceller != '' )
            {
                $tmp = explode( ',', $AutoEchoCanceller );
                $size = sizeof( $tmp );
                for( $k = 0; $k < $size; $k++ )
                {
                    $aux = explode( '.', $tmp[$k] );
                    if( $aux[0] == $serial )
                        unset( $tmp[$k] );
                }
                $final = sizeof( $tmp ) > 0 ? implode( ',', $tmp ) : '';
                $yaml['Features']['AutoEchoCanceller'] = $final;
            }

            $AutoDtmfSuppression = $yaml['Features']['AutoDtmfSuppression'];
            if( $AutoDtmfSuppression != '' )
            {
                $tmp = explode( ',', $AutoDtmfSuppression );
                $size = sizeof( $tmp );
                for( $l = 0; $l < $size; $l++ )
                {
                    $aux = explode( '.', $tmp[$l] );
                    if( $aux[0] == $serial )
                        unset( $tmp[$l] );
                }
                $final = sizeof( $tmp ) > 0 ? implode( ',', $tmp ) : '';
                $yaml['Features']['AutoDtmfSuppression'] = $final;
            }

            $AutoAGC = $yaml['Features']['AutoAGC'];
            if( $AutoAGC != '' )
            {
                $tmp = explode( ',', $AutoAGC );
                $size = sizeof( $tmp );
                for( $m = 0; $m < $size; $m++ )
                {
                    $aux = explode( '.', $tmp[$m] );
                    if( $aux[0] == $serial )
                        unset( $tmp[$m] );
                }
                $final = sizeof( $tmp ) > 0 ? implode( ',', $tmp ) : '';
                $yaml['Features']['AutoAGC'] = $final;
            }

            $DropCollectCalls = $yaml['Features']['DropCollectCalls'];
            if( sizeof( $DropCollectCalls ) > 0 )
            {
                $keys = array_keys( $DropCollectCalls );
                foreach( $keys as $k )
                {
                    $tmp = explode( '.', $k );
                    if( $tmp[0] == $serial )
                        unset( $DropCollectCalls[$k] );
                }
            }
            $yaml['Features']['DropCollectCalls'] = $DropCollectCalls;

            $GSMPin = $yaml['System']['GSMPin'];
            if( sizeof( $GSMPin ) > 0 )
            {
                $keys = array_keys( $GSMPin );
                foreach( $keys as $k )
                {
                    $tmp = explode( '.', $k );
                    if( $tmp[0] == $serial )
                        unset( $GSMPin[$k] );
                }
            }
            $yaml['System']['GSMPin'] = $GSMPin;

            $FXOLineSensibility = $yaml['System']['FXOLineSensibility'];
            if( sizeof( $FXOLineSensibility ) > 0 )
            {
                $keys = array_keys( $FXOLineSensibility );
                foreach( $keys as $k )
                {
                    $tmp = explode( '.', $k );
                    if( $tmp[0] == $serial )
                        unset( $FXOLineSensibility[$k] );
                }
            }
            $yaml['System']['FXOLineSensibility'] = $FXOLineSensibility;

            $systemYAML->setYaml( $yaml );
            $systemYAML->save( );
        }

        function getSerialId( $serial, $array )
        {
            $keys = array_keys( $array['Devices'] );
            for( $x = 0; $x < sizeof( $keys ); $x++ )
                if( isset( $array['Devices'][$keys[$x]][$serial] ) && is_array( $array['Devices'][$keys[$x]][$serial] ) )
                    return $keys[$x];
        }

        function getNotInstalledDevices( $time = '' )
        {

            Global $devices, $boardNames;
            $this->setPlacas( );

            $installedBoards = $this->placas;

            $this->setDevices( );
            $dev = $this->devices;

            $notInstalledDevices = array( );

            for( $x = 0; $x < sizeof( $installedBoards ); $x++ )
            {
                $testBoard = false;
                for( $y = 0; $y < sizeof( $dev ); $y++ )
                {
                    if( $installedBoards[$x][0] == $dev[$y][0] )
                        $testBoard = true;
                }
                if( !$testBoard )
                    $notInstalledDevices[] = $installedBoards[$x];
            }
            $kibsDevices = $this->getKibsDevices( $time );

            for( $x = 0; $x < sizeof( $kibsDevices ); $x++ )
            {
                $testBoard = false;
                for( $y = 0; $y < sizeof( $dev ); $y++ )
                {
                    if( $kibsDevices[$x][0] == $dev[$y][0] )
                        $testBoard = true;
                }
                if( !$testBoard )
                    $notInstalledDevices[] = $kibsDevices[$x];
            }

            return $notInstalledDevices;
        }

        function getKibsDevices( $time = '' )
        {
            Global $invDevices;

            if( $time === '' )
                $time = 30 * 60;
            else
                $time = $time * 60;

            $workDirectory = $this->getInstallDir( );
            $targetFile = $workDirectory . "devices";

            if( is_file( $targetFile ) )
                $kibsData = file( $targetFile );
            else
                return array( );
            $count = 0;
            for( $x = 0; $x < sizeof( $kibsData ); $x++ )
            {
                $exFileData = explode( " ", $kibsData[$x] );
                $addedTime = trim( $exFileData[0] );
                $timeNow = time( );
                $restTime = $timeNow - $addedTime;
                if( ($restTime <= $time) || ($time == 0) )
                {
                    $requiredData[$count][0] = trim( $exFileData[1] );

                    // ebs version > 0
                    if( isset( $exFileData[2] ) )
                    {
                        $name = trim( $exFileData[2] );
                        $requiredData[$count][1] = $name;

                        //ebs version > 1
                        if( strpos( $name, 'EBS_' ) === 0 )
                        {
                            $version = substr( $name, 4, 2 );
                            $requiredData[$count][5] = $version;
                            $name = str_replace( '_' . $version, '', $name );
                        }
                        else
                        {
                            $requiredData[$count][5] = 'V1';
                        }

                        $requiredData[$count][1] = $name;
                    }
                    //ebs version = 0
                    else
                    {
                        $requiredData[$count][1] = "";
                        $requiredData[$count][5] = "V0";
                    }

                    $requiredData[$count][2] = "";
                    $requiredData[$count][3] = 22;
                    $requiredData[$count][4] = "";

                    if( isset( $invDevices[$requiredData[$count][1]] ) )
                    {
                        $invDeviceData = $invDevices[$requiredData[$count][1]];
                        $requiredData[$count][3] = $invDeviceData[0];
                        $requiredData[$count][4] = $invDeviceData[1];
                    }
                    $count++;
                }
            }

            return $requiredData;
        }

        function getEBSName( $kibsName )
        {
            $data = array( );
            $data2 = array( );

            $firstPart = trim( substr( $kibsName, 0, 4 ) );
            $strSize = strlen( $kibsName );
            $lastPart = trim( substr( $kibsName, 4, $strSize ) );
            if( @ereg( "CTI", $lastPart ) )
            {
                $lastPart = str_replace( '_CTI', '', $lastPart );
            }
            $lastPart = str_replace( '-', '', $lastPart );
            $lastPart = str_replace( '_', '', $lastPart );
            $kibsName = $firstPart . $lastPart;

            return $kibsName;
        }

        function getNotInstaledDevice( $serial )
        {
            $notInstalledDevices = $this->getNotInstalledDevices( );
            for( $x = 0; $x < sizeof( $notInstalledDevices ); $x++ )
            {
                if( $notInstalledDevices[$x][0] == $serial )
                    return $notInstalledDevices[$x];
            }
        }

        function getDeviceType( $device )
        {
            $deviceType = $device[3];

            if( ($deviceType >= 0 && $deviceType <= 13) || ($deviceType == 16) )
                $type = "Placa";
            elseif( $deviceType == 14 || $deviceType == 15 )
                $type = "USB";
            elseif( $deviceType > 16 && $deviceType < 22 )
                $type = "Modulo";
            elseif( $deviceType > 21 )
                $type = "Modular";
            return $type;
        }

        function getYamlGeneralDeviceType( $deviceDef )
        {
            $device[0] = "";
            $device[1] = "";
            $device[2] = "";
            $device[3] = $deviceDef[0];

            $pType = $this->getDeviceType( $device );
            switch($pType)
            {
                case 'Placa':
                    return 'Board';
                    break;
                case 'USB':
                    return 'USB';
                    break;
                case 'Modulo':
                    return 'Module';
                    break;
            }
        }

        function getRealDeviceType( $device )
        {
            if( is_array( $device ) )
            {
                $deviceType = $device[0];
                $deviceSubType = $device[1];
                if( $deviceType == 0 || $deviceType == 13 || ($deviceType >= 3 && $deviceType <= 4) || ($deviceType >= 6 && $deviceType <= 8) || $deviceType == 16 || $deviceType == 21 )
                {
                    return 'E1';
                }
                if( $deviceType == 1 || $deviceType == 5 || ($deviceType == 20 && $deviceSubType == 0) || ($deviceType == 20 && $deviceSubType == 1) )
                {
                    return 'FXO';
                }
                if( $deviceType == 2 )
                {
                    return 'CONF';
                }
                if( ($deviceType >= 11 && $deviceType <= 12) || ($deviceType >= 14 && $deviceType <= 15) || ($deviceType == 18 && $deviceSubType == 0) || ($deviceType == 18 && $deviceSubType == 1) )
                {
                    return 'GSM';
                }
                if( ($deviceType == 19 && $deviceSubType == 1) || ($deviceType == 19 && $deviceSubType == 0) || $deviceType == 9 || $deviceType == 10 )
                {
                    return 'FXS';
                }
                if( $deviceType == 22 )
                {
                    return 'Modular';
                }
            }
            return false;
        }

        function getLinkProfile( $devType )
        {
            switch($devType)
            {
                case 'E1':
                    return 'ISDN';
                    break;
                case 'FXO':
                    return 'FXO';
                    break;
                case 'GSM':
                    return 'GSM';
                    break;
                case 'FXS':
                    return 'FXS';
                    break;
            }
        }

        function getDevice( $serial )
        {
            Global $devices, $boardNames;
            $this->setDevices( );
            $dev = $this->devices;
            for( $x = 0; $x < sizeof( $dev ); $x++ )
            {
                if( $dev[$x][0] == $serial )
                    return $dev[$x];
            }
        }

        function getDeviceSubType( $device )
        {
            return;
        }

        function getSpecificLinkType( $type )
        {
            Global $boardNames, $_SESSION;
            $return = array( );

            if( !isset( $_SESSION['devicesYaml'] ) )
                $devicesYaml = new KYaml( $this->getConfigDir( ) . "/devices.yaml" );
            else
                $devicesYaml = unserialize( $_SESSION['devicesYaml'] );

            $rawArray = $devicesYaml->getYaml( );
            $totalDevices = sizeof( $rawArray['Devices'] );
            $deviceKeys = array_keys( $rawArray['Devices'] );

            for( $x = 0; $x < $totalDevices; $x++ )
            {
                $preSerial = array_keys( $rawArray['Devices'][$deviceKeys[$x]] );
                $serial = $preSerial[0];
                if( $type != "GSM" && $type != "FXO" )
                {
                    if( isset( $rawArray['Devices'][$deviceKeys[$x]][$serial]['Links'] ) )
                    {
                        $linkSize = sizeof( $rawArray['Devices'][$deviceKeys[$x]][$serial]['Links'] );
                        for( $y = 0; $y < $linkSize; $y++ )
                        {
                            if( $rawArray['Devices'][$deviceKeys[$x]][$serial]['Links'][$y][$y]['Type'] == $type )
                            {
                                $return[] = $serial . "." . $y;
                            }
                        }
                    }
                }
                else
                {
                    if( isset( $rawArray['Devices'][$deviceKeys[$x]][$serial]['ChannelGroups'] ) )
                    {
                        $linkSize = sizeof( $rawArray['Devices'][$deviceKeys[$x]][$serial]['ChannelGroups'] );
                        for( $i = 0; $i < $linkSize; $i++ )
                        {
                            $channelGroupsKeys = array_keys( $rawArray['Devices'][$deviceKeys[$x]][$serial]['ChannelGroups'][$i] );
                            for( $y = 0; $y < $linkSize; $y++ )
                            {
                                if( $rawArray['Devices'][$deviceKeys[$x]][$serial]['ChannelGroups'][$i][$channelGroupsKeys[$y]]['Type'] == $type )
                                {
                                    $CG = $channelGroupsKeys[$y];
                                    $channelSize = explode( "-", $CG );
                                    for( $w = $channelSize[0]; $w < $channelSize[(sizeof( $channelSize ) - 1)] + 1; $w++ )
                                        $return[] = $serial . "." . $w;
                                }
                            }
                        }
                    }
                }
            }
            return $return;
        }

        function getSpecificDevice( $deviceName )
        {
            Global $boardNames, $_SESSION;
            $return = array( );

            if ( !is_array( $deviceName ) )
            {
                $deviceName = array( $deviceName );
            }

            $devicesYaml = new KYaml( $this->getConfigDir( ) . "/devices.yaml" );

            $rawArray = $devicesYaml->getYaml( );
            $totalDevices = sizeof( $rawArray['Devices'] );
            $deviceKeys = array_keys( $rawArray['Devices'] );

            for( $x = 0; $x < $totalDevices; $x++ )
            {
                $preSerial = array_keys( $rawArray['Devices'][$deviceKeys[$x]] );
                $serial = $preSerial[0];

                //device E1 com profile type casPBX = FXO
                if( in_array( 'FXO', $deviceName ) && isset( $rawArray['Devices'][$deviceKeys[$x]][$serial]['Links'] ) )
                {
                    $qtde_link = sizeof( $rawArray['Devices'][$deviceKeys[$x]][$serial]['Links'] );

                    for( $j = 0; $j < $qtde_link; $j++ )
                    {
                        $profileLink = $rawArray['Devices'][$deviceKeys[$x]][$serial]['Links'][$j][$j]['Profile'];
                        $Profiles = new KProfiles( );
                        $pTmp = $Profiles->getProfile( $profileLink );

                        if( $pTmp[$profileLink]["Type"] == "CAS PBX" )
                        {
                            if( !in_array( $serial, $return ) )
                                $return[] = $serial;
                        }
                    }
                }

                if( in_array( 'E1', $deviceName) || in_array( 'T1', $deviceName ) )
                {
                    if( isset( $rawArray['Devices'][$deviceKeys[$x]][$serial]['Links'] ) )
                    {
                        if( !in_array( $serial, $return ) )
                            $return[] = $serial;
                    }
                }
                else if( isset( $rawArray['Devices'][$deviceKeys[$x]][$serial]['ChannelGroups'] ) )
                {

                    $qtde_Ch_gr = sizeof( $rawArray['Devices'][$deviceKeys[$x]][$serial]['ChannelGroups'] );
                    for( $i = 0; $i < $qtde_Ch_gr; $i++ )
                    {
                        $linkSize = sizeof( $rawArray['Devices'][$deviceKeys[$x]][$serial]['ChannelGroups'] );
                        $channelGroupsKeys = array_keys( $rawArray['Devices'][$deviceKeys[$x]][$serial]['ChannelGroups'][$i] );

                        if( in_array( $rawArray['Devices'][$deviceKeys[$x]][$serial]['ChannelGroups'][$i][$channelGroupsKeys[0]]['Type'], $deviceName ) )
                        {
                            if( !in_array( $serial, $return ) )
                                $return[] = $serial;
                        }
                    }
                }
            }
            return $return;
        }
        function serialToBoard($serial)
        {
            $devicesYaml = new KYaml( $this->getConfigDir( ) . "/devices.yaml" );
            $rawArray = $devicesYaml->getYaml( );
            $devs = $rawArray['Devices'];
            for($i=0; $i < sizeof($devs); ++$i)
            {
                foreach($devs[$i] as $s => $dev)
                {
                    if($serial == $s)
                        return $i;
                }
            }
            return -1;
        }
        // Returns a mapping Serial -> [chan_first, chan_last, board_number] from the given tech;
        function getTechChannels($tech)
        {
            $devicesYaml = new KYaml( $this->getConfigDir( ) . "/devices.yaml" );
            $rawArray = $devicesYaml->getYaml( );
            $devs = $rawArray['Devices'];
            $ret=array();
            for($i=0; $i < sizeof($devs); ++$i)
            {
                //$dev = array_pop($devs[$i]); // this shit always have only one item.
                foreach($devs[$i] as $serial => $dev)
                {
                    foreach($dev['ChannelGroups'] as $group)
                    {
                        foreach($group as $range => $info)
                            if($info['Type'] == $tech)
                            {
                                //$ret .= 'B'.$i.'C'.$range.';';
                                $r = explode('-', $range);
                                $r['board'] = $i;
                                $ret[$serial] = $r;
                            }
                    }
                }
            }
            return $ret;
        }

        function getDeviceLinks( $device )
        {
            Global $boardNames, $_SESSION;
            $return = array( );

            if( !isset( $_SESSION['devicesYaml'] ) )
                $devicesYaml = new KYaml( $this->getConfigDir( ) . "/devices.yaml" );
            else
                $devicesYaml = unserialize( $_SESSION['devicesYaml'] );

            $rawArray = $devicesYaml->getYaml( );
            $totalDevices = sizeof( $rawArray['Devices'] );
            $deviceKeys = array_keys( $rawArray['Devices'] );

            for( $x = 0; $x < $totalDevices; $x++ )
            {
                $preSerial = array_keys( $rawArray['Devices'][$deviceKeys[$x]] );
                $serial = $preSerial[0];
                if( $serial == $device )
                {
                    $selectedDeviceLinks = $rawArray['Devices'][$deviceKeys[$x]][$serial]['Links'];
                    for( $i = 0; $i < sizeof( $selectedDeviceLinks ); $i++ )
                    {
                        $linksKeys = array_keys( $selectedDeviceLinks[$i] );
                        $return[] = $linksKeys[0];
                    }
                }
            }
            return $return;
        }

        function getE1T1Links( )
        {
            $e1 = $this->getSpecificLinkType( 'E1' );
            $t1 = $this->getSpecificLinkType( 'T1' );
            return array_merge( $e1, $t1 );
        }

        function getE1Links( )
        {
            return $this->getSpecificLinkType( 'E1' );
        }

        function getE1Devices( )
        {
            return $this->getSpecificDevice( 'E1' );
        }

        function getFirstFXODevice( )
        {
            $FXOs = $this->getSpecificDevice( 'FXO' );
            return $FXOs[0];
        }

        function getFXODeviceChannels( $serial )
        {
            Global $boardNames, $_SESSION;
            $return = array( );
            $deviceName = "FXO";

            $devicesYaml = new KYaml( $this->getConfigDir( ) . "/devices.yaml" );

            $rawArray = $devicesYaml->getYaml( );
            $totalDevices = sizeof( $rawArray['Devices'] );
            $deviceKeys = array_keys( $rawArray['Devices'] );

            $retorno = "";

            for( $x = 0; $x < $totalDevices; $x++ )
            {
                $preSerial = array_keys( $rawArray['Devices'][$deviceKeys[$x]] );
                if( isset( $rawArray['Devices'][$deviceKeys[$x]][$serial]['ChannelGroups'] ) )
                {
                    $linkSize = sizeof( $rawArray['Devices'][$deviceKeys[$x]][$serial]['ChannelGroups'] );

                    $qtde_CG = sizeof( $rawArray['Devices'][$deviceKeys[$x]][$serial]['ChannelGroups'] );

                    for( $i = 0; $i < $qtde_CG; $i++ )
                    {
                        $channelGroupsKeys = array_keys( $rawArray['Devices'][$deviceKeys[$x]][$serial]['ChannelGroups'][$i] );
                        if( $rawArray['Devices'][$deviceKeys[$x]][$serial]['ChannelGroups'][$i][$channelGroupsKeys[0]]['Type'] == $deviceName )
                            $retorno .= $channelGroupsKeys[0] . '-';
                    }
                }

                if( isset( $rawArray['Devices'][$deviceKeys[$x]][$serial]['Links'] ) )
                {
                    $qtde_link = sizeof( $rawArray['Devices'][$deviceKeys[$x]][$serial]['Links'] );

                    for( $j = 0; $j < $qtde_link; $j++ )
                    {
                        $profileLink = $rawArray['Devices'][$deviceKeys[$x]][$serial]['Links'][$j][$j]['Profile'];
                        $Profiles = new KProfiles( );
                        $pTmp = $Profiles->getProfile( $profileLink );

                        if( $pTmp[$profileLink]["Type"] == "CAS PBX" )
                        {
                            $first = (30 * ($j));
                            $last = (30 * ($j + 1)) - 1;
                            $retorno .= $first . '-' . $last . "-";
                        }
                    }
                }

            }
            if( strlen( $retorno ) > 0 )
            {
                $retorno = explode( '-', $retorno );
                $min = min( $retorno );
                $min = $min == 0 ? '0' : $min;
                $max = max( $retorno );
                $retorno = $min . '-' . $max;
            }
            return $retorno;
        }

        function getFXOChannels( )
        {
            return $this->getSpecificDevice( 'FXO' );
        }

        function getGSMPins( )
        {
            return $this->getSpecificLinkType( 'GSM' );
        }

        function getTypeSubTypeLinkCount( $dev )
        {
            Global $boardNames;

            for( $x = 0; $x <= 22; $x++ )
            {
                if( isset( $boardNames[$x] ) )
                {
                    $fkeys = array_keys( $boardNames[$x] );
                    for( $y = 0; $y < sizeof( $fkeys ); $y++ )
                    {
                        $skeys = array_keys( $boardNames[$x][$fkeys[$y]] );
                        for( $z = 0; $z < sizeof( $skeys ); $z++ )
                        {
                            if( $boardNames[$x][$fkeys[$y]][$skeys[$z]] == $dev[1] )
                                return array( $x, $fkeys[$y], $skeys[$z] );
                        }
                    }
                }
            }
            return array( 0, 0, 0 );
        }

        function createDevice( $serial, $extra = "" )
        {
            Global $devices, $boardNames, $_SESSION;
            $device = $this->getNotInstaledDevice( $serial );

            if( $extra != "" )
            {
                $device[1] = $boardNames[$extra[0]][$extra[1]][$extra[2]];
                $device[4] = $extra[2];
                $device[5] = isset( $extra[3] ) ? $extra[3] : 'V1';
            }

            $fxo = array( 'KFXO-IP', 'KFXO-80', 'KFXO-80HI' );
            $gsm = array( 'KGSM-40', 'KGSM-40_SPX' );
            $fxohi = array( 'KFXO-160HI', 'KFXO-240HI' );
            $fxs = array( 'KFXS-150', 'KFXS-300', 'KFXS-150_SPX', 'KFXS-300_SPX' );

            if( in_array( $device[1], $gsm ) )
                $device[4] = 4;
            elseif( in_array( $device[1], $fxo ) )
                $device[4] = 8;
            elseif( in_array( $device[1], $fxohi ) )
                $device[4] = 24;
            elseif( in_array( $device[1], $fxs ) )
                $device[4] = 15;

            $configData = array( );

            $regularDevice = $this->getTypeSubTypeLinkCount( $device );
            $devType = $this->getRealDeviceType( array( $regularDevice[0], $regularDevice[1], '' ) );
            $linkProfile = $this->getLinkProfile( $devType );

            if( $device[1] != "" )
            {
                $linkCount = $device[4];

                $configData[$serial]['Type'] = $this->getDeviceRealName( $device[1] );

                if( @ereg( "EBS", $device[1] ) )
                {
                    $configData[$serial]['IP'] = '';

                    /** ALERT **/
                    /* Modificação temporária até que haja configuração! */
                    $configData[$serial]['CustomTonesProfile'] = 'Profile1';
                    /** fim da modificação **/

                    if( (strpos( $configData[$serial]['Type'], 'E1' ) !== FALSE) )
                        $configData[$serial]['LinkOperationMode'] = '';

                    $configData[$serial]['Replacer'] = '';
                    $configData[$serial]['Version'] = $device[5];
                }
                if( $devType != "GSM" && $devType != "FXO" && $devType != "FXS" && $devType != "Modular" )
                {
                    if( $configData[$serial]['Type'] == "KPR-300" )
                        $linkCount = 1;
                    for( $x = 0; $x < $linkCount; $x++ )
                    {
                        $configData[$serial]['Links'][$x][$x]['Type'] = $devType;
                        $configData[$serial]['Links'][$x][$x]['Name'] = '';
                        $configData[$serial]['Links'][$x][$x]['Profile'] = 'Deactivated';
                    }
                }
                elseif( $devType != "Modular" )
                {
                    $CGSubName = "0-" . ($linkCount - 1);
                    $configData[$serial]['ChannelGroups'][0][$CGSubName]['Type'] = $devType;
                    $configData[$serial]['ChannelGroups'][0][$CGSubName]['Profile'] = $devType;
                    if( $device[1] == "KFXS-300_SPX" || $device[1] == "KFXS-300" )
                    {
                        $CGSubName = "15-29";
                        $configData[$serial]['ChannelGroups'][1][$CGSubName]['Type'] = $devType;
                        $configData[$serial]['ChannelGroups'][1][$CGSubName]['Profile'] = $devType;
                    }
                }
                elseif( $devType == "Modular" )
                {
                    $partString = @ereg_replace( 'EBS-Modular', '', $configData[$serial]['Type'] );
                    $partString = @ereg_replace( '_', '', $partString );
                    $partString = @ereg_replace( 'SPX', '', $partString );
                    $partString = @ereg_replace( 'CTI', '', $partString );
                    $deviceParts = explode( '/', $partString );
                    $linkC = 0;
                    for( $i = 0; $i < sizeof( $deviceParts ); $i++ )
                    {
                        if( @ereg( 'E1', $deviceParts[$i] ) )
                        {
                            $linkC += intval( substr( $deviceParts[$i], 0, 2 ) / 30 );
                            for( $j = 0; $j < $linkC; $j++ )
                            {
                                $configData[$serial]['Links'][$j][$j]['Type'] = 'E1';
                                $configData[$serial]['Links'][$j][$j]['Name'] = '';
                                $configData[$serial]['Links'][$j][$j]['Profile'] = 'Deactivated';
                            }
                        }
                        else
                        {
                            $linkCount = 0;
                            $channelC = substr( $deviceParts[$i], 0, 1 );
                            $totalChannel = $channelC;
                            $dType = substr( $deviceParts[$i], 1 );
                            if( isset( $deviceParts[$i + 1] ) && $deviceParts[$i] == $deviceParts[$i + 1] )
                            {
                                $channelC += $totalChannel;
                                $i++;
                                if( isset( $deviceParts[$i + 1] ) && $deviceParts[$i] == $deviceParts[$i + 1] )
                                {
                                    $channelC += $totalChannel;
                                    $i++;
                                }
                            }
                            if( isset( $configData[$serial]['Links'] ) )
                                $linkCount = sizeof( $configData[$serial]['Links'] );
                            
                            $cGNumber = sizeof( $configData[$serial]['ChannelGroups'] );
                            if( $cGNumber == 0 )
                                $lastChannel = 0 + ($linkCount * 30);
                            else
                            {
                                $keyPart = array_keys( $configData[$serial]['ChannelGroups'][$cGNumber - 1] );
                                $tmp_LC = explode( '-', $keyPart[0] );
                                $lastChannel = $tmp_LC[1] + 1;
                            }
                            
                            $linkProfile = $this->getLinkProfile( $dType );
                            $configData[$serial]['ChannelGroups'][$cGNumber][$lastChannel . '-' . ($lastChannel + $channelC - 1)]['Type'] = $dType;
                            $configData[$serial]['ChannelGroups'][$cGNumber][$lastChannel . '-' . ($lastChannel + $channelC - 1)]['Profile'] = $linkProfile;
                        }
                    }
                }

                $devicesYaml = new KYaml( $this->getConfigDir( ) . "/devices.yaml" );
                $rawArray = $devicesYaml->getYaml( );
                if( isset( $rawArray['Devices'] ) )
                    $deviceNumber = array_keys( $rawArray['Devices'] );
                $rawArraysize = sizeof( $rawArray['Devices'] );
                if( $rawArraysize > 0 )
                {
                    $deviceNumber = $deviceNumber[sizeof( $deviceNumber ) - 1];
                    $deviceNumber++;
                }
                else
                    $deviceNumber = 0;

                $devicesYaml->createVar( $deviceNumber, 'Devices', $configData );
                $devicesYaml->saveYaml( true );
            }
        }

        function manualCreateDevice( $serial, $devDef )
        {
            Global $boardNames;

            $configData = array( );
            $device = explode( '|', $devDef );

            $devType = $this->getRealDeviceType( $device );
            $linkProfile = $this->getLinkProfile( $devType );

            if( $serial != "" )
            {
                $linkCount = $device[2];
                $configData[$serial]['Type'] = $this->getDeviceRealName( $boardNames[$device[0]][$device[1]][$device[2]] );
                if( $configData[$serial]['Type'] == "KPR-300" )
                    $linkCount = 1;

                if( $device[0] > 17 )
                {
                    $configData[$serial]['IP'] = '';

                    /** ALERT **/
                    /* Modificação temporária até que haja configuração! */
                    $configData[$serial]['CustomTonesProfile'] = 'Profile1';
                    /** fim da modificação **/

                    if( (strpos( $configData[$serial]['Type'], 'E1' ) !== FALSE) )
                        $configData[$serial]['LinkOperationMode'] = '';

                    $configData[$serial]['Replacer'] = '';
                }
                if( $devType != "GSM" && $devType != "FXO" && $devType != "FXS" && $devType != "Modular" )
                {
                    for( $x = 0; $x < $linkCount; $x++ )
                    {
                        $configData[$serial]['Links'][$x][$x]['Type'] = $devType;
                        $configData[$serial]['Links'][$x][$x]['Name'] = '';
                        $configData[$serial]['Links'][$x][$x]['Profile'] = 'Deactivated';
                    }
                }
                elseif( $devType != "Modular" )
                {
                    $configData[$serial]['Version'] = '';

                    if( $device[0] == 11 || $device[0] == 12 )
                        $linkCount = 4;
                    elseif( $device[0] == 1 || $device[0] == 5 )
                    {
                        if( $device[1] == 0 || $device[1] == 1 || $device[1] == 5 )
                            $linkCount = 8;
                        elseif( $device[1] == 2 )
                            $linkCount = 16;
                        elseif( $device[1] == 3 )
                            $linkCount = 24;
                    }
                    elseif( $device[0] == 9 || $device[0] == 10 )
                        $linkCount = 15;
                    $CGSubName = "0-" . ($linkCount - 1);

                    $configData[$serial]['ChannelGroups'][0][$CGSubName]['Type'] = $devType;
                    $configData[$serial]['ChannelGroups'][0][$CGSubName]['Profile'] = $linkProfile;
                    if( ($device[0] == 9 && $device[2] == 2) || ($device[0] == 10 && $device[2] == 2) )
                    {
                        $CGSubName = "15-29";
                        $configData[$serial]['ChannelGroups'][1][$CGSubName]['Type'] = $devType;
                        $configData[$serial]['ChannelGroups'][1][$CGSubName]['Profile'] = $linkProfile;
                    }
                }
                elseif( $devType == "Modular" )
                {
                    $partString = @ereg_replace( 'EBS-Modular', '', $configData[$serial]['Type'] );
                    $partString = @ereg_replace( '_', '', $partString );
                    $partString = @ereg_replace( 'SPX', '', $partString );
                    $partString = @ereg_replace( 'CTI', '', $partString );
                    $deviceParts = explode( '/', $partString );
                    $configData[$serial]['Version'] = '';

                    $linkC = 0;
                    for( $i = 0; $i < sizeof( $deviceParts ); $i++ )
                    {
                        if( @ereg( 'E1', $deviceParts[$i] ) )
                        {
                            $linkC += intval( substr( $deviceParts[$i], 0, 2 ) / 30 );
                            for( $j = 0; $j < $linkC; $j++ )
                            {
                                $configData[$serial]['Links'][$j][$j]['Type'] = 'E1';
                                $configData[$serial]['Links'][$j][$j]['Name'] = '';
                                $configData[$serial]['Links'][$j][$j]['Profile'] = 'Deactivated';
                            }

                            if( (strpos( $configData[$serial]['Type'], 'E1' ) !== FALSE) )
                                $configData[$serial]['LinkOperationMode'] = '';
                        }
                        else
                        {
                            $linkCount = 0;
                            $channelC = substr( $deviceParts[$i], 0, 1 );
                            $totalChannel = $channelC;
                            $dType = substr( $deviceParts[$i], 1 );
                            if( isset( $deviceParts[$i + 1] ) && $deviceParts[$i] == $deviceParts[$i + 1] )
                            {
                                $channelC += $totalChannel;
                                $i++;
                                if( isset( $deviceParts[$i + 1] ) && $deviceParts[$i] == $deviceParts[$i + 1] )
                                {
                                    $channelC += $totalChannel;
                                    $i++;
                                }
                            }
                            $cGNumber = isset( $configData[$serial]['ChannelGroups'] ) ? sizeof( $configData[$serial]['ChannelGroups'] ) : 0;
                            if( isset( $configData[$serial]['Links'] ) )
                                $linkCount = sizeof( $configData[$serial]['Links'] );
                            if( $cGNumber == 0 )
                                $lastChannel = 0 + ($linkCount * 30);
                            else
                            {
                                $keyPart = array_keys( $configData[$serial]['ChannelGroups'][$cGNumber - 1] );
                                $lastChannel = explode( '-', $keyPart[0] );
                                $lastChannel = $lastChannel[1] + 1;
                            }

                            $linkProfile = $this->getLinkProfile( $dType );
                            $configData[$serial]['ChannelGroups'][$cGNumber][$lastChannel . '-' . ($lastChannel + $channelC - 1)]['Type'] = $dType;
                            $configData[$serial]['ChannelGroups'][$cGNumber][$lastChannel . '-' . ($lastChannel + $channelC - 1)]['Profile'] = $linkProfile;
                        }
                    }
                }

                $devicesYaml = new KYaml( $this->getConfigDir( ) . "/devices.yaml" );
                $rawArray = $devicesYaml->getYaml( );
                $deviceNumber = array_keys( $rawArray['Devices'] );
                $rawArraysize = sizeof( $rawArray['Devices'] );
                if( $rawArraysize > 0 )
                {
                    $deviceNumber = $deviceNumber[sizeof( $deviceNumber ) - 1];
                    $deviceNumber++;
                }
                else
                    $deviceNumber = 0;
                $devicesYaml->createVar( $deviceNumber, 'Devices', $configData );

                $devicesYaml->saveYaml( true );

            }
        }

        function getDeviceRealName( $device )
        {
            if( @ereg( "EBS", $device ) )
            {
                if( @!ereg( "Modular", $device ) )
                    $device = @ereg_replace( "_CTI", "", $device );
                if( $device == "EBS-E1600" || $device == "EBS-E1 600" )
                {
                    $device = "EBS-E1_6000";
                }
                elseif( $device == "EBS-E1300" || $device == "EBS-E1 300" )
                {
                    $device = "EBS-E1_3000";
                }
                elseif( $device == "EBS-E1300_SPX" || $device == "EBS-E1 300_SPX" )
                {
                    $device = "EBS-E1_SPX_300";
                }
                elseif( $device == "EBS-E1600_SPX" || $device == "EBS-E1 600_SPX" )
                {
                    $device = "EBS-E1_SPX_600";
                }
            }

            return $device;
        }

        function getDeviceList( )
        {
            $return = array( );
            $deviceList = $this->devices;
            for( $x = 0; $x < sizeof( $deviceList ); $x++ )
            {
                $deviceSerial = $deviceList[$x][0];
                $linkSize = $deviceList[$x][4];

                if( $deviceList[$x][3] == 5 || $deviceList[$x][3] == 11 || $deviceList[$x][3] == 12 || $deviceList[$x][3] == 1 || $deviceList[$x][3] == 14 || $deviceList[$x][3] == 15 || ($deviceList[$x][1] == "EBS-FXO_120") | ($deviceList[$x][1] == "EBS-FXS_240") | ($deviceList[$x][1] == "EBS-FXS240_SPX") | ($deviceList[$x][1] == "EBS-GSM_160") )
                    $return[] = $deviceSerial;
                else
                {
                    for( $y = 0; $y < $linkSize; $y++ )
                        $return[] = $deviceSerial . "." . $y;
                }
            }
            return $return;
        }

        function setDevices( $excludeType = '' )
        {
            Global $devices, $boardNames, $_SESSION;

            $devicesYaml = new KYaml( $this->getConfigDir( ) . "/devices.yaml" );
            $dev = $devicesYaml->getYaml( );

            $a = array( );
            if( isset( $dev['Devices'] ) )
            {
                $keys = array_keys( $dev['Devices'] );
                $devCount = 0;
                for( $x = 0; $x < sizeof( $dev['Devices'] ); $x++ )
                {
                    $seriais = array_keys( $dev['Devices'][$keys[$x]] );

                    if( $excludeType == '' )
                    {
                        $a[$devCount][0] = $seriais[0];
                        $a[$devCount][1] = $boardNames[0][0][1];
                        $a[$devCount][2] = "";
                        $a[$devCount][3] = $devices[$seriais[0]]['type'];
                        $a[$devCount][4] = $devices[$seriais[0]]['linkcount'];
                        $devCount++;
                    }
                    elseif( !@ereg( $excludeType, $dev['Devices'][$keys[$x]][$seriais[0]]['Type'] ) && !@ereg( 'Modular', $dev['Devices'][$keys[$x]][$seriais[0]]['Type'] ) )
                    {
                        $a[$devCount][0] = $seriais[0];
                        $a[$devCount][1] = $boardNames[$devices[$seriais[0]]['type']][$devices[$seriais[0]]['subtype']][$devices[$seriais[0]]['linkcount']];
                        $a[$devCount][2] = "";
                        $a[$devCount][3] = $devices[$seriais[0]]['type'];
                        $a[$devCount][4] = $devices[$seriais[0]]['linkcount'];
                        $devCount++;
                    }
                    elseif( @ereg( 'Modular', $dev['Devices'][$keys[$x]][$seriais[0]]['Type'] ) && @ereg( $excludeType, $dev['Devices'][$keys[$x]][$seriais[0]]['Type'] ) )
                    {
                        $a[$devCount][0] = $seriais[0];
                        $a[$devCount][1] = $boardNames[$devices[$seriais[0]]['type']][$devices[$seriais[0]]['subtype']][$devices[$seriais[0]]['linkcount']];
                        $a[$devCount][2] = "";
                        $a[$devCount][3] = $devices[$seriais[0]]['type'];
                        $a[$devCount][4] = $devices[$seriais[0]]['linkcount'];
                        $devCount++;
                    }
                }
            }
            $this->devices = $a;
        }

        function getBoardsLinks( )
        {
            $n = 0;
            for( $x = 0; $x < sizeof( $this->placas ); $x++ )
            {
                $channels = $this->placas[$x][2] / 15;
                for( $y = 0; $y < $channels; $y++ )
                    $boardsLinks[] = $this->placas[$x][0] . ".$y";
            }
            return $boardsLinks;
        }

        function numberChannels( $serial )
        {
            for( $x = 0; $x < sizeof( $this->placas ); $x++ )
            {
                if( $this->placas[$x][0] == $serial )
                    return $this->placas[$x][2];
            }
        }

        function checkIP( $serial )
        {
            $device = $this->getYAMLDevice( $serial );
            if( $device[$serial]['IP'] == '' )
                return true;
        }

        function checkIPConflict( $serial )
        {
            if( !isset( $_SESSION['devicesYaml'] ) )
                $devicesYaml = new KYaml( $this->getConfigDir( ) . "/devices.yaml" );
            else
                $devicesYaml = unserialize( $_SESSION['devicesYaml'] );
            $device = $this->getYAMLDevice( $serial );

            $deviceYAML = $devicesYaml->getYaml( );
            for( $i = 0; $i < sizeof( $deviceYAML['Devices'] ); $i++ )
            {
                if( !isset( $deviceYAML['Devices'][$i][$serial] ) )
                {
                    $keys = array_keys( $deviceYAML['Devices'][$i] );
                    if( isset( $deviceYAML['Devices'][$i][$keys[0]]['IP'] ) && ($deviceYAML['Devices'][$i][$keys[0]]['IP'] == $device[$serial]['IP']) && ($device[$serial]['IP'] != '') )
                        return true;
                }
            }
            return false;

        }

        function getIP( $serial )
        {
            if( !isset( $_SESSION['devicesYaml'] ) )
                $devicesYaml = new KYaml( $this->getConfigDir( ) . "/devices.yaml" );
            else
                $devicesYaml = unserialize( $_SESSION['devicesYaml'] );
            $device = $this->getYAMLDevice( $serial );
            return $device[$serial]['IP'];
        }

        function getYAMLDevice( $serial )
        {
            if( !isset( $_SESSION['devicesYaml'] ) )
                $devicesYaml = new KYaml( $this->getConfigDir( ) . "/devices.yaml" );
            else
                $devicesYaml = unserialize( $_SESSION['devicesYaml'] );
            $deviceYAML = $devicesYaml->getYaml( );
            for( $i = 0; $i < sizeof( $deviceYAML['Devices'] ); $i++ )
            {
                if( isset( $deviceYAML['Devices'][$i][$serial] ) )
                    return $deviceYAML['Devices'][$i];
            }
        }

        function getDeactivatedE1( $serial )
        {
            $devicesYaml = new KYaml( $this->getConfigDir( ) . "/devices.yaml" );
            $rawArray = $devicesYaml->getValue( $serial );
            if(!array_key_exists('Links', $rawArray))
                return false;
            $linkCount = sizeof($rawArray['Links']);
            for ($i=0; $i < $linkCount  ; $i++)
            {
                if( isset($rawArray['Links'][$i][$i]['Type']))
                {
                    $type =  $rawArray['Links'][$i][$i]['Type'];
                    if(($type == 'E1' || $type == 'T1') && $rawArray['Links'][$i][$i]['Profile'] == "Deactivated")
                        return true;
                }
            }
            return false;
        }

        function setSS7Boards( )
        {
            $placas = $this->getDevices( );
            $SS7Boards = Array( );
            $count = 0;
            $SS7Types = Array( 0, 3, 4, 6, 7, 13, 21 );
            for( $i = 0; $i < sizeof( $placas ); $i++ )
            {
                if( (in_array( trim( $placas[$i][3] ), $SS7Types )) || ($placas[$i][3] == 21) )
                {
                    $SS7Boards[$count][0] = $placas[$i][0];
                    $SS7Boards[$count][1] = $placas[$i][1];
                    $SS7Boards[$count][2] = $placas[$i][2];
                    $SS7Boards[$count][3] = $i;
                    $SS7Boards[$count][4] = $placas[$i][4];
                    $count++;
                }
            }
            $this->SS7Boards = $SS7Boards;
        }

        function getSS7Boards( )
        {
            return $this->SS7Boards;
        }

        function setCTbusBoards( )
        {
            $placas = $this->getPlacas( );
            $CTbusBoards = Array( );
            $count = 0;
            $ctbusTypes = Array( 0, 3, 4, 6, 8, 9, 11, 13 );
            for( $i = 0; $i < sizeof( $placas ); $i++ )
            {
                if( in_array( trim( $placas[$i][3] ), $ctbusTypes ) )
                {
                    $CTbusBoards[$count][0] = $placas[$i][0];
                    $CTbusBoards[$count][1] = $placas[$i][1];
                    $CTbusBoards[$count][2] = $placas[$i][2];
                    $CTbusBoards[$count][3] = $i;
                    $CTbusBoards[$count][4] = $placas[$i][4];
                    $count++;
                }
            }
            $this->CTbusBoards = $CTbusBoards;
        }

        function translate( $word )
        {
            echo $this->retTranslate( $word );
        }

        function retTranslateOrigin( $word )
        {
            Global $dicionario;
            if( isset( $dicionario['pt-br'][$word] ) )
            {
                return $dicionario['pt-br'][$word];
            }
            return $word;
        }

        function getIPv4Interfaces( )
        {
            $result = '';
            if( PHP_OS == "WINNT" )
            {
                $systemRoot = exec( "echo %systemRoot%" );
                $prt = "$systemRoot/system32/ipconfig";

                exec( "$prt", $result );

                for( $x = 0; $x < sizeof( $result ); $x++ )
                {
                    $tmp = utf8_encode( $result[$x] );
                    if( @ereg( "IPv4", $tmp ) || @ereg( "IP Address", $tmp ) || @ereg( "Endereço IP", $tmp ) || (@ereg( "inet addr", $tmp ) && @ereg( "Bcast", $tmp )) || (@ereg( "inet end", $tmp ) && @ereg( "Bcast", $tmp )) )
                    {
                        $subStr = explode( ":", $result[$x] );
                        $tmpIP = explode( " ", trim( $subStr[1] ) );
                        $tmpVar[] = trim( $tmpIP[0] );
                    }
                }
                return $tmpVar;
            }
            else
            {
                exec( "/usr/sbin/kwebhelper getifaces ipv4", $data);
                $data = explode('|', $data[0]);
                $result = array();
                foreach ($data as $k => $v)
                {
                    $tmp = explode(';', $v);
                    $result[] = $tmp[1];
                }
                return $result;
            }
        }

        function getIPv6Interfaces( )
        {
            if( PHP_OS == "WINNT" )
            {
                $systemRoot = exec( "echo %systemRoot%" );
                $prt = "$systemRoot/system32/ipconfig";
                exec( $prt, $result );
                $tmpVar = array();
                foreach ($result as $k => $v)
                {
                    $tmp = utf8_encode( $v);
                    if( @ereg( "IPv6", $tmp ))
                    {
                        $subStr = explode( ": ", $tmp );
                        $subStr = explode( '%', $subStr[1]);
                        $tmpVar[] = strtoupper(trim( $subStr[0]));
                    }
                }
                return $tmpVar;
            }
            else
            {
                exec( "/usr/sbin/kwebhelper getifaces ipv6", $data);
                $data = explode('|', $data[0]);
                $result = array();
                foreach ($data as $k => $v)
                {
                    $tmp = explode(';', $v);
                    $result[] = strtoupper($tmp[1]);
                }
                return $result;
            }
        }
        
        function getAllInterfaces()
        {
            return array_merge($this->getIPv4Interfaces(), $this->getIPv6Interfaces());
        }

        function getInterfaceNames()
        {
            if( PHP_OS == "WINNT" )
                return '';

            exec( "/usr/sbin/kwebhelper getifaces ipv4", $data);
            $data = explode('|', $data[0]);
            $result = array();
            foreach ($data as $k => $v)
            {
                $tmp = explode(';', $v);
                $result[] = $tmp[0].' ('.$tmp[1].')';
            }
            return $result;
        }
        function isPassiveDevice( $serial )
        {
            return false;
            $dev = $this->DevicesYAML->getValue( $serial );

            if(strpos($dev['Type'], 'HI') !== false || strpos($dev['Type'], 'KPR') !== false)
            {
                return true;
            }

            return false;
        }
    }
