<?php
    class SS7Exception extends Exception
    {
    }

    class SS7 extends KYaml
    {
        /**
         * array MTP2 links (reference to yaml['ss7']['Mtp2'])
         */
        protected $mtp2 = null;
        /**
         * array PointCodes (reference to yaml['ss7']['PointCodes'])
         */
        protected $pointCodes = null;
        /**
         * array Link sets (reference to yaml['ss7']['Mtp3']['LinkSets'])
         */
        protected $mtp3LinkSets = null;
        /**
         * array Routes (reference to yaml['ss7']['Mtp3']['Routes'])
         */
        protected $mtp3Routes = null;
        /**
         * array Circuit Groups (reference to
         * yaml['ss7']['Isup']['CircuitGroups'])
         */
        protected $isupCircuitGroups = null;
        /**
         * array Timers (reference to yaml['ss7']['Isup']['Timers'])
         */
        protected $isupTimers = null;
        /**
         * array Reserved timeslots inside ISUP Circuit Group
         *
         * @see KSs7Config.h
         */
        static public $isupCircuitGroupsTimeslotReserved = array( 0, 16 );
        /**
         * int Default ISUP Circuit Group Timeslot configuration
         */
        const ISUP_CIRCUITGROUP_DEFAUL_TIMESLOT = '0xFFFEFFFE';
        /**
         * int Number of timeslots
         */
        const ISUP_CIRCUITGROUP_AVAILABLE_TIMESLOTS = 31;

        public function __construct( )
        {
            $app = KSystem::getInstance( );

            // validation rules
            //@formatter:off
            $this->rules = array(
                "ss7" => array(
                    'type' => KYaml::TYPE_LISTHASH,
                    'children' => array(
                        'Mtp2' => array(
                            'type' => KYaml::TYPE_LIST,
                            'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                            'default' => array(),
                            'children' => array(
                                '?' => array(
                                    'type' => KYaml::TYPE_LIST,
                                    'children' => array(
                                        '?' => array(
                                            'type' => KYaml::TYPE_LISTHASH,
                                            'children' => array(
                                                'Address' => array(
                                                    'type' => KYaml::TYPE_STRING
                                                ),
                                                'Device' => array(
                                                    'type' => KYaml::TYPE_INTEGER
                                                ),
                                                'Link' => array(
                                                    'type' => KYaml::TYPE_INTEGER
                                                ),
                                                'Timeslot' => array(
                                                    'type' => KYaml::TYPE_INTEGER,
                                                    'default' => 16
                                                ),
                                                'Passive' => array(
                                                    'type' => KYaml::TYPE_BOOLEAN
                                                ),
                                                'LssuLength' => array(
                                                    'type' => KYaml::TYPE_INTEGER,
                                                    'default' => 2
                                                ),
                                                'Timers' => array(
                                                    'type' => KYaml::TYPE_LISTHASH,
                                                    'children' => array(
                                                        'T1' => array(
                                                            'type' => KYaml::TYPE_INTEGER
                                                        ),
                                                        'T2' => array(
                                                            'type' => KYaml::TYPE_INTEGER
                                                        ),
                                                        'T3' => array(
                                                            'type' => KYaml::TYPE_INTEGER
                                                        ),
                                                        'T5' => array(
                                                            'type' => KYaml::TYPE_INTEGER
                                                        ),
                                                        'T6' => array(
                                                            'type' => KYaml::TYPE_INTEGER
                                                        ),
                                                        'T7' => array(
                                                            'type' => KYaml::TYPE_INTEGER
                                                        ),
                                                        'ProvingEmergency' => array(
                                                            'type' => KYaml::TYPE_INTEGER
                                                        ),
                                                        'ProvingNormal' => array(
                                                            'type' => KYaml::TYPE_INTEGER
                                                        )
                                                    ) // ss7.MTP2.?.?.Timers
                                                )
                                            ) // ss7.MTP2.?.?
                                        )
                                    ) // ss7.MTP2.?
                                )
                            ) // ss7.MTP2
                        ),
                        'PointCodes' => array(
                            'type' => KYaml::TYPE_LIST,
                            'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                            'default' => array(),
                            'children' => array(
                                '?' => array(
                                    'type' => KYaml::TYPE_LIST,
                                    'children' => array(
                                        '?' => array(
                                            'type' => KYaml::TYPE_LIST,
                                            'children' => array(
                                                '?' => array(
                                                    'type' => KYaml::TYPE_INTEGER
                                                )
                                            ) // ss7.PointCodes.?.?
                                        )
                                    ) // ss7.PointCodes.?
                                )
                            ) // ss7.PointCodes
                        ),
                        'Mtp3' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'flags' => KYaml::FLAG_EMPTY,
                            'default' => array(),
                            'children' => array(
                                'SignalingPointType' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => 'SP'
                                ),
                                'LinkSets' => array(
                                    'type' => KYaml::TYPE_LIST,
                                    'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                                    'default' => array(),
                                    'children' => array(
                                        '?' => array(
                                            'type' => KYaml::TYPE_LIST,
                                            'children' => array(
                                                '?' => array(
                                                    'type' => KYaml::TYPE_LISTHASH,
                                                    'children' => array(
                                                        'Origin' => array(
                                                            'type' => KYaml::TYPE_STRING
                                                        ),
                                                        'Adjacent' => array(
                                                            'type' => KYaml::TYPE_STRING
                                                        ),
                                                        'NetworkIndicator' => array(
                                                            'type' => KYaml::TYPE_INTEGER,
                                                            'default' => 2
                                                        ),
                                                        'Links' => array(
                                                            'type' => KYaml::TYPE_LIST,
                                                            'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                                                            'children' => array(
                                                                '?' => array(
                                                                    'type' => KYaml::TYPE_LIST,
                                                                    'children' => array(
                                                                        '?' => array(
                                                                            'type' => KYaml::TYPE_LISTHASH,
                                                                            'children' => array(
                                                                                'Mtp2' => array(
                                                                                    'type' => KYaml::TYPE_STRING
                                                                                ),
                                                                                'Slc' => array(
                                                                                    'type' => KYaml::TYPE_INTEGER
                                                                                ),
                                                                                'Timers' => array(
                                                                                    'type' => KYaml::TYPE_LISTHASH,
                                                                                    'children' => array(
                                                                                        'Q707T1' => array(
                                                                                            'type' => KYaml::TYPE_INTEGER
                                                                                        ),
                                                                                        'Q707T2' => array(
                                                                                            'type' => KYaml::TYPE_INTEGER
                                                                                        ),
                                                                                        'Q704T17' => array(
                                                                                            'type' => KYaml::TYPE_INTEGER
                                                                                        )
                                                                                    ) // ss7.Mtp3.LinkSets.?.?.Links.?.?.Timers
                                                                                )
                                                                            ) // ss7.Mtp3.LinkSets.?.?.Links.?.?
                                                                        )
                                                                    ) // ss7.Mtp3.LinkSets.?.?.Links.?
                                                                )
                                                            ) // ss7.Mtp3.LinkSets.?.?.Links
                                                        )
                                                    ) // ss7.Mtp3.LinkSets.?.?
                                                )
                                            ) // ss7.Mtp3.LinkSets.?
                                        )
                                    ) // ss7.Mtp3.LinkSets
                                ),
                                'Routes' => array(
                                    'type' => KYaml::TYPE_LIST,
                                    'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                                    'default' => array(),
                                    'children' => array(
                                        '?' => array(
                                            'type' => KYaml::TYPE_LIST,
                                            'children' => array(
                                                '?' => array(
                                                    'type' => KYaml::TYPE_LISTHASH,
                                                    'children' => array(
                                                        'PointCode' => array(
                                                            'type' => KYaml::TYPE_STRING
                                                        ),
                                                        'LinkSets' => array(
                                                            'type' => KYaml::TYPE_LIST,
                                                            'children' => array(
                                                                '?' => array(
                                                                    'type' => KYaml::TYPE_STRING
                                                                )
                                                            ) // ss7.Mtp3.Routes.?.?.LinkSets
                                                        )
                                                    ) // ss7.Mtp3.Routes.?.?
                                                )
                                            ) // ss7.Mtp3.Routes.?
                                        )
                                    ) // ss7.Mtp3.Routes
                                )
                            ) // ss7.Mtp3
                        ),
                        'Isup' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'children' => array(
                                'CircuitGroups' => array(
                                    'type' => KYaml::TYPE_LIST,
                                    'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                                    'default' => array(),
                                    'children' => array(
                                        '?' => array(
                                            'type' => KYaml::TYPE_LIST,
                                            'children' => array(
                                                '?' => array(
                                                    'type' => KYaml::TYPE_LISTHASH,
                                                    'children' => array(
                                                        'Device' => array(
                                                            'type' => KYaml::TYPE_INTEGER
                                                        ),
                                                        'Link' => array(
                                                            'type' => KYaml::TYPE_INTEGER
                                                        ),
                                                        'Address' => array(
                                                            'type' => KYaml::TYPE_STRING
                                                        ),
                                                        'BaseCIC' => array(
                                                            'type' => KYaml::TYPE_INTEGER
                                                        ),
                                                        'Origin' => array(
                                                            'type' => KYaml::TYPE_STRING
                                                        ),
                                                        'Destination' => array(
                                                            'type' => KYaml::TYPE_STRING
                                                        ),
                                                        'Passive' => array(
                                                            'type' => KYaml::TYPE_BOOLEAN
                                                        ),
                                                        'TimeslotMap' => array(
                                                            'type' => KYaml::TYPE_STRING
                                                        )
                                                    ) // ss7.Isup.CircuitGroups.?.?
                                                )
                                            ) // ss7.Isup.CircuitGroups.?
                                        )
                                    ) // ss7.Isup.CircuitGroups
                                ),
                                'Timers' => array(
                                    'type' => KYaml::TYPE_LISTHASH,
                                    'children' => array(
                                        'T1' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 45000
                                        ),
                                        'T5' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 600000
                                        ),
                                        'T6' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 35000
                                        ),
                                        'T7' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 25000
                                        ),
                                        'T8' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 13000
                                        ),
                                        'T9' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 120000
                                        ),
                                        'T12' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 45000
                                        ),
                                        'T13' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 600000
                                        ),
                                        'T14' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 45000
                                        ),
                                        'T15' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 600000
                                        ),
                                        'T16' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 45000
                                        ),
                                        'T17' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 600000
                                        ),
                                        'T18' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 45000
                                        ),
                                        'T19' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 600000
                                        ),
                                        'T20' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 45000
                                        ),
                                        'T21' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 600000
                                        ),
                                        'T22' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 45000
                                        ),
                                        'T23' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 600000
                                        ),
                                        'T24' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 1500
                                        ),
                                        'T25' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 5000
                                        ),
                                        'T26' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 120000
                                        ),
                                        'T27' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 240000
                                        ),
                                        'T28' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 10000
                                        ),
                                        'T34' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 3000
                                        ),
                                        'T36' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 13000
                                        ),
                                        'T37' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 3000
                                        ),
                                        'T38' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'default' => 120000
                                        )
                                    ) // ss7.Isup.Timers
                                )
                            ) // ss7.Isup
                        )
                    ) // ss7
                )
            );
            //@formatter:on

            parent::__construct( $app->getConfigDir( ) . DIRECTORY_SEPARATOR . 'ss7.yaml' );

            // easy access to some YAML array positions
            $this->mtp2 = &$this->yaml['ss7']['Mtp2'];
            $this->pointCodes = &$this->yaml['ss7']['PointCodes'];
            $this->mtp3LinkSets = &$this->yaml['ss7']['Mtp3']['LinkSets'];
            $this->mtp3Routes = &$this->yaml['ss7']['Mtp3']['Routes'];
            $this->isupCircuitGroups = &$this->yaml['ss7']['Isup']['CircuitGroups'];
            $this->isupTimers = &$this->yaml['ss7']['Isup']['Timers'];
        }

        ///////////////////////////////////////////////////////////////////////
        // MTP2 manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Check if MTP2 link name is defined
         *
         * @param string $name MTP2 name
         * @return boolean
         */
        public function hasMtp2( $name )
        {
            foreach( $this->mtp2 as $idx => &$info )
            {
                if( array_key_exists( $name, $info ) )
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * Lookup mtp2 by its name
         *
         * @param string $name MTP2 name
         * @return null|array
         */
        public function getMtp2( $name )
        {
            $mtp2 = null;
            foreach( $this->mtp2 as $idx => &$info )
            {
                // found
                if( array_key_exists( $name, $info ) )
                {
                    $mtp2 = $info[$name];
                    break;
                }
            }
            return $mtp2;
        }

        /**
         * Get all defined MTP2 as array
         *
         * @return array
         */
        public function getAllMtp2( )
        {
            return $this->mtp2;
        }

        /**
         * Set new or update mtp2
         *
         * If oldName is not null and is not eaquals name, then the oldName
         * data will be removed first
         *
         * @param string $name MTP2 name (new name)
         * @param array $data MTP2 data array (all fields required)
         * @param string $oldName MTP2 old name
         */
        public function setMtp2( $name, $data, $oldName = '' )
        {
            // new name already exists
            if( $name !== $oldName && $this->hasMtp2( $name ) )
            {
                throw new SS7Exception( 'ss7-mtp2-error-name-already-exists' );
            }
            elseif( !$this->isValidName( $name ) )
            {
                throw new SS7Exception( 'ss7-error-invalid-name' );
            }
            // renamed MTP2 configuration
            elseif( !empty( $oldName ) && $name !== $oldName )
            {
                // rename where it is used
                foreach( $this->mtp3LinkSets as &$item )
                {
                    $_name = array_keys( $item );
                    $_name = $_name[0];
                    $_data = &$item[$_name];
                    foreach( $_data['Links'] as $idx => &$_link )
                    {
                        $_lname = array_keys( $_link );
                        $_lname = $_lname[0];
                        $_ldata = &$_link[$_lname];
                        if( $_ldata['Mtp2'] == $oldName )
                        {
                            $_ldata['Mtp2'] = $name;
                            $_data['Links'][$idx] = array( $name => $_ldata );
                        }
                    }
                }
                // delete old Mtp2
                if( !$this->deleteMtp2( $oldName ) )
                {
                    throw new SS7Exception( 'ss7-mtp2-error-not-found' );
                }
                // append new
                $this->mtp2[] = array( $name => $data );
            }
            elseif( $name === $oldName )
            {
                // update existing
                $notfound = true;
                foreach( $this->mtp2 as $idx => &$mtp2 )
                {
                    if( array_key_exists( $name, $mtp2 ) )
                    {
                        $mtp2[$name] = &$data;
                        $notfound = false;
                        break;
                    }
                }
                if( $notfound )
                {
                    throw new SS7Exception( 'ss7-mtp2-error-not-found' );
                }
            }
            else
            {
                // append new
                $this->mtp2[] = array( $name => $data );
            }
        }

        /**
         * Check if MTP2 is in use
         *
         * @param string $name MTP2 name
         * @return boolean true if in use
         */
        public function isMtp2InUse( $name )
        {
            foreach( $this->getAllMtp3LinkSets() as $item )
            {
                $_data = array_shift( $item );
                foreach( $_data['Links'] as $links )
                {
                    $_link = array_shift( $links );
                    if( $_link['Mtp2'] == $name )
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        /**
         * Delete mtp2 from configuration
         *
         * @param string $name MTP2 name
         * @return boolean true if was removed
         */
        public function deleteMtp2( $name )
        {
            if( $this->isMtp2InUse( $name ) )
            {
                throw new SS7Exception( 'ss7-mtp2-error-in-use' );
            }
            foreach( $this->mtp2 as $idx => &$info )
            {
                if( array_key_exists( $name, $info ) )
                {
                    unset( $this->mtp2[$idx] );
                    return true;
                }
            }
            return false;
        }

        /**
         * Count defined MTP2 links
         *
         * @return int
         */
        public function countMtp2( )
        {
            return count( $this->mtp2 );
        }

        /**
         * Get defined MTP2 names
         *
         * @param boolean $ignoreInUse ignore MTP2 in use
         * @return array
         */
        public function getMtp2Names( $ignoreInUse = false )
        {
            $names = array( );
            foreach( $this->getAllMtp2() as $idx => $item )
            {
                $_name = array_keys( $item );
                $_name = $_name[0];
                if( $ignoreInUse && $this->isMtp2InUse( $_name ) )
                {
                    continue;
                }
                $names[] = $_name;
            }
            return $names;
        }

        ///////////////////////////////////////////////////////////////////////
        // PointCodes link sets manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Check if PointCode is defined
         *
         * @param string $name PointCode name
         * @return boolean
         */
        public function hasPointCode( $name )
        {
            foreach( $this->pointCodes as &$info )
            {
                if( array_key_exists( $name, $info ) )
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * Lookup PointCode by its name
         *
         * @param string $name PointCode name
         * @return null|array
         */
        public function getPointCode( $name )
        {
            $pointcode = null;
            foreach( $this->pointCodes as $idx => &$_pointcode )
            {
                if( array_key_exists( $name, $_pointcode ) )
                {
                    $pointcode = $_pointcode[$name];
                    break;
                }
            }
            return $pointcode;
        }

        /**
         * Get all defined PointCodes as array
         *
         * @return array
         */
        public function getAllPointCodes( )
        {
            return $this->pointCodes;
        }

        /**
         * Set new or update PointCode
         *
         * @param string $name PointCode name
         * @param array $data PointCode data
         * @param string $oldName PointCode old name (if not empty and
         *   different from $name, then rename the PointCode)
         */
        public function setPointCode( $name, $data, $oldName = '' )
        {
            if( $oldName !== $name && $this->hasPointCode( $name ) )
            {
                throw new SS7Exception( 'ss7-pointcode-error-name-already-exists' );
            }
            elseif( !$this->isValidName( $name ) )
            {
                throw new SS7Exception( 'ss7-error-invalid-name' );
            }
            // rename
            elseif( !empty( $oldName ) && $oldName !== $name )
            {
                //rename entries using this point code
                foreach( $this->mtp3LinkSets as &$item )
                {
                    $_name = array_keys( $item );
                    $_name = $_name[0];
                    $_data = &$item[$_name];
                    if( $_data['Origin'] == $oldName )
                    {
                        $_data['Origin'] = $name;
                    }
                    if( $_data['Adjacent'] == $oldName )
                    {
                        $_data['Adjacent'] = $name;
                    }
                }
                foreach( $this->mtp3Routes as &$item )
                {
                    $_name = array_keys( $item );
                    $_name = $_name[0];
                    $_data = &$item[$_name];
                    if( $_data['PointCode'] == $oldName )
                    {
                        $_data['PointCode'] = $name;
                    }
                }
                foreach( $this->isupCircuitGroups as &$item )
                {
                    $_name = array_keys( $item );
                    $_name = $_name[0];
                    $_data = &$item[$_name];
                    if( $_data['Origin'] == $oldName )
                    {
                        $_data['Origin'] = $name;
                    }
                    if( $_data['Destination'] == $oldName )
                    {
                        $_data['Destination'] = $name;
                    }
                }
                // delete old point code
                if( !$this->deletePointCode( $oldName ) )
                {
                    throw new SS7Exception( 'ss7-ponitcode-error-not-found' );
                }
                $this->pointCodes[] = array( $name => $data );
            }
            // update
            elseif( $oldName === $name )
            {
                $notfound = true;
                foreach( $this->pointCodes as &$pointcode )
                {
                    if( array_key_exists( $name, $pointcode ) )
                    {
                        $pointcode[$name] = $data;
                        $notfound = false;
                        break;
                    }
                }
                if( $notfound )
                {
                    throw new SS7Exception( 'ss7-pointcode-error-not-found' );
                }
            }
            // append
            else
            {
                $this->pointCodes[] = array( $name => $data );
            }
        }

        /**
         * Check if the PointCode is in use
         *
         * @param string $name PointCode name
         * @return boolean
         */
        public function isPointCodeInUse( $name )
        {
            foreach( $this->getAllMtp3LinkSets() as $item )
            {
                $_data = array_shift( $item );
                if( $_data['Origin'] == $name || $_data['Adjacent'] == $name )
                {
                    return true;
                }
            }
            foreach( $this->getAllMtp3Routes() as $item )
            {
                $_data = array_shift( $item );
                if( $_data['PointCode'] == $name )
                {
                    return true;
                }
            }
            foreach( $this->getAllIsupCircuitGroups() as $item )
            {
                $_data = array_shift( $item );
                if( $_data['Origin'] == $name || $_data['Destination'] == $name )
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * Delete PointCode
         *
         * @param string $name PointCode name
         * @return boolean
         */
        public function deletePointCode( $name )
        {
            if( $this->isPointCodeInUse( $name ) )
            {
                throw new SS7Exception( 'ss7-pointcode-error-in-use' );
            }
            foreach( $this->pointCodes as $idx => &$_pointcode )
            {
                if( array_key_exists( $name, $_pointcode ) )
                {
                    unset( $this->pointCodes[$idx] );
                    return true;
                }
            }

            return false;
        }

        /**
         * Count defined PointCodes
         *
         * @return int
         */
        public function countPointCodes( )
        {
            return count( $this->pointCodes );
        }

        /**
         * Convert PointCode data to decimal
         *
         * @param array $data PointCode data
         * @return int
         */
        public function pointCodeToDec( $data )
        {
            if( count( $data ) !== 3 )
            {
                throw new SS7Exception( 'ss7-pointcode-error-invalid' );
            }

            $first = $this->pointCodeZeroFill( $data[0], 3 );
            $middle = $this->pointCodeZeroFill( $data[1], 8 );
            $last = $this->pointCodeZeroFill( $data[2], 3 );

            return bindec( "{$first}{$middle}{$last}" );
        }

        /**
         * Convert PointCode to string
         *
         * @param string $name
         * @param array $info Point code data (as in setPointCode)
         * @param boolean $includeName Include name in resulting string
         * @return string
         */
        public function pointCodeToString( $name, $info = null, $includeName = true )
        {
            if( is_null( $info ) )
            {
                $info = $this->getPointCode( $name );
                if( is_null( $info ) )
                {
                    throw new SS7Exception( 'ss7-pointcode-error-not-found' );
                }
            }

            if( $includeName )
            {
                return "{$name} ({$info[0]}-{$info[1]}-{$info[2]} / {$this->pointCodeToDec( $info )})";
            }
            else
            {
                return "{$info[0]}-{$info[1]}-{$info[2]}  / {$this->pointCodeToDec( $info )}";
            }
        }

        /**
         * Convert from decimal do binary, filling zeros on left, if needed
         *
         * @param int|string $var Variable to be converted
         * @param int $size Binary representation size
         * @return string
         */
        protected function pointCodeZeroFill( $var, $size )
        {
            $var = decbin( $var );
            if( strlen( $var ) >= $size )
            {
                return (string)$var;
            }
            return str_repeat( '0', $size - strlen( $var ) ) . $var;
        }

        /**
         * Get defined point code names
         *
         * @param boolean $ignoreInUse ignore point codes in use
         * @return array
         */
        public function getPointCodeNames( $ignoreInUse = false )
        {
            $names = array( );
            foreach( $this->getAllPointCodes() as $idx => $item )
            {
                $_name = array_keys( $item );
                $_name = $_name[0];
                $info = &$item[$_name];
                if( $ignoreInUse && $this->isPointCodeInUse( $_name ) )
                {
                    continue;
                }
                $names[$_name] = $this->pointCodeToString( $_name, $info );
            }
            return $names;
        }

        ///////////////////////////////////////////////////////////////////////
        // MTP3 link sets manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Check if MTP3 link set is defined
         *
         * @param string $name MTP3 link set name
         * @return boolean
         */
        public function hasMtp3LinkSet( $name )
        {
            foreach( $this->mtp3LinkSets as $idx => &$info )
            {
                if( array_key_exists( $name, $info ) )
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * Lookup MTP3 link set by its name
         *
         * @param string $name MTP3 link set name
         * @return null|array
         */
        public function getMtp3LinkSet( $name )
        {
            $linkset = null;
            foreach( $this->mtp3LinkSets as $idx => &$info )
            {
                if( array_key_exists( $name, $info ) )
                {
                    $linkset = $info[$name];
                    break;
                }
            }
            return $linkset;
        }

        /**
         * Get all defined MTP3 link sets as array
         *
         * @return array
         */
        public function getAllMtp3LinkSets( )
        {
            return $this->mtp3LinkSets;
        }

        /**
         * Set new or update MTP3 link set
         *
         * @param string $name link set name
         * @param array $data link set data
         * @param string $oldName link set old name
         */
        public function setMtp3LinkSet( $name, $data, $oldName = '' )
        {
            if( $oldName !== $name && $this->hasMtp3LinkSet( $name ) )
            {
                throw new SS7Exception( 'ss7-mtp3-linkset-error-name-already-exists' );
            }
            elseif( !$this->isValidName( $name ) )
            {
                throw new SS7Exception( 'ss7-error-invalid-name' );
            }
            // rename
            if( !empty( $oldName ) && $oldName !== $name )
            {
                // rename where its used
                foreach( $this->mtp3Routes as &$item )
                {
                    $_name = array_keys( $item );
                    $_name = $_name[0];
                    $_data = &$item[$_name];
                    foreach( $_data['LinkSets'] as &$_linkset )
                    {
                        if( $_linkset == $oldName )
                        {
                            $_linkset = $name;
                        }
                    }
                }
                // delete old link set
                if( !$this->deleteMtp3LinkSet( $oldName ) )
                {
                    throw new SS7Exception( 'ss7-mtp3-linkset-error-not-found' );
                }
                // append new
                $this->mtp3LinkSets[] = array( $name => $data );
            }
            elseif( $name === $oldName )
            {
                // update existing
                $notfound = true;
                foreach( $this->mtp3LinkSets as $idx => &$linkset )
                {
                    if( array_key_exists( $name, $linkset ) )
                    {
                        $linkset[$name] = &$data;
                        $notfound = false;
                        break;
                    }
                }
                if( $notfound )
                {
                    throw new SS7Exception( 'ss7-mtp3-linkset-error-not-found' );
                }
            }
            else
            {
                // append new
                $this->mtp3LinkSets[] = array( $name => $data );
            }
        }

        /**
         * Check if the link set is in use
         *
         * @param string $name link set name
         * @return boolean
         */
        public function isMtp3LinkSetInUse( $name )
        {
            foreach( $this->getAllMtp3Routes() as $item )
            {
                $_data = array_shift( $item );
                foreach( $_data['LinkSets'] as $_name )
                {
                    if( $name == $_name )
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        /**
         * Delete the link set
         *
         * @param string $name link set name
         * @return boolean
         */
        public function deleteMtp3LinkSet( $name )
        {
            if( $this->isMtp3LinkSetInUse( $name ) )
            {
                throw new SS7Exception( 'ss7-mtp3-linkset-error-in-use' );
            }
            foreach( $this->mtp3LinkSets as $idx => &$item )
            {
                if( array_key_exists( $name, $item ) )
                {
                    unset( $this->mtp3LinkSets[$idx] );
                    return true;
                }
            }
            return false;
        }

        /**
         * Count defined link sets
         *
         * @return int
         */
        public function countMtp3LinkSets( )
        {
            return count( $this->mtp3LinkSets );
        }

        /**
         * Get possible values (array key) and names (array value) for
         * NetworkIndicator
         *
         * @return array
         */
        public function getMtp3LinkSetNetworkIndicators( )
        {
            $indicators = array( );
            $indicators[] = $this->getMtp3LinkSetNetworkIndicatorName( 0 );
            $indicators[] = $this->getMtp3LinkSetNetworkIndicatorName( 1 );
            $indicators[] = $this->getMtp3LinkSetNetworkIndicatorName( 2 );
            $indicators[] = $this->getMtp3LinkSetNetworkIndicatorName( 3 );
            return $indicators;
        }

        /**
         * Translate network indicator ID to name
         *
         * @param int $indicator indicator ID
         * @return string
         */
        public function getMtp3LinkSetNetworkIndicatorName( $indicator )
        {
            //@formatter:off
            /* DECL_ENUM_ELEMENT_VAL_STR( niInternationalNetwork, 0x00, "International Network")
             * DECL_ENUM_ELEMENT_VAL_STR( niSpareInternational, 0x01, "Spare (International)")
             * DECL_ENUM_ELEMENT_VAL_STR( niNationalNetwork, 0x02, "National" )
             * DECL_ENUM_ELEMENT_VAL_STR( noSpareNational, 0x03, "Spare (National)" )
             */
            //@formatter:on
            $app = KSystem::getInstance( );

            switch( $indicator )
            {
                case 0:
                    return $app->translate( 'ss7-mtp3-linkset-network-indicator-international' );
                    break;
                case 1:
                    return $app->translate( 'ss7-mtp3-linkset-network-indicator-international-spare' );
                    break;
                case 2:
                    return $app->translate( 'ss7-mtp3-linkset-network-indicator-national' );
                    break;
                case 3:
                    return $app->translate( 'ss7-mtp3-linkset-network-indicator-national-spare' );
                    break;
                default:
                    return $app->translate( 'ss7-mtp3-linkset-network-indicator-unknown' );
                    break;
            }
        }

        /**
         * Get possible values for MTP2 associated Link SLC, optionally
         * removing values already in use (supplied as Mtp3 LinkSet array)
         *
         * @param array $inUseLinkSet linkset data (as in setMtp3LinkSet) array
         * @return array
         */
        public function getMtp3LinkSetMtp2LinkSlcs( $inUseLinkSet = null )
        {
            $slcs = range( 0, 15 );
            if( !is_null( $inUseLinkSet ) )
            {
                foreach( $inUseLinkSet['Links'] as $idx => &$link )
                {
                    $_name = array_keys( $link );
                    $_name = $_name[0];

                    $pos = array_search( $link[$_name]['Slc'], $slcs );
                    unset( $slcs[$pos] );
                }
            }
            return $slcs;
        }

        /**
         * Get MTP3 defined linkset names
         *
         * @param boolean $ignoreInUse ignore linksets in use
         * @param array $exclude Exclude names listed in this array
         * @return array
         */
        public function getMtp3LinkSetNames( $ignoreInUse = false, $exclude = array() )
        {
            $names = array( );
            foreach( $this->getAllMtp3LinkSets() as $idx => $item )
            {
                $_name = array_keys( $item );
                $_name = $_name[0];
                $info = &$item[$_name];
                if( ($ignoreInUse && $this->isMtp3LinkSetInUse( $_name )) || in_array( $_name, $exclude ) )
                {
                    continue;
                }
                $names[] = $_name;
            }
            return $names;
        }

        ///////////////////////////////////////////////////////////////////////
        // MTP3 routes manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Check if MTP3 route is defined
         *
         * @param string $name route name
         * @return boolean
         */
        public function hasMtp3Route( $name )
        {
            foreach( $this->mtp3Routes as $idx => &$info )
            {
                if( array_key_exists( $name, $info ) )
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * Get MTP3 route
         *
         * @param string $name route name
         * @return null|array
         */
        public function getMtp3Route( $name )
        {
            $mtp3route = null;
            foreach( $this->mtp3Routes as $idx => &$route )
            {
                if( array_key_exists( $name, $route ) )
                {
                    $mtp3route = $route[$name];
                    break;
                }
            }
            return $mtp3route;
        }

        /**
         * Get all defined MTP3 routes
         *
         * @return array
         */
        public function getAllMtp3Routes( )
        {
            return $this->mtp3Routes;
        }

        /**
         * Set new or update existing MTP3 route
         *
         * @param string $name route name
         * @param array $data route information data array
         * @param string $oldName route old name
         */
        public function setMtp3Route( $name, $data, $oldName = '' )
        {
            // new name already exists
            if( $name !== $oldName && $this->hasMtp3Route( $name ) )
            {
                throw new SS7Exception( 'ss7-mtp3-route-error-already-exists' );
            }
            elseif( !$this->isValidName( $name ) )
            {
                throw new SS7Exception( 'ss7-error-invalid-name' );
            }
            // renamed MTP3 route
            elseif( !empty( $oldName ) && $name !== $oldName )
            {
                //TODO: rename where this entry was used
                // delete old Mtp3
                if( !$this->deleteMtp3Route( $oldName ) )
                {
                    throw new SS7Exception( 'ss7-mtp3-route-error-not-found' );
                }
                // append new
                $this->mtp3Routes[] = array( $name => $data );
            }
            elseif( $name === $oldName )
            {
                // update existing
                $notfound = true;
                foreach( $this->mtp3Routes as $idx => &$mtp3route )
                {
                    if( array_key_exists( $name, $mtp3route ) )
                    {
                        $mtp3route[$name] = &$data;
                        $notfound = false;
                        break;
                    }
                }
                if( $notfound )
                {
                    throw new SS7Exception( 'ss7-mtp3-route-error-not-found' );
                }
            }
            else
            {
                // append new
                $this->mtp3Routes[] = array( $name => $data );
            }
        }

        /**
         * Check if the MTP3 route is being used
         *
         * TODO: implement
         *
         * @param string $name route name
         * @return boolean
         */
        public function isMtp3RouteInUse( $name )
        {
            return false;
        }

        /**
         * Delete the MTP3 route
         *
         * @param string $name route name
         * @return boolean
         */
        public function deleteMtp3Route( $name )
        {
            if( $this->isMtp3RouteInUse( $name ) )
            {
                throw new SS7Exception( 'ss7-mtp3-route-error-in-use' );
            }
            foreach( $this->mtp3Routes as $idx => &$info )
            {
                if( array_key_exists( $name, $info ) )
                {
                    unset( $this->mtp3Routes[$idx] );
                    return true;
                }
            }
            return false;
        }

        /**
         * Count how many MTP3 routes are defined
         *
         * @return int
         */
        public function countMtp3Routes( )
        {
            return count( $this->mtp3Routes );
        }

        ///////////////////////////////////////////////////////////////////////
        // ISUP Circuit Group methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Check if ISUP Circuit Group is defined
         *
         * @param string $name Circuit Group name
         * @return boolean
         */
        public function hasIsupCircuitGroup( $name )
        {
            foreach( $this->isupCircuitGroups as $idx => &$info )
            {
                if( array_key_exists( $name, $info ) )
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * Check if ISUP Circuit Group BaseCIC is already in use
         *
         * @param array $data data array as in setIsupCircuitGroup
         * @param string|array $ignore ignore the named circuit group
         *     configuration (used to overwrite configuration)
         * @return boolean
         */
        public function isIsupCircuitGroupBaseCicInUse( $data, $ignore = '' )
        {
            if( !is_array( $data ) )
            {
                throw new SS7Exception( 'ss7-invalid-data' );
            }
            foreach( $this->getAllIsupCircuitGroups( ) as $item )
            {
                $_name = array_keys( $item );
                $_name = $_name[0];
                $_data = &$item[$_name];
                if( (is_array( $ignore ) && in_array( $_name, $ignore )) || (is_string( $ignore ) && $_name === $ignore) )
                {
                    continue;
                }
                elseif( $_data['Origin'] == $data['Origin'] && $_data['Destination'] == $data['Destination'] && $_data['BaseCIC'] == $data['BaseCIC'] )
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * Get ISUP Circuit Group
         *
         * @param string $name Circuit Group name
         * @return null|array
         */
        public function getIsupCircuitGroup( $name )
        {
            $circuitgroup = null;
            foreach( $this->isupCircuitGroups as $idx => &$item )
            {
                $_name = array_keys( $item );
                $_name = $_name[0];
                if( $name === $_name )
                {
                    $circuitgroup = $item[$name];
                    break;
                }
            }
            return $circuitgroup;
        }

        /**
         * Get all ISUP Circuit Groups
         *
         * @return array
         */
        public function getAllIsupCircuitGroups( )
        {
            return $this->isupCircuitGroups;
        }

        /**
         * Set new or update ISUP Circuit Group
         *
         * @param string $name name
         * @param array $data configuration array
         * @param string $oldName old name (to rename)
         */
        public function setIsupCircuitGroup( $name, $data, $oldName )
        {
            // new name already exists
            if( $name !== $oldName && $this->hasIsupCircuitGroup( $name ) )
            {
                throw new SS7Exception( 'ss7-isup-circuit-group-error-name-already-exists' );
            }
            // ignore BaseCIC from (old) name when checking
            elseif( $this->isIsupCircuitGroupBaseCicInUse( $data, array( $name, $oldName ) ) )
            {
                throw new SS7Exception( 'ss7-isup-circuit-group-error-basecic-in-use' );
            }
            elseif( !$this->isValidName( $name ) )
            {
                throw new SS7Exception( 'ss7-error-invalid-name' );
            }
            // renamed configuration
            elseif( !empty( $oldName ) && $name !== $oldName )
            {
                //TODO: rename where this entry was used
                // delete old configuration
                if( !$this->deleteIsupCircuitGroup( $oldName ) )
                {
                    throw new SS7Exception( 'ss7-isup-circuit-group-error-not-found' );
                }
                // append new
                $this->isupCircuitGroups[] = array( $name => $data );
            }
            // update existing
            elseif( $name === $oldName )
            {
                $notfound = true;
                foreach( $this->isupCircuitGroups as $idx => &$item )
                {
                    if( array_key_exists( $name, $item ) )
                    {
                        $item[$name] = &$data;
                        $notfound = false;
                        break;
                    }
                }
                if( $notfound )
                {
                    throw new SS7Exception( 'ss7-isup-circuit-group-error-not-found' );
                }
            }
            // append new
            else
            {
                $this->isupCircuitGroups[] = array( $name => $data );
            }
        }

        /**
         * Check if the ISUP Circuit Group is in use
         *
         * TODO: implement
         *
         * @param string $name name
         * @return boolean
         */
        public function isIsupCircuitGroupInUse( $name )
        {
            return false;
        }

        /**
         * Remove ISUP Circuit Group
         *
         * @param string $name name
         * @return boolean
         */
        public function deleteIsupCircuitGroup( $name )
        {
            if( $this->isIsupCircuitGroupInUse( $name ) )
            {
                throw new SS7Exception( 'ss7-isup-circuit-group-error-in-use' );
            }
            else
            {
                foreach( $this->getAllIsupCircuitGroups() as $idx => $item )
                {
                    $_name = array_keys( $item );
                    $_name = $_name[0];
                    if( $name === $_name )
                    {
                        unset( $this->isupCircuitGroups[$idx] );
                        return true;
                    }
                }
            }
            return false;
        }

        /**
         * Count how many ISUP Circuit Groups are defined
         *
         * @return int
         */
        public function countIsupCircuitGroups( )
        {
            return count( $this->isupCircuitGroups );
        }

        /**
         * Check if timeslot is set
         *
         * @param int $timeslotmap
         * @param int $timeslot
         * @return boolean
         */
        public function isIsupCirctuitGroupTimeslotSet( $timeslotmap, $timeslot )
        {
            // reserved values are never set
            if( $this->isIsupCircuitGroupTimeslotReserved( $timeslot ) )
            {
                return false;
            }
            $timeslot = (1<<$timeslot);
            return ($timeslotmap & $timeslot) === $timeslot;
        }

        /**
         * Set timeslot bit
         *
         * @param int $timeslotmap
         * @param int $timeslot
         * @return int
         */
        public function setIsupCircuitGroupTimeslotBit( $timeslotmap, $timeslot )
        {
            if( $timeslot < 0 || $timeslot > 31 )
            {
                throw new SS7Exception( 'ss7-isup-circuit-group-error-invalid-timeslot' );
            }
            // check reserved values
            if( $this->isIsupCircuitGroupTimeslotReserved( $timeslot ) )
            {
                throw new SS7Exception( 'ss7-isup-circuit-group-error-cant-set-reserved-bits' );
            }
            $timeslotmap |= (1<<$timeslot);
            return $timeslotmap;
        }

        /**
         * Check if timeslot is reserved
         *
         * @param int $timeslot
         */
        public function isIsupCircuitGroupTimeslotReserved( $timeslot )
        {
            return in_array( $timeslot, self::$isupCircuitGroupsTimeslotReserved );
        }

        ///////////////////////////////////////////////////////////////////////
        // ISUP Timers methods
        ///////////////////////////////////////////////////////////////////////
        /**
         * Get timer
         *
         * @param string $name
         * @return integer
         */
        public function getTimer( $name )
        {
            if( array_key_exists( $name, $this->isupTimers ) )
            {
                return $this->isupTimers[$name];
            }
            else
            {
                throw new SS7Exception( 'ss7-isup-timer-error-not-found' );
            }
        }

        /**
         * Get all timers
         *
         * @return array
         */
        public function getAllTimers( )
        {
            return $this->isupTimers;
        }

        /**
         * Set timer
         *
         * @param string $name
         * @param integer $value
         */
        public function setTimer( $name, $value )
        {
            if( array_key_exists( $name, $this->isupTimers ) )
            {
                $this->isupTimers[$name] = $value;
            }
            else
            {
                throw new SS7Exception( 'ss7-isup-timer-error-not-found' );
            }
        }

        /**
         * Set all timers
         *
         * @param array $timers
         */
        public function setAllTimers( $timers )
        {
            if( array_keys( $timers ) === array_keys( $this->isupTimers ) )
            {
                $this->isupTimers = $timers;
            }
            else
            {
                throw new SS7Exception( 'ss7-isup-timer-error-invalid-array' );
            }
        }

    }
