(function (Khomp) {

    /// ----------------------------------------------------------------------------------------------
    /// Utils: funções auxiliares

    var Util = Khomp.Util = {

        paddingLeft : function (value, length) {

            length = length || 0;

            return (length > value.toString().length) ? this.paddingLeft("0" + value, length) : value;
        },

        binToNumber : function (num) {
            return parseInt(num, 2);
        },

        binToHex : function (num) {
            return this.numberToHex(this.binToNumber(num));
        },

        numberToBin : function (num, paddingLeft) {
            paddingLeft = paddingLeft || 8;
            num = (typeof num === 'string') ? this.hexToNumber(num) : num;
            return this.paddingLeft((num).toString(2), paddingLeft);
        },

        numberToHex : function (number) {
            return this.paddingLeft(number.toString(16).toUpperCase(), 2);
        },

        hexToNumber : function (hexaString) {
            return parseInt(hexaString, 16);
        },

        hexToBin : function (hexString) {
            return this.numberToBin(this.hexToNumber(hexString));
        },

        arrayMapHexToNumber : function (arrayHexa) {

            var aux = [];

            for (var i = 0; i < arrayHexa.length; i++) {
                aux.push(this.hexToNumber(arrayHexa[i]));
            }

            return aux;
        },

        /// bits contados de 1 a 8. Exemplo:
        /// 1 0 1 0 1 1 1 0   -> número de exemplo
        /// | | | | | | | |
        /// 8 7 6 5 4 3 2 1   -> posição dos bits
        testBit : function (num, bit) {
            return ((num >> (bit - 1)) % 2 !== 0);
        },

        setBit : function (num, bit) {
            return num | 1 << (bit - 1);
        },

        clearBit : function (num, bit) {
            return num & ~(1 << (bit - 1));
        },

        getBitsString : function (num, posicaoBits) {

            num = (typeof num === 'string') ? this.hexToNumber(num) : num;
            var bits = [];

            for (var i = 0; i < posicaoBits.length; i++) {
                bits.push((this.testBit(num, posicaoBits[i])) ? 1 : 0);
            }

            return bits.join('');
        },

        toAscii : function (code) {
            code = (typeof code === 'string') ? this.hexToNumber(code) : code;
            return String.fromCharCode(code);
        },
        /// ---------------------------------------------------------------------------------------------------
        /// Extraído de: http://djh1.webfactional.com/html/ascii.html
        /**
            @method translateIA5
            @param {Array} data Lista com os charcodes...
            @return {String}
        */
        translateIA5 : function (data) {

            data = (typeof data === 'string') ? data.split(' ') : data;

            return String.fromCharCode.apply(null, data);
        }
    };

})(Khomp);


(function (Khomp) {
    /**
    Decodificação do protocolo ISDN
    Webanalisador de log de mensagens Q.931

    Documentação auxiliar: http://devserver/svn/API/docs/Interno/ISDN/Normas/Normas%20ITU-T/

    @module isdn
    @submodule isdn-decoder
    @class ISDN
     */
    var ISDN = Khomp.ISDN = {

        logObj : null,
        logSource : '',
        messagesSuccess : {},
        busyChannels : {},
        usedChannels : {},
        /// Total de mensagens que foram apresentadas sem uma mensagem de SETUP. Isso indica que a mensagem de SETUP deve estar em outro arquivo de log mais antigo.
        totalMessagesWithoutSetup : 0

    };

    /**
    Limpa dos dados para uma novo processamento.

    @method clear
    @static
     */
    ISDN.clear = function () {

        this.logObj = null;
        this.logSource = '';
        this.messagesSuccess = {};
        this.totalMessagesWithoutSetup = 0;

        this.usedChannels = {};
        this.busyChannels = {};
    };

    /**
    Inicia o processamento parcial ou total do arquivo de log.
    Para iniciar uma nova análise deve ser chamado o método clear.

    Feito com base no formato de mensagens explicado na pg 47 da ITU Q.931

    @method extractData
    @static
    @param {String} logSource Conteúdo do arquivo isdn.log
    @return {Object} Log object com todos os dados das chamadas e mensagens

     */
    ISDN.extractData = function (logSource, callback) {

        console.time('TOTAL extractData:');

        callback = callback || function () {};

        logSource = (typeof logSource === 'string') ? logSource.split('\n') : logSource;

        var _this = this;
        var call;
        var callId;
        var line;
        var Message = this.Message;
        var messageObj;
        var InformationElements = this.InformationElements;

        console.log('Processando arquivo...', logSource.length);

        var logObj = this.logObj = (this.logObj !== null) ? this.logObj : {
            calls : [],
            messageTypes : {},
            links : {}
        };

        for (var i = 0, lines_len = logSource.length; i < lines_len; i++) {

            messageObj = new Message(logSource[i]);

            if (messageObj && messageObj.isValid !== false) {

                /// Para ter referência da linha do log
                messageObj.logLine = i;

                var deviceId = messageObj.getDeviceId();
                var datetime = messageObj.getDatetime();
                var type = messageObj.getType();
                var type_name = messageObj.getTypeName();
                var infos = messageObj.getInformationElements();
                var link = messageObj.getLink();
                var crv = messageObj.getCallReference();

                var key = deviceId + link + crv;

                call = this.busyChannels[key] || null;

                /// Considerar nova chamada quando a mensagem é SETUP (0x05)
                if (!call || type === 0x05) {

                    var crvDirection = messageObj.getCallReferenceDirection();

                    call = {
                        id : logObj.calls.length + 1,
                        Status : 'ERROR',
                        CallReference : crv,
                        Link : link,
                        Start : datetime,
                        // Nai : messageObj.nai,
                        deviceId : deviceId,
                        Direction : crvDirection,
                        // From : '',
                        // To : '',
                        Messages : []
                    };

                    logObj.calls.push(call);
                    this.busyChannels[key] = call;
                    logObj.links[link] = (logObj.links[link]) ? (logObj.links[link] + 1) : 1;

                }

                if (!call) {
                    // break;
                    continue;
                }
                
                callId = call.id;
                
                logObj.messageTypes[type_name] = (logObj.messageTypes[type_name]) ? logObj.messageTypes[type_name] + 1 : 1;

                _this.messagesSuccess[callId] = _this.messagesSuccess[callId] || {
                    'SETUP' : false,
                    'CONNECT' : false,
                    'DISCONNECT' : false,
                    'RELEASE' : false,
                    'RELEASE COMPLETE' : false
                };

                /// Vai associando a data da mensagem como sendo a última.
                call.Stop = datetime;

                call.Messages.push(messageObj);

                if (type_name in _this.messagesSuccess[callId]) {
                    _this.messagesSuccess[callId][type_name] = true;
                }
            }
        }

        callback();

        console.timeEnd('TOTAL extractData:');

        // this.logSource += logSource;

        return logObj;
    };

    /**
    Analisa cada uma das chamadas extraídas

    @method makeCallAnalysys
    @static
    @param {Function} callback
     */
    ISDN.makeCallAnalysys = function (callback) {

        /// Verifica se as chamadas têm erro a partir da análise das mensagens.
        console.time('TOTAL makeCallAnalysys:');

        var logObj = this.logObj;
        var calls = logObj.calls;
        var call;
        var messagesSuccess;

        logObj.totalCalls = 0;
        logObj.totalErrors = 0;
        logObj.totalAlerts = 0;
        logObj.totalSuccess = 0;
        logObj.errorCalls = [];
        logObj.alertCalls = [];
        logObj.firstCall = null;
        logObj.lastCall = null;
        logObj.messagesLengthByCall = [];

        callback = callback || function () {};

        for (var callPos = 0, calls_len = calls.length; callPos < calls_len; callPos++) {

            logObj.totalCalls++;
            call = calls[callPos];
            var callId = call.id;

            if (!logObj.firstCall) {
                logObj.firstCall = call;
            }

            logObj.lastCall = call;
            
            var msgs_length = logObj.lastCall.Messages.length;

            logObj.messagesLengthByCall.push(msgs_length);
            messagesSuccess = this.messagesSuccess[callId] || {};

            if (messagesSuccess.SETUP === true && messagesSuccess.RELEASE === true && messagesSuccess['RELEASE COMPLETE'] === true) {

                /// SUCCESS
                if (messagesSuccess.CONNECT === true && messagesSuccess.DISCONNECT === true) {

                    logObj.totalSuccess++;

                    call.Status = 'OK';

                } else {

                    /// ALERT

                    logObj.totalAlerts++;
                    logObj.alertCalls.push(callId);

                    call.Status = 'ALERT';
                }
            } else {

                logObj.errorCalls.push(callId);
                logObj.totalErrors++;
                call.Status = 'ERROR';
            }
        }

        console.timeEnd('TOTAL makeCallAnalysys:');
        callback();
    };

    /**
    Identifica se o formato do log é antigo (anterior à versão 3).

    @method isOldFormat
    @static
    @param {Mixed} logSource Conteúdo do arquivo de log em texto ou lista.
    @return {Number
     */
    ISDN.isOldFormat = function (logSource) {

        logSource = (typeof logSource === 'string') ? logSource.split('\n') : logSource;
        var line;

        for (var i = 0, lines_len = logSource.length; i < lines_len; i++) {

            line = logSource[i];

            /// Confere a barra '/' depois do dia que não tem na linha do formato antigo.
            if (line.indexOf('T|') === 0 && line.substr(4, 1) !== '/') {
                return true;
            }

        }

        return false;
    };

    /**
    Extrai o número de A (From) e B (To) da chamada.

    @method getCallFromTo
    @static
     */
    ISDN.getCallFromTo = function (call) {

        call.From = '';
        call.To = '';

        var msgs = call.Messages;
        var infos;
        var info;

        for (var msg_pos = 0, msgs_len = msgs.length; msg_pos < msgs_len; msg_pos++) {

            infos = msgs[msg_pos].infos;
            if (!infos) {
                continue;
            }

            for (var info_pos = 0, infos_len = infos.length; info_pos < infos_len; info_pos++) {
                info = infos[info_pos];
                if (info[0] === 0x6c) {
                    call.From += this.InformationElements.decodeInfo(0x6c, info.slice(1))['Number digits'];
                }
                if (info[0] === 0x70) {
                    call.To += this.InformationElements.decodeInfo(0x70, info.slice(1))['Number digits'];
                }
            }
        }
    };

    /**
    Verifica se a chamada tem todas os tipos de mensagens desejados

    @method callHasAllThisMessages
    @static
    @param {Array} messageTypes Lista com o código ou texto do tipo da mensagem
    @param {Object} call
    @return {Boolean}
     */
    ISDN.callHasAllThisMessages = function (messageTypes, call) {

        messageTypes = (typeof messageTypes === 'string' || typeof messageTypes === 'number') ? [messageTypes] : messageTypes;

        var found = 0;
        var msgs = call.Messages;
        var type;
        var regex_number = /^\d+$/;

        for (var i = 0, types_len = messageTypes.length; i < types_len; i++) {

            type = (regex_number.test(messageTypes[i])) ? parseInt(messageTypes[i], 10) : messageTypes[i];

            for (var j = 0, msgs_len = msgs.length; j < msgs_len; j++) {

                if (typeof type === 'number') {
                    if (msgs[j].type === type) {

                        found++;
                        break;
                    }
                } else {

                    if (msgs[j].getTypeName() === type) {

                        found++;
                        break;
                    }
                }
            }
        }

        return (types_len === found) ? true : false;

    };

    /**
    Procura por chamadas que tem uma das infos desejadas

    @method getCallsHasOneOfThisMessageInfos
    @static
    @param {Object} infos
    @param {Function} fnMatch Função para refinar a busca.
    @return {Array} Lista de chamadas que têm uma  das infos passadas
     */
    ISDN.getCallsHasOneOfThisMessageInfos = function (infos, fnMatch) {

        infos = (typeof infos === 'string') ? [infos] : infos;

        var found = 0;
        var calls = this.logObj.calls;
        var call;
        var msgs;
        var msg_pos;
        var msg_infos;
        var info;
        var info_pos;
        var calls_found = [];

        for (var call_pos = 0, calls_len = calls.length; call_pos < calls_len; call_pos++) {

            call = calls[call_pos];
            call.sequence = call_pos;
            msgs = call.Messages;
            found = false;

            for (msg_pos = 0, msgs_len = msgs.length; msg_pos < msgs_len; msg_pos++) {

                msg_infos = msgs[msg_pos].infos;

                if (!msg_infos) {
                    continue;
                }

                for (var msg_info_pos = 0, msg_infos_len = msg_infos.length; msg_info_pos < msg_infos_len; msg_info_pos++) {

                    info = msg_infos[msg_info_pos];

                    for (info_pos = 0, infos_len = infos.length; info_pos < infos_len; info_pos++) {

                        if (infos[info_pos] === info[0]) {

                            if (typeof fnMatch === 'function' && !fnMatch(info)) {
                                continue;
                            }

                            found = true;
                            break;
                        }
                    }
                    if (found) {
                        break;
                    }
                }
                if (found) {
                    calls_found.push(call);
                    break;
                }
            }

        }

        return calls_found;

    };

    /**
    Procura por chamadas que tem mensagens que têm parte do source desejado

    @method getCallsByMessageSource
    @static
    @param {String} source Parte do source para procurar.
    @return {Array} Lista de chamadas
     */
    ISDN.getCallsByMessageSource = function (source) {

        var calls = this.logObj.calls;
        var matchCalls = [];
        var call;
        var msg;

        for (var i = 0, calls_len = calls.length; i < calls_len; i++) {
            call = calls[i];

            for (var j = 0, msgs_len = call.Messages.length; j < msgs_len; j++) {

                msg = call.Messages[j];

                if (msg.source.indexOf(source) !== -1) {
                    matchCalls.push(call);
                }
            }
        }
        return matchCalls;
    };

    /**
    Procura por chamadas que tem mensagens com diferentes devices ou links. Se encontrar alguma está ocorrendo algum erro na separação das mensagens por chamada.

    @method getCallsWithDifferentsDevicesOrLinks
    @static
    @return {Array} Lista de chamadas com problema
     */
    ISDN.getCallsWithDifferentsDevicesOrLinks = function () {

        var calls = this.logObj.calls;
        var matchCalls = [];
        var call;
        var msg;
        var link;
        var deviceId;

        for (var call_pos = 0, calls_len = calls.length; call_pos < calls_len; call_pos++) {

            call = calls[call_pos];
            deviceId = call.deviceId;
            link = call.Link;

            for (var j = 0, msgs_len = call.Messages.length; j < msgs_len; j++) {

                msg = call.Messages[j];

                if (deviceId !== msg.getDeviceId() || link !== msg.getLink()) {
                    matchCalls.push(call);
                    break;
                }
            }
        }

        return matchCalls;
    };

    /**
    Procura por chamadas que tem mensagens com CRV diferente da chamada (considerar envio ou recebimento). Se encontrar alguma está ocorrendo algum erro na separação das mensagens por chamada.

    @method getCallsDifferentCRV
    @static
    @return {Array} Lista de chamadas com problema
     */
    ISDN.getCallsDifferentCRV = function () {

        var calls = this.logObj.calls;
        var matchCalls = [];
        var call;
        var msg;
        var crv;

        for (var call_pos = 0, calls_len = calls.length; call_pos < calls_len; call_pos++) {

            call = calls[call_pos];
            crv = call.CallReference;

            for (var j = 0, msgs_len = call.Messages.length; j < msgs_len; j++) {

                msg = call.Messages[j];

                if (crv === msg.getCallReference() || crv === msg.getCallReferenceOther()) {
                    continue;
                }
                matchCalls.push(call);
                break;
            }
        }

        return matchCalls;
    };

    /**
    Feito com base no formato de mensagens explicado na pg 47 da ITU Q.931

    @method getCallsWithError
    @static
    @param {String} Conteúdo do arquivo isdn.log
    @return {Object} Log object com todos os dados das chamadas e mensagens
     */
    ISDN.getCallsWithError = function () {

        var messages = [];
        var calls = this.logObj.calls;
        var errorIds = this.logObj.errorCalls;

        for (var i in errorIds) {
            messages.push(calls[errorIds[i]]);
        }

        return messages;
    };

    /**
    Para testes mandar reprocessar a decodificação

    @method reprocess
    @static
    @return {Boolean}
     */
    ISDN.reprocess = function () {

        if (this.logSource) {
            var logSource = this.logSource;
            this.clear();
            this.extractData(logSource, function () {
                console.log('Processado!');
            });
            return true;
        }
        return false;
    };

})(Khomp);


(function (Khomp) {

    var ISDN = Khomp.ISDN;

    /**
    Decodificação dos 'Information Elements'

    @module isdn
    @submodule isdn-decoder
    @class InformationElements
     */
    var InformationElements = ISDN.InformationElements = {

        /**
        Tipos conhecidos de information elements

        Tradução dos códigos extraída da ITU-T
        Complementado com: http://www.zytrax.com/tech/protocols/isdn/ie.htm

        @attribute types
        @final
         */
        types : {

            /// 1:::---- Single octet information elements (type a (1) e b (2))
            0x80 : 'Reserved', // 1[000]---- - type 1
            0x90 : 'Shift', // 1[001]---- - type 1

            0xa0 : 'More data', // 1[010]0000 - type 2
            0xa1 : 'Sending complete', // 1[010]0001 - type 2

            0xb0 : 'Congestion level', // 1[011]---- - type 1
            0xd0 : 'Repeat indicator', // 1[101]---- - type 1

            /// 0::::::: Variable length information element  (type c (3))
            0x00 : 'Segmented message',
            0x04 : 'Bearer capability',
            0x08 : 'Cause',
            0x0d : 'Extended facility',
            0x10 : 'Call identity',
            0x14 : 'Call state',
            0x18 : 'Channel identification',
            0x1c : 'Facility',
            0x1e : 'Progress indicator',
            0x20 : 'Network-specific facilities',
            0x27 : 'Notification indicator',
            0x28 : 'Display',
            0x29 : 'Date/time',
            0x2c : 'Keypad facility',
            0x32 : 'Information request',
            0x34 : 'Signal',
            0x38 : 'Feature activation',
            0x39 : 'Feature indication',
            0x3a : 'SPID - Service Profile Identification',
            0x3b : 'Endpoint identifier',
            0x40 : 'Information rate',
            0x42 : 'End-to-end transit delay',
            0x43 : 'Transit delay selection and indication',
            0x44 : 'Packet layer binary parameters',
            0x45 : 'Packet layer window size',
            0x46 : 'Packet size',
            0x47 : 'Closed user group',
            0x4a : 'Reverse charging indication',
            0x4c : 'Connected number',
            0x6c : 'Calling party number',
            0x6d : 'Calling party subaddress',
            0x70 : 'Called party number',
            0x71 : 'Called party subaddress',
            0x74 : 'Redirecting number',
            0x76 : 'Redirection number',
            0x78 : 'Transit network selection',
            0x79 : 'Restart indicator',
            0x7c : 'Low layer compatibility',
            0x7d : 'High layer compatibility',
            0x7e : 'User-user',
            0x7f : 'Escape for extension'
        },

        /**
        Se o bit é 0 (ZERO) continua, se é 1 (UM) é o último octeto
        Esta regra é utilizada e descrita detalhadamente na ITU Q.931

        @method isLastOctet
        @param {Number} num Octeto para verificar o bit de continuidade de informações
        @return {Boolean}
         */
        isLastOctet : function (num) {

            num = (typeof num === 'number') ? num : parseInt(num, 16);
            return ((num >> (7)) % 2 !== 0) ? true : false;
        },

        /**
        O elemento de informação pode ser representado de três formas (pg 52):
        Os três tipos são identificados como 1,2,3

        @method getOctetFormat
        @param elementCode
        @return {Number} Número associado ao tipo do elemento
         */
        getOctetFormat : function (elementCode) {

            var code = (typeof elementCode === 'string') ? parseInt(elementCode, 16) : elementCode;

            /// Se o primeiro bit é ZERO
            if ((code & 0x80) >> 7 === 0) {
                return 3;
            }

            /// Se o primeiro bit é UM
            if (code === 0xa1 || code === 0xa0) {
                return 2;
            }

            return 1;
        },

        /**
        Decodifica um Information element

        @method decodeInfo
        @param {Number} elementCode Código do Information element
        @param {Array} data Dados do Information element
        @return {Object} Decoded data
         */
        extractInfos : function (data) {

            var octetFormat;
            var elementCode;
            var infoSlice;
            var obj = {};
            var infos = [];
            var info;
            while (data.length) {

                elementCode = data.shift();

                octetFormat = this.getOctetFormat(elementCode);

                switch (octetFormat) {

                case 1:
                case 2:
                    infos.push([elementCode]);
                    // obj[elementCode] = null;
                    break;

                default:
                    infoSlice = data.shift();

                    info = data.splice(0, infoSlice);
                    info.unshift(elementCode);

                    infos.push(info);

                    // obj[elementCode] = data.splice(0, infoSlice);
                }
            }
            return infos;
            // return obj;
        },

        /**
        Decodifica um Information element

        @method decodeInfo
        @param {Number} elementCode Código do Information element
        @param {Array} data Dados do Information element
        @return {Object} Decoded data
         */
        decodeInfo : function (elementCode, data) {

            var key;
            var value;
            var elementName;

            var code = (typeof elementCode === 'number') ? elementCode : parseInt(elementCode, 16);

            /// Extension element (0x7f), see details in ITU Q.931
            if (code === 0x7f) {

                key = data[2];
                var length = data[1];

                return data.slice(3, length + 2);
            }

            var octetFormat = this.getOctetFormat(code);

            switch (octetFormat) {

            case 1:

                key = code;
                value = (code & 0x0f);

                /// TODO: Tratamento especial para quando Shift. Lock e Non-lock

                value = (key in this.elementsDecoders) ? this.elementsDecoders[key](value) : value;

                break;

            case 2:

                key = code;
                value = true;

                value = (key in this.elementsDecoders) ? this.elementsDecoders[key](value) : value;

                break;

            case 3:

                key = code;
                value = data.slice(0);

                if (!(key in this.elementsDecoders)) {
                    console.log('UNKNOW ELEMENT:', key, value);
                }

                value = (key in this.elementsDecoders) ? this.elementsDecoders[key](value) : value;

                break;

            default:
                return;
            }

            return value;
        },

        /**
        Atributos suportados pelos elementos
        Chave valor numérica para performance

        @attribute attributeNames
        @final
         */
        attributeNames : {
            0x00 : 'Additional layer 3 protocol information',
            0x01 : 'Assignor/assignee',
            0x02 : 'Call state value',
            0x03 : 'Causes',
            0x04 : 'Channel type/map element type',
            0x05 : 'Class',
            0x06 : 'Coding standard',
            0x07 : 'D-channel indicator',
            0x08 : 'Default packet size',
            0x09 : 'Extended audiovisual characteristics identification',
            0x0a : 'Extended high layer characteristics identification',
            0x0b : 'Flow control on Rx',
            0x0c : 'Flow control on Tx',
            0x0d : 'High layer characteristics identification',
            0x0e : 'In-band/out-band negotiation',
            0x0f : 'Information channel selection',
            0x10 : 'Information transfer capability',
            0x11 : 'Information transfer rate',
            0x12 : 'Interface identifier present',
            0x13 : 'Interface type',
            0x14 : 'Intermediate rate',
            0x15 : 'Interpretation',
            0x16 : 'Location',
            0x17 : 'Logical link identifier negotiation',
            0x18 : 'Mode duplex',
            0x19 : 'Mode of operation',
            0x1a : 'Mode of operation layer 2',
            0x1b : 'Mode of operation layer 3',
            0x1c : 'Modem type',
            0x1d : 'Multiframe',
            0x1e : 'Negotiation',
            0x1f : 'Negotiation indicator',
            0x20 : 'Network identification plan',
            0x21 : 'NIC on Rx',
            0x22 : 'NIC on Tx',
            0x23 : 'Notification description',
            0x24 : 'Number/map',
            0x25 : 'Number of data bits excluding parity Bit if present',
            0x26 : 'Number of stop bits',
            0x27 : 'Numbering plan',
            0x28 : 'Odd/even indicator',
            0x29 : 'Parity information',
            0x2a : 'Preferred/Exclusive',
            0x2b : 'Presentation indicator',
            0x2c : 'Presentation method of protocol profile',
            0x2d : 'Progress description',
            0x2e : 'Protocol discriminator',
            0x2f : 'Q.933 use',
            0x30 : 'Rate adaption header',
            0x31 : 'Screening indicator',
            0x32 : 'Signal value',
            0x33 : 'Synchronous/Asynchronous',
            0x34 : 'Transfer mode',
            0x35 : 'Type of network identification',
            0x36 : 'Type of number',
            0x37 : 'Type of subaddress',
            0x38 : 'User information layer 1 protocol',
            0x39 : 'User information layer 2 protocol',
            0x3a : 'User information layer 3 protocol',
            0x3b : 'User rate'
        },

        /**
        Atributos suportados pelos elementos
        Alguns podem ser reaproveitados entre os elementos

        @attribute attributes
         */
        attributes : {},

        /**
        Elementos suportados. Nesta é alimentada a lógica específica para cada tipo.
        Os itens estão separados em arquivos na pasta information_elements

        @attribute elementsDecoders
         */
        elementsDecoders : {}
    };

})(Khomp);


(function (Khomp) {

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;

    /**
    Attributos possíveis para os information elements

    @module isdn
    @submodule isdn-decoder
     */

    InformationElements.attributes = {
        /// Additional layer 3 protocol information
        0x00 : {
            0x0c : {
                0x0c : 'Internet Protocol (RFC 791) (ISO/IEC TR 9577 [82])',
                0x0f : 'Point-to-point Protocol (RFC 1548)'
            }
        },

        /// Assignor/assignee
        0x01 : {
            0x00 : 'Message originator is "Default assignee"',
            0x01 : 'Message originator is "Assignor only"'
        },

        /// Call state value
        0x02 : {
            ///    [USER] / [NETWORK]
            0x00 : 'U0 - Null / N0 - Null',
            0x01 : 'U1 - Call initiated / N1 - Call initiated',
            0x02 : 'U2 - Overlap sending / N2 - Overlap sending',
            0x03 : 'U3 - Outgoing call proceeding / N3 - Outgoing call proceeding',
            0x04 : 'U4 - Call delivered / N4 - Call delivered',
            0x06 : 'U6 - Call present / N6 - Call present',
            0x07 : 'U7 - Call received / N7 - Call received',
            0x08 : 'U8 - Connect request / N8 - Connect request',
            0x09 : 'U9 - Incoming call proceeding / N9 - Incoming call proceeding',
            0x0a : 'U10 - Active / N10 - Active',
            0x0b : 'U11 - Disconnect request / N11 - Disconnect request',
            0x0c : 'U12 - Disconnect indication / N12 - Disconnect indication',
            0x0f : 'U15 - Suspend request / N15 - Suspend request',
            0x11 : 'U17 - Resume request / N17 - Resume request',
            0x13 : 'U19 - Release request / N19 - Release request',
            0x16 : '------ / N22 - Call abort',
            0x19 : 'U25 - Overlap receiving / N25 - Overlap receiving'

        },

        /// Causes
        0x03 : {
            0x01 : 'Unassigned (unallocated) number',
            0x02 : 'No route to specified transit network',
            0x03 : 'No route to destination',
            0x06 : 'Channel unacceptable',
            0x07 : 'Call awarded and being delivered in an established channel',
            0x10 : 'Normal call clearing',
            0x11 : 'User busy',
            0x12 : 'No user responding',
            0x13 : 'User alerting, no answer',
            0x15 : 'Call rejected',
            0x16 : 'Number changed',
            0x1a : 'Non-selected user clearing',
            0x1b : 'Destination out of order',
            0x1c : 'Invalid number format (incomplete number)',
            0x3d : 'Facility rejected',
            0x1e : 'Response to STATUS ENQUIRY',
            0x1f : 'Normal, unspecified',
            0x22 : 'No circuit/channel available',
            0x26 : 'Network out of order',
            0x29 : 'Temporary failure',
            0x2a : 'Switching equipment congestion',
            0x2b : 'Access information discarded',
            0x2c : 'Requested circuit/channel not available',
            0x2f : 'Resource unavailable, unspecified',
            0x39 : 'Bearer capability not authorized',
            0x3a : 'Bearer capability not presently available',
            0x3f : 'Service or option not available, unspecified',
            0x41 : 'Bearer capability not implemented',
            0x42 : 'Channel type not implemented',
            0x45 : 'Requested facility not implemented',
            0x46 : 'Only restricted digital information bearer capability is available',
            0x4f : 'Service or option not implemented, unspecified',
            0x51 : 'Invalid call reference value',
            0x52 : 'Identified channel does not exist',
            0x53 : 'A suspended call exists, but this call identity does not',
            0x54 : 'Call identity in use',
            0x55 : 'No call suspended',
            0x56 : 'Call having the requested call identity has been cleared',
            0x58 : 'Incompatible destination',
            0x5b : 'Invalid transit network selection',
            0x5f : 'Invalid message, unspecified',
            0x60 : 'Mandatory information element is missing',
            0x61 : 'Message type non-existent or not implemented',
            0x62 : 'Message not compatible with call state or message type non-existent or not implemented',
            0x63 : 'Information element non-existent or not implemented',
            0x64 : 'Invalid information element contents',
            0x65 : 'Message not compatible with call state',
            0x66 : 'Recovery on time expire',
            0x6f : 'Protocol error, unspecified',
            0x7f : 'Interworking, unspecified'
        },

        /// Channel type/map element type
        0x04 : {
            0x03 : 'B-channel units (time slot number)',
            0x06 : 'H0-channel units',
            0x08 : 'H11-channel units',
            0x09 : 'H12-channel units'
        },

        /// Class
        0x05 : {
            0x00 : 'Indicated channels', // (Note 1)
            0x06 : 'Single interface', // (Note 2)
            0x07 : 'All interfaces' // (Note 3)

            /// Table 4-22/Q.931
            /// NOTE 1 – The channel identification information element must be included and indicates which channels are to be restarted.
            /// NOTE 2 – If non-associated signalling is used, the channel identification information element must be included to indicate the interface to be restarted if it is other than the one on which the D-channel is present.
            /// NOTE 3 – May be used when there are two or more interfaces controlled by the D-channel. The channel identification information element must not be included with this coding
        },

        /// Coding standard
        0x06 : {
            0x00 : 'ITU-T standardized coding', //00
            0x01 : 'ISO/IEC Standard', // NOTE 1 // 01
            0x02 : 'National standard', // NOTE 1 // 10
            0x03 : 'Standard defined for the network (either public or private) present on the network side of the interface' // NOTE 1 // 11

            /// TODO ver se esta opção é a mesma: '11' : 'Standard specific to identified location (Note 1)'

            /// NOTE 1 – These other coding standards should only be used only when the desired (bearer capability / call states /high layer compatibility / low layer compatibility / progress indication) cannot be represented by ITU-T-standardized coding
        },

        /// D-channel indicator
        0x07 : {
            0x00 : 'False', //'The channel identified is not the D-channel',
            0x01 : 'True' //'The channel identified is the D-channel'
        },

        /// Default packet size
        0x08 : {
            0x04 : 'Default packet size 16 octets',
            0x05 : 'Default packet size 32 octets',
            0x06 : 'Default packet size 64 octets',
            0x07 : 'Default packet size 128 octets',
            0x08 : 'Default packet size 256 octets',
            0x09 : 'Default packet size 512 octets',
            0x0a : 'Default packet size 1024 octets',
            0x0b : 'Default packet size 2048 octets',
            0x0c : 'Default packet size 4096 octets'
        },

        /// Extended audiovisual characteristics identification
        0x09 : {
            0x01 : 'Capability set of initial channel of H.221',
            0x02 : 'Capability set of subsequent channel of H.221',
            0x21 : 'Capability set of initial channel associated with an active 3.1 kHz audio or speech call.'
        },

        /// Extended high layer characteristics identification
        0x0a : {
            0x01 : 'Telephony',
            0x04 : 'Facsimile Group 2/3 (Recommendation F.182)',
            0x21 : 'Facsimile Group 4 Class I (Recommendation F.184)',
            0x24 : 'Facsimile service Group 4, Classes II and III (Recommendation F.184)',
            0x28 : '(Note 11)',
            0x31 : '(Note 11)',
            0x32 : 'Syntax based Videotex (Recommendations F.300 and T.102)',
            0x33 : 'International Videotex interworking via gateways or interworking units (Recommendations F.300 and T.101)',
            0x35 : 'Telex service (Recommendation F.60)',
            0x38 : 'Message Handling Systems (MHS) (X.400-series Recommendations [77])',
            0x41 : 'OSI application (Note 6) (X.200-series Recommendations [78])',
            0x42 : 'FTAM application (ISO 8571)',
            0x5e : 'Not available for assignment',
            0x5f : 'Not available for assignment',
            0x60 : 'Videotelephony (Recommendations F.720 [91] and F.721 [79] and F.731, profile 1a)',
            0x61 : 'Videoconferencing Recommendations F.702 [94] and F.731 [97], profile 1b',
            0x62 : 'Audiographic conferencing Recommendations F.702 [94] and F.731 [97] (including at least profile 2a2 and optionally 2a1, 2a3, 2b1, 2b2 and 2bc)',

            /// ---------------
            0x63 : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x64 : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x65 : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x66 : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x67 : 'Reserved for audiovisual services (F.700-series Recommendations [80])',

            /// ---------------
            0x68 : 'Multimedia services (F.700-series Recommendations [80])',

            /// ---------------
            0x69 : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x6a : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x6b : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x6c : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x6d : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x6e : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x6f : 'Reserved for audiovisual services (F.700-series Recommendations [80])',

            /// ---------------
            0x7f : 'Reserved'
        },

        /// Flow control on Rx
        0x0b : {
            0x00 : 'Cannot accept data with flow control mechanism',
            0x01 : 'Can accept data with flow control mechanism'
        },

        /// Flow control on Tx
        0x0c : {
            0x00 : 'Not required to send data with flow control mechanism',
            0x01 : 'Required to send data with flow control mechanism'
        },

        /// High layer characteristics identification
        0x0d : {
            0x01 : 'Telephony',
            0x04 : 'Facsimile Group 2/3 (Recommendation F.182 [68])',
            0x21 : 'Facsimile Group 4 Class I (Recommendation F.184 [69])',
            0x24 : 'Facsimile service Group 4, Classes II ad III (Recommendation F.184)',
            0x28 : 'Only to those services for which ITU-T (Note 7)',
            0x31 : 'Only to those services for which ITU-T (Note 7)',
            0x32 : 'Syntax based Videotex (Recommendation F.300 [73] and T.102 [74])',
            0x33 : 'International Videotex interworking via gateways or interworking units (Recommendation F.300 and T.101 [75])',
            0x35 : 'Telex service (Recommendation F.60 [76])',
            0x38 : 'Message Handling Systems (MHS) (X.400-series Recommendation [77])',
            0x41 : 'OSI application (Note 6) (X.200-series Recommendations [78])',
            0x42 : 'FTAM application (ISO 8571)',
            0x5e : 'Reserved for maintenance (Note 8)',
            0x5f : 'Reserved for management (Note 8)',
            0x60 : 'Videotelephony (Recommendations F.720 [91] and F.721 [79]) and F.731 profile 1a)(Note 9)',
            0x61 : 'Videoconferencing Recommendation F.702 [94] and F.731 [97] Profile 1b (Note 9)',
            0x62 : 'Audiographic conferencing Recommendations F.702 [94] and F.731 [97] (including at least profile 2a2 and optionally 2a1, 2a3, 2b1, 2b2, and 2bc) (Notes 9 and 10)',
            ///-----
            0x63 : 'Reserved for audiovisual service (F.700-series Recommendations [80])',
            0x64 : 'Reserved for audiovisual service (F.700-series Recommendations [80])',
            0x65 : 'Reserved for audiovisual service (F.700-series Recommendations [80])',
            0x66 : 'Reserved for audiovisual service (F.700-series Recommendations [80])',
            0x67 : 'Reserved for audiovisual service (F.700-series Recommendations [80])',
            ///-----
            0x68 : 'Multimedia services F.700-series Recommendations [80] (Note 9)',
            ///-----
            0x7f : 'Reserved'
        },

        /// In-band/out-band negotiation
        0x0e : {
            0x00 : 'Negotiation is done with USER INFORMATION messages on a temporary signalling connection',
            0x01 : 'Negotiation is done in-band using logical link zero'
        },

        /// Information channel selection
        0x0f : {
            /// Basic interface
            0x00 : {
                0x00 : 'No channel',
                0x01 : 'B1 channel',
                0x02 : 'B2 channel',
                0x03 : 'Any channel'
            },
            /// Other interfaces
            0x01 : {
                0x00 : 'No channel',
                0x01 : 'As indicated in following octets',
                0x02 : 'Reserved',
                0x03 : 'Any channel'
            }
        },

        /// Information transfer capability
        0x10 : {
            0x00 : 'Speech',
            0x08 : 'Unrestricted digital information',
            0x09 : 'Restricted digital information',
            0x10 : '3.1 kHz audio',
            0x11 : 'Unrestricted digital information with tones/announcements', // NOTE 2
            0x18 : 'Video'

            /// NOTE 2 – Unrestricted digital information with tones/announcements (UDI-TA) is the new informationtransfer attribute value that had previously been named "7 kHz audio" in Recommendation Q.931 (1988).
        },

        /// Information transfer rate
        0x11 : {
            /// A escolha do Circuit ou Packet é feita com base no valor do 'Transfer mode'
            /// Circuit
            0x00 : {
                0x00 : '–',
                0x10 : '64 kbit/s',
                0x11 : '2x64 kbit/s',
                0x13 : '384 kbit/s',
                0x15 : '1536 kbit/s',
                0x17 : '1920 kbit/s',
                0x18 : 'Multirate (64 kbit/s base rate)'
            },
            /// Packet
            0x02 : {
                0x00 : 'This code shall be used for all packet calls',
                0x10 : '–',
                0x11 : '–',
                0x13 : '–',
                0x15 : '–',
                0x17 : '–',
                0x18 : 'Multirate (64 kbit/s base rate)'
            }
        },

        /// Interface identifier present
        0x12 : {
            0x00 : 'False - implicitly identified ', //'(Note 1)'
            0x01 : 'True - explicitly identified in one or more octets, beginning with octet 3.1'
        },

        /// Interface type
        0x13 : {
            0x00 : 'Basic interface',
            0x01 : 'Primary rate interface'
        },

        /// Intermediate rate
        0x14 : {
            0x00 : 'Not used',
            0x01 : '8 kbit/s',
            0x02 : '16 kbit/s',
            0x03 : '32 kbit/s'
        },

        /// Interpretation
        /// Atualmente esta informação possui apenas uma opção de valor que pode ser expandida futuramente pela norma
        0x15 : {
            0x04 : 'First (primary or only) high layer characteristics identification (in octet 4) to be used in the call'
        },

        /// Location
        0x16 : {
            0x00 : 'User',
            0x01 : 'Private network serving the local user',
            0x02 : 'Public network serving the local user',
            0x03 : 'Transit network (Note 2)',
            0x04 : 'Public network serving the remote user',
            0x05 : 'Private network serving the remote user',
            0x0a : 'Network beyond the interworking point'
        },

        /// Logical link identifier negotiation
        0x17 : {
            0x00 : 'Default, LLI = 256 only',
            0x01 : 'Full protocol negotiation' /// NOTE 18 – A connection over which protocol negotiation will be executed is indicated in bit 2 of octet 5b
        },

        /// Mode duplex
        0x18 : {
            0x00 : 'Half duplex',
            0x01 : 'Full duplex'
        },

        /// Mode of operation
        0x19 : {
            0x00 : 'Bit transparent mode of operation',
            0x01 : 'Protocol sensitive mode of operation'
        },

        /// Mode of operation layer 2
        0x1a : {
            0x01 : 'Normal mode of operation',
            0x02 : 'Extended mode of operation'
        },

        /// Mode of operation layer 3
        0x1b : {
            0x01 : 'Normal packet sequence numbering',
            0x02 : 'Extended packet sequence numbering'
        },

        /// Modem type
        0x1c : {
            0x00 : 'National use',
            0x01 : 'National use',
            0x02 : 'National use',
            0x03 : 'National use',
            0x04 : 'National use',
            0x05 : 'National use',
            0x11 : 'Recommendation V.21 [55]',
            0x12 : 'Recommendation V.22 [56]',
            0x13 : 'Recommendation V.22 bis[57]',
            0x14 : 'Recommendation V.23 [58]',
            0x15 : 'Recommendation V.26 [59]',
            0x16 : 'Recommendation V.26 bis[60]',
            0x17 : 'Recommendation V.26 ter[61]',
            0x18 : 'Recommendation V.27 [62]',
            0x19 : 'Recommendation V.27 bis[63]',
            0x1a : 'Recommendation V.27 ter [64]',
            0x1b : 'Recommendation V.29 [65]',
            0x1d : 'Recommendation V.32 [66]',
            0x1e : 'Recommendation V.34 [90]',
            ///-------------------
            0x20 : 'National use',
            0x21 : 'National use',
            0x22 : 'National use',
            0x23 : 'National use',
            0x24 : 'National use',
            0x25 : 'National use',
            0x26 : 'National use',
            0x27 : 'National use',
            0x28 : 'National use',
            0x29 : 'National use',
            0x2a : 'National use',
            0x2b : 'National use',
            0x2c : 'National use',
            0x2d : 'National use',
            0x2e : 'National use',
            0x2f : 'National use',

            ///-------------------
            0x30 : 'User specified',
            0x31 : 'User specified',
            0x32 : 'User specified',
            0x33 : 'User specified',
            0x34 : 'User specified',
            0x35 : 'User specified',
            0x36 : 'User specified',
            0x37 : 'User specified',
            0x38 : 'User specified',
            0x39 : 'User specified',
            0x3a : 'User specified',
            0x3b : 'User specified',
            0x3c : 'User specified',
            0x3d : 'User specified',
            0x3e : 'User specified',
            0x3f : 'User specified'
        },

        /// Multiframe
        0x1d : {
            0x00 : 'Multiple frame establishment not supported. Only UI frames allowed',
            0x01 : 'Multiple frame establishment supported'
        },

        /// Negotiation
        0x1e : {
            0x00 : 'In-band negotiation not possible',
            0x01 : 'In-band negotiation possible'
        },

        /// Negotiation indicator
        0x1f : {
            0x00 : 'Out-band negotiation not possible',
            0x01 : 'Out-band negotiation possible'

            /// NOTE: See Annex J for description of low layer compatibility negotiation.
        },

        /// Network identification plan
        0x20 : {

            0x00 : 'Unknown',
            0x01 : 'Carrier Identification Code', // (Note 2)
            0x03 : 'Data network identification code (Recommendation X.121 [21])'

            /// Table 4-18/Q.931 NOTE 2 – Carrier Identification Codes may be an appropriate method of identifying the network serving the remote user.
        },

        /// NIC on Rx
        0x21 : {

            0x00 : 'Cannot accept data with network independent clock',
            0x01 : 'Can accept data with network independent clock'
        },

        /// NIC on Tx
        0x22 : {

            0x00 : 'Not required to send data with network independent clock',
            0x01 : 'Required to send data with network independent clock'
        },

        /// Notification description
        0x23 : {

            0x00 : 'User suspended',
            0x01 : 'User resumed',
            0x02 : 'Bearer service change',

            /// Os abaixo foram extraídos de:
            /// http://www.etsi.org/deliver/etsi_i_ets/300100_300199/30014101/01_60/ets_30014101e01p.pdf
            /// No wireshark são apresentadas os mesmos valores, que não constam na ITU Q.931
            0x79 : 'Remote hold',
            0x7a : 'Remote retrieval'
        },

        /// Number/map
        0x24 : {
            0x00 : 'Channel is indicated by the number',
            0x01 : 'Channel is indicated by the slot map'
        },

        /// Number of data bits excluding parity Bit if present
        0x25 : {
            0x00 : 'Not used',
            0x01 : '5 bits',
            0x02 : '7 bits',
            0x03 : '8 bits'
        },

        /// Number of stop bits
        0x26 : {
            0x00 : 'Not used',
            0x01 : '1 bit',
            0x02 : '1.5 bits',
            0x03 : '2 bits'
        },

        /// Numbering plan
        0x27 : {
            0x00 : 'Unknown',
            0x01 : 'ISDN/telephony numbering plan',
            0x03 : 'Data numbering plan',
            0x04 : 'Telex numbering plan',
            0x08 : 'National standard numbering plan',
            0x09 : 'Private numbering plan',
            0x0f : 'Reserved for extension'
        },

        /// Odd/even indicator
        0x28 : {
            0x00 : 'Even number of address signals',
            0x01 : 'Odd number of address signals'
        },

        /// Parity information
        0x29 : {
            0x00 : 'Odd',
            0x01 : 'Even',
            0x02 : 'None',
            0x03 : 'Forced to 0',
            0x04 : 'Forced to 1'
        },

        /// Preferred/Exclusive
        0x2a : {

            0x00 : 'Indicated channel is preferred',
            0x01 : 'Exclusive; only the indicated channel is acceptable'
        },

        /// Presentation indicator
        0x2b : {

            0x00 : 'Presentation allowed',
            0x01 : 'Presentation restricted',
            0x02 : 'Number not available due to interworking',
            0x03 : 'Reserved'
        },

        /// Presentation method of protocol profile
        0x2c : {
            0x01 : 'High layer protocol profile'
        },

        /// Progress description
        0x2d : {

            0x01 : '1. Call is not end-to-end ISDN; further call progress information may be available in-band',
            0x02 : '2. Destination address is non-ISDN',
            0x03 : '3. Origination address is non-ISDN',
            0x04 : '4. Call has returned to the ISDN',
            0x05 : '5. Interworking has occurred and has resulted in a telecommunication service change (Note 5)',
            0x08 : '8. In-band information or an appropriate pattern is now available'
        },

        /// Protocol discriminator
        0x2e : {

            0x00 : 'User-specific protocol', // (Note 1)
            0x01 : 'OSI high layer protocols',
            0x02 : 'Recommendation X.244 [44]', // (Note 2)
            0x03 : 'Reserved for system management convergence function',
            0x04 : 'IA5 characters', // (Note 4)
            0x05 : 'X.208 and X.209 coded user information', // (Note 5)
            0x07 : 'Recommendation V.120 [9] rate adaption',
            0x08 : 'Q.931/I.451 user-network call control messages',
            /// -------
            0x10 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x11 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x12 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x13 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x14 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x15 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x16 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x17 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x18 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x19 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x1a : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x1b : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x1c : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x1d : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x1e : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x1f : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x20 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x21 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x22 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x23 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x24 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x25 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x26 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x27 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x28 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x29 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x2a : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x2b : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x2c : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x2d : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x2e : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x2f : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x30 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x31 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x32 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x33 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x34 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x35 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x36 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x37 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x38 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x39 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x3a : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x3b : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x3c : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x3d : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x3e : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x3f : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            /// -------
            0x40 : 'National use',
            0x41 : 'National use',
            0x42 : 'National use',
            0x43 : 'National use',
            0x44 : 'National use',
            0x45 : 'National use',
            0x46 : 'National use',
            0x47 : 'National use',
            0x48 : 'National use',
            0x49 : 'National use',
            0x4a : 'National use',
            0x4b : 'National use',
            0x4c : 'National use',
            0x4d : 'National use',
            0x4e : 'National use',
            0x4f : 'National use',

            /// -------
            0x50 : 'Reserved for other network layer or layer 3 protocols, including Recommendation through X.25', // (Note 3)
            // through
            /// TODO: ver como fazer aqui nesta faixa. É muito grande.
            0xfe : 'Reserved for other network layer or layer 3 protocols, including Recommendation through X.25' // (Note 3)

            /// Table 4-26/Q.931
            /// NOTE 1 – The user information is structured according to user needs.
            /// NOTE 2 – The user information is structured according to Recommendation X.244 which specifies the structure of X.25 call user data.
            /// NOTE 3 – These values are reserved to discriminate these protocol discriminators from the first octet of an X.25 packet including general format identifier.
            /// NOTE 4 – The user information consists of IA5 characters.
            /// NOTE 5 – The number of X.208 and X.209 components contained in a User-user information element as well as their semantics and use are user-application dependent and may be subject to other Recommendations.
        },

        /// Q.933 use
        0x2f : {
            0x00 : 'For use when the coding defined in Recommendation Q.933 is not use'
        },

        /// Rate adaption header
        0x30 : {
            0x00 : 'Header not included',
            0x01 : 'Header included'
        },

        /// Screening indicator
        0x31 : {
            0x00 : 'User-provided, not screened',
            0x01 : 'User-provided, verified and passed',
            0x02 : 'User-provided, verified and failed',
            0x03 : 'Network provided'
        },

        /// Signal value
        0x32 : {

            0x00 : 'Dial tone on',
            0x01 : 'Ring back tone on',
            0x02 : 'Intercept tone on',
            0x03 : 'Network congestion tone on',
            0x04 : 'Busy tone on',
            0x05 : 'Confirm tone on',
            0x06 : 'Answer tone on',
            0x07 : 'Call waiting tone',
            0x08 : 'Off-hook warning tone',
            0x09 : 'Pre-emption tone on',
            0x3f : 'Tones off',
            0x40 : 'Alerting on – pattern 0', //  (Note 1)
            0x41 : 'Alerting on – pattern 1', // (Note 1)
            0x42 : 'Alerting on – pattern 2', // (Note 2)
            0x43 : 'Alerting on – pattern 3', // (Note 1)
            0x44 : 'Alerting on – pattern 4', // (Note 1)
            0x45 : 'Alerting on – pattern 5', // (Note 1)
            0x46 : 'Alerting on – pattern 6', // (Note 1)
            0x47 : 'Alerting on – pattern 7', // (Note 1)
            0x4f : 'Alerting off'

            /// Table 4-24/Q.931
            /// NOTE 1 – The use of these patterns is network dependent.
            /// NOTE 2 – Used for special/priority alerting.
        },

        /// Synchronous/Asynchronous
        0x33 : {
            0x00 : 'Synchronous data',
            0x01 : 'Asynchronous data'
        },

        /// Transfer mode
        0x34 : {

            0x00 : 'Circuit',
            0x02 : 'Packet'

        },

        /// Type of network identification
        0x35 : {

            0x00 : 'User specified',
            0x02 : 'National network identification', // (Note 1)
            0x03 : 'International network identification'

            /// NOTE 1 – In the case that "type of network identification" is coded as 010, "national network identification ", " national identification plan " is coded according to national specification.
        },

        /// Type of number
        0x36 : {
            0x00 : 'Unknown',
            0x01 : 'International number',
            0x02 : 'National number',
            0x03 : 'Network specific number',
            0x04 : 'Subscriber number',
            0x06 : 'Abbreviated number',
            0x07 : 'Reserved for extension'
        },

        /// Type of subaddress
        0x37 : {
            0x00 : 'NSAP',
            0x02 : 'User specified'
        },

        /// User information layer 1 protocol
        0x38 : {
            0x01 : 'ITU-T standardized rate adaption V.110, I.460 and X.30. This implies the presence of octet 5a and optionally octets 5b, 5c and 5d as defined below',
            0x02 : 'Recommendation G.711 [10] µ-law',
            0x03 : 'Recommendation G.711 A-law',
            0x04 : 'Recommendation G.721 [11] 32 kbit/s ADPCM and Recommendation I.460',
            0x05 : 'Recommendations H.221 and H.242',
            0x06 : 'Recommendations H.223 [92] and H.245 [93]',
            0x07 : 'Non-ITU-T standardized rate adaption. This implies the presence of octet 5a and, optionally, octets 5b, 5c and 5d. The use of this codepoint indicates that the user rate specified in octet 5a is defined by the user. Additionally, octets 5b, 5c and 5d, if present, are defined in accordance with the user specified rate adaption',
            0x08 : 'ITU-T standardized rate adaption V.120 [9]. This implies the presence of octets 5a and 5b as defined below, and optionally octets 5c and 5d',
            0x09 : 'ITU-T standardized rate adaption X.31 [14] HDLC flag stuffing'
        },

        /// User information layer 2 protocol
        0x39 : {
            0x01 : 'Basic mode ISO 1745 [36]',
            0x02 : 'Recommendation Q.921/I.441 [3]',
            0x06 : 'Recommendation X.25 [5], link layer (Notes 20, 23)',
            0x07 : 'Recommendation X.25 Multilink',
            0x08 : 'Extended LAPB; for half duplex operation (Recommendation T.71 [37])',
            0x09 : 'HDLC ARM (ISO/IEC 4335 [38])',
            0x0a : 'HDLC NRM (ISO/IEC 4335)',
            0x0b : 'HDLC ABM (ISO/IEC 4335)',
            0x0c : 'LAN logical link control (ISO/IEC 8802-2 [39])',
            0x0d : 'Recommendation X.75 [40]. Single Link Procedure (SLP)',
            0x0e : 'Recommendation Q.922',
            0x0f : 'Core aspects of Recommendation Q.922',
            0x10 : 'User specified',
            0x11 : 'ISO/IEC 7776 DTE-DCE operation'
        },

        /// User information layer 3 protocol
        0x3a : {

            0x02 : 'Recommendation Q.931/I.451',
            0x06 : 'Recommendation X.25, packet layer',
            0x07 : 'ISO/IEC 8208 [41] (X.25 packet level protocol for data terminal equipment)',
            0x08 : 'ITU-T Rec. X.223 and ISO/IEC 8878 [81] (use of ISO/IEC 8208 [41] and Recommendation X.25 to provide the OSI-CONS)',
            0x09 : 'ISO/IEC 8473 [43] (OSI connectionless mode protocol)',
            0x0a : 'Recommendation T.70[32]minimum network layer ',
            0x0b : 'ISO/IEC TR 9577[82](Protocol identification in the network layer)',
            0x10 : 'User specified'
        },

        /// User rate
        0x3b : {
            0x00 : 'For I.460, rate is specified by bits 7, 6 of octet 5b, intermediate rate. For V.110 and X.30, rate is indicated by E-bits (synchronous data only) or may be negotiated in-band. For V.120, rate is unspecified or may be negotiated in-band.',
            0x01 : '0.6 kbit/s Recommendation X.1 [17]',
            0x02 : '1.2 kbit/s',
            0x03 : '2.4 kbit/s Recommendation X.1',
            0x04 : '3.6 kbit/s',
            0x05 : '4.8 kbit/s Recommendation X.1',
            0x06 : '7.2 kbit/s',
            0x07 : '8 kbit/s Recommendation I.460',
            0x08 : '9.6 kbit/s Recommendation X.1',
            0x09 : '14.4 kbit/s',
            0x0a : '16 kbit/s Recommendation I.460',
            0x0b : '19.2 kbit/s',
            0x0c : '32 kbit/s Recommendation I.460',
            0x0d : '38.4 kbit/s Recommendation V.110 [87]',
            0x0e : '48 kbit/s Recommendations X.1',
            0x0f : '56 kbit/s',
            0x12 : '57.6 kbit/s Recommendation V.14 extended [88]',
            0x13 : '28.8 kbit/s Recommendation V.110 [89]',
            0x14 : '24 kbit/s Recommendation V.110 [89]',
            0x15 : '0.1345 kbit/s Recommendation X.1',
            0x16 : '0.100 kbit/s Recommendation X.1',
            0x17 : '0.075/1.2 kbit/s Recommendation X.1 (Note 9)',
            0x18 : '1.2/0.075 kbit/s Recommendation X.1 (Note 9)',
            0x19 : '0.050 kbit/s Recommendation X.1',
            0x1a : '0.075 kbit/s Recommendation X.1',
            0x1b : '0.110 kbit/s Recommendation X.1',
            0x1c : '0.150 kbit/s Recommendation X.1',
            0x1d : '0.200 kbit/s Recommendation X.1',
            0x1e : '0.300 kbit/s Recommendation X.1',
            0x1f : '12 kbit/s'
        }

    };

})(Khomp);


(function (Khomp) {

    /// Bearer capability
    /// A partir da página 58 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;

    InformationElements.elementsDecoders[0x04] = function (octets) {

        /// Objeto com a informação decodificada que será retornado.
        var objValue = {};

        ///----------------------------------------
        /// Octet 3
        var octet3 = octets.shift();

        /// Coding standard: bits 7,6 octet 3
        var codingStandard = (octet3 & 0x60) >> 5;
        objValue[InformationElements.attributeNames[0x06]] = attributes[0x06][codingStandard] || codingStandard;

        /// Information transfer capability: bits 5,4,3,2,1 octet 3
        var informationTransferCapability = (octet3 & 0x1f);
        objValue['Information transfer capability'] = attributes[0x10][informationTransferCapability] || informationTransferCapability;

        ///----------------------------------------
        /// Octet 4
        var octet4 = octets.shift();

        /// Transfer mode: bits 7,6 octet 4
        var transferMode = (octet4 & 0x60) >> 5;
        objValue['Transfer mode'] = attributes[0x34][transferMode] || transferMode;

        /// Information transfer rate: análise dos bits 5,4,3,2,1 octet 4
        var informationTransferRate = (octet4 & 0x1f);
        objValue['Information transfer rate'] = attributes[0x11][transferMode][informationTransferRate] || informationTransferRate;

        ///----------------------------------------
        /// Octet 4.1*
        
        /// Rate multiplier
        /// NOTE 1 - Estará presente somente se o Information transfer rate tiver como valor 'Multirate'
        if (informationTransferRate === 0x18) {

            var octet41 = octets.shift();

            objValue['Rate multiplier'] = octet41;
        }
        
        /// Daqui em diante os octetos são opcionais. Então se não tem mais nada...
        if (octets.length === 0) {return objValue;}

        ///----------------------------------------
        /// Octet 5*
        var octet5 = octets.shift();

        /// Layer identifier: bits 7,6 octet 5
        objValue['Layer 1 identifier'] = (octet5 & 0x60) >> 5;

        /// User information layer 1 protocol: bits 5,4,3,2,1 octet 5
        var userInfoLayer1Protocol = (octet5 & 0x1f);
        
        objValue['User information layer 1 protocol'] = attributes[0x38][userInfoLayer1Protocol] || userInfoLayer1Protocol;

        ///----------------------------------------
        /// Octet 5a*
        /// NOTE 2 – This octet may be present if octet 3 indicates unrestricted digital information and octet 5 indicates either of the ITU-T standardized rate adaptions V.110, I.460 and X.30 or V.120 [9]. It may also be present if octet 3 indicates 3.1 kHz audio and octet 5 indicates G.711.
        var conditionNote2 = (
            (informationTransferCapability === 0x08 && (userInfoLayer1Protocol === 0x01 || userInfoLayer1Protocol === 0x08)) ||
            (informationTransferCapability === 0x10 && (userInfoLayer1Protocol === 0x02 || userInfoLayer1Protocol === 0x03))) ? true : false;

        var extensionIndicatorOct5 = (((octet5 & 0x80) >> 7) === 0) ? true : false;

        if (!extensionIndicatorOct5) {
            return objValue;
        }

        ///----------------------------------------
        var octet5a = octets.shift();

        /// Synchronous/Asynchronous: bit 7 octet 5a
        /// NOTE 7 – Octets 5b-5d may be omitted in the case of synchronous user rates.
        var syncData = (octet5a & 0x40) >> 6;
        objValue['Synchronous/Asynchronous'] = attributes[0x33][syncData] || syncData;

        /// Negotiation: bit 6 octet 5a
        var negotiation = (octet5a & 0x20) >> 5;
        objValue.Negotiation = attributes[0x1e][negotiation] || negotiation;

        /// User rate: bits 5,4,3,2,1 octet 5a
        var userRate = (octet5a & 0x1f);
        objValue['User rate'] = attributes[0x3b][userRate] || userRate;

        ///----------------------------------------
        /// Octet 5b*
        /// NOTE 3 – This structure of octet 5b only applies if octet 5 indicates ITU-T standardized rate adaption (see Recommendations V.110 [7], I.460 [15] and X.30 [8])
        if (octets.length && userInfoLayer1Protocol === 0x01) {

            var octet5b = octets.shift();

            /// Intermediate rate: bits 7, 6 do octeto 5b
            var intermediateRate = (octet5b & 0x60) >> 5;
            objValue['Intermediate rate'] = attributes[0x14][intermediateRate] || intermediateRate;

            /// NIC on Tx: bit 5 do octet 5b
            /// NOTE 10 – Refers to transmission in the forward direction of the call.
            /// NOTE 11 – See Recommendations V.110 [7], I.460 [15] and X.30 [8].
            var nicOnTx = (octet5b & 0x10) >> 4;
            objValue['NIC on Tx'] = attributes[0x22][nicOnTx] || nicOnTx;

            /// NIC on Rx: bit 4 do octet 5b
            /// NOTE 12 – Refers to transmission in the backward direction of the call.
            /// NOTE 13 – See Recommendations V.110 [7], I.460 [15] and X.30 [8].
            var nicOnRx = (octet5b & 0x08) >> 3;
            objValue['NIC on Rx'] = attributes[0x21][nicOnRx] || nicOnRx;

            /// Flow control on Tx: bit 3 do octet 5b
            /// NOTE 14 – Refers to transmission in the forward direction of the call.
            /// NOTE 15 – See Recommendations V.110, I.460 and X.30.
            var flowControlOnTx = (octet5b & 0x04) >> 2;
            objValue['Flow control on Tx'] = attributes[0x0c][flowControlOnTx] || flowControlOnTx;

            /// Flow control on Rx: bit 2 do octet 5b
            /// NOTE 16 – Refers to transmission in the backward direction of the call.
            /// NOTE 17 – See Recommendations V.110, I.460 and X.30
            var flowControlOnRx = (octet5b & 0x02) >> 1;
            objValue['Flow control on Rx'] = attributes[0x0b][flowControlOnRx] || flowControlOnRx;

            /// NOTE 4 – This structure of octet 5b only applies if octet 5 indicates ITU-T standardized rate adaption (see Recommendation V.120 [9]).
            if (!InformationElements.isLastOctet(octet5b) && userInfoLayer1Protocol === 0x08) {

                ///----------------------------------------
                /// Octet 5ba
                var octet5ba = octets.shift();

                /// Rate adaption header: bit 7 do octet 5b*
                var rateAdaptionHeader = (octet5ba & 0x40) >> 6;
                objValue['Rate adaption header'] = attributes[0x30][rateAdaptionHeader] || rateAdaptionHeader;

                /// Multiframe: bit 6 do octet 5b*
                var multiframe = (octet5ba & 0x20) >> 5;
                objValue.Multiframe = attributes[0x1d][multiframe] || multiframe;

                /// Mode of operation: bit 5 do octet 5b*
                var modeOfOperation = (octet5ba & 0x10) >> 4;
                objValue['Mode of operation'] = attributes[0x19][modeOfOperation] || modeOfOperation;

                /// Logical link identifier negotiation: bit 4 octet 5b*
                var logicalLinkIdentifierNegotiation = (octet5ba & 0x08) >> 3;
                objValue['Logical link identifier negotiation'] = attributes[0x17][logicalLinkIdentifierNegotiation] || logicalLinkIdentifierNegotiation;

                /// Assignor/assignee: bit 3 do octet 5b*
                var assignorAssignee = (octet5ba & 0x04) >> 2;
                objValue[InformationElements.attributeNames[0x01]] = attributes[0x01][assignorAssignee] || assignorAssignee;

                /// In-band/out-band negotiation: bit 2 do octet 5b*
                var inBandOutBandNegotiation = (octet5ba & 0x02) >> 1;
                objValue['In-band/out-band negotiation'] = attributes[0x0e][inBandOutBandNegotiation] || inBandOutBandNegotiation;

                ///----------------------------------------
                /// Octet 5c*
                /// NOTE 2
                if (octets.length && !InformationElements.isLastOctet(octet5ba) && conditionNote2) {

                    var octet5c = octets.shift();

                    /// Number of stop bits: bits 7,6 do octet 5c*
                    var numberOfStopBits = (octet5c & 0x60) >> 5;
                    objValue['Number of stop bits'] = attributes[0x26][numberOfStopBits] || numberOfStopBits;

                    /// Number of data bits excluding parity Bit if present: bits 5,4 do octet 5c*
                    var numberOfDataBitsExcludingParityBitIfPresent = (octet5c & 0x18) >> 3;
                    objValue['Number of data bits excluding parity Bit if present'] = attributes[0x25][numberOfDataBitsExcludingParityBitIfPresent] || numberOfDataBitsExcludingParityBitIfPresent;

                    /// Parity information: bits 3,2,1 do octet 5c*
                    var parityInformation = (octet5c & 0x07);
                    objValue['Parity information'] = attributes[0x29][parityInformation] || parityInformation;

                    ///----------------------------------------
                    /// Octet 5d*
                    /// NOTE 2
                    if (octets.length && !InformationElements.isLastOctet(octet5c) && conditionNote2) {

                        var octet5d = octets.shift();

                        /// Mode duplex: bit 7 do octet 5d*
                        var modeDuplex = (octet5d & 0x40) >> 6;
                        objValue['Mode duplex'] = attributes[0x18][modeDuplex] || modeDuplex;

                        /// Modem type: bits 6,5,4,3,2,1 octet 5d*
                        var modemType = (octet5d & 0x3f);
                        objValue['Modem type'] = attributes[0x1c][modemType] || modemType;

                    }
                }
            }

            ///----------------------------------------
            /// Octet 6*
            if (octets.length) {

                var octet6 = octets.shift();

                var layer2Ident = (octet6 & 0x60) >> 5;
                objValue['Layer 2 identifier'] = layer2Ident;

                /// User information layer 2 protocol: bits 5,4,3,2,1 octet 6
                var userInfoLayer2Protocol = (octet6 & 0x1f);
                objValue['User information layer 2 protocol'] = attributes[0x39][userInfoLayer2Protocol] || userInfoLayer2Protocol;

            }

            ///----------------------------------------
            /// Octet 7*
            if (octets.length) {
                var octet7 = octets.shift();

                var layer3Ident = (octet7 & 0x60) >> 5;
                objValue['Layer 3 identifier'] = layer3Ident;

                /// User information layer 3 protocol: bits 5,4,3,2,1 octeto 7
                var userInfoLayer3Protocol = (octet7 & 0x1f);
                objValue['User information layer 3 protocol'] = attributes[0x3a][userInfoLayer3Protocol] || userInfoLayer3Protocol;

                ///----------------------------------------
                /// Octet 7a* e 7b*
                /// NOTE 5: This octet may be included if octet 7 indicates ISO/IEC TR 9577 (Protocol Identification in the network layer).
                var note5Condition = (userInfoLayer3Protocol === 0x0b) ? true : false;

                if (octets.length && note5Condition) {

                    var octet7a = octets.shift();
                    var octet7b = octets.shift();

                    /// Additional layer 3 protocol information: bits 4,3,2,1 octetos 7a* e 7b*
                    var bits7a = (octet7a & 0xf);
                    var bits7b = (octet7b & 0xf);

                    objValue[InformationElements.attributeNames[0x00]] = attributes[0x00][bits7a][bits7b] || (bits7a + ' ' + bits7b);

                }

            }
        }

        return objValue;
    };

})(Khomp);


(function (Khomp) {

    /// Call identity
    /// Página 66 ITU Q.931

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var Util = Khomp.Util;

    InformationElements.elementsDecoders[0x10] = function (values) {

        var objValue = {
            'Call identity' : Util.translateIA5(values)
        };

        return objValue;
    };
})(Khomp);


(function (Khomp) {

    /// Call state
    /// Página 66 ITU Q.931

    /// TODO: ver como pegar/apresentar o global value. Não está claro (nota ou algo assim) como extrair esta informação no mesmo octeto do call state value.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;

    InformationElements.elementsDecoders[0x14] = function (values) {

        var objValue = {};

        /// Octet 3
        var octet3 = values.shift();

        var codingStandard = (octet3 & 0xc0) >> 6;
        var callStateValue = (octet3 & 0x3f);

        /// Octet 3 bits 8,7
        objValue[InformationElements.attributeNames[0x06]] = attributes[0x06][codingStandard] || codingStandard;

        /// Octet 3 bits 6,5,4,3,2,1
        objValue['Call state value (User / Network)'] = attributes[0x02][callStateValue] + ' - ' + callStateValue.toString(16) || callStateValue.toString(16);

        return objValue;
    };

})(Khomp);

(function (Khomp) {

    /// Called party number (pg 67)

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;
    var Util = Khomp.Util;

    InformationElements.elementsDecoders[0x70] = function (values) {

        var objValue = {};

        /// Octet 3
        var octet3 = values.shift();

        /// Type of number (bits 7,6,5)
        var typeOfNumber = (octet3 & 0x70) >> 4;

        /// Numbering plan identification (octeto 3, bits 4,3,2,1)
        var numberPlanIdentification = (octet3 & 0x0f);

        objValue['Numbering plan'] = attributes[0x27][numberPlanIdentification] || numberPlanIdentification;
        objValue['Type of number'] = attributes[0x36][typeOfNumber] || typeOfNumber;

        /// Octet 4 em diante
        // var octet4 = values;

        /// Number digits (octeto 4) tabela IA5 characters
        objValue['Number digits'] = Util.translateIA5(values);

        return objValue;
    };
})(Khomp);

(function (Khomp) {

    /// Called party subaddress (pg 69)

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;
    var Util = Khomp.Util;

    InformationElements.elementsDecoders[0x71] = function (values) {

        var objValue = {};

        var octet3;
        var octet4x;

        /// octet3
        octet3 = values.shift();

        var typeOfSubaddress = (octet3 & 0x70) >> 4;

        /// bits 7,6,5 octeto 3
        objValue['Type of subaddress'] = attributes[0x37][typeOfSubaddress] || typeOfSubaddress;

        if (typeOfSubaddress === 0x02) {

            /// NOTE 1 – The odd/even indicator is used when the type of subaddress is "user specified" and the coding is BCD.
            /// bit 4 octeto 3
            var oddEvenIndicator = (octet3 & 0x08) >> 3;
            objValue['Odd/even indicator'] = attributes[0x28][oddEvenIndicator] || oddEvenIndicator;
        }

        objValue['Subaddress information'] = '';
        while (values.length) {
            octet4x = values.shift();
            objValue['Subaddress information'] += Util.translateIA5(octet4x);
        }

        return objValue;

    };

})(Khomp);

(function (Khomp) {

    /// Calling party number (pg 70)

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;
    var Util = Khomp.Util;

    InformationElements.elementsDecoders[0x6c] = function (values) {

        var objValue = {};

        /// Octet 3
        var octet3 = values.shift();

        /// Type of number (bits 7,6,5)
        var typeOfNumber = (octet3 & 0x70) >> 4;

        objValue['Type of number'] = attributes[0x36][typeOfNumber] || typeOfNumber;

        if (typeOfNumber === 0x00 || typeOfNumber === 0x01 || typeOfNumber === 0x02 || typeOfNumber === 0x04) {

            /// Numbering plan (applies for type of number = 000, 001, 010 and 100)
            /// Numbering plan identification

            var numberPlanIdentification = (octet3 & 0x0f);

            objValue['Numbering plan'] = attributes[0x27][numberPlanIdentification] || numberPlanIdentification;
        }

        var extensionIndicator = (((octet3 & 0x80) >> 7) === 0) ? true : false;

        if (extensionIndicator) {
            /// Octet 3a
            var octet3a = values.shift();

            /// Presentation indicator (bits 7,6) (octeto 3a)
            var presentationIndicator = (octet3a & 0x60) >> 5;

            /// Screening indicator (bits 2,1) (octeto 3a)
            var screeningIndicator = (octet3a & 0x03);

            objValue['Screening indicator'] = attributes[0x31][screeningIndicator] || screeningIndicator;
            objValue['Presentation indicator'] = attributes[0x2b][presentationIndicator] || presentationIndicator;
        }

        /// Octet 4
        var octet4x;

        /// Number digits (octeto 4) IA5 characters
        objValue['Number digits'] = '';

        /// Traduzido de acordo com tabela IA5
        objValue['Number digits'] = Util.translateIA5(values);
        // while (values.length) {

            // octet4x = values.shift();

            // objValue['Number digits'] += Util.translateIA5(octet4x);
        // }

        return objValue;
    };

})(Khomp);


(function (Khomp) {

    /// Calling party subaddress (pg 71)

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;
    var Util = Khomp.Util;

    InformationElements.elementsDecoders[0x6d] = function (values) {

        var objValue = {};
        var octet3;
        var octet4x;

        /// octet3
        octet3 = values.shift();

        /// typeOfSubaddress: octet3 bits 7,6,5
        var typeOfSubaddress = (octet3 & 0x70) >> 4;

        objValue['Type of subaddress'] = attributes[0x37][typeOfSubaddress] || typeOfSubaddress;

        if (typeOfSubaddress === 0x02) {

            /// NOTE 1 – The odd/even indicator is used when the type of subaddress is "user specified" and the coding is BCD.
            /// oddEvenIndicator: octet3 bit 4
            var oddEvenIndicator = (octet3 & 0x08) >> 3;
            objValue['Odd/even indicator'] = attributes[0x28][oddEvenIndicator] || oddEvenIndicator;
        }

        objValue['Subaddress information'] = '';
        while (values.length) {
            octet4x = values.shift();
            objValue['Subaddress information'] += Util.translateIA5(octet4x, [8, 7, 6, 5, 4, 3, 2, 1]);
        }

        return objValue;

    };

})(Khomp);


(function (Khomp) {

    /// Cause
    /// Página 72 ITU Q.931. Detalhes na Q.850.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;

    InformationElements.elementsDecoders[0x08] = function (values) {

        var objValue = {};
        var octet3 = values.shift();
        var codingStandard = (octet3 & 0x60) >> 5;
        var location = (octet3 & 0x0f);

        objValue['Coding standard'] = attributes[0x06][codingStandard] || codingStandard;
        objValue.Location = attributes[0x16][location] || location;

        var extensionIndicatorOct3 = (((octet3 & 0x80) >> 7) === 0) ? true : false;

        if (extensionIndicatorOct3) {
            /// Recommendation... acho que não tem na Q.931. Ver Q.850 Cause format.
        }

        var octet4 = values.shift();
        var causeValue = (octet4 & 0x7f);

        objValue.Cause = attributes[0x03][causeValue] || causeValue;

        var octet;
        var key;
        /// Diagnostics
        while (values.length) {

            octet = values.shift();

            objValue.Diagnostics = (objValue.Diagnostics) ? (objValue.Diagnostics + ', ' + octet) : octet;

        }

        return objValue;
    };

})(Khomp);


(function (Khomp) {

    /// Channel identification (18)
    /// A partir da página 72 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;
    var Util = Khomp.Util;

    InformationElements.elementsDecoders[0x18] = function (values) {

        var objValue = {};
        var octet3;
        var octet31;
        var octet32;
        var octet33;

        /// Octeto 3
        octet3 = values.shift();

        /// Interface identifier present: Bit 7 octet 3
        var interfaceIdentifierPresent = (octet3 & 0x40) >> 6;
        objValue['Interface identifier present'] = attributes[0x12][interfaceIdentifierPresent] || interfaceIdentifierPresent;

        /// Interface type: Bit 6 octeto 3
        var interfaceType = (octet3 & 0x20) >> 5;
        objValue['Interface type'] = attributes[0x13][interfaceType] || interfaceType;

        var spare = (octet3 & 0x10) >> 4;

        /// Preferred/Exclusive: Bit 4 octeto 3
        var preferredExclusive = (octet3 & 0x08) >> 3;
        objValue['Preferred/Exclusive'] = attributes[0x2a][preferredExclusive] || preferredExclusive;

        /// D-channel indicator: Bit 3 octeto 3
        var dchannel = (octet3 & 0x04) >> 2;
        objValue['D-channel indicator'] = attributes[0x07][dchannel] || dchannel;

        /// Information channel selection: Bit 2,1 octeto 3
        var channel_selection = (octet3 & 0x03);
        var subtype = (interfaceType === 0) ? 0 : 1;
        objValue['Information channel selection'] = attributes[0x0f][subtype][channel_selection] || channel_selection;

        /// Octeto 3.1*
        /// NOTE 1 – When the "interface identifier present" field in octet 3 indicates "interface implicitly identified" octet 3.1 is omitted. When octet 3.1 is present, it may be extended by using the extension bit (bit 8)

        if (interfaceIdentifierPresent !== 0) {

            octet31 = values.shift();

            var interfaceIdentifier = (octet31 & 0x7f);
            objValue['Interface identifier'] = interfaceIdentifier;
        }

        /// Octeto 3.2*
        /// NOTE 2 – When the "interface type" field in octet 3 indicates "basic interface", octets 3.2 and 3.3 are functionally replaced by the "information channel selection" field in octet 3, and thus omitted
        if (interfaceType !== 0) {

            octet32 = values.shift();

            /// Coding standard: bits 7,6 octeto 3.2
            var codingStandard = (octet32 & 0x60) >> 5;
            objValue[InformationElements.attributeNames[0x06]] = attributes[0x06][codingStandard] || codingStandard;

            /// Number/map: bit 5 octeto 3.2
            var numberMap = (octet32 & 0x10) >> 4;
            objValue['Number/map'] = attributes[0x24][numberMap] || numberMap;

            /// Channel type/map element type: bits 4,3,2,1 do octeto 3.2
            var channelTypeOrMapElementType = (octet32 & 0x0f);
            objValue['Channel type/map element type'] = attributes[0x04][channelTypeOrMapElementType] || channelTypeOrMapElementType;

            if (values.length) {

                /// Octeto 3.3*

                if (numberMap === 0x00) {
                    /// -----------------------------------------
                    /// Quando o canal é um número
                    while (values.length) {

                        octet33 = values.shift();

                        /// NOTE 3 – When channel number is used and a single channel is indicated, bit 8 shall be set to "1". When channel number is used and multiple channels are indicated, bit 8 shall be used as an extension bit to indicate an extension to subsequent channels and coded according to the rules specified in 4.5.1
                        /// NOTE 4 – When channel number is used, this octet may be repeated to indicate multiple channels
                        var channelNumber = (octet33 & 0x7f);
                        objValue['Channel number'] = (objValue['Channel number']) ? objValue['Channel number'] + ', ' + channelNumber : channelNumber;

                        if (InformationElements.isLastOctet(octet33)) {
                            break;
                        }
                    }
                } else {

                    /// -----------------------------------------
                    /// Quando o canal é um slot map

                    /// NOTE 13 – The length of the slot map (in bits) is defined by the capacity of the interface type (e.g. 1534 kbit/s or 2048 kbit/s for a primary rate interface) divided by the capacity of the channel type/mapelement type (e.g. 64 kbit/s for a B-channel). The length of the slot map is the smallest number of complete octets that contain the length in bits
                    var slotMap;
                    while (values.length) {

                        octet33 = values.shift();

                        // if (channelTypeOrMapElementType === '0011') {
                        /// B-channel

                        slotMap = Util.getBitsString(octet33, [8, 7, 6, 5, 4, 3, 2, 1]);

                        objValue['Slot map'] = (objValue['Slot map']) ? objValue['Slot map'] + ', ' + slotMap : slotMap;

                        // } else if (channelTypeOrMapElementType === '0110') {
                        /// H0 channel
                        /// TODO: ver como tratar o e(5), d(4), c(3), b(2), a(1)
                        // } else {
                        /// TODO: ver como tratar o H11(1), H12(1), etc
                        /// H1x channel
                        // }
                    }
                }
            }

            if (values.length) {
                objValue['Não decodificado'] = values.join(' ');
            }

            return objValue;
        }
    };
})(Khomp);


(function (Khomp) {

    /// Congestion level
    /// A partir da página 76 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;

    InformationElements.elementsDecoders[0xb0] = function (value) {

        var attribute = {

            /// Somente bits 4,3,2,1 do octeto 1
            0x00 : 'Receiver ready',
            0x0f : 'Receiver not ready'
        };

        return attribute[value] || value;

    };

})(Khomp);


(function (Khomp) {

    /// Connected number
    /// Não consta na ITU Q931.
    /// Este é um elemento particular e não será decodificado.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var Util = Khomp.Util;

    InformationElements.elementsDecoders[0x4c] = function (values) {

        var octet3 = values.shift();

        if (!InformationElements.isLastOctet(octet3)) {
            var octet4 = values.shift();
        }

        return Util.translateIA5(values);

    };

})(Khomp);


(function (Khomp) {

    /// Date/time
    /// A partir da página 77 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;

    InformationElements.elementsDecoders[0x29] = function (values) {

        var y,
        m,
        d,
        h,
        i,
        s,
        datetime;

        y = values.shift();
        m = values.shift();
        d = values.shift();

        datetime = y + '-' + m + '-' + d;

        if (values.length) {
            h = values.shift();
            datetime += ' ' + h;
        }

        if (values.length) {
            i = values.shift();
            datetime += ':' + i;
        }

        if (values.length) {
            s = values.shift();
            datetime += ':' + s;
        }

        return datetime;
    };

})(Khomp);


(function (Khomp) {

    /// Display
    /// Página 77 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;

    InformationElements.elementsDecoders[0x28] = function (values) {

        return Khomp.Util.translateIA5(values);
    };

})(Khomp);

(function (Khomp) {

    /// Facility
    /// Não consta na ITU Q931.
    /// Este é um elemento particular e não será decodificado.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;

    InformationElements.elementsDecoders[0x1c] = function (values) {

        return values;

    };
})(Khomp);


(function (Khomp) {

    /// High layer compatibility - hlc (pg 78)
    /// Obs: aqui o primeiro octeto dos values já é o octeto 3 indicado na ITU.

    var Util = Khomp.Util;
    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;

    InformationElements.elementsDecoders[0x7d] = function (values) {

        var objValue = {};
        var octet3 = values.shift();

        /// Coding standard (octeto 3 bits 7, 6)
        var codingStandard = (octet3 & 0x60) >> 5;

        /// Interpretation (octeto 3 bits 5, 4, 3)
        var interpretation = (octet3 & 0x1c) >> 2;

        /// Presentation method of protocol profile (octeto 3 bits 2,1)
        var presentationMethod = (octet3 & 0x03);

        objValue[InformationElements.attributeNames[0x06]] = attributes[0x06][codingStandard] || codingStandard;

        objValue.Interpretation = attributes[0x15][interpretation] || interpretation;

        objValue['Presentation method of protocol profile'] = attributes[0x2c][presentationMethod] || presentationMethod;

        var octet4 = values.shift();

        /// High layer characteristics identification (octeto 4 bits 7,6,5,4,3,2,1)
        var highLayerCharacteristicsIdentification = (octet4 & 0x7f);

        objValue['High layer characteristics identification'] = attributes[0x0d][highLayerCharacteristicsIdentification] || highLayerCharacteristicsIdentification;

        if (!InformationElements.isLastOctet(octet4)) {

            var octet4a = values.shift();

            /// octet4a bits 7,6,5,4,3,2,1
            var extendedCharacteristicsIdentification = (octet4a & 0x7f);

            if (highLayerCharacteristicsIdentification === 0x5e || highLayerCharacteristicsIdentification === 0x5f) {

                /// Extended high layer characteristics identification: bits 7,6,5,4,3,2,1 do octeto 4a
                objValue['Extended high layer characteristics identification'] = attributes[0x0a][extendedCharacteristicsIdentification] || extendedCharacteristicsIdentification;

            } else if (highLayerCharacteristicsIdentification === 0x60 || highLayerCharacteristicsIdentification === 0x61 || highLayerCharacteristicsIdentification === 0x68) {

                /// Extended audiovisual characteristics identification: bits 7,6,5,4,3,2,1 octeto 4a
                objValue['Extended audiovisual characteristics identification'] = attributes[0x09][extendedCharacteristicsIdentification] || extendedCharacteristicsIdentification;

            } else {
                console.log('Valor não esperado');
            }
        }

        if (values.length) {
            objValue['Não decodificado'] = values.join(' ');
        }

        return objValue;

    };

})(Khomp);


(function (Khomp) {

    /// Keypad facility
    /// Página 81 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var Util = Khomp.Util;

    InformationElements.elementsDecoders[0x2c] = function (values) {

        return Util.translateIA5(values);

    };

})(Khomp);


(function (Khomp) {

    /// Low layer compability
    /// Página 81 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;

    InformationElements.elementsDecoders[0x7c] = function (values) {

        /// Objeto com a informação decodificada que será retornado.
        var objValue = {};

        /// Octet 3

        var octet3 = values.shift();

        /// Coding standard: bits 7,6 octet 3
        var codingStandard = (octet3 & 0x60) >> 5;

        /// Information transfer capability: bits 5,4,3,2,1 octet 3
        var informationTransferCapability = (octet3 & 0x1f);

        objValue[InformationElements.attributeNames[0x06]] = attributes[0x06][codingStandard] || codingStandard;
        objValue['Information transfer capability'] = attributes[0x10][informationTransferCapability] || informationTransferCapability;

        /// Octet 3a

        /// Negotiation indicator: bit 7 octet 3a
        /// TABLE 4-16/Q.931 NOTE 4 – When octet 3a is omitted, "out-band negotiation not possible" shall be assumed.
        objValue['Negotiation indicator'] = attributes[0x1f][0x00];

        if (!InformationElements.isLastOctet(octet3)) {

            var octet3a = values.shift();
            var negotiationIndicator = (octet3a & 0x40) >> 6;

            objValue['Negotiation indicator'] = attributes[0x1f][negotiationIndicator] || negotiationIndicator;

        }

        /// Octet 4

        var octet4 = values.shift();

        /// Transfer mode: bits 7,6 octet 4
        var transferMode = (octet4 & 0x60) >> 5;

        /// Information transfer rate: bits 5,4,3,2,1 octet 4
        /// TABLE 4-16/Q.931 NOTE 5 – When the information transfer rate 2×64 kbit/s is used, the coding of octets 3 and 4 refer to both 64 kbit/s channels.
        /// TABLE 4-16/Q.931 NOTE 6 – Additional attributes are defined in Table 4-17
        var informationTransferRate = (octet4 & 0x1f);

        objValue['Transfer mode'] = attributes[0x34][transferMode] || transferMode;
        objValue['Information transfer rate'] = attributes[0x11][transferMode][informationTransferRate] || informationTransferRate;

        /// Octet 4.1*
        /// NOTE 1 – This octet is required if octet 4 indicates multirate (64 kbit/s base rate). Otherwise, it shall not be present.

        if (informationTransferRate === 0x18) { // multirate (64 kbit/s base rate)

            var octet41 = values.shift();

            /// Rate multiplier
            objValue['Rate multiplier'] = octet41;
        }

        /// Octet 5*

        if (values.length === 0) {

            /// low layer compability - Caiu nesta condição porque normalmente deve ter mais dados (CONFIRMAR ISSO!!)

            return objValue;
        }

        var octet5 = values.shift();
        var layer1Ident = (octet5 & 0x60) >> 5;

        /// User information layer 1 protocol: bits 5,4,3,2,1 octet 5
        var userInfoLayer1Protocol = (octet5 & 0x1f);

        objValue['Layer 1 identifier'] = layer1Ident;
        objValue['User information layer 1 protocol'] = attributes[0x38][userInfoLayer1Protocol] || userInfoLayer1Protocol;

        /// Octet 5a*
        /// NOTE 2 – This octet may be present
        /* if octet 3 indicates unrestricted digital information
        and octet 5 indicates either of the
        ITU - T standardized rate adaptions V.110,
        I.460
        and X.30 or V.120[9].
        It may also be present
        if (octet 3 indicates 3.1 kHz audio)
        and(octet 5 indicates G.711).
         */
        var conditionNote2 = (
            (informationTransferCapability === 0x11 && (userInfoLayer1Protocol === 0x01 || userInfoLayer1Protocol === 0x08)) ||
            (informationTransferCapability === 0x11 && (userInfoLayer1Protocol === 0x02 || userInfoLayer1Protocol === 0x03))) ? true : false;

        var extensionIndicatorOct5 = (((octet5 & 0x80) >> 7) === 0) ? true : false;

        if (!extensionIndicatorOct5) {
            return objValue;
        }

        var octet5a = values.shift();

        /// Synchronous/Asynchronous: bit 7 octet 5a
        var syncData = (octet5a & 0x40) >> 6;
        objValue['Synchronous/Asynchronous'] = attributes[0x33][syncData] || syncData;

        /// Negotiation: bit 6 octet 5a
        var negotiation = (octet5a & 0x20) >> 5;
        objValue.Negotiation = attributes[0x1e][negotiation] || negotiation;

        /// User rate: bits 5,4,3,2,1 octet 5a
        var userRate = (octet5a & 0x1f);
        objValue['User rate'] = attributes[0x3b][userRate] || userRate;

        /// Octet 5b*
        /// NOTE 3 – This structure of octet 5b only applies if octet 5 indicates ITU-T standardized rate adaption (see Recommendations V.110 [7], I.460 [15] and X.30 [8])
        if (values.length && userInfoLayer1Protocol === 0x01) {

            var octet5b = values.shift();

            /// Intermediate rate bits 7,6 octet 5b
            var intermediateRate = (octet5b & 0x60) >> 5;
            objValue['Intermediate rate'] = attributes[0x14][intermediateRate] || intermediateRate;

            /// NIC on Tx: bit 5 octet 5b
            var nicOnTx = (octet5b & 0x10) >> 4;
            objValue['NIC on Tx'] = attributes[0x22][nicOnTx] || nicOnTx;

            /// NIC on Rx: bit 4 octet 5b
            var nicOnRx = (octet5b & 0x08) >> 3;
            objValue['NIC on Rx'] = attributes[0x21][nicOnRx] || nicOnRx;

            /// Flow control on Tx: bit 3 octet 5b
            var flowControlOnTx = (octet5b & 0x04) >> 2;
            objValue['Flow control on Tx'] = attributes[0x0c][flowControlOnTx] || flowControlOnTx;

            /// Flow control on Rx: bit 2 octet 5b
            var flowControlOnRx = (octet5b & 0x02) >> 1;
            objValue['Flow control on Rx'] = attributes[0x0b][flowControlOnRx] || flowControlOnRx;

            /// NOTE 4 – This structure of octet 5b only applies if octet 5 indicates ITU-T standardized rate adaption (see Recommendation V.120 [9]).
            if (!InformationElements.isLastOctet(octet5b) && userInfoLayer1Protocol === 0x08) {

                var octet5ba = values.shift();

                /// Rate adaption header: bit 7 octet 5b*
                var rateAdaptionHeader = (octet5ba & 0x40) >> 6;
                objValue['Rate adaption header'] = attributes[0x30][rateAdaptionHeader] || rateAdaptionHeader;

                /// Multiframe: bit 6 octet 5b*
                var multiframe = (octet5ba & 0x20) >> 5;
                objValue.Multiframe = attributes[0x1d][multiframe] || multiframe;

                /// Mode of operation: bit 5 octet 5b*
                var modeOfOperation = (octet5ba & 0x10) >> 4;
                objValue['Mode of operation'] = attributes[0x19][modeOfOperation] || modeOfOperation;

                /// Logical link identifier negotiation: bit 4 octet 5b*
                var logicalLinkIdentifierNegotiation = (octet5ba & 0x08) >> 3;
                objValue['Logical link identifier negotiation'] = attributes[0x17][logicalLinkIdentifierNegotiation] || logicalLinkIdentifierNegotiation;

                /// Assignor/assignee: bit 3 octet 5b*
                var assignorAssignee = (octet5ba & 0x04) >> 2;
                objValue[InformationElements.attributeNames[0x01]] = attributes[0x01][assignorAssignee] || assignorAssignee;

                /// In-band/out-band negotiation: bit 2 octet 5b*
                var inBandOutBandNegotiation = (octet5ba & 0x02) >> 1;
                objValue['In-band/out-band negotiation'] = attributes[0x0e][inBandOutBandNegotiation] || inBandOutBandNegotiation;

                /// Octet 5c*
                /// NOTE 2
                if (values.length && !InformationElements.isLastOctet(octet5ba) && conditionNote2) {

                    var octet5c = values.shift();

                    /// Number of stop bits: bits 7,6 octet 5c*
                    var numberOfStopBits = (octet5c & 0x60) >> 5;
                    objValue['Number of stop bits'] = attributes[0x26][numberOfStopBits] || numberOfStopBits;

                    /// Number of data bits excluding parity Bit if present: bits 5,4 octet 5c*
                    var numberOfDataBitsExcludingParityBitIfPresent = (octet5c & 0x18) >> 3;
                    objValue['Number of data bits excluding parity Bit if present'] = attributes[0x25][numberOfDataBitsExcludingParityBitIfPresent] || numberOfDataBitsExcludingParityBitIfPresent;

                    /// Parity information: bits 3,2,1 octet 5c*
                    var parityInformation = (octet5c & 0x07);
                    objValue['Parity information'] = attributes[0x29][parityInformation] || parityInformation;

                    /// Octet 5d*
                    /// NOTE 2
                    if (values.length && !InformationElements.isLastOctet(octet5c) && conditionNote2) {

                        var octet5d = values.shift();

                        /// Mode duplex: bit 7 octet 5d*
                        var modeDuplex = (octet5d & 0x40) >> 6;
                        objValue['Mode duplex'] = attributes[0x18][modeDuplex] || modeDuplex;

                        /// Modem type: bits 6,5,4,3,2,1 octet 5d*
                        var modemType = (octet5d & 0x3f);
                        objValue['Modem type'] = attributes[0x1c][modemType] || modemType;

                    }
                }
            }

            /// Octet 6*
            if (values.length) {

                var octet6 = values.shift();

                var layer2Ident = (octet6 & 0x60) >> 5;
                objValue['Layer 2 identifier'] = layer2Ident;

                /// User information layer 2 protocol:  bits 5,4,3,2,1 octet 6
                /// NOTE 20 – This Recommendation is compatible with ISO/IEC 7776 DTE-DCE operation.
                /// NOTE 21 – When this coding is included, octet 6a will include user coding for the user specified Layer 2 protocol.
                /// NOTE 22 – This Standard is compatible with Recommendation X.75 modified by the application rules defined in Recommendation T.90.
                /// NOTE 23 – When this coding is included, octets 6a and 6b with ITU-T encoding may be included.
                var userInfoLayer2Protocol = (octet6 & 0x1f);
                objValue['User information layer 2 protocol'] = attributes[0x39][userInfoLayer2Protocol] || userInfoLayer2Protocol;

                /// NOTE 5 – This octet may be present only if octet 6 indicates certain acknowledged mode HDLC elements of procedure as indicated in Table 4-16.
                if (!InformationElements.isLastOctet(octet6)) {

                    /// NOTE 23 – When this coding is included, octets 6a and 6b with ITU-T encoding may be included.

                    if (userInfoLayer2Protocol === 0x02 ||
                        userInfoLayer2Protocol === 0x06 ||
                        userInfoLayer2Protocol === 0x07 ||
                        userInfoLayer2Protocol === 0x09 ||
                        userInfoLayer2Protocol === 0x0a ||
                        userInfoLayer2Protocol === 0x0b ||
                        userInfoLayer2Protocol === 0x0d ||
                        userInfoLayer2Protocol === 0x0e ||
                        userInfoLayer2Protocol === 0x11) {

                        /// Octet 6a*
                        var octet6a = values.shift();

                        /// Mode of operation: bits 7,6 octet 6a
                        var modeOfOperationLayer2 = (octet6a & 0x60) >> 5;

                        objValue['Mode of operation layer 2'] = attributes[0x1a][modeOfOperationLayer2] || modeOfOperationLayer2;

                        /// Q.933 use: bits 2,1 octet 6a
                        var q933 = (octet6a & 0x03);

                        objValue['Q.933 use'] = attributes[0x2f][q933] || q933;

                        if (!InformationElements.isLastOctet(octet6a)) {

                            /// Octet 6a2*
                            var octet6a2 = values.shift();
                            var userSpecifiedLayer2ProtocolInformation = (octet6a2 & 0x7f);
                            objValue['User specified layer 2 protocol information '] = userSpecifiedLayer2ProtocolInformation;

                        }
                        /// Octet 6b*
                        var octet6b = values.shift();
                        var windowSize = (octet6b & 0x7f);
                        objValue['Window size'] = windowSize + 'k';
                    }

                    /// Octet 7
                    var octet7 = values.shift();

                    var layer3Identifier = (octet7 & 0x60) >> 5;
                    objValue['Layer 3 identifier'] = layer3Identifier;

                    /// User information layer 3 protocol: bits 5,4,3,2,1 octet 7
                    var userInfoLayer3Protocol = (octet7 & 0x1f);
                    objValue['User information layer 3 protocol'] = attributes[0x3a][userInfoLayer3Protocol] || userInfoLayer3Protocol;

                    /// Octet 7a* - primeiro caso
                    var octet7a;

                    if (userInfoLayer3Protocol === 0x10) {
                        /// NOTE 8 – This octet may be present only if octet 7 indicates 'user specified' layer 3 protocol.
                        /// Table 4-16/Q.931 NOTE 24 – When this coding is included, octet 7a will include user coding for the user specified layer 3 protocol.

                        octet7a = values.shift();

                        objValue['Optional layer 3 protocol information'] = (octet7a);

                        /// Octet 7a* - segundo caso
                    } else if (
                        /// NOTE 7 – This octet may be present only if octet 7 indicates a layer 3 protocol based on ITU-T X.25 [5], ISO/IEC 8208 [41] or ITU-T Rec. X.223 [96] and ISO/IEC 8878 [81] as indicated in Table 4-16.
                        /// Table 4-16/Q.931 NOTE 25 – When this coding is included, octets 7a, 7b and 7c with ITU-T X.25, X.223 and ISO/IEC TR 9577 encoding may be included.

                        userInfoLayer3Protocol === 0x06 ||
                        userInfoLayer3Protocol === 0x07 ||
                        userInfoLayer3Protocol === 0x08) {

                        octet7a = values.shift();

                        /// Mode of operation layer 3: bits 7,6 octet 7a
                        var modeOfOperationLayer3 = (octet7a & 0x60) >> 5;

                        objValue['Mode of operation layer 3'] = attributes[0x1b][modeOfOperationLayer3] || modeOfOperationLayer3;

                        if (!InformationElements.isLastOctet(octet7a)) {

                            var octet7b = values.shift();

                            /// Default packet size: bits 4,3,2,1 octet 7b
                            var defaultPacketSize = (octet7b & 0x0f);

                            objValue['Default packet size'] = attributes[0x08][defaultPacketSize] || defaultPacketSize;

                            if (!InformationElements.isLastOctet(octet7b)) {

                                var octet7c = values.shift();
                                /// Packet window size: octet7c bits 7,6,5,4,3,2,1
                                objValue['Packet window size'] = (octet7c & 0x7f);
                            }
                        }

                    } else if (userInfoLayer3Protocol === 0x0b) {
                        /// Octet 7a* - terceiro caso
                        /// NOTE 9 – This may be included if octet 7 indicates ISO/IEC TR 9577.
                        /// NOTE 26 – When this coding is included, octets 7a and 7b with ITU-T X.25, X.223 [96] and ISO/IEC TR 9577 encoding may be included
                    }

                }
            }
        }

        if (values.length) {
            objValue['Não decodificado'] = values.join(' ');
        }
        return objValue;
    };

})(Khomp);


(function (Khomp) {

    /// More data
    /// Página 93 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;

    InformationElements.elementsDecoders[0xa0] = function (values) {

        return null;

    };
})(Khomp);


(function (Khomp) {

    /// Network-specific facilities
    /// Página 93 ITU Q.931

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;
    var Util = Khomp.Util;

    InformationElements.elementsDecoders[0x20] = function (values) {

        var objValue = {};

        /// Length of network identification (bits 8,7,6,5,4,3,2,1 octet 3)
        /// This field contains the length, in octets, of the network identification found in octet 3.1 and the repetition of octet 3.2. If the value is "0000 0000", then the default provider (see E.1) is assumed and octets 3.1 and 3.2 are omitted.
        var octet3 = values.shift();

        if (octet3 <= 0) {
            /// ITU Q.931 - E.1
            return 'Default provider';
        }

        var octet31 = values.shift();

        /// Type of network identification (bits 7,6,5 octet 3.1*)
        var typeOfNetworkIdentification = (octet31 & 0x70) >> 4;

        /// Network Identification (bits 4,3,2,1 octet31)
        var networkIdentificationPlan = (octet31 & 0x0f);

        objValue['Type of network identification'] = attributes[0x35][typeOfNetworkIdentification] || typeOfNetworkIdentification;
        objValue['Network identification plan'] = attributes[0x20][networkIdentificationPlan] || networkIdentificationPlan;

        var octet32;
        objValue['Network identification'] = '';
        while (values.length > 1) {

            octet32 = values.shift();
            objValue['Network identification'] += Util.translateIA5(octet32);
        }

        var octet4 = values.shift();
        objValue['Network-specific facility specification'] = octet4;

        return objValue;

    };
})(Khomp);


(function (Khomp) {

    /// Notification indicator
    /// Página 94 ITU Q.931

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;

    InformationElements.elementsDecoders[0x27] = function (values) {

        var objValue = {};
        var octet3 = values.shift();

        /// Notification description octeto 3 bits 7,6,5,4,3,2,1
        var key = (octet3 & 0x7f);

        objValue['Notification description'] = attributes[0x23][key] || key;

        return objValue;
    };

})(Khomp);

(function (Khomp) {

    /// Progress indicator (1E)
    /// Página 95 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;

    InformationElements.elementsDecoders[0x1e] = function (values) {

        var objValue = {};

        var octet3 = values.shift();
        var octet4 = values.shift();

        /// Octeto 3 bits 7,6
        var codingStandard = (octet3 & 0x60) >> 5;

        /// Octeto 3 bits 4,3,2,1
        var location = (octet3 & 0x0f);

        /// Octeto 4 bits 7,6,5,4,3,2,1
        var progressDescription = (octet4 & 0x7f);

        objValue[InformationElements.attributeNames[0x06]] = attributes[0x06][codingStandard] || codingStandard;
        objValue.Location = attributes[0x16][location] || location;
        objValue['Progress description'] = attributes[0x2d][progressDescription] || progressDescription;

        if (values.length) {
            objValue['Não decodificado'] = values.join(' ');
        }

        return objValue;
    };
})(Khomp);


(function (Khomp) {

    /// Repeat indicator
    /// Página 96 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;

    InformationElements.elementsDecoders[0xd0] = function (values) {

        var octet = values;

        return ((octet & 0x0f) !== 0x02) ? (octet & 0x0f) : 'Prioritized list for selecting one possibility'; // (Note)
        /// Table 4-21/Q.931 NOTE – Used for Bearer service change procedures (see Annex L).
    };

})(Khomp);

(function (Khomp) {

    /// Restart indicator
    /// Página 97 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;

    InformationElements.elementsDecoders[0x79] = function (value) {

        var objValue = {};
        var classe = (value & 0x07);

        objValue.Class = attributes[0x05][classe] || classe;

        return objValue;
    };

})(Khomp);


(function (Khomp) {

    /// Reverse charging indication
    /// Página 111 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;

    InformationElements.elementsDecoders[0x4a] = function (value) {

        var octet3 = value.shift();

        return (octet3 & 0x07);

    };

})(Khomp);


(function (Khomp) {

    /// Segmented message
    /// Página 98 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;

    InformationElements.elementsDecoders[0x00] = function (values) {

        var objValue = {};
        var attributes = {
            'First segment indicator' : {
                0x00 : 'Subsequent segment to first segment',
                0x01 : 'First segment of segmented message'
            }
        };

        var octet3 = values.shift();
        var octet4 = values.shift();

        var firstSegmentIndicator = (octet3 & 0x80) >> 7;
        var numberOfSegmentsRemaining = (octet3 & 0x7f);

        objValue['First segment indicator'] = attributes['First segment indicator'][firstSegmentIndicator] || firstSegmentIndicator;
        objValue['Number of segments remaining'] = numberOfSegmentsRemaining;

        var segmentedMessageType = (octet4 & 0x7f);
        objValue['Segmented message type'] = (segmentedMessageType);

        return objValue;

    };

})(Khomp);


(function (Khomp) {

    /// Sending complete

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;

    InformationElements.elementsDecoders[0xa1] = function (value) {
        return null;
    };

})(Khomp);


(function (Khomp) {

    /// Signal
    /// Página 99 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;

    InformationElements.elementsDecoders[0x34] = function (values) {

        var octet3 = values.shift();
        var signalValue = octet3;

        return {
            'Signal value' : attributes[0x32][signalValue] || signalValue
        };
    };
})(Khomp);


(function (Khomp) {

    /// Transit network selection
    /// Página 100 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;

    InformationElements.elementsDecoders[0x78] = function (values) {

        var objValue = {};
        var octet3 = values.shift();

        var typeOfNetworkIdentification = (octet3 & 0x70) >> 4;
        var networkIdentificationPlan = (octet3 & 0x0f);

        objValue['Type of network identification'] = attributes[0x35][typeOfNetworkIdentification] || typeOfNetworkIdentification;
        objValue['Network identification plan'] = attributes[0x20][networkIdentificationPlan] || networkIdentificationPlan;

        objValue['Network identification'] = values.join(' ');

        return objValue;
    };
})(Khomp);


(function (Khomp) {

    /// User-user
    /// Página 101 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;

    InformationElements.elementsDecoders[0x7e] = function (values) {

        var objValue = {};
        var octet3 = values.shift();
        var protocolDiscriminator = (octet3);

        objValue['Protocol discriminator'] = attributes[0x2e][protocolDiscriminator] || protocolDiscriminator;

        objValue['User information'] = values.join(' ');

        return objValue;
    };

})(Khomp);


(function (Khomp) {

    /// ----------------------------------------------------------------------------------------------
    /// Trecho relacionado à identificação das Messages

    var Util = Khomp.Util;
    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;

    /**
    Classe mensagem ISDN

    @module isdn
    @submodule isdn-decoder
    @class Message
    @contructor
     */
    var Message = Khomp.ISDN.Message = function (lapdMessage) {

        if (matchMessage(lapdMessage)) {
            this.oldFormat = false;
        } else if (matchMessageV2(lapdMessage)) {
            this.oldFormat = true;
        } else  {
            this.isValid = false;
            return null;
        }

        this.isValid = true;
        this.source = lapdMessage.replace(/[\r\n]/g, '');

    };

    /// Tipos de mensagens conhecidas. Tem tipos além dos que constam na ITU
    var messageTypes = {

        0x00 : 'Escape to nationally specific message type (Note)', //00

        /// 000----- Call establishment message:
        0x01 : 'ALERTING',
        0x02 : 'CALL PROCEEDING',
        0x07 : 'CONNECT',
        0x0f : 'CONNECT ACKNOWLEDGE',
        0x03 : 'PROGRESS',
        0x05 : 'SETUP',
        0x0d : 'SETUP ACKNOWLEDGE',

        /// 001----- Call information phase message:
        0x26 : 'RESUME',
        0x2e : 'RESUME ACKNOWLEDGE',
        0x22 : 'RESUME REJECT',
        0x25 : 'SUSPEND',
        0x2d : 'SUSPEND ACKNOWLEDGE',
        0x21 : 'SUSPEND REJECT',
        0x20 : 'USER INFORMATION',

        ///010----- Call clearing messages:
        0x45 : 'DISCONNECT',
        0x4d : 'RELEASE',
        0x5a : 'RELEASE COMPLETE',
        0x46 : 'RESTART',
        0x4e : 'RESTART ACKNOWLEDGE',

        /// 011----- Miscellaneous messages:
        0x60 : 'SEGMENT',
        0x79 : 'CONGESTION CONTROL',
        0x7b : 'INFORMATION',
        0x6e : 'NOTIFY',
        0x7d : 'STATUS',
        0x75 : 'STATUS ENQUIRY',
        
        /// Additional reference: http://www.zytrax.com/tech/protocols/isdn/msg.htm
        0x62 : 'FACILITY',
        0x64 : 'REGISTER',
        0x24 : 'HOLD',
        0x28 : 'HOLD ACKNOWLEDGE',
        0x30 : 'HOLD REJECT',
        0x31 : 'RETRIEVE',
        0x33 : 'RETRIEVE ACKNOWLEDGE',
        0x37 : 'RETRIEVE REJECT',
        0x7e : 'NOTIFY',
        0xfb : 'KEY HOLD',
        0xfc : 'KEY RELEASE',
        0xff : 'KEY SETUP',
        0xfe : 'KEY SETUP ACKNOWLEDGE',
        /// Custom Lucent
        0x90 : 'CONFERENCE',
        0x91 : 'CONFERENCE ACKNOWLEDGE',
        0x92 : 'CONFERENCE REJECT',
        0x93 : 'TRANSFER',
        0x94 : 'TRANSFER ACKNOWLEDGE',
        0x95 : 'TRANSFER REJECT',
        0x96 : 'ASSOCIATED',
        0x97 : 'ASSOCIATED ACKNOWLEDGE',
        0x98 : 'DROP',
        0x99 : 'DROP ACKNOWLEDGE',
        0x9a : 'DROP REJECT',
        0x9b : 'HOLD',
        0x9c : 'HOLD ACKNOWLEDGE',
        0x9d : 'HOLD REJECT',
        0x9e : 'RECONNECT',
        0x9f : 'RECONNECT ACKNOWLEDGE',
        0xb0 : 'RECONNECT REJECT',
        0xb1 : 'REDIRECT',
        0xf7 : 'MANAGEMENT INFORMATION'
    };

    /// Informação da etapa chamada que a mensagem pertence
    var stepsMap = {

        0x00 : -1, //'Escape to nationally specific message type (Note)', //00

        /// 000----- Call establishment message:
        0x01 : 0, //'ALERTING',
        0x02 : 0, //'CALL PROCEEDING',
        0x07 : 0, //'CONNECT',
        0x0f : 0, //'CONNECT ACKNOWLEDGE',
        0x03 : 0, //'PROGRESS',
        0x05 : 0, //'SETUP',
        0x0d : 0, //'SETUP ACKNOWLEDGE'

        /// 001----- Call information phase message:
        0x26 : 1, //'RESUME',
        0x2e : 1, //'RESUME ACKNOWLEDGE',
        0x22 : 1, //'RESUME REJECT',
        0x25 : 1, //'SUSPEND',
        0x2d : 1, //'SUSPEND ACKNOWLEDGE',
        0x21 : 1, //'SUSPEND REJECT',
        0x20 : 1, //'USER INFORMATION',

        ///010----- Call clearing messages:
        0x45 : 2, //'DISCONNECT',
        0x4d : 2, //'RELEASE',
        0x5a : 2, //'RELEASE COMPLETE',
        0x46 : 2, //'RESTART',
        0x4e : 2, //'RESTART ACKNOWLEDGE',

        /// 011----- Miscellaneous messages:
        0x60 : 3, //'SEGMENT',
        0x79 : 3, //'CONGESTION CONTROL',
        0x7b : 3, //'INFORMATION',
        0x6e : 3, //'NOTIFY',
        0x7d : 3, //'STATUS',
        0x75 : 3 //'STATUS ENQUIRY'
    };

    /**
    Este método valida e separa os dados básicos da mensagem para versões antigas do log
    @method matchMessageV2
    @param {String} lapdMessage
    @return {Object} Message object
     */
    var actualDate;
    var matchMessageV2 = function (lapdMessage) {

        if (lapdMessage.indexOf('#DATE') !== -1) {

            /// #DATE 31/03/2014
            actualDate = lapdMessage.substr(6, 10);
            return false;

        } else if (lapdMessage.indexOf('LAPD MSG I', 23) === -1) {

            /// T|08:39:02.638 |N1| <- LAPD MSG I [C (P=0), N(S)=115, N(R)= 85] 02 01 E6 AA 08 02 80 5E 01 1E 02 82 82 1E 02 82 88
            return false;
        }

        return true;
    };

    /**
    Este método valida e separa os dados básicos da mensagem

    @method matchMessage
    @param {String} lapdMessage
    @return {Object} Message object
     */
    var matchMessage = function (lapdMessage) {

        /// T|04/11/2014 21:43:30.400|D51385 L2| -> LAPD MSG I [C (P=0), N(S)= 32, N(R)=  0] 02 01 40 00 08 02 3E CF 0F
        /// T|23/09/2014 12:38:34.867|D48088 L0| <- LAPD MSG I [C (P=0), N(S)=  0, N(R)= 15] 02 01 00 1E 08 02 49 2C 45 08 02 84 90 1E 02 82 88 1E 02 84 83

        if (lapdMessage.indexOf('LAPD MSG I', 35) === -1) {
            return null;
        } else {
            return true;
        }

    };

    /**
    Retorna o texto que representa a etapa da mensagem

    @method getStep
    @return {String}
     */
    Message.prototype.getStep = function () {

        var code = this.type;

        var steps = ['Call establishment message', 'Call information phase message', 'Call clearing messages', 'Miscellaneous messages'];

        var mapped = (code in stepsMap) ? stepsMap[code] : null;
        var step_text = (mapped >= 0) ? steps[mapped] : code;

        return step_text || 'UNKNOW';

    };

    /**
    Retorna a data e hora do source da mensagem

    @method getDatetime
    @return {String} Data e hora da mensagem
     */
    Message.prototype.getDatetime = function () {

        if (this.oldFormat) {
            return actualDate + ' ' + this.source.substr(2, 12);
        } else {
            return this.source.substr(2, 23);
        }
    };

    /**
    DeviceId utilizado na mensagem

    @method getDeviceId
    @return {String}
     */
    Message.prototype.getDeviceId = function () {

        if (this.oldFormat) {
            return this.source.substr(16, 2);
        } else {
            return this.source.substr(26, 6);
        }
    };

    /**
    Link utilizado na mensagem

    @method getLink
    @return {String}
     */
    Message.prototype.getLink = function () {

        if (this.oldFormat) {
            return this.source.substr(16, 2);
        } else {
            return this.source.substr(33, 2);
        }
    };

    /**
    Direção da mensagem

    -> (Send) para quando envia
    <- (Receive) para quando recebe

    @method getDirection
    @return {String}
     */
    Message.prototype.getDirection = function () {

        if (this.oldFormat) {
            return this.source.substr(20, 2);
        } else {
            return this.source.substr(37, 2);
        }
    };

    /**
    Pega os octetos com os dados da mensagem

    @method getData
    @return {String}
     */
    Message.prototype.getData = function () {

        if (!this.data) {

            this.data = (this.oldFormat) ? this.source.substr(64).split(' ') : this.source.substr(81).split(' ');

            for (var data_pos = 0, data_len = this.data.length; data_pos < data_len; data_pos++) {
                this.data[data_pos] = parseInt(this.data[data_pos], 16);
            }
        }

        return this.data;
    };

    /**
    Pega o CRV da mensagem

    @method getCallReference
    @return {String}
     */
    Message.prototype.getCallReference = function () {

        var data = this.getData();
        var crvLength = data[5];
        var crv = data.slice(6, 6 + crvLength);

        crv[0] = Util.numberToHex(Util.clearBit(crv[0], 8));
        if (crv[1]) {
            crv[1] = Util.numberToHex(crv[1]);
        }

        return crv.join('');
    };

    /**
    Pega o CRV da mensagem da outra ponta

    @method getCallReferenceOther
    @return {String}
     */
    Message.prototype.getCallReferenceOther = function () {

        var data = this.getData();
        var crvLength = data[5];
        var crv = data.slice(6, 6 + crvLength);

        crv[0] = Util.numberToHex(Util.setBit(crv[0], 8));

        return crv.join('');
    };

    /**
    Pega a direção da chamada com base no CRV

    @method getCallReferenceDirection
    @return {String}
     */
    Message.prototype.getCallReferenceDirection = function () {

        var data = this.getData();

        var test = data[6];

        return ((test & 0x80) >> 7) ? '<-' : '->';
    };

    /**
    Armazena o posicionamento dos information elements na mensagem
    */
    Message.prototype.info_elements_pos = [];

    /**
    Separa os information elements da mensagem

    @method getInformationElements
    @return {Object}
     */
    Message.prototype.getInformationElements = function () {

        this.info_elements_pos = [];

        var data = this.getData();
        var crvLength = data[5];
        var pos = 6 + crvLength + 1;

        var infos = data.slice(pos);

        if (infos.length) {

            this.infos = InformationElements.extractInfos(infos);

        }
        if (this.infos) {
            var pos_begin;
            for (var i = 0; i < this.infos.length; ++i) {
                pos_begin = pos;
                pos = ((this.infos[i].length === 1) ? pos_begin : pos_begin + this.infos[i].length);
                this.info_elements_pos.push([pos_begin, pos]);
                pos = pos + 1;
            }
        }

        return this.infos;
    };

    /**
    Decodifica os information elements da mensagem

    @method getDecodedInformationElements
    @return {Object}
     */
    Message.prototype.getDecodedInformationElements = function () {

        this.infos = this.infos || this.getInformationElements();
        var decodedInfos = [];
        var infoName;
        var infoData;
        var infoCode;

        for (var info_pos = 0, infos_len = this.infos.length; info_pos < infos_len; info_pos++) {

            info = this.infos[info_pos];
            infoCode = info[0];
            infoData = info.slice(1);

            infoName = InformationElements.types[infoCode] || infoCode;
            decodedInfos.push({name: infoName, value: InformationElements.decodeInfo(infoCode, infoData) });
        }

        return decodedInfos;
    };

    /**
    Pega o código do tipo da mensagem

    @method getType
    @return {String}
     */
    Message.prototype.getType = function () {

        if (this.type === 'number') {
            return this.type;
        }

        var data = this.getData();
        var crvLength = data[5];
        var pos = 6 + crvLength;
        this.type = data[pos];

        return this.type;
    };

    /**
    Pega o nome do tipo da mensagem

    @method getTypeName
    @return {String}
     */
    Message.prototype.getTypeName = function () {

        var type = this.getType();

        return messageTypes[type] || type;
    };

})(Khomp);


