(function (Khomp) {

    /// Network-specific facilities
    /// Página 93 ITU Q.931

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;
    var Util = Khomp.Util;

    InformationElements.elementsDecoders[0x20] = function (values) {

        var objValue = {};

        /// Length of network identification (bits 8,7,6,5,4,3,2,1 octet 3)
        /// This field contains the length, in octets, of the network identification found in octet 3.1 and the repetition of octet 3.2. If the value is "0000 0000", then the default provider (see E.1) is assumed and octets 3.1 and 3.2 are omitted.
        var octet3 = values.shift();

        if (octet3 <= 0) {
            /// ITU Q.931 - E.1
            return 'Default provider';
        }

        var octet31 = values.shift();

        /// Type of network identification (bits 7,6,5 octet 3.1*)
        var typeOfNetworkIdentification = (octet31 & 0x70) >> 4;

        /// Network Identification (bits 4,3,2,1 octet31)
        var networkIdentificationPlan = (octet31 & 0x0f);

        objValue['Type of network identification'] = attributes[0x35][typeOfNetworkIdentification] || typeOfNetworkIdentification;
        objValue['Network identification plan'] = attributes[0x20][networkIdentificationPlan] || networkIdentificationPlan;

        var octet32;
        objValue['Network identification'] = '';
        while (values.length > 1) {

            octet32 = values.shift();
            objValue['Network identification'] += Util.translateIA5(octet32);
        }

        var octet4 = values.shift();
        objValue['Network-specific facility specification'] = octet4;

        return objValue;

    };
})(Khomp);
