(function (Khomp) {

    /// ----------------------------------------------------------------------------------------------
    /// Trecho relacionado à identificação das Messages

    var Util = Khomp.Util;
    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;

    /**
    Classe mensagem ISDN

    @module isdn
    @submodule isdn-decoder
    @class Message
    @contructor
     */
    var Message = Khomp.ISDN.Message = function (lapdMessage) {

        if (matchMessage(lapdMessage)) {
            this.oldFormat = false;
        } else if (matchMessageV2(lapdMessage)) {
            this.oldFormat = true;
        } else  {
            this.isValid = false;
            return null;
        }

        this.isValid = true;
        this.source = lapdMessage.replace(/[\r\n]/g, '');

    };

    /// Tipos de mensagens conhecidas. Tem tipos além dos que constam na ITU
    var messageTypes = {

        0x00 : 'Escape to nationally specific message type (Note)', //00

        /// 000----- Call establishment message:
        0x01 : 'ALERTING',
        0x02 : 'CALL PROCEEDING',
        0x07 : 'CONNECT',
        0x0f : 'CONNECT ACKNOWLEDGE',
        0x03 : 'PROGRESS',
        0x05 : 'SETUP',
        0x0d : 'SETUP ACKNOWLEDGE',

        /// 001----- Call information phase message:
        0x26 : 'RESUME',
        0x2e : 'RESUME ACKNOWLEDGE',
        0x22 : 'RESUME REJECT',
        0x25 : 'SUSPEND',
        0x2d : 'SUSPEND ACKNOWLEDGE',
        0x21 : 'SUSPEND REJECT',
        0x20 : 'USER INFORMATION',

        ///010----- Call clearing messages:
        0x45 : 'DISCONNECT',
        0x4d : 'RELEASE',
        0x5a : 'RELEASE COMPLETE',
        0x46 : 'RESTART',
        0x4e : 'RESTART ACKNOWLEDGE',

        /// 011----- Miscellaneous messages:
        0x60 : 'SEGMENT',
        0x79 : 'CONGESTION CONTROL',
        0x7b : 'INFORMATION',
        0x6e : 'NOTIFY',
        0x7d : 'STATUS',
        0x75 : 'STATUS ENQUIRY',
        
        /// Additional reference: http://www.zytrax.com/tech/protocols/isdn/msg.htm
        0x62 : 'FACILITY',
        0x64 : 'REGISTER',
        0x24 : 'HOLD',
        0x28 : 'HOLD ACKNOWLEDGE',
        0x30 : 'HOLD REJECT',
        0x31 : 'RETRIEVE',
        0x33 : 'RETRIEVE ACKNOWLEDGE',
        0x37 : 'RETRIEVE REJECT',
        0x7e : 'NOTIFY',
        0xfb : 'KEY HOLD',
        0xfc : 'KEY RELEASE',
        0xff : 'KEY SETUP',
        0xfe : 'KEY SETUP ACKNOWLEDGE',
        /// Custom Lucent
        0x90 : 'CONFERENCE',
        0x91 : 'CONFERENCE ACKNOWLEDGE',
        0x92 : 'CONFERENCE REJECT',
        0x93 : 'TRANSFER',
        0x94 : 'TRANSFER ACKNOWLEDGE',
        0x95 : 'TRANSFER REJECT',
        0x96 : 'ASSOCIATED',
        0x97 : 'ASSOCIATED ACKNOWLEDGE',
        0x98 : 'DROP',
        0x99 : 'DROP ACKNOWLEDGE',
        0x9a : 'DROP REJECT',
        0x9b : 'HOLD',
        0x9c : 'HOLD ACKNOWLEDGE',
        0x9d : 'HOLD REJECT',
        0x9e : 'RECONNECT',
        0x9f : 'RECONNECT ACKNOWLEDGE',
        0xb0 : 'RECONNECT REJECT',
        0xb1 : 'REDIRECT',
        0xf7 : 'MANAGEMENT INFORMATION'
    };

    /// Informação da etapa chamada que a mensagem pertence
    var stepsMap = {

        0x00 : -1, //'Escape to nationally specific message type (Note)', //00

        /// 000----- Call establishment message:
        0x01 : 0, //'ALERTING',
        0x02 : 0, //'CALL PROCEEDING',
        0x07 : 0, //'CONNECT',
        0x0f : 0, //'CONNECT ACKNOWLEDGE',
        0x03 : 0, //'PROGRESS',
        0x05 : 0, //'SETUP',
        0x0d : 0, //'SETUP ACKNOWLEDGE'

        /// 001----- Call information phase message:
        0x26 : 1, //'RESUME',
        0x2e : 1, //'RESUME ACKNOWLEDGE',
        0x22 : 1, //'RESUME REJECT',
        0x25 : 1, //'SUSPEND',
        0x2d : 1, //'SUSPEND ACKNOWLEDGE',
        0x21 : 1, //'SUSPEND REJECT',
        0x20 : 1, //'USER INFORMATION',

        ///010----- Call clearing messages:
        0x45 : 2, //'DISCONNECT',
        0x4d : 2, //'RELEASE',
        0x5a : 2, //'RELEASE COMPLETE',
        0x46 : 2, //'RESTART',
        0x4e : 2, //'RESTART ACKNOWLEDGE',

        /// 011----- Miscellaneous messages:
        0x60 : 3, //'SEGMENT',
        0x79 : 3, //'CONGESTION CONTROL',
        0x7b : 3, //'INFORMATION',
        0x6e : 3, //'NOTIFY',
        0x7d : 3, //'STATUS',
        0x75 : 3 //'STATUS ENQUIRY'
    };

    /**
    Este método valida e separa os dados básicos da mensagem para versões antigas do log
    @method matchMessageV2
    @param {String} lapdMessage
    @return {Object} Message object
     */
    var actualDate;
    var matchMessageV2 = function (lapdMessage) {

        if (lapdMessage.indexOf('#DATE') !== -1) {

            /// #DATE 31/03/2014
            actualDate = lapdMessage.substr(6, 10);
            return false;

        } else if (lapdMessage.indexOf('LAPD MSG I', 23) === -1) {

            /// T|08:39:02.638 |N1| <- LAPD MSG I [C (P=0), N(S)=115, N(R)= 85] 02 01 E6 AA 08 02 80 5E 01 1E 02 82 82 1E 02 82 88
            return false;
        }

        return true;
    };

    /**
    Este método valida e separa os dados básicos da mensagem

    @method matchMessage
    @param {String} lapdMessage
    @return {Object} Message object
     */
    var matchMessage = function (lapdMessage) {

        /// T|04/11/2014 21:43:30.400|D51385 L2| -> LAPD MSG I [C (P=0), N(S)= 32, N(R)=  0] 02 01 40 00 08 02 3E CF 0F
        /// T|23/09/2014 12:38:34.867|D48088 L0| <- LAPD MSG I [C (P=0), N(S)=  0, N(R)= 15] 02 01 00 1E 08 02 49 2C 45 08 02 84 90 1E 02 82 88 1E 02 84 83

        if (lapdMessage.indexOf('LAPD MSG I', 35) === -1) {
            return null;
        } else {
            return true;
        }

    };

    /**
    Retorna o texto que representa a etapa da mensagem

    @method getStep
    @return {String}
     */
    Message.prototype.getStep = function () {

        var code = this.type;

        var steps = ['Call establishment message', 'Call information phase message', 'Call clearing messages', 'Miscellaneous messages'];

        var mapped = (code in stepsMap) ? stepsMap[code] : null;
        var step_text = (mapped >= 0) ? steps[mapped] : code;

        return step_text || 'UNKNOW';

    };

    /**
    Retorna a data e hora do source da mensagem

    @method getDatetime
    @return {String} Data e hora da mensagem
     */
    Message.prototype.getDatetime = function () {

        if (this.oldFormat) {
            return actualDate + ' ' + this.source.substr(2, 12);
        } else {
            return this.source.substr(2, 23);
        }
    };

    /**
    DeviceId utilizado na mensagem

    @method getDeviceId
    @return {String}
     */
    Message.prototype.getDeviceId = function () {

        if (this.oldFormat) {
            return this.source.substr(16, 2);
        } else {
            return this.source.substr(26, 6);
        }
    };

    /**
    Link utilizado na mensagem

    @method getLink
    @return {String}
     */
    Message.prototype.getLink = function () {

        if (this.oldFormat) {
            return this.source.substr(16, 2);
        } else {
            return this.source.substr(33, 2);
        }
    };

    /**
    Direção da mensagem

    -> (Send) para quando envia
    <- (Receive) para quando recebe

    @method getDirection
    @return {String}
     */
    Message.prototype.getDirection = function () {

        if (this.oldFormat) {
            return this.source.substr(20, 2);
        } else {
            return this.source.substr(37, 2);
        }
    };

    /**
    Pega os octetos com os dados da mensagem

    @method getData
    @return {String}
     */
    Message.prototype.getData = function () {

        if (!this.data) {

            this.data = (this.oldFormat) ? this.source.substr(64).split(' ') : this.source.substr(81).split(' ');

            for (var data_pos = 0, data_len = this.data.length; data_pos < data_len; data_pos++) {
                this.data[data_pos] = parseInt(this.data[data_pos], 16);
            }
        }

        return this.data;
    };

    /**
    Pega o CRV da mensagem

    @method getCallReference
    @return {String}
     */
    Message.prototype.getCallReference = function () {

        var data = this.getData();
        var crvLength = data[5];
        var crv = data.slice(6, 6 + crvLength);

        crv[0] = Util.numberToHex(Util.clearBit(crv[0], 8));
        if (crv[1]) {
            crv[1] = Util.numberToHex(crv[1]);
        }

        return crv.join('');
    };

    /**
    Pega o CRV da mensagem da outra ponta

    @method getCallReferenceOther
    @return {String}
     */
    Message.prototype.getCallReferenceOther = function () {

        var data = this.getData();
        var crvLength = data[5];
        var crv = data.slice(6, 6 + crvLength);

        crv[0] = Util.numberToHex(Util.setBit(crv[0], 8));

        return crv.join('');
    };

    /**
    Pega a direção da chamada com base no CRV

    @method getCallReferenceDirection
    @return {String}
     */
    Message.prototype.getCallReferenceDirection = function () {

        var data = this.getData();

        var test = data[6];

        return ((test & 0x80) >> 7) ? '<-' : '->';
    };

    /**
    Armazena o posicionamento dos information elements na mensagem
    */
    Message.prototype.info_elements_pos = [];

    /**
    Separa os information elements da mensagem

    @method getInformationElements
    @return {Object}
     */
    Message.prototype.getInformationElements = function () {

        this.info_elements_pos = [];

        var data = this.getData();
        var crvLength = data[5];
        var pos = 6 + crvLength + 1;

        var infos = data.slice(pos);

        if (infos.length) {

            this.infos = InformationElements.extractInfos(infos);

        }
        if (this.infos) {
            var pos_begin;
            for (var i = 0; i < this.infos.length; ++i) {
                pos_begin = pos;
                pos = ((this.infos[i].length === 1) ? pos_begin : pos_begin + this.infos[i].length);
                this.info_elements_pos.push([pos_begin, pos]);
                pos = pos + 1;
            }
        }

        return this.infos;
    };

    /**
    Decodifica os information elements da mensagem

    @method getDecodedInformationElements
    @return {Object}
     */
    Message.prototype.getDecodedInformationElements = function () {

        this.infos = this.infos || this.getInformationElements();
        var decodedInfos = [];
        var infoName;
        var infoData;
        var infoCode;

        for (var info_pos = 0, infos_len = this.infos.length; info_pos < infos_len; info_pos++) {

            info = this.infos[info_pos];
            infoCode = info[0];
            infoData = info.slice(1);

            infoName = InformationElements.types[infoCode] || infoCode;
            decodedInfos.push({name: infoName, value: InformationElements.decodeInfo(infoCode, infoData) });
        }

        return decodedInfos;
    };

    /**
    Pega o código do tipo da mensagem

    @method getType
    @return {String}
     */
    Message.prototype.getType = function () {

        if (this.type === 'number') {
            return this.type;
        }

        var data = this.getData();
        var crvLength = data[5];
        var pos = 6 + crvLength;
        this.type = data[pos];

        return this.type;
    };

    /**
    Pega o nome do tipo da mensagem

    @method getTypeName
    @return {String}
     */
    Message.prototype.getTypeName = function () {

        var type = this.getType();

        return messageTypes[type] || type;
    };

})(Khomp);
