(function (Khomp) {

    /**
    Khomp 2014 - R2 decoder

    @module r2
    @submodule r2-decoder
    @class R2
     */

    var R2 = Khomp.R2 = {
        logObj : null,
        calls : [],
        busyChannels : {},
        usedChannels : {},
        mfcs : {},
        lins : {},
        trimRegExp : /^\s+|\s+$/g
    };

    var __ = function () {
        return Khomp.translate.apply(null, arguments);
    };

    /**
    Limpa o objeto para um novo processamento

    @method clear
    @static
     */
    R2.clear = function () {
        this.logObj = null;
        this.logSource = '';
        this.calls = [];
        this.busyChannels = {};
        this.usedChannels = {};
        this.messageTypes = {};
        this.mfcs = {};
        this.lins = {};
    };

    /**
    Possui as possíveis etapas das chamadas

    @attribute callSteps
     */
    R2.callSteps = [{
            step : __('Tronco livre'),
            signalType : __('Livre'),
            signal : '10'
        }, {
            step : __('Tronco livre'),
            signalType : __('Livre (confirmação)'),
            signal : '10'
        }, {
            step : __('Ocupação do tronco'),
            signalType : __('Sinal de ocupação'),
            signal : '00'
        }, {
            step : __('Ocupação do tronco'),
            signalType : __('Sinal de confirmação de ocupação'),
            signal : '11'
        }, {
            /// Se não vier o sinal de atendimento (01), vai vir (10) que é sinal de desligamento.
            step : __('Atendimento da chamada'),
            signalType : __('Sinal de atendimento'),
            signal : '01'
        }, {
            step : __('Desligamento da chamada'),
            signalType : __('Sinal de desligar para trás'),
            signal : '11'
        }, {
            step : __('Desligamento da chamada'),
            signalType : __('Sinal de desligar para frente'),
            signal : '10'
        }, {
            step : __('Desligamento da chamada'),
            signalType : __('Sinal de confirmação de desconexão'),
            signal : '10'
        }, {
            step : __('Desligamento da chamada'),
            signalType : __('Sinal de desconexão forçada'),
            signal : '00'
        }, {
            step : __('Situações especiais'),
            signalType : __('Sinal de confirmação de desconexão forçada'),
            signal : '10'
        }, {
            step : __('Situações especiais'),
            signalType : __('Sinal de bloqueio'),
            signal : '10'
        }, {
            step : __('Situações especiais'),
            signalType : __('Sinal de falha'),
            signal : '11'
        }
    ];

    /**
    Sinais para frente

    @attribute forwardSignals
     */
    R2.forwardSignals = {
        /// Grupo I
        'I1' : __('Algarismo 1'),
        'I2' : __('Algarismo 2'),
        'I3' : __('Algarismo 3'),
        'I4' : __('Algarismo 4'),
        'I5' : __('Algarismo 5'),
        'I6' : __('Algarismo 6'),
        'I7' : __('Algarismo 7'),
        'I8' : __('Algarismo 8'),
        'I9' : __('Algarismo 9'),
        'I10' : __('Algarismo 0'),
        'I11' : __('Inserção de semi supressor de eco na origem'),
        'I12' : __('Pedido recusado ou indicação de trânsito internacional'),
        'I13' : __('Acesso a equipamento de teste'),
        'I14' : __('Inserção de semi supressor de eco de destino ou indicação de trânsito internacional'),
        'I15' : __('Fim de número ou indicação de que a chamada cursou enlace via satélite.'),

        /// Grupo II
        'II1' : __('Assinante comum'),
        'II2' : __('Assinante com tarifação especial'),
        'II3' : __('Equipamento de manutenção'),
        'II4' : __('Telefone público local'),
        'II5' : __('Telefonista'),
        'II6' : __('Equipamento de comunicação de dados'),
        'II7' : __('Telefone público interurbano'),
        'II8' : __('Chamada à cobrar'),
        'II9' : __('Assinante comum - Serv. entrante internacional'),
        'II10' : __('Reserva'),
        'II11' : __('Indicativo de chamada transferida'),
        'II12' : __('Reserva'),
        'II13' : __('Reserva'),
        'II14' : __('Reserva'),
        'II15' : __('Reserva')
    };

    /**
    Sinais para trás

    @attribute backwardSignals
     */
    R2.backwardSignals = {
        /// Grupo A
        'A1' : __('Enviar o próximo algarismo'),
        'A2' : __('Necessidade de semi-supressor de eco no destino ou enviar o primeiro algarismo enviado'),
        'A3' : __('Preparar para a recepção de sinais do grupo B'),
        'A4' : __('Congestionamento'),
        'A5' : __('Enviar categoria e identidade do assinante chamador'),
        'A6' : __('Reserva'),
        'A7' : __('Enviar o algarismo N-2'),
        'A8' : __('Enviar o algarismo N-3'),
        'A9' : __('Enviar o algarismo N-1'),
        'A10' : __('Reserva'),
        'A11' : __('Reserva'),
        'A12' : __('Reserva'),
        'A13' : __('Reserva'),
        'A14' : __('Reserva'),
        'A15' : __('Reserva'),
        /// Grupo B
        'B1' : __('Linha livre com tarifação'),
        'B2' : __('Linha de assinante ocupada'),
        'B3' : __('Linha do assinante com número mudado'),
        'B4' : __('Congestionamento'),
        'B5' : __('Linha de assinante livre sem tarifação'),
        'B6' : __('Linha de assinante livre com tarifação e colocar retenção sob controle do assinante chamado'),
        'B7' : __('Nível ou número vago'),
        'B8' : __('Linha do assinante fora de serviço para tráfego terminado'),
        'B9' : __('Reserva'),
        'B10' : __('Reserva'),
        'B11' : __('Reserva'),
        'B12' : __('Reserva'),
        'B13' : __('Reserva'),
        'B14' : __('Reserva'),
        'B15' : __('Reserva')

    };

    /**
    Tradução para mensagens dos algarismos

    @attribute numbersMap
     */
    R2.numbersMap = {
        'I1' : '1',
        'I2' : '2',
        'I3' : '3',
        'I4' : '4',
        'I5' : '5',
        'I6' : '6',
        'I7' : '7',
        'I8' : '8',
        'I9' : '9',
        'I10' : '0'
    };

    /**
    Armazena os padrões de mensagens

    @attribute patterns
     */
    R2.patterns = [];

    /**
    Inicia a extração dos dados de uma mensagem

    @method extractDataFromMessage
    @static
    @param {String} message
    @return {Object} Mensagem decodificada ou null se não é válida
     */
    R2.extractDataFromMessage = function (message) {

        var messageObj;

        if (!message) {
            return null;
        }

        if (message.indexOf('R2_SIG', 26) === -1) {
            return null;
        }
        
        /// Limpa caracteres de final de linha
        message = message.replace(/[\r\n]/g, '');
        var message_splited = message.split('|');
        var dev_channel = message_splited[3].split(' ');

        messageObj = {
            type : 'INDEFINIDO',
            source : message
        };

        messageObj.time = message_splited[1];
        messageObj.deviceId = dev_channel[0].replace(/^\s+|\s+$/g, '');
        messageObj.channel = dev_channel[1].replace(/^\s+|\s+$/g, '');
        messageObj.message = message_splited[4].replace(/^\s+|\s+$/g, '');
        messageObj.direction = (messageObj.message.indexOf('->') !== -1) ? '->' : '<-';
        
        message_splited = null;
        dev_channel = null;

        var call = this.busyChannels[messageObj.channel] || null;

        /// Ocupação de tronco

        /// A enviando do pedido inicial
        var envioPedidoSinalDeOcupacao = messageObj.message.indexOf('X:LIN -> 00');
        var envioPedidoSinalDeOcupacao2 = messageObj.message.indexOf('A:LIN -> 00');

        /// B recebendo o pedido de ocupação
        var recebeuPedidoDoSinalDeOcupacao = messageObj.message.indexOf('X:LIN <- 00');
        var recebeuPedidoDoSinalDeOcupacao2 = messageObj.message.indexOf('B:LIN <- 00');

        /// TODO:
        ///   - Desconsiderar as mensagens 'X:LIN -> 1001' quando antes do sinal de ocupação. Mensagem tronco livre boot do sistema.
        ///   - Desconsiderar as mensagens 'X:LIN <- 1001' quando antes do sinal de ocupação.
        ///   - Há um caso de exceção para ver no futuro.
        if ((messageObj.message.indexOf('X:LIN -> 10') !== -1 || messageObj.message.indexOf('X:LIN <- 10') !== -1) && call) {

            return null;
        }

        //! Podem ser enviados mais de um pedido de desconexão.
        var isNewCall = (
            envioPedidoSinalDeOcupacao !== -1 ||
            envioPedidoSinalDeOcupacao2 !== -1 ||
            recebeuPedidoDoSinalDeOcupacao !== -1 ||
            recebeuPedidoDoSinalDeOcupacao2 !== -1) ? true : false;

        if (isNewCall) {

            call = {
                id : this.calls.length + 1,
                Status : 'INDEFINIDO',
                recebendoNumA : false,
                recebendoNumB : true,
                deviceId : messageObj.deviceId,
                channel : messageObj.channel,
                From : '',
                To : '',
                Messages : []
            };

            this.calls.push(call);
            this.usedChannels[call.channel] = (this.usedChannels[call.channel]) ? this.usedChannels[call.channel] + 1 : 1;
            this.busyChannels[messageObj.channel] = call;

            /// Pedindo para conectar
            if (envioPedidoSinalDeOcupacao !== -1 || envioPedidoSinalDeOcupacao2 !== -1) {

                /// Este é o chamador
                call.User = 'A';
                call.Direction = '->';
            }

            /// Recebendo pedido de conexão
            if (recebeuPedidoDoSinalDeOcupacao !== -1 || recebeuPedidoDoSinalDeOcupacao2 !== -1) {

                call.User = 'B';
                call.Direction = '<-';

            }

        } else if (!call) {
            return null;
        }

        var call_msgs = call.Messages;
        var msgs_len = call_msgs.length;
        var prevMessage = (msgs_len > 0) ? call_msgs[msgs_len - 1] : null;
        var dataPositionLIN = messageObj.message.indexOf(':LIN ');

        call_msgs.push(messageObj);

        if (dataPositionLIN !== -1) {

            messageObj.data = messageObj.message.substr(dataPositionLIN + 8, 2);

            /// TODO:
            ///   - Considerar mensagens 'Long pulse' que não mensagens de tarifação enviadas pela operadora. É identificada quando é recebido um 'A:LIN <- 0001' e em seguida 'A:LIN <- 0101'
            if (prevMessage && prevMessage.message.indexOf('A:LIN <- 0001') !== -1 && messageObj.message.indexOf('A:LIN <- 0101') !== -1) {

                messageObj.type = 'Long pulse';
                prevMessage.type = 'Long pulse';
                return messageObj;

            }

        } else {

            var dataPositionMFC = messageObj.message.indexOf(':MFC ');

            if (dataPositionMFC !== -1) {
                messageObj.data = messageObj.message.substr(dataPositionMFC + 8);
                messageObj.type = this.translateData(messageObj.data);
            }
        }

        /// --------------------------------------------------------------------------------------
        /// Começa o MFC

        /// O envio de A e o recebimento de B têm o mesmo tratamento.
        if ((call.User === 'A' && messageObj.direction === '->') || (call.User === 'B' && messageObj.direction === '<-')) {

            if (messageObj.data === 'I15') {

                call.recebendoNumA = false;
                call.recebendoNumB = false;

            } else if (messageObj.data in this.numbersMap) {

                if (call.recebendoNumA) {

                    call.From += this.numbersMap[messageObj.data];

                } else if (call.recebendoNumB) {

                    call.To += this.numbersMap[messageObj.data];
                }
            }

            /// O recebimento de A e o envio de B têm o mesmo tratamento.
        } else if ((call.User === 'A' && messageObj.direction === '<-') || (call.User === 'B' && messageObj.direction === '->')) {

            if (messageObj.data === 'A5') {

                /// Recebendo A

                call.recebendoNumA = true;
                call.recebendoNumB = false;

            } else if (messageObj.data === 'A3') {

                /// Acabou identidade de B

                delete call.recebendoNumA;
                delete call.recebendoNumB;

            } else if (messageObj.data === 'A1') {

                call.recebendoNumA = false;
                call.recebendoNumB = true;
            }

        } else {
            console.error('ERRO. Não foram identificados A nem B');
        }

        return messageObj;
    };

    /**
    Método principal para iniciar a leitura dos dados de log do R2

    @method extractData
    @static
    @param {Mixed} logSource Linhas do arquivo de log em formato string ou array com as linhas
    @param {Function} callback
     */
    R2.extractData = function (logSource, callback) {

        logSource = (typeof logSource === 'string') ? logSource.split('\n') : logSource;

        var line;

        this.oldFormat = this.isOldFormat(logSource);

        if (this.oldFormat) {

            console.time('Converte para novo formato');
            logSource = this.convertR2v2Tov3(logSource);
            console.timeEnd('Converte para novo formato');
        }

        console.time('extractData R2');

        for (var i = 0, ln_len = logSource.length; i < ln_len; i++) {

            line = logSource[i];

            this.extractDataFromMessage(line);

        }
        console.timeEnd('extractData R2');
        callback(null, this.calls);

        return this.calls;

    };

    /**
    Faz a converão da informação de uma mensagem

    @method translateData
    @static
    @param {String} data Informação vinda na mensagem
    @return {String} Mensagem traduzida. Null no caso de não encontrada
     */
    R2.translateData = function (data) {

        if (data === '0') {

            return 'Silence';

        } else if (data in this.forwardSignals) {

            return this.forwardSignals[data];

        } else if (data in this.backwardSignals) {

            return this.backwardSignals[data];
        }

        return null;
    };

    /**
    Adiciona no R2.patterns um padrão de mensagens de sinalização.

    Se o 'pattern' já existir, é feito incremento do número de chamadas por 'pattern'

    @method addCallPattern
    @static
    @param {Object} call Chamada para adicionar ou posição no array de chamadas
     */
    R2.addCallPattern = function (call) {

        var msgs;
        var msg;
        var callPattern = [];
        var pattern_item;

        call = (typeof call === 'object') ? call : this.calls[call];

        for (var msg_pos = 0, msgs_len = call.Messages.length; msg_pos < msgs_len; msg_pos++) {

            msg = call.Messages[msg_pos].message;

            if (msg.indexOf(':LIN ') !== -1) {
                callPattern.push(msg);
            }
        }

        var found = false;

        for (var i = 0, patterns_len = R2.patterns.length; i < patterns_len; i++) {

            pattern_item = R2.patterns[i];
            found = true;

            if (pattern_item.messages.length === callPattern.length) {

                for (var j = 0, actual_msg_len = pattern_item.messages.length; j < actual_msg_len; j++) {

                    if (pattern_item.messages[j] !== callPattern[j]) {
                        found = false;
                        break;
                    }
                }
            } else {
                found = false;
            }

            if (found) {
                pattern_item.calls.push(call);
                break;
            }
        }

        if (!found) {
            R2.patterns.push({
                calls : [call],
                messages : callPattern
            });
        }

    };

    /**
    Faz a análise das mensagens decodificadas

    @method makeCallAnalysys
    @static
    @param {Function} callback
     */
    R2.makeCallAnalysys = function (callback) {

        console.time('R2.makeCallAnalysys');
        callback = callback || function () {};

        var calls = this.calls;
        var call;
        var step;
        var step_pos;
        var msg;
        var msg_str;
        var msg_pos;
        var call_pos;
        var foundStep;
        var callStepsLength = this.callSteps.length;
        var confirmDisconnect = ':LIN <- 10';
        var requestDisconnect = ':LIN -> 10';
        var total_calls = calls.length;
        var total_msgs;
        this.logObj = this.logObj || {};
        this.logObj.totalSuccess = 0;
        this.logObj.totalAlerts = 0;
        this.logObj.totalErrors = 0;
        this.logObj.messagesLengthByCall = [];
        var ultima;
        var prevMessage;
        var callPattern;
        var maxEndTimesLoopSteps;
        var lastStepAndHasMoreMessages = false;

        if (total_calls > 0) {
            this.logObj.Start = calls[0].Messages[0].time;
            this.logObj.Stop = calls[total_calls - 1].Messages[calls[total_calls - 1].Messages.length - 1].time;
        }

        for (call_pos = 0; call_pos < total_calls; call_pos++) {

            callPattern = [];
            lastStepAndHasMoreMessages = false;
            step_pos = 0;
            call = calls[call_pos];
            call.Status = 'ERROR';
            call.Steps = [];
            total_msgs = call.Messages.length;
            call.Start = call.Messages[0].time;
            call.Stop = call.Messages[total_msgs - 1].time;

            this.logObj.messagesLengthByCall.push(total_msgs);
            maxEndTimesLoopSteps = 10;

            /// Bits para o STATUS:
            var STATUS = 0;
            ///  000
            ///  || ` etapa ocupação
            ///  | ` atendimento
            ///   ` desligamento

            for (msg_pos = 0; msg_pos < total_msgs; msg_pos++) {

                foundStep = false;
                msg = call.Messages[msg_pos];
                prevMessage = (msg_pos > 1) ? call.Messages[msg_pos - 1] : null;

                msg_str = msg.message;
                switch (msg.data) {

                case '00':
                    STATUS = STATUS | 1 << 0;

                    break;
                case '01':
                    STATUS = STATUS | 1 << 1;
                    break;
                case '10':
                    STATUS = STATUS | 1 << 2;
                    break;
                }

                /// MFC messages
                if (msg_str.indexOf(':LIN') === -1) {

                    this.mfcs[msg.data] = (this.mfcs[msg.data]) ? this.mfcs[msg.data] + 1 : 1;

                    if (!msg.type) {
                        msg.type = __('Desconhecido');
                    }

                    this.messageTypes[msg.type] = (this.messageTypes[msg.type]) ? this.messageTypes[msg.type] + 1 : 1;

                    continue;
                }

                this.lins[msg.data] = (this.lins[msg.data]) ? this.lins[msg.data] + 1 : 1;

                if (prevMessage && msg.direction === prevMessage.direction && msg.data === prevMessage.data) {

                    ///   - Quando vem uma mensagem o mesmo valor com mesma direção considerar como sinal repetido
                    msg.step = step.step;
                    msg.type = step.signalType;
                    msg.type += ' (' + __('repetido') + ')';

                } else if (msg.type !== 'Long pulse') {

                    for (; step_pos < callStepsLength; step_pos++) {

                        ultima = (step) ? step.signalType : ' - ';
                        step = this.callSteps[step_pos];

                        var linMatch = step.signal;

                        if (msg.data === linMatch) {

                            foundStep = true;
                            step_pos++;

                            break;

                        }
                    }

                    if (foundStep === true) {

                        msg.step = step.step;
                        msg.type = step.signalType;

                    } else if (step_pos === callStepsLength) {
                        /// Se chegou no final das steps e não acabaram as mensagens.
                        step_pos = 3;
                        msg_pos--;
                        continue;
                    } else {
                        console.warn('Call: %s, Message: %s - Step não encontrada: %s (%s) - step_pos anterior: %s, maxEndTimesLoopSteps: %s', call_pos, msg_pos, msg.source, ultima, step_pos, maxEndTimesLoopSteps);
                    }

                }

                this.messageTypes[msg.type] = (this.messageTypes[msg.type]) ? this.messageTypes[msg.type] + 1 : 1;

                /// verifica as fases da chamada
            }

            if (STATUS >= 0x07) {
                call.Status = 'OK';
            } else if (STATUS === 0x05) {
                call.Status = 'ALERT';
            }

            switch (call.Status) {

            case 'OK':
                this.logObj.totalSuccess++;
                break;
            case 'ALERT':
                this.logObj.totalAlerts++;
                break;
            default:
                this.logObj.totalErrors++;
            }

            this.addCallPattern(call);
        }

        console.timeEnd('R2.makeCallAnalysys');
        callback(null, this.logObj);

    };

    /**
    Verifica se uma chamada tem todas as mensagens de uma lista

    @method callHasAllThisMessages
    @static
    @param {Array} messageTypes Lista com os tipos de mensagens. Ex: ['A:LIN -> 0001', 'B:LIN <- 1000']
    @param {Object} call
    @return {Boolean} Se tem todas ou não
     */
    R2.callHasAllThisMessages = function (messageTypes, call) {

        messageTypes = (typeof messageTypes === 'string') ? [messageTypes] : messageTypes;

        var found = 0;

        for (var i = 0; i < messageTypes.length; i++) {

            for (var j = 0; j < call.Messages.length; j++) {

                if (call.Messages[j].type === messageTypes[i]) {

                    found++;
                    break;
                }
            }
        }

        return (messageTypes.length === found) ? true : false;
    };

    /**
    Ordena os padrões de chamadas pelo total de chamadas que batem com o padrão

    @method sortPatternsByTotalCalls
    @static
     */
    R2.sortPatternsByTotalCalls = function () {

        R2.patterns.sort(function (a, b) {

            return (a.callIds.length === b.callIds.length) ? 0 :
            (a.callIds.length < b.callIds.length) ? -1 : 1;
        });
    };

    /**
    Procura as mensagens com um certo número de mensagens

    @method findCallsByNumberOfMessages
    @static
    @param {Number} n Número de chamadas
    @return {Array} Lista com as chamadas que tem o número de mensagens
     */
    R2.findCallsByNumberOfMessages = function (n) {

        var found = [];
        var calls = R2.calls;

        for (var i = 0; i < calls.length; i++) {
            if (n === calls[i].Messages.length) {
                found.push(calls[i]);
            }
        }
        return found;
    };

    /**
    Procura por chamadas com problemas

    @method findForCallsWithDecodeProblems
    @static
    @return {Array} Lista de chamadas com problemas
     */
    R2.findForCallsWithDecodeProblems = function () {

        /// Chamadas com mensagens de deviceId diferentes.
        var calls = R2.calls;
        var found = [];
        var msg;
        var call;
        var msgs;

        for (var i = 0; calls.length; i++) {

            found = false;
            call = calls[i];
            msgs = call.Messages;
            var deviceId = msgs[0].deviceId;

            /// Não pode ter uma chamada com mensagens de devices diferentes
            for (var j = 0, msgs_len = msgs.length; j < msgs_len; j++) {

                if (deviceId !== msgs[j].deviceId) {

                    console.log('DEVICES DIFERENTES NA MESMA CHAMADA >> ', msgs[j - 1].source, msgs[j].source);
                    found.push(call);
                    // break;
                }
            }
            // if (found.length > 0) {
            // console.log('Device diferente: ', i, calls[i]);
            // break;
            // }
        }
        return found;
    };

    /**
    Procura por chamadas com From e To iguais

    @method findCallsWhereFromIsTheSameOfTo
    @static
    @return {Array} Lista de chamadas
     */
    R2.findCallsWhereFromIsTheSameOfTo = function () {

        var calls = R2.calls;
        var call;
        var found = [];

        for (var i = 0, calls_len = calls.length; i < calls_len; i++) {

            call = calls[i];

            if (call.From === call.To) {
                found.push(call);
            }
        }
        return found;
    };

    /**
    Procura por chamadas com mensagens de dias diferentes. Para auxiliar no debug.

    @method findCallsWhereMessagesHasDifferentDays
    @static
    @return {Array} Lista de mensagens
     */
    R2.findCallsWhereMessagesHasDifferentDays = function () {

        var calls = R2.calls;
        var call;
        var date;
        var found = [];
        var msg;

        for (var i = 0, calls_len = calls.length; i < calls_len; i++) {

            call = calls[i];
            date = call.Start.substr(0, 10);

            for (var msg_pos = 0, msgs_len = call.Messages.length; msg_pos < msgs_len; msg_pos++) {

                msg = call.Messages[msg_pos];

                if (date !== msg.time.substr(0, 10)) {

                    found.push(call);
                    break;
                }
            }
        }
        return found;
    };

    /**
    Lista uma faixa de mensagens de uma chamada

    @method listRangeOfMessages
    @static
    @param {Object} call
    @param {Number} from Índice de início
    @param {Number} [to] Índice de fim. Se não for passado vai até o final das mensagens
    @return {Array} Lista com o fonte das mensagens na faixa desejada
     */
    R2.listRangeOfMessages = function (call, from, to) {

        from = from || 0;
        to = to || call.Messages.length;
        var list = [];

        for (var i = from; i < to; i++) {
            list.push(call.Messages[i].source);
        }
        return list;
    };

    /**
    Identifica se o formato do log é antigo (anterior à versão 3).

    @method isOldFormat
    @static
    @param {Mixed} logSource Conteúdo do arquivo de log em texto ou lista.
    @return {Number
     */
    R2.isOldFormat = function (logSource) {

        logSource = (typeof logSource === 'string') ? logSource.split('\n') : logSource;
        var line;

        for (var i = 0, lines_len = logSource.length; i < lines_len; i++) {

            line = logSource[i];

            if (line.indexOf('<-(RX)--') !== -1 || line.indexOf('--(TX)->') !== -1) {
                return true;
            }

        }

        return false;
    };

    /**
    Converte o arquivo do formato antigo para o novo.

    @method convertR2v2Tov3
    @static
    @param {Mixed} fileSource String ou Array com as linhas do arquivo
    @return {Array} Lista com as linhas do novo arquivo.
     */
    R2.convertR2v2Tov3 = function (fileSource) {

        fileSource = (typeof fileSource === 'string') ? fileSource.split('\n') : fileSource;

        var regex = {

            RX_LIN : /T\|(.+)\|SOFTR2\s\s\|(.+)\s(.+)\|\s<-\(RX\)--\sABCD=(\d+)\s/,
            TX_LIN : /T\|(.+)\|SOFTR2\s\s\|(.+)\s(.+)\|\s--\(TX\)->\sABCD=(\d+)\s/,

            RX_MFC : /T\|(.+)\|SOFTR2\s\s\|(.+)\s(.+)\|\s<-\(RX\)--\s(.+)/,
            TX_MFC : /T\|(.+)\|SOFTR2\s\s\|(.+)\s(.+)\|\s--\(TX\)->\s(.+)/
        };

        var match;
        var newLines = [];
        var busyChannels = {};
        var element;
        var call;
        var time;
        var channel;
        var device;
        var signal;
        var signal_sub;

        var paddingLeft = function (value, length) {

            length = length || 0;

            return (length > value.toString().length) ? paddingLeft("0" + value, length) : value;
        };

        var regex_group = /(\w+)h\s\-\sGrupo\s(\w+)\s/;

        var translateSignalMFCv2 = function (signal) {
            // if (signal === '0h - Fim de MFC') {
            if (signal.indexOf('0h - F') !== -1) {
                return 0;
            }

            var match = regex_group.exec(signal);
            var group;

            if (match) {
                switch (match[2]) {
                case '1':
                    group = 'I';
                    break;
                case '2':
                    group = 'II';
                    break;
                default:
                    group = match[2];
                }
                return group + parseInt(match[1], 16);
            }

            return signal;
        };

        for (var i = 0, source_len = fileSource.length; i < source_len; i++) {

            if (fileSource[i].indexOf('R2_SIG', 26) !== -1) {
                newLines.push(fileSource[i]);
                continue;
            }

            for (var j in regex) {

                match = regex[j].exec(fileSource[i]);

                if (match) {

                    time = match[1];
                    device = match[2];
                    channel = 'C' + paddingLeft(match[3].substr(1), 3);
                    signal = match[4];
                    signal_sub = signal.substr(0, 2);

                    if (signal.length === 2) {
                        console.log('ERRO: SINAL MENOR ', signal);
                    }

                    call = busyChannels[channel];
                    element = call || 'X';

                    switch (j) {

                    case 'RX_LIN':

                        if (signal_sub === '00') {
                            element = 'B';
                        }

                        newLines.push('T|' + time + '|R2_SIG  |' + device + ' ' + channel + '| ' + element + ':LIN <- ' + signal);

                        break;

                    case 'TX_LIN':

                        if (signal_sub === '00') {
                            element = 'A';
                        }

                        newLines.push('T|' + time + '|R2_SIG  |' + device + ' ' + channel + '| ' + element + ':LIN -> ' + signal);
                        break;

                    case 'RX_MFC':

                        signal = translateSignalMFCv2(signal);
                        newLines.push('T|' + time + '|R2_SIG  |' + device + ' ' + channel + '| ' + element + ':MFC <- ' + signal);

                        break;

                    case 'TX_MFC':

                        signal = translateSignalMFCv2(signal);
                        newLines.push('T|' + time + '|R2_SIG  |' + device + ' ' + channel + '| ' + element + ':MFC -> ' + signal);

                        break;

                    }

                    busyChannels[channel] = element;

                    /// Parar o loop regex se encontrou.
                    break;

                }
            }
        }
        return newLines;
    };

})(Khomp);
