<?php
/// Khomp 2015 
/// Este script procura por textos para tradução no KWebLogViewer em arquivos js e tpl
/// Será gerado um arquivo para cada idioma na pasta KWebLogViewer/languages
/// Traduções já realizadas são reaproveitadas.

require_once  '../../constants.php';

ini_set('memory_limit','512M');

if ( ! function_exists('glob_recursive')) {
    
    function glob_recursive($pattern, $flags = 0) {
        $files = glob($pattern, $flags);

        foreach (glob(dirname($pattern).'/*', GLOB_ONLYDIR|GLOB_NOSORT) as $dir) {
            $files = array_merge($files, glob_recursive($dir.'/'.basename($pattern), $flags));
        }

        return $files;
    }
}

class TranslationCapture {
    
    public static $languages = array('pt-br', 'es-es', 'en-us');
    public static $default_language = 'pt-br';
    
    public static function capture_from($path) {
            
        $words_by_lang_map = array();

        foreach(TranslationCapture::$languages as $lang) {
            $words_by_lang_map[$lang] = array();
        }

        foreach (glob_recursive($path . "/*.{js,tpl}", GLOB_BRACE) as $filepath) {

            $path_parts = pathinfo($filepath);

            $file_extension = $path_parts['extension'];
            
            if (strpos($filepath, 'translation_') !== false) {
                /// Nao processar este arquivo.
                echo 'PULANDO ARQUIVO: ' . __FILE__;
                continue;
            }

            // echo "Analisando arquivo: $filepath\n";
            
            if ($file_extension === 'tpl') {
                $words = TranslationCapture::translation_capture_from_dom($filepath);
            } else if ($file_extension === 'js') {
                $words = TranslationCapture::translation_capture_from_js($filepath);
            }

            if (!empty($words) && is_array($words)) {
                foreach(TranslationCapture::$languages as $lang) {
                    $words_by_lang_map[$lang] = array_merge($words_by_lang_map[$lang], $words);
                }
            }
        }

        /// Ordenar e remover repetidos
        foreach(TranslationCapture::$languages as $lang) {
            $words_by_lang_map[$lang] = array_unique($words_by_lang_map[$lang]);
            ksort($words_by_lang_map[$lang]);
        }
        
        return $words_by_lang_map;
    }
    
    public static function translate($words_by_lang_map, $path) {
        
        /// Verificar se já está traduzido
        foreach(TranslationCapture::$languages as $lang) {

            $translated[$lang] = array();

            $filepath = $path . "/translation.{$lang}.js";

            if (file_exists($filepath)) {
                
                $file_content = trim(file_get_contents($filepath));
                $pos = strpos($file_content, '{');
                $rpos = strrpos($file_content, '}');

                if ($pos === false || $rpos === false) {
                    echo "JSON invalido";
                    exit();
                }
                
                //echo "Inicio: {$pos}, Fim: {$rpos}";

                $file_content = substr($file_content, $pos, $rpos-$pos+1);

                if ($file_content === false) {
                    echo "Substr error";
                    exit();
                }
                
                $translated[$lang] = json_decode($file_content, true);
                
                if ($translated[$lang] === null) {
                    echo 'ERROR JSON parse ' . $lang;
                    file_put_contents('para_json_parse.json', $file_content);
                    exit();
                }

            } else {
                echo "Arquivo nao encontrado: $filepath";
            }
        }

        foreach(TranslationCapture::$languages as $lang) {

            foreach($words_by_lang_map[$lang] as $key => $value) {
                
                if (isset($translated[$lang][$key]) && $translated[$lang][$key] !== "") {
                    $words_by_lang_map[$lang][$key] = $translated[$lang][$key];
                } else if ($lang !== TranslationCapture::$default_language) {
                    /// Limpar a tradução do texto quando o idioma é diferente do $default_language
                    $words_by_lang_map[$lang][$key] = '';
                }
            }
        }
        
        return $words_by_lang_map;
    }
    
    public static function translation_capture_from_dom($filepath) {
        $dom = new DOMDocument();
        $dom->load($filepath);

        $finder = new DomXPath($dom);
        $words_map = array();
        $classname = 'i18n';

        /// Pega todos elementos com a classe i18n
        $elements = $finder->query("//*[contains(concat(' ', normalize-space(@class), ' '), ' $classname ')]");

        if (!empty($elements)) {
            foreach($elements as $element) {
                /// Get title
                $title = $element->getAttribute('title');
                if ( !empty( $title ) ) {
                    $words_map[$title] = $title;
                }
                
                /// Key translation
                $key = $element->getAttribute('key');
                $texto = trim($element->nodeValue);
                $key = (empty($key)) ? $texto : $key;
                
                if ($key === '') continue;
                
                $words_map[$key] = $texto;
            }
            //echo "File: " . $filepath . " / Elements: " . print_r($words_map);
        }
        return $words_map;
    }

    public static function translation_capture_from_js($filepath) {

        $handle = fopen($filepath, 'r');
        $words_map = array();

        while (($buffer = fgets($handle)) !== false) {

            if (empty($buffer)) continue;

            /// TODO: ver se está pegando quando tem mais em uma linha

            /// Com aspas simples
            preg_match_all("/__\(\s*'(.*?)'\s*\)/", $buffer, $matches);

            if (!empty($matches) && !empty($matches[0])) {

                foreach($matches[1] as $match) {

                    $words_map[$match] = $match;
                }
            }

            /// Com aspas duplas
            preg_match_all('/__\(\s*"(.*?)"\s*\)/', $buffer, $matches);

            if (!empty($matches) && !empty($matches[0])) {

                foreach($matches[1] as $match) {

                    $words_map[$match] = $match;
                }
            }
        }

        fclose($handle);

        return $words_map;
    }

    public static function save_translated($words_by_lang_map, $path = '') {
        
        $new_files_content = array();
        
        foreach($words_by_lang_map as $lang => $words_map) {
            
            // $words_map = array_map('utf8_decode', $words_map);
            $fileAppend = '';
            $new_files_content[$lang] = array();
            foreach($words_map as $key => $value) {
                $new_files_content[$lang][] = "  \"" . ($key) ."\"" . ": \"". ($value) . "\"";
            }
            file_put_contents($path . 'translation.'.$lang.'.js', "Khomp.translations[\"" . $lang . "\"] = {\n" . implode(",\n", $new_files_content[$lang]) . "\n};\n");
        }
    }
}

$words_map = TranslationCapture::capture_from(KROOT . '/KWebLogViewer');

$words_map = TranslationCapture::translate($words_map, KROOT . '/KWebLogViewer/languages');

TranslationCapture::save_translated($words_map);
echo 'Finished!';

