<?php
    require_once '../constants.php';
    require_once KROOT . '/classes/KSystem.php';
    $app = KSystem::getInstance( );
    $app->redirectIfNotLogged( );

    require_once KROOT . '/smarty-libs/Smarty.class.php';
    require_once KROOT . '/topo.php';
    require_once KROOT . '/rodape.php';
    require_once KROOT . '/util.php';
    require_once KMONITOR . '/mapDevices.php';
    require_once KMONITOR . '/monitorService.php';
    require_once KMONITOR . '/monitorDevice.php';
    require_once KMONITOR . '/monitorLink.php';
    require_once KMONITOR . '/monitorChannel.php';
    require_once KMONITOR . '/languages/dictionary.php';
    if( IS_GATEWAY )
    {
        require_once KROOT . '/classes/KFeatures.php';
    }

    $smarty = new Smarty( );

    if( $app->hasPermission( 'monitoring', array( 'read', 'full' ) ) )
    {
        $location = isset( $_REQUEST['location'] ) ? $_REQUEST['location'] : NULL;
        if( isset( $location ) )
        {
            switch($location)
            {
                case 'services':
                    try
                    {
                        $monitor = new MonitorService( );
                    }
                    catch(Exception $e)
                    {
                        unset( $_SESSION['DeviceList'] );
                        $_SESSION['errorMessage'] = $app->translateErrMessage( $e->getMessage( ) );
                        MonitorService::checkAllServices( );
                    }
                    $smarty->assign( 'CONTENT', MonitorService::showServices( ) );
                    $smarty->assign( 'LEGEND', $app->translate( 'services' ) );
                    $smarty->assign( 'MODULE', 'Services' );
                    $smarty->assign( 'JSmodule', 'Services' );
                    break;

                case 'devices':
                    try
                    {
                        $monitor = new MonitorDevice( );
                    }
                    catch(Exception $e)
                    {
                        unset( $_SESSION['DeviceList'] );
                        $_SESSION['errorMessage'] = $app->translateErrMessage( $e->getMessage( ) );
                    }
                    $smarty->assign( 'CONTENT', MonitorDevice::showDevices( ) );
                    $smarty->assign( 'LEGEND', $app->translate( 'devices' ) );
                    $smarty->assign( 'MODULE', 'Devices' );
                    $smarty->assign( 'JSmodule', 'Devices' );
                    break;

                case 'links':
                    try
                    {
                        $monitor = new MonitorLink( );
                    }
                    catch(Exception $e)
                    {
                        unset( $_SESSION['DeviceList'] );
                        $_SESSION['errorMessage'] = $app->translateErrMessage( $e->getMessage( ) );
                    }
                    $smarty->assign( 'CONTENT', MonitorLink::showLinks( ) );
                    $smarty->assign( 'LEGEND', $app->translate( 'links' ) );
                    $smarty->assign( 'MODULE', 'Links' );
                    $smarty->assign( 'JSmodule', 'Links' );
                    break;

                case 'channels':
                    try
                    {
                        $monitor = new MonitorChannel( );
                    }
                    catch(Exception $e)
                    {
                        unset( $_SESSION['DeviceList'] );
                        $_SESSION['errorMessage'] = $app->translateErrMessage( $e->getMessage( ) );
                    }
                    $content = MonitorChannel::getMessages( );
                    $content .= MonitorChannel::getDevices( );
                    $smarty->assign( 'CONTENT', $content );
                    $smarty->assign( 'LEGEND', $app->translate( 'channels' ) );
                    $smarty->assign( 'MODULE', 'Channels' );
                    $smarty->assign( 'JSmodule', 'Channels' );
                    break;
                default:

                    if( IS_GATEWAY )
                    {
                        if( $location == 'naps' )
                        {
                            $content = "<div id='kgateway-monitor-nap-status-canvas'><table id='kgateway-monitor-nap-status-table' class='tabelaDevice zebra sortable'><thead><tr><th rowspan='2'>NAP</th><th rowspan='2'>{$app->translate( 'kgateway-monitor-nap-type' )}</th><th rowspan='2'>{$app->translate( 'kgateway-monitor-nap-status' )}</th><th colspan='3'>{$app->translate( 'channels' )}</th><th rowspan='2'>{$app->translate( 'kgateway-monitor-nap-details' )}</th></tr>";
                            $content .= "<tr><th>{$app->translate( 'fail' )}</th><th>{$app->translate( 'idle' )}</th><th>{$app->translate( 'call' )}</th></tr></thead>";
                            $content .= '<tbody></tbody></table><input type="hidden" value="naps" id="current-monitoring" /></div><script type="text/javascript">( function($) { kgatewayMonitorNapStatus(); }( jQuery ) );</script>';

                            $smarty->assign( 'CONTENT', $content );
                            $smarty->assign( 'LEGEND', $app->translate( 'kgateway-monitor-nap-status-legend' ) );
                            $smarty->assign( 'MODULE', 'NAPs' );
                            $smarty->assign( 'JSmodule', 'NAPs' );
                            break;
                        }
                        else if( $location == 'simcards' && KFeatures::checkFeature( KFeatures::GSM_SUPPORT ) )
                        {
                            $content = "<div id='kgateway-monitor-sim-cards-status-canvas'><table id='kgateway-monitor-sim-cards-status-table' class='tabelaDevice zebra sortable'><thead><tr><th>{$app->translate( 'kgateway-monitor-sim-card' )}</th><th>NAP</th><th>{$app->translate( 'kgateway-monitor-sim-card-status' )}</th>
                            <th>{$app->translate( 'kgateway-monitor-sms-status' )} </th>
                            <th>{$app->translate( 'kgateway-monitor-sim-card-minutes-left' )}</th><th>{$app->translate( 'kgateway-monitor-sim-card-minutes-configured' )}</th><th>{$app->translate( 'kgateway-monitor-sim-card-sms-left' )}</th><th>{$app->translate( 'kgateway-monitor-sim-card-sms-configured' )}</th><th>{$app->translate( 'kgateway-monitor-sim-card-details' )}</th></tr></thead>";
                            $content .= '<tbody></tbody></table><input type="hidden" value="simcards" id="current-monitoring" /></div><script type="text/javascript">( function($) { kgatewayMonitorSimCardsStatus(); }( jQuery ) );</script>';

                            $smarty->assign( 'CONTENT', $content );
                            $smarty->assign( 'LEGEND', $app->translate( 'kgateway-sim-cards-status-legend' ) );
                            $smarty->assign( 'MODULE', 'SIMCards' );
                            $smarty->assign( 'JSmodule', 'SIMCards' );
                            break;
                        }
                    }

                    try
                    {
                        $monitor = new MonitorDevice( );
                    }
                    catch(Exception $e)
                    {
                        unset( $_SESSION['DeviceList'] );
                        $_SESSION['errorMessage'] = $app->translateErrMessage( $e->getMessage( ) );
                    }
                    $smarty->assign( 'CONTENT', MonitorDevice::showDevices( ) );
                    $smarty->assign( 'LEGEND', $app->translate( 'devices' ) );
                    $smarty->assign( 'MODULE', 'Devices' );
                    $smarty->assign( 'JSmodule', 'Devices' );
                    break;
            }
        }

        $ajaxTo = isset( $_REQUEST['ajaxTo'] ) ? $_REQUEST['ajaxTo'] : NULL;
        if( isset( $ajaxTo ) )
        {
            require_once KROOT . '/json_util.php';
            require_once KMONITOR . '/ajax/includeAjaxFiles.php';
            if( function_exists( $AjaxFunctions[$ajaxTo] ) )
            {
                echo call_user_func_array( $AjaxFunctions[$ajaxTo], array( $_REQUEST ) );
            }
            else
            {
                echo "Undefined function {$ajaxTo}";
            }
            exit ;
        }
    }
    else
        $smarty->assign( 'CONTENT', $app->translate( 'noPermissionToThisPage' ) );

    $smarty->assign( 'MODULO_PAI', '..' );
    $smarty->assign( 'MODULO', '.' );

    $topo = Topo::printTopo( 'monitoring', $_SESSION['loginAccess']['configuration'], $_SESSION['loginAccess']['monitoring'], $_SESSION['loginAccess']['diagnostic'], $_SESSION['loginAccess']['administration'], '..' );
    $smarty->assign( 'TOP', $topo );

    $rodape = Rodape::printRodape( );
    $smarty->assign( 'FOOTER', $rodape );

    $SO = PHP_OS == 'WINNT' ? 'WIN' : 'LINUX';
    $smarty->assign( 'SO', $SO );

    $smarty->assign( 'DESVIO_X', '40' );
    $smarty->assign( 'DESVIO_Y', '300' );

    $smarty->assign( 'IS_GATEWAY', IS_GATEWAY );
    $smarty->assign( 'IS_CHANNEL', IS_CHANNEL );
    $smarty->assign( 'HAVE_LINK', true );
    if( IS_GATEWAY )
    {
        $smarty->assign( 'NAPS', $app->translate( 'kgateway-monitor-button-naps' ) );
        $smarty->assign( 'SIMCARDS', $app->translate( 'kgateway-monitor-button-sim-cards' ) );

        //TODO retirar a dependencia da classe KSistema do KWebConfig
        require_once KCONFIG . '/classes/classKSistema.php';
        global $sistema;
        if( !isset( $_SESSION['sis'] ) )
        {
            $sistema = new KSistema( $app->getLanguage( ) );
            $_SESSION['sis'] = serialize( $sistema );
        }
        else
        {
            $sistema = unserialize( $_SESSION['sis'] );
            $sistema->setLanguage( $app->getLanguage( ) );
        }
        $smarty->assign( 'HAVE_GSM', KFeatures::checkFeature( KFeatures::GSM_SUPPORT ) );
        $smarty->assign( 'HAVE_LINK', KFeatures::checkFeature( KFeatures::E1_SUPPORT ) );
    }
    else
    {
        $smarty->assign( 'HAVE_LINK', true );
    }

    $smarty->assign( 'DEVICES', $app->translate( 'devices' ) );
    $smarty->assign( 'DEVICE', $app->translate( 'device' ) );
    $smarty->assign( 'LINKS', $app->translate( 'links' ) );
    $smarty->assign( 'CHANNELS', $app->translate( 'channels' ) );
    $smarty->assign( 'SERVICES', $app->translate( 'services' ) );
    $smarty->assign( 'REFRESH', $app->translate( 'refresh' ) );
    $smarty->assign( 'REFRESH_DELAY', $app->translate( 'refreshDelay' ) );
    $smarty->assign( 'NOT_REFRESH', $app->translate( 'notRefresh' ) );
    $smarty->assign( 'DELAY1', $app->translate( 'delay1' ) );
    $smarty->assign( 'DELAY2', $app->translate( 'delay2' ) );
    $smarty->assign( 'DELAY3', $app->translate( 'delay3' ) );

    //attaching timestamp to prevent cache
    $smarty->assign( 'KWEBMONITOR_CSS_TS', filemtime( './estilos/kwebmonitor.css' ) );
    $smarty->assign( 'GERAL_JS_TS', filemtime( '../js/geral.js' ) );
    $smarty->assign( 'UTIL_JS_TS', filemtime( './js/util.js' ) );
    $smarty->assign( 'KWEBMONITOR_JS_TS', filemtime( './js/kwebmonitor.js' ) );
    $smarty->assign( 'GERAL_CSS_TS', filemtime( '../js/geral.css' ) );
    $smarty->assign( 'CUSTOM_CSS', ( !file_exists( '../css/custom.css' ) ) ? '' : '<link type="text/css" rel="stylesheet" href="../css/custom.css"/>' );
    $smarty->assign( 'KCHART_TS', filemtime( './js/kcharts.js' ) );
    $smarty->assign( 'JQPLOT_TS', filemtime( './js/jqplot/jquery.jqplot.min.js' ) );
    $smarty->assign( 'JQPLOT_JS_TS', filemtime( './js/jqplot/jquery.jqplot.min.js' ) );
    $smarty->assign( 'JQPLOT_PIE_JS_TS', filemtime( './js/jqplot/jqplot.pieRenderer.min.js' ) );
    $smarty->assign( 'JQPLOT_CSS_TS', filemtime( './estilos/jquery.jqplot.min.css' ) );
    $smarty->assign( 'JQPLOT_BAR_RENDER', filemtime( './js/jqplot/jqplot.barRenderer.js' ) );
    $smarty->assign( 'JQPLOT_HIGHLIGHTER', filemtime( './js/jqplot/jqplot.highlighter.js' ) );
    $smarty->assign( 'JQPLOT_AXIS_RENDER', filemtime( './js/jqplot/jqplot.categoryAxisRenderer.min.js' ) );
    $smarty->assign( 'JQPLOT_POINT_LABELS', filemtime( './js/jqplot/jqplot.pointLabels.min.js' ) );
    $smarty->assign( 'JQUERY_JS_TS', filemtime( '../js/jquery/jquery.js' ) );
    $smarty->assign( 'CHOSEN_JS_TS', filemtime( '../js/jquery/chosen.jquery.min.js' ) );

    // Charts
    $smarty->assign( 'BACK', $app->translate( 'back' ) );
    $smarty->assign( 'CHANNEL', $app->translate( 'channel' ) );
    $smarty->assign( 'CALLS', $app->translate( 'chart_calls' ) );
    $smarty->assign( 'REFRESH_CHARTS', $app->translate( 'refresh_charts' ) );
    $smarty->assign( 'CH_STATUS_CHART_TITLE', $app->translate( 'ch_status_chart_title' ) );
    $smarty->assign( 'INC_OUT_CHART_TITLE', $app->translate( 'inc_out_chart_title' ) );
    $smarty->assign( 'STATUS_CHART_TITLE', $app->translate( 'call_status_chart_title' ) );
    $smarty->assign( 'FAIL_CHART_TITLE', $app->translate( 'fail_chart_title' ) );
    $smarty->assign( 'OF', $app->translate( 'minichart_of' ) );
    $smarty->assign( 'LOADING', $app->translate( 'minichart_loading' ) );
    $smarty->assign( 'NODATA', $app->translate( 'minichart_nodata' ) );
    $smarty->assign( 'SUCCEED', $app->translate( 'minichart_succeed' ) );
    $smarty->assign( 'FAILED', $app->translate( 'minichart_failed' ) );
    $smarty->assign( 'NOTANSWERED', $app->translate( 'minichart_not_answered' ) );
    $smarty->assign( 'STATISTICS', $app->translate( 'statistics' ) );
    $smarty->assign( 'CHANNELSTATUS', $app->translate( 'channelStatus' ) );

    if( IS_GATEWAY )
    {
        $smarty->assign( 'JQUERY_UI_JS_TS', filemtime( '../js/jquery/jquery.ui.js' ) );
        $smarty->assign( 'UI_CSS_TS', filemtime( '../js/jquery/css/ui-khomp/jquery-ui-1.10.4.custom.min.css' ) );
        $smarty->assign( 'K_UI_CSS_TS', filemtime( '../js/jquery/css/k-ui.css' ) );
        $smarty->assign( 'COMPATIBILITY_JS_TS', filemtime( '../js/compatibility.js' ) );
        $smarty->assign( 'RPC_JS_TS', filemtime( '../js/jquery/rpc-0.2.1.min.js' ) );
        $smarty->assign( 'KGATEWAY_JS_TS', filemtime( 'js/kgateway.js' ) );
        $smarty->assign( 'MASKINPUT_JS_TS', filemtime( '../js/jquery/jquery.maskedinput-1.3.min.js' ) );
        $smarty->assign( 'TABLESORT_JS_TS', filemtime( '../js/jquery/jquery.tablesorter.min.js' ) );
        $smarty->assign( 'COOKIE_JS_TS', filemtime( '../js/jquery/jquery.cookie.js' ) );
    }

    $smarty->display( 'themes/default/default.tpl' );
