<?php
    require_once ('populator.php');
    require_once ('monitorService.php');
    require_once ('querist.php');

    class MonitorDevice
    {
        function __construct( )
        {
            if( Populator::isK3LConnected( ) )
            {
                $CountDevices = Populator::getDeviceCount( );
                $deviceList = null;
                for( $i = 0; $i < (int)$CountDevices; $i++ )
                {
                    $device = array( 'model' => '', 'board' => $i, 'serial' => '', 'status' => '', 'links' => array( ) );
                    $device = Populator::populateDevice( $device );
                    $deviceList[$i] = $device;
                }
                unset($_SESSION['DeviceMiniCharts']);
                $_SESSION['DeviceList'] = $deviceList;
                $_SESSION['errorMessage'] = MonitorService::checkMediaServerMessages( );
            }
            else
            {
                throw new Exception( 'k3lDown' );
            }
        }

        public static function showDevices( )
        {
            $app = KSystem::getInstance( );
            Global $image_dir;

            $alertMessage = $app->translate( MonitorService::getWarningMessage( ) );

            if( isset( $_SESSION['errorMessage'] ) && $_SESSION['errorMessage'] != '' )
            {
                $style1 = '';
                $style2 = 'displayNone';
            }
            elseif( isset( $alertMessage ) && $alertMessage != '' )
            {
                $style2 = '';
                $style1 = 'displayNone';
            }
            else
            {
                $style1 = 'displayNone';
                $style2 = 'displayNone';
            }

            $buffer = '';
            $buffer .= "<div id='divErrorMessage' class='errorMessage {$style1}' >";
            $buffer .= "<img class='errorIco' src='{$image_dir}/imagens/error.png' />";
            $buffer .= '<div id="divErrorMessageLine">' . $_SESSION['errorMessage'] . '</div>';
            $buffer .= '</div>';

            $buffer .= "<div id='divAlertMessage' class='errorMessage {$style2}' >";
            $buffer .= "<img class='errorIco' src='{$image_dir}/imagens/error.png' />";
            $buffer .= '<div id="divAlertMessageLine">' . $alertMessage . '</div>';
            $buffer .= '</div>';
            $buffer .= '<input type="hidden" value="devices" id="current-monitoring" />';

            if( isset( $_SESSION['DeviceList'][0] ) && sizeof( $_SESSION['DeviceList'] > 0 ) )
            {
                $deviceList = $_SESSION['DeviceList'];
                $canResetDevice = MonitorDevice::canResetDevice( $deviceList );

                $buffer .= '<div id="devices">';
                $buffer .= '<table id="devicesTable" border="1" class="tabelaDevice zebra">';
                $buffer .= '<thead><tr>';
                $buffer .= '<th rowspan="2" class="board" scope="col">' . $app->translate( 'board' ) . '</th>';
                $buffer .= '<th rowspan="2" class="serial" scope="col">' . $app->translate( 'serial' ) . '</th>';
                $buffer .= '<th rowspan="2" class="model" scope="col">' . $app->translate( 'model' ) . '</th>';
                if( $canResetDevice && !IS_GATEWAY )
                    $buffer .= '<th rowspan="2" class="ip" scope="col">' . $app->translate( 'IP' ) . '</th>';
                $buffer .= '<th rowspan="2" class="status" scope="col">' . $app->translate( 'status' ) . '</th>';
                $buffer .= '<th colspan="3" class="channel" scope="col">' . $app->translate( 'channel' ) . '</th>';
                $buffer .= '<th rowspan="2" class="links" scope="col">' . $app->translate( 'links' ) . '</th>';

                $buffer .= '<th rowspan="2" class="actions" scope="col">' . $app->translate( 'options' ) . '</th>';

                $buffer .= '</tr>';
                $buffer .= '<tr>';
                $buffer .= '<th class="fail" scope="col">' . $app->translate( 'fail' ) . '</th>';
                $buffer .= '<th class="idle" scope="col">' . $app->translate( 'idle' ) . '</th>';
                $buffer .= '<th class="call" scope="col">' . $app->translate( 'call' ) . '</th>';
                $buffer .= '</tr></thead><tbody>';

                $arr_seriais = array( );

                for( $i = 0; $i < count( $deviceList ); $i++ )
                {
                    $device = $deviceList[$i];
                    $serial = $device['serial'];
                    $arr_seriais[] = $serial;

                    $buffer .= "<tr><td id='board-{$serial}' >" . $device['board'] . '</td>';
                    $buffer .= "<td id='serial-{$serial}' >" . $device['serial'] . '</td>';
                    $buffer .= "<td id='modelo-{$serial}' >" . $app->translate( trim( $device['model'] ) ) . '</td>';

                    if( $canResetDevice && !IS_GATEWAY )
                    {
                        $buffer .= "<td id='ip-{$serial}' >";
                        $buffer .= isset( $device['IP'] ) ? $device['IP'] : '-';
                        $buffer .= '</td>';
                    }

                    $status = $device['status'];
                    $buffer .= "<td id='status-{$serial}' ><b>{$status}</b></td>";

                    $buffer .= '<td >';
                    if( count( $device['channels'] ) == 0 )
                        $buffer .= '-';
                    else
                    {
                        $fail = $device['channels']['fail'];
                        $buffer .= "<span id='channels-{$serial}-fail' ><b>{$fail}</b></span>";
                    }
                    $buffer .= '</td><td >';
                    if( count( $device['channels'] ) == 0 )
                        $buffer .= '-';
                    else
                    {
                        $idle = $device['channels']['idle'];
                        $buffer .= "<span id='channels-{$serial}-idle' ><b>{$idle}</b></span>";
                    }
                    $buffer .= '</td><td >';
                    if( count( $device['channels'] ) == 0 )
                        $buffer .= '-';
                    else
                    {
                        $call = $device['channels']['call'];
                        $buffer .= "<span id='channels-{$serial}-call' ><b>{$call}</b></span>";
                    }
                    $buffer .= '</td>';

                    $buffer .= "<td id='links-{$serial}'>";
                    if( count( $device['links'] ) == 0 )
                        $buffer .= ' - ';
                    else
                    {
                        foreach( $device['links'] as $key => $value )
                        {
                            $buffer .= "<span id='links-{$serial}-link-{$key}'>";
                            $lnk = $device['type'] != 24 && $device['type'] != 3 ? 'Link ' . $key . ' : ' : 'Link ' . MonitorDevice::getRX( $key ) . ' : ';
                            $buffer .= $lnk;
                            $buffer .= '<label><b>' . $value['status'] . '</b></label>';
                            $buffer .= '</br></span>';
                        }
                    }

                    $buffer .= '<td class="displayFlex" id="options-' . $serial . '" >';
                    if( !IS_GATEWAY )
                    {
                        if( $canResetDevice && strpos( $device['model'], 'EBS' ) !== false && ($_SESSION['loginAccess']['monitoring'] == 'full') )
                        {
                            $buffer .= "<a href='#' onclick='resetDevice({$i});'>";
                            $buffer .= "<img class='marginTop12px' title='{$app->translate('resetDevice')}' src='$image_dir/imagens/reset.png' />";
                            $buffer .= '</a>';
                        }
                        else
                            $buffer .= "<a><img class='marginTop12px' title='{$app->translate('resetDeviceDenied')}' src='$image_dir/imagens/resetdenied.png' /></a>";
                    }
                    $buffer .= "<div class='miniChart' id='chart{$serial}' ></div>";
                    $buffer .= '</td>';
                    $buffer .= '</tr>';
                }
                $buffer .= '</tbody></table>';
                $buffer .= '</div>';
                $buffer .= "<script>updateDataDevices(true);</script>";
            }
            else
                $buffer .= $app->translate( 'noDevice' );

            return $buffer;
        }

        public static function getRX( $pos )
        {
            switch($pos)
            {
                case 0:
                    return '0 Rx0 ';
                case 1:
                    return '0 Rx1 ';
                case 2:
                    return '1 Rx0 ';
                case 3:
                    return '1 Rx1 ';
                default:
                    return '-';
            }
        }

        public function canResetDevice( $deviceList )
        {
            foreach( $deviceList as $k => $v )
            {
                if( strpos( $v['model'], 'EBS' ) !== false )
                    return true;
            }
            return false;
        }
    }