<?php
    require_once KROOT . '/util.php';
    require_once KCONFIG . '/classes/classKSistema.php';
    require_once KMONITOR . '/languages/dictionary.php';
    require_once KMONITOR . '/querist.php';
    header( 'Content-Type: text/html; charset=UTF-8' );

    class MonitorService
    {
        function __construct( )
        {
            Global $_SESSION;

            if( Populator::isK3LConnected( ) )
            {
                $_SESSION['errorMessage'] = MonitorService::checkMediaServerMessages( );
                MonitorService::checkAllServices( );
            }
            else
            {
                throw new Exception( 'k3lDown' );
            }
        }
        public static function getSpxProcessName()
        {
            //if(isset($_SESSION['softpbxProcess']))
                //return $_SESSION['softpbxProcess'];
            if(!is_file("/etc/asterisk/khomp.conf"))
                return false;
            $ret = array();
            $file = file_get_contents('/etc/asterisk/khomp.conf');
            preg_match("/\n\s*spx-process-name\s*=\s*([^;\s]*)/", $file, $ret);

            if(isset($ret[1]))
            {
                $_SESSION['softpbxProcess'] = $ret[1];
                return $ret[1];
            }
            else
            {
                unset($_SESSION['softpbxProcess']);
                return false;
            }
        }

        public static function checkAllServices( )
        {
            if( PHP_OS == 'WINNT' )
            {
                $service['Khomp Integrated Boot System'] = MonitorService::checkService( 'kibs' );
                $service['Khomp Log Server'] = MonitorService::checkService( 'klogserver' );
                $service['Khomp Media Server'] = MonitorService::checkService( 'kmpserver' );
                $service['Khomp API Server'] = MonitorService::checkService( 'K3LServer' );
                $service['Khomp Query Server'] = MonitorService::checkService( 'KQueryServer' );
				if ( win32_query_service_status( 'KLicenseServer' ) != 1060 )
		        {
		            $service['Khomp License Server'] = MonitorService::checkService("KLicenseServer");
		        }
            }
            else
            {
                $service['Khomp Integrated Boot System'] = MonitorService::checkService( 'kibs' );
                $service['Khomp Log Server'] = MonitorService::checkService( 'klogserver' );
				if (is_file("/usr/sbin/klicenseserver"))
		        {
		            $service['Khomp License Server'] = MonitorService::checkService("klicenseserver");
		        }

                if( is_file( '/usr/sbin/kmpserver' ) )
                    $service['Khomp Media Server'] = MonitorService::checkService( 'kmpserver' );
                elseif( MonitorService::checkPID( 'asterisk' ) )
                    $service['SoftPBX']['Asterisk'] = true;
                elseif( MonitorService::checkPID( 'freeswitch' ) )
                    $service['SoftPBX']['FreeSWITCH'] = true;
                elseif( MonitorService::getSpxProcessName() and MonitorService::checkPID($_SESSION['softpbxProcess']))
                    $service['SoftPBX'][$_SESSION['softpbxProcess']] = true;
                else
                    $service['SoftPBX'] = false;

                $service['Khomp API Server'] = MonitorService::checkService( 'k3lserver' );
                $service['Khomp Query Server'] = MonitorService::checkService( 'kqueryserver' );

                if( is_file( '/usr/sbin/kgateway' ) )
                    $service['Khomp Gateway'] = MonitorService::checkService( 'kgateway' );
            }
            $_SESSION['ServiceList'] = $service;
        }

        public static function checkEssentialServiceAvailable( )
        {
            if( PHP_OS == 'WINNT' )
                return MonitorService::checkService( 'k3lserver' ) && MonitorService::checkService( 'kqueryserver' ) && MonitorService::checkService( 'kmpserver' );
            else
            {
                if( is_file( '/usr/sbin/kmpserver' ) )
                    return MonitorService::checkService( 'k3lserver' ) && MonitorService::checkService( 'kqueryserver' ) && MonitorService::checkService( 'kmpserver' );
                elseif( MonitorService::checkPID( 'asterisk' ) )
                    return MonitorService::checkService( 'k3lserver' ) && MonitorService::checkService( 'kqueryserver' ) && MonitorService::checkPID( 'asterisk' );
                elseif( MonitorService::checkPID( 'freeswitch' ) )
                    return MonitorService::checkService( 'k3lserver' ) && MonitorService::checkService( 'kqueryserver' ) && MonitorService::checkPID( 'freeswitch' );
            }
        }

        public static function isSoftPBXActive( )
        {
            if( PHP_OS == 'WINNT' )
                return MonitorService::checkService( 'kmpserver' );
            else
            {
                if( is_file( '/usr/sbin/kmpserver' ) )
                    return MonitorService::checkService( 'kmpserver' );
                elseif(MonitorService::getSpxProcessName())
                {
                    if(MonitorService::checkPID($_SESSION['softpbxProcess']))
                        return true;
                    return false;
                }
                elseif( MonitorService::checkPID( 'asterisk' ) )
                    return true;
                elseif( MonitorService::checkPID( 'freeswitch' ) )
                    return true;
                return false;
            }
        }

        public static function getWarningMessage( )
        {
            switch (MonitorService::checkNonEssentialService())
            {
                case '0':
                    return 'LogServerInactive';
                case '1':
                    return 'KibsInactive';
                case '2':
                    return 'KibsAndLogServerAreInactive';
                default:
                    return '';
            }
        }

        public static function checkNonEssentialService( )
        {
            $log = MonitorService::checkService( 'klogserver' );
            $kibs = MonitorService::checkService( 'kibs' );

            if( !$log && !$kibs )
                return '2';
            else
            {
                if( !$log )
                    return '0';
                if( !$kibs )
                    return '1';
                return '3';
            }
        }

        public static function getProcessName( $service )
        {
            switch ($service)
            {
                case 'Khomp Media Server':
                    return 'kmpserver';
                case 'Khomp Integrated Boot System':
                    return 'kibs';
                case 'Khomp Log Server':
                    return 'klogserver';
                case 'Khomp Query Server':
                    return 'kqueryserver';
				case 'Khomp License Server' :
                    return "klicenseserver";
                case 'Khomp API Server':
                    return 'k3lserver';
                case 'Asterisk':
                    return 'asterisk';
                case 'FreeSWITCH':
                    return 'freeswitch';
                case 'Khomp Gateway':
                    return 'kgateway';
                default:
                    Global $_SESSION;
                    if(isset($_SESSION['softpbxProcess']) and $service == 'SoftPBX')
                        return $_SESSION['softpbxProcess'];
                    return '-';
            }
        }

        public static function checkMediaServerMessages( )
        {
            $app = KSystem::getInstance( );
            Global $_SESSION;
            if( MonitorService::checkService( 'k3lserver' ) )
            {
                if( !MonitorService::isSoftPBXActive( ) )
                {
                    if( PHP_OS != 'WINNT' )
                    {
                        if( is_file( '/usr/sbin/kmpserver' ) )
                            return $app->translate( 'KMPDown' );
                        else
                            return $app->translate( 'softPBXDown' );
                    }
                    else
                        return $app->translate( 'KMPDown' );
                }
                else
                    return '';
            }
            else
                return $app->translate( 'k3lDown' );
        }

        public static function checkPID( $service )
        {
            return (bool) exec( 'ps cax | grep -i -w ' . $service );
        }

        public static function checkService( $service_name )
        {
            if( PHP_OS == 'WINNT' )
            {
                $scCheck = win32_query_service_status( $service_name );
                return (bool)$scCheck['ProcessId'];
            }
            else
                return (bool) exec( 'ps cax | grep -i -w ' . $service_name );
        }

        public static function showServices( )
        {
            Global $_SESSION, $image_dir;
            $app = KSystem::getInstance( );
            $alertMessage = $app->translate( MonitorService::getWarningMessage( ) );

            if( isset( $_SESSION['errorMessage'] ) && $_SESSION['errorMessage'] != '' )
            {
                $style1 = '';
                $style2 = 'displayNone';
            }
            elseif( isset( $alertMessage ) && $alertMessage != '' )
            {
                $style2 = '';
                $style1 = 'displayNone';
            }
            else
            {
                $style2 = 'displayNone';
                $style1 = 'displayNone';
            }

            $buffer = '';
            $buffer .= '<input type="hidden" value="services" id="current-monitoring" />';
            $buffer .= "<div class='errorMessage divErrorMessage $style1' >";
            $buffer .= "<img class='errorIco' src='$image_dir/imagens/error.png' />";
            $buffer .= '<div id="divErrorMessageLine">' . $_SESSION['errorMessage'] . '</div>';
            $buffer .= "</div>";

            $buffer .= "<div class='errorMessage divErrorMessage $style2' >";
            $buffer .= "<img class='errorIco' src='$image_dir/imagens/error.png' />";
            $buffer .= "<div id='divAlertMessageLine'>" . $alertMessage . "</div>";
            $buffer .= '</div>';

            $serviceList = $_SESSION['ServiceList'];
            $access = $_SESSION['loginAccess']['monitoring'];
            $buffer .= '<input type="hidden" id="lang" value="' . $app->getLanguage( ) . '" name="lang" />';
            $buffer .= "<input type='hidden' id='access' value='$access' name='access' />";
            $buffer .= "<input type='hidden' id='softpbxProcess' value='' name='access' />";

            $buffer .= '<table border=1 class="tabelaService zebra"><thead><tr>';
            $buffer .= '<th class="service">' . $app->translate( 'service' ) . '</th>';
            $buffer .= '<th class="status">' . $app->translate( 'status' ) . '</th>';
            $buffer .= '<th class="process">' . $app->translate( 'process' ) . '</th>';
            $buffer .= '<th class="options">' . $app->translate( 'options' ) . '</th></tr></thead><tbody>';

            $i = 0;

            foreach( $serviceList as $key => $value )
            {
                if( $key == 'SoftPBX' )
                {
                    if( isset( $_SESSION['softpbxProcess'] ))
                    {
                        $value = $serviceList['SoftPBX'][$_SESSION['softpbxProcess']];
                    }
                    elseif( isset( $serviceList['SoftPBX']['Asterisk'] ) )
                    {
                        $key = 'Asterisk';
                        $value = $serviceList['SoftPBX']['Asterisk'];
                    }
                    elseif( isset( $serviceList['SoftPBX']['FreeSWITCH'] ) )
                    {
                        $key = 'FreeSWITCH';
                        $value = $serviceList['SoftPBX']['FreeSWITCH'];
                    }
                }

                $buffer .= "<tr><td>{$key}</td><td>";
                $k = str_replace( ' ', '_', $key );
                if( $value == 0 )
                {
                    $buffer .= "<span id='status-{$k}'>";
                    $buffer .= "<input type='hidden' id='real-status-{$k}' value='false' />";
                    $buffer .= '<b>' . $app->translate( 'inactive' ) . '</b>';
                    $buffer .= '</span>';
                }
                else
                {
                    $buffer .= "<span id='status-{$k}'>";
                    $buffer .= "<input type='hidden' id='real-status-{$k}' value='true' />";
                    $buffer .= '<b>' . $app->translate( 'active' ) . '</b>';
                    $buffer .= '</span>';
                }
                $buffer .= '</td>';
                if( $key == 'SoftPBX' or $key == 'Asterisk' or $key == 'FreeSWITCH')
                    $buffer .= '<td><span id="pbxname">' . MonitorService::getProcessName( $key ) . '</span></td>';
                else
                    $buffer .= '<td>' . MonitorService::getProcessName( $key ) . '</td>';
                $buffer .= '<td>';

                if( PHP_OS == 'WINNT' )
                {
                    if( $_SESSION['loginAccess']['monitoring'] == 'full' )
                    {
                        if( $value == 0 )
                        {
                            $buffer .= "<span id='button-{$k}'>";
                            $buffer .= "<a onclick=\"startService('" . $key . "');updateDataServices(true);updateDataServices(true);\" >";
                            $buffer .= "<img class='cursorHand' title='" . $app->translate( 'startService' ) . "' src='$image_dir/imagens/play.png' border='0'></a>";
                            $buffer .= '</span>';
                        }
                        else
                        {
                            $buffer .= "<span id='button-{$k}'>";
                            $buffer .= "<a onclick=\"stopService('" . $key . "');updateDataServices(true);\" >";
                            $buffer .= "<img title='" . $app->translate( 'stopService' ) . "' class='cursorHand' src='$image_dir/imagens/stop.png' border='0' ></a>";
                            $buffer .= '</span>';
                        }
                    }
                    else
                        $buffer .= "<span id='button-{$k}'>-</span>";
                }
                else
                {
                    if( $_SESSION['loginAccess']['monitoring'] == 'full')
                    {
                        if($key != 'Asterisk' && $key != 'FreeSWITCH' && $key != 'SoftPBX')
                        {
                            $ret = shell_exec( 'whoami' );
                            if( strcasecmp( $ret, 'root' ) )
                            {
                                if( $value == 0 )
                                {
                                    $buffer .= "<span id='button-$k'>";
                                    $buffer .= "<a onclick=\"startService('" . $key . "');updateDataServices(true);updateDataServices(true);\" >";
                                    $buffer .= "<img title='" . $app->translate( 'startService' ) . "' class='cursorHand' src='$image_dir/imagens/play.png' border='0'></a>";
                                    $buffer .= '</span>';
                                }
                                else
                                {
                                    $buffer .= "<span id='button-$k'>";
                                    $buffer .= "<a onclick=\"stopService('" . $key . "');updateDataServices(true);updateDataServices(true);\" >";
                                    $buffer .= "<img title='" . $app->translate( 'stopService' ) . "' class='cursorHand' src='$image_dir/imagens/stop.png' border='0' ></a>";
                                    $buffer .= '</span>';
                                }
                            }
                            else
                                $buffer .= "<img src='$image_dir/imagens/noRoot.png' border='0' title='" . $app->translate( 'noPermission' ) . "' >";
                        }
                        else
                        {
                            $buffer .= "<span id='pbxnamebutton'><a href='#!' onclick=\"editPbxName();\">";
                            $buffer .= "<img src='$image_dir/imagens/edit.png' title='edit' /></a></span>";
                        }
                    }
                    else
                        $buffer .= "<span id='button-{$k}'>-</span>";
                }
                $buffer .= '</td>';
                $i++;
            }
            $buffer .= '</tr></tbody>';
            $buffer .= '</table>';

            return $buffer;
        }
    }
