<?php

    require_once KCONFIG . '/classes/KGatewayYaml.php';

    class KLicenseLimitsException extends Exception
    {
    }

    class KLicenseLimits
    {
        private static $limits = array();

        private static $limits_ebs = array(
            "e1" => 4,      // links
            "gsm" => 16,    // channels
            "link" => 30,   // channels
        );

        public function __construct( )
        {
            if( !isset( $_SESSION['KLicenseLimits::limits'] ) )
            {
                $output = array();

                $errcode = -1;

                exec( "/usr/sbin/khomp-sudo /etc/init.d/kgateway limits 2>&1", $output, $errcode );

                if( $errcode == 0 )
                {
                    if ( !is_string( $output[0] ) || $output[0] === '' )
                    {
                        throw new KLicenseLimitsException(
                            'license-manager-error-limits-not-available' );
                    }

                    $gateway_limits = explode( ",", $output[0] );

                    foreach( $gateway_limits as $limit )
                    {
                        $value = explode( "=", $limit );

                        if ( count( $value ) === 2 )
                        {
                            self::$limits[ $value[0] ] = intval( $value[1] );
                        }
                        else
                        {
                            trigger_error( "KLicenseLimits: could not use limit => " . $limit, E_USER_WARNING );
                        }
                    }

                    if ( count( self::$limits ) === 0 )
                    {
                        throw new KLicenseLimitsException(
                            'license-manager-error-limits-not-available' );
                    }

                    $_SESSION['KLicenseLimits::limits'] = self::$limits;
                }
            }
            else
            {
                self::$limits = $_SESSION['KLicenseLimits::limits'];
            }

            if(IS_CHANNEL)
            {
                self::$limits['PCAP_MAXIMUM_STORAGE_SIZE'] = 3*1024*1024*1024; //3GB
                self::$limits['EBS'] = 9999;
            }
        }

        /**
         * Get Limit of EBS
         */
        public function getEBSLimit()
        {
            return intval( self::$limits[ "EBS" ] );
        }

        /**
         * Get Limit of EBS E1
         */
        public function getEBSLimitE1()
        {
            return intval( self::$limits[ "EBS_E1" ] );
        }

        /**
         * Get Limit of EBS
         */
        public function getEBSLimitGSM()
        {
            return intval( self::$limits[ "EBS_GSM" ] );
        }

        /**
         * Check EBS E1 Link limits
         *
         * @param int number of links E1
         */
        public function checkEBSE1LinkLimits( $links )
        {
            if ( intval( self::$limits[ "EBS_E1" ] ) === 0 )
            {
                if ( $links > 0 )
                {
                    throw new KLicenseLimitsException( 'license-manager-error-exceeds-e1-links' );
                }

                return;
            }

            $limit = ( intval( self::$limits[ "EBS_E1" ] ) *
                ( intval( self::$limits[ "E1_CHANNELS" ] ) /
                    ( intval( self::$limits[ "EBS_E1" ] ) * self::$limits_ebs[ "link" ] ) ) );

            if ( $links > $limit )
            {
                throw new KLicenseLimitsException( 'license-manager-error-exceeds-e1-links' );
            }
        }

        /**
         * Get Limit of VoIP Calls TRANSCODING_ON
         */
        public function getVoIPCallLimitTRANSCODING()
        {
            return intval( self::$limits["VOIP_CALLS_TRANSCODING_ON"] );
        }

        /**
         * Check EBS type limits
         *
         * @param string EBS type (e1 or gsm)
         * @param int available
         */
        public function checkEBSLimits( $type, $available )
        {
            if ( $type !== "e1" && $type !== "gsm" )
            {
                throw new KLicenseLimitsException( 'license-manager-error-upload' );
            }
            if ( is_int($available) === false )
            {
                throw new KLicenseLimitsException( 'license-manager-error-upload' );
            }

            if ( $available > self::$limits_ebs[ $type ] )
            {
                throw new KLicenseLimitsException(
                    'license-manager-error-exceeds-ebs-'.$type.'-channels' );
            }
        }

        /**
         * Check Gateway Pure Model limits
         *
         * @see KGatewayYaml::KMG* for KMG models
         *
         * @param int number of channels E1
         * @param int number of channels GSM
         * @param int number of VoIP calls
         */
        public function checkGatewayLimits(
            $e1_channels,
            $gsm_channels,
            $voip_calls,
            $kgateway )
        {
            if ( ( is_int($e1_channels) && is_int($gsm_channels) && is_int($voip_calls) ) === false )
            {
                throw new KLicenseLimitsException( 'license-manager-error-upload' );
            }

            $transcoding = true;

            if ( $kgateway instanceof KGatewayYaml )
            {
                $transcoding = $kgateway->checkSettingsSIPTranscoding();
            }

            $this->checkModelLimits( $e1_channels, $gsm_channels, $voip_calls, $transcoding );
        }

        /**
         * Check Gateway Model limits
         *
         * @param int number of channels E1
         * @param int number of channels GSM
         * @param int number of VoIP calls
         * @param bool transcoding enabled
         */
        private function checkModelLimits(
            $e1_channels,
            $gsm_channels,
            $voip_calls,
            $transcoding )
        {
            $_transcoding = "OFF";

            if ( $transcoding === true )
            {
                $_transcoding = "ON";
            }

            if ( $e1_channels > intval( self::$limits["E1_CHANNELS"] ) )
            {
                throw new KLicenseLimitsException( 'license-manager-error-exceeds-e1-channels' );
            }
            if ( $gsm_channels > intval( self::$limits["GSM_CHANNELS"] ) )
            {
                throw new KLicenseLimitsException( 'license-manager-error-exceeds-gsm-channels' );
            }
            if ( ( $e1_channels + $gsm_channels ) > intval( self::$limits["CHANNELS"] ) )
            {
                throw new KLicenseLimitsException( 'license-manager-error-exceeds-max-channels' );
            }
            if ( $voip_calls > intval( self::$limits["VOIP_CALLS_TRANSCODING_".$_transcoding] ) )
            {
                throw new KLicenseLimitsException( 'license-manager-error-exceeds-voip-calls' );
            }
        }

        /**
         * Get Limit of PCAP storage
         */
        public function getPacketCaptureStorageLimit()
        {
            return intval( self::$limits[ "PCAP_MAXIMUM_STORAGE_SIZE" ] );
        }

    }

