#!/bin/bash -l

###############################################################################

REVISION_NUMB="$Rev: 9271 $"
REVISION_DATE="$LastChangedDate: 2017-01-25 19:42:31 -0200 (Wed, 25 Jan 2017) $"

export PATH=${PATH}
#DEFAULT_LINE_STRING="-----------------------------------------------------------------------------"
DEFAULT_LINE_STRING=""
CHARSET_TRANSLATION="iconv -f 'UTF-8'"

if ! which 'iconv' &> /dev/null
then
    CHARSET_TRANSLATION="cat"
fi

PROG_NAME="$0"

FIX_TMP_DIR="/tmp/.kwebportal-log-tmpdir.txt"
LOG_DIR="/var/log/khomp"
LOG_SOFTPBX="${LOG_DIR}/softpbx.log"
LOGFILE="${LOG_DIR}/diagnostic.log"
DIALPLAN="${LOG_DIR}/dialplan.log"

SOFTPBX=0
SOFTPBXPROCNAME=asterisk
if pidof ncall &>/dev/null
then
    SOFTPBXPROCNAME=ncall
fi

############################## CODES ###########################################
SUCCESS=0 # 
ERROR=1   # Generic error
EROOT=2   # user != root
ENFIND=3  # Programs not found
EK3L=4    # k3lserver is not actived
ECHAN=5   # chan_khomp is not loaded
EMOD=6    # mod_khomp is not loaded
ESOFT=7   # Softpbx is not actived
###############################################################################
main_tabulate ()
{
    local count="$1"
    local tabs=""

    for ((i=0;i<count;i++))
    do
        tabs="${tabs}  "
    done

    while read LINE
    do
        echo "${tabs}${LINE}"
    done
}

msg_prefix ()
{
    echo "[$(date '+%d/%m %H:%M:%S')]"
}

msg_nonl ()
{
    (echo -n "$@" | ${CHARSET_TRANSLATION}) >&2

    (echo "$@" | ${CHARSET_TRANSLATION}) >> "${LOGFILE}"
}

msg ()
{
    (echo "$@" | ${CHARSET_TRANSLATION}) >&2

    (echo "$@" | ${CHARSET_TRANSLATION}) >> "${LOGFILE}"
}

###############################################################################
asterisk_remote ()
{
#   asterisk -r -q -m -x "$@"
    $SOFTPBXPROCNAME -r -x "$@"
}

freeswitch_remote ()
{
    ${FREESWITCH_BIN_PATH}/fs_cli -x "$@"
}

softpbx_remote()
{
    if [ ${SOFTPBX} == 1 ]
    then
        asterisk_remote "$@"
    elif [ ${SOFTPBX} == 2 ]
    then
        freeswitch_remote "$@"
    fi
}

softpbx_check()
{
    if pidof asterisk &>/dev/null || pidof ncall &>/dev/null
    then
        SOFTPBX=1
        if asterisk_remote "khomp summary" | grep -q "No such command"
        then
            msg "${PROG_NAME}: ERROR: \"chan_khomp\" is not loaded in the Asterisk"
            exit ${ECHAN}
        fi

    elif pidof freeswitch &>/dev/null
    then
        SOFTPBX=2
        if [ "${FREESWITCH_INSTALL_PATH}w" == "w" ]
        then
            FREESWITCH_INSTALL_PATH="/usr/local/freeswitch"
        fi

        if ! which 'freeswitch' &> /dev/null
        then
            FREESWITCH_BIN_PATH="${FREESWITCH_INSTALL_PATH}/bin"
        else
            FREESWITCH=`which freeswitch`
            FREESWITCH_BIN_PATH=`dirname ${FREESWITCH}`
            FREESWITCH_INSTALL_PATH=`echo ${FREESWITCH_BIN_PATH} | sed 's/\/bin[/]\?$//'`
        fi

        if [ ! -x ${FREESWITCH_BIN_PATH}/fs_cli ]
        then
            FS_PID=`pidof freeswitch`
            FREESWITCH=`ls -la /proc/${FS_PID}/exe | grep "\->" | sed 's/.* -> \(.*\)$/\1/'`
            FREESWITCH_BIN_PATH=`dirname ${FREESWITCH}`
            FREESWITCH_INSTALL_PATH=`echo ${FREESWITCH_BIN_PATH} | sed 's/\/bin[/]\?$//'`
        fi
    
        if freeswitch_remote "khomp summary" | grep -qi "not found"
        then
            msg "${PROG_NAME}: ERROR: \"mod_khomp\" is not loaded in the FreeSWITCH"
            exit ${EMOD}
        fi
    else
        msg "${PROG_NAME}: ERROR: SoftPBX is not running"
        exit ${ESOFT}
    fi
}

###############################################################################
asterisk_log_commands()
{
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo -n "${SOFTPBXPROCNAME} -V: " >> "${LOG_SOFTPBX}"
    ${SOFTPBXPROCNAME} -V >> "${LOG_SOFTPBX}"
    
    msg -n "."
    
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo -n "khomp revision: " >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp revision" | grep '^<K>' >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}">> "${LOG_SOFTPBX}"
    echo "khomp summary:" >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp summary" | grep '^<K>' | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}">> "${LOG_SOFTPBX}"
    echo "khomp dump pvt (disponível no channel 5.0):" >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp dump pvt" | grep '^<K>' | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."
    echo "${DEFAULT_LINE_STRING}"	 >> "${LOG_SOFTPBX}"
    echo "khomp network:" >> "${LOG_SOFTPBX}"
    checkconflict_ebs  | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}">> "${LOG_SOFTPBX}"
    echo "khomp sysinfo:" >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp sysinfo" | grep '^<K>' | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}">> "${LOG_SOFTPBX}"
    echo "manager show connected:" >> "${LOG_SOFTPBX}"
    asterisk_remote "manager show connected" | grep 'users connected' | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp links errors show:" >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp links errors show" | grep '^<K>' | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp links show:" >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp links show" | grep '^<K>' | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp channels show:" >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp channels show" | grep '^<K>' | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp channels statistics show:" >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp channels statistics show" | grep '^<K>' | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp calls show:" >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp calls show" | grep '^<K>' | main_tabulate 1 >> "${LOG_SOFTPBX}"


    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "Asterisk process limits: " >> "${LOG_SOFTPBX}"
    cat /proc/`pidof ${SOFTPBXPROCNAME}`/limits >> "${LOG_SOFTPBX}"

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "Asterisk process stat: " >> "${LOG_SOFTPBX}"
    cat /proc/`pidof ${SOFTPBXPROCNAME}`/stat >> "${LOG_SOFTPBX}"


    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "Khomp dump config: " >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp dump config" | grep '^<K>' | main_tabulate 1 >> "${LOG_SOFTPBX}"

    if [ `${SOFTPBXPROCNAME} -V |awk '{print $2}'|awk -F. '{print $1 $2}'` -ge 18 ]; then
        echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}" 
        echo "Asterisk core show settings: " >> "${LOG_SOFTPBX}"
        asterisk_remote "core show settings" | main_tabulate 1 >> "${LOG_SOFTPBX}"

        echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}" 
        echo "Asterisk core show channels verbose: " >> "${LOG_SOFTPBX}"
        asterisk_remote "core show channels verbose" | main_tabulate 1 >> "${LOG_SOFTPBX}"
    fi

    msg -n "."
}
        
freeswitch_log_commands()
{
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo -n "freeswitch -version: " >> "${LOG_SOFTPBX}"
    ${FREESWITCH_BIN_PATH}/freeswitch -version >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo -n "khomp revision: " >> "${LOG_SOFTPBX}"
    freeswitch_remote "khomp revision" >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}">> "${LOG_SOFTPBX}"
    echo "khomp summary:" >> "${LOG_SOFTPBX}"
    freeswitch_remote "khomp summary" | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}">> "${LOG_SOFTPBX}"
    echo "khomp sysinfo:" >> "${LOG_SOFTPBX}"
    freeswitch_remote "khomp sysinfo" | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}">> "${LOG_SOFTPBX}"
    echo "manager show connected:" >> "${LOG_SOFTPBX}"
    freeswitch_remote "manager show connected" | grep 'users connected' | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp show links errors:" >> "${LOG_SOFTPBX}"
    freeswitch_remote "khomp show links errors" | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp show links:" >> "${LOG_SOFTPBX}"
    freeswitch_remote "khomp show links" | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp show channels:" >> "${LOG_SOFTPBX}"
    freeswitch_remote "khomp show channels" | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp show statistics:" >> "${LOG_SOFTPBX}"
    freeswitch_remote "khomp show statistics" | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp show calls:" >> "${LOG_SOFTPBX}"
    freeswitch_remote "khomp show calls" | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."
}

generic_log_commands()
{
    if [ ${SOFTPBX} == 1 ]
    then
        asterisk_log_commands $@
    else
        freeswitch_log_commands $@
    fi
}

asterisk_log_start()
{
    #ROTATE
    if [ "w${LOGGER_ROTATE}" != "w--no-rotate" ]
    then
        echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
        echo "Rotating Asterisk log files" | main_tabulate 1 >> "${LOG_SOFTPBX}"
        asterisk_remote "logger rotate" | main_tabulate 2 >> "${LOG_SOFTPBX}"
    fi

    #LER NO KLOG.CFG SE e' PARA ATIVAR DEBUGGING OU ALL
    #level=`cat /etc/khomp/config/klog.cfg | grep -A 1 "\\[Channel\\]" | grep Value | cut -d= -f2`
    #HABILITA OS LOGS DA API/CHANNEL
    #echo "asterisk_remote \"khomp log disk just $level\"" | main_tabulate 1 >> "${LOG_SOFTPBX}"
    #asterisk_remote "khomp log disk just $level" | grep '^<K>' | main_tabulate 2 >> "${LOG_SOFTPBX}"
    #echo "asterisk_remote \"khomp log disk diagnostic\"" | main_tabulate 1 >> "${LOG_SOFTPBX}"


    # Ncall uses a different path for khomp.conf
    if [ $SOFTPBXPROCNAME = "ncall" ]
    then 
        KHOMP_CONF='/sistema/ncallprod/arquivos/asterisk/configBase/khomp.conf'
    else
        KHOMP_CONF='/etc/asterisk/khomp.conf'
    fi
    if [ ! -f $KHOMP_CONF ]; then
        touch $KHOMP_CONF
    fi

    if ! grep -q "^\s*log-to-disk\s*=\s*diagnostic" $KHOMP_CONF ;
    then
        if ! grep "\[globals\]" ${KHOMP_CONF};
        then
            echo "[globals]" >> ${KHOMP_CONF}
        fi
        #check if option exists
        if grep -q "log-to-disk" ${KHOMP_CONF};
        then
            sed -e '/log-to-disk/ s/^/log-to-disk=diagnostic;/' -i ${KHOMP_CONF} >> "${LOG_SOFTPBX}"
        else
            sed -e '/\[globals\]/a log-to-disk=diagnostic;' -i ${KHOMP_CONF}
        fi
    fi

    #asterisk_remote "module reload chan_khomp.so" | grep '^<K>' | main_tabulate 1 >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp log disk diagnostic" | grep '^<K>' | main_tabulate 2 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo >> "${LOG_SOFTPBX}"

    # HABILITA LOGS DO ASTERISK (TODO: CHECAR VERSAO)
    echo "asterisk_remote \"set verbose 99\"" | main_tabulate 1 >> "${LOG_SOFTPBX}"
    asterisk_remote "set verbose 99" | main_tabulate 2 >> "${LOG_SOFTPBX}"
    asterisk_remote "core set verbose 99" | main_tabulate 2 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo >> "${LOG_SOFTPBX}"

    asterisk_remote "dialplan show" > "${DIALPLAN}"

    msg -n "."
}

freeswitch_log_start()
{
    #ROTATE
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "Rotating FreeSWITCH log files" | main_tabulate 1 >> "${LOG_SOFTPBX}"
    freeswitch_remote "fsctl send_sighup" | main_tabulate 2 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo >> "${LOG_SOFTPBX}"

    # HABILITA LOGS DO FREESWITCH
    if [ ! -f /tmp/loglevel ]
    then
        fs_cli -x "fsctl loglevel" | sed 's/.* //' | tr -d '[[:space:]]' > /tmp/loglevel
    fi

    echo "freeswitch_remote \"fsctl loglevel debug\"" | main_tabulate 1 >> "${LOG_SOFTPBX}"
    freeswitch_remote "fsctl loglevel debug" | main_tabulate 2 >> "${LOG_SOFTPBX}"
    
    msg -n "."

    echo >> "${LOG_SOFTPBX}"

    cp "${FREESWITCH_INSTALL_PATH}/log/freeswitch.xml.fsxml" "${DIALPLAN}"

    msg -n "."
}

generic_log_start()
{
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "Enabling SoftPBX logs:" >> "${LOG_SOFTPBX}"

    if [ ${SOFTPBX} == 1 ]
    then
        asterisk_log_start $@
    else
        freeswitch_log_start $@
    fi
}

k3l_log_start()
{
    echo "Enabling K3L FullLog" >> "${LOG_SOFTPBX}"
    cat /etc/khomp/config/klog.cfg | sed 's/FullLog=false/FullLog=true/g' > /tmp/.start.klog.cfg.tmp
    mv /tmp/.start.klog.cfg.tmp /etc/khomp/config/klog.cfg

    /usr/sbin/kwebhelper reload
    
    echo "OK" | main_tabulate 1 >> "${LOG_SOFTPBX}"
}

main_log_start ()
{

    softpbx_remote "khomp log console just standard" > /dev/null

    # ACHA A ULTIMA PASTA DE LOGS CRIADA E VERSÃO DA API
    local k3l_summary=`softpbx_remote 'khomp summary' | grep 'K3L API'`

    local k3l_version=`expr "${k3l_summary}" : '.\+K3L API \([0-9.]\+\).\+' | cut -d. -f1-2`

    msg # empty line.

    msg -n "$(msg_prefix) Enabling logs (version ${k3l_version}), waiting..."

    # INFORMACOES DO SISTEMA, VERSOES, FERRAMENTAS

    echo "${PROG_NAME} (${REVISION_NUMB}/${REVISION_DATE})" > "${LOG_SOFTPBX}"
    
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "################################################################################" >> "${LOG_SOFTPBX}"    
    echo "[$(date '+%d/%m %H:%M:%S')] START" >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "uname -a: " >> "${LOG_SOFTPBX}"
    uname -a | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."
    
    if [ -f /etc/os-release ]
    then
        echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
        echo "cat /etc/os-release: " >> "${LOG_SOFTPBX}"
        cat /etc/os-release | main_tabulate 1 >> "${LOG_SOFTPBX}"
    fi

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "cat /etc/issue: " >> "${LOG_SOFTPBX}"
    cat /etc/issue | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."
    
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "gcc --version: " >> "${LOG_SOFTPBX}"
    gcc --version | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."
    
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "ldd --version: " >> "${LOG_SOFTPBX}"
    ldd --version | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    generic_log_commands
    
    msg -n "."
    
    generic_log_start

    msg -n "."

    k3l_log_start
   
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "################################################################################" >> "${LOG_SOFTPBX}"    
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"

    msg "."

    msg "$(msg_prefix) Logs enabled."
    msg # empty line.

    # Coloca nos logs a versão do firmware e do flex. Put on end of log to do time to log activation
    if [ ${SOFTPBX} == 1 ]
    then
        OUT=`asterisk_remote "khomp summary" | grep "GSM" | cut -d ']' -f 1 | cut -d '[' -f 3`

        list=$(echo $OUT | tr " " "\n")
        for i in $list
        do
             asterisk_remote "khomp send command $i 0 28 AT+CGMR?"
             asterisk_remote "khomp send command $i 0 28 ati9"
        done
    fi
}

###############################################################################
asterisk_log_stop()
{
    msg -n "."
    # DLITA LOG DE COMANDOS/EVENTOS DA API E DO CHANNEL DA KHOMP
    asterisk_remote "khomp log disk just standard" > /dev/null
    if [ $SOFTPBXPROCNAME == "asterisk" ]
    then
        sed -e '/^log-to-disk=diagnostic;$/d' -e 's/^\s*log-to-disk=diagnostic;//' -i /etc/asterisk/khomp.conf
    else # ncall
        sed -e '/^log-to-disk=diagnostic;$/d' -e 's/^\s*log-to-disk=diagnostic;//' -i /sistema/ncallprod/arquivos/asterisk/configBase/khomp.conf
    fi
    # Due to some unknown black magic, the stop command won't work if you
    # uncommnet the following line.
    #asterisk_remote "module reload chan_khomp.so"

    msg -n "."

    # DESABILITA LOGS DO ASTERISK
    asterisk_remote "set verbose 0" > /dev/null
    asterisk_remote "core set verbose 0" > /dev/null
    asterisk_remote "set debug 0" > /dev/null
    asterisk_remote "core set debug 0" > /dev/null
    

    msg -n "."
}

freeswitch_log_stop()
{
    msg -n "."

    # DESABILITA LOGS DO FREESWITCH
    if [ -f /tmp/loglevel ]
    then
        LOGLEVEL=`cat /tmp/loglevel`
        rm -rf /tmp/loglevel
        freeswitch_remote "fsctl loglevel $LOGLEVEL" > /dev/null
    else
        freeswitch_remote "fsctl loglevel notice" > /dev/null
    fi

    msg -n "."
}

generic_log_stop()
{
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "Disabling SoftPBX logs:" >> "${LOG_SOFTPBX}"

    if [ ${SOFTPBX} == 1 ]
    then
        asterisk_log_stop $@
    else
        freeswitch_log_stop $@
    fi
    
    echo "OK" | main_tabulate 1 >> "${LOG_SOFTPBX}"
}

k3l_log_stop()
{
    echo "Disabling K3L FullLog" >> "${LOG_SOFTPBX}"
    cat /etc/khomp/config/klog.cfg | sed 's/FullLog=true/FullLog=false/g' > /tmp/.stop.klog.cfg.tmp
    mv /tmp/.stop.klog.cfg.tmp /etc/khomp/config/klog.cfg

    /usr/sbin/kwebhelper reload

    echo "OK" | main_tabulate 1 >> "${LOG_SOFTPBX}"
}

main_log_stop ()
{
    msg # empty line.
    
    echo "[$(date '+%d/%m %H:%M:%S')] STOP" >> "${LOG_SOFTPBX}"

    msg -n "$(msg_prefix) Disabling logs..."
    
    generic_log_stop

    msg -n "."

    k3l_log_stop

    msg "."

    msg "$(msg_prefix) Logs disabled."
    msg # empty line.
    
    echo "################################################################################" >> "${LOG_SOFTPBX}"    
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
}

generic_log_copy_output ()
{
    cp -n $DIR/output.* $DUMPDIR/ 2> /dev/null
}

main_log_copy_output ()
{
    DUMPDIR="/var/log/khomp/outputs"
    if [ ! -d $DUMPDIR ]
    then
        mkdir $DUMPDIR
    fi
    DIR="/tmp"
    generic_log_copy_output
    DIR="/var/tmp/khomp/k3lserver"
    generic_log_copy_output
    DIR="/var/tmp/khomp/kibs"
    generic_log_copy_output
    DIR="/var/tmp/khomp/klogserver"
    generic_log_copy_output
    DIR="/var/tmp/khomp/kqueryserver"
    generic_log_copy_output
    
    if [ "$(ls -A $DUMPDIR)" == "" ]
    then
        rm -rf $DUMPDIR
    fi
}

aux_log_change_filename ()
{
    for filename in `ls -l $DIR`
    do
        if [ `echo $filename | grep -c "core"` -gt 0 ]
        then
            if [ `echo $filename | grep -c "-"` -eq 0 ]
            then
                PID=`echo $filename | cut -d '.' -f 2`
                DATE_TIME=`date '+%Y-%m-%d_%H-%M-%S'`
                mv $DIR/$filename $DIR/core.$DATE_TIME.$PID
            fi
        fi
    done
}

generic_log_gen_output ()
{
    for file in `ls -l $DIR`
    do
        if [ `echo $file | grep -c "core"` -gt 0 ]
        then
            output=`echo $file |  sed 's/core/output/g'`
            if [ ! -e $DIR/$output ]
            then
                gdb $DUMPAPP -c $DIR/$file -batch -x /tmp/cmd > $DIR/$output 2> /dev/null
            fi
        fi
    done
}

main_log_gen_output ()
{
    touch /tmp/cmd
    echo "set pagination off" >> /tmp/cmd
    echo "thread apply all bt full" >> /tmp/cmd

    if [ ${SOFTPBX} == 1 ]
    then
        DUMPAPP="asterisk"
        DIR="/var/tmp/asterisk"
        generic_log_gen_output
    else
        DUMPAPP="freeswitch"
    fi
    DIR="/tmp"
    generic_log_gen_output
    DUMPAPP="k3lserver"
    DIR="/var/tmp/khomp/k3lserver"
    aux_log_change_filename
    generic_log_gen_output
    DUMPAPP="kibs"
    DIR="/var/tmp/khomp/kibs"
    aux_log_change_filename
    generic_log_gen_output
    DUMPAPP="klogserver"
    DIR="/var/tmp/khomp/klogserver"
    aux_log_change_filename
    generic_log_gen_output
    DUMPAPP="kqueryserver"
    DIR="/var/tmp/khomp/kqueryserver"
    aux_log_change_filename
    generic_log_gen_output

    rm -rf /tmp/cmd
}

generic_log_clear_output ()
{
    rm -rf $DIR/core.*
    rm -rf $DIR/output.*
}

main_log_clear_output ()
{
    DIR="/tmp"
    generic_log_clear_output
    DIR="/var/tmp/khomp/k3lserver"
    generic_log_clear_output
    DIR="/var/tmp/khomp/kibs"
    generic_log_clear_output
    DIR="/var/tmp/khomp/klogserver"
    generic_log_clear_output
    DIR="/var/tmp/khomp/kqueryserver"
    generic_log_clear_output

    rm -rf /var/log/khomp/outputs
}

asterisk_status()
{
    local log_to_disk=`asterisk_remote "khomp log status"| grep "Enabled log-on-disk"`

    if echo ${log_to_disk} | grep -q "diagnostic"
    then
        return 0
    fi

    if [[ "${log_to_disk}" == *"errors"* ]] &&
       [[ "${log_to_disk}" == *"warnings"* ]] &&
       [[ "${log_to_disk}" == *"messages"* ]] &&
       [[ "${log_to_disk}" == *"events"* ]] &&
       [[ "${log_to_disk}" == *"commands"* ]] &&
       [[ "${log_to_disk}" == *"audio"* ]] &&
       [[ "${log_to_disk}" == *"modem"* ]] &&
       [[ "${log_to_disk}" == *"link"* ]] &&
       [[ "${log_to_disk}" == *"cas"* ]] &&
       [[ "${log_to_disk}" == *"general-debug"* ]] &&
       [[ "${log_to_disk}" == *"functions"* ]] &&
       [[ "${log_to_disk}" == *"stream-warning"* ]] &&
       [[ "${log_to_disk}" == *"states"* ]];
    then
        return 0
    fi

    return 1
}

freeswitch_status()
{
    local log_to_disk=`freeswitch_remote "fsctl loglevel"`

    if echo ${log_to_disk} | grep -q "DEBUG"
    then
        return 0
    fi
    
    return 1
}


checkconflict_ebs()
{
	#-------------------------------------------------------------------
	COUNT="2" # -c count : how many packets to send (arping)
	SUF_MAC_KHOMP="f8:03:32"
	RET_TMP_OUT="/tmp/.khomp_retorno_arping.tmp"
	ARPING_BIN="`which arping`"
	IFCONFIG_BIN="`which ifconfig`"
	ASTERISK_BIN="`which asterisk`"
	#-------------------------------------------------------------------
	if [ -z ${ARPING_BIN} ] ; then echo "Error: Command arping not found. Please install."; return ; fi
	#-----------------------------------------------
	OutDevices=$(${IFCONFIG_BIN} | grep "Link encap:Ether" -n1 | grep "inet end" -n1 | grep "Link encap:Ether" | awk -F\: '{print $2}' | awk '{print $1}' | sort | uniq)
	OutIpsEBS=$(cat /etc/khomp/config/devices.yaml|grep "IP:"|awk '{print $2}')
	#-----------------------------------------------

	for device in ${OutDevices} # INICIO Loop FOR em todos os devices (ifconfig)
	do
		for END_IP in ${OutIpsEBS} # INICIO Loop FOR em todos os IPS de EBS
		do
			${ARPING_BIN} -I $device ${END_IP} -c${COUNT} > ${RET_TMP_OUT} # salva o retorno do arping no diretorio temporario
			cat ${RET_TMP_OUT} | grep reply >/dev/null
			if [ ${?} == 0 ]; then # INICIO IF (Verifica se exite reply)

				CheckMacAdd() {	cat ${RET_TMP_OUT}| grep reply | tail -1 | grep -i "\[${SUF_MAC_KHOMP}" >/dev/null ;}
				# Atribui a variavel o ultimo ip que não contem o MacAddress Khomp (Possível MAC ADDRESS CONFLITANTE)
				MacConflito=$(cat $RET_TMP_OUT|grep reply|grep -v "\[${SUF_MAC_KHOMP}" |tail -1|awk -F\[ '{print $2}'|awk -F\] '{print $1}')

				CheckMacAdd
				if [ ${?} != '0' ] 
				then
					echo "Warning! EBS IP conflict: \"${END_IP}\", in the device: \"${device}\". MAC address conflicting: \"${MacConflito}\""
				else
					echo "EBS IP: \"${END_IP}\" working perfectly in device: \"${device}\""
				fi
			else 
				echo "EBS IP: \"${END_IP}\" unreachable in the device: \"${device}\""
			fi # FIM IF (Verifica se existe reply)
		done # Fim Loop FOR nos IPS
	done # FIM Loop FOR dos devices
}




generic_status()
{
    if [ ${SOFTPBX} == 1 ]
    then
        return $(asterisk_status $@)
    else
        return $(freeswitch_status $@)
    fi
}

k3l_status()
{
    if cat /etc/khomp/config/klog.cfg | grep -q "FullLog[ ]*=[ ]*true"
    then
        return 0
    fi

    return 1
}

main_status()
{
    return $(generic_status && k3l_status)
}

freswitch_log_path()
{
    msg `freeswitch_remote "khomp show log path"`
    return 0
}

main_log_path()
{
    if [ ${SOFTPBX} == 2 ]
    then
        freswitch_log_path
    fi
    return 0
}
###############################################################################
main_help()
{
    echo
    echo "$0: Enable logs tool."
    echo
    echo "Usage: $0 [options]"
    echo
    echo "Options:"
    echo
    echo "  --start        Start logs."
    echo "  --stop         Stop logs."
    echo "  --copyoutput   Copy all output files previously created."
    echo "  --genoutput    Generate output files from core files."
    echo "  --clearoutput  Clear core files and output files."
    echo "  --dumpstart    Activate dumpcheck daemon to monitor and generate outputs from new dumps"
    echo "  --dumpstop     Deactivate dumpcheck daemon to monitor and generate outputs from new dumps"
    echo "  --no-rotate    Do not rotate the ${SOFTPBXPROCNAME} logs. Ex: kdiagnostic_run.sh --no-rotate start"
    echo "  --help | -h    This help screen."
    echo
}
###############################################################################

ARG=""
ARGS=""
LOGGER_ROTATE='yes'

if [ $# == 1 ] && ([ "$1" == "start" ] || [ "$1" == "--start" ] || [ "$1" == "stop" ] || [ "$1" == "--stop" ])
then
    ARGS=$1
    file=/etc/khomp/config/klog.cfg
    ARGSFILE=`sed '/^\[SPX]/,/^\ *$/!d;//d' $file`
    if [ -z "$ARGFILE" ]
    then
        ARGSTMP=$ARGS
        ARGS=$ARGSFILE
        ARGS+=" "
        ARGS+=$ARGSTMP
    fi
else
    ARGS=$@
fi

for ARG in $ARGS
do
    if [ "$ARG" = "help" ] || [ "$ARG" = "--help" ] || [ "$ARG" = "-h" ]
    then
        main_help
        continue
    fi

    # VERIFICA SE EH SUPER-USUARIO
    if [ `whoami` != root ]
    then
        msg "${PROG_NAME}: ERROR: Only root can do that."
        exit ${EROOT}
    fi

    # VERIFICA SE HA SOFTPBX RODANDO 
    softpbx_check

    if [ "$ARG" == "copyoutput" ] || [ "$ARG" == "--copyoutput" ]
    then
        main_log_copy_output
        continue

    elif [ "$ARG" == "genoutput" ] || [ "$ARG" == "--genoutput" ]
    then
        main_log_gen_output
        continue

    elif [ "$ARG" == "clearoutput" ] || [ "$ARG" == "--clearoutput" ]
    then
        main_log_clear_output

    elif [ "$ARG" == "dumpstart" ] || [ "$ARG" == "--dumpstart" ] || [ "$ARG" == "MonitorDump=true" ]
    then
        pid=`pidof dumpcheck`
        if [ -f /usr/sbin/dumpcheck ] && [[ -z $pid ]]
        then
            /usr/sbin/dumpcheck &
        fi
        continue

    elif [ "$ARG" == "dumpstop" ] || [ "$ARG" == "--dumpstop" ] || [ "$ARG" == "MonitorDump=false" ]
    then
        pid=`pidof dumpcheck`
        if [ -f /usr/sbin/dumpcheck ] && [[ ! -z $pid ]]
        then
            killall -15 dumpcheck
        fi
        continue
    fi

    # VERIFICA SE TEM DEPENDENCIAS
    if ! which 'find' 'expr' &> /dev/null
    then
        msg "${PROG_NAME}: ERROR: Programs 'find' and 'expr' not found."
        msg
        exit ${ENFIND}
    fi

    if ! pidof k3lserver &>/dev/null
    then
        msg "${PROG_NAME}: ERROR: k3lserver is not running."
        exit ${EK3L}
    fi

    if [ "$ARG" == "checkip" ] || [ "$ARG" == "--checkip" ]
    then
        checkconflict_ebs
        continue
    fi

    # ASTERISK LOGGER ROTATE? 
    if [ "$ARG" = 'no-rotate' ] || [ "$ARG" = "--no-rotate" ] || [ "$ARG" == "RotateAsteriskLog=false" ]
    then
        LOGGER_ROTATE='--no-rotate'
        continue
    elif [ "$ARG" == "RotateAsteriskLog=true" ]
    then
        continue
    fi

    if [ "$ARG" == "start" ] || [ "$ARG" == "--start" ]
    then
        main_log_start
        continue

    elif [ "$ARG" == "stop" ] || [ "$ARG" == "--stop" ]
    then
        main_log_stop
        continue

    elif [ "$ARG" == "logpath" ] || [ "$ARG" == "--logpath" ]
    then
        main_log_path

    elif [ "$ARG" == "status" ] || [ "$ARG" == "--status" ]
    then
        if (main_status)
        then
            (echo "ACTIVE" | ${CHARSET_TRANSLATION}) 
        else
            (echo "INACTIVE" | ${CHARSET_TRANSLATION}) 
        fi
    
    else
        main_help
        exit ${ERROR}
    fi
done

exit ${SUCCESS}

