<?php
    require_once KCONFIG . '/classes/classSS7.php';

    $AjaxFunctions['showMTP2'] = 'showMTP2';
    //TODO: create two functions, one to add, and other to edit?
    $AjaxFunctions['addMTP2'] = 'showAddMTP2';
    $AjaxFunctions['editMTP2'] = 'showAddMTP2';
    $AjaxFunctions['saveMTP2'] = 'saveMTP2';
    $AjaxFunctions['removeMTP2'] = 'removeMTP2';

    $AjaxFunctions['showPointCodes'] = 'showPointCodes';
    $AjaxFunctions['addPointCode'] = 'showAddPointCode';
    $AjaxFunctions['editPointCode'] = 'showAddPointCode';
    $AjaxFunctions['savePointCode'] = 'savePointCode';
    $AjaxFunctions['removePointCode'] = 'removePointCode';

    $AjaxFunctions['showLinkSets'] = 'showLinkSets';
    $AjaxFunctions['addLinkSet'] = 'showAddLinkSet';
    $AjaxFunctions['editLinkSet'] = 'showAddLinkSet';
    $AjaxFunctions['saveLinkSet'] = 'saveLinkSet';
    $AjaxFunctions['removeLinkSet'] = 'removeLinkSet';

    $AjaxFunctions['showRoutes'] = 'showRoutes';
    $AjaxFunctions['addRoute'] = 'showAddRoute';
    $AjaxFunctions['editRoute'] = 'showAddRoute';
    $AjaxFunctions['removeRoute'] = 'removeRoute';
    $AjaxFunctions['saveRoute'] = 'saveRoute';

    $AjaxFunctions['showTimers'] = 'showTimers';
    $AjaxFunctions['saveTimers'] = 'saveTimers';

    $AjaxFunctions['showCircuitGroups'] = 'showCircuitGroups';
    $AjaxFunctions['addCircuitGroup'] = 'showAddCircuitGroup';
    $AjaxFunctions['editCircuitGroup'] = 'showAddCircuitGroup';
    $AjaxFunctions['saveCircuitGroup'] = 'saveCircuitGroup';
    $AjaxFunctions['removeCircuitGroup'] = 'removeCircuitGroup';

    /**
     * If has licenses warnings, return the <div> with them
     *
     * @return string
     */
    function _licenseWarnings( )
    {
        require_once KROOT . '/classes/KLicense.php';
        $buf = '';
        $manager = new KLicenseManager( );
        $app = KSystem::getInstance( );

        if( !$manager->anyLicenseHasType( KLicense::TYPE_ISUP ) )
        {
            $buf .= "<div class='license-warning'>{$app->translate( 'ss7-isup-license-error-license-required' )}</div><br />";
        }
        //TODO: count ISUP circuits and check if has licenses for all of them

        return $buf;
    }

    /**
     * Create HTML select
     *
     * Behaviours:
     *
     * @param array $items array (numeric)
     * @param int|string $selected selected value
     * @param string $options HTML options (will be appended to select tag)
     * @param boolean $addSelect Add SELECT option in beginning (if selected is
     * null)
     * @param boolean $useKeyAsValue Use array key as option value
     */
    function _createSelect( $items, $selected = null, $options = '', $addSelect = true, $useKeyAsValue = false )
    {
        $app = KSystem::getInstance( );
        $buf = "<select {$options}>";
        if( is_null( $selected ) && $addSelect )
        {
            $buf .= "<option value='<SELECT>'>{$app->translate( 'Select' )}</option>";
        }
        foreach( $items as $idx => $value )
        {
            if( $useKeyAsValue )
            {
                $select = ($idx === $selected) ? ' selected=\'selected\'' : '';
                $_v = $useKeyAsValue ? $idx : $value;
            }
            else
            {
                $select = ($value === $selected) ? ' selected=\'selected\'' : '';
                $_v = $useKeyAsValue ? $idx : $value;
            }
            $buf .= "<option value='{$_v}'{$select}>{$value}</option>";
        }
        $buf .= '</select>';
        return $buf;
    }

    /**
     * MTP2 main list
     */
    function showMTP2( )
    {
        global $image_dir;
        $app = KSystem::getInstance( );

        $buff = "<fieldset><legend>{$app->translate( 'ss7-mtp2' )}</legend>";
        $buff .= _licenseWarnings( );
        $buff .= '<div id="ss7-mtp2-content">';

        $ss7 = new SS7( );

        if( $ss7->countMtp2( ) > 0 )
        {
            $buff .= "<table class='ss7-list-table zebra'><thead>
                <tr>
                    <th width='25%'>{$app->translate( 'name' )}</th>
                    <th width='20%'>{$app->translate( 'ss7-mtp2-device' )}</th>
                    <th width='25%'>{$app->translate( 'ss7-location' )}</th>
                    <th width='20%'>{$app->translate( 'ss7-mtp2-passive' )}</th>
                    <th width='10%'>{$app->translate( 'options' )}</th>
                </tr>
            </thead><tbody>";

            foreach( $ss7->getAllMtp2() as $idx => $info )
            {
                $_name = array_keys( $info );
                $_name = $_name[0];
                $_data = &$info[$_name];

                $passive = $_data['Passive'] == 'true' ? $app->translate( 'yes' ) : $app->translate( 'no' );
                if( $_data['Address']  == "127.0.0.1")
                {
                    $location = $app->translate( 'ss7-location-local' );
                }
                else
                {
                    $location = "{$app->translate( 'ss7-location-remote' )} (IP: {$_data['Address']})";
                }

                $buff .= "<tr>";
                $buff .= "<td>{$_name}</td>";
                $buff .= "<td>K{$_data['Device']} - Link {$_data['Link']}</td>";
                $buff .= "<td>{$location}</td>";
                $buff .= "<td>{$passive}</td>";
                $buff .= "<td class='options'>";
                $buff .= "<img title='{$app->translate( 'edit' )}' alt='{$app->translate( 'edit' )}' src='{$image_dir}/imagens/edit.png' class='ss7-mtp2-edit cursorHand' />";
                $buff .= "<img title='{$app->translate( 'delete' )}' alt='{$app->translate( 'delete' )}' src='{$image_dir}/imagens/remove.png' class='ss7-mtp2-delete cursorHand' />";
                $buff .= "</td>";
                $buff .= "</tr>";
            }
            $buff .= "</tbody></table>";
        }
        else
            $buff .= "<div class='textAlignCenter'>{$app->translate( 'ss7-mtp2-empty-list' )}</div>";

        $buff .= "<br /><input value='{$app->translate( 'add' )}' class='ss7-mtp2-add floatRight' type='button' />";
        $buff .= '</fieldset></div>';

        return $buff;
    }

    /**
     * Point Code main list
     */
    function showPointCodes( )
    {
        global $image_dir;

        $app = KSystem::getInstance( );

        $buff = "<fieldset><legend>{$app->translate( 'ss7-pointcode' )}</legend>";
        $buff .= _licenseWarnings( );
        $buff .= '<div id=\'ss7-pointcode-content\'>';

        $ss7 = new SS7( );

        if( $ss7->countPointCodes( ) > 0 )
        {
            $buff .= "<table class='ss7-list-table zebra'><thead><tr><th width='40%'>{$app->translate( 'name' )}</th><th width='40%'>{$app->translate( 'ss7-pointcode' )}</th><th width='10%'>{$app->translate( 'options' )}</th></tr></thead><tbody>";
            foreach( $ss7->getAllPointCodes() as $idx => $info )
            {
                $_name = array_keys( $info );
                $_name = $_name[0];
                $_data = &$info[$_name];

                $buff .= "<tr>";
                $buff .= "<td>{$_name}</td>";
                $buff .= "<td>{$ss7->pointCodeToString( $_name, $_data, false )}</td>";
                $buff .= "<td class='options'>";
                $buff .= "<img title='{$app->translate( 'edit' )}' alt='{$app->translate( 'edit' )}' src='{$image_dir}/imagens/edit.png' class='ss7-pointcode-edit cursorHand' />";
                $buff .= "<img title='{$app->translate( 'delete' )}' alt='{$app->translate( 'delete' )}' src='{$image_dir}/imagens/remove.png' class='ss7-pointcode-delete cursorHand' />";
                $buff .= "</td>";
                $buff .= "</tr>";
            }
            $buff .= '</tbody></table>';
        }
        else
            $buff .= "<div class='textAlignCenter'>{$app->translate( 'ss7-pointcode-empty-list' )}</div>";

        $buff .= '<div class="floatRight" >';
        $buff .= "<br><input value='{$app->translate( 'add' )}' class='ss7-pointcode-add' type='button' />";
        $buff .= '</div>';
        $buff .= '</div></fieldset>';

        return $buff;
    }

    /**
     * Return the Timer form > td element as string
     *
     * @param yaml array loaded yaml
     * @param name string timer name
     * @param default int timer default value
     * @return string
     */
    function _timerFormTd( &$yaml, $name, $default )
    {
        $value = array_key_exists( $name, $yaml ) ? $yaml[$name] : $default;
        return "<td><label for='isup{$name}'>{$name}:</label></td><td><input class='selector5' value='{$value}' id='isup{$name}' name='{$name}' type='text' /></td>";
    }

    /**
     * ISUP Timers main list
     */
    function showTimers( )
    {
        $app = KSystem::getInstance( );

        $ss7 = new SS7( );
        $timers = $ss7->getAllTimers( );

        $buff = "<div id='ss7-isup-timers'><fieldset><legend>{$app->translate( 'ss7-isup-timers' )}</legend>";
        $buff .= _licenseWarnings( );

        $buff .= "<div>{$app->translate( 'ss7-isup-timer-time-unit-warning' )}</div><br />";

        $buff .= '<table class=\'zebra\' id=\'ss7-isup-timer-table\'><tbody>';

        $buff .= '<tr>';
        $buff .= _timerFormTd( $timers, 'T1', 45000 );
        $buff .= _timerFormTd( $timers, 'T5', 60000 );
        $buff .= '</tr>';

        $buff .= '<tr>';
        $buff .= _timerFormTd( $timers, 'T6', 35000 );
        $buff .= _timerFormTd( $timers, 'T7', 25000 );
        $buff .= '</tr>';

        $buff .= '<tr>';
        $buff .= _timerFormTd( $timers, 'T8', 13000 );
        $buff .= _timerFormTd( $timers, 'T9', 120000 );
        $buff .= '</tr>';

        $buff .= '<tr>';
        $buff .= _timerFormTd( $timers, 'T12', 45000 );
        $buff .= _timerFormTd( $timers, 'T13', 600000 );
        $buff .= '</tr>';

        $buff .= '<tr>';
        $buff .= _timerFormTd( $timers, 'T14', 45000 );
        $buff .= _timerFormTd( $timers, 'T15', 600000 );
        $buff .= '</tr>';

        $buff .= '<tr>';
        $buff .= _timerFormTd( $timers, 'T16', 45000 );
        $buff .= _timerFormTd( $timers, 'T17', 600000 );
        $buff .= '</tr>';

        $buff .= '<tr>';
        $buff .= _timerFormTd( $timers, 'T18', 45000 );
        $buff .= _timerFormTd( $timers, 'T19', 600000 );
        $buff .= '</tr>';

        $buff .= '<tr>';
        $buff .= _timerFormTd( $timers, 'T20', 45000 );
        $buff .= _timerFormTd( $timers, 'T21', 600000 );
        $buff .= '</tr>';

        $buff .= '<tr>';
        $buff .= _timerFormTd( $timers, 'T22', 45000 );
        $buff .= _timerFormTd( $timers, 'T23', 600000 );
        $buff .= '</tr>';

        $buff .= '<tr>';
        $buff .= _timerFormTd( $timers, 'T24', 1500 );
        $buff .= _timerFormTd( $timers, 'T25', 5000 );
        $buff .= '</tr>';

        $buff .= '<tr>';
        $buff .= _timerFormTd( $timers, 'T26', 120000 );
        $buff .= _timerFormTd( $timers, 'T27', 240000 );
        $buff .= '</tr>';

        $buff .= '<tr>';
        $buff .= _timerFormTd( $timers, 'T28', 10000 );
        $buff .= _timerFormTd( $timers, 'T34', 3000 );
        $buff .= '</tr>';

        $buff .= '<tr>';
        $buff .= _timerFormTd( $timers, 'T36', 13000 );
        $buff .= _timerFormTd( $timers, 'T37', 3000 );
        $buff .= '</tr>';

        $buff .= '<tr>';
        $buff .= _timerFormTd( $timers, 'T38', 120000 );
        $buff .= '<td>&nbsp;</td><td>&nbsp;</td>';
        $buff .= '</tr>';
        $buff .= '</tbody></table><br />';
        $buff .= "<input type='button' value='{$app->translate( 'save' )}' class='ss7-isup-timer-save floatRight' />";
        $buff .= '</fieldset></div>';

        return $buff;
    }

    /**
     * ISUP Circuit Groups main list
     */
    function showCircuitGroups( )
    {
        global $image_dir;
        $app = KSystem::getInstance( );

        $ss7 = new SS7( );

        $buff = "<fieldset><legend>{$app->translate( 'ss7-isup-circuit-groups' )}</legend>";
        $buff .= _licenseWarnings( );
        $buff .= '<div id=\'ss7-isup-circuit-groups\'>';

        if( $ss7->countIsupCircuitGroups( ) > 0 )
        {
            $buff .= "<table class='ss7-list-table zebra'><thead>
            <tr>
                <th width='14%'>{$app->translate( 'name' )}</th>
                <th width='20%'>{$app->translate( 'ss7-isup-circuit-group-origin-point-code' )}</th>
                <th width='20%'>{$app->translate( 'ss7-isup-circuit-group-destination-point-code' )}</th>
                <th width='8%'>{$app->translate( 'ss7-isup-circuit-group-basecic' )}</th>
                <th width='10%'>{$app->translate( 'ss7-isup-circuit-group-device' )}</th>
                <th width='12%'>{$app->translate( 'ss7-location' )}</th>
                <th width='8%'>{$app->translate( 'ss7-isup-circuit-group-passive' )}</th>
                <th width='8%'>{$app->translate( 'options' )}</th>
            </tr>
            </thead><tbody>";
            foreach( $ss7->getAllIsupCircuitGroups() as $idx => $info )
            {
                $_name = array_keys( $info );
                $_name = $_name[0];
                $_data = &$info[$_name];

                $passive = $_data['Passive'] == 'true' ? $app->translate( 'yes' ) : $app->translate( 'no' );
                if( $_data['Address']  == "127.0.0.1")
                {
                    $location = $app->translate( 'ss7-location-local' );
                }
                else
                {
                    $location = "{$app->translate( 'ss7-location-remote' )} (IP: {$_data['Address']})";
                }

                $buff .= "<tr>";
                $buff .= "<td>{$_name}</td>";
                $buff .= "<td>{$ss7->pointCodeToString( $_data['Origin'] )}</td>";
                $buff .= "<td>{$ss7->pointCodeToString( $_data['Destination'] )}</td>";
                $buff .= "<td>{$_data['BaseCIC']}</td>";
                $buff .= "<td>K{$_data['Device']} - Link {$_data['Link']}</td>";
                $buff .= "<td>{$location}</td>";
                $buff .= "<td>{$passive}</td>";
                $buff .= "<td class='options'>";
                $buff .= "<img title='{$app->translate( 'edit' )}' alt='{$app->translate( 'edit' )}' src='{$image_dir}/imagens/edit.png' class='ss7-isup-circuit-group-edit cursorHand' />";
                $buff .= "<img title='{$app->translate( 'delete' )}' alt='{$app->translate( 'delete' )}' src='{$image_dir}/imagens/remove.png' class='ss7-isup-circuit-group-delete cursorHand' />";
                $buff .= "</td>";
                $buff .= "</tr>";
            }
            $buff .= '</tbody></table>';
        }
        else
            $buff .= "<div class='textAlignCenter'>{$app->translate( 'ss7-isup-circuit-group-empty-list' )}</div>";

        $buff .= '<br /><div class="floatRight">';
        $buff .= "<input value='{$app->translate( 'add' )}' type='button' class='ss7-isup-circuit-group-add' />";
        $buff .= '</div>';
        $buff .= '</div></fieldset>';

        return $buff;
    }

    /**
     * MTP3 LinkSets main list
     */
    function showLinkSets( )
    {
        global $image_dir;
        $app = KSystem::getInstance( );

        $ss7 = new SS7( );

        $buff = "<fieldset><legend>{$app->translate( 'ss7-mtp3-linkset' )}</legend>";
        $buff .= _licenseWarnings( );
        $buff .= '<div id=\'ss7-mtp3-linkset\'>';

        if( $ss7->countMtp3LinkSets( ) > 0 )
        {
            $buff .= "<table class='ss7-list-table zebra'><thead><tr><th width='20%'>{$app->translate( 'name' )}</th><th width='20%'>{$app->translate( 'ss7-mtp3-linkset-origin-point-code' )}</th><th width='20%'>{$app->translate( 'ss7-mtp3-linkset-adjacent-point-code' )}</th><th width='20%'>{$app->translate( 'ss7-mtp3-linkset-links' )}</th><th width='10%'>{$app->translate( 'options'  )}</th></tr></thread><tbody>";
            foreach( $ss7->getAllMtp3LinkSets() as $idx => $info )
            {
                $_name = array_keys( $info );
                $_name = $_name[0];
                $_data = &$info[$_name];

                $links = array( );
                foreach( $_data['Links'] as $idx => $link )
                {
                    $_lname = array_keys( $link );
                    $_lname = $_lname[0];
                    $links[] = "{$_lname} ({$link[$_lname]['Slc']})";
                }
                $links = implode( ', ', $links );

                $buff .= "<tr>";
                $buff .= "<td>{$_name}</td>";
                $buff .= "<td>{$ss7->pointCodeToString( $_data['Origin'] )}</td>";
                $buff .= "<td>{$ss7->pointCodeToString( $_data['Adjacent'] )}</td>";
                $buff .= "<td>{$links}</td>";
                $buff .= '<td class=\'options\'>';
                $buff .= "<img title='{$app->translate( 'edit' )}' alt='{$app->translate( 'edit' )}' src='{$image_dir}/imagens/edit.png' class='ss7-mtp3-linkset-edit cursorHand' />";
                $buff .= "<img title='{$app->translate( 'delete' )}' alt='{$app->translate( 'delete' )}' src='{$image_dir}/imagens/remove.png' class='ss7-mtp3-linkset-delete cursorHand' />";
                $buff .= '</td>';
                $buff .= '</tr>';
            }
            $buff .= '</tbody></table>';
        }
        else
            $buff .= "<div class='textAlignCenter'>{$app->translate( 'ss7-mtp3-linkset-empty-list' )}</div>";

        $buff .= '<div class="floatRight">';
        $buff .= "<br /><input value='{$app->translate( 'add' )}' type='button' class='ss7-mtp3-linkset-add' />";
        $buff .= '</div>';
        $buff .= '</fieldset></div>';

        return $buff;
    }

    /**
     * MTP3 Routes main list
     */
    function showRoutes( )
    {
        global $image_dir;
        $app = KSystem::getInstance( );

        $ss7 = new SS7( );

        $buff = "<fieldset><legend>{$app->translate( 'ss7-mtp3-route' )}</legend>";
        $buff .= _licenseWarnings( );
        $buff .= '<div id=\'ss7-mtp3-routes\'>';

        if( $ss7->countMtp3Routes( ) > 0 )
        {
            $buff .= "<table class='ss7-list-table zebra simpleTable'><thead><tr><th width='20%'>{$app->translate('name')}</th><th width='20%'>{$app->translate('ss7-mtp3-route-destination-point-code')}</th><th width='20%'>{$app->translate('ss7-mtp3-route-linksets')}</th><th width='10%'>{$app->translate('options')}</th></tr></thead><tbody>";
            foreach( $ss7->getAllMtp3Routes() as $idx => $info )
            {
                $_name = array_keys( $info );
                $_name = $_name[0];
                $_data = &$info[$_name];

                $linksets = implode( ', ', $_data['LinkSets'] );
                $buff .= "<tr>";
                $buff .= "<td>{$_name}</td>";
                $buff .= "<td>{$ss7->pointCodeToString( $_data['PointCode'] )}</td>";
                $buff .= "<td>{$linksets}</td>";
                $buff .= "<td class='options'>";
                $buff .= "<img title='{$app->translate( 'edit' )}' alt='{$app->translate( 'edit' )}' src='{$image_dir}/imagens/edit.png' class='ss7-mtp3-route-edit cursorHand' /><img title='{$app->translate( 'delete' )}' alt='{$app->translate( 'delete' )}' src='{$image_dir}/imagens/remove.png' class='ss7-mtp3-route-delete cursorHand' />";
                $buff .= "</td>";
                $buff .= "</tr>";
            }
            $buff .= '</tbody></table>';
        }
        else
            $buff .= "<div class='textAlignCenter'>{$app->translate( 'ss7-mtp3-route-empty-list' )}</div>";

        $buff .= '<br><div class="floatRight">';
        $buff .= "<input value='{$app->translate( 'add' )}' type='button' class='ss7-mtp3-route-add' />";
        $buff .= '</div>';

        $buff .= '</div></fieldset>';

        return $buff;
    }

    /**
     * MTP3 LinkSet add/edit form
     */
    function showAddLinkSet( )
    {
        require_once KROOT . '/json_util.php';
        global $image_dir;
        $app = KSystem::getInstance( );

        $name = array_key_exists( 'name', $_POST ) ? $_POST['name'] : null;

        $ss7 = new SS7( );
        $pointcodes = $ss7->getPointCodeNames( );

        if( is_null( $name ) )
        {
            $linkset = array( 'Links' => array( ), 'Origin' => null, 'Adjacent' => null, 'NetworkIndicator' => 2 );
            $name = '';
        }
        else
        {
            $linkset = $ss7->getMtp3LinkSet( $name );
            if( is_null( $linkset ) )
                return $app->translate( 'ss7-mtp3-linkset-error-not-found' );
        }

        $buf = '<table id="ss7-mtp3-linkset-form-table" class="ss7-form-table zebra"><tbody><tr>';

        $buf .= "<td><label for='ss7-mtp3-linkset-name'>{$app->translate( 'name' )}</label>:</td>";
        $buf .= "<td><input type='hidden' name='__oldname__' value='{$name}' /><input value='{$name}' id='ss7-mtp3-linkset-name' name='__name__' type='text' /></td>";

        $buf .= '</tr><tr>';

        $select = _createSelect( $pointcodes, $linkset['Origin'], 'id=\'ss7-mtp3-linkset-origin-point-code\' name=\'Origin\'', true, true );
        $buf .= "<td><label for='ss7-mtp3-linkset-origin-point-code'>{$app->translate( 'ss7-mtp3-linkset-origin-point-code' )}</label>:</td>";
        $buf .= "<td>{$select}</td>";

        $buf .= '</tr><tr>';

        $select = _createSelect( $pointcodes, $linkset['Adjacent'], 'id=\'ss7-mtp3-linkset-adjacent-point-code\' name=\'Adjacent\'', true, true );
        $buf .= "<td><label for='ss7-mtp3-linkset-adjacent-point-code'>{$app->translate( 'ss7-mtp3-linkset-adjacent-point-code' )}</label>:</td>";
        $buf .= "<td>{$select}</td>";

        $buf .= '</tr><tr>';

        $select = _createSelect( $ss7->getMtp3LinkSetNetworkIndicators( ), $linkset['NetworkIndicator'], 'id=\'ss7-mtp3-linkset-network-indicator\' name=\'NetworkIndicator\'', false, true );
        $buf .= "<td><label for='ss7-mtp3-linkset-network-indicator'>{$app->translate( 'ss7-mtp3-linkset-network-indicator' )}</label>:</td>";
        $buf .= "<td>{$select}</td>";

        $buf .= '</tr></tbody></table><br />';

        if( count( $linkset['Links'] ) > 0 )
        {
            $displayassociated = array( 'msg' => ' style=\'display: none;\'', 'content' => '' );
            $links = '';
            foreach( $linkset['Links'] as $idx => &$link )
            {
                $_name = array_keys( $link );
                $_name = $_name[0];
                $_data = &$link[$_name];
                $json = JSONencode( $_data );
                $links .= "<li>SLC {$_data['Slc']}: {$_name}<input class='ss7-mtp3-linkset-mtp2-link-data' type='hidden' value='{$json}' /><img title='" . $app->translate( 'remove' ) . "' class='ss7-mtp3-linkset-mtp3-link-delete' src='{$image_dir}/imagens/remove.png' /></li>";
            }
        }
        else
        {
            $displayassociated = array( 'msg' => '', 'content' => ' style=\'display: none;\'' );
            $links = '';
        }

        $mtp2links = $ss7->getMtp2Names( true );
        $displayavailable = array( 'msg' => count( $mtp2links ) === 0 ? '' : ' style=\'display: none;\'', 'content' => count( $mtp2links ) === 0 ? ' style=\'display: none;\'' : '' );

        $buf .= "<fieldset><legend>{$app->translate( 'ss7-mtp3-linkset-links' )}</legend>";
        $buf .= "<div id='ss7-mtp3-linkset-links'><h3>{$app->translate( 'ss7-mtp3-linkset-associated-links' )}</h3><p{$displayassociated['msg']}>{$app->translate( 'ss7-mtp3-linkset-associated-links-empty-list' )}</p><ul{$displayassociated['content']}>{$links}</ul></div>";
        $buf .= "<div id='ss7-mtp3-linkset-link-form'><h3>{$app->translate( 'ss7-mtp3-linkset-available-links' )}</h3><p{$displayavailable['msg']}>{$app->translate( 'ss7-mtp3-linkset-available-links-empty-list' )}</p><div{$displayavailable['content']}><table class='ss7-form-table simpleTable'><tbody><tr>";

        $select = _createSelect( $mtp2links, null, 'id=\'ss7-mtp3-linkset-mtp2-link\'', false );
        $buf .= "<td><label for='ss7-mtp3-linkset-mtp2-link'>{$app->translate( 'ss7-mtp3-linkset-mtp2-link' )}</label></td>";
        $buf .= "<td>{$select}</td>";

        $buf .= '</tr><tr>';

        $select = _createSelect( $ss7->getMtp3LinkSetMtp2LinkSlcs( $linkset ), null, 'id=\'ss7-mtp3-linkset-slc\'', false );
        $buf .= "<td><label for='ss7-mtp3-linkset-slc'>SLC</label></td>";
        $buf .= "<td>{$select}</td>";

        $buf .= '</tr></tbody></table>';

        $buf .= "<div class='expand-next'>+ {$app->translate( 'ss7-mtp3-linkset-mtp2-link-advanced-configuration' )}</div>";
        $buf .= '<table style="display: none;" class="ss7-form-table simpleTable"><tbody><tr>';

        $buf .= "<td><label for='ss7-mtp3-linkset-q707t1'>Q707T1</label></td>";
        $buf .= "<td><input type='text' id='ss7-mtp3-linkset-q707t1' name='Q707T1' value='4000' /></td>";

        $buf .= '</tr><tr>';

        $buf .= "<td><label for='ss7-mtp3-linkset-q707t2'>Q707T2</label></td>";
        $buf .= "<td><input type='text' id='ss7-mtp3-linkset-q707t2' name='Q707T2' value='30000' /></td>";

        $buf .= '</tr><tr>';

        $buf .= "<td><label for='ss7-mtp3-linkset-q704t17'>Q704T17</label></td>";
        $buf .= "<td><input type='text' id='ss7-mtp3-linkset-q704t17' name='Q704T17' value='800' /></td>";

        $buf .= '</tr></tbody></table>';

        $buf .= "<br /><input type='button' value='{$app->translate( 'ss7-mtp3-linkset-add-link' )}' class='ss7-mtp3-linkset-add-link floatRight' /></div></div>";
        $buf .= '<div style=\'clear:both;\'>&nbsp;</div></fieldset>';

        $buf .= '</fieldset><br />';
        $buf .= '<div class="floatRight">';
        $buf .= "<input value='{$app->translate( 'save' )}' type='button' class='ss7-mtp3-linkset-save' />";
        $buf .= "<input value='{$app->translate( 'back' )}' type='button' class='ss7-mtp3-linkset-back' />";
        $buf .= '</div>';

        return $buf;
    }

    /**
     * Save MTP3 LinkSet
     */
    function saveLinkSet( )
    {
        require_once KROOT . '/json_util.php';

        $app = KSystem::getInstance( );
        $reply = array( 'success' => false, 'message' => $app->translate( 'ss7-mtp3-linkset-error-save' ) );
        $name = array_key_exists( '__name__', $_POST ) ? $_POST['__name__'] : null;
        $oldName = array_key_exists( '__oldname__', $_POST ) ? $_POST['__oldname__'] : null;

        try
        {
            if( is_null( $name ) )
            {
                throw new Exception( 'ss7-mtp3-linkset-name-is-required' );
            }
            unset( $_POST['__name__'] );
            unset( $_POST['__oldname__'] );

            $ss7 = new SS7( );
            $ss7->setMtp3LinkSet( $name, $_POST, $oldName );
            $ss7->save( );
            $reply = array( 'success' => true, 'message' => $app->translate( 'ss7-mtp3-linkset-saved' ) );
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }

        return JSONencode( $reply );
    }

    /**
     * Remove MTP3 LinkSet
     */
    function removeLinkSet( )
    {
        require_once KROOT . '/json_util.php';

        $app = KSystem::getInstance( );
        $reply = array( 'success' => false, 'message' => $app->translate( 'ss7-mtp3-linkset-error-delete' ) );
        $name = array_key_exists( 'name', $_POST ) ? $_POST['name'] : null;

        try
        {
            $ss7 = new SS7( );
            if( $ss7->deleteMtp3LinkSet( $name ) )
            {
                $ss7->save( );
                $reply = array( 'success' => true, 'message' => $app->translate( 'ss7-mtp3-linkset-deleted' ) );
            }
            else
            {
                throw new Exception( 'ss7-mtp3-linkset-error-not-found' );
            }
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }

        return JSONencode( $reply );
    }

    /**
     * Form to add/edit MTP2 link
     */
    function showAddMTP2( )
    {
        global $sistema;
        $app = KSystem::getInstance( );

        $name = array_key_exists( 'name', $_POST ) ? $_POST['name'] : null;
        $ss7 = new SS7( );

        // new -- set defaults
        if( is_null( $name ) )
        {
            //@formatter:off
            $mtp2 = array(
                '__name__' => '',
                'Address' => '127.0.0.1',
                'Timers' => array(
                    'T1' => 40000,
                    'T2' => 5000,
                    'T3' => 1500,
                    'T5' => 100,
                    'T6' => 3000,
                    'T7' => 1000,
                    'ProvingEmergency' => 500,
                    'ProvingNormal' => 8200,
                ),
                'Device' => '',
                'Link' => '',
                'Passive' => 'false'
            );
            //@formatter:on
        }
        else
        {
            $mtp2 = $ss7->getMtp2( $name );
            if( is_null( $mtp2 ) )
            {
                return $app->translate( 'ss7-mtp2-error-not-found' );
            }
            $mtp2['__name__'] = $name;
        }

        $buf = '<table class="ss7-form-table zebra"><tbody><tr>';

        $buf .= "<td align='left'><label for='ss7-mtp2-name'>{$app->translate( 'name' )}</label>:</td>";
        $buf .= "<td width='20%' align='center'><input type='hidden' name='__oldname__' value='{$mtp2['__name__']}' /><input value='{$mtp2['__name__']}' id='ss7-mtp2-name' name='__name__' type='text' /></td>";

        $buf .= '</tr><tr>';

        $isLocal = ($mtp2["Address"] == "127.0.0.1");
        $options = array( "local" => $app->translate( 'ss7-location-local' ),
                          "remote" => $app->translate( 'ss7-location-remote' ) );
        $select = _createSelect( $options, ($isLocal ? 'local' : 'remote'), 'id="ss7-mtp2-location"', false, true);

        $buf .= "<td align='left'><label for='ss7-mtp2-location'>{$app->translate( 'ss7-location' )}</label>:</td>";
        $buf .= "<td>$select</td>";
        $buf .= '</tr><tr>';

        $links = array();
        foreach($sistema->getSpecificDevice('E1') as $d)
        {
            foreach($sistema->getDeviceLinks($d) as $l)
            {
                $links["{$d}.{$l}"] = "K{$d} - Link {$l}";
            }
        }
        $currentLink = "{$mtp2['Device']}.{$mtp2['Link']}";
        $linkSelect = _createSelect( $links, $currentLink, 'id="ss7-mtp2-local-device-link" name="local_Link" ', is_null($name), true );

        $passiveChecked = $mtp2['Passive'] == 'true' ? 'checked ' : '';

        $buf .= "<td align='left' colspan='2'>
            <div id='ss7-mtp2-link-data-local'>
                <table class='ss7-form-table zebra'><tbody><tr>
                <tr>
                    <td align='left'><label for='ss7-mtp2-local-device-link'>{$app->translate( 'ss7-mtp2-link' )}</label>:</td>
                    <td>$linkSelect</td>
                </tr>
                <tr>
                    <td align='left'><label for='ss7-mtp2-local-passive'>{$app->translate( 'ss7-mtp2-passive' )}</label>:</td>
                    <td><input id='ss7-mtp2-local-passive' name='local_Passive' type='checkbox' $passiveChecked/></td>
                </tr>
                </tbody></table>
            </div>
            <div id='ss7-mtp2-link-data-remote'>
                <table class='ss7-form-table zebra'><tbody><tr>
                <tr>
                    <td align='left'><label for='ss7-mtp2-address'>{$app->translate( 'ss7-mtp2-address' )}</label>:</td>
                    <td><input value='{$mtp2['Address']}' id='ss7-mtp2-address' name='remote_Address' type='text' class='ipv4' /></td>
                </tr>
                <tr>
                    <td align='left'><label for='ss7-mtp2-remote-device'>{$app->translate( 'ss7-mtp2-device' )}</label>:</td>
                    <td><input value='{$mtp2['Device']}' id='ss7-mtp2-remote-device' name='remote_Device' type='text' /></td>
                </tr>
                <tr>
                    <td align='left'><label for='ss7-mtp2-remote-device-link'>{$app->translate( 'ss7-mtp2-link' )}</label>:</td>
                    <td><input value='{$mtp2['Link']}' id='ss7-mtp2-remote-device-link' name='remote_Link' type='text' /></td>
                </tr>
                <tr>
                    <td align='left'><label for='ss7-mtp2-remote-passive'>{$app->translate( 'ss7-mtp2-passive' )}</label>:</td>
                    <td><input id='ss7-mtp2-local-passive' name='remote_Passive' type='checkbox' $passiveChecked/></td>
                </tr>
                </tbody></table>
            </div>
        </td>";

        $buf .= '</tr><tr>';
        $buf .= '</tr></tbody></table><br/>';

        $buf .= "<fieldset><legend>{$app->translate('ss7-mtp2-timers')}</legend>";

        $buf .= '<table class="ss7-form-table zebra"><tbody><tr>';

        $buf .= "<td align='left'><label for='ss7-mtp2-timer-t1'>T1</label>:</td>";
        $buf .= "<td><input value='{$mtp2['Timers']['T1']}' id='ss7-mtp2-timer-t1' name='T1' type='text' class='integer' /></td>";

        $buf .= '</tr><tr>';

        $buf .= "<td align='left'><label for='ss7-mtp2-timer-t2'>T2</label>:</td>";
        $buf .= "<td><input value='{$mtp2['Timers']['T2']}' id='ss7-mtp2-timer-t2' name='T2' type='text' class='integer' /></td>";

        $buf .= '</tr><tr>';

        $buf .= "<td align='left'><label for='ss7-mtp2-timer-t3'>T3</label>:</td>";
        $buf .= "<td><input value='{$mtp2['Timers']['T3']}' id='ss7-mtp2-timer-t3' name='T3' type='text' class='integer' /></td>";

        $buf .= '</tr><tr>';

        $buf .= "<td align='left'><label for='ss7-mtp2-timer-t5'>T5</label>:</td>";
        $buf .= "<td><input value='{$mtp2['Timers']['T5']}' id='ss7-mtp2-timer-t5' name='T5' type='text' class='integer' /></td>";

        $buf .= '</tr><tr>';

        $buf .= "<td align='left'><label for='ss7-mtp2-timer-t6'>T6</label>:</td>";
        $buf .= "<td><input value='{$mtp2['Timers']['T6']}' id='ss7-mtp2-timer-t6' name='T6' type='text' class='integer' /></td>";

        $buf .= '</tr><tr>';

        $buf .= "<td align='left'><label for='ss7-mtp2-timer-t7'>T7</label>:</td>";
        $buf .= "<td><input value='{$mtp2['Timers']['T7']}' id='ss7-mtp2-timer-t7' name='T7' type='text' class='integer' /></td>";

        $buf .= '</tr><tr>';

        $buf .= "<td align='left'><label for='ss7-mtp2-timer-proving-emergency'>{$app->translate( 'ss7-mtp2-timer-proving-emergency' )}</label>:</td>";
        $buf .= "<td><input value='{$mtp2['Timers']['ProvingEmergency']}' id='ss7-mtp2-timer-proving-emergency' name='ProvingEmergency' type='text' class='integer' /></td>";

        $buf .= '</tr><tr>';

        $buf .= "<td align='left'><label for='ss7-mtp2-timer-proving-normal'>{$app->translate( 'ss7-mtp2-timer-proving-normal' )}</label>:</td>";
        $buf .= "<td><input value='{$mtp2['Timers']['ProvingNormal']}' id='ss7-mtp2-timer-proving-normal' name='ProvingNormal' type='text' class='integer' /></td>";

        $buf .= '</tr></tbody></table>';

        $buf .= '</fieldset>';

        $buf .= '<br />';

        $buf .= '<div class="floatRight">';
        $buf .= "<input class='ss7-mtp2-back' value='{$app->translate( 'back' )}' type='button'>";
        $buf .= "<input class='ss7-mtp2-save' value='{$app->translate( 'save' )}' type='button'>";
        $buf .= '</div>';

        return $buf;
    }

    /**
     * Save MTP2 link
     */
    function saveMTP2( )
    {
        require_once KROOT . '/json_util.php';

        $app = KSystem::getInstance( );

        $reply = array( 'success' => false, 'message' => $app->translate( 'ss7-mtp2-error-save' ) );
        $name = array_key_exists( '__name__', $_POST ) ? $_POST['__name__'] : null;
        $oldName = array_key_exists( '__oldname__', $_POST ) ? $_POST['__oldname__'] : null;

        try
        {
            if( is_null( $name ) || is_null( $oldName ) )
            {
                throw new Exception( 'ss7-mtp2-error-name-is-required' );
            }
            unset( $_POST['__name__'] );
            unset( $_POST['__oldname__'] );

            $ss7 = new SS7( );
            $ss7->setMtp2( $name, $_POST, $oldName );
            $ss7->save( );
            $reply = array( 'success' => true, 'message' => $app->translate( 'ss7-mtp2-saved' ) );
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }

        return JSONencode( $reply );
    }

    /**
     * Delete MTP2 link
     */
    function removeMTP2( )
    {
        require_once KROOT . '/json_util.php';

        $app = KSystem::getInstance( );

        $reply = array( 'success' => false, 'message' => $app->translate( 'ss7-mtp2-error-delete' ) );

        $name = array_key_exists( 'name', $_POST ) ? $_POST['name'] : null;

        try
        {
            if( is_null( $name ) )
            {
                throw new Exception( 'ss7-mtp2-error-not-found' );
            }
            $ss7 = new SS7( );
            $ss7->deleteMtp2( $name );
            $ss7->save( );
            $reply = array( 'success' => true, 'message' => $app->translate( 'ss7-mtp2-deleted' ) );
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }
        return JSONencode( $reply );
    }

    /**
     * Form to add/edit Point Code
     */
    function showAddPointCode( )
    {
        $app = KSystem::getInstance( );

        $name = array_key_exists( 'name', $_POST ) ? $_POST['name'] : null;

        $ss7 = new SS7( );

        if( is_null( $name ) )
        {
            //@formatter:off
            $pointcode = array(
                '__name__' => '',
                '__first__' => 0,
                '__middle__' => 0,
                '__last__' => 0,
                '__decimal__' => 0
            );
            //@formatter:on
        }
        else
        {
            $_pointcode = $ss7->getPointCode( $name );
            if( is_null( $_pointcode ) )
            {
                throw new Exception( 'ss7-pointcode-error-not-found' );
            }
            // @formatter:off
            $pointcode = array(
                '__name__' => $name,
                '__first__' => $_pointcode[0],
                '__middle__' => $_pointcode[1],
                '__last__' => $_pointcode[2],
                '__decimal__' => $ss7->pointCodeToDec( $_pointcode )
            );
            // @formatter:on
            unset( $_pointcode );
        }

        $buff = '<table class="ss7-form-table zebra"><tbody>';
        $buff .= '<tr>';
        $buff .= "<td><label for='ss7-pointcode-form-name'>{$app->translate( 'name' )}</label>:</td>";
        $buff .= "<td><input name='__oldname__' value='{$pointcode['__name__']}' type='hidden' /><input id='ss7-pointcode-form-name' name='__name__' type='text' value='{$pointcode['__name__']}' /></td>";
        $buff .= '</tr><tr>';
        $buff .= "<td><label for='ss7-pointcode-form-decimal'>{$app->translate( "ss7-pointcode-decimal" )}</label>:</td>";
        $buff .= "<td><input id='ss7-pointcode-form-decimal' name='__decimal__' type='text' value='{$pointcode['__decimal__']}' /></td>";
        $buff .= '</tr><tr>';
        $buff .= "<td><label for='ss7-pointcode-form-first'>{$app->translate( "ss7-pointcode-3-8-3" )}</label>:</td><td><input id='ss7-pointcode-form-first' name='__first__' type='text' value='{$pointcode['__first__']}' class='ss7-pointcode-383' /> - ";
        $buff .= "<input id='ss7-pointcode-form-middle' name='__middle__' type='text' value='{$pointcode['__middle__']}' class='ss7-pointcode-383' /> - ";
        $buff .= "<input id='ss7-pointcode-form-last' name='__last__' type='text' value='{$pointcode['__last__']}' class='ss7-pointcode-383' /></td>";
        $buff .= "</tr>";
        $buff .= '</tbody></table><br />';

        $buff .= '<div class="floatRight">';
        $buff .= "<input class='ss7-pointcode-save' value='{$app->translate( 'save' )}' type='button' />";
        $buff .= "<input class='ss7-pointcode-back' value='{$app->translate( 'back' )}' type='button' />";
        $buff .= '</div>';

        return $buff;
    }

    /**
     * Save Point Code
     */
    function savePointCode( )
    {
        require_once KROOT . '/json_util.php';

        $app = KSystem::getInstance( );
        $ss7 = new SS7( );
        $reply = array( 'success' => false, 'message' => $app->translate( 'ss7-pointcode-error-save' ) );

        try
        {
            $name = array_key_exists( '__name__', $_POST ) ? $_POST['__name__'] : null;
            $oldName = array_key_exists( '__oldname__', $_POST ) ? $_POST['__oldname__'] : null;
            $decimal = array_key_exists( '__decimal__', $_POST ) ? $_POST['__decimal__'] : null;
            $first = array_key_exists( '__first__', $_POST ) ? $_POST['__first__'] : null;
            $middle = array_key_exists( '__middle__', $_POST ) ? $_POST['__middle__'] : null;
            $last = array_key_exists( '__last__', $_POST ) ? $_POST['__last__'] : null;

            if( is_null( $name ) )
            {
                throw new Exception( 'ss7-pointcode-error-name-is-required' );
            }
            if( is_null( $decimal ) )
            {
                throw new Exception( 'ss7-pointcode-error-decimal-is-required' );
            }
            if( is_null( $first ) || is_null( $middle ) || is_null( $last ) )
            {
                throw new Exception( 'ss7-pointcode-error-3-8-3-is-required' );
            }
            $pointcode = array( $first, $middle, $last );
            $ss7->setPointCode( $name, $pointcode, $oldName );
            $ss7->save( );
            $reply = array( 'success' => true, 'message' => $app->translate( 'ss7-pointcode-saved' ) );
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }
        return JSONencode( $reply );
    }

    /**
     * Delete Point Code
     */
    function removePointCode( )
    {
        require_once KROOT . '/json_util.php';
        $app = KSystem::getInstance( );
        $ss7 = new SS7( );
        $reply = array( 'success' => false, 'message' => $app->translate( 'ss7-pointcode-error-delete' ) );

        $name = array_key_exists( 'name', $_POST ) ? $_POST['name'] : null;
        if( is_null( $name ) )
        {
            $reply['message'] = $app->translate( 'ss7-pointcode-error-not-found' );
        }
        else
        {
            try
            {
                $ss7->deletePointCode( $name );
                $ss7->save( );
                $reply = array( 'success' => true, 'message' => $app->translate( 'ss7-pointcode-deleted' ) );
            }
            catch( Exception $e )
            {
                $reply['message'] = $app->translate( $e->getMessage( ) );
            }
        }
        return JSONencode( $reply );
    }

    /**
     * Save ISUP Timers
     */
    function saveTimers( )
    {
        require_once KROOT . '/json_util.php';
        $app = KSystem::getInstance( );
        $ss7 = new SS7( );
        $reply = array( 'success' => false, 'message' => $app->translate( 'ss7-isup-timer-error-save' ) );

        try
        {
            $ss7->setAllTimers( $_POST );
            $ss7->save( );
            $reply = array( 'success' => true, 'message' => $app->translate( 'ss7-isup-timer-saved' ) );
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }
        return JSONencode( $reply );
    }

    /**
     * Form to add/edit ISUP Circuit Group
     */
    function showAddCircuitGroup( )
    {
        global $sistema;
        $app = KSystem::getInstance( );

        $name = array_key_exists( 'name', $_GET ) ? $_GET['name'] : '';
        $ss7 = new SS7( );

        if( empty( $name ) )
        {
            $circuitGroup = array(
                'Origin' => null,
                'Destination' => null,
                'Device' => '',
                'Link' => 0,
                'Address' => '127.0.0.1',
                'BaseCIC' => 1,
                'Passive' => 'false',
                'TimeslotMap' => SS7::ISUP_CIRCUITGROUP_DEFAUL_TIMESLOT
            );
        }
        else
        {
            $circuitGroup = $ss7->getIsupCircuitGroup( $name );
            if( is_null( $circuitGroup ) )
            {
                return $app->translate( 'ss7-isup-circuit-group-error-not-found' );
            }
        }

        $pointCodes = $ss7->getPointCodeNames( );

        $buf = '<table class="ss7-form-table zebra"><tbody><tr>';
        $buf .= "<td><label for='ss7-isup-circuit-group-form-name'>{$app->translate( 'name' )}</label>:</td>";
        $buf .= "<td><input type='hidden' name='__oldname__' value='{$name}' /><input id='ss7-isup-circuit-group-form-name' value='{$name}' name='__name__' type='text' /></td>";

        $buf .= '</tr><tr>';

        $select = _createSelect( $pointCodes, $circuitGroup['Origin'], 'id=\'ss7-isup-circuit-group-form-origin-point-code\' name=\'Origin\'', true, true );
        $buf .= "<td><label for='ss7-isup-circuit-group-form-origin-point-code'>{$app->translate( 'ss7-isup-circuit-group-origin-point-code' )}</label>:</td>";
        $buf .= "<td>{$select}</td>";

        $buf .= '</tr><tr>';

        $select = _createSelect( $pointCodes, $circuitGroup['Destination'], 'id=\'ss7-isup-circuit-group-form-destination-point-code\' name=\'Destination\'', true, true );
        $buf .= "<td><label for='ss7-isup-circuit-group-form-destination-point-code'>{$app->translate( 'ss7-isup-circuit-group-destination-point-code' )}</label>:</td>";
        $buf .= "<td>{$select}</td>";

        $buf .= '</tr><tr>';

        $isLocal = ($circuitGroup["Address"] == "127.0.0.1");
        $options = array( "local" => $app->translate( 'ss7-location-local' ),
                          "remote" => $app->translate( 'ss7-location-remote' ) );
        $select = _createSelect( $options, ($isLocal ? 'local' : 'remote'), 'id="ss7-isup-circuit-group-location"', false, true);

        $buf .= "<td align='left'><label for='ss7-isup-circuit-group-address'>{$app->translate( 'ss7-location' )}</label>:</td>";
        $buf .= "<td>$select</td>";
        $buf .= '</tr><tr>';

        $links = array();
        foreach($sistema->getSpecificDevice('E1') as $d)
        {
            foreach($sistema->getDeviceLinks($d) as $l)
            {
                $links["{$d}.{$l}"] = "K{$d} - Link {$l}";
            }
        }
        $currentLink = "{$circuitGroup['Device']}.{$circuitGroup['Link']}";
        $linkSelect = _createSelect( $links, $currentLink, 'id="ss7-isup-circuit-group-local-device-link" name="local_Link" ', is_null($name), true );

        $passiveChecked = $circuitGroup['Passive'] == 'true' ? 'checked ' : '';

        $buf .= "<td align='left' colspan='2'>
            <div id='ss7-isup-circuit-group-link-data-local'>
                <table class='ss7-form-table zebra'><tbody><tr>
                <tr>
                    <td align='left'><label for='ss7-isup-circuit-group-local-device-link'>{$app->translate( 'ss7-isup-circuit-group-link' )}</label>:</td>
                    <td>$linkSelect</td>
                </tr>
                <tr>
                    <td align='left'><label for='ss7-isup-circuit-group-local-passive'>{$app->translate( 'ss7-isup-circuit-group-passive' )}</label>:</td>
                    <td><input id='ss7-isup-circuit-group-local-passive' name='local_Passive' type='checkbox' $passiveChecked/></td>
                </tr>
                </tbody></table>
            </div>
            <div id='ss7-isup-circuit-group-link-data-remote'>
                <table class='ss7-form-table zebra'><tbody><tr>
                <tr>
                    <td align='left'><label for='ss7-isup-circuit-group-address'>{$app->translate( 'ss7-isup-circuit-group-address' )}</label>:</td>
                    <td><input value='{$circuitGroup['Address']}' id='ss7-isup-circuit-group-address' name='remote_Address' type='text' class='ipv4' /></td>
                </tr>
                <tr>
                    <td align='left'><label for='ss7-isup-circuit-group-remote-device'>{$app->translate( 'ss7-isup-circuit-group-device' )}</label>:</td>
                    <td><input value='{$circuitGroup['Device']}' id='ss7-isup-circuit-group-remote-device' name='remote_Device' type='text' /></td>
                </tr>
                <tr>
                    <td align='left'><label for='ss7-isup-circuit-group-remote-device-link'>{$app->translate( 'ss7-isup-circuit-group-link' )}</label>:</td>
                    <td><input value='{$circuitGroup['Link']}' id='ss7-isup-circuit-group-remote-device-link' name='remote_Link' type='text' /></td>
                </tr>
                <tr>
                    <td align='left'><label for='ss7-isup-circuit-group-remote-passive'>{$app->translate( 'ss7-isup-circuit-group-passive' )}</label>:</td>
                    <td><input id='ss7-isup-circuit-group-local-passive' name='remote_Passive' type='checkbox' $passiveChecked/></td>
                </tr>
                </tbody></table>
            </div>
        </td>";

        $buf .= '</tr><tr>';

        $buf .= "<td><label for='ss7-isup-circuit-group-form-basecic'>{$app->translate( 'ss7-isup-circuit-group-basecic' )}</label>:</td>";
        $buf .= "<td><input type='text' value='{$circuitGroup['BaseCIC']}' id='ss7-isup-circuit-group-form-basecic' name='BaseCIC' /></td>";

        $buf .= '</tr><tr>';

        $buf .= '</tr></tbody></table><br>';

        $buf .= "<fieldset><legend>{$app->translate( 'ss7-isup-circuit-group-timeslotmap' )}</legend><div id='ss7-isup-circuit-group-timeslot-map'>";
        $buf .= "<div><br /><span class='ss7-isup-circuit-group-timeslot-select-all'>{$app->translate( 'ss7-isup-circuit-group-timeslot-select-all' )}</span><br /><br /><span class='ss7-isup-circuit-group-timeslot-deselect-all'>{$app->translate( 'ss7-isup-circuit-group-timeslot-deselect-all' )}</span></div>";
        $buf .= '<ul>';

        $timeslotmap = hexdec( substr( $circuitGroup['TimeslotMap'], 2 ) );
        for( $timeslot = 0; $timeslot <= SS7::ISUP_CIRCUITGROUP_AVAILABLE_TIMESLOTS; $timeslot++ )
        {
            if( $ss7->isIsupCircuitGroupTimeslotReserved( $timeslot ) )
            {
                $extra = ' disabled=\'true\'';
                $extralabel = ' class=\'disabled\'';
            }
            else
            {
                $extra = $ss7->isIsupCirctuitGroupTimeslotSet( $timeslotmap, $timeslot ) ? ' checked=\'true\'' : '';
                $extralabel = '';
            }
            $buf .= "<li><label for='ss7-isup-circuit-group-timeslot-{$timeslot}'{$extralabel}>{$timeslot}</label> <input id='ss7-isup-circuit-group-timeslot-{$timeslot}' type='checkbox'{$extra} /></li>";
            if( $timeslot === 15 )
            {
                $buf .= '</ul><ul>';
            }
        }
        $buf .= '</ul>';
        $buf .= '</div></fieldset><br />';

        $buf .= '<div class="floatRight">';
        $buf .= "<input value='{$app->translate( 'back' )}' type='button' class='ss7-isup-circuit-group-back' />";
        $buf .= "<input value='{$app->translate( 'save' )}' type='button' class='ss7-isup-circuit-group-save' />";
        $buf .= '</div>';

        return $buf;
    }

    /**
     * Save ISUP Circuit Group
     */
    function saveCircuitGroup( )
    {
        require_once KROOT . '/json_util.php';
        $app = KSystem::getInstance( );
        $reply = array( 'success' => false, 'message' => $app->translate( 'ss7-isup-circuit-group-error-save' ) );
        $ss7 = new SS7( );

        try
        {
            $name = array_key_exists( '__name__', $_POST ) ? $_POST['__name__'] : null;
            $oldName = array_key_exists( '__oldname__', $_POST ) ? $_POST['__oldname__'] : null;
            $_timeslotmap = array_key_exists( '__timeslotmap__', $_POST ) ? $_POST['__timeslotmap__'] : null;
            $timeslotmap = 0;

            unset( $_POST['__name__'] );
            unset( $_POST['__oldname__'] );
            unset( $_POST['__timeslotmap__'] );

            foreach( $_timeslotmap as $timeslot )
            {
                $timeslotmap = $ss7->setIsupCircuitGroupTimeslotBit( $timeslotmap, $timeslot );
            }
            $_POST['TimeslotMap'] = '0x' . strtoupper( dechex( $timeslotmap ) );

            $ss7->setIsupCircuitGroup( $name, $_POST, $oldName );
            $ss7->save( );
            $reply = array( 'success' => true, 'message' => $app->translate( 'ss7-isup-circuit-group-saved' ) );
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }

        return JSONencode( $reply );
    }

    /**
     * Delete ISUP Circuit Group
     */
    function removeCircuitGroup( )
    {
        require_once KROOT . '/json_util.php';
        $app = KSystem::getInstance( );
        $reply = array( 'success' => false, 'message' => $app->translate( 'ss7-isup-circuit-group-error-delete' ) );
        $ss7 = new SS7( );

        try
        {
            $name = array_key_exists( 'name', $_GET ) ? $_GET['name'] : null;
            if( !is_null( $name ) && $ss7->deleteIsupCircuitGroup( $name ) )
            {
                $ss7->save( );
                $reply = array( 'success' => true, 'message' => $app->translate( 'ss7-isup-circuit-group-deleted' ) );
            }
            else
            {
                throw new Exception( 'ss7-isup-circuit-group-error-not-found' );
            }
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }

        return JSONencode( $reply );
    }

    /**
     * Form to add/edit MTP3 Route
     */
    function showAddRoute( )
    {
        global $image_dir;

        $app = KSystem::getInstance( );
        $ss7 = new SS7( );

        $name = array_key_exists( 'name', $_POST ) ? $_POST['name'] : '';
        $buf = '';

        if( empty( $name ) )
        {
            $mtp3route = array( 'PointCode' => null, 'LinkSets' => array( ) );
        }
        else
        {
            $mtp3route = $ss7->getMtp3Route( $name );
            if( is_null( $mtp3route ) )
            {
                return $app->translate( 'ss7-mtp3-route-error-not-found' );
            }
        }

        $buf .= '<div id=\'ss7-mtp3-route-form-table\'><table class=\'ss7-form-table simpleTable\'><tbody><tr>';

        $buf .= "<td><label for='ss7-mtp3-route-form-name'>{$app->translate( 'name' )}</label>:</td>";
        $buf .= "<td><input type='hidden' name='__oldname__' value='{$name}' /><input type='text' id='ss7-mtp3-route-form-name' name='__name__' value='{$name}' /></td>";

        $buf .= '</tr><tr>';

        $select = _createSelect( $ss7->getPointCodeNames( ), $mtp3route['PointCode'], 'id=\'ss7-mtp3-route-form-point-code\' name=\'PointCode\'', true, true );
        $buf .= "<td><label for='ss7-mtp3-route-form-point-code'>{$app->translate( 'ss7-mtp3-route-destination-point-code' )}</label>:</td>";
        $buf .= "<td>{$select}</td>";

        $buf .= '</tr></table></div>';

        if( count( $mtp3route['LinkSets'] ) > 0 )
        {
            $displaylinksetlist = array( 'msg' => ' style=\'display: none;\'', 'content' => '' );
            $linksetlist = '';
            foreach( $mtp3route['LinkSets'] as &$linkset )
            {
                $linksetlist .= "<li>{$linkset}<img src='{$image_dir}/imagens/remove.png' class='ss7-mtp3-route-linkset-delete' /></li>";
            }
        }
        else
        {
            $linksetlist = '';
            $displaylinksetlist = array( 'msg' => '', 'content' => ' style=\'display: none;\'' );
        }
        $linksets = $ss7->getMtp3LinkSetNames( false, $mtp3route['LinkSets'] );
        $displaylinksets = array( 'msg' => count( $linksets ) === 0 ? '' : ' style=\'display: none;\'', 'content' => count( $linksets ) === 0 ? ' style=\'display: none;\'' : '' );

        $buf .= "<fieldset><legend>{$app->translate( 'ss7-mtp3-route-linksets' )}</legend>";
        $buf .= "<div id='ss7-mtp3-route-linksets'><h3>{$app->translate( 'ss7-mtp3-route-associated-linksets' )}</h3><p{$displaylinksetlist['msg']}>{$app->translate( 'ss7-mtp3-route-associated-linksets-empty-list' )}</p><ul{$displaylinksetlist['content']}>{$linksetlist}</ul></div>";
        $buf .= "<div id='ss7-mtp3-route-linksets-form'><h3>{$app->translate( 'ss7-mtp3-route-available-linksets' )}</h3><p{$displaylinksets['msg']}>{$app->translate( 'ss7-mtp3-route-available-linksets-empty-list' )}</p><div{$displaylinksets['content']}><table class='ss7-form-table simpleTable'><tr>";
        $select = _createSelect( $linksets, null, 'id=\'ss7-mtp3-route-form-linkset\'', false );
        $buf .= "<td><label for='ss7-mtp3-route-form-linkset'>{$app->translate( 'ss7-mtp3-route-linkset' )}</label></td>";
        $buf .= "<td>{$select}</td>";
        $buf .= '</tr></tbody></table>';
        $buf .= "<br><input value='{$app->translate( 'ss7-mtp3-route-add-linkset' )}' type='button' class='ss7-mtp3-route-add-linkset floatRight' />";
        $buf .= '</div></div>';
        $buf .= '<div style=\'clear:both;\'>&nbsp;</div>';

        $buf .= '</fieldset><br />';
        $buf .= '<div class="floatRight">';
        $buf .= "<input value='{$app->translate( 'save' )}' type='button' class='ss7-mtp3-route-save' />";
        $buf .= "<input value='{$app->translate( 'back' )}' type='button' class='ss7-mtp3-route-back' />";
        $buf .= '</div>';

        return $buf;
    }

    /**
     * Delete MTP3 Route
     */
    function removeRoute( )
    {
        require_once KROOT . '/json_util.php';

        $app = KSystem::getInstance( );
        $name = array_key_exists( 'name', $_POST ) ? $_POST['name'] : null;
        $reply = array( 'success' => false, 'message' => $app->translate( 'ss7-mtp3-route-error-delete' ) );

        try
        {
            if( is_null( $name ) )
            {
                throw new Exception( 'ss7-mtp3-route-error-not-found' );
            }
            $ss7 = new SS7( );
            if( $ss7->deleteMtp3Route( $name ) )
            {
                $ss7->save( );
                $reply = array( 'success' => true, 'message' => $app->translate( 'ss7-mtp3-route-deleted' ) );
            }
            else
            {
                throw new Exception( 'ss7-mtp3-route-error-not-found' );
            }
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }
        return JSONencode( $reply );
    }

    /**
     * Save MTP3 Route
     */
    function saveRoute( )
    {
        require_once KROOT . '/json_util.php';

        $app = KSystem::getInstance( );
        $reply = array( 'success' => false, 'message' => $app->translate( 'ss7-mtp3-route-error-save' ) );

        try
        {
            $ss7 = new SS7( );
            $name = array_key_exists( '__name__', $_POST ) ? $_POST['__name__'] : null;
            $oldName = array_key_exists( '__oldname__', $_POST ) ? $_POST['__oldname__'] : '';

            unset( $_POST['__name__'] );
            unset( $_POST['__oldname__'] );

            $ss7->setMtp3Route( $name, $_POST, $oldName );
            $ss7->save( );
            $reply = array( 'success' => true, 'message' => $app->translate( 'ss7-mtp3-route-saved' ) );
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }
        return JSONencode( $reply );
    }
