<?php
    require_once KCONFIG . '/classes/classKService.php';

    function loadSystemYAML( $configDir = '' )
    {
        Global $sistema;

        if( $configDir == '' )
            $configDir = $sistema->getConfigDir( );

        $filename = $configDir . '/system.yaml';

        if( !is_file( $filename ) || filesize( $filename ) < 1 )
            copy( dirname( __FILE__ ) . "../../defaultConfig/defaultSystem.yaml", $configDir . "/system.yaml" );

        return new KYaml( $filename );
    }

    function loadKIBSYAML( $configDir = '' )
    {
        Global $sistema;

        if( $configDir == '' )
            $configDir = $sistema->getConfigDir( );

        $filename = $configDir . '/kibs.yaml';

        return new KYaml( $filename );
    }
	
    function loadKLSYAML( $configDir = '' )
    {
        Global $sistema;

        if( $configDir == '' )
            $configDir = $sistema->getConfigDir( );

        $filename = $configDir . '/licenseclient.yaml';

        return new KYaml( $filename );
    }

    function getSelectorLoadService( $serviceName )
    {
        Global $sistema;

        $service = new KService( $serviceName );
        $serviceStartupType = $service->getStartupType( );
        $validStartupTypes = $service->getValidTypes( );
        $buff = '';

        $disable = PHP_OS != 'WINNT' ? "disabled='disabled'" : '';

        $buff .= "<select name='load-{$serviceName}' id='load-{$serviceName}' class='selector2' {$disable}>";
        foreach( $validStartupTypes as $validType )
        {
            $selected = $validType == $serviceStartupType ? 'selected' : '';
            $buff .= "<option value='$validType' $selected>" . $sistema->retTranslate( $validType ) . "</option>\n";
        }
        $buff .= '</select>';
        return $buff;
    }

    function loadKLogConfigs( $configDir = '' )
    {

        Global $sistema;

        if( $configDir == '' )
            $configDir = $sistema->getConfigDir( );

        $logConfigs = array( );

        $fileContent = file( $configDir . "/klog.cfg" );
        $sectionName = '';
        for( $x = 0; $x < sizeof( $fileContent ); $x++ )
        {
            if( @ereg( "\[", $fileContent[$x] ) )
            {

                $sectionName = explode( "]", $fileContent[$x] );
                $sectionName = explode( "[", $sectionName[0] );
                $sectionName = $sectionName[1];
            }
            elseif( @ereg( "=", $fileContent[$x] ) )
            {
                $varName = explode( "=", $fileContent[$x] );
                $value = trim( $varName[1] );
                $logConfigs[$sectionName . $varName[0]] = $value;
            }
        }
        return $logConfigs;
    }

    function getFXODevices( )
    {
        Global $sistema;
        $FXOChannels = $sistema->getFXOChannels( );
        $FXOChannelsSize = sizeof( $FXOChannels );
        $return = "";
        $return .= "<select id='fxodevice' onChange='selectFXODeviceChannels(this.value);'>";

        for( $i = 0; $i < $FXOChannelsSize; $i++ )
            $return .= "<option value='$FXOChannels[$i]'>" . $FXOChannels[$i] . "</option>";

        $return .= "</select>";
        return $return;
    }

    function getFXODeviceChannels( $device )
    {
        Global $sistema;
        $FXOChannels = $sistema->getFXODeviceChannels( $device );
        $FXOChannels = explode( '-', $FXOChannels );
        $return = "";
        $return .= "<select id='fxoDeviceChannel'>";
        for( $i = $FXOChannels[0]; $i <= $FXOChannels[1]; $i++ )
            $return .= "<option value='$i'>Channel " . $i . "</option>";

        $return .= "</select>";
        return $return;
    }

    function getDevicelinks( $device )
    {
        Global $sistema;

        $links = $sistema->getDeviceLinksList( $device );
        $return = "<option value=''>" . $sistema->retTranslate( 'SelectValue' ) . "</option>";
        for( $i = 0; $i < sizeof( $links ); $i++ )
            $return .= "<option value='$links[$i]'>$links[$i]</option>";
        return $return;
    }

    function getDeviceChannels( $device, $sChannel, $eChannel = '' )
    {
        Global $sistema;
        $channels = $sistema->getDeviceChannelList( $device );
        $return = "<option value=''>" . $sistema->retTranslate( 'SelectValue' ) . "</option>";
        $toSelect = $sChannel;
        if( $eChannel != '' )
            $toSelect = $eChannel;
        for( $i = $sChannel; $i < sizeof( $channels ); $i++ )
        {
            if( $i == $toSelect )
                $selected = "selected";
            else
                $selected = "";

            $return .= "<option value='$channels[$i]' $selected>$channels[$i]</option>";
        }
        return $return;
    }

    function createWC( $type, $wc )
    {
        switch($type)
        {
            case 'ACAnalyzer':
                $sysYAML = loadSystemYaml( );
                $WC = $sysYAML->getValue( 'AutoCallAnalyzer' );

                if( $WC != '' )
                {
                    $tmpWC = str_replace( ' ', '', $WC );
                    $arr_WC = explode( ',', $tmpWC );
                    if( !in_array( $wc, $arr_WC ) )
                    {
                        $WC .= ', ' . $wc;
                        $sysYAML->changeVarValue( 'AutoCallAnalyzer', $WC, 'Features' );
                    }
                }
                else
                    $sysYAML->changeVarValue( 'AutoCallAnalyzer', $wc, 'Features' );

                $sysYAML->save( );
                break;

            case 'AECancel':
                $sysYAML = loadSystemYaml( );
                $AECancel = $sysYAML->getValue( 'AutoEchoCanceller' );
                if( $AECancel != '' )
                {
                    $tmpAE = str_replace( ' ', '', $AECancel );
                    $arr_AE = explode( ',', $tmpAE );
                    if( !in_array( $wc, $arr_AE ) )
                    {
                        $AECancel .= ', ' . $wc;
                        $sysYAML->changeVarValue( 'AutoEchoCanceller', $AECancel, 'Features' );
                    }
                }
                else
                    $sysYAML->changeVarValue( 'AutoEchoCanceller', $wc, 'Features' );

                $sysYAML->save( );
                break;

            case 'ADSup':
                $sysYAML = loadSystemYaml( );
                $AECancel = $sysYAML->getValue( 'AutoDtmfSuppression' );
                if( $AECancel != '' )
                {
                    $tmpAE = str_replace( ' ', '', $AECancel );
                    $arr_AE = explode( ',', $tmpAE );
                    if( !in_array( $wc, $arr_AE ) )
                    {
                        $AECancel .= ', ' . $wc;
                        $sysYAML->changeVarValue( 'AutoDtmfSuppression', $AECancel, 'Features' );
                    }
                }
                else
                    $sysYAML->changeVarValue( 'AutoDtmfSuppression', $wc, 'Features' );

                $sysYAML->save( );
                break;

            case 'AAGC':
                $sysYAML = loadSystemYaml( );
                $AECancel = $sysYAML->getValue( 'AutoAGC' );
                if( $AECancel != '' )
                {
                    $tmpAE = str_replace( ' ', '', $AECancel );
                    $arr_AE = explode( ',', $tmpAE );
                    if( !in_array( $wc, $arr_AE ) )
                    {
                        $AECancel .= ', ' . $wc;
                        $sysYAML->changeVarValue( 'AutoAGC', $AECancel, 'Features' );
                    }
                }
                else
                    $sysYAML->changeVarValue( 'AutoAGC', $wc, 'Features' );

                $sysYAML->save( );

                break;

            case 'DCCall':
                $sysYAML = loadSystemYaml( );
                $DCCalls = $sysYAML->getValue( 'DropCollectCalls' );
                $DCCalls[$wc] = array( 'AutoBlock' => false, 'BlockResponse' => 'Busy', 'DropCallAnalyzer' => false, 'AutoDoubleAnswer' => false, 'DoubleAnswerDelay' => 500 );
                $DCCalls = precedSort( $DCCalls );
                $sysYAML->changeVarValue( 'DropCollectCalls', $DCCalls );
                $sysYAML->save( );
                break;
        }
    }

    function removeWC( $type, $wc )
    {
        switch($type)
        {
            case 'ACAnalyzer':
                removeACAnalyzer( $wc );
                break;
            case 'AECancel':
                removeAECancel( $wc );
                break;
            case 'ADSup':
                removeADSupression( $wc );
                break;
            case 'AAGC':
                removeAAGC( $wc );
                break;
            case 'DCCall':
                deleteDCCall( $wc );
                break;
        }
    }

    function saveDCCallConfigs( $data )
    {
        $SysYAML = loadSystemYAML( );
        $yaml = $SysYAML->yaml;

        $data = explode( ',', $data );
        foreach( $data as $d )
        {
            $dcc = explode( '|', $d );
            for( $i = 1; $i < sizeof( $dcc ); $i++ )
            {
                $cv = explode( ';', $dcc[$i] );
                $yaml['Features']['DropCollectCalls'][$dcc[0]][$cv[0]] = $cv[1];
            }
        }
        $SysYAML->yaml = $yaml;
        $SysYAML->save( );
    }

    function removeACAnalyzer( $wc )
    {
        $SysYAML = loadSystemYAML( );
        $aca = $SysYAML->getValue( 'AutoCallAnalyzer' );
        $finalACA = explode( ", ", $aca );

        $tmpACA = array( );

        for( $i = 0; $i < sizeof( $finalACA ); $i++ )
        {
            if( $finalACA[$i] != $wc )
                $tmpACA[] = $finalACA[$i];
        }

        if( sizeof( $tmpACA ) == 1 )
            $tmpACA = $tmpACA[0];
        elseif( sizeof( $tmpACA ) == 0 )
            $tmpACA = '';
        else
            $tmpACA = implode( ", ", $tmpACA );

        $SysYAML->changeVarValue( 'AutoCallAnalyzer', $tmpACA );
        $SysYAML->save( );
    }

    function removeAECancel( $wc )
    {
        $SysYAML = loadSystemYAML( );
        $AECancel = $SysYAML->getValue( 'AutoEchoCanceller' );
        $finalACE = explode( ", ", $AECancel );

        $nAECancel = array();

        for( $i = 0; $i < sizeof( $finalACE ); $i++ )
        {
            if( $finalACE[$i] != $wc )
                $nAECancel[] = $finalACE[$i];
        }

        if( sizeof( $nAECancel ) == 1 )
            $nAECancel = $nAECancel[0];
        elseif( sizeof( $nAECancel ) == 0 )
            $nAECancel = '';
        else
            $nAECancel = implode( ", ", $nAECancel );
        $SysYAML->changeVarValue( 'AutoEchoCanceller', $nAECancel );
        $SysYAML->save( );
    }

    function removeADSupression( $wc )
    {
        $SysYAML = loadSystemYAML( );
        $ADS = $SysYAML->getValue( 'AutoDtmfSuppression' );

        $ADS = explode( ", ", $ADS );
        $nADS = array();

        for( $i = 0; $i < sizeof( $ADS ); $i++ )
        {
            if( $ADS[$i] != $wc )
                $nADS[] = $ADS[$i];
        }

        if( sizeof( $nADS ) == 1 )
            $nADS = $nADS[0];
        elseif( sizeof( $nADS ) == 0 )
            $nADS = '';
        else
            $nADS = implode( ", ", $nADS );
        $SysYAML->changeVarValue( 'AutoDtmfSuppression', $nADS );
        $SysYAML->save( );
    }

    function removeAAGC( $wc )
    {
        $SysYAML = loadSystemYAML( );
        $AGC = $SysYAML->getValue( 'AutoAGC' );
        $AGC = explode( ", ", $AGC );

        $nAGC = array();

        for( $i = 0; $i < sizeof( $AGC ); $i++ )
        {
            if( $AGC[$i] != $wc )
                $nAGC[] = $AGC[$i];
        }

        if( sizeof( $nAGC ) == 1 )
            $nAGC = $nAGC[0];
        elseif( sizeof( $nAGC ) == 0 )
            $nAGC = '';
        else
            $nAGC = implode( ", ", $nAGC );

        $SysYAML->changeVarValue( 'AutoAGC', $nAGC );
        $SysYAML->save( );
    }

    function deleteDCCall( $wc )
    {
        $SysYAML = loadSystemYAML( );
        $DCC = $SysYAML->getValue( 'DropCollectCalls' );
        if( isset( $DCC[$wc] ) )
        {
            $SysYAML->deleteVar( $wc, 'Features|DropCollectCalls' );
            $SysYAML->save( );
        }
    }

    function precedSort( $sortable )
    {
        $newArray = array( );

        $keys = array_keys( $sortable );
        $size = sizeof( $keys );
        for( $j = 0; $j < 5; $j++ )
        {
            for( $i = 0; $i < $size; $i++ )
            {
                $subPart = explode( ".", $keys[$i] );
                switch($j)
                {
                    case 0:
                        //channel
                        if( @ereg( "\[", $subPart[1] ) )
                            $newArray[$keys[$i]] = $sortable[$keys[$i]];
                        break;

                    case 1:
                        //link
                        if( $subPart[0] != "*" && $subPart[1] != "*" && !@ereg( "\[", $subPart[1] ) )
                            $newArray[$keys[$i]] = $sortable[$keys[$i]];
                        break;

                    case 2:
                        //device
                        if( $subPart[0] != "*" && $subPart[1] == "*" )
                            $newArray[$keys[$i]] = $sortable[$keys[$i]];
                        break;

                    case 3:
                        //tech
                        if( $subPart[1] == "FXO" || $subPart[1] == "FXS" || $subPart[1] == "E1" || $subPart[1] == "GSM" )
                            $newArray[$keys[$i]] = $sortable[$keys[$i]];
                        break;

                    case 4:
                        //all
                        if( $subPart[0] == "*" && $subPart[1] == "*" )
                            $newArray[$keys[$i]] = $sortable[$keys[$i]];
                        break;
                }
            }
        }
        return $newArray;
    }

    function getBooleanValue( $string )
    {
        if( $string == 'true' )
            return TRUE;
        else
            return false;
    }

    //**************************** telephony

    function printProfile( $profile, $type, $name, $subType, $edit = "" )
    {
        switch($type)
        {
            case 'R2':
                $return = printR2( $profile, $name, $edit, $subType );
                break;
            case 'ISDN':
                $return = printDefaultIsdn( $profile, $name, $edit, $subType );
                break;
            case 'ISUP':
                $return = printDefaultIsup( $profile, $name, $edit );
                break;
            case 'GSM':
                $return = printDefaultGSM( $profile, $name, $edit );
                break;
            case 'FXS':
                $return = printDefaultFXS( $profile, $name, $edit );
                break;
            case 'CAS PBX':
                $return = printDefaultLineSide( $profile, $name, $edit, $subType );
                break;
            case 'FXO':
                $return = printDefaultFXO( $profile, $name, $edit );
                break;
            default:
                break;
        }
        return $return;
    }

    function printR2( $profile, $name, $edit, $subType )
    {
        Global $sistema;
        $return = "";
        $return .= "<table class='zebra' id='defaultProfile'><tbody>";

        $return .= "<tr><td align='left'><label for='name'>" . $sistema->retTranslate( 'Nome' ) . "</label></td>";
        $return .= "<td width='15%' align='center'>";
        if( $edit == "" )
        {
            $profileName = $name . "_copy";
            while( testProfile( $profileName ) != "Ok" )
                $profileName .= "_copy";
            $return .= " <input class='selector2' name='name' id='name' value='$profileName' type='text' >";
        }
        else
            $return .= $name . " <input type='hidden' id='name' name='name' value='$name'/><span id='profileAnswer'></span>";

        $return .= "</td></tr>";

        $return .= "<tr><td align='left'>" . $sistema->retTranslate( 'Type' ) . "</td>";
        $return .= "<td width='15%' align='center'>" . $profile[$name]['Type'] . "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'><label for='SubType'>" . $sistema->retTranslate( 'SubType' ) . "</label></td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['SubType'];
        $return .= "<select class='selector2' name='SubType' id='SubType' >";
        $possibleValues = array( 'MFC' => 'MFC', 'Passive' => 'Passive' );
        $keys = array_keys( $possibleValues );
        for( $x = 0; $x <= 1; $x++ )
        {
            $selected = ( $value == $keys[$x] ) ? 'selected' : '';
            $return .= "<option value='$keys[$x]' $selected>" . $keys[$x] . "</option>";
        }
        $return .= "</select>";
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $ForcedDisconnection = $profile[$name]['ForcedDisconnection'];
        $checked = $ForcedDisconnection ? 'checked' : '';

        $return .= '<label for="ForcedDisconnection">' . $sistema->retTranslate( 'ForcedDisconnection' ) . '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='center'>";
        $return .= "<input class='check' name='ForcedDisconnection' id='ForcedDisconnection' value='' type='checkbox' $checked>";
        $return .= "</td>";
        $return .= "</tr>";
        $checked = "";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="SeizeConfirmationTimeout">' . $sistema->retTranslate( 'SeizeConfirmationTimeout' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['SeizeConfirmationTimeout'];
        $return .= intSelectMaker( 'SeizeConfirmationTimeout', '', $value, 1000, 160000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="DestEoNStandard">' . $sistema->retTranslate( 'DestEoNStandard' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $DestEoNStandardvalue = isset($profile[$name]['DestEoNStandard']) ? $profile[$name]['DestEoNStandard'] : 0;
        $return .= "<select id='DestEoNStandard' class='selector2' name='DestEoNStandard'>";

        switch($DestEoNStandardvalue)
        {
            //waitA3
            case 0:
                $return .= "<option selected='' value='0'>" . $sistema->retTranslate( 'waitA3' ) . "</option>";
                $return .= "<option value='1'>" . $sistema->retTranslate( 'sendI15' ) . "</option>";
                $return .= "<option value='2'>" . $sistema->retTranslate( 'sendI12' ) . "</option>";
                break;

            //sendI15
            case 1:
                $return .= "<option value='0'>" . $sistema->retTranslate( 'waitA3' ) . "</option>";
                $return .= "<option selected='' value='1'>" . $sistema->retTranslate( 'sendI15' ) . "</option>";
                $return .= "<option value='2'>" . $sistema->retTranslate( 'sendI12' ) . "</option>";
                break;

            //sendI12
            default:
                $return .= "<option value='0'>" . $sistema->retTranslate( 'waitA3' ) . "</option>";
                $return .= "<option value='1'>" . $sistema->retTranslate( 'sendI15' ) . "</option>";
                $return .= "<option selected='' value='2'>" . $sistema->retTranslate( 'sendI12' ) . "</option>";
                break;
        }

        $return .= "</select>";
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="OrigEoNStandard">' . $sistema->retTranslate( 'OrigEoNStandard' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $OrigEoNStandardvalue = isset( $profile[$name]['OrigEoNStandard'] ) ? $profile[$name]['OrigEoNStandard'] : true;
        $return .= "<select id='OrigEoNStandard' class='selector2' name='OrigEoNStandard'>";

        switch ($OrigEoNStandardvalue)
        {
            case true:
                $return .= "<option selected='' value='true'>" . $sistema->retTranslate( 'sendI15' ) . "</option>";
                $return .= "<option value='false'>" . $sistema->retTranslate( 'sendI12' ) . "</option>";
                break;

            default:
                $return .= "<option value='true'>" . $sistema->retTranslate( 'sendI15' ) . "</option>";
                $return .= "<option selected='' value='false'>" . $sistema->retTranslate( 'sendI12' ) . "</option>";
                break;
        }

        $return .= "</select>";
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="AnswerSignalReceptionTimeout">' . $sistema->retTranslate( 'AnswerSignalReceptionTimeout' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['AnswerSignalReceptionTimeout'];
        $return .= intSelectMaker( 'AnswerSignalReceptionTimeout', '', $value, 1000, 160000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="DisconnectConfirmationTime">' . $sistema->retTranslate( 'DisconnectConfirmationTime' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['DisconnectConfirmationTime'];
        $return .= intSelectMaker( 'DisconnectConfirmationTime', '', $value, 1000, 160000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="SeizeTimeout">' . $sistema->retTranslate( 'SeizeTimeout' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['SeizeTimeout'];
        $return .= intSelectMaker( 'SeizeTimeout', '', $value, 1000, 160000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="R2MinLineSendTime">' . $sistema->retTranslate( 'R2MinLineSendTime' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['R2MinLineSendTime'];
        $return .= intSelectMaker( 'R2MinLineSendTime', '', $value, 0, 2000, 5 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="BillingPulseMaxTime">' . $sistema->retTranslate( 'BillingPulseMaxTime' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['BillingPulseMaxTime'];
        $return .= intSelectMaker( 'BillingPulseMaxTime', '', $value, 0, 2000, 5 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="IdentityRequestPosition">' . $sistema->retTranslate( 'IdentityRequestPosition' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['IdentityRequestPosition'];
        $return .= intSelectMaker( 'IdentityRequestPosition', '', $value, 1, 32, 1 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="ForwardMfcReceptionTimeout">' . $sistema->retTranslate( 'ForwardMfcReceptionTimeout' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['ForwardMfcReceptionTimeout'];
        $return .= intSelectMaker( 'ForwardMfcReceptionTimeout', '', $value, 1000, 160000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="ForwardMfcSendingTimeout">' . $sistema->retTranslate( 'ForwardMfcSendingTimeout' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['ForwardMfcSendingTimeout'];
        $return .= intSelectMaker( 'ForwardMfcSendingTimeout', '', $value, 1000, 160000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="ForwardMfcSilenceTimeout">' . $sistema->retTranslate( 'ForwardMfcSilenceTimeout' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['ForwardMfcSilenceTimeout'];
        $return .= intSelectMaker( 'ForwardMfcSilenceTimeout', '', $value, 1000, 160000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="MfcPulseTime">' . $sistema->retTranslate( 'MfcPulseTime' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['MfcPulseTime'];
        $return .= intSelectMaker( 'MfcPulseTime', '', $value, 0, 2000, 5 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="RingbackSilenceTime">' . $sistema->retTranslate( 'RingbackSilenceTime' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['RingbackSilenceTime'];
        $return .= intSelectMaker( 'RingbackSilenceTime', '', $value, 1000, 160000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="RingbackToneTime">' . $sistema->retTranslate( 'RingbackToneTime' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['RingbackToneTime'];
        $return .= intSelectMaker( 'RingbackToneTime', '', $value, 0, 2000, 5 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="ChannelsDefault">' . $sistema->retTranslate( 'ChannelsDefault' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['Channels']['Default'];
        $return .= "<select name='ChannelsDefault' id='ChannelsDefault' >";
        $possibleValues = array( 1 => $sistema->retTranslate( 'incoming' ), 2 => $sistema->retTranslate( 'outgoing' ), 3 => $sistema->retTranslate( 'bidirectional' ) );
        for( $x = 1; $x <= 3; $x++ )
        {
            $selected = ($value == $x) ? "selected" : "";
            $return .= "<option value='$x' $selected>" . $possibleValues[$x] . "</option>";
        }

        $return .= "</select>";
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td colspan='2' >";
        $return .= "<div id='ManuallyChannelsLink'>";

        $return .= "<a href='#' onclick='showManuallyChannels();'>";
        $return .= $sistema->retTranslate( 'Channels' ) . "</a></div>";
        $showDiv = false;

        $aux = "<table class='canaisManual zebra'><tbody>";
        for( $x = 0; $x < 30; $x++ )
        {
            if( isset( $profile[$name]['Channels'][$x] ) )
                $value = $profile[$name]['Channels'][$x];
            else
                $value = "";

            $entrante = "";
            $sainte = "";
            switch ($value)
            {
                case 1:
                    $entrante = "checked";
                    $showDiv = true;
                    break;
                case 2:
                    $sainte = "checked";
                    $showDiv = true;
                    break;
                case 3:
                    $entrante = "checked";
                    $sainte = "checked";
                    $showDiv = true;
                    break;
            }
            $aux .= '<tr>';
            $aux .= "<td>" . $sistema->retTranslate( 'Channel' ) . " $x </td>";
            $aux .= "<td align='right'><input class='check' name='entrante$x' id='entrante$x' value='1' type='checkbox' $entrante><label for='entrante$x'>" . $sistema->retTranslate( 'incoming' ) . "</label></td>";
            $aux .= "<td width='5%'><td>";
            $aux .= "<td align='left'><input class='check' name='sainte$x' id='sainte$x' value='2' type='checkbox' $sainte><label for='sainte$x'>" . $sistema->retTranslate( 'outgoing' ) . "</label></td>";

            $aux .= "</tr>";
        }
        $aux .= "</tbody></table>";

        $style = $showDiv ? 'style="display: block;"' : 'style="display: none;"';
        $return .= "<div id='ManuallyChannels' $style>";
        $return .= $aux;
        $return .= "</div>";
        $return .= '</td></tr></tbody></table></br>';

        $JSParam = $edit == "" ? "'copy'" : "'edit'";
        $return .= '<div class="floatRight">';
        $return .= "<input value='" . $sistema->retTranslate( "Cancelar" ) . "' onclick='listProfiles();' type='button'>";
        $return .= "<input value='" . $sistema->retTranslate( "Salvar" ) . "' onclick=\"createR2($JSParam);\" type='button'>";
        $return .= '</div>';

        return $return;
    }

    function printDefaultIsdn( $profile, $name, $edit, $subType )
    {
        Global $sistema;
        $return = "";
        $return .= "<table class='zebra' id='defaultProfile'><tbody>";

        $return .= "<tr><td width='85%' align='left'>";
        $return .= '<label for="name">' . $sistema->retTranslate( 'Nome' ) . '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";

        if( $edit == "" )
        {
            $profileName = $name . "_copy";
            while( testProfile( $profileName ) != "Ok" )
                $profileName .= "_copy";
            $return .= " <input class='selector2' name='name' id='name' value='$profileName' type='text' >";
        }
        else
            $return .= $name . "<input type='hidden' id='name' name='name' value='$name'/><span class='hidden' id='profileAnswer'></span>";

        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td width='85%' align='left'>";
        $return .= $sistema->retTranslate( 'Type' );
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";
        $return .= $profile[$name]['Type'];
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td width='85%' align='left'>";
        $return .= '<label for="SubType">' . $sistema->retTranslate( 'SubType' ) . '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";
        $value = $profile[$name]['SubType'];
        $possibleValues = array( 'EndPoint' => 'EndPoint', 'Network' => 'Network', 'Passive' => 'Passive' );
        $return .= make_select('SubType', $possibleValues, $value);
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td width='85%' align='left'>";
        $checked = ( $profile[$name]['AllowTransmissionInTimeoutRecovery'] ) ? 'checked' : '';
        $return .= '<label for="AllowTransmissionInTimeoutRecovery">' . $sistema->retTranslate( 'AllowTransmissionInTimeoutRecovery' ) . '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";
        $return .= "<input class='check' name='AllowTransmissionInTimeoutRecovery' id='AllowTransmissionInTimeoutRecovery' value='' type='checkbox' $checked>";
        $return .= "</td>";
        $return .= "</tr>";
        $checked = "";

        $return .= "<tr><td width='85%' align='left'>";
        $return .= '<label for="TimeslotOffsetOption">' . $sistema->retTranslate( 'TimeslotOffsetOption' ) . '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";
        $value = $profile[$name]['TimeslotOffsetOption'];
        $possibleValues = array('0 - Padrão(1-15 17-31)', '1 - Modificado (1-30)');
        $return .= make_select('TimeslotOffsetOption', $possibleValues, $value);
        $return .= "</td>";
        $return .= "</tr>";

        $BearerCapabilityOptions = array( "0x00 - Speech", "0x08 - Unrestricted Digital", "0x10 - Audio 3.1 KHz", "0x11 - Audio 7 KHz", "0x18 - Video" );
        $HLCOptions = array( "0x00 - Não Enviar", "0x81 - Telephony", "0x84 - Fax Group 23", "0xa1 - Fax Group 4", "0xa4 - Teletex F.184", "0xa8 - Teletex F.220", "0xb1 - Teletex F.200", "0xb2 - Videotex", "0xb5 - Teletex F.60", "0xb8 - MHS", "0xc1 - OSI Application", "0xde - Maintenance", "0xdf - Management" );
        $RestartProcedure = array('single', 'none', 'multiple', 'slotmap');
        $TypeOfNumberOptions = array('0x00 - Unknown', '0x10 - International', '0x20 - National', '0x30 - Network Specific', '0x40 - Subscriber', '0x60 - Abbreviated', '0x70 - Reserved' );
        $NumberingPlanOptions = array('0x00 - Unknown', '0x01 - Isdn', '0x03 - Data', '0x04 -Telex', '0x08 - National', '0x09 - Private', '0x0F - Reserved' );
        $PresentationOptions = array('0x00 - Presentation Allowed', '0x01 - Presentation Restricted', '0x02 - Number Not Available Due To Interworking', '0x03 - Reserved' );
        $ScreeningOptions = array('0x00 - UserProvidedNotScreened', '0x01 - UserProvidedVerifiedAndPassed', '0x02 - UserProvidedVerifiedAndFailed', '0x03 - NetworkProvided' );

        $return .= "<tr><td width='85%' align='left'>";
        $return .= '<label for="RestartProcedureOnLinkEstablish">'. $sistema->retTranslate('RestartProcedureOnLinkEstablish'). '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";
        $return .= make_select('RestartProcedureOnLinkEstablish', $RestartProcedure, $profile[$name]['RestartProcedureOnLinkEstablish']);
        $return .= "</td>";
        $return .= "</tr>";
        $return .= '</tbody></table>';

        $return .= '<br/><fieldset><legend>'. $sistema->retTranslate('SETUPOptions'). '</legend>';

        $return .= '<table class="zebra"><tbody>';
        $return .= "<tr><td width='85%' align='left'>";
        $checked = ( $profile[$name]['AlwaysSendChannelId'] ) ? 'checked' : '';
        $return .= '<label for="AlwaysSendChannelId">' . $sistema->retTranslate( 'AlwaysSendChannelId' ) . '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";
        $return .= "<input class='check' name='AlwaysSendChannelId' id='AlwaysSendChannelId' value='' type='checkbox' $checked>";
        $return .= "</td>";
        $return .= "</tr>";
        $checked = "";

        $return .= "<tr><td width='85%' align='left'>";
        $return .= '<label for="DefaultBearerCapability">' . $sistema->retTranslate( 'DefaultBearerCapability' ) . '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";
        $value = $profile[$name]['DefaultBearerCapability'];
        $return .= make_select('DefaultBearerCapability', $BearerCapabilityOptions, $value);
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td width='85%' align='left'>";
        $return .= '<label for="DefaultHLC">' . $sistema->retTranslate( 'DefaultHLC' ) . '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";
        $value = $profile[$name]['DefaultHLC'];
        $return .= make_select('DefaultHLC' , $HLCOptions, $value);
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td width='85%' align='left'>";
        $return .= '<label for="DefaultOrigTypeOfNumber">'. $sistema->retTranslate('DefaultOrigTypeOfNumber'). '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";
        $return .= make_select('DefaultOrigTypeOfNumber', $TypeOfNumberOptions, $profile[$name]['DefaultOrigTypeOfNumber']);
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td width='85%' align='left'>";
        $return .= '<label for="DefaultOrigNumberingPlan">'. $sistema->retTranslate('DefaultOrigNumberingPlan'). '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";
        $return .= make_select('DefaultOrigNumberingPlan', $NumberingPlanOptions, $profile[$name]['DefaultOrigNumberingPlan']);
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td width='85%' align='left'>";
        $return .= '<label for="SendOrigPresentationAndScreening">'. $sistema->retTranslate('SendOrigPresentationAndScreening'). '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";
        $checked = $profile[$name]['SendOrigPresentationAndScreening'] ? 'checked' : '';
        $return .= "<input class='check' name='SendOrigPresentationAndScreening' id='SendOrigPresentationAndScreening' value='' type='checkbox' $checked>";
        $return .= "</td>";
        $return .= "</tr>";
        $checked = '';

        $return .= "<tr><td width='85%' align='left'>";
        $return .= '<label for="DefaultOrigPresentation">'. $sistema->retTranslate('DefaultOrigPresentation'). '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";
        $return .= make_select( 'DefaultOrigPresentation', $PresentationOptions, $profile[$name]['DefaultOrigPresentation']);
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td width='85%' align='left'>";
        $return .= '<label for="DefaultOrigScreening">'. $sistema->retTranslate('DefaultOrigScreening'). '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";
        $return .= make_select( 'DefaultOrigScreening', $ScreeningOptions, $profile[$name]['DefaultOrigScreening']);
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td width='85%' align='left'>";
        $return .= '<label for="DefaultDestTypeOfNumber">'. $sistema->retTranslate('DefaultDestTypeOfNumber'). '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";
        $return .= make_select( 'DefaultDestTypeOfNumber', $TypeOfNumberOptions, $profile[$name]['DefaultDestTypeOfNumber']);
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td width='85%' align='left'>";
        $return .= '<label for="DefaultDestNumberingPlan">'. $sistema->retTranslate('DefaultDestNumberingPlan'). '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";
        $return .= make_select( 'DefaultDestNumberingPlan', $NumberingPlanOptions, $profile[$name]['DefaultDestNumberingPlan'] );
        $return .= "</td>";
        $return .= "</tr>";
        $return .= '</tbody></table>';
        $return .= '</fieldset>';

        $return .= '<br/><fieldset><legend>'. $sistema->retTranslate( 'IsdnTimers' ) . '</legend>';
        //$return .= "<tr><td colspan='2' width='100%' align='left'>";
        //$return .= $sistema->retTranslate( 'IsdnTimers' ) . ": ";

        $return .= "<table class='zebra'><tbody>";

        $return .= "<tr><td width='85%' align='left'>";
        $return .= '<label for="RejectRecovery">' . $sistema->retTranslate( 'LAPDtRejectRecovery' ) . '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='right'>";
        $value = $profile[$name]['IsdnTimers']['RejectRecovery'];
        $return .= intSelectMaker( 'RejectRecovery', '', $value, 1000, 180000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="SSCTt1">' . $sistema->retTranslate( 'SSCTt1' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['IsdnTimers']['SSCTt1'];
        $return .= intSelectMaker( 'SSCTt1', '', $value, 1000, 180000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="CTt1">' . $sistema->retTranslate( 'CTt1' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['IsdnTimers']['CTt1'];
        $return .= intSelectMaker( 'CTt1', '', $value, 1000, 180000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="CTt3">' . $sistema->retTranslate( 'CTt3' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['IsdnTimers']['CTt3'];
        $return .= intSelectMaker( 'CTt3', '', $value, 1000, 180000, 1000 );
        $return .= '</td>';
        $return .= '</tr>';

        $return .= "<tr><td align='left'>";
        $return .= '<label for="T301">' . $sistema->retTranslate( 'Q931t301' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['IsdnTimers']['T301'];
        $return .= intSelectMaker( 'T301', '', $value, 1000, 180000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="T302">' . $sistema->retTranslate( 'Q931t302' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['IsdnTimers']['T302'];
        $return .= intSelectMaker( 'T302', '', $value, 1000, 180000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="T303">' . $sistema->retTranslate( 'Q931t303' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['IsdnTimers']['T303'];
        $return .= intSelectMaker( 'T303', '', $value, 1000, 180000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="T304">' . $sistema->retTranslate( 'Q931t304' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['IsdnTimers']['T304'];
        $return .= intSelectMaker( 'T304', '', $value, 1000, 180000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="T305">' . $sistema->retTranslate( 'Q931t305' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['IsdnTimers']['T305'];
        $return .= intSelectMaker( 'T305', '', $value, 1000, 180000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="T308">' . $sistema->retTranslate( 'Q931t308' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['IsdnTimers']['T308'];
        $return .= intSelectMaker( 'T308', '', $value, 1000, 180000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="T309">' . $sistema->retTranslate( 'Q931t309' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['IsdnTimers']['T309'];
        $return .= intSelectMaker( 'T309', '', $value, 1000, 180000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="T310">' . $sistema->retTranslate( 'Q931t310' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['IsdnTimers']['T310'];
        $return .= intSelectMaker( 'T310', '', $value, 1000, 180000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="T313">' . $sistema->retTranslate( 'Q931t313' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['IsdnTimers']['T313'];
        $return .= intSelectMaker( 'T313', '', $value, 1000, 180000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="T200">' . $sistema->retTranslate( 'LAPDt200' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['IsdnTimers']['T200'];
        $return .= intSelectMaker( 'T200', '', $value, 1000, 180000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="T203">' . $sistema->retTranslate( 'LAPDt203' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['IsdnTimers']['T203'];
        $return .= intSelectMaker( 'T203', '', $value, 1000, 180000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";
        $return .= "</tbody></table>";

        $return .= '</fieldset>';

        // $return .= "</td>";
        // $return .= "</tr>";
        // $return .= "</tbody></table><br/>";

        $JSParam = $edit == "" ? "'copy'" : "'edit'";

        $return .= '<br/><div class="floatRight">';
        $return .= "<input value='" . $sistema->retTranslate( "Cancelar" ) . "' onclick='listProfiles();' type='button'>";
        $return .= "<input value='" . $sistema->retTranslate( "Salvar" ) . "' onclick=\"createISDN($JSParam);\" type='button'>";
        $return .= '</div>';

        return $return;
    }

    function printDefaultGSM( $profile, $name, $edit )
    {
        Global $sistema;
        $return = "";
        $return .= "<table class='zebra' id='defaultProfile'><tbody>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="name">' . $sistema->retTranslate( 'Nome' ) . '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='center'>";

        if( $edit == "" )
        {
            $profileName = $name . "_copy";
            while( testProfile( $profileName ) != "Ok" )
                $profileName .= "_copy";

            $return .= " <input class='selector2' name='name' id='name' value='$profileName' type='text' >";
        }
        else
            $return .= $name . "<input type='hidden' id='name' name='name' value='$name'/><span class='hidden' id='profileAnswer'></span>";

        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= $sistema->retTranslate( 'Type' );
        $return .= "</td>";
        $return .= "<td width='15%' align='center'>";
        $return .= $profile[$name]['Type'];
        $return .= "</td>";
        $return .= "</tr>";

        $checked = "";

        $SendDtmfBySignaling = $profile[$name]['SendDtmfBySignaling'];
        $checked = ( $SendDtmfBySignaling ) ? 'checked' : '';
        $return .= '<tr><td align="left"><label for="SendDtmfBySignaling">'. $sistema->retTranslate( 'SendDtmfBySignaling' ).'</label></td>';
        $return .= '<td width="15%" align="center">';
        $return .= "<input class='check' name='SendDtmfBySignaling' id='SendDtmfBySignaling' value='' type='checkbox' $checked>";
        $return .= "</td>";
        $return .= "</tr>";
        $checked = "";

        $HexEncodeReceived8bitSms = $profile[$name]['HexEncodeReceived8bitSms'];
        $checked = ( $HexEncodeReceived8bitSms ) ? 'checked' : '';
        $return .= '<tr><td align="left"><label for="HexEncodeReceived8bitSms">'. $sistema->retTranslate( 'HexEncodeReceived8bitSms' ).'</label></td>';
        $return .= '<td width="15%" align="center">';
        $return .= "<input class='check' name='HexEncodeReceived8bitSms' id='HexEncodeReceived8bitSms' value='' type='checkbox' $checked>";
        $return .= "</td>";
        $return .= "</tr>";
        $checked = "";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="DefaultSmsTextCoding">' . $sistema->retTranslate( 'DefaultSmsTextCoding' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['DefaultSmsTextCoding'];
        $return .= "<select class='selector2' name='DefaultSmsTextCoding' id='DefaultSmsTextCoding' >";
        $possibleValues = array( 'utf-8' => 'utf-8', 'iso8859-1' => 'iso8859-1');
        $keys = array_keys( $possibleValues );
        for( $x = 0; $x <= 1; $x++ )
        {
            $selected = ( $value == $keys[$x] ) ? 'selected' : '';
            $return .= "<option value='$keys[$x]' $selected>" . $keys[$x] . "</option>";
        }
        $return .= "</select>";
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="CheckNetworkTimeOut">' . $sistema->retTranslate( 'CheckNetworkTimeOut' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['CheckNetworkTimeOut'];
        $return .= intSelectMaker( 'CheckNetworkTimeOut', '', $value, 1000, 160000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";
        $JSParam = $edit == "" ? "'copy'" : "'edit'";
        $return .= "</tbody></table><br/>";

        $return .= '<div class="floatRight" >';
        $return .= "<input value='" . $sistema->retTranslate( "Cancelar" ) . "' onclick='listProfiles();' type='button'>";
        $return .= "<input value='" . $sistema->retTranslate( "Salvar" ) . "' onclick=\"createGSM($JSParam);\" type='button'>";
        $return .= '</div>';

        return $return;
    }

    function printDefaultFXS( $profile, $name, $edit )
    {
        $app = KSystem::getInstance();

        $return = "";
        $return .= "<table class='zebra' id='defaultProfile'><tbody>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="name">' . $app->translate( 'Nome' ) . '</label>';
        $return .= "</td><td width='15%' align='center'>";

        if( $edit == "" )
        {
            $profileName = $name . "_copy";
            while( testProfile( $profileName ) != "Ok" )
                $profileName .= "_copy";

            $return .= " <input class='selector2' name='name' id='name' value='$profileName' type='text' >";
        }
        else
            $return .= $name . " <input type='hidden' id='name' name='name' value='$name'/><span class='hidden' id='profileAnswer'></span>";

        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= $app->translate( 'Type' );
        $return .= "</td>";
        $return .= "<td width='15%' align='center'>";
        $return .= $profile[$name]['Type'];
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="CadenceTimesA">' . $app->translate( 'CadenceTimes' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $valueA = $profile[$name]['CadenceTimes'][0];
        $valueB = $profile[$name]['CadenceTimes'][1];
        $valueC = $profile[$name]['CadenceTimes'][2];
        $valueD = $profile[$name]['CadenceTimes'][3];
        $return .= intSelectMaker( 'CadenceTimesA', 'CadenceTime', $valueA, 50, 16000, 50 );
        $return .= intSelectMaker( 'CadenceTimesB', 'CadenceTime', $valueB, 50, 16000, 50 );
        $return .= "<br>";
        $return .= intSelectMaker( 'CadenceTimesC', 'CadenceTime', $valueC, 50, 16000, 50 );
        $return .= intSelectMaker( 'CadenceTimesD', 'CadenceTime', $valueD, 50, 16000, 50 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="FlashThresholdTime">' . $app->translate( 'FlashThresholdTime' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['FlashThresholdTime'];
        $return .= intSelectMaker( 'FlashThresholdTime', 'FlashValidationTime', $value, 0, 5000, 5 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="FlashValidationTime">' . $app->translate( 'FlashValidationTime' ) . '</span>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['FlashValidationTime'];
        $return .= intSelectMaker( 'FlashValidationTime', 'FlashValidationTime', $value, 0, 5000, 5 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="CallerIdProfile">' . $app->translate('callerIdProfileFXS') . '</label>';

        $return .= "</td>";
        $return .= "<td align='right'>";
        $callerid = new KYaml( $app->getConfigDir() . '/callerid.yaml');
        $value = $profile[$name]['CallerIdProfile'];
        $callerYaml = $callerid->getYaml();
        $opts = array();
        foreach (array('DTMFGenerator', 'FSKGenerator' ) as $k => $v)
        {
            foreach ($callerYaml[$v]as $key => $val)
            {
                if(!in_array(key($val), $opts))
                    $opts[] = key($val);
            }
        }
        $return .= '<select id="CallerIdProfile" class="" name="CallerIdProfile">';
        sort($opts);
        foreach ($opts as $k => $v)
        {
            $selected = $v == $value ? 'selected' : '';
            $return .= "<option {$selected} value='{$v}'>{$v}</option>";
        }
        $return .= '</select>';
        $return .= "</td>";
        $return .= "</tr>";

        $callerid = new KYaml( $app->getConfigDir() . '/profiles.yaml');
        $loop = $profile[$name]['LineDistance'];

        $return .= "<tr><td align='left'><label for='LineDistance'>" . $app->translate('LineDistance').'</label></td>';
        $return .= "<td align='right'>";
        $return .= '<select id="LineDistance" name="LineDistance">';

        foreach (array('1 - Normal', '2 - Long') as $k => $v)
        {
            $selected = $v == $loop ? 'selected' : '';
            $return .= "<option {$selected} value='{$v}'>{$v}</option>";
        }
        $return .= '</select></td></tr>';

        $return .= "</tbody></table><br/>";

        $JSParam = $edit == "" ? "'copy'" : "'edit'";

        $return .= '<div class="floatRight" >';
        $return .= "<input value='" . $app->translate( "Cancelar" ) . "' onclick='listProfiles();' type='button'>";
        $return .= "<input value='" . $app->translate( "Salvar" ) . "' onclick=\"createFXS($JSParam);\" type='button'>";
        $return .= '</div>';

        return $return;
    }

    function printDefaultLineSide( $profile, $name, $edit, $subType )
    {
        Global $sistema;
        $return = "";
        $return .= "<table class='zebra' id='defaultProfile'><tbody>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="name">' . $sistema->retTranslate( 'Nome' ) . '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='center'>";

        if( $edit == "" )
        {
            $profileName = $name . "_copy";
            while( testProfile( $profileName ) != "Ok" )
                $profileName .= "_copy";

            $return .= " <input class='selector2' name='name' id='name' value='$profileName' type='text' >";
        }
        else
            $return .= $name . " <input type='hidden' id='name' name='name' value='$name'/><span class='hidden' id='profileAnswer'></span>";

        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= $sistema->retTranslate( 'Type' );
        $return .= "</td>";
        $return .= "<td width='15%' align='center'>";
        $return .= $profile[$name]['Type'];
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="SubType">' . $sistema->retTranslate( 'SubType' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['SubType'];
        $return .= "<select class='selector2' name='SubType' id='SubType' >";
        $possibleValues = array( 'LineSide' => 'LineSide', 'EL7' => 'EL7', 'LC' => 'LC' );
        $keys = array_keys( $possibleValues );
        for( $x = 0; $x < sizeof( $keys ); $x++ )
        {
            $selected = ( $value == $keys[$x] ) ? 'selected' : '';
            $return .= "<option value='$keys[$x]' $selected>" . $keys[$x] . "</option>";
        }
        $return .= "</select>";
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $UseAddressSignaling = $profile[$name]['UseAddressSignaling'];
        $checked = ( $UseAddressSignaling ) ? 'checked' : '';
        $return .= '<label for="UseAddressSignaling">' . $sistema->retTranslate( 'UseAddressSignaling' ) . '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='center'>";
        $return .= "<input class='check' name='UseAddressSignaling' id='UseAddressSignaling' value='' type='checkbox' $checked>";
        $return .= "</td>";
        $return .= "</tr>";
        $checked = "";

        $return .= "<tr><td align='left'>";
        $UseAnswerSupervision = $profile[$name]['UseAnswerSupervision'];
        $checked = ( $UseAnswerSupervision ) ? 'checked' : '';
        $return .= '<label for="UseAnswerSupervision">' . $sistema->retTranslate( 'UseAnswerSupervision' ) . '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='center'>";
        $return .= "<input class='check' name='UseAnswerSupervision' id='UseAnswerSupervision' value='' type='checkbox' $checked>";
        $return .= "</td>";
        $return .= "</tr>";
        $checked = "";

        $return .= "<tr><td align='left'>";
        $UseFarEndDisconnect = $profile[$name]['UseFarEndDisconnect'];
        $checked = ( $UseFarEndDisconnect ) ? 'checked' : '';
        $return .= '<label for="UseFarEndDisconnect">' . $sistema->retTranslate( 'UseFarEndDisconnect' ) . '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='center'>";
        $return .= "<input class='check' name='UseFarEndDisconnect' id='UseFarEndDisconnect' value='' type='checkbox' $checked>";
        $return .= "</td>";
        $return .= "</tr>";
        $checked = "";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="ReleaseTime">' . $sistema->retTranslate( 'ReleaseTime' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['ReleaseTime'];
        $return .= intSelectMaker( 'ReleaseTime', '', $value, 0, 10000, 500 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="AbandonTime">' . $sistema->retTranslate( 'AbandonTime' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['AbandonTime'];
        $return .= intSelectMaker( 'AbandonTime', '', $value, 1000, 16000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="RingsToNotify">' . $sistema->retTranslate( 'RingsToNotify' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['RingsToNotify'];
        $return .= intSelectMaker( 'RingsToNotify', '', $value, 1, 10, 1 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="FlashTime">' . $sistema->retTranslate( 'FlashTime' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['FlashTime'];
        $return .= intSelectMaker( 'FlashTime', '', $value, 100, 16000, 100 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="LongSignalingTime">' . $sistema->retTranslate( 'LongSignalingTime' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['LongSignalingTime'];
        $return .= intSelectMaker( 'LongSignalingTime', '', $value, 0, 16000, 100 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="ShortSignalingTime">' . $sistema->retTranslate( 'ShortSignalingTime' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['ShortSignalingTime'];
        $return .= intSelectMaker( 'ShortSignalingTime', '', $value, 0, 16000, 100 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="SeizeTimeout">' . $sistema->retTranslate( 'SeizeTimeout' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['SeizeTimeout'];
        $return .= intSelectMaker( 'SeizeTimeout', '', $value, 1000, 160000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";
        $return .= "</tbody></table><br/>";

        $JSParam = $edit == "" ? "'copy'" : "'edit'";

        $return .= '<div class="floatRight">';
        $return .= "<input value='" . $sistema->retTranslate( "Cancelar" ) . "' onclick='listProfiles();' type='button'>";
        $return .= "<input value='" . $sistema->retTranslate( "Salvar" ) . "' onclick=\"createLineSide($JSParam);\" type='button'>";
        $return .= '</div>';

        return $return;
    }

    function printDefaultFXO( $profile, $name, $edit )
    {
        Global $sistema;
        $return = "";
        $return .= "<table class='zebra' id='defaultProfile'>";

        $return .= '<tr><td align="left"><label for="name">'.$sistema->retTranslate( 'Nome' ).'</label></td><td width="15%" align="center">';
        if( $edit == '' )
        {
            $profileName = "{$name}_copy";
            while( testProfile( $profileName ) != 'Ok' )
                $profileName .= '_copy';

            $return .= " <input class='selector2' name='name' id='name' value='$profileName' type='text' >";
        }
        else
            $return .= "{$name} <input type='hidden' id='name' name='name' value='$name'/><span class='hidden' id='profileAnswer'></span>";

        $return .= '</td></tr>';

        $return .= "<tr><td align='left'>";
        $return .= $sistema->retTranslate( 'Type' );
        $return .= "</td>";
        $return .= "<td width='15%' align='center'>";
        $return .= $profile[$name]['Type'];
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr>";
        $return .= "<td align='left'>";
        $DisconnectOnPolarityReversal = $profile[$name]['DisconnectOnPolarityReversal'];
        $checked = ( $DisconnectOnPolarityReversal ) ? 'checked' : '';
        $return .= '<label for="DisconnectOnPolarityReversal">' . $sistema->retTranslate( 'DisconnectOnPolarityReversal' ) . '</label>';
        $return .= "</td>";
        $return .= "<td width='15%' align='center'>";
        $return .= "<input class='check' name='DisconnectOnPolarityReversal' id='DisconnectOnPolarityReversal' value='' type='checkbox' $checked>";
        $return .= "</td>";
        $return .= "</tr>";
        $checked = "";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="RingsToNotify">' . $sistema->retTranslate( 'RingsToNotify' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['RingsToNotify'];
        $return .= intSelectMaker( 'RingsToNotify', '', $value, 1, 10, 1 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="SeizeTimeout">' . $sistema->retTranslate( 'SeizeTimeout' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['SeizeTimeout'];
        $return .= intSelectMaker( 'SeizeTimeout', '', $value, 1000, 160000, 1000 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="FlashTime">' . $sistema->retTranslate( 'FlashTime' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['FlashTime'];
        $return .= intSelectMaker( 'FlashTime', '', $value, 100, 16000, 100 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="FlashValidationTime">' . $sistema->retTranslate( 'FlashValidationTime' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['FlashValidationTime'];
        $return .= intSelectMaker( 'FlashValidationTime', '', $value, 100, 16000, 100 );
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "<tr><td align='left'>";
        $return .= '<label for="LineTerminationImpedance">' . $sistema->retTranslate( 'LineTerminationImpedance' ) . '</label>';
        $return .= "</td>";
        $return .= "<td align='right'>";
        $value = $profile[$name]['LineTerminationImpedance'];
        $return .= intSelectMaker( 'LineTerminationImpedance', '', $value, 600, 900, 300 );
        $return .= "</td>";
        $return .= "</tr>";

        $app = KSystem::getInstance();
        $return .= "<tr><td align='left'>";
        $return .= '<label for="CallerIdProfile">' . $sistema->retTranslate('callerIdProfileFXO') . '</label>';

        $return .= "</td>";
        $return .= "<td align='right'>";
        $callerid = new KYaml( $app->getConfigDir() . '/callerid.yaml');
        $value = $profile[$name]['CallerIdProfile'];
        $callerYaml = $callerid->getYaml();
        $opts = array();
        foreach (array('DTMFDetector', 'FSKDetector' ) as $k => $v)
        {
            foreach ($callerYaml[$v]as $key => $val)
            {
                if(!in_array(key($val), $opts))
                    $opts[] = key($val);
            }
        }
        $return .= '<select id="CallerIdProfile" class="" name="CallerIdProfile">';
        sort($opts);
        foreach ($opts as $k => $v)
        {
            $selected = $v == $value ? 'selected' : '';
            $return .= "<option {$selected} value='{$v}'>{$v}</option>";
        }
        $return .= '</select>';
        $return .= "</td>";
        $return .= "</tr>";

        $return .= "</tbody></table><br/>";

        $JSParam = $edit == "" ? "'copy'" : "'edit'";

        $return .= '<div class="floatRight">';
        $return .= "<input value='" . $sistema->retTranslate( "Cancelar" ) . "' onclick='listProfiles();' type='button'>";
        $return .= "<input value='" . $sistema->retTranslate( "Salvar" ) . "' onclick=\"createFXO($JSParam);\" type='button'>";
        $return .= '</div>';

        return $return;
    }

    function createR2( $variables, $newProfile )
    {
        $Profiles = new KProfiles( );
        $name = $variables['name'];
        $pos = $Profiles->getProfilePosition( $name );
        $edit = strlen( $pos ) >= 1 && $pos > -1 ? true : false;

        if( $newProfile && $edit )
            return false;

        $profile = array( );
        $profile['Type'] = 'R2';
        $profile['SubType'] = $variables['SubType'];
        $profile['ForcedDisconnection'] = checkBooleanValue( $variables['ForcedDisconnection'] );
        $profile['SeizeConfirmationTimeout'] = checkBooleanValue( $variables['SeizeConfirmationTimeout'] );
        $profile['DestEoNStandard'] = checkBooleanValue( $variables['DestEoNStandard'] );
        $profile['OrigEoNStandard'] = checkBooleanValue( $variables['OrigEoNStandard'] );
        $profile['AnswerSignalReceptionTimeout'] = checkBooleanValue( $variables['AnswerSignalReceptionTimeout'] );
        $profile['DisconnectConfirmationTime'] = checkBooleanValue( $variables['DisconnectConfirmationTime'] );
        $profile['SeizeTimeout'] = checkBooleanValue( $variables['SeizeTimeout'] );
        $profile['R2MinLineSendTime'] = checkBooleanValue( $variables['R2MinLineSendTime'] );
        $profile['BillingPulseMaxTime'] = checkBooleanValue( $variables['BillingPulseMaxTime'] );
        $profile['IdentityRequestPosition'] = checkBooleanValue( $variables['IdentityRequestPosition'] );
        $profile['ForwardMfcReceptionTimeout'] = checkBooleanValue( $variables['ForwardMfcReceptionTimeout'] );
        $profile['ForwardMfcSendingTimeout'] = checkBooleanValue( $variables['ForwardMfcSendingTimeout'] );
        $profile['ForwardMfcSilenceTimeout'] = checkBooleanValue( $variables['ForwardMfcSilenceTimeout'] );
        $profile['MfcPulseTime'] = checkBooleanValue( $variables['MfcPulseTime'] );
        $profile['RingbackSilenceTime'] = checkBooleanValue( $variables['RingbackSilenceTime'] );
        $profile['RingbackToneTime'] = checkBooleanValue( $variables['RingbackToneTime'] );
        $profile['Channels']['Default'] = checkBooleanValue( $variables['ChannelsDefault'] );

        for( $x = 0; $x < 30; $x++ )
        {
            if( isset( $variables['Channels' . $x] ) )
                $profile['Channels'][$x] = $variables['Channels' . $x];
        }

        $tmp = $Profiles->profilesYaml->getYaml( );
        if( $edit )
            $tmp['ChannelGroupProfiles'][$pos][$name] = $profile;
        else
            $tmp['ChannelGroupProfiles'][] = array( $name => $profile );
        $Profiles->profilesYaml->yaml = $tmp;

        $Profiles->saveProfiles( );
        return true;
    }

    function createIsdn( $variables, $newProfile )
    {
        $Profiles = new KProfiles( );
        $name = $variables['name'];
        $pos = $Profiles->getProfilePosition( $name );
        $edit = strlen( $pos ) >= 1 && $pos > -1 ? true : false;

        if( $newProfile && $edit )
            return false;

        $profile = array( );
        $profile['Type'] = 'ISDN';
        $profile['SubType'] = $variables['SubType'];
        $profile['DefaultBearerCapability'] = checkBooleanValue( $variables['DefaultBearerCapability'] );
        $profile['DefaultHLC'] = checkBooleanValue( $variables['DefaultHLC'] );
        $profile['AlwaysSendChannelId'] = checkBooleanValue( $variables['AlwaysSendChannelId'] );
        $profile['T1Mode'] = checkBooleanValue( $variables['T1Mode'] );
        $profile['AllowTransmissionInTimeoutRecovery'] = checkBooleanValue( $variables['AllowTransmissionInTimeoutRecovery'] );
        $profile['TimeslotOffsetOption'] = checkBooleanValue( $variables['TimeslotOffsetOption'] );
        $profile['RestartProcedureOnLinkEstablish'] = checkBooleanValue( $variables['RestartProcedureOnLinkEstablish'] );
        $profile['DefaultOrigTypeOfNumber'] = checkBooleanValue( $variables['DefaultOrigTypeOfNumber'] );
        $profile['DefaultOrigNumberingPlan'] = checkBooleanValue( $variables['DefaultOrigNumberingPlan'] );
        $profile['SendOrigPresentationAndScreening'] = checkBooleanValue( $variables['SendOrigPresentationAndScreening'] );
        $profile['DefaultOrigPresentation'] = checkBooleanValue( $variables['DefaultOrigPresentation'] );
        $profile['DefaultOrigScreening'] = checkBooleanValue( $variables['DefaultOrigScreening'] );
        $profile['DefaultDestTypeOfNumber'] = checkBooleanValue( $variables['DefaultDestTypeOfNumber'] );
        $profile['DefaultDestNumberingPlan'] = checkBooleanValue( $variables['DefaultDestNumberingPlan'] );
        $profile['IsdnTimers']['T301'] = checkBooleanValue( $variables['T301'] );
        $profile['IsdnTimers']['T302'] = checkBooleanValue( $variables['T302'] );
        $profile['IsdnTimers']['T303'] = checkBooleanValue( $variables['T303'] );
        $profile['IsdnTimers']['T304'] = checkBooleanValue( $variables['T304'] );
        $profile['IsdnTimers']['T305'] = checkBooleanValue( $variables['T305'] );
        $profile['IsdnTimers']['T308'] = checkBooleanValue( $variables['T308'] );
        $profile['IsdnTimers']['T309'] = checkBooleanValue( $variables['T309'] );
        $profile['IsdnTimers']['T310'] = checkBooleanValue( $variables['T310'] );
        $profile['IsdnTimers']['T313'] = checkBooleanValue( $variables['T313'] );
        $profile['IsdnTimers']['T200'] = checkBooleanValue( $variables['T200'] );
        $profile['IsdnTimers']['T203'] = checkBooleanValue( $variables['T203'] );
        $profile['IsdnTimers']['RejectRecovery'] = checkBooleanValue( $variables['RejectRecovery'] );
        $profile['IsdnTimers']['SSCTt1'] = checkBooleanValue( $variables['SSCTt1'] );
        $profile['IsdnTimers']['CTt1'] = checkBooleanValue( $variables['CTt1'] );
        $profile['IsdnTimers']['CTt3'] = checkBooleanValue( $variables['CTt3'] );

        $tmp = $Profiles->profilesYaml->getYaml( );
        if( $edit )
            $tmp['ChannelGroupProfiles'][$pos][$name] = $profile;
        else
            $tmp['ChannelGroupProfiles'][] = array( $name => $profile );
        $Profiles->profilesYaml->yaml = $tmp;

        $Profiles->saveProfiles( );
        return true;
    }

    function createGSM( $variables, $newProfile )
    {
        $Profiles = new KProfiles( );
        $name = $variables['name'];
        $pos = $Profiles->getProfilePosition( $name );
        $edit = strlen( $pos ) >= 1 && $pos > -1 ? true : false;

        if( $newProfile && $edit )
            return false;

        $profile = array( );
        $profile['Type'] = 'GSM';
        $profile['CheckNetworkTimeOut'] = checkBooleanValue( $variables['CheckNetworkTimeOut'] );
        $profile['SendDtmfBySignaling'] = checkBooleanValue( $variables['SendDtmfBySignaling'] );
        $profile['HexEncodeReceived8bitSms'] = checkBooleanValue( $variables['HexEncodeReceived8bitSms'] );
        $profile['DefaultSmsTextCoding'] = $variables['DefaultSmsTextCoding'];

        $tmp = $Profiles->profilesYaml->getYaml( );
        if( $edit )
            $tmp['ChannelGroupProfiles'][$pos][$name] = $profile;
        else
            $tmp['ChannelGroupProfiles'][] = array( $name => $profile );
        $Profiles->profilesYaml->yaml = $tmp;

        $Profiles->saveProfiles( );
        return true;
    }

    function createFXS( $variables, $newProfile )
    {
        $Profiles = new KProfiles( );
        $name = $variables['name'];
        $pos = $Profiles->getProfilePosition( $name );
        $edit = strlen( $pos ) >= 1 && $pos > -1 ? true : false;

        if( $newProfile && $edit )
            return false;

        $profile = array( );
        $profile['Type'] = 'FXS';
        $profile['CadenceTimes'][0] = checkBooleanValue( $variables['CadenceTimesA'] );
        $profile['CadenceTimes'][1] = checkBooleanValue( $variables['CadenceTimesB'] );
        $profile['CadenceTimes'][2] = checkBooleanValue( $variables['CadenceTimesC'] );
        $profile['CadenceTimes'][3] = checkBooleanValue( $variables['CadenceTimesD'] );
        $profile['FlashThresholdTime'] = checkBooleanValue( $variables['FlashThresholdTime'] );
        $profile['FlashValidationTime'] = checkBooleanValue( $variables['FlashValidationTime'] );
        $profile['CallerIdProfile'] =  $variables['CallerIdProfile'] ;
        $profile['LineDistance'] =  $variables['LineDistance'] ;

        $tmp = $Profiles->profilesYaml->getYaml( );
        if( $edit )
            $tmp['ChannelGroupProfiles'][$pos][$name] = $profile;
        else
            $tmp['ChannelGroupProfiles'][] = array( $name => $profile );
        $Profiles->profilesYaml->yaml = $tmp;

        $Profiles->saveProfiles( );
        return true;
    }

    function createLineSide( $variables, $newProfile )
    {
        $Profiles = new KProfiles( );
        $name = $variables['name'];
        $pos = $Profiles->getProfilePosition( $name );
        $edit = strlen( $pos ) >= 1 && $pos > -1 ? true : false;

        if( $newProfile && $edit )
            return false;

        $profile = array( );
        $profile['Type'] = 'CAS PBX';
        $profile['SubType'] = $variables['SubType'];
        $profile['UseAddressSignaling'] = checkBooleanValue( $variables['UseAddressSignaling'] );
        $profile['UseAnswerSupervision'] = checkBooleanValue( $variables['UseAnswerSupervision'] );
        $profile['UseFarEndDisconnect'] = checkBooleanValue( $variables['UseFarEndDisconnect'] );
        $profile['ReleaseTime'] = checkBooleanValue( $variables['ReleaseTime'] );
        $profile['AbandonTime'] = checkBooleanValue( $variables['AbandonTime'] );
        $profile['RingsToNotify'] = checkBooleanValue( $variables['RingsToNotify'] );
        $profile['FlashTime'] = checkBooleanValue( $variables['FlashTime'] );
        $profile['LongSignalingTime'] = checkBooleanValue( $variables['LongSignalingTime'] );
        $profile['ShortSignalingTime'] = checkBooleanValue( $variables['ShortSignalingTime'] );
        $profile['SeizeTimeout'] = checkBooleanValue( $variables['SeizeTimeout'] );

        $tmp = $Profiles->profilesYaml->getYaml( );
        if( $edit )
            $tmp['ChannelGroupProfiles'][$pos][$name] = $profile;
        else
            $tmp['ChannelGroupProfiles'][] = array( $name => $profile );
        $Profiles->profilesYaml->yaml = $tmp;

        $Profiles->saveProfiles( );
        return true;
    }

    function createFXO( $variables, $newProfile )
    {
        $Profiles = new KProfiles( );
        $name = $variables['name'];
        $pos = $Profiles->getProfilePosition( $name );
        $edit = strlen( $pos ) >= 1 && $pos > -1 ? true : false;

        if( $newProfile && $edit )
            return false;

        $profile['Type'] = 'FXO';
        $profile['RingsToNotify'] = checkBooleanValue( $variables['RingsToNotify'] );
        $profile['SeizeTimeout'] = checkBooleanValue( $variables['SeizeTimeout'] );
        $profile['DisconnectOnPolarityReversal'] = checkBooleanValue( $variables['DisconnectOnPolarityReversal'] );
        $profile['FlashTime'] = checkBooleanValue( $variables['FlashTime'] );
        $profile['FlashValidationTime'] = checkBooleanValue( $variables['FlashValidationTime'] );
        $profile['LineTerminationImpedance'] = $variables['LineTerminationImpedance'];
        $profile['CallerIdProfile'] =  $variables['CallerIdProfile'] ;

        $tmp = $Profiles->profilesYaml->getYaml( );
        if( $edit )
            $tmp['ChannelGroupProfiles'][$pos][$name] = $profile;
        else
            $tmp['ChannelGroupProfiles'][] = array( $name => $profile );
        $Profiles->profilesYaml->yaml = $tmp;

        $Profiles->saveProfiles( );
        return true;
    }

    function checkBooleanValue( $var )
    {
        if( $var == 'true' )
            return true;
        elseif( $var == 'false' )
            return false;
        else
            return $var;
    }

    function testProfile( $profileName )
    {
        Global $sistema;
        $Profiles = new KProfiles( );
        $profile = $Profiles->getProfile( $profileName );
        if( $profile == NULL && $profileName != "" )
            return "Ok";
        else
            return $sistema->retTranslate( 'invalidName' );
    }

    function intSelectMaker( $name, $class, $value, $min, $max, $step, $fator = 1 )
    {
        $return = '';

        if( $class != '' )
            $return .= "<select class='$class' name='$name' id='$name' >";
        else
            $return .= "<select class='selector2' name='$name' id='$name' >";
        for( $x = $min; $x <= $max; $x = $x + $step )
        {
            if( $x == $value )
                $selected = "selected";
            else
                $selected = "";
            $return .= "<option value='$x' $selected>" . ($fator * $x) . "</option>";
        }
        $return .= "</select>";
        return $return;
    }

    function make_select( $id, $options, $selected, $associative = false, $htmlattributes = array() )
    {
        $app = KSystem::getInstance();

        if( !isset( $htmlattributes['id'] ) )
        {
            $htmlattributes['id'] = $id;
            $htmlattributes['name'] = $id;
        }

        $attributes = '';
        foreach( $htmlattributes as $attrname => $attrvalue )
        {
            $attributes .= " {$attrname}=\"{$attrvalue}\"";
        }

        $ret = "<select {$attributes}>";
        unset( $attributes );

        if( ( !$associative && $selected === '' ) || ( $associative && !array_key_exists( $selected, $options ) ) )
        {
            $ret .= "<option selected disabled>". $app->translate( 'Select' ) . "</option>";
        }

        foreach( $options as $key => &$value )
        {
            if( !$associative )
            {
                $sel = $value == $selected ? 'selected=\'selected\'' : '';
                $ret .= "<option {$sel}>{$value}</option>";
            }
            else
            {
                $sel = $key == $selected? 'selected=\'selected\'' : '';
                $ret .= "<option value='{$key}' {$sel}>{$value}</option>";
            }
        }
        $ret .= "</select>";
        return $ret;
    }


    function cadenceAssistant( $first = true )
    {
        Global $sistema;

        $return = '';

        if( $first )
            $return .= "<div id='dialog-modal' title='" . $sistema->retTranslate( 'CadencesAssistant' ) . "'>";

        $return .= "<p>" . $sistema->retTranslate( 'InitCadenceAssistant' ) . "</p>";
        $return .= "<div id='looper'>";
        $FXOChannels = $sistema->getFXOChannels( );

        if( sizeof( $FXOChannels ) > 0 )
        {
            $return .= "<form>";
            $return .= "<table id='cadenceAssistantTable' >";
            $return .= "<tr>";
            $return .= "<td align='left'>" . $sistema->retTranslate( "Device" ) . "</td>";
            $return .= "<td align='left'>";
            $return .= getFXODevices( );
            $return .= "</td>";
            $return .= "</tr>";
            $return .= "<td align='left'>" . $sistema->retTranslate( "Channel" ) . "</td>";
            $return .= "<td align='left' id='fxoDeviceChannelsList'>";
            $FXO = $sistema->getFirstFXODevice( );
            $return .= getFXODeviceChannels( $FXO );
            $return .= "</td>";
            $return .= "</tr>";
            $return .= "<tr>";
            $return .= "<td align='left'>" . $sistema->retTranslate( "NumeroRamalLigacoes" ) . "</td>";
            $return .= "<td align='left'>";
            $return .= "<input id='busy' type='text' onkeyup='validateNumber(this);' onmouseup='return numberMask(this, event);' onKeyPress='return numberMask(this, event);'>";
            $return .= "</td>";
            $return .= "</tr>";
            $return .= "<tr>";
            $return .= "<td align='left'>" . $sistema->retTranslate( "NumeroRingBack" ) . "</td>";
            $return .= "<td align='left'>";
            $return .= "<input id='ringback' type='text' onkeyup='validateNumber(this);' onmouseup='return numberMask(this, event);' onKeyPress='return numberMask(this, event);'>";
            $return .= "</td>";
            $return .= "</tr>";
            $return .= "<tr>";
            $return .= "<td align='left'>" . $sistema->retTranslate( "NumeroInvalido" ) . "</td>";
            $return .= "<td align='left'>";
            $return .= "<input id='invalid' type='text' onkeyup='validateNumber(this);' onmouseup='return numberMask(this, event);' onKeyPress='return numberMask(this, event);'>";
            $return .= "</td>";
            $return .= "</tr>";
            $return .= "</table>";
            $return .= "</br>";

            $return .= "<input type='button' onClick='initAssintant();' value='" . $sistema->retTranslate( "gerarCadencias" ) . "' />";

            $return .= "</form>";
        }
        else
            $return .= $sistema->retTranslate( "NDevice4Assitance" );

        $return .= "</div>";
        if( $first )
            $return .= "</div>";

        return $return;
    }

    function replace_key_function($array, $key1, $key2)
    {
        $keys = array_keys($array);
        $index = array_search($key1, $keys);

        if ($index !== false) {
            $keys[$index] = $key2;
            $array = array_combine($keys, $array);
        }

        return $array;
    }
