<?php
    require_once KCONFIG . '/classes/classKGatewayUtil.php';
    require_once KCONFIG . '/classes/KGatewayYaml.php';
    require_once KCONFIG . '/classes/KFTPYaml.php';

    $AjaxFunctions['showCDROptions'] = 'showCDROptions';
    $AjaxFunctions['saveCDROptions'] = 'saveCDROptions';
    $AjaxFunctions['showCDRRadius'] = 'showCDRRadius';
    $AjaxFunctions['saveCDRRadius'] = 'saveCDRRadius';
    $AjaxFunctions['showFTPOptions'] = 'showFTPOptions';
    $AjaxFunctions['saveFTPOptions'] = 'saveFTPOptions';
    $AjaxFunctions['saveCDRFTPOptions'] = 'saveCDRFTPOptions';
    $AjaxFunctions['testFTPConnection'] = 'testFTPConnection';

    function CDROption( $app, $name, $format )
    {
        $trvar = "gateway-cdr-{$name}";
        $cdrval = is_null( $format ) ? $name : "{$name}:{$format}";
        return "<li cdr-value='{$cdrval}' class='gateway-cdr-option ui-dragable'>&nbsp;<div>{$name}</div><img src='/imagens/tooltip.png' class='gateway-tooltip' alt='tooltip' title='{$app->translate( $trvar )}' /></li>";
    }

    function CDRRotateOption( $app, $name, $current )
    {
        $s = $name == $current ? ' selected=\'selected\'' : '';
        $n = 'gateway-cdr-' . str_replace( ' ', '-', $name );
        return "<option value='{$name}'{$s}>{$app->translate( $n )}</option>";
    }

    function __CDRtooltip( $textid, $text = null )
    {
        $app = KSystem::getInstance( );
        if( ! empty( $textid ) )
        {
            $text = $app->translate( $textid );
        }
        return "<img src='../imagens/tooltip.png' title='{$text}' class='gateway-tooltip' alt='tooltip' />";
    }

    function showCDROptions( )
    {
        require_once KROOT . '/classes/KLicense.php';
        require_once KROOT . '/classes/KFeatures.php';

        $app = KSystem::getInstance( );
        $gatewayyaml = new KGatewayYaml( );
        $lm = new KLicenseManager();

        //@formatter:off
        // available options to gateway-cdr-select
        $options = array(
            'callid', 'leg_a', 'leg_b', 'orig_addr', 'dest_addr',
            'leg_a_id', 'leg_b_id', 'leg_a_signaling', 'leg_b_signaling',
            'leg_a_sid', 'leg_b_sid',
            'leg_b_orig_addr', 'leg_b_dest_addr', 'start_stamp', 'audio_stamp',
            'answer_stamp', 'end_stamp', 'duration', 'billsec', 'hangup_origin',
            'hangup_cause', 'hangup_cause_q850', 'hangup_cause_sent',
            'orig_nap', 'dest_nap', 'route_name', 'leg_a_profile',
            'leg_b_profile', 'leg_b_profile_origin',
            'portability_service_provider_name',
            'portability_service_provider_code', 'portability_checked_by_route',
            'portability_checked_using_number', 'time_track_account',
            'retry_count'
        );

        if( KFeatures::checkFeature( KFeatures::CALL_ANALYZER ) && $lm->anyLicenseHasAnyType( KLicense::TYPES_GATEWAY_ANALYTICS ) )
        {
            $options = array_merge( $options, array(
                'analyzer_stamp', 'analyzer_conclusion', 'analyzer_code',
                'analyzer_action', 'analyzer_pattern', 'analyzer_locale'
            ) );
        }

        if( KFeatures::checkFeature( KFeatures::GSM_SUPPORT ) )
        {
            $options = array_merge( $options, array(
                'leg_a_gsm_iccid', 'leg_b_gsm_iccid', 'leg_a_gsm_imsi',
                'leg_b_gsm_imsi'
            ) );
        }

        if( KFeatures::checkFeature( KFeatures::E1_SUPPORT ) )
        {
            $options = array_merge( $options, array(
                'leg_a_trunk_name', 'leg_b_trunk_name', 'leg_a_timeslot_number',
                'leg_b_timeslot_number'
            ) );
        }

        // default option format
        $options_fmt = array(
            'start_stamp'=>'%d-%m-%y %H:%M:%S',
            'audio_stamp'=>'%d-%m-%y %H:%M:%S',
            'answer_stamp'=>'%d-%m-%y %H:%M:%S',
            'end_stamp'=>'%d-%m-%y %H:%M:%S'
        );

        if( KFeatures::checkFeature( KFeatures::CALL_ANALYZER ) && $lm->anyLicenseHasAnyType( KLicense::TYPES_GATEWAY_ANALYTICS ) )
        {
            $options_fmt['analyzer_stamp'] = '%d-%m-%y %H:%M:%S';
        }

        // samples (array of array, the second array has the sample name on the
        // first position, the second has the sample format and from the third
        // position to the end of array are the sample outputs
        $samples = array(
            array(
                'CSV',
                '"${callid}", "${leg_a}", "${leg_b}", "${orig_addr}", "${dest_addr}", "${leg_b_orig_addr}", "${leg_b_dest_addr}", "${start_stamp:%d/%m/%y %H:%M:%S}", "${answer_stamp:%d/%m/%y %H:%M:%S}", "${end_stamp:%d/%m/%y %H:%M:%S}", "${duration}", "${billsec}", "${hangup_origin}", "${hangup_cause}", "${dest_nap}", "${route_name}"',
                '"0526870051344016125586952140", "D52687 C5", "D52687 C6", "551151823411", "554837222900", "551151823411", "554837222900", "03/08/12 14:46:17", "03/08/12 14:46:27", "03/08/12 14:48:19", "122", "112", "leg a", "0", "nap a", "default"',
                '"0526870161344016811385295214", "D52687 C16", "D52687 C4", "554837222900", "551151823411", "554837222900", "551151823411", "03/08/12 15:00:07", "03/08/12 15:00:25", "03/08/12 15:32:25", "1938", "1920", "leg a", "0", "nap a", "default"',
                '"0526870011344020110345605789", "D52687 C1", "D19432 C9", "551151823411", "541159176025", "551151823411", "541159176025", "03/08/12 21:12:55", "03/08/12 21:13:16", "03/08/12 21:22:00", "545", "524", "leg b", "4", "nap c", "argentina"'
            ),
            array(
                'SQL',
                'INSERT INTO CallLog (`callid`, `orig_addr`, `dest_addr`, `start`, `answer`, `end`) VALUES ("${callid}", "${orig_addr}", "${dest_addr}", "${start_stamp:%Y-%m-%d %H:%M:%S}", "${answer_stamp:%Y-%m-%d %H:%M:%S}", "${end_stamp:%Y-%m-%d %H:%M:%S}");',
                'INSERT INTO CallLog (`callid`, `orig_addr`, `dest_addr`, `start`, `answer`, `end`) VALUES ("0526870051344016125586952140", "551151823411", "554837222900", "2012-08-03 14:46:17", "2012-08-03 14:46:27", "2012-08-03 14:48:19");',
                'INSERT INTO CallLog (`callid`, `orig_addr`, `dest_addr`, `start`, `answer`, `end`) VALUES ("0526870161344016811385295214", "554837222900", "551151823411", "2012-08-03 15:00:07", "2012-08-03 15:00:25", "2012-08-03 15:32:25");',
                'INSERT INTO CallLog (`callid`, `orig_addr`, `dest_addr`, `start`, `answer`, `end`) VALUES ("0526870011344020110345605789", "551151823411", "541159176025", "2012-08-03 21:12:55", "2012-08-03 21:13:16", "2012-08-03 21:22:00");'
            ),
            array(
                $app->translate( 'gateway-cdr-sample-custom' ),
                '${callid}: (${leg_a}) [${orig_addr} -> ${dest_addr}] <---> ${route_name} <---> (${leg_b}) [${leg_b_orig_addr} -> ${leg_b_dest_addr}] -- start=${start_stamp:%c}, answer=${answer_stamp:%c}, end=${end_stamp:%c}, total=${duration}, bill=${billsec}',
                '0526870051344016125586952140: (D52687 C5) [551151823411 -> 554837222900] <---> default <---> (D52687 C6) [551151823411 -> 554837222900] -- start=Fri Aug 03 14:46:17 2012, answer=Fri Aug 03 14:46:27 2012, end=Fri Aug 03 14:48:19 2012, total=122, bill=112',
                '0526870161344016811385295214: (D52687 C16) [554837222900 -> 551151823411] <---> default <---> (D52687 C4) [554837222900 -> 551151823411] -- start=Fri Aug 03 15:00:07 2012, answer=Fri Aug 03 15:00:25 2012, end=Fri Aug 03 15:32:25 2012, total=1938, bill=1920',
                '0526870011344020110345605789: (D52687 C1) [551151823411 -> 541159176025] <---> argentina <---> (D19432 C9) [551151823411 -> 541159176025] -- start=Fri Aug 03 21:12:55 2012, answer=Fri Aug 03 21:13:16 2012, end=Fri Aug 03 21:22:00 2012, total=545, bill=524'
            )

        );
        //@formatter:on

        $kyaml = new KGatewayYaml( );
        $cdr = $kyaml->getCDR( );

        $enabled = $kyaml->isCDREnabled( ) ? ' checked=\'checked\'' : '';
        $buf = "<div id='gateway-cdr-config'><fieldset name='gateway-cdr' id='gateway-cdr'><legend><input type='checkbox' id='gateway-cdr-enabled'{$enabled} /> <label for='gateway-cdr-enabled'>{$app->translate( 'gateway-cdr-file' )}</label></legend>";
        $buf .= '<table name="gateway-cdr-table" id="gateway-cdr-table" class="simpleTable">';
        $buf .= "<tr><td class='gateway-cdr-td-align-left'><label for='gateway-cdr-rotate'>{$app->translate( 'gateway-cdr-rotate-title' )}</label>: <select name='gateway-cdr-rotate' id='gateway-cdr-rotate'>";
        foreach( KGatewayYaml::$cdrrotateoptions as &$opt )
        {
            $buf .= CDRRotateOption( $app, $opt, $cdr['Rotate'] );
        }
        $buf .= '</select></td>';
        // end of rotation select
        $buf .= "<td width='150px' rowspan='4'><p>{$app->translate( 'gateway-cdr-options-title' )} <img src='/imagens/tooltip.png' class='gateway-tooltip' alt='tooltip' title='{$app->translate( 'gateway-cdr-options-help' )}' /></p><ul id='gateway-cdr-select' name='gateway-cdr-select'>";
        foreach( $options as &$opt )
        {
            $fmt = array_key_exists( $opt, $options_fmt ) ? $options_fmt[$opt] : null;
            $buf .= CDROption( $app, $opt, $fmt );
        }
        $buf .= '</ul></td></tr>';
        // end of pattern editor variables
        $buf .= "<tr><td class='gateway-cdr-td-align-left'><label for='gateway-cdr-style'>{$app->translate( 'gateway-cdr-style' )}</label>: <select id='gateway-cdr-style' name='gateway-cdr-style'>";
        $selected = isset( $cdr['Style'] ) ? $cdr['Style'] : 'per try';
        foreach( KGatewayYaml::$cdrstyleoptions as &$opt )
        {
            $optselected = $selected == $opt ? ' selected=\'selected\'' : '';
            $key = 'gateway-cdr-style-' . str_replace( ' ', '-', $opt );
            $buf .= "<option value='{$opt}'{$optselected}>{$app->translate( $key )}</option>";
        }
        $buf .= '</select></td></tr>';
        // end of style select
        $buf .= "<tr><td class='gateway-cdr-td-align-left'><label for='gateway-cdr-pattern'>{$app->translate( 'gateway-cdr-pattern-title' )}</label>:</td></tr><tr><td><div><textarea name='gateway-cdr-pattern' id='gateway-cdr-pattern'>{$cdr['Pattern']}</textarea></div></td></tr>";
        // end of pattern editor
        $buf .= "<tr><td colspan='2'><div id='gateway-cdr-help' name='gateway-cdr-help'><span class='gateway-cdr-help-title'>+ {$app->translate( 'gateway-cdr-fmt-title' )} <img src='/imagens/tooltip.png' alt='tooltip' /></span><table class='help'>{$app->translate( 'gateway-cdr-fmt' )}</table><br /><span class='gateway-cdr-help-title'>+ {$app->translate( 'gateway-cdr-sample-title' )} <img src='/imagens/tooltip.png' alt='tooltip' /></span><div class='help'>";
        foreach( $samples as &$sample )
        {
            $buf .= "<p>{$app->translate( 'gateway-cdr-sample-fmt' )} <span class='bld'>{$sample[0]}</span>: <span class='gateway-cdr-help-sample'>{$sample[1]}</span> <span class='gateway-cdr-help-sample-copy'>{$app->translate( 'gateway-cdr-sample-copy' )}</span><br />{$app->translate( 'gateway-cdr-sample-result' )}:<br />";
            for( $r = 2; $r < count( $sample ); $r++ )
            {
                $buf .= "<span class='gateway-cdr-help-sample'>{$sample[$r]}</span><br />";
            }
            $buf .= '</p>';
        }
        $buf .= '</div></div></td></tr>';
        // end of help sections
        $buf .= "</table><br /><div class='textAlignRight'>";
        $buf .= "<input type='button' id='gateway-cdr-save' name='gateway-cdr-save' value='{$app->translate( 'save' )}' /></div></fieldset>";
        // end of fieldset

        return $buf;
    }

    function saveCDROptions( )
    {
        require_once KROOT . '/json_util.php';

        $app = KSystem::getInstance( );

        $rt = array( 'message' => $app->translate( 'gateway-cdr-save-error' ), 'success' => false );
        if( !array_key_exists( 'Pattern', $_POST ) || !array_key_exists( 'Rotate', $_POST ) )
        {
            $rt['message'] = $app->translate( 'gateway-cdr-save-error-notnull' );
        }
        elseif( strpos( $_POST['Pattern'], "\n" ) !== false )
        {
            $rt['message'] = $app->translate( 'gateway-cdr-save-error-nolinebreak' );
        }
        else
        {
            try
            {
                $kyaml = new KGatewayYaml( );
                $cdr = $kyaml->getCDR( );
                foreach( $_POST as $key => $value )
                {
                    $cdr[$key] = $value;
                }
                $kyaml->setCDR( $cdr );
                $kyaml->save( );

                $rt = array( 'message' => $app->translate( 'gateway-cdr-saved' ), 'success' => true );
            }
            catch( Exception $e )
            {
                $rt['message'] = $app->translate( $e->getMessage() );
            }
        }

        return JSONencode( $rt );
    }

    function showCDRRadius( )
    {
        require_once KROOT . '/classes/KFeatures.php';

        $app = KSystem::getInstance( );

        if( ! KFeatures::checkFeature( KFeatures::RADIUS_SUPPORT ) )
        {
            return $app->translate( 'gateway-cdr-radius-error-not-supported' );
        }

        $yaml = new KGatewayYaml( );
        $cdr = $yaml->getCDR( );
        $conf =& $cdr['Radius'];
        $tooltip = '__CDRtooltip';

        $date_fmt_tooltip = '<table class="zebra simpleTable"><tbody>';
        $date_fmt_tooltip .= $app->translate( 'gateway-cdr-fmt' );
        $date_fmt_tooltip .= $app->translate( 'gateway-cdr-radius-date-fmt' );
        $date_fmt_tooltip .= '</tbody></table><br />';
        $date_fmt_tooltip .= $app->translate( 'gateway-cdr-radius-date-format-utc' );

        $enabled = $conf['Enabled'] ? ' checked=\'checked\'' : '';

        $buf = "<div id='gateway-cdr-radius-config'><fieldset id='gateway-cdr-radius'><legend><input type='checkbox' id='gateway-cdr-radius-enabled' name='Enabled' {$enabled}/> <label for='gateway-cdr-radius-enabled'>{$app->translate( 'gateway-cdr-radius' )}</label></legend>";
        // server list
        $buf .= "<fieldset><legend>{$app->translate( 'gateway-cdr-radius-servers' )}</legend>";
        $buf .= '<table id="gateway-cdr-radius-servers-table" class="zebra simpleTable gateway-table-form">';
        $buf .= '<thead>';
        $buf .= "<tr><th>{$app->translate( 'gateway-cdr-radius-server-host' )} {$tooltip( 'gateway-cdr-radius-server-host-tooltip' )}</th><th>{$app->translate( 'gateway-cdr-radius-server-port' )} {$tooltip( 'gateway-cdr-radius-server-port-tooltip' )}</th><th>{$app->translate( 'gateway-cdr-radius-server-secret' )} {$tooltip( 'gateway-cdr-radius-server-secret-tooltip' )}</th><th>{$app->translate( 'options' )}</th></tr>";
        $buf .= '</thead><tbody>';
        $buf .= "<tr class='displayNone' id='gateway-cdr-radius-server-tpl'>";
        $buf .= "<td><input type='text' name='Host' value='' required /></td>";
        $buf .= "<td><input type='text' name='Port' value='0' required /></td>";
        $buf .= "<td><input type='text' name='Secret' value='' required /></td>";
        $buf .= "</tr>";
        foreach( $conf['Servers'] as $server )
        {
            $buf .= "<tr class='gateway-cdr-radius-server'>";
            $buf .= "<td><input type='text' name='Host' value='{$server['Host']}' required /></td>";
            $buf .= "<td><input type='text' name='Port' value='{$server['Port']}' required /></td>";
            $buf .= "<td><input type='text' name='Secret' value='{$server['Secret']}' required /></td>";
            $buf .= "</tr>";
        }
        // end of servers list
        $buf .= '</tbody></table><br />';
        $buf .= "<div class='textAlignRight'><span id='gateway-cdr-radius-add-server' class='link'>{$app->translate( 'gateway-cdr-radius-add-server' )}</span></div>";
        $buf .= '</fieldset><br />';

        // advanced options
        $buf .= "<fieldset><legend id='gateway-cdr-radius-advanced-options'><span class='ui-icon k-ui-icon-inline ui-icon-circle-plus k-ui-icon-force-green' /> {$app->translate( 'gateway-cdr-radius-advanced-options' )}</legend>";
        $buf .= '<table id="gateway-cdr-radius-table" class="zebra simpleTable gateway-table-form displayNone"><tbody>';
        $buf .= "<tr><td class='label'><label for='gateway-cdr-radius-timeout'>{$app->translate( 'gateway-cdr-radius-timeout' )}</label>:</td><td><input id='gateway-cdr-radius-timeout' name='Timeout' type='text' value='{$conf['Timeout']}' required /></td></tr>";
        $buf .= "<tr><td class='label'><label for='gateway-cdr-radius-retries'>{$app->translate( 'gateway-cdr-radius-retries' )}</label>:</td><td><input id='gateway-cdr-radius-retries' name='Retries' type='text' value='{$conf['Retries']}' required /></td></tr>";
        $buf .= "<tr><td class='label'><label for='gateway-cdr-radius-dead-server-disabled-time'>{$app->translate( 'gateway-cdr-radius-dead-server-disabled-time' )}</label> {$tooltip( 'gateway-cdr-radius-dead-server-disabled-time-tooltip' )}:</td><td><input id='gateway-cdr-radius-dead-server-disabled-time' name='DeadServerDisabledTime' type='text' value='{$conf['DeadServerDisabledTime']}' required /></td></tr>";
        $buf .= "<tr><td class='label'><label for='gateway-cdr-radius-date-format'>{$app->translate( 'gateway-cdr-radius-date-format' )}</label> {$tooltip( null, $date_fmt_tooltip )}:</td><td><input id='gateway-cdr-radius-date-format' name='DateFormat' type='text' value='{$conf['DateFormat']}' required /></td></tr>";
        $buf .= "<tr><td class='label'><label for='gateway-cdr-radius-parallel-requests'>{$app->translate( 'gateway-cdr-radius-parallel-requests' )}</label>:</td><td><input id='gateway-cdr-radius-parallel-requests' name='ParallelRequests' type='text' value='{$conf['ParallelRequests']}' /></td></tr>";
        $buf .= "<tr><td class='label'><label for='gateway-cdr-radius-queue-size-discard-start'>{$app->translate( 'gateway-cdr-radius-queue-size-discard-start' )}</label> {$tooltip( 'gateway-cdr-radius-queue-size-discard-start-tooltip' )}:</td><td><input id='gateway-cdr-radius-queue-size-discard-start' name='QueueSizeDiscardStart' type='text' value='{$conf['QueueSizeDiscardStart']}' /></td></tr>";
        $buf .= "<tr><td class='label'><label for='gateway-cdr-radius-queue-size-discard-all'>{$app->translate( 'gateway-cdr-radius-queue-size-discard-all' )} {$tooltip( 'gateway-cdr-radius-queue-size-discard-all-tooltip' )}:</label></td><td><input id='gateway-cdr-radius-queue-size-discard-all' name='QueueSizeDiscardAll' type='text' value='{$conf['QueueSizeDiscardAll']}' /></td></tr>";
        $buf .= '</tbody></table></fieldset>';
        // end of advanced options

        $buf .= '</fieldset></div><br />';

        $buf .= "<div class='textAlignRight'><input type='button' id='gateway-cdr-radius-save' value='{$app->translate( 'save' )}' /></div>";

        return $buf;
    }

    function saveCDRRadius( )
    {
        require_once KROOT . '/json_util.php';

        $app = KSystem::getInstance( );
        $ret = array( 'success' => false, 'message' => 'gateway-cdr-radius-save-error' );

        if( ! KFeatures::checkFeature( KFeatures::RADIUS_SUPPORT ) )
        {
            $ret['message'] = $app->translate( 'gateway-cdr-radius-error-not-supported' );
            return JSONencode( $ret );
        }

        try
        {
            $yaml = new KGatewayYaml( );
            $cdr = $yaml->getCDR( );
            $cdr['Radius'] = $_POST;
            $yaml->setCDR( $cdr );
            $yaml->save( );
            $ret = array( 'success' => true, 'message' => $app->translate( 'gateway-cdr-radius-saved' ) );
        }
        catch( Exception $e )
        {
            $ret['message'] = $app->translate( $e->getMessage( ) );
        }
        return JSONencode( $ret );
    }

    //TODO: this function should be elsewhere
    function __toHumanSize( $size )
    {
        if( $size > 1073741824 )
        {
            $unit = 'GiB';
            $size = $size / 1073741824;
        }
        elseif( $size > 1048576 )
        {
            $unit = 'MiB';
            $size = $size / 1048576;
        }
        elseif( $size > 1024 )
        {
            $unit = 'KiB';
            $size = $size / 1024;
        }
        else
        {
            $unit = 'B';
        }

        $size = round( $size, 2 );
        return "{$size} {$unit}";
    }

    function showFTPOptions( )
    {
        $app = KSystem::getInstance();

        $yaml = new KFTPYaml( );

        $ftp = $yaml->getFTP();

        if( !$ftp )
        {
            throw new Exception( 'gateway-profile-error-not-found' );
        }

        $enabled = $ftp['Enabled'] ? ' checked=\'checked\'' : '';

        $buff = '';
        $buff .= "<div id='FTPConfig'>";
        $buff .= "<fieldset name='gateway-ftp-config id='gateway-ftp-config'>";
        $buff .= "<legend><input type='checkbox' id='gateway-ftp-enabled'{$enabled} /> <label for='gateway-ftp-enabled'>{$app->translate( 'gateway-send-to-ftp' )}</label></legend>";

        $buff .= "<table class='zebra'><tbody>";

        $buff .= "<tr><td align='left'>" . $app->translate( 'ServerAddress' ) . "</td>";
        $buff .= "<td align='right'>";
        $buff .= "<input id='gateway-ftp-server' name='gateway-ftp-server' class='inputText' type='text' value='{$ftp['Server']}'>";
        $buff .= "</td>";
        $buff .= "</tr>";

        $buff .= "<tr><td align='left'>" . $app->translate( 'ServerPort' ) . "</td>";
        $buff .= "<td align='right'>";
        $buff .= "<input id='gateway-ftp-port' class='inputText' type='text' onkeypress='return portMask(this, event);' onkeyup='validatePort(this);' value='{$ftp['Port']}'>";
        $buff .= "</td>";
        $buff .= "</tr>";

        $buff .= "<tr><td align='left'>" . $app->translate( 'Username' ) . "</td>";
        $buff .= "<td align='right'>";
        $buff .= "<input id='gateway-ftp-user' class='inputText' type='text' value='{$ftp['User']}'>";
        $buff .= "</td>";
        $buff .= "</tr>";

        $buff .= "<tr><td align='left'>" . $app->translate( 'password' ) . "</td>";
        $buff .= "<td align='right'>";
        $buff .= "<input name='gateway-ftp-password' id='gateway-ftp-password' class='inputText' type='password' value='{$ftp['Passwd']}'>";
        $buff .= "</td>";
        $buff .= "</tr>";

        $buff .= "<tr><td align='left'>" . $app->translate( 'gateway-ftp-dir-dst' ) . "</td>";
        $buff .= "<td align='right'>";
        $buff .= "<input id='gateway-ftp-dir-dst' class='inputText' type='text' value='{$ftp['Path']}'>";
        $buff .= "</td>";
        $buff .= "</tr>";

        $buff .= "</tbody></table>";
        $buff .= '<br />';

        $buff .= "</fieldset>";

        $buff .= '<br />';
        $buff .= "<fieldset name='gateway-ftp-local id='gateway-ftp-local'>";
        $buff .= "<legend><label class='labelFTP'>{$app->translate( 'gateway-cdr-local-config' )}</label></legend>";

        $buff .= "<table class='zebra'><tbody>";

        $limit = __toHumanSize( $yaml->getCDRSize() );
        $buff .= "<tr><td align='left'>" . $app->translate( 'gateway-cdr-time-retention' ) . " <img src='../imagens/tooltip.png' title='{$app->translate( 'gateway-cdr-retention-size-limit' )} {$limit}' class='gateway-tooltip' alt='tooltip' /></td>";
        $buff .= "<td align='right'>";
        $buff .= "<input id='gateway-cdr-time-retention' class='inputText' type='text' value='{$yaml->getCDRRetention()}'>";
        $buff .= "</td>";
        $buff .= "</tr>";

        $buff .= "</tbody></table>";
        $buff .= "</fieldset>";

        $buff .= '<br />';

        $buff .= "<div class='textAlignRight'>";
        $buff .= "<input type='button' id='gateway-ftp-save' name='gateway-ftp-save' value='{$app->translate( 'save' )}' />";
        $buff .= "</div>";
        $buff .= "</div>";

        return $buff;

    }

function saveFTPOptions( )
{
    require_once KROOT . '/json_util.php';

    $app = KSystem::getInstance( );

    $rt = array( 'message' => $app->translate( 'gateway-cdr-save-error' ), 'success' => false );

    if( !array_key_exists( 'Server', $_POST ) ||
        !array_key_exists( 'Port', $_POST ) ||
        !array_key_exists( 'User', $_POST ) ||
        !array_key_exists( 'Passwd', $_POST ) ||
        !array_key_exists( 'Path', $_POST ) )
    {
        $rt['message'] = $app->translate( 'gateway-cdr-save-error-notnull' );
    }

    else
    {
        try
        {
            $kyaml = new KFTPYaml( );
            $kyaml->setFTP( $_POST );
            $kyaml->save( );

            $rt = array( 'message' => $app->translate( 'gateway-cdr-saved' ), 'success' => true );
        }
        catch( Exception $e )
        {
            $rt['message'] = $app->translate( $e->getMessage() );
        }
   	}

    return JSONencode( $rt );
}

function saveCDRFTPOptions( )
{
    require_once KROOT . '/json_util.php';

    $app = KSystem::getInstance( );

    $rt = array( 'message' => $app->translate( 'gateway-cdr-save-error' ), 'success' => false );

    if(
        !array_key_exists( 'Retention', $_POST )
        || $_POST['Retention'] == ''
    )
    {
        $rt['message'] = $app->translate( 'gateway-ftp-save-error-notnull' );
    }
    else
    {
        try
        {
            $kyaml = new KFTPYaml( );
            $kyaml->setCDRRetention( $_POST['Retention'] );
            $kyaml->save( );

            $rt = array( 'message' => $app->translate( 'gateway-cdr-saved' ), 'success' => true );
        }
        catch( Exception $e )
        {
            $rt['message'] = $app->translate( $e->getMessage() );
        }
   	}

    return JSONencode( $rt );
}

function testFTPConnection ( )
{
    require_once KROOT . '/json_util.php';
    $app = KSystem::getInstance( );

    $rt = array( 'message' => $app->translate( 'gateway-ftp-no-permission' ), 'success' => false );

    $permission_folder_test = '__khomp-ftp_permission-test_create_folder';
    $conn_id = null;

    try
    {
        if( $conn_id = ftp_connect( $_POST['Server'], $_POST['Port'] ) )
        {
           if( ftp_login( $conn_id, $_POST['User'], $_POST['Passwd'] ) )
           {
               if ( $_POST['Path'] == "" )
               {
                   if( ftp_mkdir( $conn_id, $path_ftp . $permission_folder_test ) ==  true )
                   {
                       if( ftp_rmdir( $conn_id, $path_ftp . $permission_folder_test ) )
                       {
                           $rt = array( 'message' => $app->translate( 'gateway-ftp-connected' ), 'success' => true );
                       }
                   }
                   else
                   {
                       $rt = array( 'message' => $app->translate( 'gateway-ftp-no-permission' ), 'success' => false );
                   }
               }
               else
               {
                   $path_ftp = $_POST['Path'];

                   if( ftp_mkdir( $conn_id, $path_ftp . "/" . $permission_folder_test ) ==  true )
                   {
                       if( ftp_rmdir( $conn_id, $path_ftp . "/" . $permission_folder_test ) )
                       {
                           $rt = array( 'message' => $app->translate( 'gateway-ftp-connected' ), 'success' => true );
                       }
                   }
                   else
                   {
                       $rt = array( 'message' => $app->translate( 'gateway-ftp-no-permission' ), 'success' => false );
                   }
               }

           }
           else
           {
               $rt = array( 'message' => $app->translate( 'gateway-ftp-invalid-user-or-passwd' ), 'success' => false );
           }
        }
        else
        {
            $rt = array( 'message' => $app->translate( 'gateway-ftp-invalid-ip-or-port' ), 'success' => false );
        }

    }
    catch( Exception $e )
    {
        $rt['message'] = $app->translate( $e->getMessage() );
    }

    if( !is_null( $conn_id ) )
    {
        ftp_close( $conn_id );
    }

    return JSONencode( $rt );
}
