<?php
    require_once KCONFIG . '/classes/KGatewayYaml.php';

    $AjaxFunctions['showAuditGW'] = 'showAuditGW';
    $AjaxFunctions['enableAuditGW'] = 'enableAuditGW';
    $AjaxFunctions['disableAuditGW'] = 'disableAuditGW';

    $g_app = KSystem::getInstance( );
    function tr($key)
    {
        global $g_app;
        return $g_app->translate($key);
    }

    function checkbx($name, $checked = false)
    {
        $suffix = $checked? 'checked="checked"': '';
        return "<input type='checkbox' id='{$name}' {$suffix} />";
    }

    function label_for($name)
    {
        $str = tr($name);
        return "<label for='{$name}'>{$str}</label>";
    }

    function showAuditGW( )
    {
        $app = KSystem::getInstance( );
        $gyaml = new KGatewayYaml( );
        $enabled = $gyaml->isAuditEnabled( );
        $disablefield = $enabled ? ' disabled=\'disabled\'' : '';

        $buf = "<fieldset><legend>{$app->translate( 'gateway-audit' )}</legend>";
        $buf .= '<div id=\'gateway-audit\'>';
        if( $enabled )
        {
            $startdate = strftime( "%d/%m/%Y, %H:%M", $gyaml->getAuditStartAsUnixTimestamp() );
            $enddate = strftime( "%d/%m/%Y, %H:%M", $gyaml->getAuditEndAsUnixTimestamp() );
            $buf .= "<div class='warning'>{$app->translate( 'gateway-audit-running' )}<br /><br />{$app->translate( 'gateway-audit-started-at' )}: {$startdate}<br />{$app->translate( 'gateway-audit-end-at' )}: {$enddate}</div>";
        }
        $buf .= '<table class=\'simpleTable gateway-table-form zebra\'><tbody>';
        //@formatter:off
        //$buf .= "<tr><td class='label'><label for='gateway-audit-start'>{$app->translate( 'gateway-audit-start')}: <img src='/imagens/tooltip.png' class='gateway-tooltip' id='gateway-audit-start-tooltip' /></label><br /><span class='link' id='gateway-audit-start-now'>{$app->translate( 'gateway-audit-start-now')}</span></td><td><input type='text' id='gateway-audit-start' name='Start' /></td></tr>";
        //@formatter:on
        $buf .= "<tr><td class='label' style='width:auto;'><label for='gateway-audit-end'>{$app->translate( 'gateway-audit-end')} <img src='/imagens/tooltip.png' class='gateway-tooltip' id='gateway-audit-end-tooltip' /></label></td><td style='text-align:right;'><select id='gateway-audit-end' name='End'{$disablefield}>";
        $audit = $gyaml->getAudit( );
        $end = !empty( $audit['End'] ) ? $audit['End'] / 60 : 5;
        for( $i = 5; $i <= 60; $i += 5 )
        {
            $select = $end == $i ? ' selected=\'selected\'' : '';
            $v = $i * 60;
            $buf .= "<option value='{$v}'{$select}>{$i}</option>";
        }
        $buf .= '</select></td></tr>';
        if( $enabled )
        {
            $actions = "<span>{$app->translate( 'gateway-audit-select-all-routes' )}</span> | <span>{$app->translate( 'gateway-audit-remove-all-routes' )}</span>";
        }
        else
        {
            $actions = "<span class='link' id='gateway-audit-select-all-routes'>{$app->translate( 'gateway-audit-select-all-routes' )}</span> | <span class='link' id='gateway-audit-remove-all-routes'>{$app->translate( 'gateway-audit-remove-all-routes' )}</span>";
        }
        $buf .= "<tr><td class='label' style='width:auto;'>".label_for('gateway-audit-whole-call')."</td>";
        $buf .= "<td class='checkbox'>".checkbx('gateway-audit-whole-call', $audit[WholeCall])."</td></tr>";

        $buf .= "<tr><td colspan='2' class='label' style='width:auto;'>{$app->translate( 'gateway-audit-routes')}:<br />{$actions}</td></tr>";

        $buf .= "<tr><td colspan='2'><table id='gateway-audit-routes' class='gateway-table'><thead><tr><th class='checkbox'>&nbsp;</th><th>{$app->translate( 'Name' )}</th><th>{$app->translate( 'OriginNAP' )}</th><th>{$app->translate( 'DestinationNAP' )}</th></tr></thead><tbody>";
        foreach( $gyaml->getAllRoutes() as $route )
        {
            $originNAP = $route['OriginNAP'] == '<ANY>' ? $app->translate( 'gateway-route-any' ) : $route['OriginNAP'];

            if( $gyaml->routeHasCallAnalyzerEnabled( $route['Name'] ) )
            {
                $checked = $gyaml->isRouteAudited( $route['Name'] ) ? ' checked=\'checked\'' : '';
                $buf .= "<tr><td class='checkbox'><input type='checkbox' class='gateway-audit-route'{$checked}{$disablefield} /></td><td>{$route['Name']}</td><td>{$originNAP}</td><td>{$route['DestinationNAP']}</td></tr>";
            }
            else
            {
                $buf .= "<tr><td class='checkbox'>&nbsp;</td><td><span class='gateway-route-unavailable'>{$route['Name']}</span></td><td>{$originNAP}</td><td>{$route['DestinationNAP']}</td></tr>";
            }
        }
        $buf .= '</tbody></table></td></tr></tbody></table><br />';
        $buf .= "<div class='textAlignRight'>";
        if( $enabled )
        {
            $buf .= "<input id='gateway-audit-disable' type='button' value='{$app->translate( 'gateway-audit-disable' )}' />";
        }
        else
        {
            $buf .= "<input id='gateway-audit-enable' type='button' value='{$app->translate( 'gateway-audit-enable' )}' />";
        }
        $buf .= '</div>';
        $buf .= '</div></fieldset>';
        return $buf;
    }

    function enableAuditGW( )
    {
        require_once KROOT . '/json_util.php';

        $app = KSystem::getInstance( );
        $gyaml = new KGatewayYaml( );
        $ret = array( 'success' => false, 'message' => $app->translate( 'gateway-audit-enable-error' ) );

        try
        {
            $gyaml->enableAudit( $_POST );
            $gyaml->save( );
            if( reload( 'k3lreload' ) === 'OK' )
            {
                $ret = array( 'success' => true, 'message' => $app->translate( 'gateway-audit-enabled' ) );
            }
        }
        catch( Exception $e )
        {
            $ret['message'] = $app->translate( $e->getMessage( ) );
        }

        return JSONencode( $ret );
    }

    function disableAuditGW( )
    {
        require_once KROOT . '/json_util.php';

        $app = KSystem::getInstance( );
        $gyaml = new KGatewayYaml( );
        $ret = array( 'success' => false, 'message' => $app->translate( 'gateway-audit-disable-error' ) );

        try
        {
            $gyaml->disableAudit( );
            $gyaml->save( );
            if( reload( 'k3lreload' ) === 'OK' )
            {
                $ret = array( 'success' => true, 'message' => $app->translate( 'gateway-audit-disabled' ) );
            }
        }
        catch( Exception $e )
        {
            $ret['message'] = $app->translate( $e->getMessage( ) );
        }

        return JSONencode( $ret );
    }
