<?php
    require_once KCONFIG . '/classes/classKYaml.php';

    class KGatewayProfileYamlException extends KYamlException
    {
        /* VOID */
    }

    class KGatewayProfileYaml extends KYaml
    {
        static $hmpAnalyticsEvents = array( 'OnVoiceMail', 'OnAnsweringMachine', 'OnInterceptionSignaling', 'OnInterceptionAnnouncement', 'OnForwardAnnouncement', 'OnUnknownAnnouncement', 'OnPortabilityIdentification', 'OnRingbackSignal', 'OnShortHumanAnswer', 'OnLongHumanAnswer', 'OnFax', 'OnNoSpeech' );
        //@formatter:off
        //TODO: Implement notification
        static $hmpAnalyticsActions = array( 'Connect', 'Drop', 'PlayAndDrop', 'Notify', 'Ignore' );
        static $features = array( 'EchoCanceller' , 'DTMFSupression', 'AutoGainControl', 'ManualGainControl' );
        //@formatter:on
        static $faxModes = array( 'Disabled', 'T38', 'PassThrough' );

        private static $kgatewayyaml = null;
        /**
         * If true, chain save to KGatewayYaml instance before commit
         */
        protected $chainSave = false;

        public function __construct( )
        {
            // validation rules
            //@formatter:off
            $this->rules = array(
                'Default' => array(
                    'type' => KYaml::TYPE_STRING
                ), // Default
                'CallAnalyzerBehaviors' => array(
                    'type' => KYaml::TYPE_LISTHASH,
                    'children' => array(
                        'DropBehaviors' => array(
                            'type' => KYaml::TYPE_LIST,
                            'default' => array(),
                            'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                            'children' => array(
                                '?' => array(
                                    'type' => KYaml::TYPE_LISTHASH,
                                    'children' => array(
                                        'Name' => array(
                                            'type' => KYaml::TYPE_STRING
                                        ),
                                        'SIPCode' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            //'default' => false,
                                            'flags' => KYaml::FLAG_NOTREQUIRED
                                        ),
                                        'SIPHeader' => array(
                                            'type' => KYaml::TYPE_STRING,
                                            //'default' => '',
                                            'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NOTREQUIRED
                                        ),
                                        'ISDNCause' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'flags' => KYaml::FLAG_NOTREQUIRED
                                            //'default' => 31 // kq931cNormalUnspecified @ KISDN.h
                                        )
                                    )
                                ) // CallAnalyzerBehaviors.DropBehaviors.?
                            )
                        ), // CallAnalyzerBehaviors.DropBehaviors
                        'NotifyBehaviors' => array(
                            'type' => KYaml::TYPE_LIST,
                            'default' => array(),
                            'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                            'children' => array(
                                '?' => array(
                                    'type' => KYaml::TYPE_LISTHASH,
                                    'children' => array(
                                        'Name' => array(
                                            'type' => KYaml::TYPE_STRING
                                        ),
                                        'SIPHeader' => array(
                                            'type' => KYaml::TYPE_STRING
                                        )
                                    )
                                ) // CallAnalyzerBehaviors.NotifyBehaviors.?
                            )
                        ), // CallAnalyzerBehaviors.NotifyBehaviors
                        'AnalyticsBehaviors' => array(
                            'type' => KYaml::TYPE_LIST,
                            'default' => array(),
                            'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                            'children' => array(
                                '?' => array(
                                    'type' => KYaml::TYPE_LISTHASH,
                                    'children' => array(
                                        'Name' => array(
                                            'type' => KYaml::TYPE_STRING
                                        ),
                                        'Locale' => array(
                                            'type' => KYaml::TYPE_STRING,
                                            'default' => 'pt-BR'
                                        ),
                                        'Classification' => array(
                                            'type' => KYaml::TYPE_LISTHASH,
                                            'children' => array(
                                                'OnTimeout' => $this->hmpAnalyticsEventRules(),
                                                'OnVoiceMail' => $this->hmpAnalyticsEventRules(),
                                                'OnAnsweringMachine' => $this->hmpAnalyticsEventRules(),
                                                'OnInterceptionSignaling' => $this->hmpAnalyticsEventRules(),
                                                'OnInterceptionAnnouncement' => $this->hmpAnalyticsEventRules(),
                                                'OnForwardAnnouncement' => $this->hmpAnalyticsEventRules(),
                                                'OnUnknownAnnouncement' => $this->hmpAnalyticsEventRules(),
                                                'OnPortabilityIdentification' => $this->hmpAnalyticsEventRules(),
                                                'OnRingbackSignal' => $this->hmpAnalyticsEventRules(),
                                                'OnShortHumanAnswer' => $this->hmpAnalyticsEventRules(),
                                                'OnLongHumanAnswer' => $this->hmpAnalyticsEventRules(),
                                                'OnFax' => $this->hmpAnalyticsEventRules(),
                                                'OnNoSpeech' => $this->hmpAnalyticsEventRules()
                                            ),
                                            'default' => array(
                                                'OnTimeout' => array( 'Action' => 'Connect' ),
                                                'OnVoiceMail' => array( 'Action' => 'Connect' ),
                                                'OnAnsweringMachine' => array( 'Action' => 'Connect' ),
                                                'OnInterceptionSignaling' => array( 'Action' => 'Connect' ),
                                                'OnInterceptionAnnouncement' => array( 'Action' => 'Connect' ),
                                                'OnForwardAnnouncement' => array( 'Action' => 'Connect' ),
                                                'OnUnknownAnnouncement' => array( 'Action' => 'Connect' ),
                                                'OnPortabilityIdentification' => array( 'Action' => 'Connect' ),
                                                'OnRingbackSignal' => array( 'Action' => 'Connect' ),
                                                'OnShortHumanAnswer' => array( 'Action' => 'Connect' ),
                                                'OnLongHumanAnswer' => array( 'Action' => 'Connect' ),
                                                'OnFax' => array( 'Action' => 'Connect' ),
                                                'OnNoSpeech' => array( 'Action' => 'Connect' ),
                                            )
                                        )
                                    )
                                ) // CallAnalyzerBehaviors.AnalyticsBehaviors.?
                            )
                        ) // CallAnalyzerBehaviors.AnalyticsBehaviors
                    )
                ), // CallAnalyzerBehaviors
                'Profiles' => array(
                    'type' => KYaml::TYPE_LIST,
                    'default' => array(),
                    'flags' => KYaml::FLAG_NORMALIZE,
                    'children' => array(
                        '?' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'children' => array(
                                'Name' => array(
                                    'type' => KYaml::TYPE_STRING
                                ),
                                'ReadOnly' => array(
                                    'type' => KYaml::TYPE_BOOLEAN,
                                    'default' => false
                                ),
                                'EnableFeatures' => array(
                                    'type' => KYaml::TYPE_LIST,
                                    'default' => array(),
                                    'flags' => KYaml::FLAG_EMPTY,
                                    'children' => array(
                                        '?' => array(
                                            'type' => KYaml::TYPE_STRING
                                        ) // Profiles.?.EnableFeatures.?
                                    )
                                ), // Profiles.?.EnableFeatures
                                'DisableFeatures' => array(
                                    'type' => KYaml::TYPE_LIST,
                                    'default' => array(),
                                    'flags' => KYaml::FLAG_EMPTY,
                                    'children' => array(
                                        '?' => array(
                                            'type' => KYaml::TYPE_STRING
                                        ) // Profiles.?.DisableFeatures.?
                                    )
                                ), // Profiles.?.DisableFeatures
                                'FAX' => array(
                                    'type' => KYaml::TYPE_LISTHASH,
                                    'children' => array(
                                        'Mode' => array(
                                            'type' => KYaml::TYPE_STRING,
                                            'default' => '',
                                            'flags' => KYaml::FLAG_EMPTY
                                        ),
                                        'EnableFeatures' => array(
                                            'type' => KYaml::TYPE_LIST,
                                            'default' => array(),
                                            'flags' => KYaml::FLAG_EMPTY,
                                            'children' => array(
                                                '?' => array(
                                                    'type' => KYaml::TYPE_STRING
                                                ) // Profiles.?.FAX.EnableFeatures.?
                                            )
                                        ), // Profiles.?.FAX.EnableFeatures
                                        'DisableFeatures' => array(
                                            'type' => KYaml::TYPE_LIST,
                                            'default' => array(),
                                            'flags' => KYaml::FLAG_EMPTY,
                                            'children' => array(
                                                '?' => array(
                                                    'type' => KYaml::TYPE_STRING
                                                ) // Profiles.?.FAX.DisableFeatures.?
                                            )
                                        ), // Profiles.?.FAX.DisableFeatures
                                    )
                                ), // Profiles.?.FAX
                                'R2' => array(
                                    'type' => KYaml::TYPE_LISTHASH,
                                    'children' => array(
                                        'DoubleAnswer' => array(
                                            'type' => KYaml::TYPE_LISTHASH,
                                            'children' => array(
                                                'Enabled' => array(
                                                    'type' => KYaml::TYPE_BOOLEAN,
                                                    'default' => true
                                                ),
                                                'Timeout' => array(
                                                    'type' => KYaml::TYPE_INTEGER,
                                                    'default' => 1000
                                                )
                                            )
                                        ) // Profiles.?.R2.DoubleAnswer
                                    )
                                ), // Profiles.?.R2
                                'SIP' => array(
                                    'type' => KYaml::TYPE_LISTHASH,
                                    'children' => array(
                                        'CustomHeaders' => array(
                                            'type' => KYaml::TYPE_LIST,
                                            'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_NORMALIZE,
                                            'children' => array(
                                            	'?' => array(
                                            		'type' => KYaml::TYPE_LISTHASH,
                                            		'children' => array( 
														'Name' => array(
															'type' => KYaml::TYPE_STRING,
															'default' => ''
														),
														'Value' => array(
															'type' => KYaml::TYPE_STRING,
															'default' => ''
														)
													)
												) // //Profiles.?.SIP.CustomHeaders.?
											) // //Profiles.?.SIP.CustomHeaders
                                         ),
                                        'RelayExtensionHeaders' => array(
                                            'type' => KYaml::TYPE_BOOLEAN,
                                            'default' => false
                                        ) //Profiles.?.SIP.RelayExtensionHeaders
                                    ) 
                                ), // Profiles.?.SIP
                                'Cadences' => array(
                                    'type' => KYaml::TYPE_LISTHASH,
                                    'children' => array(
                                        'Enabled' => array(
                                            'type' => KYaml::TYPE_BOOLEAN,
                                            'default' => true
                                        ),
                                        'Trying' => array(
                                            'type' => KYaml::TYPE_STRING,
                                            'default' => '25,500,25,2000'
                                        ),
                                        'Ringback' => array(
                                            'type' => KYaml::TYPE_STRING,
                                            'default' => '1000,4000'
                                        )
                                    )
                                ), // Profiles.?.Cadences
                                'VolumeGain' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'default' => 0
                                ),
                                'CallAnalyzer' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => ''
                                ), // Profiles.?.CallAnalyzer
                                'CauseMap' => array(
                                    'type' => KYaml::TYPE_LIST,
                                    'flags' => KYaml::FLAG_NORMALIZE,
                                    'children' => array(
                                        '?' => array(
                                            'type' => KYaml::TYPE_LISTHASH,
                                            'children' => array(
                                                'Id' => array(
                                                    'type' => KYaml::TYPE_INTEGER
                                                ),
                                                'ISDN' => array(
                                                    'type' => KYaml::TYPE_INTEGER
                                                ),
                                                'R2' => array(
                                                    'type' => KYaml::TYPE_INTEGER
                                                ),
                                                'SIP' => array(
                                                    'type' => KYaml::TYPE_INTEGER
                                                ),
                                                // issue #7596
                                                'Retry' => array(
                                                    'type' => KYaml::TYPE_BOOLEAN,
                                                    'default' => false
                                                )
                                            )
                                        ) // Profiles.?.CauseMap.?
                                    )
                                ) // Profiles.?.CauseMap
                            )
                        ) // Profiles.?
                    ),
                ),
            );
            //@formatter:on

            $app = KSystem::getInstance( );
            if( PHP_OS != 'WINNT' )
            {
                $this->validationProgram = '/usr/sbin/khomp-sudo /etc/init.d/kgateway validate-config';
            }
            parent::__construct( $app->getConfigDir( ) . DIRECTORY_SEPARATOR . 'gatewayprofile.yaml' );
        }

        ///////////////////////////////////////////////////////////////////////
        // Misc methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Get KGatewayYaml instance
         *
         * @return KGatewayYaml
         */
        public function getKGatewayYaml( )
        {
            if( is_null( self::$kgatewayyaml ) )
            {
                require_once KCONFIG . '/classes/KGatewayYaml.php';
                $this->setKGatewayYaml( new KGatewayYaml( ) );
                self::$kgatewayyaml->setKGatewayProfileYaml( $this );
            }
            return self::$kgatewayyaml;
        }

        /**
         * Set KGatewayYaml instance
         *
         * @param KGatewayYaml $instance
         */
        public function setKGatewayYaml( KGatewayYaml $instance )
        {
            self::$kgatewayyaml = $instance;
        }

        /**
         * If configured, chain the save to the inner KGatewayYaml object
         *
         * @see KYaml::commit
         */
        protected function commit( )
        {
            if( $this->chainSave )
            {
                try
                {
                    $this->chainSave = false;
                    $this->getKGatewayYaml( )->save( );
                }
                catch( Exception $e )
                {
                    $this->rollback( );
                    throw $e;
                }
            }
            parent::commit( );
        }

        ///////////////////////////////////////////////////////////////////////
        // Internal use only methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Rules for call analyzer hmp analytics events
         *
         * @return array
         */
        protected function hmpAnalyticsEventRules( )
        {
            //@formatter:off
            return array(
                'type' => KYaml::TYPE_LISTHASH,
                'children' => array(
                    'Action' => array(
                        'type' => KYaml::TYPE_STRING,
                        'default' => 'Connect'
                    ),
                    'Timeout' => array(
                        'type' => KYaml::TYPE_INTEGER,
                        'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NOTREQUIRED
                    ),
                    'NotifyBehavior' => array(
                        'type' => KYaml::TYPE_STRING,
                        'default' => '',
                        'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NOTREQUIRED
                    ),
                    'DropBehavior' => array(
                        'type' => KYaml::TYPE_STRING,
                        'default' => '',
                        'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NOTREQUIRED
                    ),
                    'AudioFile' => array(
                        'type' => KYaml::TYPE_STRING,
                        'default' => '',
                        'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NOTREQUIRED
                    ),
                    'Patterns' => array(
                        'type' => KYaml::TYPE_LIST,
                        'default' => array(),
                        'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                        'children' => array(
                            '?' => array(
                                'type' => KYaml::TYPE_LISTHASH,
                                'children' => array(
                                    'Action' => array(
                                        'type' => KYaml::TYPE_STRING,
                                        'default' => 'Connect'
                                    ),
                                    'NotifyBehavior' => array(
                                        'type' => KYaml::TYPE_STRING,
                                        'default' => '',
                                        'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NOTREQUIRED
                                    ),
                                    'DropBehavior' => array(
                                        'type' => KYaml::TYPE_STRING,
                                        'default' => '',
                                        'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NOTREQUIRED
                                    ),
                                    'AudioFile' => array(
                                        'type' => KYaml::TYPE_STRING,
                                        'default' => '',
                                        'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NOTREQUIRED
                                    ),
                                    'Pattern' => array(
                                        'type' => KYaml::TYPE_STRING,
                                    )
                                )
                            )
                        )
                    )
                ),
                'default' => array(
                    'Action' => 'Connect',
                    'Patterns' => array()
                )
            );
            //@formatter:on
        }

        /**
         * Check if named behavior exists
         *
         * @param string $type behavior type
         * @param string $name behavior name
         * @return boolean
         */
        protected function hasBehavior( $type, $name )
        {
            foreach( $this->yaml['CallAnalyzerBehaviors']["{$type}Behaviors"] as &$behavior )
            {
                if( $behavior['Name'] === $name )
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * Get the named behavior
         *
         * @param string $type behavior type
         * @param string $name behavior name
         * @return null|array
         */
        protected function getBehavior( $type, $name )
        {
            foreach( $this->yaml['CallAnalyzerBehaviors']["{$type}Behaviors"] as &$behavior )
            {
                if( $behavior['Name'] === $name )
                {
                    return $behavior;
                }
            }
            return null;
        }

        /**
         * Count defined behaviors
         *
         * @param string $type behavior type
         * @return int
         */
        protected function countBehaviors( $type )
        {
            return count( $this->yaml['CallAnalyzerBehaviors']["{$type}Behaviors"] );
        }

        /**
         * Set new or replace named behavior
         *
         * @param string $type behavior type
         * @param array $data behavior data array
         * @param string $oldName behavior previous name (if renamed)
         * @throws KGatewayProfileYamlException
         */
        protected function setBehavior( $type, $data, $oldName )
        {
            if( $oldName != $data['Name'] && $this->hasBehavior( $type, $data['Name'] ) )
            {
                throw new KGatewayProfileYamlException( 'gateway-profile-error-duplicated-behavior' );
            }
            // new behavior
            if( empty( $oldName ) )
            {
                $this->yaml['CallAnalyzerBehaviors']["{$type}Behaviors"][] = $data;
            }
            // update behavior
            else
            {
                $notfound = true;
                foreach( $this->yaml['CallAnalyzerBehaviors']["{$type}Behaviors"] as &$behavior )
                {
                    if( $behavior['Name'] == $oldName )
                    {
                        $this->renameBehavior( $type, $data['Name'], $oldName );
                        $behavior = $data;
                        $notfound = false;
                        break;
                    }
                }
                if( $notfound )
                {
                    throw new KGatewayProfileYamlException( 'gateway-profile-error-behavior-not-found' );
                }
            }
        }

        /**
         * Delete named behavior
         *
         * @param string $type behavior type
         * @param string $name behavior name
         * @return boolean
         * @throws KGatewayProfileYamlException
         */
        protected function deleteBehavior( $type, $name )
        {
            if( $this->isBehaviorInUse( $type, $name ) )
            {
                throw new KGatewayProfileYamlException( 'gateway-profile-error-behavior-in-use' );
            }

            foreach( $this->yaml['CallAnalyzerBehaviors']["{$type}Behaviors"] as $idx => &$behavior )
            {
                if( $behavior['Name'] == $name )
                {
                    unset( $this->yaml['CallAnalyzerBehaviors']["{$type}Behaviors"][$idx] );
                    return true;
                }
            }
            return false;
        }

        /**
         * Check if the named behavior is in use by any profile
         *
         * @param string $type behavior type
         * @param string $name behavior name
         * @return boolean
         */
        protected function isBehaviorInUse( $type, $name )
        {
            $key = "{$type}Behavior";
            if ( $type === "Analytics" )
            {
                foreach( $this->getAllProfiles() as $profile )
                {
                    if( isset( $profile['CallAnalyzer'] ) && $profile['CallAnalyzer'] === $name )
                    {
                        return true;
                    }
                }
            }
            else
            {
                foreach( $this->getAllAnalyticsBehaviors() as $behavior )
                {
                    foreach( KGatewayProfileYaml::$hmpAnalyticsEvents as $event )
                    {
                        if( isset( $behavior['Classification'][$event][$key] )
                            && $behavior['Classification'][$event][$key] === $name )
                        {
                            return true;
                        }

                        foreach( $behavior['Classification'][$event]["Patterns"] as $pattern )
                        {
                            if( isset( $pattern[$key] ) && $pattern[$key] === $name )
                            {
                                return true;
                            }
                        }
                    }
                }
            }
            return false;
        }

        /**
         * Behavior was renamed
         *
         * @param string $type behavior type
         * @param string $name behavior new name
         * @param string $oldName behavior old name
         */
        public function renameBehavior( $type, $name, $oldName )
        {
            $key = "{$type}Behavior";
            if ( $type === "Analytics" )
            {
                foreach( $this->yaml['Profiles'] as &$profile )
                {
                    if( isset( $profile['CallAnalyzer'] ) && $profile['CallAnalyzer'] === $oldName )
                    {
                        $profile['CallAnalyzer'] = $name;
                    }
                }
            }
            else
            {
                foreach( $this->yaml['CallAnalyzerBehaviors']['AnalyticsBehaviors'] as &$behavior )
                {
                    foreach( KGatewayProfileYaml::$hmpAnalyticsEvents as $event )
                    {
                        if( isset( $behavior['Classification'][$event][$key] )
                            && $behavior['Classification'][$event][$key] === $oldName )
                        {
                            $behavior['Classification'][$event][$key] = $name;
                        }

                        foreach( $behavior['Classification'][$event]["Patterns"] as $pattern )
                        {
                            if( isset( $pattern[$key] ) && $pattern[$key] === $oldName )
                            {
                                $pattern[$key] = $name;
                            }
                        }
                    }
                }
            }
        }

        ///////////////////////////////////////////////////////////////////////
        // DropBehaviors manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Check if named behavior exists
         *
         * @param string $name behavior name
         * @return boolean
         */
        public function hasDropBehavior( $name )
        {
            return $this->hasBehavior( 'Drop', $name );
        }

        /**
         * Get the named behavior
         *
         * @param string $name behavior name
         * @return null|array
         */
        public function getDropBehavior( $name )
        {
            return $this->getBehavior( 'Drop', $name );
        }

        /**
         * Get all behaviors
         *
         * @return array
         */
        public function getAllDropBehaviors( )
        {
            return $this->yaml['CallAnalyzerBehaviors']['DropBehaviors'];
        }

        /**
         * Count defined behaviors
         *
         * @return int
         */
        public function countDropBehaviors( )
        {
            return $this->countBehaviors( 'Drop' );
        }

        /**
         * Set new or replace named behavior
         *
         * @param array $data behavior data array
         * @param string $oldName behavior previous name (if renamed)
         * @throws KGatewayProfileYamlException
         */
        public function setDropBehavior( $data, $oldName = '' )
        {
            return $this->setBehavior( 'Drop', $data, $oldName );
        }

        /**
         * Delete named behavior
         *
         * @param string $type behavior type
         * @param string $name behavior name
         * @return boolean
         * @throws KGatewayProfileYamlException
         */
        public function deleteDropBehavior( $name )
        {
            return $this->deleteBehavior( 'Drop', $name );
        }

        ///////////////////////////////////////////////////////////////////////
        // NotifyBehaviors manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Check if named behavior exists
         *
         * @param string $name behavior name
         * @return boolean
         */
        public function hasNotifyBehavior( $name )
        {
            return $this->hasBehavior( 'Notify', $name );
        }

        /**
         * Get the named behavior
         *
         * @param string $name behavior name
         * @return null|array
         */
        public function getNotifyBehavior( $name )
        {
            return $this->getBehavior( 'Notify', $name );
        }

        /**
         * Get all behaviors
         *
         * @return array
         */
        public function getAllNotifyBehaviors( )
        {
            return $this->yaml['CallAnalyzerBehaviors']['NotifyBehaviors'];
        }

        /**
         * Count defined behaviors
         *
         * @return int
         */
        public function countNotifyBehaviors( )
        {
            return $this->countBehaviors( 'Notify' );
        }

        /**
         * Set new or replace named behavior
         *
         * @param array $data behavior data array
         * @param string $oldName behavior previous name (if renamed)
         * @throws KGatewayProfileYamlException
         */
        public function setNotifyBehavior( $data, $oldName = '' )
        {
            return $this->setBehavior( 'Notify', $data, $oldName );
        }

        /**
         * Delete named behavior
         *
         * @param string $type behavior type
         * @param string $name behavior name
         * @return boolean
         * @throws KGatewayProfileYamlException
         */
        public function deleteNotifyBehavior( $name )
        {
            return $this->deleteBehavior( 'Notify', $name );
        }

        ///////////////////////////////////////////////////////////////////////
        // AnalyticsBehaviors manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Check if named behavior exists
         *
         * @param string $name behavior name
         * @return boolean
         */
        public function hasAnalyticsBehavior( $name )
        {
            return $this->hasBehavior( 'Analytics', $name );
        }

        /**
         * Get the named behavior
         *
         * @param string $name behavior name
         * @return null|array
         */
        public function getAnalyticsBehavior( $name )
        {
            return $this->getBehavior( 'Analytics', $name );
        }

        /**
         * Get all behaviors
         *
         * @return array
         */
        public function getAllAnalyticsBehaviors( )
        {
            return $this->yaml['CallAnalyzerBehaviors']['AnalyticsBehaviors'];
        }

        /**
         * Count defined behaviors
         *
         * @return int
         */
        public function countAnalyticsBehaviors( )
        {
            return $this->countBehaviors( 'Analytics' );
        }

        /**
         * Set new or replace named behavior
         *
         * @param array $data behavior data array
         * @param string $oldName behavior previous name (if renamed)
         * @throws KGatewayProfileYamlException
         */
        public function setAnalyticsBehavior( $data, $oldName = '' )
        {
            return $this->setBehavior( 'Analytics', $data, $oldName );
        }

        /**
         * Delete named behavior
         *
         * @param string $type behavior type
         * @param string $name behavior name
         * @return boolean
         * @throws KGatewayProfileYamlException
         */
        public function deleteAnalyticsBehavior( $name )
        {
            return $this->deleteBehavior( 'Analytics', $name );
        }

        ///////////////////////////////////////////////////////////////////////
        // Profiles manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Check if named profile exists
         *
         * @param string $name profile name
         * @return boolean
         */
        public function hasProfile( $name )
        {
            foreach( $this->yaml['Profiles'] as &$profile )
            {
                if( $profile['Name'] === $name )
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * Get the named profile
         *
         * @param string $name profile name
         * @return null|array
         */
        public function getProfile( $name )
        {
            foreach( $this->yaml['Profiles'] as &$profile )
            {
                if( $profile['Name'] === $name )
                {
                    return $profile;
                }
            }
            return null;
        }

        /**
         * Get all profiles
         *
         * @return array
         */
        public function getAllProfiles( )
        {
            return $this->yaml['Profiles'];
        }

        /**
         * Count defined profiles
         *
         * @return int
         */
        public function countProfiles( )
        {
            return count( $this->yaml['Profiles'] );
        }

        /**
         * Compare two cause map items
         * 
         * This function is compatible with usort, uksort and uasort
         * 
         * @param array $left Left operator
         * @param array $right Right operator
         * @return int 0 if equals, 1 if left is grater, -1 if right is grater
         */
        static public function causeMapSortComparator( $left, $right )
        {
            if( $left['Id'] == $right['Id'] )
            {
                return 0;
            }
            return $left['Id'] < $right['Id'] ? -1 : 1;
        }

        /**
         * Set new or replace profile
         *
         * @param array $data profile data
         * @param string $oldName profile previous name (if renamed)
         * @param boolean $force force read only update
         * @throws KGatewayProfileYamlException
         */
        public function setProfile( $data, $oldName = '', $force = false )
        {
            if( $oldName != $data['Name'] && $this->isProfileInUse( $data['Name'] ) )
            {
                throw new KGatewayProfileYamlException( 'gateway-profile-error-duplicated-profile' );
            }

            // set FAX mode features
            switch( $data['FAX']['Mode'] )
            {
                case 'Disabled':
                    $data['FAX']['EnableFeatures'] = array( );
                    $data['FAX']['DisableFeatures'] = array( );
                    break;
                case 'T38':
                    $data['FAX']['EnableFeatures'] = array( 'EchoCanceller' );
                    $data['FAX']['DisableFeatures'] = array( 'All' );
                    break;
                case 'PassThrough':
                    $data['FAX']['EnableFeatures'] = array( );
                    $data['FAX']['DisableFeatures'] = array( 'All' );
                    break;
                default:
                    throw new KGatewayProfileYamlException( 'gateway-profile-error-invalid-fax-mode' );
            }

            // issue #5308, cause map implementation
            foreach( $data['CauseMap'] as &$causes )
            {
                if( $causes['ISDN'] < 1 || $causes['ISDN'] > 127 )
                {
                    trigger_error( "Profile = {$data['Name']}, Id = {$causes['Id']}, ISDN = {$causes['ISDN']} out of range", E_USER_NOTICE );
                    throw new KGatewayProfileYamlException( 'gateway-profile-cause-map-error-isdn-out-of-range' );
                }
                if( $causes['R2']  < 0 || $causes['R2'] > 8 )
                {
                    trigger_error( "Profile = {$data['Name']}, Id = {$causes['Id']}, R2 = {$causes['R2']} out of range", E_USER_NOTICE );
                    throw new KGatewayProfileYamlException( 'gateway-profile-cause-map-error-r2-out-of-range' );
                }
                if( $causes['SIP'] < 200 || $causes['SIP'] > 699 )
                {
                    trigger_error( "Profile = {$data['Name']}, Id = {$causes['Id']}, SIP = {$causes['SIP']} out of range", E_USER_NOTICE );
                    throw new KGatewayProfileYamlException( 'gateway-profile-cause-map-error-sip-out-of-range' );
                }
            }

            $data[ 'VolumeGain' ] = ( empty( $data[ 'VolumeGain' ] ) ) ? 0 : (int)$data[ 'VolumeGain' ];
            if ( $data[ 'VolumeGain' ] > 10 ) {
                $data[ 'VolumeGain' ] = 10;
            } else if ( $data[ 'VolumeGain' ] < -10) {
                $data['VolumeGain'] = -10;
            }

            // new
            if( empty( $oldName ) )
            {
                $this->yaml['Profiles'][] = $data;
            }
            // update
            else
            {
                $notfound = true;
                foreach( $this->yaml['Profiles'] as &$profile )
                {
                    if( $profile['Name'] === $oldName )
                    {
                        if( $profile['ReadOnly'] && !$force )
                        {
                            throw new KGatewayProfileYamlException( 'gateway-profile-error-read-only' );
                        }

                        $this->chainSave = true;
                        $gatewayyaml = $this->getKGatewayYaml( );
                        $gatewayyaml->renameNapsProfile( $data['Name'], $oldName );
                        $gatewayyaml->renameRoutesProfile( $data['Name'], $oldName );
                        if( $this->isDefaultProfile( $oldName ) )
                        {
                            $this->yaml['Default'] = $data['Name'];
                        }
                        $profile = $data;
                        $notfound = false;
                        break;
                    }
                }
                if( $notfound )
                {
                    throw new KGatewayProfileYamlException( 'gateway-profile-error-not-found' );
                }
            }
        }

        /**
         * Delete named profile
         *
         * @param string $name profile name
         * @throws KGatewayProfileYamlException
         */
        public function deleteProfile( $name )
        {
            if( $this->isProfileInUse( $name ) )
            {
                throw new KGatewayProfileYamlException( 'gateway-profile-error-in-use' );
            }

            foreach( $this->yaml['Profiles'] as $idx => $profile )
            {
                if( $profile['Name'] === $name )
                {
                    if( $profile['ReadOnly'] )
                    {
                        throw new KGatewayProfileYamlException( 'gateway-profile-error-read-only' );
                    }

                    unset( $this->yaml['Profiles'][$idx] );
                    return true;
                }
            }
            return false;
        }

        /**
         * Check if a profile is in use by Default, any NAP or any Route
         *
         * @param string $name profile name
         * @return boolean
         */
        public function isProfileInUse( $name )
        {
            // is default
            if( $name === $this->yaml['Default'] )
            {
                return true;
            }

            $_gyaml = $this->getKGatewayYaml( );

            // is being used by any nap
            foreach( $_gyaml->getAllNaps() as $nap )
            {
                if( isset( $nap['Profile'] ) && $nap['Profile'] === $name )
                {
                    return true;
                }
            }

            // is being used by any route
            foreach( $_gyaml->getAllRoutes() as $route )
            {
                if( isset( $route['DestinationProfile'] ) && $route['DestinationProfile'] === $name )
                {
                    return true;
                }
            }

            return false;
        }

        /**
         * Get default profile
         * 
         * @return null|array
         */
        public function getDefaultProfile( )
        {
            return $this->getProfile( $this->yaml['Default'] );
        }

        /**
         * Set new default profile
         *
         * @param string $name profile name
         * @return boolean
         */
        public function setDefaultProfile( $name )
        {
            if( !$this->hasProfile( $name ) )
            {
                return false;
            }
            $this->yaml['Default'] = $name;
            return true;
        }

        /**
         * Check if named profile is the default
         *
         * @param string $name profile name
         * @return boolean
         */
        public function isDefaultProfile( $name )
        {
            return $this->yaml['Default'] === $name;
        }

    }
