<?php
    require_once KCONFIG . '/classes/classKYaml.php';

    if( PHP_OS == 'WINNT' )
    {
        //  strptime is not implemented in Windows systems
        // implementation from: http://php.net/manual/en/function.strptime.php
        function strptime( $date, $format )
        {
            $masks = array( '%d' => '(?P<d>[0-9]{2})', '%m' => '(?P<m>[0-9]{2})', '%Y' => '(?P<Y>[0-9]{4})', '%H' => '(?P<H>[0-9]{2})', '%M' => '(?P<M>[0-9]{2})', '%S' => '(?P<S>[0-9]{2})',
            // usw..
            );

            $rexep = "#" . strtr( preg_quote( $format ), $masks ) . "#";
            if( !preg_match( $rexep, $date, $out ) )
                return false;

            $ret = array( "tm_sec" => (int)$out['S'], "tm_min" => (int)$out['M'], "tm_hour" => (int)$out['H'], "tm_mday" => (int)$out['d'], "tm_mon" => $out['m'] ? $out['m'] - 1 : 0, "tm_year" => $out['Y'] > 1900 ? $out['Y'] - 1900 : 0, );
            return $ret;
        }

    }

    class KGatewayYamlException extends KYamlException
    {
    }

    class KGatewayYaml extends KYaml
    {
        private static $kgatewayprofileyaml = null;
        private static $audittimefmt = '%Y-%m-%d %H:%M:%S';
        public static $cdrrotateoptions = array( 'every minute', 'hourly', 'daily' );
        public static $cdrstyleoptions = array( 'per try', 'per call' /* , 'kmedia' */ );
        public static $channelallocatoralgorithms = array( 'AUTOMATIC', 'FIRSTFREE', 'ROUNDROBIN', 'LIMIT' );
        public static $g729b = array( 'G729', 'G729B' );
        public static $nattraversalmodes = array( 'DISABLED', 'STUN', 'EXTERNAL_IP_ADDRESSES' );
        public static $radiusdictionaries = array( 'KHOMP', 'CISCO', 'TELCOBRIDGES' );

        const GSMNAP_CONTRACT_RENEW_DISABLED = 0;
        const GSMNAP_SIM_CARD_SWAP_DISABLED = 0;

        const DEFAULT_RADIUS_DATE_FORMAT = '%H:%M:%S.@m %z %a %b %d %Y';

        public function __construct( )
        {
            // validation rules
            //@formatter:off
            $this->rules = array(
                'NAPs' => array(
                    'type' => KYaml::TYPE_LIST,
                    'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                    'default' => array(),
                    'children' => array(
                        '?' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'children' => array(
                                'Name' => array(
                                    'type' => KYaml::TYPE_STRING,
                                ),
                                'Type' => array(
                                    'type' => KYaml::TYPE_STRING,
                                ),
                                'ChannelAllocator' => array(
                                    'type' => KYaml::TYPE_LISTHASH,
                                    'default' => array( 'Algorithm' => 'AUTOMATIC' ),
                                    'children' => array(
                                        'Algorithm' => array(
                                            'type' => KYaml::TYPE_STRING,
                                            'default' => 'AUTOMATIC'
                                        ),
                                        // LIMIT algorithm option
                                        'Limit' => array(
                                            'type' => KYaml::TYPE_INTEGER,
                                            'flags' => KYaml::FLAG_NOTREQUIRED
                                        ),
                                        'Sort' => array(
                                            'type' => KYaml::TYPE_STRING,
                                            'flags' => KYaml::FLAG_NOTREQUIRED
                                        )
                                    ) // NAPs.?.ChannelAllocator
                                ),
                                'Profile' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_NOTREQUIRED
                                ),
                                //SIP
                                'Register' => array(
                                    'type' => KYaml::TYPE_BOOLEAN,
                                    'flags' => KYaml::FLAG_NOTREQUIRED,
                                    'default' => 'false'
                                ),
                                'Proxy' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'ProxyPort' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'default' => 0,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'Username' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_NOTREQUIRED
                                ),
                                'Password' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'Domain' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_NOTREQUIRED
                                ),
                                'DomainPort' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'default' => 0,
                                    'flags' => KYaml::FLAG_NOTREQUIRED
                                ),
                                'DomainUseIpPacketAddress' => array(
                                    'type' => KYaml::TYPE_BOOLEAN,
                                    'default' => false,
                                    'flags' => KYaml::FLAG_NOTREQUIRED
                                ),
                                'IgnoreSourcePort' => array(
                                    'type' => KYaml::TYPE_BOOLEAN,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'AuthorizationUser' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'Expires' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'default' => 0,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'UseCalledFromRequestURI' => array(
                                    'type' => KYaml::TYPE_BOOLEAN,
                                    'default' => false,
                                    'flags' => KYaml::FLAG_NOTREQUIRED
                                ),
                                'SipTransportType' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_EMPTY,
                                    'default' => 'UDP'
                                ),
                                'AudioTransportType' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_EMPTY,
                                    'default' => 'UDP'
                                ),
                                'StunServer' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'StunServerPort' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'default' => 0,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                // issues #15497, #11726
                                'NATTraversalMode' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                // issue #15497
                                'ExternalRtpAddress' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'ExternalSipAddress' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                // issue #15694
                                'ForceRtpInterface' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'ForceSipInterface' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                // issue #7247
                                'KeepAlive' => array(
                                    'type' => KYaml::TYPE_BOOLEAN,
                                    'default' => false,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'KeepAliveInterval' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'default' => 0,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                // TRUNK
                                'Links' => array(
                                    'type' => KYaml::TYPE_LIST,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                                    'default' => array(),
                                    'children' => array(
                                        '?' => array(
                                            'type' => KYaml::TYPE_STRING
                                        )
                                    ) // NAPs.?.Links
                                ),
                                //GSM
                                'TimeTrackPattern' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_EMPTY,
                                    'default' => ''
                                ),
                                'MonthlyMinutes' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                 'MonthlySms' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'MinutesToSwap' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'flags' => KYaml::FLAG_NOTREQUIRED
                                ),
                                'ContractDay' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'SharedMinutes' => array(
                                    'type' => KYaml::TYPE_BOOLEAN,
                                    'default' => false,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'SIMs' => array(
                                    'type' => KYaml::TYPE_LIST,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                                    'children' => array(
                                        '?' => array(
                                            'type' => KYaml::TYPE_STRING
                                        )
                                    )
                                ) //NAPs.?.SIMs
                            ) // NAPs.?.children
                        ) // NAPs.?
                    ) // NAPs.children
                ), //NAPs
                'CodecProfiles' => array(
                    'type' => KYaml::TYPE_LIST,
                    'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                    'default' => array(),
                    'children' => array(
                        '?' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'children' => array(
                                'Name' => array(
                                    'type' => KYaml::TYPE_STRING
                                ),
                                'CodecList' => array(
                                    'type' => KYaml::TYPE_LIST,
                                    'flags' => KYaml::FLAG_NORMALIZE,
                                    'children' => array(
                                        '?' => array(
                                            'type' => KYaml::TYPE_STRING
                                        )
                                    ) // CodecProfiles.?.CodecList
                                )
                            ) // CodecProfiles.?
                        )
                    ) // CodecProfiles
                ),
                'Routes' => array(
                    'type' => KYaml::TYPE_LIST,
                    'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                    'default' => array(),
                    'children' => array(
                        '?' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'children' => array(
                                'Name' => array(
                                    'type' => KYaml::TYPE_STRING
                                ),
                                'OriginNAP' => array(
                                    'type' => KYaml::TYPE_STRING
                                ),
                                'Called' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_EMPTY
                                ),
                                'Caller' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_EMPTY
                                ),
                                'RemapCalled' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_EMPTY
                                ),
                                'RemapCaller' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_EMPTY
                                ),
                                'DestinationNAP' => array(
                                    'type' => KYaml::TYPE_STRING
                                ),
                                'Time' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_EMPTY
                                ),
                                'Priority' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'default' => 0
                                ),
                                //SIP
                                'CodecProfile' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'DestinationProfile' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_NOTREQUIRED
                                ),
                                // Portability
                                'DestinationServiceProvider' => array(
                                    'type' => KYaml::TYPE_LIST,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY,
                                    'children' => array(
                                        '?' => array(
                                            'type' => KYaml::TYPE_STRING
                                        ) // Routes.?.DestinationServiceProvider.?
                                    )
                                ), // Routes.?.DestinationServiceProvider
                                'PortabilityRegex' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                )
                            ) // Routes.?
                        )
                    ) // Routes
                ),
                'RoutesSMS' => array(
                    'type' => KYaml::TYPE_LIST,
                    'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                    'default' => array(),
                    'children' => array(
                        '?' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'children' => array(
                                'Name' => array(
                                    'type' => KYaml::TYPE_STRING
                                ),
                                'DestinationNumber' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_EMPTY
                                ),
                                'DestinationNAP' => array(
                                    'type' => KYaml::TYPE_STRING
                                ),
                                'Priority' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'default' => 0
                                ),
                                // Portability
                                'DestinationServiceProvider' => array(
                                    'type' => KYaml::TYPE_LIST,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY,
                                    'children' => array(
                                        '?' => array(
                                            'type' => KYaml::TYPE_STRING
                                        ) // Routes.?.DestinationServiceProvider.?
                                    )
                                ), // Routes.?.DestinationServiceProvider
                                'PortabilityRegex' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                )
                            )
                        )
                    ) // RoutesSMS
                ),
                'TimeRestraints' => array(
                    'type' => KYaml::TYPE_LIST,
                    'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                    'default' => array(),
                    'children' => array(
                        '?' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'children' => array(
                                'Name' => array(
                                    'type' => KYaml::TYPE_STRING
                                ),
                                'Hour' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_EMPTY
                                ),
                                'WDay' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_EMPTY
                                ),
                                'Day' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_EMPTY
                                ),
                                'Month' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'default' => '',
                                    'flags' => KYaml::FLAG_EMPTY
                                )
                            ) // TimeRestraints.?
                        )
                    ) // TimeRestraints
                ),
                'CDR' => array(
                    'type' => KYaml::TYPE_LISTHASH,
                    'children' => array(
                        'Pattern' => array(
                            'type' => KYaml::TYPE_STRING
                        ),
                        'Rotate' => array(
                            'type' => KYaml::TYPE_STRING
                        ),
                        'BasePath' => array(
                            'type' => KYaml::TYPE_STRING
                        ),
                        'Enabled' => array(
                            'type' => KYaml::TYPE_BOOLEAN,
                            'default' => true,
                            'flags' => KYaml::FLAG_NOTREQUIRED
                        ),
                        'Style' => array(
                            'type' => KYaml::TYPE_STRING,
                            'default' => 'per try',
                            'flags' => KYaml::FLAG_NOTREQUIRED
                        ),
                        // issue #15667
                        'Radius' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'flags' => KYaml::FLAG_NOTREQUIRED,
                            'children' => array(
                                'Enabled' => array(
                                    'type' => KYaml::TYPE_BOOLEAN,
                                    'default' => false
                                ),
                                'DictionaryEnabled' => array(
                                    'type' => KYaml::TYPE_LIST,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_NORMALIZE,
                                    'children' => array(
                                        '?' => array(
                                            'type' => KYaml::TYPE_STRING
                                        ) // CDR.Radius.DictionaryEnabled.?
                                    )
                                ), // CDR.Radius.DictionaryEnabled
                                'Timeout' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'default' => 10,
                                    'validator' => 'RadiusTimeout'
                                ),
                                'Retries' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'default' => 3,
                                    'validator' => 'RadiusRetries'
                                ),
                                'ParallelRequests' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'default' => 10,
                                    'validator' => 'RadiusParallelRequests'
                                ),
                                'QueueSizeDiscardStart' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'default' => 3000,
                                    'validator' => 'RadiusQueueSizeDiscard'
                                ),
                                'QueueSizeDiscardAll' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'default' => 5000,
                                    'validator' => 'RadiusQueueSizeDiscard'
                                ),
                                'DeadServerDisabledTime' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'default' => 0,
                                    'validator' => 'RadiusDeadServerDisabledTime'
                                ),
                                'DateFormat' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    // KMedia default format
                                    'default' => self::DEFAULT_RADIUS_DATE_FORMAT
                                ),
                                'Servers' => array(
                                    'type' => KYaml::TYPE_LIST,
                                    'flags' => KYaml::FLAG_NORMALIZE | KYaml::FLAG_EMPTY,
                                    'default' => array( ),
                                    'children' => array(
                                        '?' => array(
                                            'type' => KYaml::TYPE_LISTHASH,
                                            'children' => array(
                                                'Host' => array(
                                                    'type' => KYaml::TYPE_STRING
                                                ),
                                                'Port' => array(
                                                    'type' => KYaml::TYPE_INTEGER,
                                                    'default' => 0, // automatic
                                                    'flags' => KYaml::FLAG_NOTREQUIRED,
                                                    'validator' => 'RadiusServerPort'
                                                ),
                                                'Secret' => array(
                                                    'type' => KYaml::TYPE_STRING
                                                )
                                            )
                                        ) // CDR.Radius.Servers.?
                                    )
                                ), // CDR.Radius.Servers
                            )
                        ) // CDR.Radius
                    ) // CDR
                ),
                //TODO: mark as deprecated??
                'MaxEbs' => array(
                    'type' => KYaml::TYPE_INTEGER,
                    'default' => 1
                ), // MaxEbs
                'Audit' => array(
                    'type' => KYaml::TYPE_LISTHASH,
                    'flags' => KYaml::FLAG_NOTREQUIRED,
                    'children' => array(
                        'Start' => array(
                            'type' => KYaml::TYPE_STRING,
                            'flags' => KYaml::FLAG_EMPTY,
                            'default' => ''
                        ),
                        'End' => array(
                            'type' => KYaml::TYPE_INTEGER,
                            'default' => 5
                        ),
                        'WholeCall' => array(
                            'type' => KYaml::TYPE_BOOLEAN,
                            'default' => false
                        ),
                        'Routes' => array(
                            'type' => KYaml::TYPE_LIST,
                            'flags' => KYaml::FLAG_EMPTY,
                            'default' => array( ),
                            'children' => array(
                                '?' => array(
                                    'type' => KYaml::TYPE_STRING
                                )
                            ) // Audit.Routes.?
                        ) // Audit.Routes
                    )
                ), // Audit
                'Portability' => array(
                    'type' => KYaml::TYPE_LISTHASH,
                    'flags' => KYaml::FLAG_NOTREQUIRED,
                    'children' => array(
                        'Driver' => array(
                            'type' => KYaml::TYPE_STRING,
                            'flags' => KYaml::FLAG_NOTREQUIRED,
                            'default' => 'DISABLED'
                        ),
                        'DriverOptions' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY,
                            'children' => array(
                                'Url' => array(
                                    'type' => KYaml::TYPE_STRING
                                ),
                                'Method' => array(
                                    'type' => KYaml::TYPE_STRING
                                ),
                                'ResponsePattern' => array(
                                    'type' => KYaml::TYPE_STRING,
                            'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'Timeout' => array(
                                    'type' => KYaml::TYPE_INTEGER
                                ),
                                'ErrorMaxToSleep' => array(
                                    'type' => KYaml::TYPE_INTEGER
                                ),
                                'ErrorSleepTime' => array(
                                    'type' => KYaml::TYPE_INTEGER
                                ),
                                'Username' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'Password' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_NOTREQUIRED | KYaml::FLAG_EMPTY
                                ),
                                'AdditionalParams' => array(
                                    'type' => KYaml::TYPE_LISTHASH,
                                    'children' => array(),
                                    'flags' => KYaml::FLAG_NOTREQUIRED
                        )
                    )
                        )
                    )
                ), // Portability
                'SIMCards' => array(
                    'type' => KYaml::TYPE_LIST,
                    'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE,
                    'default' => array(),
                    'children' => array(
                        '?' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'children' => array(
                                'ICCID' => array(
                                    'type' => KYaml::TYPE_STRING
                                ),
                                'Number' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'flags' => KYaml::FLAG_EMPTY
                                )
                            ) // SIMCards.?
                        )
                    )
                ), // SIMCards
                'TimeTrackPatterns' => array(
                    'type' => KYaml::TYPE_LIST,
                    'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NORMALIZE | KYaml::FLAG_NOTREQUIRED,
                    'default' => array(),
                    'children' => array(
                        '?' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'children' => array(
                                'Name' => array(
                                    'type' => KYaml::TYPE_STRING
                                ),
                                'Delay' => array(
                                    'type' => KYaml::TYPE_INTEGER
                                ),
                                'First' => array(
                                    'type' => KYaml::TYPE_INTEGER
                                ),
                                'Next' => array(
                                    'type' => KYaml::TYPE_INTEGER
                                )
                            ) // TimeTrackPatterns.?
                        )
                    )
                ), // TimeTrackPatterns
                'Settings' => array(
                    'type' => KYaml::TYPE_LISTHASH,
                    'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NOTREQUIRED,
                    'default' => array(),
                    'children' => array(
                        'EventHandlerThreads' => array(
                            'type' => KYaml::TYPE_INTEGER,
                            'default' => 0,
                            'flags' => KYaml::FLAG_NOTREQUIRED
                        ),
                        'RetryTimeout' => array(
                            'type' => KYaml::TYPE_INTEGER,
                            'default' => 30000,
                            'flags' => KYaml::FLAG_NOTREQUIRED
                        ),
                        'SIP' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'flags' => KYaml::FLAG_EMPTY | KYaml::FLAG_NOTREQUIRED,
                            'children' => array(
                                'Transcoding' => array(
                                    'type' => KYaml::TYPE_BOOLEAN,
                                    'flags' => KYaml::FLAG_NOTREQUIRED
                                )
                            )
                        )
                    )
                ) // Settings
            );
            //@formatter:on

            $app = KSystem::getInstance( );

            if( PHP_OS != 'WINNT' )
            {
                $this->validationProgram = '/usr/sbin/khomp-sudo /etc/init.d/kgateway validate-config';
            }

            $this->addValidator( new KYamlIntegerRangeValidator( 1, 99, 'RadiusTimeout' ) );
            $this->addValidator( new KYamlIntegerRangeValidator( 0, 99, 'RadiusRetries' ) );
            $this->addValidator( new KYamlIntegerRangeValidator( 0, 999, 'RadiusDeadServerDisabledTime' ) );
            $this->addValidator( new KYamlIntegerRangeValidator( 0, 65535, 'RadiusServerPort' ) );
            $this->addValidator( new KYamlIntegerRangeValidator( 1, 99, 'RadiusParallelRequests' ) );
            $this->addValidator( new KYamlIntegerRangeValidator( 1, 50000, 'RadiusQueueSizeDiscard' ) );

            parent::__construct( $app->getConfigDir( ) . DIRECTORY_SEPARATOR . 'gateway.yaml' );
        }

        ///////////////////////////////////////////////////////////////////////
        // Misc methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Get KGatewayProfileYaml instance
         *
         * @return KGatewayProfileYaml
         */
        public function getKGatewayProfileYaml( )
        {
            if( is_null( self::$kgatewayprofileyaml ) )
            {
                require_once KCONFIG . '/classes/KGatewayProfileYaml.php';
                $this->setKGatewayProfileYaml( new KGatewayProfileYaml( ) );
                self::$kgatewayprofileyaml->setKGatewayYaml( $this );
            }
            return self::$kgatewayprofileyaml;
        }

        /**
         * Set the KGatewayProfileYaml instance
         *
         * @param KGatewayProfileYaml $instance
         */
        public function setKGatewayProfileYaml( KGatewayProfileYaml $instance )
        {
            self::$kgatewayprofileyaml = $instance;
        }

        /**
         * Check if array has element
         *
         * @param string $key YAML array key
         * @param string $name Child node name
         * @param string $infokey Key to be used to compare against node $name
         * @return boolean
         */
        protected function has__( $key, $name, $infokey = 'Name' )
        {
            if( !isset( $this->yaml[$key] ) )
            {
                trigger_error( "Invalid key {$key} in my YAML array", E_USER_WARNING );
                return false;
            }

            foreach( $this->yaml[$key] as &$info )
            {
                if( !isset( $info[$infokey] ) )
                {
                    tirgger_error( "Invalid child node in key {$key}, expected '{$infokey}' to be defined", E_USER_WARNING );
                    return false;
                }

                if( $info[$infokey] === $name )
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * Get array element
         *
         * @param string $key YAML array key
         * @param string $name Child node name
         * @param string $infokey Key to be used to compare against node $name
         */
        protected function get__( $key, $name, $infokey = 'Name' )
        {
            if( $this->has__( $key, $name, $infokey ) )
            {
                foreach( $this->yaml[$key] as &$info )
                {
                    if( $info[$infokey] === $name )
                    {
                        return $info;
                    }
                }
            }
            return null;
        }

        ///////////////////////////////////////////////////////////////////////
        // NAPs manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Check if NAP exists
         *
         * @param string $name NAP name
         * @return boolean
         */
        public function hasNap( $name )
        {
            return $this->has__( 'NAPs', $name );
        }

        /**
         * Get nap
         *
         * @param string $name Nap name
         * @return null|array
         */
        public function getNap( $name )
        {
            return $this->get__( 'NAPs', $name );
        }

        /**
         * Get all naps
         *
         * @return array
         */
        public function getAllNaps( )
        {
            return $this->yaml['NAPs'];
        }

        /**
         * Set NAP
         *
         * @param array $data NAP info
         */
        public function setNap( $data, $oldName = '' )
        {
            // duplicated route name
            if( $oldName != $data['Name'] && $this->hasNap( $data['Name'] ) )
            {
                throw new KGatewayYamlException( 'gateway-nap-save-error-duplicated' );
            }
            // add NAP
            if( empty( $oldName ) )
            {
                $this->yaml['NAPs'][] = $data;
            }
            // update NAP
            else
            {
                $notfound = true;
                foreach( $this->yaml['NAPs'] as $idx => &$info )
                {
                    if( $oldName == $info['Name'] )
                    {
                        $info = $data;
                        $notfound = false;
                        break;
                    }
                }
                if( $notfound )
                {
                    throw new KGatewayYamlException( 'gateway-nap-error-not-found' );
                }
            }
        }

        /**
         * Profile was renamed, check if need to update naps profiles
         *
         * @param string $name profile new name
         * @param string $oldName profile old name
         */
        public function renameNapsProfile( $name, $oldName )
        {
            trigger_error( "Renaming NAPs profile {$oldName} to {$name}", E_USER_NOTICE );
            foreach( $this->yaml['NAPs'] as &$nap )
            {
                if( isset( $nap['Profile'] ) && $nap['Profile'] === $oldName )
                {
                    $nap['Profile'] = $name;
                }
            }
        }

        /**
         * Time track pattern was renamed, check if need to update naps
         * reference
         *
         * @param string $name time track pattern new name
         * @param string $oldName pattern old name
         */
        public function renameNapsTimeTrackPattern( $name, $oldName )
        {
            trigger_error( "Renaming NAPs time track pattern {$oldName} to {$name}", E_USER_NOTICE );
            foreach( $this->yaml['NAPs'] as &$nap )
            {
                if( isset( $nap['TimeTrackPattern'] ) && $nap['TimeTrackPattern'] === $oldName )
                {
                    $nap['TimeTrackPattern'] = $name;
                }
            }
        }

        ///////////////////////////////////////////////////////////////////////
        // Routes manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Check if route exists
         *
         * @param string $name Route name
         * @return boolean
         */
        public function hasRoute( $name )
        {
            foreach( $this->yaml['Routes'] as &$info )
            {
                if( $info['Name'] === $name )
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * Count defined routes
         *
         * @return int
         */
        public function countRoutes( )
        {
            return count( $this->yaml['Routes'] );
        }

        /**
         * Get route from YAML
         *
         * @param string $name Route name
         * @return null|array
         */
        public function getRoute( $name )
        {
            foreach( $this->yaml['Routes'] as &$info )
            {
                if( $info['Name'] === $name )
                {
                    return $info;
                }
            }
            return null;
        }

        /**
         * Get all routes
         *
         * @return array
         */
        public function getAllRoutes( )
        {
            return $this->yaml['Routes'];
        }

        /**
         * Set route to YAML
         *
         * If route exists, update it
         *
         * @param array $data Route info
         * @param string $oldName Route old name
         */
        public function setRoute( $data, $oldName = '' )
        {
            // duplicated route name
            if( $oldName != $data['Name'] && $this->hasRoute( $data['Name'] ) )
            {
                throw new KGatewayYamlException( 'gateway-route-error-already-exists' );
            }
            // add new route
            if( empty( $oldName ) )
            {
                $this->yaml['Routes'][] = $data;
            }
            // update route
            else
            {
                $notfound = true;
                foreach( $this->yaml['Routes'] as $idx => &$route )
                {
                    if( $oldName == $route['Name'] )
                    {
                        $route = $data;
                        $notfound = false;
                        break;
                    }
                }
                if( $notfound )
                {
                    throw new KGatewayYamlException( 'gateway-route-error-not-found' );
                }
            }
        }

        /**
         * Delete route
         *
         * @param string $name Route name
         */
        public function deleteRoute( $name )
        {
            foreach( $this->yaml['Routes'] as $idx => &$route )
            {
                if( $name == $route['Name'] )
                {
                    unset( $this->yaml['Routes'][$idx] );
                    return true;
                }
            }
            return false;
        }

        /**
         * Set routes order
         *
         * @param array $order Array of route names with the new order
         * @return boolean
         */
        public function setRoutesOrder( $order )
        {
            if( count( $order ) != count( $this->yaml['Routes'] ) )
            {
                throw new KGatewayYamlException( 'gateway-route-error-reorder-invalid-size' );
            }
            $tmp = array( );
            foreach( $order as $name )
            {
                $route = $this->getRoute( $name );
                if( is_null( $route ) )
                {
                    throw new KGatewayYamlException( 'gateway-route-error-reorder-invalid-name' );
                }
                else
                {
                    $tmp[] = $route;
                }
            }
            $this->yaml['Routes'] = &$tmp;
        }

        /**
         * Profile was renamed, check if need to update routes profiles
         *
         * @param string $name profile new name
         * @param string $oldName profile old name
         */
        public function renameRoutesProfile( $name, $oldName )
        {
            trigger_error( "Renaming ROUTE profile {$oldName} to {$name}", E_USER_NOTICE );
            foreach( $this->yaml['Routes'] as &$route )
            {
                if( isset( $route['DestinationProfile'] ) && $route['DestinationProfile'] === $oldName )
                {
                    $route['DestinationProfile'] = $name;
                }
            }
        }

        /**
         * Check if codec profile is being used
         *
         * @param string $name Codec profile name
         * @return boolean
         */
        public function isCodecProfileInUse( $name )
        {
            foreach( $this->yaml['Routes'] as &$route )
            {
                if( $route['CodecProfile'] === $name )
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * Codec profile was renamed, check if need to update routes
         *
         * @param string $name Codec profile new name
         * @param string $oldName Codec profile old name
         */
        public function renameRoutesCodecProfile( $name, $oldName )
        {
            trigger_error( "Renaming ROUTE codec profile {$oldName} to {$name}", E_USER_NOTICE );
            foreach( $this->yaml['Routes'] as &$route )
            {
                if( $route['CodecProfile'] === $oldName )
                {
                    $route['CodecProfile'] = $name;
                }
            }
        }

        /**
         * Check if device is being used by any NAP
         *
         * @param string|integer $serial Device serial number
         * @return boolean
         */
        public function checkIfDeviceIsBeingUsedByAnyNap( $serial )
        {
            foreach( $this->yaml['NAPs'] as &$nap )
            {
                if( $nap['Type'] === 'Trunk' )
                {
                    foreach( $nap['Links'] as &$l )
                    {
                        // serial will always be the first part
                        $_serial = explode( '.', $l );
                        if( is_array( $_serial ) && count( $_serial ) > 0 && $_serial[0] === $serial )
                        {
                            return true;
                        }
                    }
                }
            }
            return false;
        }

        /**
         * Check if the route has call analyzer enabled by its profile
         *
         * Note: The profile is checked recursively
         *
         * @param string $name Route name
         * @return boolean
         */
        public function routeHasCallAnalyzerEnabled( $name )
        {
            $route = $this->getRoute( $name );

            if( is_null( $route ) )
            {
                throw new KGatewayYamlException( 'gateway-route-error-not-found' );
            }

            // WARNING: false means profile not set, null (returned from
            // getProfile) means profile not found
            $profile = isset( $route['DestinationProfile'] ) && $route['DestinationProfile'] ? $this->getKGatewayProfileYaml( )->getProfile( $route['DestinationProfile'] ) : false;
            if( $profile === false )
            {
                $nap = $this->getNap( $route['DestinationNAP'] );
                if( is_null( $nap ) )
                {
                    throw new KGatewayYamlException( 'gateway-nap-error-not-found' );
                }

                $profile = isset( $nap['Profile'] ) && $nap['Profile'] ? $this->getKGatewayProfileYaml( )->getProfile( $nap['Profile'] ) : false;
            }

            if( $profile === false )
            {
                $profile = $this->getKGatewayProfileYaml( )->getDefaultProfile( );
            }

            if( is_null( $profile ) )
            {
                throw new KGatewayYamlException( 'gateway-route-error-profile-not-found' );
            }

            return isset( $profile['CallAnalyzer'] ) && $profile['CallAnalyzer'] != '';
        }


        ///////////////////////////////////////////////////////////////////////
        // Routes SMS manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Check if route exists
         *
         * @param string $name Route SMS name
         * @return boolean
         */
        public function hasRouteSMS( $name )
        {
            foreach( $this->yaml['RoutesSMS'] as &$info )
            {
                if( $info['Name'] === $name )
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * Count defined routes SMS
         *
         * @return int
         */
        public function countRoutesSMS( )
        {
            return count( $this->yaml['RoutesSMS'] );
        }

        /**
         * Get route SMS from YAML
         *
         * @param string $name Route name
         * @return null|array
         */
        public function getRouteSMS( $name )
        {
            foreach( $this->yaml['RoutesSMS'] as &$info )
            {
                if( $info['Name'] === $name )
                {
                    return $info;
                }
            }
            return null;
        }

         /**
         * Get all routes SMS
         *
         * @return array
         */
        public function getAllRoutesSMS( )
        {
            return $this->yaml['RoutesSMS'];
        }

        /**
         * Set route SMS to YAML
         *
         * If route exists, update it
         *
         * @param array $data Route info
         * @param string $oldName Route old name
         */
        public function setRouteSMS( $data, $oldName = '' )
        {
            // duplicated route name
            if( $oldName != $data['Name'] && $this->hasRouteSMS( $data['Name'] ) )
            {
                throw new KGatewayYamlException( 'gateway-route-error-already-exists' );
            }
            // add new route
            if( empty( $oldName ) )
            {
                $this->yaml['RoutesSMS'][] = $data;
            }
            // update route
            else
            {
                $notfound = true;
                foreach( $this->yaml['RoutesSMS'] as $idx => &$route )
                {
                    if( $oldName == $route['Name'] )
                    {
                        $route = $data;
                        $notfound = false;
                        break;
                    }
                }
                if( $notfound )
                {
                    throw new KGatewayYamlException( 'gateway-route-error-not-found' );
                }
            }
        }

        /**
         * Delete route SMS
         *
         * @param string $name Route name
         */
        public function deleteRouteSMS( $name )
        {
            foreach( $this->yaml['RoutesSMS'] as $idx => &$route )
            {
                if( $name == $route['Name'] )
                {
                    unset( $this->yaml['RoutesSMS'][$idx] );
                    return true;
                }
            }
            return false;
        }

        /**
         * Set routes SMS order
         *
         * @param array $order Array of route names with the new order
         * @return boolean
         */
        public function setRoutesOrderSMS ( $order )
        {
            if( count( $order ) != count( $this->yaml['RoutesSMS'] ) )
            {
                throw new KGatewayYamlException( 'gateway-route-error-reorder-invalid-size' );
            }
            $tmp = array( );
            foreach( $order as $name )
            {
                $route = $this->getRoute( $name );
                if( is_null( $route ) )
                {
                    throw new KGatewayYamlException( 'gateway-route-error-reorder-invalid-name' );
                }
                else
                {
                    $tmp[] = $route;
                }
            }
            $this->yaml['RoutesSMS'] = &$tmp;
        }

        ///////////////////////////////////////////////////////////////////////
        // CDR manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Check if CDR is enabled
         *
         * @return boolean
         */
        public function isCDREnabled( )
        {
            return isset( $this->yaml['CDR']['Enabled'] ) ? $this->yaml['CDR']['Enabled'] : true;
        }

        /**
         * Get CDR data
         *
         * @return array
         */
        public function getCDR( )
        {
            // issue #15667
            if( ! isset( $this->yaml['CDR']['Radius'] ) )
            {
                $this->yaml['CDR']['Radius'] = $this->defaultFromPath( 'CDR.Radius' );
            }
            return $this->yaml['CDR'];
        }

        /**
         * Set CDR data
         *
         * @param array $data CDR data
         */
        public function setCDR( $data )
        {
            // always keep BasePath key value!
            $currentData = $this->getCDR( );
            $data['BasePath'] = $currentData['BasePath'];
            $this->yaml['CDR'] = $data;
        }

        ///////////////////////////////////////////////////////////////////////
        // Audit manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Check if audit is enabled
         *
         * @return boolean
         */
        public function isAuditEnabled( )
        {
            $ts = $this->getAuditEndAsUnixTimestamp( );
            if( $ts !== null && $ts > time( ) )
            {
                return true;
            }
            return false;
        }

        /**
         * Check if route is being audited
         *
         * @param string $name Route name
         * @return boolean
         */
        public function isRouteAudited( $name )
        {
            return isset( $this->yaml['Audit']['Routes'] ) && in_array( $name, $this->yaml['Audit']['Routes'] );
        }

        /**
         * Set audit parameters
         *
         * @param array $data Audit data
         */
        public function enableAudit( $data )
        {
            $data['Start'] = strftime( self::$audittimefmt );
            $this->yaml['Audit'] = $data;
        }

        /**
         * Set audit Start time to empty (stop audit)
         */
        public function disableAudit( )
        {
            $this->yaml['Audit']['Start'] = '';
        }

        /**
         * Get audit parameters
         *
         * @return array
         */
        public function getAudit( )
        {
            return isset( $this->yaml['Audit'] ) ? $this->yaml['Audit'] : null;
        }

        /**
         * Get audit start date as UNIX timestamp
         *
         * @return integer UNIX timestamp or null if an error occured
         */
        public function getAuditStartAsUnixTimestamp( )
        {
            if( empty( $this->yaml['Audit']['Start'] ) )
            {
                return null;
            }
            $t = strptime( $this->yaml['Audit']['Start'], self::$audittimefmt );
            if( $t === false )
            {
                return null;
            }
            $t = mktime( $t['tm_hour'], $t['tm_min'], $t['tm_sec'], $t['tm_mon'] + 1, $t['tm_mday'], $t['tm_year'] + 1900 );
            if( $t === false )
            {
                return null;
            }
            return $t;
        }

        /**
         * Get audit end date as UNIX timestamp
         *
         * @return integer UNIX timestamp or null if an error occured
         */
        public function getAuditEndAsUnixTimestamp( )
        {
            $start = $this->getAuditStartAsUnixTimestamp( );
            if( $start === null )
            {
                return null;
            }
            return $start + $this->yaml['Audit']['End'];
        }

        ///////////////////////////////////////////////////////////////////////
        // Codec profile manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Check if codec profile exists
         *
         * @param string $name Codec profile name
         * @return boolean
         */
        public function hasCodecProfile( $name )
        {
            foreach( $this->yaml['CodecProfiles'] as &$profile )
            {
                if( $profile['Name'] === $name )
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * Count defined codec profiles
         *
         * @return int
         */
        public function countCodecProfiles( )
        {
            return count( $this->yaml['CodecProfiles'] );
        }

        /**
         * Get codec profile
         *
         * @param string $name Codec profile name
         * @return array
         */
        public function getCodecProfile( $name )
        {
            $found = false;
            foreach( $this->yaml['CodecProfiles'] as &$profile )
            {
                if( $name === $profile['Name'] )
                {
                    $found = true;
                    break;
                }
            }

            if( !$found )
            {
                return null;
            }

            $this->checkCodecProfile( $profile );
            return $profile;
        }

        /**
         * Get all codec profiles
         *
         * @return array
         */
        public function getAllCodecProfiles( )
        {
            foreach( $this->yaml['CodecProfiles'] as &$profile )
            {
                $this->checkCodecProfile( $profile );
            }

            return $this->yaml['CodecProfiles'];
        }

        /**
         * Set codec profile
         *
         * @param array $data Codec profile info
         * @param string $oldName Codec profile old name
         */
        public function setCodecProfile( $data, $oldName = '' )
        {
            // duplicated codec profile name
            if( $oldName != $data['Name'] && $this->hasCodecProfile( $data['Name'] ) )
            {
                throw new KGatewayYamlException( 'gateway-codec-profile-save-error-duplicated' );
            }
            // new codec profile
            if( empty( $oldName ) )
            {
                $this->yaml['CodecProfiles'][] = $data;
            }
            // update codec profile
            else
            {
                $notfound = true;
                foreach( $this->yaml['CodecProfiles'] as $idx => &$profile )
                {
                    if( $oldName === $profile['Name'] )
                    {
                        if( $profile['Name'] != $data['Name'] )
                        {
                            $this->renameRoutesCodecProfile( $data['Name'], $oldName );
                        }
                        $profile = $data;
                        $notfound = false;
                        break;
                    }
                }
                if( $notfound )
                {
                    throw new KGatewayYamlException( 'gateway-codec-profile-error-not-found' );
                }
            }
        }

        /**
         * Delete codec profile
         *
         * @param string $name Codec profile name
         */
        public function deleteCodecProfile( $name )
        {
            if( $this->isCodecProfileInUse( $name ) )
            {
                throw new KGatewayYamlException( 'gateway-codec-profile-delete-error-inuse' );
            }

            foreach( $this->yaml['CodecProfiles'] as $idx => $profile )
            {
                if( $profile['Name'] === $name )
                {
                    unset( $this->yaml['CodecProfiles'][$idx] );
                    break;
                }
            }
        }

        /**
         * Check the profile's codec list
         *
         * @param array $profile The profile.
         */
        protected function checkCodecProfile( &$profile )
        {
            // codec G729 will enable G729B, but we don't support it, so, rename
            // G729 and G729B to G729A
            foreach( $profile['CodecList'] as $idx => $codec )
            {
                if( in_array( $codec, self::$g729b ) )
                {
                    trigger_error( "Renamed CODEC from {$codec} to G729A, refrenced by profile {$profile['Name']}", E_USER_NOTICE );
                    $profile['CodecList'][$idx] = 'G729A';
                }
            }
        }

        ///////////////////////////////////////////////////////////////////////
        // Portability manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Get portability configuration
         *
         * @return array
         */
        public function getPortability( )
        {
            if( isset( $this->yaml['Portability'] ) )
            {
                return $this->yaml['Portability'];
            }
            else
            {
                return array( 'Driver' => 'DISABLED', 'DriverOptions' => array() );
            }
        }

        /**
         * Set portability configuration
         *
         * @param string $driver Driver name
         * @param array $options Driver options
         */
        public function setPortability( $driver, $options )
        {
            switch( $driver )
            {
                case 'DISABLED':
                case 'LOCAL-DATABASE':
                case 'WEBSERVICE':
                    break;
                default:
                    throw new KGatewayYamlException( 'gateway-portability-error-unknown-driver' );
            }
            $this->yaml['Portability'] = array( 'Driver' => $driver, 'DriverOptions' => $options );
        }

        ///////////////////////////////////////////////////////////////////////
        // SIM cards manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Check if SIM card exists
         *
         * @param string $iccid SIM card ICCID
         * @return boolean
         */
        public function hasSIMCard( $iccid )
        {
            return $this->has__( 'SIMCards', $iccid, 'ICCID' );
        }

        /**
         * Get all SIM cards
         *
         * @return array
         */
        public function getAllSIMCards( )
        {
            return $this->yaml['SIMCards'];
        }

        /**
         * Set SIM card
         *
         * @param array $data SIM card data
         */
        public function setSIMCard( $data )
        {
            foreach( $this->yaml['SIMCards'] as &$sim )
            {
                if( $sim['ICCID'] === $data['ICCID'] )
                {
                    // update sim card info
                    $sim = $data;
                    return;
                }
            }
            // add sim card info
            $this->yaml['SIMCards'][] = $data;
        }

        /**
         * Delete SIM card
         *
         * @param string $iccid SIM card ICCID
         */
        public function deleteSIMCard( $iccid )
        {
            if( !$this->hasSIMCard( $iccid ) )
            {
                throw new KGatewayYamlException( 'gateway-nap-error-gsm-sim-card-not-found' );
            }
            $nap = $this->getSIMCardNap( $iccid );
            if( !is_null( $nap ) )
            {
                foreach( $nap['SIMs'] as $idx=>$sim )
                {
                    if( $sim === $iccid )
                    {
                        trigger_error( "Removing SIM [{$iccid}] from NAP {$nap['Name']}. SIM is being deleted!", E_USER_NOTICE );
                        unset( $nap['SIMs'][$idx] );
                        $this->setNap( $nap, $nap['Name'] );
                        break;
                    }
                }
            }
            foreach( $this->yaml['SIMCards'] as $idx=>$sim )
            {
                if( $sim['ICCID'] === $iccid )
                {
                    trigger_error( "SIM [{$iccid}] deleted!", E_USER_NOTICE );
                    unset( $this->yaml['SIMCards'][$idx] );
                    break;
                }
            }
        }

        /**
         * Get SIM card's NAP
         *
         * @param string $iccid SIM card ICCID
         * @return array|null
         */
        public function getSIMCardNap( $iccid )
        {
            if( !$this->hasSIMCard( $iccid ) )
            {
                throw new KGatewayYamlException( 'gateway-nap-error-gsm-sim-card-not-found' );
            }
            foreach( $this->getAllNaps() as $nap )
            {
                if( $nap['Type'] == 'GSM' && in_array( $iccid, $nap['SIMs'], true ) )
                {
                    return $nap;
                }
            }
            return null;
        }

        /**
         * Get all configured ICCIDs
         */
        public function getAllSIMCardICCIDs( )
        {
            $iccids = array();
            foreach( $this->yaml['SIMCards'] as &$simcard )
            {
                $iccids[] = $simcard['ICCID'];
            }
            return $iccids;
        }

        ///////////////////////////////////////////////////////////////////////
        // Time track patterns manipulation methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Get all Time Tracking Patterns
         *
         * @return array
         */
        public function getAllTimeTrackPatterns()
        {
            if( isset( $this->yaml['TimeTrackPatterns'] ) )
            {
                return $this->yaml['TimeTrackPatterns'];
            }
            return array();
        }

        /**
         * Get all Time Tracking Patterns names
         *
         * @return array
         */
        public function getAllTimeTrackPatternsNames()
        {
            $r = array();
            foreach( $this->yaml['TimeTrackPatterns'] as &$p )
            {
                $r[] = $p['Name'];
            }
            return $r;
        }

        /**
         * Set time track pattern
         *
         * @param array $data Pattern data
         * @param string $oldName Pattern old name
         */
        public function setTimeTrackPattern( $data, $oldName = '' )
        {
            // duplicated time track pattern
            if( $oldName != $data['Name'] && $this->hasTimeTrackPattern( $data['Name'] ) )
            {
                throw new KGatewayYamlException( 'gateway-time-track-pattern-save-error-duplicated' );
            }
            // new pattern
            if( empty( $oldName ) )
            {
                $this->yaml['TimeTrackPatterns'][] = $data;
            }
            // update pattern
            else
            {
                $notfound = true;
                foreach( $this->yaml['TimeTrackPatterns'] as $idx => &$pattern )
                {
                    if( $oldName === $pattern['Name'] )
                    {
                        if( $pattern['Name'] != $data['Name'] )
                        {
                            $this->renameNapsTimeTrackPattern( $data['Name'], $oldName );
                        }
                        $pattern = $data;
                        $notfound = false;
                        break;
                    }
                }
                if( $notfound )
                {
                    throw new KGatewayYamlException( 'gateway-time-track-pattern-error-not-found' );
                }
            }
        }

        /**
         * Check if time track pattern exists
         *
         * @param string $name Time track pattern name
         * @return boolean
         */
        public function hasTimeTrackPattern( $name )
        {
            foreach( $this->yaml['CodecProfiles'] as &$pattern )
            {
                if( $pattern['Name'] === $name )
                {
                    return true;
                }
            }
            return false;
        }

        ///////////////////////////////////////////////////////////////////////
        // Settings methods
        ///////////////////////////////////////////////////////////////////////

        /**
         * Get Settings
         *
         * @return array
         */
        public function getSettings()
        {
            if( isset( $this->yaml['Settings'] ) )
            {
                return $this->yaml['Settings'];
            }
            return array();
        }

        /**
         * Get Settings SIP
         *
         * @return array
         */
        public function getSettingsSIP()
        {
            $settings = $this->getSettings();
            if( isset( $settings['SIP'] ) )
            {
                return $settings['SIP'];
            }
            return array();
        }

        /**
         * Set Settings SIP transcoding
         *
         * @param bool
         */
        public function setSettingsSIPTranscoding( $transcoding )
        {
            $this->yaml['Settings']['SIP']['Transcoding'] = $transcoding;
        }

        /**
         * Check Settings SIP transcoding
         *
         * @return bool
         */
        public function checkSettingsSIPTranscoding()
        {
            $sip = $this->getSettingsSIP();
            if( isset( $sip['Transcoding'] ) )
            {
                return $sip['Transcoding'];
            }
            return true;
        }

    }
