(function($) {

/// Plugin para auxiliar a seleção de um input numerico
/// Para funcionar automaticamente, definir os attributos min e max no input. Ex: <input type="text" min="-10" max="10">
$.fn.KVolumeSlider = function(options) {

    options = $.extend({
        animate: 'fast',
        max: null,
        min: null,
        orientation: 'vertical'
    }, options || {});

    var $slider = $('.Kslider-element');
    var regExpNum = /^\-?[\d]+$/;
    var forceInt = function(v) {
        return (regExpNum.test(v)) ? parseInt(v, 10) : 0;
    }

    if ($slider.length === 0) {
        $slider = null;
    }

    var onclick = function(event) {

        $slider = $slider || $('<span />').addClass('Kslider-element').appendTo(document.body);

        var _this = this;
        var max = (typeof options.max !== 'number' && regExpNum.test($(this).attr('max'))) ? parseInt($(this).attr('max'), 10) : options.max;
        var min = (typeof options.min !== 'number' && regExpNum.test($(this).attr('min'))) ? parseInt($(this).attr('min'), 10) : options.min;
        var pos = $(this).offset();
        var value = $(this).val();

        pos.top = pos.top - 40;
        pos.left = pos.left + $(this).width() + 20;

        $slider
        .css({
            position : 'absolute',
            '-webkit-box-shadow': '0px 0px 2px 2px rgba(51, 153, 102, 0.75)',
            '-moz-box-shadow': '0px 0px 2px 2px rgba(51, 153, 102, 0.75)',
            'box-shadow': '0px 0px 2px 2px rgba(51, 153, 102, 0.75)'
        })
        .css(pos)
        .show()
        .slider($.extend(options, {
            value: value,
            disabled: false,
            max: max,
            min: min,
            slide: function(event, ui) {
                var n = forceInt(ui.value);
                $(_this).val(n);
            }
        }));
    };

    var onchange = function(event) {
        $slider.slider({
            value: $(this).val()
        });
    };

    $(this).each(function() {
        $(this)
        .addClass('kslider')
        .off('.kslider')
        .on({
            'click.kslider' : onclick,
            'change.kslider': onchange
        });
    });

    /// Para remover o slider quando clicado fora do input e slider.
    $(document.body)
        .off('.kslider')
        .on('click.kslider', function(event) {
        if ($slider && event.target !== $slider.get(0) && !$(event.target).hasClass('kslider')) {
            $slider.hide().slider({disabled:true});
        }
    });
};

})(jQuery);