<?php

$AjaxFunctions['partialDownload'] = 'partialDownload';
$AjaxFunctions['getLogFileDetails'] = 'getLogFileDetails';
set_time_limit(120);

 /// ------------------------------------------------------------------------------------------------------------
/// Ajax

function partialDownload( ) {

    $response = extractPartOfFile($_REQUEST['file'], $_REQUEST['begin'], $_REQUEST['end'],$_REQUEST['offset'], $_REQUEST['max']);
    
    return JSONencode( $response );
}

function getLogFileDetails( ) {

    return JSONencode( logFileDetails( $_REQUEST['file'] ) );

}


 /// ------------------------------------------------------------------------------------------------------------
/// Outras funções

/*
 Compara se o primeira data é menor ou igual a segunda data.
 
 @method compareDateTimeString
 @param {String} $a Formato dd/mm/yyyy H:i:s
 @param {String} $b Formato dd/mm/yyyy H:i:s
 @return {Boolean}
*/
function compareDateTimeString( $a, $b ) {

     /// Year
    $a_aux = (int)substr($a, 6, 4);
    $b_aux = (int)substr($b, 6, 4);
    
    if ( $a_aux > $b_aux ) { return false; }
    if ( $a_aux < $b_aux ) { return true;  }
    
    /// Month
    $a_aux = (int)substr($a, 3, 2);
    $b_aux = (int)substr($b, 3, 2);
    
    if ( $a_aux > $b_aux ) { return false; }
    if ( $a_aux < $b_aux ) { return true;  }
    
    /// Day
    $a_aux = (int)substr($a, 0, 2);
    $b_aux = (int)substr($b, 0, 2);
    
    if ( $a_aux > $b_aux ) { return false; }
    if ( $a_aux < $b_aux ) { return true;  }

    /// Hour
    $a_aux = (int)substr($a, 11, 2);
    $b_aux = (int)substr($b, 11, 2);
    
    if ( $a_aux > $b_aux ) { return false; }
    if ( $a_aux < $b_aux ) { return true;  }

    /// Minutes
    $a_aux = (int)substr($a, 14, 2);
    $b_aux = (int)substr($b, 14, 2);
    
    if ( $a_aux > $b_aux ) { return false; }
    if ( $a_aux < $b_aux ) { return true;  }

    /// Seconds
    $a_aux = (int)substr($a, 17, 2);
    $b_aux = (int)substr($b, 17, 2);
    
    if ( $a_aux > $b_aux ) { return false; }
    if ( $a_aux < $b_aux ) { return true;  }
    
    return true;
    
}

/*
 Esta função sugere uma linha que pode ser iniciada a a leitura do arquivo de log com base na data pesquisada.
 
 AINDA NÃO IMPLEMENTADA
 
 A ideia:
 
 1) Lê uma linha na metade do arquivo. 
 2) Se a data extraída do início da linha, for menor que a data inicial de pesquisa, começa a ler a partir deste ponto.
 3) Se não, volta o cursor do arquivo linhas até achar uma data menor que a da pesquisa.
 
 A intenção é melhorar a performance e não percorrer o arquivo todo a cada consulta.
 
 @method escolhePosicaoArquivoParaContinuar
 @param {String} $date
 @param {Object} $buffer Buffer do arquivo.
 @return {Number} Posição para continuar.
*/
function escolhePosicaoArquivoParaContinuar( $date, &$buffer ) {

    if (empty($begin)) {
        return 0;
    }
}

/*
 Pega a última linha do arquivo do log que seja válida (ISDN ou R2)
 
 @method getLastLineOfFile
 @param {String} $filename Nome do arquivo
 @param {Object} $buffer Buffer do arquivo já aberto
 @return {Array|String} Linha e datetime ou string vazia
*/
function getLastLineOfFile( $filename, &$buffer ) {

    fseek($buffer, 0, SEEK_END);

    $cursor = ftell( $buffer );
    $cursor--;

    for($x_pos = 0; fseek($buffer, $x_pos, SEEK_END) !== -1; $x_pos--) {
        
        $line = fgets($buffer);

        if (strpos($line, '#DATE') !== false) {
            return array(
                "source" => $line,
                "datetime" => substr($line, 6, 11) . ' 00:00:00'
            );
        }

        $match = logLineMatch($filename, $line);
    
        if ($match === true) {
            return array(
                "source" => $line,
                "datetime" => substr($line, 2, 19)
            );
        }
    }

    return '';
}

/*
 Pega a primeira linha do arquivo do log que seja válida (ISDN ou R2)
 
 @method getFirstLineOfFile
 @param {String} $filename Nome do arquivo (isdn.log ou r2.log)
 @param {Object} $buffer Buffer do arquivo
 @return {Array} Linha e datetime ou string vazia
*/
function getFirstLineOfFile( $filename, &$buffer ) {

    while (($line = fgets($buffer)) !== false) {

        if (strpos($line, '#DATE') !== false) {
            return array(
                "source" => $line,
                "datetime" => substr($line, 6, 11) . ' 00:00:00'
            );
        }
    
        $match = logLineMatch($filename, $line);
        
        if ($match === true) {
            return array(
                "source" => $line,
                "datetime" => substr($line, 2, 19)
            );
        }
    }

    return '';

}

/*
 Recebe o nome do arquivo de log e passa informações úteis.
 
 @method logFileDetails
 @param {String} $filename Nome do arquivo.
 @return {Array} file details.
*/
function logFileDetails( $filename ) {

    $filePath = LogUtil::getLogDir( ) . $filename;
    
    if ($fileBuffer = fopen($filePath, 'r')) {
    
        $firstLine = getFirstLineOfFile($filename, $fileBuffer);
        $lastLine = getLastLineOfFile($filename, $fileBuffer);

        fclose($fileBuffer);
    }
    
    return array(
        "firstLine" => $firstLine,
        "lastLine" => $lastLine,
        "filesize" => filesize($filePath)
    );
}

/*
Verifica é uma linha válida para ISDN ou R2

@method logLineMatch
@param {String} $filename Nome do arquivo
@param {Object} $line Referência para a linha do arquivo lida.
@return {Boolean} Se for válida vai retornar um Array com informações
*/
function logLineMatch( $filename, &$line ) {

    $line = trim($line);
    
    /// ISDN
    if ($filename === 'isdn.log') {

        if (strpos($line, 'T|') === false || strpos($line, 'LAPD MSG I') === false) {
            return false;
        }
        
        return true;
        
    /// R2
    } else if ($filename === 'r2.log') {
    
        if (strpos($line, 'T|') === false || (strpos($line, 'R2_SIG', 26) === false && strpos($line, 'SOFTR2', 26) === false)) {
            return false;
        }
        
        return true;
    }
}

/*
Extrai parte de um arquivo. Foi criado para particionar o download do arquivo de log.

@method extractPartOfFile
@param {String} $filename Nome do arquivo de log
@param {String} $begin Data de início formato dd/mm/yyyy H:m:i Ex: 28/02/2014 12:28:10
@param {String} $end Data de fim formato dd/mm/yyyy H:m:i Ex: 29/02/2014 16:00:00
@param {Number} $offset Ponto a partir do qual deve ser iniciada a leitura do arquivo.
@param {Number} $max Máximo de linhas lidas. A função vai retornar se atingir este valor ou final do arquivo.
@return {Array} No padrão com success/content
*/
function extractPartOfFile( $filename, $begin, $end, $offset = 0, $max = 100 ) {

    /// Foi necessária para arquivos grandes (~100MB)
    ini_set('memory_limit', '256M');
    
    /// Caminho completo para o arquivo de log
    $file = LogUtil::getLogDir( ) . $filename;
    
    /// Controle da continuação. É a linha para continuar a leitura.
    $offset = (empty($offset) || (int)$offset < 0) ? 0 : (int)$offset;
    
    $matchStr = 'LAPD MSG I';
    $isdn_data_old_format;

    if ($filename === 'r2.log') {
    
        $matchStr = 'R2_SIG';
        $matchStrOldLog = 'SOFTR2';
    }
    

    /// Máximo de linhas por vez.
    $maxBufferSize = (empty($max)) ? 100 : (int)$max;
    
    $bufferSize = 0;

    $response = array();

    $handle = fopen($file, "r");
    
    fseek($handle, $offset);
    
    if ($handle) {
    
        $response['success'] = true;
        $response['content'] = array(
            "begin" => $begin,
            "end" => $end,
            "offset" => $offset,
            "filesize" => filesize($file),
            "finish" => false,
            "file" => ""
        );
        
        while (($buffer = fgets($handle)) !== false) {

            if (substr($buffer, 0, 2) !== 'T|' || strpos($buffer, $matchStr) === false) {
                
                /// Específico versões antigas R2 e ISDN. issue #12064
                if (strpos($buffer, '#DATE', 0) !== false) {
                    $response['content']['file'] .= $buffer;
                    $isdn_data_old_format = substr($buffer, 6, 11);
                    continue;
                } else if (!empty($matchStrOldLog) && strpos($buffer, $matchStrOldLog) !== false) {
                    /// Linha válida
                } else {
                    continue;
                }
            } else if (strpos($buffer, '/') === 4) {
                /// Começou novo formato. Este trecho é necessário para o caso de log isdn antigo e novo misturado no mesmo arquivo.
                $isdn_data_old_format = null;
            }
            
            if (!empty($isdn_data_old_format) && strpos($buffer, ':') === 4) {
                $datetime = $isdn_data_old_format . ' ' . substr($buffer, 2, 8);
            } else {
                $datetime = substr($buffer, 2, 19);
            }
            
            /// Compara se chegou no início do período pretendido
            if ($datetime && $begin && compareDateTimeString($begin, $datetime) === false) {
            
                continue;
                
            /// Compara se passou do período pretendido
            } else if ($datetime && $end && compareDateTimeString($datetime, $end) === false) {
            
                $response["content"]["finish"] = true;
                break;

            } else {
        
                $response['content']['file'] .= $buffer;
                
                $bufferSize = $bufferSize + strlen($buffer);

                if ($bufferSize >= $maxBufferSize && !$isdn_data_old_format) {
                    break;
                }
            }
        }
        
        if (feof($handle)) {
            $response["content"]["finish"] = true;
        }
        
        
        $response['content']['offset'] = ftell($handle);
        
        fclose($handle);
        
    } else {
    
        $response = errorMessage("Não foi possível ler o arquivo.");
    }
    
    return $response;

}
