(function (Khomp) {

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;

    /**
    Attributos possíveis para os information elements

    @module isdn
    @submodule isdn-decoder
     */

    InformationElements.attributes = {
        /// Additional layer 3 protocol information
        0x00 : {
            0x0c : {
                0x0c : 'Internet Protocol (RFC 791) (ISO/IEC TR 9577 [82])',
                0x0f : 'Point-to-point Protocol (RFC 1548)'
            }
        },

        /// Assignor/assignee
        0x01 : {
            0x00 : 'Message originator is "Default assignee"',
            0x01 : 'Message originator is "Assignor only"'
        },

        /// Call state value
        0x02 : {
            ///    [USER] / [NETWORK]
            0x00 : 'U0 - Null / N0 - Null',
            0x01 : 'U1 - Call initiated / N1 - Call initiated',
            0x02 : 'U2 - Overlap sending / N2 - Overlap sending',
            0x03 : 'U3 - Outgoing call proceeding / N3 - Outgoing call proceeding',
            0x04 : 'U4 - Call delivered / N4 - Call delivered',
            0x06 : 'U6 - Call present / N6 - Call present',
            0x07 : 'U7 - Call received / N7 - Call received',
            0x08 : 'U8 - Connect request / N8 - Connect request',
            0x09 : 'U9 - Incoming call proceeding / N9 - Incoming call proceeding',
            0x0a : 'U10 - Active / N10 - Active',
            0x0b : 'U11 - Disconnect request / N11 - Disconnect request',
            0x0c : 'U12 - Disconnect indication / N12 - Disconnect indication',
            0x0f : 'U15 - Suspend request / N15 - Suspend request',
            0x11 : 'U17 - Resume request / N17 - Resume request',
            0x13 : 'U19 - Release request / N19 - Release request',
            0x16 : '------ / N22 - Call abort',
            0x19 : 'U25 - Overlap receiving / N25 - Overlap receiving'

        },

        /// Causes
        0x03 : {
            0x01 : 'Unassigned (unallocated) number',
            0x02 : 'No route to specified transit network',
            0x03 : 'No route to destination',
            0x06 : 'Channel unacceptable',
            0x07 : 'Call awarded and being delivered in an established channel',
            0x10 : 'Normal call clearing',
            0x11 : 'User busy',
            0x12 : 'No user responding',
            0x13 : 'User alerting, no answer',
            0x15 : 'Call rejected',
            0x16 : 'Number changed',
            0x1a : 'Non-selected user clearing',
            0x1b : 'Destination out of order',
            0x1c : 'Invalid number format (incomplete number)',
            0x3d : 'Facility rejected',
            0x1e : 'Response to STATUS ENQUIRY',
            0x1f : 'Normal, unspecified',
            0x22 : 'No circuit/channel available',
            0x26 : 'Network out of order',
            0x29 : 'Temporary failure',
            0x2a : 'Switching equipment congestion',
            0x2b : 'Access information discarded',
            0x2c : 'Requested circuit/channel not available',
            0x2f : 'Resource unavailable, unspecified',
            0x39 : 'Bearer capability not authorized',
            0x3a : 'Bearer capability not presently available',
            0x3f : 'Service or option not available, unspecified',
            0x41 : 'Bearer capability not implemented',
            0x42 : 'Channel type not implemented',
            0x45 : 'Requested facility not implemented',
            0x46 : 'Only restricted digital information bearer capability is available',
            0x4f : 'Service or option not implemented, unspecified',
            0x51 : 'Invalid call reference value',
            0x52 : 'Identified channel does not exist',
            0x53 : 'A suspended call exists, but this call identity does not',
            0x54 : 'Call identity in use',
            0x55 : 'No call suspended',
            0x56 : 'Call having the requested call identity has been cleared',
            0x58 : 'Incompatible destination',
            0x5b : 'Invalid transit network selection',
            0x5f : 'Invalid message, unspecified',
            0x60 : 'Mandatory information element is missing',
            0x61 : 'Message type non-existent or not implemented',
            0x62 : 'Message not compatible with call state or message type non-existent or not implemented',
            0x63 : 'Information element non-existent or not implemented',
            0x64 : 'Invalid information element contents',
            0x65 : 'Message not compatible with call state',
            0x66 : 'Recovery on time expire',
            0x6f : 'Protocol error, unspecified',
            0x7f : 'Interworking, unspecified'
        },

        /// Channel type/map element type
        0x04 : {
            0x03 : 'B-channel units (time slot number)',
            0x06 : 'H0-channel units',
            0x08 : 'H11-channel units',
            0x09 : 'H12-channel units'
        },

        /// Class
        0x05 : {
            0x00 : 'Indicated channels', // (Note 1)
            0x06 : 'Single interface', // (Note 2)
            0x07 : 'All interfaces' // (Note 3)

            /// Table 4-22/Q.931
            /// NOTE 1 – The channel identification information element must be included and indicates which channels are to be restarted.
            /// NOTE 2 – If non-associated signalling is used, the channel identification information element must be included to indicate the interface to be restarted if it is other than the one on which the D-channel is present.
            /// NOTE 3 – May be used when there are two or more interfaces controlled by the D-channel. The channel identification information element must not be included with this coding
        },

        /// Coding standard
        0x06 : {
            0x00 : 'ITU-T standardized coding', //00
            0x01 : 'ISO/IEC Standard', // NOTE 1 // 01
            0x02 : 'National standard', // NOTE 1 // 10
            0x03 : 'Standard defined for the network (either public or private) present on the network side of the interface' // NOTE 1 // 11

            /// TODO ver se esta opção é a mesma: '11' : 'Standard specific to identified location (Note 1)'

            /// NOTE 1 – These other coding standards should only be used only when the desired (bearer capability / call states /high layer compatibility / low layer compatibility / progress indication) cannot be represented by ITU-T-standardized coding
        },

        /// D-channel indicator
        0x07 : {
            0x00 : 'False', //'The channel identified is not the D-channel',
            0x01 : 'True' //'The channel identified is the D-channel'
        },

        /// Default packet size
        0x08 : {
            0x04 : 'Default packet size 16 octets',
            0x05 : 'Default packet size 32 octets',
            0x06 : 'Default packet size 64 octets',
            0x07 : 'Default packet size 128 octets',
            0x08 : 'Default packet size 256 octets',
            0x09 : 'Default packet size 512 octets',
            0x0a : 'Default packet size 1024 octets',
            0x0b : 'Default packet size 2048 octets',
            0x0c : 'Default packet size 4096 octets'
        },

        /// Extended audiovisual characteristics identification
        0x09 : {
            0x01 : 'Capability set of initial channel of H.221',
            0x02 : 'Capability set of subsequent channel of H.221',
            0x21 : 'Capability set of initial channel associated with an active 3.1 kHz audio or speech call.'
        },

        /// Extended high layer characteristics identification
        0x0a : {
            0x01 : 'Telephony',
            0x04 : 'Facsimile Group 2/3 (Recommendation F.182)',
            0x21 : 'Facsimile Group 4 Class I (Recommendation F.184)',
            0x24 : 'Facsimile service Group 4, Classes II and III (Recommendation F.184)',
            0x28 : '(Note 11)',
            0x31 : '(Note 11)',
            0x32 : 'Syntax based Videotex (Recommendations F.300 and T.102)',
            0x33 : 'International Videotex interworking via gateways or interworking units (Recommendations F.300 and T.101)',
            0x35 : 'Telex service (Recommendation F.60)',
            0x38 : 'Message Handling Systems (MHS) (X.400-series Recommendations [77])',
            0x41 : 'OSI application (Note 6) (X.200-series Recommendations [78])',
            0x42 : 'FTAM application (ISO 8571)',
            0x5e : 'Not available for assignment',
            0x5f : 'Not available for assignment',
            0x60 : 'Videotelephony (Recommendations F.720 [91] and F.721 [79] and F.731, profile 1a)',
            0x61 : 'Videoconferencing Recommendations F.702 [94] and F.731 [97], profile 1b',
            0x62 : 'Audiographic conferencing Recommendations F.702 [94] and F.731 [97] (including at least profile 2a2 and optionally 2a1, 2a3, 2b1, 2b2 and 2bc)',

            /// ---------------
            0x63 : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x64 : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x65 : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x66 : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x67 : 'Reserved for audiovisual services (F.700-series Recommendations [80])',

            /// ---------------
            0x68 : 'Multimedia services (F.700-series Recommendations [80])',

            /// ---------------
            0x69 : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x6a : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x6b : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x6c : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x6d : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x6e : 'Reserved for audiovisual services (F.700-series Recommendations [80])',
            0x6f : 'Reserved for audiovisual services (F.700-series Recommendations [80])',

            /// ---------------
            0x7f : 'Reserved'
        },

        /// Flow control on Rx
        0x0b : {
            0x00 : 'Cannot accept data with flow control mechanism',
            0x01 : 'Can accept data with flow control mechanism'
        },

        /// Flow control on Tx
        0x0c : {
            0x00 : 'Not required to send data with flow control mechanism',
            0x01 : 'Required to send data with flow control mechanism'
        },

        /// High layer characteristics identification
        0x0d : {
            0x01 : 'Telephony',
            0x04 : 'Facsimile Group 2/3 (Recommendation F.182 [68])',
            0x21 : 'Facsimile Group 4 Class I (Recommendation F.184 [69])',
            0x24 : 'Facsimile service Group 4, Classes II ad III (Recommendation F.184)',
            0x28 : 'Only to those services for which ITU-T (Note 7)',
            0x31 : 'Only to those services for which ITU-T (Note 7)',
            0x32 : 'Syntax based Videotex (Recommendation F.300 [73] and T.102 [74])',
            0x33 : 'International Videotex interworking via gateways or interworking units (Recommendation F.300 and T.101 [75])',
            0x35 : 'Telex service (Recommendation F.60 [76])',
            0x38 : 'Message Handling Systems (MHS) (X.400-series Recommendation [77])',
            0x41 : 'OSI application (Note 6) (X.200-series Recommendations [78])',
            0x42 : 'FTAM application (ISO 8571)',
            0x5e : 'Reserved for maintenance (Note 8)',
            0x5f : 'Reserved for management (Note 8)',
            0x60 : 'Videotelephony (Recommendations F.720 [91] and F.721 [79]) and F.731 profile 1a)(Note 9)',
            0x61 : 'Videoconferencing Recommendation F.702 [94] and F.731 [97] Profile 1b (Note 9)',
            0x62 : 'Audiographic conferencing Recommendations F.702 [94] and F.731 [97] (including at least profile 2a2 and optionally 2a1, 2a3, 2b1, 2b2, and 2bc) (Notes 9 and 10)',
            ///-----
            0x63 : 'Reserved for audiovisual service (F.700-series Recommendations [80])',
            0x64 : 'Reserved for audiovisual service (F.700-series Recommendations [80])',
            0x65 : 'Reserved for audiovisual service (F.700-series Recommendations [80])',
            0x66 : 'Reserved for audiovisual service (F.700-series Recommendations [80])',
            0x67 : 'Reserved for audiovisual service (F.700-series Recommendations [80])',
            ///-----
            0x68 : 'Multimedia services F.700-series Recommendations [80] (Note 9)',
            ///-----
            0x7f : 'Reserved'
        },

        /// In-band/out-band negotiation
        0x0e : {
            0x00 : 'Negotiation is done with USER INFORMATION messages on a temporary signalling connection',
            0x01 : 'Negotiation is done in-band using logical link zero'
        },

        /// Information channel selection
        0x0f : {
            /// Basic interface
            0x00 : {
                0x00 : 'No channel',
                0x01 : 'B1 channel',
                0x02 : 'B2 channel',
                0x03 : 'Any channel'
            },
            /// Other interfaces
            0x01 : {
                0x00 : 'No channel',
                0x01 : 'As indicated in following octets',
                0x02 : 'Reserved',
                0x03 : 'Any channel'
            }
        },

        /// Information transfer capability
        0x10 : {
            0x00 : 'Speech',
            0x08 : 'Unrestricted digital information',
            0x09 : 'Restricted digital information',
            0x10 : '3.1 kHz audio',
            0x11 : 'Unrestricted digital information with tones/announcements', // NOTE 2
            0x18 : 'Video'

            /// NOTE 2 – Unrestricted digital information with tones/announcements (UDI-TA) is the new informationtransfer attribute value that had previously been named "7 kHz audio" in Recommendation Q.931 (1988).
        },

        /// Information transfer rate
        0x11 : {
            /// A escolha do Circuit ou Packet é feita com base no valor do 'Transfer mode'
            /// Circuit
            0x00 : {
                0x00 : '–',
                0x10 : '64 kbit/s',
                0x11 : '2x64 kbit/s',
                0x13 : '384 kbit/s',
                0x15 : '1536 kbit/s',
                0x17 : '1920 kbit/s',
                0x18 : 'Multirate (64 kbit/s base rate)'
            },
            /// Packet
            0x02 : {
                0x00 : 'This code shall be used for all packet calls',
                0x10 : '–',
                0x11 : '–',
                0x13 : '–',
                0x15 : '–',
                0x17 : '–',
                0x18 : 'Multirate (64 kbit/s base rate)'
            }
        },

        /// Interface identifier present
        0x12 : {
            0x00 : 'False - implicitly identified ', //'(Note 1)'
            0x01 : 'True - explicitly identified in one or more octets, beginning with octet 3.1'
        },

        /// Interface type
        0x13 : {
            0x00 : 'Basic interface',
            0x01 : 'Primary rate interface'
        },

        /// Intermediate rate
        0x14 : {
            0x00 : 'Not used',
            0x01 : '8 kbit/s',
            0x02 : '16 kbit/s',
            0x03 : '32 kbit/s'
        },

        /// Interpretation
        /// Atualmente esta informação possui apenas uma opção de valor que pode ser expandida futuramente pela norma
        0x15 : {
            0x04 : 'First (primary or only) high layer characteristics identification (in octet 4) to be used in the call'
        },

        /// Location
        0x16 : {
            0x00 : 'User',
            0x01 : 'Private network serving the local user',
            0x02 : 'Public network serving the local user',
            0x03 : 'Transit network (Note 2)',
            0x04 : 'Public network serving the remote user',
            0x05 : 'Private network serving the remote user',
            0x0a : 'Network beyond the interworking point'
        },

        /// Logical link identifier negotiation
        0x17 : {
            0x00 : 'Default, LLI = 256 only',
            0x01 : 'Full protocol negotiation' /// NOTE 18 – A connection over which protocol negotiation will be executed is indicated in bit 2 of octet 5b
        },

        /// Mode duplex
        0x18 : {
            0x00 : 'Half duplex',
            0x01 : 'Full duplex'
        },

        /// Mode of operation
        0x19 : {
            0x00 : 'Bit transparent mode of operation',
            0x01 : 'Protocol sensitive mode of operation'
        },

        /// Mode of operation layer 2
        0x1a : {
            0x01 : 'Normal mode of operation',
            0x02 : 'Extended mode of operation'
        },

        /// Mode of operation layer 3
        0x1b : {
            0x01 : 'Normal packet sequence numbering',
            0x02 : 'Extended packet sequence numbering'
        },

        /// Modem type
        0x1c : {
            0x00 : 'National use',
            0x01 : 'National use',
            0x02 : 'National use',
            0x03 : 'National use',
            0x04 : 'National use',
            0x05 : 'National use',
            0x11 : 'Recommendation V.21 [55]',
            0x12 : 'Recommendation V.22 [56]',
            0x13 : 'Recommendation V.22 bis[57]',
            0x14 : 'Recommendation V.23 [58]',
            0x15 : 'Recommendation V.26 [59]',
            0x16 : 'Recommendation V.26 bis[60]',
            0x17 : 'Recommendation V.26 ter[61]',
            0x18 : 'Recommendation V.27 [62]',
            0x19 : 'Recommendation V.27 bis[63]',
            0x1a : 'Recommendation V.27 ter [64]',
            0x1b : 'Recommendation V.29 [65]',
            0x1d : 'Recommendation V.32 [66]',
            0x1e : 'Recommendation V.34 [90]',
            ///-------------------
            0x20 : 'National use',
            0x21 : 'National use',
            0x22 : 'National use',
            0x23 : 'National use',
            0x24 : 'National use',
            0x25 : 'National use',
            0x26 : 'National use',
            0x27 : 'National use',
            0x28 : 'National use',
            0x29 : 'National use',
            0x2a : 'National use',
            0x2b : 'National use',
            0x2c : 'National use',
            0x2d : 'National use',
            0x2e : 'National use',
            0x2f : 'National use',

            ///-------------------
            0x30 : 'User specified',
            0x31 : 'User specified',
            0x32 : 'User specified',
            0x33 : 'User specified',
            0x34 : 'User specified',
            0x35 : 'User specified',
            0x36 : 'User specified',
            0x37 : 'User specified',
            0x38 : 'User specified',
            0x39 : 'User specified',
            0x3a : 'User specified',
            0x3b : 'User specified',
            0x3c : 'User specified',
            0x3d : 'User specified',
            0x3e : 'User specified',
            0x3f : 'User specified'
        },

        /// Multiframe
        0x1d : {
            0x00 : 'Multiple frame establishment not supported. Only UI frames allowed',
            0x01 : 'Multiple frame establishment supported'
        },

        /// Negotiation
        0x1e : {
            0x00 : 'In-band negotiation not possible',
            0x01 : 'In-band negotiation possible'
        },

        /// Negotiation indicator
        0x1f : {
            0x00 : 'Out-band negotiation not possible',
            0x01 : 'Out-band negotiation possible'

            /// NOTE: See Annex J for description of low layer compatibility negotiation.
        },

        /// Network identification plan
        0x20 : {

            0x00 : 'Unknown',
            0x01 : 'Carrier Identification Code', // (Note 2)
            0x03 : 'Data network identification code (Recommendation X.121 [21])'

            /// Table 4-18/Q.931 NOTE 2 – Carrier Identification Codes may be an appropriate method of identifying the network serving the remote user.
        },

        /// NIC on Rx
        0x21 : {

            0x00 : 'Cannot accept data with network independent clock',
            0x01 : 'Can accept data with network independent clock'
        },

        /// NIC on Tx
        0x22 : {

            0x00 : 'Not required to send data with network independent clock',
            0x01 : 'Required to send data with network independent clock'
        },

        /// Notification description
        0x23 : {

            0x00 : 'User suspended',
            0x01 : 'User resumed',
            0x02 : 'Bearer service change',

            /// Os abaixo foram extraídos de:
            /// http://www.etsi.org/deliver/etsi_i_ets/300100_300199/30014101/01_60/ets_30014101e01p.pdf
            /// No wireshark são apresentadas os mesmos valores, que não constam na ITU Q.931
            0x79 : 'Remote hold',
            0x7a : 'Remote retrieval'
        },

        /// Number/map
        0x24 : {
            0x00 : 'Channel is indicated by the number',
            0x01 : 'Channel is indicated by the slot map'
        },

        /// Number of data bits excluding parity Bit if present
        0x25 : {
            0x00 : 'Not used',
            0x01 : '5 bits',
            0x02 : '7 bits',
            0x03 : '8 bits'
        },

        /// Number of stop bits
        0x26 : {
            0x00 : 'Not used',
            0x01 : '1 bit',
            0x02 : '1.5 bits',
            0x03 : '2 bits'
        },

        /// Numbering plan
        0x27 : {
            0x00 : 'Unknown',
            0x01 : 'ISDN/telephony numbering plan',
            0x03 : 'Data numbering plan',
            0x04 : 'Telex numbering plan',
            0x08 : 'National standard numbering plan',
            0x09 : 'Private numbering plan',
            0x0f : 'Reserved for extension'
        },

        /// Odd/even indicator
        0x28 : {
            0x00 : 'Even number of address signals',
            0x01 : 'Odd number of address signals'
        },

        /// Parity information
        0x29 : {
            0x00 : 'Odd',
            0x01 : 'Even',
            0x02 : 'None',
            0x03 : 'Forced to 0',
            0x04 : 'Forced to 1'
        },

        /// Preferred/Exclusive
        0x2a : {

            0x00 : 'Indicated channel is preferred',
            0x01 : 'Exclusive; only the indicated channel is acceptable'
        },

        /// Presentation indicator
        0x2b : {

            0x00 : 'Presentation allowed',
            0x01 : 'Presentation restricted',
            0x02 : 'Number not available due to interworking',
            0x03 : 'Reserved'
        },

        /// Presentation method of protocol profile
        0x2c : {
            0x01 : 'High layer protocol profile'
        },

        /// Progress description
        0x2d : {

            0x01 : '1. Call is not end-to-end ISDN; further call progress information may be available in-band',
            0x02 : '2. Destination address is non-ISDN',
            0x03 : '3. Origination address is non-ISDN',
            0x04 : '4. Call has returned to the ISDN',
            0x05 : '5. Interworking has occurred and has resulted in a telecommunication service change (Note 5)',
            0x08 : '8. In-band information or an appropriate pattern is now available'
        },

        /// Protocol discriminator
        0x2e : {

            0x00 : 'User-specific protocol', // (Note 1)
            0x01 : 'OSI high layer protocols',
            0x02 : 'Recommendation X.244 [44]', // (Note 2)
            0x03 : 'Reserved for system management convergence function',
            0x04 : 'IA5 characters', // (Note 4)
            0x05 : 'X.208 and X.209 coded user information', // (Note 5)
            0x07 : 'Recommendation V.120 [9] rate adaption',
            0x08 : 'Q.931/I.451 user-network call control messages',
            /// -------
            0x10 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x11 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x12 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x13 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x14 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x15 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x16 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x17 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x18 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x19 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x1a : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x1b : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x1c : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x1d : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x1e : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x1f : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x20 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x21 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x22 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x23 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x24 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x25 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x26 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x27 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x28 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x29 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x2a : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x2b : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x2c : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x2d : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x2e : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x2f : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x30 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x31 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x32 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x33 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x34 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x35 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x36 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x37 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x38 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x39 : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x3a : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x3b : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x3c : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x3d : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x3e : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            0x3f : 'Reserved for other network layer or layer 3 protocols, including Recommendation',
            /// -------
            0x40 : 'National use',
            0x41 : 'National use',
            0x42 : 'National use',
            0x43 : 'National use',
            0x44 : 'National use',
            0x45 : 'National use',
            0x46 : 'National use',
            0x47 : 'National use',
            0x48 : 'National use',
            0x49 : 'National use',
            0x4a : 'National use',
            0x4b : 'National use',
            0x4c : 'National use',
            0x4d : 'National use',
            0x4e : 'National use',
            0x4f : 'National use',

            /// -------
            0x50 : 'Reserved for other network layer or layer 3 protocols, including Recommendation through X.25', // (Note 3)
            // through
            /// TODO: ver como fazer aqui nesta faixa. É muito grande.
            0xfe : 'Reserved for other network layer or layer 3 protocols, including Recommendation through X.25' // (Note 3)

            /// Table 4-26/Q.931
            /// NOTE 1 – The user information is structured according to user needs.
            /// NOTE 2 – The user information is structured according to Recommendation X.244 which specifies the structure of X.25 call user data.
            /// NOTE 3 – These values are reserved to discriminate these protocol discriminators from the first octet of an X.25 packet including general format identifier.
            /// NOTE 4 – The user information consists of IA5 characters.
            /// NOTE 5 – The number of X.208 and X.209 components contained in a User-user information element as well as their semantics and use are user-application dependent and may be subject to other Recommendations.
        },

        /// Q.933 use
        0x2f : {
            0x00 : 'For use when the coding defined in Recommendation Q.933 is not use'
        },

        /// Rate adaption header
        0x30 : {
            0x00 : 'Header not included',
            0x01 : 'Header included'
        },

        /// Screening indicator
        0x31 : {
            0x00 : 'User-provided, not screened',
            0x01 : 'User-provided, verified and passed',
            0x02 : 'User-provided, verified and failed',
            0x03 : 'Network provided'
        },

        /// Signal value
        0x32 : {

            0x00 : 'Dial tone on',
            0x01 : 'Ring back tone on',
            0x02 : 'Intercept tone on',
            0x03 : 'Network congestion tone on',
            0x04 : 'Busy tone on',
            0x05 : 'Confirm tone on',
            0x06 : 'Answer tone on',
            0x07 : 'Call waiting tone',
            0x08 : 'Off-hook warning tone',
            0x09 : 'Pre-emption tone on',
            0x3f : 'Tones off',
            0x40 : 'Alerting on – pattern 0', //  (Note 1)
            0x41 : 'Alerting on – pattern 1', // (Note 1)
            0x42 : 'Alerting on – pattern 2', // (Note 2)
            0x43 : 'Alerting on – pattern 3', // (Note 1)
            0x44 : 'Alerting on – pattern 4', // (Note 1)
            0x45 : 'Alerting on – pattern 5', // (Note 1)
            0x46 : 'Alerting on – pattern 6', // (Note 1)
            0x47 : 'Alerting on – pattern 7', // (Note 1)
            0x4f : 'Alerting off'

            /// Table 4-24/Q.931
            /// NOTE 1 – The use of these patterns is network dependent.
            /// NOTE 2 – Used for special/priority alerting.
        },

        /// Synchronous/Asynchronous
        0x33 : {
            0x00 : 'Synchronous data',
            0x01 : 'Asynchronous data'
        },

        /// Transfer mode
        0x34 : {

            0x00 : 'Circuit',
            0x02 : 'Packet'

        },

        /// Type of network identification
        0x35 : {

            0x00 : 'User specified',
            0x02 : 'National network identification', // (Note 1)
            0x03 : 'International network identification'

            /// NOTE 1 – In the case that "type of network identification" is coded as 010, "national network identification ", " national identification plan " is coded according to national specification.
        },

        /// Type of number
        0x36 : {
            0x00 : 'Unknown',
            0x01 : 'International number',
            0x02 : 'National number',
            0x03 : 'Network specific number',
            0x04 : 'Subscriber number',
            0x06 : 'Abbreviated number',
            0x07 : 'Reserved for extension'
        },

        /// Type of subaddress
        0x37 : {
            0x00 : 'NSAP',
            0x02 : 'User specified'
        },

        /// User information layer 1 protocol
        0x38 : {
            0x01 : 'ITU-T standardized rate adaption V.110, I.460 and X.30. This implies the presence of octet 5a and optionally octets 5b, 5c and 5d as defined below',
            0x02 : 'Recommendation G.711 [10] µ-law',
            0x03 : 'Recommendation G.711 A-law',
            0x04 : 'Recommendation G.721 [11] 32 kbit/s ADPCM and Recommendation I.460',
            0x05 : 'Recommendations H.221 and H.242',
            0x06 : 'Recommendations H.223 [92] and H.245 [93]',
            0x07 : 'Non-ITU-T standardized rate adaption. This implies the presence of octet 5a and, optionally, octets 5b, 5c and 5d. The use of this codepoint indicates that the user rate specified in octet 5a is defined by the user. Additionally, octets 5b, 5c and 5d, if present, are defined in accordance with the user specified rate adaption',
            0x08 : 'ITU-T standardized rate adaption V.120 [9]. This implies the presence of octets 5a and 5b as defined below, and optionally octets 5c and 5d',
            0x09 : 'ITU-T standardized rate adaption X.31 [14] HDLC flag stuffing'
        },

        /// User information layer 2 protocol
        0x39 : {
            0x01 : 'Basic mode ISO 1745 [36]',
            0x02 : 'Recommendation Q.921/I.441 [3]',
            0x06 : 'Recommendation X.25 [5], link layer (Notes 20, 23)',
            0x07 : 'Recommendation X.25 Multilink',
            0x08 : 'Extended LAPB; for half duplex operation (Recommendation T.71 [37])',
            0x09 : 'HDLC ARM (ISO/IEC 4335 [38])',
            0x0a : 'HDLC NRM (ISO/IEC 4335)',
            0x0b : 'HDLC ABM (ISO/IEC 4335)',
            0x0c : 'LAN logical link control (ISO/IEC 8802-2 [39])',
            0x0d : 'Recommendation X.75 [40]. Single Link Procedure (SLP)',
            0x0e : 'Recommendation Q.922',
            0x0f : 'Core aspects of Recommendation Q.922',
            0x10 : 'User specified',
            0x11 : 'ISO/IEC 7776 DTE-DCE operation'
        },

        /// User information layer 3 protocol
        0x3a : {

            0x02 : 'Recommendation Q.931/I.451',
            0x06 : 'Recommendation X.25, packet layer',
            0x07 : 'ISO/IEC 8208 [41] (X.25 packet level protocol for data terminal equipment)',
            0x08 : 'ITU-T Rec. X.223 and ISO/IEC 8878 [81] (use of ISO/IEC 8208 [41] and Recommendation X.25 to provide the OSI-CONS)',
            0x09 : 'ISO/IEC 8473 [43] (OSI connectionless mode protocol)',
            0x0a : 'Recommendation T.70[32]minimum network layer ',
            0x0b : 'ISO/IEC TR 9577[82](Protocol identification in the network layer)',
            0x10 : 'User specified'
        },

        /// User rate
        0x3b : {
            0x00 : 'For I.460, rate is specified by bits 7, 6 of octet 5b, intermediate rate. For V.110 and X.30, rate is indicated by E-bits (synchronous data only) or may be negotiated in-band. For V.120, rate is unspecified or may be negotiated in-band.',
            0x01 : '0.6 kbit/s Recommendation X.1 [17]',
            0x02 : '1.2 kbit/s',
            0x03 : '2.4 kbit/s Recommendation X.1',
            0x04 : '3.6 kbit/s',
            0x05 : '4.8 kbit/s Recommendation X.1',
            0x06 : '7.2 kbit/s',
            0x07 : '8 kbit/s Recommendation I.460',
            0x08 : '9.6 kbit/s Recommendation X.1',
            0x09 : '14.4 kbit/s',
            0x0a : '16 kbit/s Recommendation I.460',
            0x0b : '19.2 kbit/s',
            0x0c : '32 kbit/s Recommendation I.460',
            0x0d : '38.4 kbit/s Recommendation V.110 [87]',
            0x0e : '48 kbit/s Recommendations X.1',
            0x0f : '56 kbit/s',
            0x12 : '57.6 kbit/s Recommendation V.14 extended [88]',
            0x13 : '28.8 kbit/s Recommendation V.110 [89]',
            0x14 : '24 kbit/s Recommendation V.110 [89]',
            0x15 : '0.1345 kbit/s Recommendation X.1',
            0x16 : '0.100 kbit/s Recommendation X.1',
            0x17 : '0.075/1.2 kbit/s Recommendation X.1 (Note 9)',
            0x18 : '1.2/0.075 kbit/s Recommendation X.1 (Note 9)',
            0x19 : '0.050 kbit/s Recommendation X.1',
            0x1a : '0.075 kbit/s Recommendation X.1',
            0x1b : '0.110 kbit/s Recommendation X.1',
            0x1c : '0.150 kbit/s Recommendation X.1',
            0x1d : '0.200 kbit/s Recommendation X.1',
            0x1e : '0.300 kbit/s Recommendation X.1',
            0x1f : '12 kbit/s'
        }

    };

})(Khomp);
