(function (Khomp) {

    /// Bearer capability
    /// A partir da página 58 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;

    InformationElements.elementsDecoders[0x04] = function (octets) {

        /// Objeto com a informação decodificada que será retornado.
        var objValue = {};

        ///----------------------------------------
        /// Octet 3
        var octet3 = octets.shift();

        /// Coding standard: bits 7,6 octet 3
        var codingStandard = (octet3 & 0x60) >> 5;
        objValue[InformationElements.attributeNames[0x06]] = attributes[0x06][codingStandard] || codingStandard;

        /// Information transfer capability: bits 5,4,3,2,1 octet 3
        var informationTransferCapability = (octet3 & 0x1f);
        objValue['Information transfer capability'] = attributes[0x10][informationTransferCapability] || informationTransferCapability;

        ///----------------------------------------
        /// Octet 4
        var octet4 = octets.shift();

        /// Transfer mode: bits 7,6 octet 4
        var transferMode = (octet4 & 0x60) >> 5;
        objValue['Transfer mode'] = attributes[0x34][transferMode] || transferMode;

        /// Information transfer rate: análise dos bits 5,4,3,2,1 octet 4
        var informationTransferRate = (octet4 & 0x1f);
        objValue['Information transfer rate'] = attributes[0x11][transferMode][informationTransferRate] || informationTransferRate;

        ///----------------------------------------
        /// Octet 4.1*
        
        /// Rate multiplier
        /// NOTE 1 - Estará presente somente se o Information transfer rate tiver como valor 'Multirate'
        if (informationTransferRate === 0x18) {

            var octet41 = octets.shift();

            objValue['Rate multiplier'] = octet41;
        }
        
        /// Daqui em diante os octetos são opcionais. Então se não tem mais nada...
        if (octets.length === 0) {return objValue;}

        ///----------------------------------------
        /// Octet 5*
        var octet5 = octets.shift();

        /// Layer identifier: bits 7,6 octet 5
        objValue['Layer 1 identifier'] = (octet5 & 0x60) >> 5;

        /// User information layer 1 protocol: bits 5,4,3,2,1 octet 5
        var userInfoLayer1Protocol = (octet5 & 0x1f);
        
        objValue['User information layer 1 protocol'] = attributes[0x38][userInfoLayer1Protocol] || userInfoLayer1Protocol;

        ///----------------------------------------
        /// Octet 5a*
        /// NOTE 2 – This octet may be present if octet 3 indicates unrestricted digital information and octet 5 indicates either of the ITU-T standardized rate adaptions V.110, I.460 and X.30 or V.120 [9]. It may also be present if octet 3 indicates 3.1 kHz audio and octet 5 indicates G.711.
        var conditionNote2 = (
            (informationTransferCapability === 0x08 && (userInfoLayer1Protocol === 0x01 || userInfoLayer1Protocol === 0x08)) ||
            (informationTransferCapability === 0x10 && (userInfoLayer1Protocol === 0x02 || userInfoLayer1Protocol === 0x03))) ? true : false;

        var extensionIndicatorOct5 = (((octet5 & 0x80) >> 7) === 0) ? true : false;

        if (!extensionIndicatorOct5) {
            return objValue;
        }

        ///----------------------------------------
        var octet5a = octets.shift();

        /// Synchronous/Asynchronous: bit 7 octet 5a
        /// NOTE 7 – Octets 5b-5d may be omitted in the case of synchronous user rates.
        var syncData = (octet5a & 0x40) >> 6;
        objValue['Synchronous/Asynchronous'] = attributes[0x33][syncData] || syncData;

        /// Negotiation: bit 6 octet 5a
        var negotiation = (octet5a & 0x20) >> 5;
        objValue.Negotiation = attributes[0x1e][negotiation] || negotiation;

        /// User rate: bits 5,4,3,2,1 octet 5a
        var userRate = (octet5a & 0x1f);
        objValue['User rate'] = attributes[0x3b][userRate] || userRate;

        ///----------------------------------------
        /// Octet 5b*
        /// NOTE 3 – This structure of octet 5b only applies if octet 5 indicates ITU-T standardized rate adaption (see Recommendations V.110 [7], I.460 [15] and X.30 [8])
        if (octets.length && userInfoLayer1Protocol === 0x01) {

            var octet5b = octets.shift();

            /// Intermediate rate: bits 7, 6 do octeto 5b
            var intermediateRate = (octet5b & 0x60) >> 5;
            objValue['Intermediate rate'] = attributes[0x14][intermediateRate] || intermediateRate;

            /// NIC on Tx: bit 5 do octet 5b
            /// NOTE 10 – Refers to transmission in the forward direction of the call.
            /// NOTE 11 – See Recommendations V.110 [7], I.460 [15] and X.30 [8].
            var nicOnTx = (octet5b & 0x10) >> 4;
            objValue['NIC on Tx'] = attributes[0x22][nicOnTx] || nicOnTx;

            /// NIC on Rx: bit 4 do octet 5b
            /// NOTE 12 – Refers to transmission in the backward direction of the call.
            /// NOTE 13 – See Recommendations V.110 [7], I.460 [15] and X.30 [8].
            var nicOnRx = (octet5b & 0x08) >> 3;
            objValue['NIC on Rx'] = attributes[0x21][nicOnRx] || nicOnRx;

            /// Flow control on Tx: bit 3 do octet 5b
            /// NOTE 14 – Refers to transmission in the forward direction of the call.
            /// NOTE 15 – See Recommendations V.110, I.460 and X.30.
            var flowControlOnTx = (octet5b & 0x04) >> 2;
            objValue['Flow control on Tx'] = attributes[0x0c][flowControlOnTx] || flowControlOnTx;

            /// Flow control on Rx: bit 2 do octet 5b
            /// NOTE 16 – Refers to transmission in the backward direction of the call.
            /// NOTE 17 – See Recommendations V.110, I.460 and X.30
            var flowControlOnRx = (octet5b & 0x02) >> 1;
            objValue['Flow control on Rx'] = attributes[0x0b][flowControlOnRx] || flowControlOnRx;

            /// NOTE 4 – This structure of octet 5b only applies if octet 5 indicates ITU-T standardized rate adaption (see Recommendation V.120 [9]).
            if (!InformationElements.isLastOctet(octet5b) && userInfoLayer1Protocol === 0x08) {

                ///----------------------------------------
                /// Octet 5ba
                var octet5ba = octets.shift();

                /// Rate adaption header: bit 7 do octet 5b*
                var rateAdaptionHeader = (octet5ba & 0x40) >> 6;
                objValue['Rate adaption header'] = attributes[0x30][rateAdaptionHeader] || rateAdaptionHeader;

                /// Multiframe: bit 6 do octet 5b*
                var multiframe = (octet5ba & 0x20) >> 5;
                objValue.Multiframe = attributes[0x1d][multiframe] || multiframe;

                /// Mode of operation: bit 5 do octet 5b*
                var modeOfOperation = (octet5ba & 0x10) >> 4;
                objValue['Mode of operation'] = attributes[0x19][modeOfOperation] || modeOfOperation;

                /// Logical link identifier negotiation: bit 4 octet 5b*
                var logicalLinkIdentifierNegotiation = (octet5ba & 0x08) >> 3;
                objValue['Logical link identifier negotiation'] = attributes[0x17][logicalLinkIdentifierNegotiation] || logicalLinkIdentifierNegotiation;

                /// Assignor/assignee: bit 3 do octet 5b*
                var assignorAssignee = (octet5ba & 0x04) >> 2;
                objValue[InformationElements.attributeNames[0x01]] = attributes[0x01][assignorAssignee] || assignorAssignee;

                /// In-band/out-band negotiation: bit 2 do octet 5b*
                var inBandOutBandNegotiation = (octet5ba & 0x02) >> 1;
                objValue['In-band/out-band negotiation'] = attributes[0x0e][inBandOutBandNegotiation] || inBandOutBandNegotiation;

                ///----------------------------------------
                /// Octet 5c*
                /// NOTE 2
                if (octets.length && !InformationElements.isLastOctet(octet5ba) && conditionNote2) {

                    var octet5c = octets.shift();

                    /// Number of stop bits: bits 7,6 do octet 5c*
                    var numberOfStopBits = (octet5c & 0x60) >> 5;
                    objValue['Number of stop bits'] = attributes[0x26][numberOfStopBits] || numberOfStopBits;

                    /// Number of data bits excluding parity Bit if present: bits 5,4 do octet 5c*
                    var numberOfDataBitsExcludingParityBitIfPresent = (octet5c & 0x18) >> 3;
                    objValue['Number of data bits excluding parity Bit if present'] = attributes[0x25][numberOfDataBitsExcludingParityBitIfPresent] || numberOfDataBitsExcludingParityBitIfPresent;

                    /// Parity information: bits 3,2,1 do octet 5c*
                    var parityInformation = (octet5c & 0x07);
                    objValue['Parity information'] = attributes[0x29][parityInformation] || parityInformation;

                    ///----------------------------------------
                    /// Octet 5d*
                    /// NOTE 2
                    if (octets.length && !InformationElements.isLastOctet(octet5c) && conditionNote2) {

                        var octet5d = octets.shift();

                        /// Mode duplex: bit 7 do octet 5d*
                        var modeDuplex = (octet5d & 0x40) >> 6;
                        objValue['Mode duplex'] = attributes[0x18][modeDuplex] || modeDuplex;

                        /// Modem type: bits 6,5,4,3,2,1 octet 5d*
                        var modemType = (octet5d & 0x3f);
                        objValue['Modem type'] = attributes[0x1c][modemType] || modemType;

                    }
                }
            }

            ///----------------------------------------
            /// Octet 6*
            if (octets.length) {

                var octet6 = octets.shift();

                var layer2Ident = (octet6 & 0x60) >> 5;
                objValue['Layer 2 identifier'] = layer2Ident;

                /// User information layer 2 protocol: bits 5,4,3,2,1 octet 6
                var userInfoLayer2Protocol = (octet6 & 0x1f);
                objValue['User information layer 2 protocol'] = attributes[0x39][userInfoLayer2Protocol] || userInfoLayer2Protocol;

            }

            ///----------------------------------------
            /// Octet 7*
            if (octets.length) {
                var octet7 = octets.shift();

                var layer3Ident = (octet7 & 0x60) >> 5;
                objValue['Layer 3 identifier'] = layer3Ident;

                /// User information layer 3 protocol: bits 5,4,3,2,1 octeto 7
                var userInfoLayer3Protocol = (octet7 & 0x1f);
                objValue['User information layer 3 protocol'] = attributes[0x3a][userInfoLayer3Protocol] || userInfoLayer3Protocol;

                ///----------------------------------------
                /// Octet 7a* e 7b*
                /// NOTE 5: This octet may be included if octet 7 indicates ISO/IEC TR 9577 (Protocol Identification in the network layer).
                var note5Condition = (userInfoLayer3Protocol === 0x0b) ? true : false;

                if (octets.length && note5Condition) {

                    var octet7a = octets.shift();
                    var octet7b = octets.shift();

                    /// Additional layer 3 protocol information: bits 4,3,2,1 octetos 7a* e 7b*
                    var bits7a = (octet7a & 0xf);
                    var bits7b = (octet7b & 0xf);

                    objValue[InformationElements.attributeNames[0x00]] = attributes[0x00][bits7a][bits7b] || (bits7a + ' ' + bits7b);

                }

            }
        }

        return objValue;
    };

})(Khomp);
