(function (Khomp) {

    /// Channel identification (18)
    /// A partir da página 72 da ITU Q931.

    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;
    var Util = Khomp.Util;

    InformationElements.elementsDecoders[0x18] = function (values) {

        var objValue = {};
        var octet3;
        var octet31;
        var octet32;
        var octet33;

        /// Octeto 3
        octet3 = values.shift();

        /// Interface identifier present: Bit 7 octet 3
        var interfaceIdentifierPresent = (octet3 & 0x40) >> 6;
        objValue['Interface identifier present'] = attributes[0x12][interfaceIdentifierPresent] || interfaceIdentifierPresent;

        /// Interface type: Bit 6 octeto 3
        var interfaceType = (octet3 & 0x20) >> 5;
        objValue['Interface type'] = attributes[0x13][interfaceType] || interfaceType;

        var spare = (octet3 & 0x10) >> 4;

        /// Preferred/Exclusive: Bit 4 octeto 3
        var preferredExclusive = (octet3 & 0x08) >> 3;
        objValue['Preferred/Exclusive'] = attributes[0x2a][preferredExclusive] || preferredExclusive;

        /// D-channel indicator: Bit 3 octeto 3
        var dchannel = (octet3 & 0x04) >> 2;
        objValue['D-channel indicator'] = attributes[0x07][dchannel] || dchannel;

        /// Information channel selection: Bit 2,1 octeto 3
        var channel_selection = (octet3 & 0x03);
        var subtype = (interfaceType === 0) ? 0 : 1;
        objValue['Information channel selection'] = attributes[0x0f][subtype][channel_selection] || channel_selection;

        /// Octeto 3.1*
        /// NOTE 1 – When the "interface identifier present" field in octet 3 indicates "interface implicitly identified" octet 3.1 is omitted. When octet 3.1 is present, it may be extended by using the extension bit (bit 8)

        if (interfaceIdentifierPresent !== 0) {

            octet31 = values.shift();

            var interfaceIdentifier = (octet31 & 0x7f);
            objValue['Interface identifier'] = interfaceIdentifier;
        }

        /// Octeto 3.2*
        /// NOTE 2 – When the "interface type" field in octet 3 indicates "basic interface", octets 3.2 and 3.3 are functionally replaced by the "information channel selection" field in octet 3, and thus omitted
        if (interfaceType !== 0) {

            octet32 = values.shift();

            /// Coding standard: bits 7,6 octeto 3.2
            var codingStandard = (octet32 & 0x60) >> 5;
            objValue[InformationElements.attributeNames[0x06]] = attributes[0x06][codingStandard] || codingStandard;

            /// Number/map: bit 5 octeto 3.2
            var numberMap = (octet32 & 0x10) >> 4;
            objValue['Number/map'] = attributes[0x24][numberMap] || numberMap;

            /// Channel type/map element type: bits 4,3,2,1 do octeto 3.2
            var channelTypeOrMapElementType = (octet32 & 0x0f);
            objValue['Channel type/map element type'] = attributes[0x04][channelTypeOrMapElementType] || channelTypeOrMapElementType;

            if (values.length) {

                /// Octeto 3.3*

                if (numberMap === 0x00) {
                    /// -----------------------------------------
                    /// Quando o canal é um número
                    while (values.length) {

                        octet33 = values.shift();

                        /// NOTE 3 – When channel number is used and a single channel is indicated, bit 8 shall be set to "1". When channel number is used and multiple channels are indicated, bit 8 shall be used as an extension bit to indicate an extension to subsequent channels and coded according to the rules specified in 4.5.1
                        /// NOTE 4 – When channel number is used, this octet may be repeated to indicate multiple channels
                        var channelNumber = (octet33 & 0x7f);
                        objValue['Channel number'] = (objValue['Channel number']) ? objValue['Channel number'] + ', ' + channelNumber : channelNumber;

                        if (InformationElements.isLastOctet(octet33)) {
                            break;
                        }
                    }
                } else {

                    /// -----------------------------------------
                    /// Quando o canal é um slot map

                    /// NOTE 13 – The length of the slot map (in bits) is defined by the capacity of the interface type (e.g. 1534 kbit/s or 2048 kbit/s for a primary rate interface) divided by the capacity of the channel type/mapelement type (e.g. 64 kbit/s for a B-channel). The length of the slot map is the smallest number of complete octets that contain the length in bits
                    var slotMap;
                    while (values.length) {

                        octet33 = values.shift();

                        // if (channelTypeOrMapElementType === '0011') {
                        /// B-channel

                        slotMap = Util.getBitsString(octet33, [8, 7, 6, 5, 4, 3, 2, 1]);

                        objValue['Slot map'] = (objValue['Slot map']) ? objValue['Slot map'] + ', ' + slotMap : slotMap;

                        // } else if (channelTypeOrMapElementType === '0110') {
                        /// H0 channel
                        /// TODO: ver como tratar o e(5), d(4), c(3), b(2), a(1)
                        // } else {
                        /// TODO: ver como tratar o H11(1), H12(1), etc
                        /// H1x channel
                        // }
                    }
                }
            }

            if (values.length) {
                objValue['Não decodificado'] = values.join(' ');
            }

            return objValue;
        }
    };
})(Khomp);
