(function (Khomp) {

    /// High layer compatibility - hlc (pg 78)
    /// Obs: aqui o primeiro octeto dos values já é o octeto 3 indicado na ITU.

    var Util = Khomp.Util;
    var ISDN = Khomp.ISDN;
    var InformationElements = ISDN.InformationElements;
    var attributes = InformationElements.attributes;

    InformationElements.elementsDecoders[0x7d] = function (values) {

        var objValue = {};
        var octet3 = values.shift();

        /// Coding standard (octeto 3 bits 7, 6)
        var codingStandard = (octet3 & 0x60) >> 5;

        /// Interpretation (octeto 3 bits 5, 4, 3)
        var interpretation = (octet3 & 0x1c) >> 2;

        /// Presentation method of protocol profile (octeto 3 bits 2,1)
        var presentationMethod = (octet3 & 0x03);

        objValue[InformationElements.attributeNames[0x06]] = attributes[0x06][codingStandard] || codingStandard;

        objValue.Interpretation = attributes[0x15][interpretation] || interpretation;

        objValue['Presentation method of protocol profile'] = attributes[0x2c][presentationMethod] || presentationMethod;

        var octet4 = values.shift();

        /// High layer characteristics identification (octeto 4 bits 7,6,5,4,3,2,1)
        var highLayerCharacteristicsIdentification = (octet4 & 0x7f);

        objValue['High layer characteristics identification'] = attributes[0x0d][highLayerCharacteristicsIdentification] || highLayerCharacteristicsIdentification;

        if (!InformationElements.isLastOctet(octet4)) {

            var octet4a = values.shift();

            /// octet4a bits 7,6,5,4,3,2,1
            var extendedCharacteristicsIdentification = (octet4a & 0x7f);

            if (highLayerCharacteristicsIdentification === 0x5e || highLayerCharacteristicsIdentification === 0x5f) {

                /// Extended high layer characteristics identification: bits 7,6,5,4,3,2,1 do octeto 4a
                objValue['Extended high layer characteristics identification'] = attributes[0x0a][extendedCharacteristicsIdentification] || extendedCharacteristicsIdentification;

            } else if (highLayerCharacteristicsIdentification === 0x60 || highLayerCharacteristicsIdentification === 0x61 || highLayerCharacteristicsIdentification === 0x68) {

                /// Extended audiovisual characteristics identification: bits 7,6,5,4,3,2,1 octeto 4a
                objValue['Extended audiovisual characteristics identification'] = attributes[0x09][extendedCharacteristicsIdentification] || extendedCharacteristicsIdentification;

            } else {
                console.log('Valor não esperado');
            }
        }

        if (values.length) {
            objValue['Não decodificado'] = values.join(' ');
        }

        return objValue;

    };

})(Khomp);
