var KLogAnalyser;

(function (Khomp, $) {

    var $dialog;
    var $paginationShowMore;
    var $sortTable;
    var maxPerPage = 10;
    var scrollAnim = 'linear';
    var scrollTime = 300;
    var dialogMinWidth = 700;
    var overflow = [];
    var filename = '';
    var __ = function () {
        return Khomp.translate.apply(null, arguments);
    };

    /**
    Para interface de análise de logs

    @module WEB
     */
    KLogAnalyser = {
        tituloDialogo : {
            'isdn.log' : __("Análise ISDN"),
            'r2.log' : __("Análise R2")
        }
    };

    /**
    Busca no servidor detalhes do arquivo de log no que será analisado

    @method KLogAnalyser.getLogFileDetails
    @param {String} file Nome do arquivo sem caminho
    @param {Function} callback
     */
    KLogAnalyser.getLogFileDetails = function (file, callback) {

        jQuery.ajax({
            beforeSend : this.loadingContent(),
            complete : this.finishedContent(),
            url : GLOBALAjaxURL,
            data : {
                'ajaxTo' : 'getLogFileDetails',
                'file' : file
            },
            dataType : "json",
            success : function (response) {
                callback(response);
            }
        });

    };

    /// Interface

    /**
    Abre o diálogo de análise do arquivo de log (ISDN ou R2).

    @method KLogAnalyser.openLogAnalyser
    @param {String} file Nome do arquivo sem caminho
     */
    KLogAnalyser.openLogAnalyser = function (file) {

        filename = file;
        var _this = this;

        this.getLogFileDetails(file, function (fileDetailsResponse) {

            jQuery.ajax({
                beforeSend : _this.loadingContent(),
                complete : _this.finishedContent(),
                url : GLOBALAjaxURL,
                data : {
                    'ajaxTo' : 'getLogFile',
                    'file' : file
                },
                dataType : "json",
                error : function (ret) {
                    alert(__('Erro na requisição.'));
                },
                success : function (ret) {

                    if (ret.success) {

                        /// Retira a barra de rolagem no modal
                        bodyOverflow = [$('body').css('overflow-x'), $('body').css('overflow-y')];
                        $('body').css({
                            'overflow-x' : 'hidden',
                            'overflow-y' : 'hidden'
                        });

                        $dialog = $('#divLogAnalyser');

                        if ($dialog.is(":ui-dialog")) {
                            $dialog.dialog('destroy');
                        }

                        $dialog
                        .html(ret.content.header)
                        .dialog({
                            title : _this.tituloDialogo[file] || file,
                            modal : true,
                            resizable : true,
                            draggable : true,
                            hide : "fast",
                            show : "fast",
                            minWidth : dialogMinWidth,
                            position : {
                                my : "center",
                                at : "center",
                                of : window
                            },
                            width : 800,
                            // height : $(window).height() - 40,
                            beforeClose : function () {
                                /// Volta a barra de rolagem quando fechar
                                $('body').css({
                                    'overflow-x' : bodyOverflow[0],
                                    'overflow-y' : bodyOverflow[1]
                                });
                            }
                        });

                        if (!fileDetailsResponse.firstLine || !fileDetailsResponse.lastLine) {
                            $dialog.find('.kwebloganalyser:first').html('<p style="text-align:center; margin-top:50px;">' + __('O conteúdo do arquivo não possui registros de chamadas.') + '</p>');
                            return;
                        }

                        $dialog.find('.i18n').translate();
                        $dialog.find('.help').tooltip();

                        /// Voltar para o topo
                        $dialog.find('.log-list-scrolltop')
                        .css('cursor', 'pointer')
                        .bind('click', function (event) {
                            event.preventDefault();
                            window.scrollTo(0, 0);
                            $dialog.animate({
                                scrollTop : 0
                            }, scrollTime);
                            return false;
                        });

                        _this.startCalendar();

                        /// Mostra a opção de carregar um arquivo local. Está 'escondido' pois deve ser utilizado somente pela Khomp.
                        $dialog.find('.log-load-form').bind('dblclick', function () {
                            $dialog.find('.log-local-input').toggle();
                        });

                        /// Nome e tamanho do arquivo
                        $dialog.find('.log-filename').html(filename);
                        $dialog.find('.log-filesize').html(_this.fileSizeHuman(fileDetailsResponse.filesize));

                        _this.fillBeginEndSelects(fileDetailsResponse.firstLine.datetime, fileDetailsResponse.lastLine.datetime);
                        _this.prepareDragFile();

                        /// Sort lista
                        $sortTable = $('<div />')
                            .hide()
                            .addClass('show-only-if-calls')
                            /// Adiciona os checkboxes e labels
                            .append('<span>' + __('Ordenação') + ': </span><input id="logTableSortAsc" type="radio" name="sort" value="0"/> <label for="logTableSortAsc">' + __('Asc.') + '</label> <input id="logTableSortDesc" type="radio" name="sort" value="1" checked="checked"/> <label for="logTableSortDesc">' + __('Desc.') + '</label>')
                            .appendTo($dialog.find('.log-table-options'))
                            /// Eventos
                            .find('input[type="radio"]').bind('click', function (event) {

                                _this.renderFilterTable($(this).val());
                            });

                        /// Form submit
                        $dialog.find('form[name="analyseLog"]').bind('submit', function (event) {
                            event.preventDefault();
                            _this.analyseLog();
                            return false;
                        });

                        $dialog.find('form[name="log_filter"]').bind('submit', function (event) {
                            event.preventDefault();
                            _this.renderFilterTable();
                            return false;
                        });

                    } else {

                        ShowNotice(ret.message, ret.success);
                    }
                }
            });
        });
    };

    /**
    Inicializa componentes de data

    @method KLogAnalyser.startCalendar
     */
    KLogAnalyser.startCalendar = function () {

        var dateOptions = {
            dateFormat : 'dd/mm/yy',
            dayNames : ['Domingo', 'Segunda', 'Terça', 'Quarta', 'Quinta', 'Sexta', 'Sábado', 'Domingo'],
            dayNamesMin : ['D', 'S', 'T', 'Q', 'Q', 'S', 'S', 'D'],
            dayNamesShort : ['Dom', 'Seg', 'Ter', 'Qua', 'Qui', 'Sex', 'Sáb', 'Dom'],
            monthNames : ['Janeiro', 'Fevereiro', 'Março', 'Abril', 'Maio', 'Junho', 'Julho', 'Agosto', 'Setembro', 'Outubro', 'Novembro', 'Dezembro'],
            monthNamesShort : ['Jan', 'Fev', 'Mar', 'Abr', 'Mai', 'Jun', 'Jul', 'Ago', 'Set', 'Out', 'Nov', 'Dez'],
            nextText : 'Próximo',
            prevText : 'Anterior'
        };

        $dialog.find("#beginDate")
        // .datepicker('destroy')
        .datepicker(dateOptions)
        // .datepicker("option", "minDate", $('#beginDate').val())
        // .datepicker("option", "maxDate", $("#endDate").val())
        .datepicker('option', 'onClose', function (selectedDate) {
            $dialog.find("#endDate").datepicker("option", "minDate", selectedDate);
        });

        $dialog.find("#endDate")
        // .datepicker('destroy')
        .datepicker(dateOptions)
        // .datepicker("option", "minDate", $('#beginDate').val())
        // .datepicker("option", "maxDate", $("#endDate").val())
        .datepicker('option', 'onClose', function (selectedDate) {
            $dialog.find("#beginDate").datepicker("option", "maxDate", selectedDate);
        });
    };

    /**
    Abre ou fecha uma chamada da lista

    @method KLogAnalyser.openCloseWraper
    @param {HTMLElement} element Elemento clicado
     */
    KLogAnalyser.openCloseWraper = function (element) {

        var $w = $(element).parents('.log-message:first').find('.wrapers:first');
        var $text = $(element).find('.arrow:first');

        var visible = $w.is(':visible');

        if (visible) {
            $w.hide();
            $text.text('[+]');
            $w.find('.isdn-information-element').off('mouseenter.highlight_information_element');
        } else {
            $w.show('fast');
            $text.text('[-]');
            
            /// Destaca o information element no fonte da mensagem de uma chamada
            $w.find('.isdn-information-element').on('mouseenter.highlight_information_element', function() {
                var call_pos = parseInt($(this).parents('.call').attr('data-position'), 10);
                var msg_pos = parseInt($(this).parents('.log-message').attr('data-position'), 10);
                var info_pos = parseInt($(this).attr('data-position'), 10);
                
                var msg = Khomp.ISDN.logObj.calls[call_pos].Messages[msg_pos];
                
                if (msg) {
                    $(this).parents('.log-message').find('.log-source-line .msg-source').html(KLogAnalyser.makeSourceHtml(msg.source, msg.info_elements_pos[info_pos]));
                }
            });
        }

        return false;
    };

    /**
    Abre ou fecha as informações de uma chamada.
    O html é gerado somente uma vez.

    @method KLogAnalyser.openCloseWrapers
    @param {String} id Identificador da chamada
    @param {String} type Tipo pode ser 'isdn' ou 'r2'
     */
    KLogAnalyser.openCloseWrapers = function (id, type) {

        type = type || 'isdn';

        $dialog.find('#wrapers' + id).toggle();

        /// Gera o html somente se vazio
        if ($dialog.find('#wrapers' + id).html() === '') {

            if (type === 'isdn') {

                KLogAnalyser.createTreeMessagesISDN(id);

            } else if (type === 'r2') {

                KLogAnalyser.createTreeMessagesR2(id);
            }

            $('.log-message')
            .off('mouseenter.log-message mouseleave.log-message')
            .on({
                'mouseenter.log-message' : function () {
                    $(this).find('.log-source-line').css('visibility', 'visible');
                },
                'mouseleave.log-message' : function () {
                    $(this).find('.log-source-line').css('visibility', 'hidden');
                }
            });
        }
        return false;
    };

    /**
    Abre ou fecha um item da árvore

    @method KLogAnalyser.openClose
    @param {Mixed} element Elemento clicado.
     */
    KLogAnalyser.openClose = function (element) {

        var $w = $(element).parent().find('.wrapers:first');
        var $text = $(element).find('.arrow:first');

        var text = $text.text();
        var visible = $w.is(':visible');

        if (visible) {
            $w.hide();
            $text.text('[+]');
        } else {
            $w.show('slow');
            $text.text('[-]');
        }

        return false;
    };

    /**
    Mostra ou não o filtro

    @method KLogAnalyser.showHideFilter
     */
    KLogAnalyser.showHideFilter = function () {

        $dialog.find('.log-filter-form,.showFilter,.hideFilter').toggle();
    };

    /**
    Alterna a interface entre carregando e parado.

    @method KLogAnalyser.statusCarregando
    @param {Boolean} carregando True para carregando e False para parado
     */
    KLogAnalyser.statusCarregando = function (carregando) {

        $dialog.find('#cancelLoadLogFile').toggle(carregando);
        $dialog.find('#loadLogFileData').toggle(!carregando);
        $dialog.find('#loadLogFileData,#beginDate,#beginHour,#endDate,#endHour')
        .toggleClass('disabled', carregando)
        .prop('disabled', carregando);

        this.centralizeDialog();
    };

    /**
    Altera a porcentagem do status de carregamento do arquivo do servidor.

    @method KLogAnalyser.changePercentageDiv
    @param {Number} partial Parte concluída
    @param {Number} total Total
     */
    KLogAnalyser.changePercentageDiv = function (partial, total) {

        var txtPercent = (partial && total && partial <= total) ? Math.round(partial / total * 100) + '%' : '';

        $dialog.find('#loadLogFileDataPercentage').html(txtPercent);
    };

    /**
    Preenche o espaço do resumo do arquivo.

    @method KLogAnalyser.fillLogSummary
    @param {Number} totalErrors
    @param {Number} totalAlerts
    @param {Number} totalSuccess
    @param {Number} totalCalls
     */
    KLogAnalyser.fillLogSummary = function (totalErrors, totalAlerts, totalSuccess, totalCalls) {

        var $logSummary = $dialog.find('.log-summary');

        $logSummary.show();

        $logSummary.find('.log-call-error-total').html(totalErrors);
        $logSummary.find('.log-call-alert-total').html(totalAlerts);
        $logSummary.find('.log-call-success-total').html(totalSuccess);
        $logSummary.find('.log-call-total').html(totalCalls);

    };

    /**
    Preenche os campos de data e hora do diálogo inicial do ISDN ou R2

    @method KLogAnalyser.fillBeginEndSelects

    @param {String} begin '17/10/2014 08:36:00'
    @param {String} end '17/10/2014 15:05:00'
     */
    KLogAnalyser.fillBeginEndSelects = function (begin, end) {

        if (!begin && !end) {
            return;
        }

        var b = (begin) ? begin.split(' ') : '';
        var e = (end) ? end.split(' ') : '';

        $dialog.find('#beginDate')
        .datepicker("option", "minDate", b[0])
        .datepicker("option", "maxDate", e[0])
        .datepicker("setDate", b[0]);

        $dialog.find('#endDate')
        .datepicker("option", "minDate", b[0])
        .datepicker("option", "maxDate", e[0])
        .datepicker("setDate", e[0]);

        $dialog.find('.log-first-message-time').html(begin);
        $dialog.find('.log-last-message-time').html(end);

    };

    /**
    Gera o html dos checkboxs dos links no filtro

    @method KLogAnalyser.printLogFilterLinksCheckbox
    @param {Object} links
     */
    KLogAnalyser.printLogFilterLinksCheckbox = function (links) {

        var html = [];
        var lnks = [];
        var linkId;

        for (linkId in links) {
            lnks.push(linkId);
        }

        lnks.sort();

        for (var i = 0, lnks_len = lnks.length; i < lnks_len; i++) {

            linkId = lnks[i];

            html[i] = '<input type="checkbox" disabled="disabled" checked="checked" id="link_' + linkId + '" class="log-filter-link" value="' + linkId + '"/><label for="link_' + linkId + '">' + (__('Link %s (%s chamadas)').replace('%s', '<span class="bold">' + linkId + '</span>').replace('%s', links[linkId])) + '</label>';
        }

        $dialog.find('.log-filter-links').html('<li>' + html.join('</li><li>') + '</li>');
    };

    /**
    Recebe o conteúdo do arquivo e inicia o processamento dos dados de acordo com o tipo

    @method KLogAnalyser.analyseLog
    @param {Array|String} logFileContent Conteúdo do arquivo
     */
    KLogAnalyser.analyseLog = function (logFileContent) {

        switch (filename) {

        case 'r2.log':
            this.analyseR2(logFileContent);
            break;

        case 'isdn.log':
            this.analyseISDN(logFileContent);
            break;
        }
    };

    /**
    Renderiza a lista de chamadas de acordo com o filtro (ISDN).

    @method KLogAnalyser.renderFilterTable
    @param {Number} sort Crescente (0) ou Decrescente (1)
    @param {Number} offset
     */
    KLogAnalyser.renderFilterTable = function (sort, offset) {

        sort = parseInt(((sort !== null && typeof sort !== 'undefined') ? sort : $dialog.find('.log-table-options input[name="sort"]:checked').val()), 10);

        switch (filename) {

        case 'r2.log':
            this.renderTable(this.filterCallsR2(), this.createCallLineR2, sort, offset);
            break;

        case 'isdn.log':
            this.renderTable(this.filterCallsISDN(), this.createCallLineISDN, sort, offset);
            break;
        }
    };

    /**
    Centraliza o diálogo e ocupa a tela toda

    @method KLogAnalyser.centralizeDialog
     */
    KLogAnalyser.centralizeDialog = function () {

        window.scrollTo(0, 0);

        $dialog
        .dialog('option', 'height', $(window).height() - 40)
        .dialog('option', 'width', $(window).width() - 60)
        .dialog("option", "position", {
            my : "center",
            at : '20px',
            of : window
        });
    };

    /**
    Checkbox desabilita ou não o select de tipos de mensagens do filtro.

    @method KLogAnalyser.enableDisableMessageFilterTypes
     */
    KLogAnalyser.enableDisableMessageFilterTypes = function () {

        $dialog.find('#messageTypes').prop('disabled', !$dialog.find('#chkMessageTypes').is(':checked'));

    };

    /**
    Checkbox desabilita ou não a seleção de canais ou links
    @method KLogAnalyser.enableDisableMessageChannel
    @param {Object} element HTML element
     */
    KLogAnalyser.enableDisableMessageChannel = function (element) {

        $(element).parent().parent().find('.log-filter.log-filter-links [type="checkbox"]')
        .prop('checked', $(element).is(':checked'))
        .prop('disabled', $(element).is(':checked'));

    };

    /**
    Gera o html para árvore das informações de uma mensagem

    @method KLogAnalyser.generateHtmlCallInformations
    @param {Object} infos Information elements de uma mensagem
     */
    KLogAnalyser.generateHtmlCallInformations = function (infos) {

        if (!infos || !infos.length) {
            return '';
        }

        var html = '';
        var info;
        var name;
        var importantInfo;
        var importantInfos = ['Number digits', 'Channel number', 'User information layer 1 protocol'];

        for (var i = 0; i < infos.length; ++i) {

            name = infos[i].name;
            info = infos[i].value;
            importantInfo = '';

            if (typeof info === 'string' || typeof info === 'number') {

                html += '<div class="isdn-information-element" data-position="' + i + '"><span class="noArrow"></span>' + name + ': ' + info + '</div>';

            } else if (info === null) {

                html += '<div class="isdn-information-element" data-position="' + i + '"><span class="noArrow"></span>' + name + '</div>';

            } else {

                if (typeof info === 'object') {

                    if (info.Facility) {

                        importantInfo = ' (no decoded)';

                    } else {

                        for (var info_pos = 0, infos_len = importantInfos.length; info_pos < infos_len; info_pos++) {

                            if (importantInfos[info_pos]in info) {

                                importantInfo = ' (' + info[importantInfos[info_pos]] + ')';
                                break;
                            }
                        }
                    }
                }

                html += '<div class="isdn-information-element" data-position="' + i + '">';
                html += '<div class="cursorHand" onclick="KLogAnalyser.openClose(this);"><span class="arrow">[+]</span>' + name + ' ' + importantInfo + '</div>';
                html += '<div class="wrapers">';

                if ($.isArray(info)) {

                    html += '<div class="log-value">' + info.join(' ') + '</div>';

                } else if (typeof info === 'object') {

                    for (var j in info) {

                        html += '<div class="isdn-information-attribute"><span class="bold">' + j + ': </span><span>' + info[j] + '</span></div>';
                    }
                } else {

                    html += '<div>' + info + '</div>';
                }
                html += '</div>';
                html += '</div>';
            }
        }

        return html;
    };

    /**
    Mostra status carregamento

    @method KLogAnalyser.loadingContent
     */
    KLogAnalyser.loadingContent = function () {

        $('#refreshing').show();
    };

    /**
    Mostra status carregamento

    @method KLogAnalyser.finishedContent
     */
    KLogAnalyser.finishedContent = function () {

        $('#refreshing').hide();
        $('#loading').hide();
    };

    /**
    Gera o html dos checkboxs dos channels no filtro

    @method KLogAnalyser.printLogFilterChannelsCheckbox
    @param {Object} usedChannels
     */
    KLogAnalyser.printLogFilterChannelsCheckbox = function (usedChannels) {

        var html = [];
        var lnks = [];
        var id;

        for (id in usedChannels) {
            lnks.push(id);
        }

        lnks.sort();

        for (var i = 0; i < lnks.length; i++) {

            id = lnks[i];

            html[i] = '<input type="checkbox" disabled="disabled" checked="checked" id="link_' + id + '" class="log-filter-link" value="' + id + '"/><label for="link_' + id + '">' + (__('Canal %s (%s chamadas)').replace('%s', '<span class="bold">' + id + '</span>').replace('%s', usedChannels[id])) + '</label>';
        }

        $dialog.find('.log-filter-links').html('<li>' + html.join('</li><li>') + '</li>');
    };

    /**
    Interrompe o download dos dados de log

    @method KLogAnalyser.stopDownloadFile
     */
    KLogAnalyser.stopDownloadFile = function () {

        KLogAnalyser.statusCarregando(false);
        PartialDownload.stoped = true;
        // ShowNotice("Carregamento de dados interrompido!", false);
    };

    /**
    Mostra os bytes de forma resumida para o usuário

    @method KLogAnalyser.fileSizeHuman
    @param {Number} bytes
     */
    KLogAnalyser.fileSizeHuman = function (bytes) {

        var exp = Math.log(bytes) / Math.log(1024) | 0;
        var result = (bytes / Math.pow(1024, exp)).toFixed(2);

        return result + ' ' + (exp === 0 ? 'bytes' : 'KMGTPEZY'[exp - 1] + 'B');
    };

    /**
    Associa evento para carrega arquivo local com recurso arrastar

    @method KLogAnalyser.prepareDragFile
     */
    KLogAnalyser.prepareDragFile = function () {

        if (typeof window.FileReader === 'undefined') {
            console.log('Recurso FileReader não disponível.');
            return;
        }
    
        var _this = this;
        var holder = document.getElementById('inputFileLocal');
        var readFile = function (file) {

            var reader = new FileReader();
            
            reader.onload = function (event) {
                $dialog.find('.log-filesize').html(KLogAnalyser.fileSizeHuman($dialog.find('#inputFileLocal')[0].files[0].size));
                _this.analyseLog(event.target.result);
            };
            reader.readAsText(file);
        };

        holder.ondragover = function () {
            this.className = 'hover';
            return false;
        };

        holder.ondragend = function () {
            this.className = '';
            return false;
        };

        holder.ondrop = function (e) {
            this.className = '';
            e.preventDefault();

            var file = e.dataTransfer.files[0];
            readFile(file);
            return false;
        };

        holder.onchange = function (e) {

            var file = holder.files[0];
            var valid_name = /^.+\.log$/;

            if (file && file.name.match(valid_name)) {
                readFile(file);
            } else {
                console.log("File not supported!");
            }
        };
    };

    /**
    Alterna a interface para carregar o arquivo do servidor ou local

    @method KLogAnalyser.loadSourceFrom
     */
    KLogAnalyser.loadSourceFrom = function (value) {

        var remote = (value === 'remote') ? true : false;

        $dialog.find('.log-remote').toggle(remote);
        $dialog.find('.log-local').toggle(!remote);
    };

    /**
    Gera o html para mostrar o fonte da mensagem (tooltip)

    @method KLogAnalyser.makeSourceHtml
    @param {String} source
    @param {Array} info_pos Para destacar o information element na mensagem
     */
    KLogAnalyser.makeSourceHtml = function (source, info_pos) {

        var aux = source.split('] ');
        var auxMessage = aux[1].split(' ');
        auxMessage[8] = '<span style="border-radius: 3px; background: white; color: gray; border: 1px solid gray; padding: 2px;" title="' + __('Código da mensagem') + '">' + auxMessage[8] + '</span>';
        if (info_pos && info_pos.length === 2) {
            auxMessage[info_pos[0]] = '<span style="border-radius: 3px; background: white; color: gray; border: 1px solid gray; padding: 2px" title="Information element">' + auxMessage[info_pos[0]];
            auxMessage[info_pos[1]] = auxMessage[info_pos[1]] + '</span>';
        }
        
        return '<span style="color:gray;">' + aux[0] + '] </span>' + auxMessage.join(' ');
    };

    /**
    Renderiza a tabela para quando é ISDN ou R2

    @method KLogAnalyser.renderTable
    @param {Array} calls Lista de chamadas para desenhar
    @param {Function} functionCreateLine Função para criar uma linha. Esta função adiciona o elemento no DOM
    @param {Number} sort  Crescente (0) ou Decrescente (1)
    @param {Number} [offset] Limpa ou adiciona mais linhas na tabela.
     */
    KLogAnalyser.renderTable = function (calls, functionCreateLine, sort, offset) {

        sort = (sort === 1) ? 1 : 0;
        offset = (typeof offset === 'undefined') ? 0 : offset;
        calls = calls || [];

        $dialog.find('.log-calls-wrapper').show();
        $dialog.find('.log-calls').show();

        var _this = this;
        var call;
        var totalVisible;
        var totalFiltradas = calls.length;
        var maxInPage = maxPerPage + offset;
        var maxIndex = (totalFiltradas - maxInPage > 0) ? totalFiltradas - maxInPage : 0;
        var msg = '';

        if (offset === 0 || totalFiltradas === 0) {

            $dialog.find('.log-calls').html('');
        }

        if (totalFiltradas > 0) {

            if (sort === 1) {

                for (call_pos = totalFiltradas - offset - 1; call_pos >= maxIndex; call_pos--) {

                    call = calls[call_pos];
                    functionCreateLine(call);

                }

            } else {

                for (call_pos = offset, match_len = totalFiltradas; call_pos < match_len; call_pos++) {

                    if (call_pos >= maxInPage) {
                        break;
                    }

                    call = calls[call_pos];
                    functionCreateLine(call);

                }
            }
        }

        if (totalFiltradas !== 0) {

            totalVisible = $dialog.find('.log-calls .call:visible').length;

            if (totalVisible < totalFiltradas) {

                msg = __("Mostrando %s de %s chamadas.").replace('%s', totalVisible).replace('%s', totalFiltradas);

                /// Botão de paginação
                $paginationShowMore = $('<div />')
                    .attr('title', __('Mostrar mais'))
                    .addClass('dotdotdot')
                    .appendTo('.log-calls')
                    .bind('click', function () {

                        var totalVisible = $dialog.find('.log-calls .call:visible').length;
                        $(this).remove();

                        _this.renderFilterTable(null, totalVisible);

                        // setTimeout(function() {
                        $dialog.animate({
                            scrollTop : $dialog[0].scrollHeight
                        }, scrollTime, scrollAnim);
                        // $('#divLogAnalyser').scrollTop($('#divLogAnalyser')[0].scrollHeight);
                        // }, 100);

                    });

            } else {

                msg = (totalFiltradas > 1) ? __("Mostrando %s chamadas.").replace('%s', totalFiltradas) : __("Mostrando uma chamada.");
            }

            $dialog.find('.show-only-if-calls').show();
            $dialog.find('.log-table-status').html(msg);

        } else {

            $dialog.find('.show-only-if-calls').hide();
            $dialog.find('.log-table-status').html('');

            msg = __("Nenhuma chamada encontrada com este filtro.");
            $dialog.find('.log-calls').html('<p class="calls-not-found">' + msg + '</p>');
        }

        $dialog.find('.log-local-filter').show();

    };

    /**
    Desenha o gráfico de mensagens por chamada

    @method KLogAnalyser.messagesByCallChart
     */
    KLogAnalyser.messagesByCallChart = function (messagesLengthByCall) {
        // $('.messagesLengthChart')
        // .sparkline(messagesLengthByCall, {
        // type : 'line',
        // width : '150px',
        // height : '30px',
        // chartRangeMin : 0
        // });
    };

    ///  Análise do log para cada tipo

    ///  ISDN

    /**
    Chama uma nova análise do conteúdo do arquivo ISDN

    @method KLogAnalyser.analyseISDN
    @param {String} logFileContent Conteúdo do arquivo
     */
    KLogAnalyser.analyseISDN = function (logFileContent) {

        var _this = this;
        var ISDN = Khomp.ISDN;
        var beginDate = $dialog.find('[name="beginDate"]').val();
        var beginHour = $dialog.find('[name="beginHour"]').val() + ':00';
        var endDate = $dialog.find('[name="endDate"]').val();
        var endHour = $dialog.find('[name="endHour"]').val() + ':00';
        var begin = beginDate + ' ' + beginHour;
        var end = endDate + ' ' + endHour;

        if (!logFileContent && (!isValidDate(beginDate) || !isValidDate(endDate))) {
            ShowNotice('Data inválida', false);
            return false;
        }

        ISDN.clear();

        this.fillLogSummary();
        this.statusCarregando(true);
        this.changePercentageDiv();
        this.centralizeDialog();

        $dialog.find('.log-load-form').hide();

        if (logFileContent) {

            /// opção arquivo local
            $dialog.find('.log-wait').show();
            $dialog.find('.log-first-message-time').html('-');
            $dialog.find('.log-last-message-time').html('-');

            setTimeout(function () {

                ISDN.extractData(logFileContent, function () {

                    _this.changePercentageDiv(50, 100);

                    ISDN.makeCallAnalysys(function () {

                        var logObj = ISDN.logObj;

                        $dialog.find('.log-wait').hide();
                        $dialog.find('.log-first-message-time').html((logObj && logObj.firstCall) ? logObj.firstCall.Start.replace(/\..+$/, '') : 'ERRO');
                        $dialog.find('.log-last-message-time').html((logObj && logObj.lastCall) ? logObj.lastCall.Stop.replace(/\..+$/, '') : 'ERRO');

                        _this.printLogFilterLinksCheckbox(logObj.links);
                        _this.fillLogSummary(logObj.totalErrors, logObj.totalAlerts, logObj.totalSuccess, logObj.totalCalls);
                        _this.fillSelectMessageTypes(logObj.messageTypes);
                        _this.renderFilterTable();
                        _this.changePercentageDiv(100, 100);
                        _this.statusCarregando(false);
                        _this.messagesByCallChart(logObj.messagesLengthByCall);

                        if (logObj.oldFormat) {

                            console.log(__('Log no formato anterior à versão 3.'));
                        }
                    });
                });
            }, 0);
            return;
        }

        PartialDownload.download({
            file : 'isdn.log',
            begin : begin,
            end : end
        }, function (response) {

            /// Callback RECEBEU PARTE do arquivo.
            // setTimeout(function () {
            ISDN.extractData(response.content.file, function () {

                ISDN.makeCallAnalysys(function () {

                    _this.printLogFilterLinksCheckbox(ISDN.logObj.links);
                    _this.fillLogSummary(ISDN.logObj.totalErrors, ISDN.logObj.totalAlerts, ISDN.logObj.totalSuccess, ISDN.logObj.totalCalls);
                });
            });
            // }, 0);

            $dialog.find('.log-filesize').html(_this.fileSizeHuman(response.content.filesize));
            _this.changePercentageDiv(response.content.offset, response.content.filesize);

        }, function () {
            /// Callback FINAL: quando termina todo o arquivo.
            // setTimeout(function () {
            $dialog.find('.log-first-message-time').html(begin);
            $dialog.find('.log-last-message-time').html(end);
            ISDN.makeCallAnalysys(function () {

                _this.printLogFilterLinksCheckbox(ISDN.logObj.links);
                _this.fillLogSummary(ISDN.logObj.totalErrors, ISDN.logObj.totalAlerts, ISDN.logObj.totalSuccess, ISDN.logObj.totalCalls);
                _this.fillSelectMessageTypes(ISDN.logObj.messageTypes);
                _this.renderFilterTable();

                _this.statusCarregando(false);
                _this.messagesByCallChart(ISDN.logObj.messagesLengthByCall);
            });
            // }, 1);
        });
    };

    /**
    Retorna as chamadas de acordo com o filtro da interface

    @method KLogAnalyser.filterCallsISDN
    @return {Array} Lista de chamadas
     */
    KLogAnalyser.filterCallsISDN = function () {

        var ISDN = Khomp.ISDN;
        var calls = ISDN.logObj.calls;
        var call;
        var totalFiltradas = 0;
        var call_pos;
        this.matchCalls = [];

        var inputNumeroA = $dialog.find('#inputNumeroA').val();
        var inputNumeroB = $dialog.find('#inputNumeroB').val();
        var inputCrv = $dialog.find('#inputCrv').val().toUpperCase();

        var errorMessages = $dialog.find('#logFilterError').is(':checked');
        var alertMessages = $dialog.find('#logFilterAlert').is(':checked');
        var successMessages = $dialog.find('#logFilterSuccess').is(':checked');

        var checkedLinks;
        var $checkedLinks = $dialog.find('.log-filter-links .log-filter-link:checked');
        var $checkedAllLinks = $dialog.find('#chkLinkOrChannel');

        if ($checkedLinks.length) {
            checkedLinks = {};
            $dialog.find('.log-filter-links .log-filter-link:checked').each(function () {
                checkedLinks[$(this).val()] = true;
            });
        }

        var chkMessageTypes = $dialog.find('#chkMessageTypes').is(':checked');
        var messageTypes = $dialog.find('#messageTypes').val();

        for (call_pos = 0, calls_len = calls.length; call_pos < calls_len; call_pos++) {

            call = calls[call_pos];

            if (typeof call.From === 'undefined' || typeof call.To === 'undefined') {
                ISDN.getCallFromTo(call);
            }

            if (!errorMessages && call.Status === "ERROR") {
                continue;
            }

            if (!alertMessages && call.Status === "ALERT") {
                continue;
            }

            if (!successMessages && call.Status === "OK") {
                continue;
            }

            if (inputNumeroA !== '' && call.From.indexOf(inputNumeroA) === -1) {
                continue;
            }

            if (inputNumeroB !== '' && call.To.indexOf(inputNumeroB) === -1) {
                continue;
            }

            if (inputCrv !== '' && call.CallReference.indexOf(inputCrv) === -1) {
                continue;
            }

            if (!$checkedAllLinks.is(':checked') && checkedLinks && !(call.Link in checkedLinks)) {
                continue;
            }

            if (chkMessageTypes) {

                if (messageTypes && messageTypes.length) {

                    var temTodasMensagens = ISDN.callHasAllThisMessages(messageTypes, call);

                    if (!temTodasMensagens) {
                        continue;
                    }
                }
            }
            // if (!(compareDateTimeString(beginDate + ' ' + beginHour, call.Start) && compareDateTimeString(call.Stop, endDate + ' ' + endHour))) {
            // continue;
            // }

            this.matchCalls.push(call);

        }

        return this.matchCalls;

    };

    /**
    Gera o html de uma chamada da lista para ISDN

    @method KLogAnalyser.createTreeMessagesISDN
    @param {String} id Identificador da chamada (call reference)
     */
    KLogAnalyser.createTreeMessagesISDN = function (id) {

        var html = '<p class="bold">' + __('Mensagens da chamada') + ': </p>';
        var position = parseInt(id, 10) - 1;
        var ISDN = Khomp.ISDN;
        var messages = ISDN.logObj.calls[position].Messages;
        var msg;

        for (var i = 0, msgs_len = messages.length; i < msgs_len; i++) {

            msg = messages[i];
            idw = 'msg_' + id + '_' + i;
            var datetime = msg.getDatetime();
            var direction = msg.getDirection();
            var directionTitle = (direction === '->') ? __("Mensagem enviada") : __('Mensagem recebida');
            var type = msg.getTypeName();

            html += '<div id="' + idw + '" class="log-message" data-position="' + i + '">';

            if (msg.infos && msg.infos.length) {

                html += '<div class="log-list-openclose cursorHand" onclick="KLogAnalyser.openCloseWraper(this);">' +

                '<table>' +
                '<tr>' +
                '<td class="log-message-decoded">' +
                '<span class="arrow">[+]</span> ' + datetime + ' <span class="log-help" title="' + directionTitle + '">' + direction + '</span> <span>' + type + '</span>' +
                '</td>' +
                '<td class="log-source-line" style="visibility:hidden;">' +
                '<span>' + __('Fonte') + ': </span><span class="msg-source">' + msg.source + '</span>' +
                '</td>' +
                '</tr>' +
                '</table>' +

                '</div>' +

                '<div class="wrapers">' + KLogAnalyser.generateHtmlCallInformations(msg.getDecodedInformationElements()) + '</div>';

            } else {

                html += '<div>' +
                '<table>' +
                '<tr>' +
                '<td class="log-message-decoded">' +
                '<span class="noArrow"></span>' + datetime + ' <span class="log-help" title="' + directionTitle + '">' + direction + '</span> <span>' + type + '</span>' +
                '</td>' +
                '<td class="log-source-line" style="visibility:hidden;">' +
                ' <span>' + __('Fonte') + ': ' + msg.source + '</span>' +
                '</td>' +
                '</tr>' +
                '</table>' +

                '</div>';
            }

            html += '</div>';
        }

        $dialog.find('#wrapers' + id).html(html);
    };

    /**
    Gera o html de uma chamada e insere na lista

    @method createCallLineISDN
    @param {Object} call
     */
    KLogAnalyser.createCallLineISDN = function (call) {

        var id = call.id;
        var call_pos = (id - 1);
        var ISDN = Khomp.ISDN;
        var direction = typeof call.Direction !== 'undefined' ? call.Direction : '-';
        var start = (call.Start) ? call.Start : '-';
        var from = (call.From) ? call.From : '-';
        // var totalMessages = call.Messages.length;
        var crv = typeof call.CallReference !== 'undefined' ? call.CallReference : '-';
        var stop = (call.Stop) ? call.Stop : '-';
        var to = (call.To) ? call.To : '-';
        var status = typeof call.Status !== 'undefined' ? call.Status : 'ERROR';
        var link = (ISDN.logObj.oldFormat) ? call.Link : call.deviceId + '/' + call.Link;
        var linkTitle = (ISDN.logObj.oldFormat) ? __('Número identificador da chamada') : __('ID do dispositivo (device ID) / Link');
        var linkLabel = (ISDN.logObj.oldFormat) ? __('NAI') : __('Disp/Link');

        var cor = (status === 'OK') ? 'CallOK'
         : (status === 'ALERT') ? 'CallAlert'
         : 'CallError';

        var statusTitle = (status === 'OK') ? __('Chamada realizada com sucesso')
         : (status === 'ALERT') ? __('Chamada com atendimento mas com finalização anormal.')
         : __('Chamada com possibilidade de erros.');

        var line = '<div id="call_' + id + '" class="call divMain lineHover" data-position="' + call_pos + '">' +

            '<table class="cursorHand" width="100%" onclick="KLogAnalyser.openCloseWrapers(\'' + id + '\');">' +
            '<tr>' +
            '<td rowspan="2" width="30"><div class="divImg ' + cor + ' log-help" title="' + statusTitle + '"></div></td>' +
            '<td width="50" align="right" class="log-help" title="' + __('Sequência atribuída para a chamada conforme encontrada nos dados analisados') + '"><span>' + __('Sequência') + '</span>:</td>' +
            '<td width="100" class="log-value">' + id + '</td>' +
            '<td width="80" align="right" class="log-help" title="' + __('Direção da chamada') + '"><span>' + __('Direção') + '</span>:</td>' +
            '<td width="40" class="log-value">' + direction + '</td>' +
            '<td width="60" align="right" class="log-help" title="' + __('Número de origem da chamada') + '"><span>' + __('De') + '</span>:</td>' +
            '<td class="log-value">' + from + '</td>' +
            '<td align="right" width="250" class="log-help" title="' + __('Data e hora do início da chamada') + '"><span>' + __('Início') + '</span>: <span class="log-value">' + start + '</span></td>' +
            '</tr>' +
            '<tr>' +
            '<td align="right" class="log-help" title="' + linkTitle + '">' + linkLabel + ': </td>' +
            '<td class="log-value"><span>' + link + '</span></td>' +
            '<td align="right" class="log-help" title="' + __('Referência da chamada (Call Reference Value)') + '"><span>CRV</span>:</td>' +
            '<td class="log-value">' + crv + '</td>' +
            '<td align="right" class="log-help" title="' + __('Número de destino da chamada') + '"><span>' + __('Para') + '</span>:</td>' +
            '<td class="log-value">' + to + '</td>' +
            '<td align="right" class="log-help" title="' + __('Data e hora do fim da chamada') + '"><span>' + __('Fim') + '</span>: <span class="log-value">' + stop + '</span></td>' +
            '</tr>' +
            '</table>' +

            '<div class="wrapers" id="wrapers' + id + '"></div>' + '</div>';

        $dialog.find('.log-calls').append(line);
    };

    /**
    Preenche o campo de tipos de mensagens no filtro (ISDN).

    @method KLogAnalyser.fillSelectMessageTypes
    @param {Object} messageTypes
     */
    KLogAnalyser.fillSelectMessageTypes = function (messageTypes) {

        var optionsMessageTypes = '';
        var title;
        var text;

        for (var i in messageTypes) {
            text = i;
            title = '';
            if (text.length > 50) {
                title = ' title="' + text + '"';
                text = text.substr(0, 47) + '...';
            }
            optionsMessageTypes += '<option value="' + i + '"' + title + '>' + text + '</option>';
        }
        $dialog.find('#messageTypes')
        .html(optionsMessageTypes)
        .sortOptions();
    };

    /// R2

    /**
    Chama uma nova análise do conteúdo do arquivo R2

    @method KLogAnalyser.analyseR2
    @param {Array|String} logFileContent Conteúdo do arquivo
     */
    KLogAnalyser.analyseR2 = function (logFileContent) {

        var _this = this;
        var beginDate = $dialog.find('[name="beginDate"]').val();
        var beginHour = $dialog.find('[name="beginHour"]').val() + ':00';
        var endDate = $dialog.find('[name="endDate"]').val();
        var endHour = $dialog.find('[name="endHour"]').val() + ':00';
        var R2 = Khomp.R2;

        if (!logFileContent && (!isValidDate(beginDate) || !isValidDate(endDate))) {
            ShowNotice('Data inválida', false);
            return false;
        }

        R2.clear();

        this.fillLogSummary();
        this.statusCarregando(true);
        this.changePercentageDiv();
        this.centralizeDialog();

        $dialog.find('.log-load-form').hide();

        if (logFileContent) {

            /// opção arquivo local
            $dialog.find('.log-wait').show();
            $dialog.find('.log-first-message-time').html('-');
            $dialog.find('.log-last-message-time').html('-');

            R2.extractData(logFileContent, function (err, calls) {

                R2.makeCallAnalysys(function (err, logObj) {

                    $dialog.find('.log-wait').hide();
                    $dialog.find('.log-load-form').hide();

                    $dialog.find('.log-first-message-time').html((logObj && logObj.Start) ? logObj.Start.replace(/\..+$/, '') : 'ERRO');
                    $dialog.find('.log-last-message-time').html((logObj && logObj.Stop) ? logObj.Stop.replace(/\..+$/, '') : 'ERRO');

                    _this.fillLogSummary(logObj.totalErrors, logObj.totalAlerts, logObj.totalSuccess, R2.calls.length);

                    _this.printLogFilterChannelsCheckbox(R2.usedChannels);
                    _this.fillSelectMessageTypesR2();

                    _this.renderFilterTable();
                    _this.changePercentageDiv(100, 100);
                    _this.statusCarregando(false);
                    _this.messagesByCallChart(logObj.messagesLengthByCall);

                    if (R2.oldFormat) {

                        console.log(__('Log no formato anterior à versão 3.'));
                    }

                });
            });

            return;
        }

        PartialDownload.download({
            file : 'r2.log',
            begin : beginDate + ' ' + beginHour,
            end : endDate + ' ' + endHour
        }, function (response) {

            R2.extractData(response.content.file, function () {

                R2.makeCallAnalysys(function (err, logObj) {
                    console.log('R2: Parte concluida!');
                    _this.printLogFilterChannelsCheckbox(R2.usedChannels);
                    _this.fillLogSummary(R2.logObj.totalErrors, R2.logObj.totalAlerts, R2.logObj.totalSuccess, R2.calls.length);
                });
            });

            $dialog.find('.log-filesize').html(KLogAnalyser.fileSizeHuman(response.content.filesize));
            _this.changePercentageDiv(response.content.offset, response.content.filesize);

        }, function () {

            R2.makeCallAnalysys(function (err, logObj) {

                $dialog.find('.log-wait').hide();
                $dialog.find('.log-load-form').hide();

                // $dialog.find('.log-first-message-time').html(logObj.Start.replace(/\..+$/, ''));
                // $dialog.find('.log-last-message-time').html(logObj.Stop.replace(/\..+$/, ''));

                _this.fillLogSummary(logObj.totalErrors, logObj.totalAlerts, logObj.totalSuccess, R2.calls.length);
                _this.printLogFilterChannelsCheckbox(R2.usedChannels);
                _this.fillSelectMessageTypesR2();

                _this.renderFilterTable();
                _this.changePercentageDiv(100, 100);
                _this.statusCarregando(false);
                _this.messagesByCallChart(logObj.messagesLengthByCall);

                if (R2.oldFormat) {

                    console.log(__('Log no formato anterior à versão 3.'));
                }

                console.log('R2: TUDO concluido!');
            });
        });
    };

    /**
    Retorna as chamadas R2 de acordo com filtro da interface
    @method KLogAnalyser.filterCallsR2
    @return {Array} Lista de chamadas
     */
    KLogAnalyser.filterCallsR2 = function () {

        var R2 = Khomp.R2;
        var calls = Khomp.R2.calls;
        var call;
        var totalFiltradas = 0;
        var call_pos;
        this.matchCalls = [];

        var inputNumeroA = $dialog.find('#inputNumeroA').val();
        var inputNumeroB = $dialog.find('#inputNumeroB').val();

        var errorMessages = $dialog.find('#logFilterError').is(':checked');
        var alertMessages = $dialog.find('#logFilterAlert').is(':checked');
        var successMessages = $dialog.find('#logFilterSuccess').is(':checked');

        var chkMessageTypes = $dialog.find('#chkMessageTypes').is(':checked');
        var messageTypes = $dialog.find('#messageTypes').val();

        var checkedLinks;
        var $checkedLinks = $dialog.find('.log-filter-links .log-filter-link:checked');
        var $checkedAllLinks = $dialog.find('#chkLinkOrChannel');

        if ($checkedLinks.length) {

            checkedLinks = {};

            $dialog.find('.log-filter-links .log-filter-link:checked').each(function () {

                checkedLinks[$(this).val()] = true;
            });
        }

        for (call_pos = 0, calls_len = calls.length; call_pos < calls_len; call_pos++) {

            call = calls[call_pos];

            if (!errorMessages && call.Status === "ERROR") {
                continue;
            }
            if (!alertMessages && call.Status === "ALERT") {
                continue;
            }
            if (!successMessages && call.Status === "OK") {
                continue;
            }

            if (inputNumeroA !== '' && call.From.indexOf(inputNumeroA) === -1) {
                continue;
            }

            if (inputNumeroB !== '' && call.To.indexOf(inputNumeroB) === -1) {
                continue;
            }

            if (!$checkedAllLinks.is(':checked') && checkedLinks && !(call.channel in checkedLinks)) {
                continue;
            }

            if (chkMessageTypes) {

                if (messageTypes && messageTypes.length) {

                    var temTodasMensagens = R2.callHasAllThisMessages(messageTypes, call);

                    if (!temTodasMensagens) {
                        continue;
                    }
                }
            }

            this.matchCalls.push(call);

        }

        return this.matchCalls;
    };

    /**
    Gera o html de uma chamada da lista para R2

    @method KLogAnalyser.createTreeMessagesR2
    @param {String} id Identificador da chamada
     */
    KLogAnalyser.createTreeMessagesR2 = function (id) {

        var position = parseInt(id, 10) - 1;
        var messages = Khomp.R2.calls[position].Messages;
        var html = '';
        var msg;
        var lastMsgStep;

        for (var i = 0, msgs_len = messages.length; i < msgs_len; i++) {

            msg = messages[i];
            idw = 'msg_' + id + '_' + i;

            if (lastMsgStep !== msg.step) {

                if (msg.step) {

                    html += '<p class="bold title-sep-right">' + __('Fase') + ': ' + msg.step + '</p>';

                } else {

                    html += '<p class="bold title-sep-right">' + __('Troca MFC') + '</p>';
                }
            }
            var message = msg.message;
            message = (message.indexOf('->') !== -1) ? message.replace('->', '<span class="log-help" title="' + __("Mensagem enviada") + '">-></span>') : message.replace('<-', '<span class="log-help" title="' + __("Mensagem recebida") + '"><-</span>');
            lastMsgStep = msg.step;

            html += '<div id="' + idw + '" class="log-message">' +
            '<table>' +
            '<tr>' +
            '<td class="log-message-decoded">' +
            '<span class="noArrow"></span>' + msg.time + ' | <div class="bold" style="display: inline-block; min-width: 90px;">' + message + '</div> ' + '<span>' + msg.type + '</span>' +
            '</td>' +
            '<td class="log-source-line" style="visibility:hidden;">' +
            '<span>' + __('Fonte') + ': ' + msg.source + '</span>' +
            '</td>' +
            '</tr>' +
            '</table>' +

            '</div>';
        }

        $dialog.find('#wrapers' + id).html(html);
    };

    /**
    Gera o html de uma chamada e insere na lista

    @method createCallLineR2
    @param {Object} call
     */
    KLogAnalyser.createCallLineR2 = function (call) {

        var id = call.id;
        var direction = typeof call.Direction !== 'undefined' ? call.Direction : '-';
        var deviceId = typeof call.deviceId !== 'undefined' ? call.deviceId : '-';
        var user = typeof call.User !== 'undefined' ? call.User : '-';
        var channel = typeof call.channel !== 'undefined' ? call.channel : '-';
        var start = (call.Start) ? call.Start : '-';
        var from = (call.From) ? call.From : '-';
        var stop = (call.Stop) ? call.Stop : '-';
        var to = (call.To) ? call.To : '-';
        var status = typeof call.Status !== 'undefined' ? call.Status : 'ERROR';

        var cor = (status === 'OK') ? 'CallOK'
         : (status === 'ALERT') ? 'CallAlert'
         : 'CallError';

        var statusTitle = (status === 'OK') ? __('Chamada realizada com sucesso')
         : (status === 'ALERT') ? __('Chamada com atendimento mas com finalização anormal.')
         : __('Chamada com possibilidade de erros.');

        var line = '<div id="call_' + id + '" class="call divMain lineHover">' +

            '<table class="cursorHand" width="100%" onclick="KLogAnalyser.openCloseWrapers(\'' + id + '\', \'r2\');">' +
            '<tr>' +
            '<td rowspan="2" width="30"><div class="divImg ' + cor + ' log-help" title="' + statusTitle + '"></div></td>' +
            '<td width="40" align="right"><span class="log-help" title="' + __('Sequência geral de acordo com as chamadas encontradas nos dados analisados') + '">' + __('Sequência') + '</span>:</td>' +
            '<td width="100" class="log-value">' + id + '</td>' +
            '<td width="80" align="right"><span class="log-help" title="' + __('Direção da chamada') + '">' + __('Direção') + '</span>:</td>' +
            '<td width="40" class="log-value">' + direction + '</td>' +
            '<td width="60" align="right"><span class="log-help" title="' + __('Número de origem da chamada') + '">' + __('De') + '</span>:</td>' +
            '<td class="log-value">' + from + '</td>' +
            '<td align="right" width="250"><span class="log-help" title="' + __('Data e hora do início da chamada') + '">' + __('Início') + '</span>: <span class="log-value">' + start + '</span></td>' +
            '</tr>' +
            '<tr>' +
            '<td align="right"><span class="log-help" title="' + __('ID do dispositivo (device ID) / Canal') + '">' + __('Disp/Canal') + '</span>:</td>' +
            '<td class="log-value">' + deviceId + '/' + channel + '</td>' +
            '<td align="right"><span class="log-help" title="' + __('Identificação do usuário na chamada. A = originador, B = destinatário') + '">' + __('Usuário') + '</span>:</td>' +
            '<td class="log-value">' + user + '</td>' +
            '<td align="right"><span class="log-help" title="' + __('Número de destino da chamada') + '">' + __('Para') + '</span>:</td>' +
            '<td class="log-value">' + to + '</td>' +
            '<td align="right"><span class="log-help" title="' + __('Data e hora do fim da chamada') + '">' + __('Fim') + '</span>: <span class="log-value">' + stop + '</span></td>' +
            '</tr>' +
            '</table>' +

            '<div class="wrapers" id="wrapers' + id + '"></div>' + '</div>';

        $dialog.find('.log-calls').append(line);
    };

    /**
    Preenche o campo de tipos de mensagens no filtro (ISDN).

    @method KLogAnalyser.fillSelectMessageTypes
     */
    KLogAnalyser.fillSelectMessageTypesR2 = function () {

        var R2 = Khomp.R2;
        var optionsMessageTypes = '';
        var title;
        var text;

        for (var i in R2.messageTypes) {
            text = i;
            title = '';
            if (text.length > 50) {
                title = ' title="' + text + '"';
                text = text.substr(0, 47) + '...';
            }
            optionsMessageTypes += '<option value="' + i + '"' + title + '>' + text + '</option>';
        }
        /*
        optionsMessageTypes += '<optgroup label="'+__('Sinalização')+'">';
        for (var i in R2.lins) {
        text = i + ' - ' + R2.translateData(i);
        title = '';
        if (text.length > 50) {
        title = ' title="' + text + '"';
        text = text.substr(0, 47) + '...';
        }
        optionsMessageTypes += '<option value="' + i + '"' + title + '>' + text + '</option>';
        }
        optionsMessageTypes += '</optgroup>';

        optionsMessageTypes += '<optgroup label="'+__('MFC')+'">';
        for (var i in R2.mfcs) {
        text = i + ' - ' + R2.translateData(i);
        title = '';
        if (text.length > 50) {
        title = ' title="' + text + '"';
        text = text.substr(0, 47) + '...';
        }
        optionsMessageTypes += '<option value="' + i + '"' + title + '>' + text + '</option>';
        }
        optionsMessageTypes += '</optgroup>';
         */
        $dialog.find('#messageTypes')
        .html(optionsMessageTypes)
        .sortOptions();
    };

    /// Download do arquivo

    /**
    Permite fazer o downlod do arquivo de log em partes
     */
    var PartialDownload = {
        stoped : false,
        maxSliceSize : 100000 * 1024, /// Bytes
        download : function (filter, preProcessCallback, finishCallback) {

            console.time('Total download');

            var totalDownloadPart = 0;

            filter = filter || {
                offset : 0
            };

            this.stoped = false;

            preProcessCallback = preProcessCallback || function (response) {
                console.log('callback preprocess: ', response.content.offset);
            };

            finishCallback = finishCallback || function (response) {
                console.log('callback finish:', response.content.finish);
            };

            var _this = this;

            var partCallback = function (response) {

                totalDownloadPart++;

                if (response) {
                    if (!response.success) {
                        ShowNotice(response.message || __('Erro na resposta.'));
                        return;
                    }
                } else {
                    ShowNotice(__('Erro na resposta.'));
                    return;
                }

                if (_this.stoped) {
                    ShowNotice('Carregamento de dados interrompido pelo usuário.');
                    // return;
                }

                filter.offset = response.content.offset;

                preProcessCallback(response);

                if (_this.stoped || response.content.finish) {
                    console.log("Total partials: ", totalDownloadPart);
                    console.timeEnd('Total download');
                    finishCallback(response);
                } else {
                    _this.downloadPart(filter, partCallback);
                }

            };

            this.downloadPart(filter, partCallback);
        },

        downloadPart : function (filter, callback) {

            console.time('>>> Download...');
            console.log('### Início Download...', new Date());

            jQuery.ajax({
                beforeSend : KLogAnalyser.loadingContent(),
                complete : KLogAnalyser.finishedContent(),
                url : GLOBALAjaxURL,
                data : {
                    ajaxTo : 'partialDownload',
                    file : filter.file,
                    max : PartialDownload.maxSliceSize,
                    offset : filter.offset,
                    begin : filter.begin || '',
                    end : filter.end || ''
                },
                dataType : "json",
                success : function (ret) {
                    console.timeEnd('>>> Download...');
                    console.log('### Fim Download...', new Date());
                    callback(ret);
                },
                error : function () {
                    callback('Erro na requisição para baixar o arquivo de log.');
                    console.error('Ocorreu algum erro.', arguments);
                }
            });
        },
        performanceTest : function () {

            var _this = this;

            _this.testLittlePartSize(function () {
                _this.testMediumPartSize(function () {
                    _this.testBigPartSize(function () {
                        _this.testOneRequest(function () {
                            console.log('Concluído.');
                        });
                    });
                });
            });

        },
        testLittlePartSize : function (callback) {

            PartialDownload.maxSliceSize = 300 * 1024;

            ISDN.clear();
            console.time("Teste testLittlePartSize - Size (" + PartialDownload.maxSliceSize + ") ");

            PartialDownload.download({
                begin : '28/02/2014 12:00',
                end : '28/02/2014 13:00'
            }, function (response) {
                ISDN.extractData(response.content.file, true);
            }, function () {
                KLogAnalyser.renderFilterTable();
                console.log('-------------------------------------------------------------------------');
                console.timeEnd("Teste testLittlePartSize - Size (" + PartialDownload.maxSliceSize + ") ");
                callback();
            });
        },
        testMediumPartSize : function (callback) {

            ISDN.clear();
            PartialDownload.maxSliceSize = 1000 * 1024;

            console.time("Teste testMediumPartSize - Size (" + PartialDownload.maxSliceSize + ") ");

            PartialDownload.download({
                begin : '28/02/2014 12:00',
                end : '28/02/2014 13:00'
            }, function (response) {
                ISDN.extractData(response.content.file, true);
            }, function () {
                KLogAnalyser.renderFilterTable();
                console.log('-------------------------------------------------------------------------');
                console.timeEnd("Teste testMediumPartSize - Size (" + PartialDownload.maxSliceSize + ") ");
                callback();
            });
        },
        testBigPartSize : function (callback) {

            ISDN.clear();
            PartialDownload.maxSliceSize = 5000 * 1024;

            callback = callback || function () {
                console.log('CALLBACK testBigPartSize');
            };

            console.time("Teste testBigPartSize - Size (" + PartialDownload.maxSliceSize + ") ");

            PartialDownload.download({
                begin : '28/02/2014 12:00',
                end : '28/02/2014 13:00'
            }, function (response) {
                ISDN.extractData(response.content.file, true);
            }, function () {
                KLogAnalyser.renderFilterTable();
                console.log('-------------------------------------------------------------------------');
                console.timeEnd("Teste testBigPartSize - Size (" + PartialDownload.maxSliceSize + ") ");
                callback();
            });
        },
        testExtraBigPartSize : function (callback) {

            ISDN.clear();
            PartialDownload.maxSliceSize = 10000 * 1024;

            callback = callback || function () {
                console.log('CALLBACK testBigPartSize');
            };

            console.time("Teste testBigPartSize - Size (" + PartialDownload.maxSliceSize + ") ");

            PartialDownload.download({
                begin : '28/02/2014 12:00',
                end : '28/02/2014 13:00'
            }, function (response) {
                ISDN.extractData(response.content.file, true);
            }, function () {
                KLogAnalyser.renderFilterTable();
                console.log('-------------------------------------------------------------------------');
                console.timeEnd("Teste testBigPartSize - Size (" + PartialDownload.maxSliceSize + ") ");
                callback();
            });
        },
        testOneRequest : function (callback) {

            console.time("Teste testOneRequest");

            jQuery.ajax({
                beforeSend : KLogAnalyser.loadingContent(),
                complete : KLogAnalyser.finishedContent(),
                url : GLOBALAjaxURL,
                data : {
                    ajaxTo : 'directDownload',
                    file : 'isdn.log'
                },
                dataType : "json",
                success : function (response) {

                    console.log('-------------------------------------------------------------------------');
                    console.timeEnd("Teste testOneRequest");

                    ISDN.clear();
                    ISDN.extractData(response.content.file);
                    KLogAnalyser.renderFilterTable();

                    callback(response);
                },
                error : function () {
                    callback('Erro na requisição para baixar o arquivo de log.');
                    console.error('Ocorreu algum erro.', arguments);
                }
            });
        }
    };

    /**
    Compara se o primeira data é menor ou igual a segunda data.
     */
    /**function compareDateTimeString(a, b) {

    var re = /\/|\s|\.|:/;
    var auxA = a.split(re);
    var auxB = b.split(re);

    auxA = [auxA[2], auxA[1], auxA[0], auxA[3], auxA[4]].join('-');
    auxB = [auxB[2], auxB[1], auxB[0], auxB[3], auxB[4]].join('-');

    return auxA <= auxB;
    }
     */

    /**
    Valida data

    @method isValidDate
    @param {String} txtDate Data no formato dd-mm-yyyy ou dd/mm/yyyy
    @return {Boolean} True se OK e false se inválida
     */
    function isValidDate(txtDate) {

        if (!txtDate) {
            return false;
        }

        var rxDatePattern = /^(\d{1,2})(\/|-)(\d{1,2})(\/|-)(\d{4})$/;
        var dtArray = txtDate.match(rxDatePattern);

        if (dtArray === null) {
            return false;
        }

        dtMonth = dtArray[3];
        dtDay = dtArray[1];
        dtYear = dtArray[5];

        if (dtMonth < 1 || dtMonth > 12) {

            return false;
        } else if (dtDay < 1 || dtDay > 31) {

            return false;
        } else if ((dtMonth == 4 || dtMonth == 6 || dtMonth == 9 || dtMonth == 11) && dtDay == 31) {

            return false;
        } else if (dtMonth == 2) {

            var isleap = (dtYear % 4 === 0 && (dtYear % 100 !== 0 || dtYear % 400 === 0));

            if (dtDay > 29 || (dtDay === 29 && !isleap)) {
                return false;
            }
        }

        return true;
    }

    $.fn.sortOptions = function () {

        $(this).each(function () {

            var op = $(this).children("option");
            op.sort(function (a, b) {
                return a.text > b.text ? 1 : -1;
            });

            return $(this).empty().append(op);
        });
    };

})(Khomp, jQuery);
