<?php
    require_once ('querist.php');

    define( 'POPULATOR_NOT_ARRAY', 'Not a array' );
    define( 'POPULATOR_INCOMPATIBLE_DEVICE', 'Device is not compatible with especifications' );

    class Checker
    {
        static function checkDevice( &$device )
        {
            if( !isset( $device['model'] ) )
                return false;
            if( !isset( $device['board'] ) )
                return false;
            if( !isset( $device['serial'] ) )
                return false;
            if( !isset( $device['status'] ) )
                return false;
            if( !isset( $device['links'] ) )
                return false;
            if( !is_array( $device['links'] ) )
                return false;
            return true;
        }
    }

    class Populator
    {
        static public $last_error = '';
        private static function basicCheck( &$device )
        {
            if( !is_array( $device ) )
            {
                Populator::$last_error = POPULATOR_NOT_ARRAY;
                return false;
            }
            if( !Checker::checkDevice( $device ) )
            {
                Populator::$last_error = POPULATOR_INCOMPATIBLE_DEVICE;
                return false;
            }
            return true;
        }

        public static function populateDevice( $device )
        {
            if( !Populator::basicCheck( $device ) )
            {
                trigger_error( Populator::$last_error, E_USER_NOTICE );
                return $device;
            }
            $board_num = $device['board'];
            $serial = Populator::getDeviceSerial( $board_num );
            $device['serial'] = $serial;

            $type = Populator::getDeviceType( $serial );
            $device['type'] = $type;

            if( $type != 22 )
                $device['model'] = Populator::getDeviceModelValue( $type, $serial );
            else
            {
                $modelo = Populator::getEBSString( $serial );
                $model = Populator::getDeviceModel( $serial );
                $device['model'] = 'EBS-Modular ' . Populator::getEBSDeviceModel( $model ) . ' ' . substr( $modelo, 0, -1 );
            }

            switch($type)
            {
                //KGSM-USB
                case 14:
                case 15:
                case 18:
                case 19:
                case 20:
                case 21:
                case 24:
                // EBS Modular
                case 22:
                case 23:
                    $status_ebs = Populator::isEBSConnected( $serial );

                    if( $status_ebs == '' )
                        $device['status'] = 'N/A';
                    else if( $status_ebs == 0 )
                        $device['status'] = 'Down';
                    else
                        $device['status'] = 'Up';
                    break;

                // VoIP
                case 17:
                    $device['status'] = MonitorService::checkService( 'kmpserver' ) ? 'Up' : 'Down';
                    break;

                //If model is a board, the status is always UP
                default:
                    $device['status'] = 'Up';
                    break;
            }

            // Only E1/T1, FXS, EBS_E1 and EBS_Mod have links
            if( $type == 0 || $type == 3 || $type == 7 || $type == 9 || $type == 10 || $type == 18 || $type == 19 || $type == 20 || $type == 21 || $type == 22 || $type == 24 )
            {
                $links_count = 0;
                $links_count = Populator::getLinkCount( $serial );

                $tp = 'E1';
                // EBSE1-HI ou KPR
                if( $type == 24 || $type == 3 )
                {
                    $links_count = $links_count * 2;
                    $tp = 'HI';
                }

                $tmp = '';
                $tmp_arr = array( );
                for( $i = 0; $i < $links_count; $i++ )
                {
                    if( $device['status'] == 'Up' )
                    {
                        $tmp = Populator::getStatusLinks( $serial, $tp, $i );
                        if( strlen( $tmp ) > 3 )
                            $tmp_arr[$i]['status'] = '-';
                        else
                            $tmp_arr[$i]['status'] = $tmp != 0 ? 'Down' : 'Up';
                    }
                    else
                        $tmp_arr[$i]['status'] = 'N/A';
                }
                $device['links'] = $tmp_arr;
            }
            $fail_total = Populator::getChannelTotalFail( $serial );
            $idle_total = Populator::getChannelTotalIdle( $serial );
            $call_total = Populator::getChannelTotalCall( $serial );
            $device['channels'] = array( 'fail' => $fail_total, 'idle' => $idle_total, 'call' => $call_total );

            $device['IP'] = strpos( $device['model'], 'EBS' ) !== false ? Populator::getIPFromDevice( $serial ) : '-';

            return $device;
        }

        public static function populateLink( $device )
        {
            $board_num = $device['board'];
            $serial = Populator::getDeviceSerial( $board_num );
            $device['serial'] = $serial;
            $type = Populator::getDeviceType( $serial );
            $device['type'] = $type;

            switch($type)
            {
                case 14:
                case 15:
                    $device['status'] = 'N/A';
                    break;
                case 18:
                case 19:
                case 20:
                case 21:
                case 22:
                case 24:
                    $status_ebs = Populator::isEBSConnected( $serial );
                    if( $status_ebs == '' )
                        $device['status'] = 'N/A';
                    else if( $status_ebs == 0 )
                        $device['status'] = 'Down';
                    else
                        $device['status'] = 'Up';
                    break;

                // VoIP
                case 17:
                    if( MonitorService::checkService( 'kmpserver' ) )
                        $device['status'] = 'Up';
                    else
                        $device['status'] = 'Down';
                default:
                    $device['status'] = 'Up';
                    break;
            }

            if( $type != 22 )
                $device['model'] = Populator::getDeviceModelValue( $type, $serial );
            else
            {
                $modelo = Populator::getEBSString( $serial );
                $model = Populator::getDeviceModel( $serial );
                $device['model'] = 'EBS-Modular' . Populator::getEBSDeviceModel( $model ) . ' ' . substr( $modelo, 0, -1 );
            }
            $links_count = 0;
            $links_count = Populator::getLinkCount( $serial );

            $tp = 'E1';
            $tp2 = '';
            if( $type == 24 || $type == 3 )
            {
                $links_count = $links_count * 2;
                $tp = 'HI';
                $tp2 = 'HI';
            }

            for( $i = 0; $i < $links_count; $i++ )
            {
                $device['links'][$i]['id'] = $i;
                $signaling = Populator::getLinkSignaling( $serial, $i, $tp2 );
                
                $device['links'][$i]['signaling'] = Populator::getSignalingValue( $signaling );

                $nick = Populator::getLinkName( $serial, $i, $tp2 );
                $device['links'][$i]['nick'] = utf8_encode( $nick );
                $link_status = Populator::getStatusLinks( $serial, $tp, $i );

                if( !isset( $device['isE1'] ) )
                    $device['isE1'] = Populator::isE1( $serial );

                $isE1 = $device['isE1'];
                if( $device['status'] == 'Up' )
                {
                    $resultVar = Populator::getLinkErrorCounter( $serial, $i );
                    if( $link_status == 0 )
                    {
                        $device['links'][$i]['status'] = 'Up';
                        $device['links'][$i]['alarms'] = Populator::getArrayAlarms( $isE1 );
                    }
                    else
                    {
                        $device['links'][$i]['status'] = 'Down';
                        $var = Populator::getVariations( $link_status );
                        $device['links'][$i]['alarms'] = Populator::getArrayAlarms( $isE1, $var );
                    }
                }
                else
                {
                    $device['links'][$i]['alarms'] = Populator::getArrayAlarms( $isE1 );
                    $device['links'][$i]['alarms']['deviceUnreacheable'] = 1;
                    $device['links'][$i]['status'] = 'Down';
                    $resultVar = 'N/A';
                }
                $device['links'][$i]['sync'] = Populator::isReceivingClock( $serial, $i, $tp2 );

                if( $resultVar == 'N/A' )
                    $var = array( 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A' );
                else if( $resultVar != '' )
                    $var = explode( ',', $resultVar );
                else
                    $var = array( '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-' );

                if( $isE1 )
                    $device['links'][$i]['variations'] = array( 'BlockedCount' => $var[0], 'SignalLostError' => $var[1], 'AlarmReceived' => $var[2], 'FrameSync' => $var[3], 'MultiFrameSync' => $var[4], 'RemoteAlarm' => $var[5], 'SlipAlarm' => $var[6], 'PRBS' => $var[7], 'WrongEBits' => $var[8], 'FIFOJitter' => $var[9], 'LostSyncTime' => $var[10], 'FrameAlignment' => $var[11], 'WrongFramesCount' => $var[12], 'BipolarViolation' => $var[13], 'CRC-4Error' => $var[14] );
                else
                    $device['links'][$i]['variations'] = array( 'ChangesToLock' => $var[0], 'LostOfSignal' => $var[1], 'PulseDensityVltn' => $var[2], 'SeverelyErrorFrame' => $var[3], 'LostOfMultiframe' => $var[4], 'YellowAlarmESF' => $var[5], 'PRBS' => $var[6], 'PRBSMultiFrame' => $var[7], 'WrongFrameBit' => $var[8], 'JitterVariation' => $var[9], 'Align' => $var[10], 'FramesWithoutSync' => $var[11], 'LCV' => $var[12], 'CRC6' => $var[13] );
            }
            return $device;
        }

        public static function prePopulateChannels( $device )
        {
            $board_num = $device['board'];
            $device['serial'] = Populator::getDeviceSerial( $board_num );
            $serial = $device['serial'];

            $type = Populator::getDeviceType( $serial );
            $device['type'] = $type;

            if( $type != 22 )
                $device['model'] = Populator::getDeviceModelValue( $type, $serial );
            else
            {
                $modelo = Populator::getEBSString( $serial );
                $model = Populator::getDeviceModel( $serial );
                $device['model'] = 'EBS-Modular';
            }
            $links_count = 0;
            $links_count = Populator::getLinkCount( $serial );

            if( $links_count == 0 )
                $device['links'][99] = array( 'channels' => array( 0 => 0 ) );
            else
            {
                for( $i = 0; $i < $links_count; $i++ )
                    $device['links'][$i] = array( 'channels' => array( 0 => 0 ) );

                if( !isset( $device['isE1'] ) )
                    $device['isE1'] = Populator::isE1( $serial );
            }

            return $device;
        }

        public static function populateChannels( $device, $link, $firstCh = null, $lastCh = null )
        {
            if( $link == 99 )
                return Populator::populateChannelsWithoutLink( $device, $firstCh, $lastCh );
            else
                return Populator::populateChannelsFromLink( $device, $link );
        }

        public static function populateChannelsFromLink( $device, $link )
        {
            $board_num = $device['board'];
            $serial = $device['serial'];
            $type = $device['type'];
            $tp = '';
            if( $type == 24 || $type == 3 )
                $tp = 'HI';

            $signaling = Populator::getLinkSignaling( $serial, $link, $tp );
            $signaling = Populator::getSignalingValue( $signaling );
            $voipCount = Populator::getVoIPChannelCount( $serial );
            $chCount = Populator::getChannelCount( $serial );
            $voipMin = $chCount - $voipCount;

            if( !isset( $device['isE1'] ) )
                $device['isE1'] = Populator::isE1( $serial );
            $multiCh = $device['isE1'] ? 30 : 24;

            for( $i = 0; $i < $multiCh; $i++ )
            {
                $ch = ($multiCh * $link) + $i;
                $status_channel = Populator::getCallStatus( $serial, $ch );
                $status_tmp = Populator::getCallStatusValue( $status_channel );
                $addInfo = Populator::getAddInfo( $serial, $ch );

                if( $device['type'] == 22 )
                {
                    if( $signaling == 'OpenCAS' )
                    {
                        $ainf = Populator::getCASRawAddInfo( $addInfo );
                        $addInfo = 'Tx ' . $ainf[0] . ' - Rx ' . $ainf[1];
                    }
                    else
                        $addInfo = Populator::getAddInfoValueModular( $addInfo, $signaling );
                }
                else
                {
                    if( $signaling == 'OpenCAS' )
                    {
                        $ainf = Populator::getCASRawAddInfo( $addInfo );
                        $addInfo = 'Tx ' . $ainf[0] . ' - Rx ' . $ainf[1];
                    }
                    else if( $signaling == 'OpenR2' )
                    {
                        $ainf = Populator::getCASRawAddInfo( $addInfo );
                        $addInfo = 'Rx1 ' . $ainf[0] . ' - Rx0 ' . $ainf[1];
                    }
                    else
                        $addInfo = Populator::getAddInfoValue( $addInfo, $device['type'], $ch, $serial, $voipMin );
                }

                $info_channel = Populator::getChannelStats( $serial, $ch );
                $infos = explode( ',', $info_channel );
                $inbound = isset( $infos[0] ) ? $infos[0] : '';
                $outbound = isset( $infos[1] ) ? $infos[1] : '';
                $succeed = isset( $infos[2] ) ? $infos[2] : '';
                $failed = isset( $infos[3] ) ? $infos[3] : '';

                $device['links'][$link]['channels'][$i] = array( 'callStatus' => $status_tmp, 'signaling' => $signaling, 'details' => $addInfo, 'inbound' => $inbound, 'outbound' => $outbound, 'succeed' => $succeed, 'failed' => $failed );
                $dial_number = Populator::getDialNumber( $serial, $ch );
                $call_duration = Populator::getCallDuration( $serial, $ch );
                $device['links'][$link]['channels'][$i]['CallDuration'] = $call_duration;

                $device['links'][$link]['channels'][$i]['DialNumber'] = $dial_number != '' ? $dial_number : '-';
                $device['links'][$link]['channels'][$i]['AverageCallTime'] = Populator::getAverageCallTime( $serial, $ch );
            }
            return $device;
        }

        public static function populateChannelsWithoutLink( $device, $firstCh, $lastCh )
        {
            $board_num = $device['board'];
            $serial = $device['serial'];
            $voipCount = Populator::getVoIPChannelCount( $serial );
            $chCount = Populator::getChannelCount( $serial );
            $voipMin = $chCount - $voipCount;

            if( isset( $firstCh ) && isset( $lastCh ) )
            {
                $i = $firstCh;
                $count_channel = $lastCh + 1;
            }
            else
            {
                $i = 0;
                $count_channel = $chCount;
            }

            $dev_string = '';
            if($device['type'] > 17 )
                $dev_string = Populator::getEBSString($serial);
            
            $countGSM = '';
            if( strpos( $device['model'], 'GSM' ) !== false)
            {
                for( $ij = 0; $ij < $count_channel; $ij++ )
                    $countGSM .= $ij . ',';
            }
            elseif(strpos( $dev_string, 'GSM' ) !== false )
                $countGSM  = Populator::getGSMChannelsCount( $serial );
            else
                $countGSM = '';
            
            $countGSM = substr( $countGSM, 0, -1 );
            $GSMch = strlen( $countGSM ) > 0 ? explode( ',', $countGSM ) : array( );

            for( $i; $i < $count_channel; $i++ )
            {
                $status_channel = Populator::getCallStatus( $serial, $i );
                $status_tmp = Populator::getCallStatusValue( $status_channel );
                $sig = Populator::getChannelSignaling( $serial, $i );
                $signaling = Populator::getSignalingValue( $sig );

                $ai = Populator::getAddInfo( $serial, $i );
                if( $device['type'] == 22 )
                    $addInfo = Populator::getAddInfoValueModular( $ai, $sig );
                else
                    $addInfo = Populator::getAddInfoValue( $ai, $device['type'], $i, $serial, $voipMin );

                $info_channel = Populator::getChannelStats( $serial, $i );
                $infos = explode( ',', $info_channel );

                $oper = '-';
                $strength = '-';
                $sim = '-';
                $IMEI = '-';
                $ICCID = '-';

                if( in_array( $i, $GSMch ) )
                {
                    $strength = Populator::getGSMSignalStrengh( $serial, $i );
                    $strength = ($strength == '255' || $strength == '') ? '-' : "{$strength}%";

                    $oper = Populator::getGSMOperName( $serial, $i );
                    if( $oper == '' )
                        $oper = '-';

                    $sim = Populator::getGSMSIM( $serial, $i );
                    if( $sim == '' )
                        $sim = '-';

                    $IMEI = Populator::getGSMIMEI( $serial, $i );
                    $IMEI = ($IMEI == '') ? '-' : $IMEI;

                    $ICCID = Populator::getGSMICCID( $serial, $i );
                    $ICCID = ($ICCID == '') ? '-' : $ICCID;
                }
                //link 99 means this device dont have links but have channels.
                $device['links'][99]['channels'][$i] = array( 'callStatus' => $status_tmp, 'signaling' => $signaling, 'details' => $addInfo, 
                                                              'inbound' => $infos[0], 'outbound' => $infos[1], 'succeed' => $infos[2], 'failed' => $infos[3], 
                                                              'sim' => $sim, 'oper' => utf8_encode( $oper ), 'strength' => $strength, 'IMEI' => $IMEI, 'ICCID' => $ICCID );

                $dial_number = Populator::getDialNumber( $serial, $i );
                $device['links'][99]['channels'][$i]['CallDuration'] = Populator::getCallDuration( $serial, $i );

                $device['links'][99]['channels'][$i]['DialNumber'] = $dial_number != '' ? $dial_number : '-';
                $device['links'][99]['channels'][$i]['AverageCallTime'] = Populator::getAverageCallTime( $serial, $i );
            }
            return $device;
        }

        public static function getVariations( $status )
        {
            $retorno = array( );
            // masks sequence: SignalLost, NetworkAlarm, FrameSyncLost,
            // MultiframeSyncLost, RemoteAlarm, HighErrorRate, UnknownAlarm,
            // E1Error,NotInitialized
            $mascaras = array( '00000001', '00000010', '00000100', '00001000', '00010000', '00100000', '01000000', '10000000', '11111111' );
            $status = decbin( $status );
            $status = Populator::zeroFill( $status, 8 );

            for( $i = 0; $i < count( $mascaras ); $i++ )
                $retorno[] = ($mascaras[$i] == $status) == $mascaras[$i] ? 1 : 0;
            return $retorno;
        }

        public static function getSignalingValue( $signal )
        {
            $signaling = array( 0 => 'Inactive', 1 => 'R2Digital', 2 => 'ContinuosEM', 3 => 'PulsedEM', 4 => 'UserR2Digital', 5 => 'Analog', 6 => 'OpenCAS', 7 => 'OpenR2', 8 => 'SIP', 9 => 'OpenCCS', 10 => 'PRI EndPoint', 11 => 'Analog Terminal', 12 => 'PRI Network', 13 => 'PRI Passive', 14 => 'Line Side', 15 => 'CAS EL7', 16 => 'GSM', 17 => 'E1LC', 18 => 'ISUP', 19 => 'ISUP Passive' );
            return $signaling[$signal];
        }

        public static function getCallStatusValue( $stats )
        {
            switch ($stats)
            {
                case '0':
                    return 'Idle';
                case '1':
                    return 'On Call';
                case '2':
                    return 'On Call';
                default:
                    return 'Fail';
            }
        }

        public static function getAddInfoValueEBS( $type, $addInfo )
        {
            switch ($type)
            {
                // E1, HI
                case 18:
                case 24:
                    return Populator::getAddInfoValueE1( $addInfo );
                case 21:
                    return Populator::getAddInfoValueGSM( $addInfo );
                case 20:
                    return Populator::getAddInfoValueFXS( $addInfo );
                case 19:
                    return Populator::getAddInfoValueFXO( $addInfo );
                default:
                    return '-';
            }
        }

        public static function getAddInfoValueModular( $addinfo, $signaling )
        {
            switch($signaling)
            {
                case 5:
                    return Populator::getAddInfoValueFXO( $addinfo );
                case 8:
                    return Populator::getAddInfoValueVoIP( $addinfo );
                case 11:
                    return Populator::getAddInfoValueFXS( $addinfo );
                case 16:
                    return Populator::getAddInfoValueGSM( $addinfo );
                default:
                    return Populator::getAddInfoValueE1( $addinfo );
            }
        }

        public static function getAddInfoValue( $addInfo, $type, $ch, $board, $voipMin )
        {
            if( $type != 17 )
            {
                switch ($type)
                {
                    case 18:
                    case 19:
                    case 20:
                    case 21:
                    case 24:
                        return Populator::getAddInfoValueEBS( $type, $addInfo );
                    case 11:
                    case 12:
                    case 14:
                    case 15:
                        return Populator::getAddInfoValueGSM( $addInfo );
                    case 1:
                    case 5:
                        return Populator::getAddInfoValueFXO( $addInfo );
                    case 9:
                    case 10:
                        return Populator::getAddInfoValueFXS( $addInfo );
                    case 0:
                    case 4:
                    case 6:
                    case 7:
                    case 13:
                        return Populator::getAddInfoValueE1( $addInfo );
                    default:
                        return '-';
                }
            }
            else
                return Populator::getAddInfoValueVoIP( $addInfo );
        }

        public static function getAddInfoValueFXO( $stats )
        {
            switch ($stats)
            {
                case '0':
                    return 'Disable';
                case '1':
                    return 'Enable';
                default:
                    return 'Fail';
            }
        }

        public static function getAddInfoValueVoIP( $stats )
        {
            $retorno = array( );
            $mascaras = array( Populator::zeroFill( decbin( 16 ), 8 ), Populator::zeroFill( decbin( 64 ), 8 ) );
            $stats = decbin( $stats );
            $stats = Populator::zeroFill( $stats, 8 );
            if( $stats == 0 )
                return 'Free';
            else
            {
                if( ($mascaras[0] & $stats) == $mascaras[0] )
                    $retorno[] = 'OutgoingLock';

                if( ($mascaras[1] & $stats) == $mascaras[1] )
                    $retorno[] = 'IncomingLock';

                return implode( ',', $retorno );
            }
        }

        public static function getAddInfoValueFXS( $status )
        {
            switch ($status)
            {
                case '0':
                    return 'On Hook';
                case '1':
                    return 'Off Hook';
                case '2':
                    return 'Ringing';
                default:
                    return 'Fail';
            }
        }

        public static function getAddInfoValueGSM( $status )
        {
            switch ($status)
            {
                case '0':
                    return 'Idle';
                case '1':
                    return 'Call In Progress';
                case '2':
                    return 'SMSInProgress';
                case '3':
                    return 'ModemError';
                case '4':
                    return 'SIMCardError';
                case '5':
                    return 'NetworkError';
                default:
                    return 'Not Ready';
            }
        }

        public static function getAddInfoValueE1( $status )
        {
            $retorno = array( );
            if( $status == -1 )
                return 'Inoperative';
            if( $status == 0 )
                return 'Free';
            else if( $status == 6 )
                return 'Locked';
            else
            {
                $mascaras = array( Populator::zeroFill( decbin( 1 ), 8 ), Populator::zeroFill( decbin( 2 ), 8 ), Populator::zeroFill( decbin( 4 ), 8 ), Populator::zeroFill( decbin( 16 ), 8 ), Populator::zeroFill( decbin( 32 ), 8 ), Populator::zeroFill( decbin( 64 ), 8 ), Populator::zeroFill( decbin( 128 ), 8 ) );
                $status = decbin( $status );
                $status = Populator::zeroFill( $status, 8 );

                if( ($mascaras[2] & $status) == $mascaras[2] )
                    $retorno[] = 'Incoming';

                if( ($mascaras[1] & $status) == $mascaras[1] )
                    $retorno[] = 'Outgoing';

                if( ($mascaras[3] & $status) == $mascaras[3] )
                    $retorno[] = 'Locked for Outgoing';

                if( ($mascaras[4] & $status) == $mascaras[4] )
                    $retorno[] = 'Fail';

                if( ($mascaras[5] & $status) == $mascaras[5] )
                    $retorno[] = 'Locked for Incoming';

                if( ($mascaras[6] & $status) == $mascaras[6] )
                    $retorno[] = 'Remote Lock';
            }
            return implode( ',', $retorno );
        }

        public static function zeroFill( $var, $size )
        {
            $aux = '';
            if( strlen( $var ) >= $size )
                return $var;
            for( $x = strlen( $var ); $x < $size; $x++ )
                $aux .= '0';
            return $aux . $var;
        }

        public static function getDeviceModelValue( $type, $serial )
        {
            Global $boardModel;

            $model = '';
            $link_count = '';
            $voipCount = '';

            if( $type == 17 )
                return 'VoIP';

            $model = Populator::getDeviceModel( $serial );
            $link_count = Populator::getLinkCount( $serial );
            $voipCount = Populator::getVoIPChannelCount( $serial );

            // echo "@".$type."@".$model."@".$link_count."@".$voipCount."<BR>";

            return $boardModel[$type][$model][$link_count][$voipCount];
        }

        public static function getEBSDeviceModel( $model )
        {
            switch ($model)
            {
                case '0':
                    return 'CTI';
                case '1':
                    return 'SPX';
                default:
                    return '-';
            }
        }

        public static function checkValue( $value )
        {
            return $value == NULL || trim( $value ) == '' ? 'N/A' : $value;
        }

        public static function isE1( $serial )
        {
            return Populator::getOperationMode( $serial ) == 'E1' ? true : false;
        }

        public static function getArrayAlarms( $isE1, $var = array(0,0,0,0,0,0,0,0,0) )
        {
            if( $isE1 )
                return array( 'SignalLost' => $var[0], 'NetworkAlarm' => $var[1], 'FrameSyncLost' => $var[2], 'MultiframeSyncLost' => $var[3], 'RemoteAlarm' => $var[4], 'HighErrorRate' => $var[5], 'UnknownAlarm' => $var[6], 'E1Error' => $var[7], 'NotInitialized' => $var[8], 'deviceUnreacheable' => 0 );
            else
                return array( 'SignalLostT1' => $var[0], 'PulseDensityViolation' => $var[1], 'SeverelyErroredFrame' => $var[2], 'MultiframeSyncError' => $var[3], 'YellowAlarmESF' => $var[4], 'YellowAlarmSLC' => $var[5], 'YellowAlarmD4' => $var[6], 'YellowAlarm' => $var[7], 'NotInitialized' => $var[8], 'deviceUnreacheable' => 0 );
        }

        public static function getCASRawAddInfo( $addInfo )
        {
            $addInfo = decbin( $addInfo );
            $addInfo = str_pad( $addInfo, 8, '0', STR_PAD_LEFT );
            return array( substr( $addInfo, 0, 4 ), substr( $addInfo, 4 ) );
        }

        /************** function calling query server ***************/
        // TODO NEED TO VALIDATE ALL THE PARAMS

        public static function getDeviceSerial( $board_num )
        {
            Querist::ask( "QUERY k3l.Config.Device.{$board_num}.SerialNumber", $serial );
            return $serial;
        }

        public static function isEBSConnected( $serial )
        {
            Querist::ask( "QUERY k3l.Status.Connected.{$serial}", $status_ebs );
            return $status_ebs;
        }

        public static function getLinkCount( $serial )
        {
            $links_count = 0;
            Querist::ask( "QUERY k3l.Config.Device.{$serial}.LinkCount", $link_count );
            return $link_count;
        }

        public static function getChannelTotalFail( $serial )
        {
            $fail_total = 0;
            Querist::ask( "QUERY k3l.Status.Channel.{$serial}.TotalFail", $fail_total );
            return $fail_total;
        }

        public static function getChannelTotalIdle( $serial )
        {
            $iddle_total = 0;
            Querist::ask( "QUERY k3l.Status.Channel.{$serial}.TotalIdle", $iddle_total );
            return $iddle_total;
        }

        public static function getChannelTotalCall( $serial )
        {
            $call_total = 0;
            Querist::ask( "QUERY k3l.Status.Channel.{$serial}.TotalCall", $call_total );
            return $call_total;
        }

        public static function getDeviceType( $serial )
        {
            Querist::ask( "QUERY k3l.DeviceType.{$serial}", $device_type );
            return $device_type;
        }

        public static function getLinkErrorCounter( $serial, $link )
        {
            $count = 0;
            Querist::ask( "QUERY k3l.Status.LinkErrorCounter.{$serial}.{$link}", $count );
            return $count;
        }

        public static function getStatusLinks( $serial, $type = 'E1', $link )
        {
            // TODO need to verify if $type == E1 ou $type == HI
            $status = 0;
            Querist::ask( "QUERY k3l.Status.Link.{$serial}.{$link}.0.{$type}", $status );
            return $status;
        }

        public static function getChannelCount( $serial )
        {
            $chCount = 0;
            Querist::ask( "QUERY k3l.Config.Device.{$serial}.ChannelCount", $chCount );
            return $chCount;
        }

        public static function getVoIPChannelCount( $serial )
        {
            $voipCount = 0;
            Querist::ask( "QUERY k3l.Config.Device.{$serial}.VoIPChannelCount", $voipCount );
            return $voipCount;
        }

        public static function getAddInfo( $serial, $channel )
        {
            $addInfo = 0;
            Querist::ask( "QUERY k3l.Status.Channel.{$serial}.{$channel}.AddInfo", $addInfo );
            return $addInfo;
        }

        public static function getChannelStats( $serial, $channel )
        {
            $stats = 0;
            Querist::ask( "QUERY k3l.ChannelStats.{$serial}.{$channel}", $stats );
            return $stats;
        }

        public static function getDialNumber( $serial, $channel )
        {
            $dial_number = '';
            Querist::ask( "QUERY k3l.Status.Channel.{$serial}.{$channel}.DialNumber", $dial_number );
            return $dial_number;
        }

        public static function getCallDuration( $serial, $channel )
        {
            $call_duration = '';
            Querist::ask( "QUERY k3l.Status.Channel.{$serial}.{$channel}.CallDuration", $call_duration );
            $duration = intval( $call_duration ) / 1000;

            if( $duration > 0 )
            {
                // ugly because cant use DateTime->diff->format on PHP 5.2 on linux
                $sec = $duration % 60;
                $min = floor( ($duration % 3600) / 60 );
                $hour = floor( ($duration % 86400) / 3600 );
                $day = floor( ($duration % 2592000) / 86400 );
                $sec_str = $sec < 10 ? "0$sec" : $sec;
                $min_str = $min < 10 ? "0$min" : $min;
                $hour_str = $hour < 10 ? "0$hour" : $hour;
                $day_str = $day > 0 ? $day . 'd ' : '';

                return "$day_str$hour_str:$min_str:$sec_str";
            }
            else
                return '-';
        }

        public static function getCallStatus( $serial, $channel )
        {
            $status = 0;
            Querist::ask( "QUERY k3l.Status.Channel.{$serial}.{$channel}.CallStatus", $status );
            return $status;
        }

        public static function getGSMChannelsCount( $serial )
        {
            $count = 0;
            Querist::ask( "QUERY k3l.Config.EBS.{$serial}.GSMChannels", $count );
            return $count == NULL ? '' : $count;
        }

        public static function getLinkSignaling( $serial, $link, $HI )
        {
            $signaling = '';
            Querist::ask( "QUERY k3l.Config.Link{$HI}.{$serial}.{$link}.Signaling", $signaling );
            return $signaling;
        }

        public static function getChannelSignaling( $serial, $ch )
        {
            $signaling = '';
            Querist::ask( "QUERY k3l.Config.Channel.{$serial}.{$ch}.Signaling", $signaling );
            return $signaling;
        }

        public static function getGSMSignalStrengh( $serial, $channel )
        {
            $strength = '';
            Querist::ask( "QUERY k3l.Status.GSMChannel.{$serial}.{$channel}.SignalStrength.0", $strength );
            return $strength;
        }

        public static function getGSMOperName( $serial, $channel )
        {
            $name = '';
            Querist::ask( "QUERY k3l.Status.GSMChannel.{$serial}.{$channel}.OperName.0", $name );
            return $name;
        }

        public static function getGSMSIM( $serial, $channel )
        {
            $sim = '';
            Querist::ask( "QUERY k3l.Status.GSMChannel.{$serial}.{$channel}.SIM.0", $sim );
            return $sim;
        }

        public static function getGSMIMEI( $serial, $channel )
        {
            $imei = '';
            Querist::ask( "QUERY k3l.Status.GSMChannel.{$serial}.{$channel}.IMEI.0", $imei );
            return $imei;
        }

        public static function getGSMICCID( $serial, $channel )
        {
            $iccid = '';
            Querist::ask( "QUERY k3l.Status.GSMChannel.{$serial}.{$channel}.ICCID.0", $iccid );
            return $iccid;
        }

        public static function getDeviceModel( $serial )
        {
            $model = '';
            Querist::ask( "QUERY k3l.Config.Device.{$serial}.DeviceModel", $model );
            return $model;
        }

        public static function getEBSString( $serial )
        {
            $model = '';
            Querist::ask( "QUERY k3l.Config.EBS.{$serial}.String", $modelo );
            return $modelo;
        }

        public static function getLinkName( $serial, $link, $type = '' )
        {
            $name = '';
            Querist::ask( "QUERY k3l.Config.Link{$type}.{$serial}.{$link}.Name", $name );
            return $name;
        }

        public static function getOperationMode( $serial )
        {
            $E1T1 = '';
            Querist::ask( "QUERY k3l.Config.Link.{$serial}.0.OperatingMode", $E1T1 );
            return $E1T1;
        }

        public static function isK3LConnected( )
        {
            if( Querist::ask( 'QUERY k3l.Connected', $op ) )
                return $op;
            else
                throw new Exception( Querist::$last_error, 1 );
        }

        public static function getDeviceCount( )
        {
            $count = 0;
            Querist::ask( 'QUERY k3l.DeviceCount', $count );
            return $count;
        }

        public static function getIPFromDevice( $serial )
        {
            // TODO KMG API 3.3
            if( !IS_GATEWAY )
            {
                $ip = '';
                Querist::ask( "QUERY k3l.Config.EBS.{$serial}.IP", $ip );
                return $ip;
            }
            else
                return '0.0.0.0';
        }

        public static function getAverageCallTime( $serial, $ch )
        {
            // TODO KMG API 3.3
            if( !IS_GATEWAY )
            {
                $time = '';
                Querist::ask( "QUERY k3l.Status.Channel.{$serial}.{$ch}.AverageCallTime", $time );
                return $time == 0 ? '-' : gmdate( "H:i:s", $time );
            }
            else
                return '-';
        }

        public static function isReceivingClock( $serial, $link, $HI )
        {
            // TODO KMG API 3.3
            if( !IS_GATEWAY )
            {
                $clock = '';
                Querist::ask( "QUERY k3l.Config.Link{$HI}.{$serial}.{$link}.ReceivingClock", $clock );
                return $clock == '1' ? true : false;
            }
            else
                return true;
        }

        /* commands */
        public static function resetLink( $serial, $link )
        {
            Querist::ask( "CMD k3l.ResetLink.{$serial}.{$link}", $ret );
            return $ret == "Executed" ? true : false;
        }

        public static function blockLink( $serial, $link )
        {
            Querist::ask( "CMD k3l.BlockLink.{$serial}.{$link}", $ret );
            return $ret == "Executed" ? true : false;
        }

        public static function clearLinkErrorCounter( $serial, $link )
        {
            Querist::ask( "CMD k3l.ClearLinkErrorCounter.{$serial}.{$link}", $ret );
            return $ret == "Executed" ? true : false;
        }

        public static function selectSIM( $selector )
        {
            Querist::ask( "CMD k3l.GSM.SelectSIM.{$selector}", $ret );
            return $ret == "Executed" ? true : false;
        }

        public static function clearChannelStats( $serial, $channel )
        {
            Querist::ask( "CMD k3l.ClearChannelStats.{$serial}.{$channel}", $ret );
            return $ret == "Executed" ? true : false;
        }

        public static function resetGSMChannel( $serial, $channel )
        {
            Querist::ask( "CMD k3l.ResetModem.{$serial}.{$channel}", $ret );
            return $ret == "Executed" ? true : false;
        }

    }
